\set{capture}{hidden}
\begin{code}

module Additional (
	toEnumeration,
	fromEnumeration,
	toBool,
	fromBool,
	discardFraction,
	unJust,
	require,
	(<|>),	
	minOfPair,
	maxOfPair,
	inRange,
	inRange3,
	inRange4,
	(%),
	(#*#),
	ceilingLogBase2,
	floorLogBase2,
	bit,
	bits,
	gcdOfPair,
	lcmOfPair,
	lcmOfList,
	composeList,
	zipList,
	lengthOfList,
	takeFromList,
	dropFromList,
	coerceList,
	SetT,
	emptySet,
	listToSet,
	setToList,
	minimumOfSet,
	maximumOfSet,
	(@@),
	(/@),
	subset,
	subseteq,
	cardinality,
	union,
	intersection,
	difference,
	crossProduct,
	unions,
	intersections,
	MapT,
	emptyMap,
	listToMap,
	mapToList,
	mapCardinality,
	dom,
	codom,
	(#),
	composeMap,
	(//),
	mapUnion,
	mapUnions,
	mapSetDifference,
	mapSetIntersection,
) where

import qualified Data.Map
import qualified Data.Set
import Ratio

infixr 9 `composeMap`
infixl 9 #, `bit`, `bits`
infixl 7 #*#, `difference`, `mapSetDifference`, `crossProduct`
infixl 6 `union`, `mapUnion`, `intersection`, `mapSetIntersection`, //
infix  4 @@, /@, `subset`, `subseteq`
infixl 0 <|>, `zipList`

\end{code}

\BeginHaskellToken	{factorial}				{Void}	{}
  \HaskellArgument							{Integer}
  \HaskellToken		{{-!-}}					{Close}	{!}
\EndHaskellToken

\BeginHaskellToken	{permutations}				{Ord}	{permutations}
  \HaskellArgument							{Integer, [_T]}
\EndHaskellToken


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Additional}
\set{title}{Assumed Notation}
\begin{appendix}

As discussed in Chapter \ref{Haskell}, the presentation of all Haskell source code included in this work omits the implementation details of many simple definitions. Most of these omitted constructs can be found in the standard Haskell library, in which case only their type signatures are presented below. For all remaining entities that are applied but not defined in the actual body of this work, this appendix contains a detailed description of both the type and the implementation of the entity in the standard syntax of the Haskell programming language.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Additional/Types}
\set{title}{Standard Types}
\begin{section}

\NewHaskellToken	{Bool}					{Ord}	{«bool»}
\NewHaskellResult	{Bool}						{Type}
\NewHaskellToken	{Integer}				{Ord}	{«integer»}
\NewHaskellResult	{Integer}					{Type}
\NewHaskellToken	{Rational}				{Ord}	{«rational»}
\NewHaskellResult	{Rational}					{Type}
\NewHaskellToken	{String}				{Ord}	{«string»}
\NewHaskellResult	{String}					{Type}
\NewHaskellToken	{undefined}				{Ord}	{\bot}
\NewHaskellResult	{undefined}					{forall _T => _T}

\NewVariable		{bool}
\NewVariable		{integer}
\NewVariable		{rational}
\NewVariable		{string}

\NewHaskellToken	{True}					{Ord}	{true}
\NewHaskellResult	{True}						{Bool}
\NewHaskellToken	{False}					{Ord}	{false}
\NewHaskellResult	{False}						{Bool}

\IndexType		{Bool}						{bool (\tstar)}
\IndexType		{Integer}					{integer (\tstar)}
\IndexType		{Rational}					{rational (\tstar)}
\IndexType		{String}					{string (\tstar)}
\IndexTerm		{undefined}					{\bot (T) ((\forall T))}
\IndexCons		{True}						{true (bool)}
\IndexCons		{False}						{false (bool)}

The standard Haskell library provides a plethora of predefined scalar types, but only four of them are actually used in this work: “Bool”, “Integer”, “Rational” and “String”. For consistency with the conventions adopted by the C standard for presentation of language grammars, the names of these types are presented simply as “"Bool"\,”, “"Integer"”, “"Rational"\,” and “"String"”, respectively. The two data constructors of the “"Bool"\,” type are written as “"True"” and “"False"”, while integers are depicted simply by numeric constants such as “"0"”, “"negate 1"” or “"7"”. Rational numbers are always introduced by the arithmetic operators described later in Section \ref{Additional/Arithmetic}, while concrete string values never appear in the presented compiler implementation at all. As mentioned in Chapter \ref{Haskell}, the “undefined” value which appears as an implicit member of every Haskell type is written as “\!"undefined"\!”.

\BeginHaskellToken	{Eq}					{Ord}	{eq}
  \HaskellArgument							{Type}
\EndHaskellToken

\BeginHaskellToken	{Ord}					{Ord}	{ord}
  \HaskellArgument							{Type}
\EndHaskellToken

\BeginHaskellToken	{Num}					{Ord}	{num}
  \HaskellArgument							{Type}
\EndHaskellToken

\BeginHaskellToken	{Enum}					{Ord}	{enum}
  \HaskellArgument							{Type}
\EndHaskellToken

\BeginHaskellToken	{Show}					{Ord}	{show}
  \HaskellArgument							{Type}
\EndHaskellToken

\BeginHaskellToken	{Monad}					{Ord}	{monad}
  \HaskellArgument							{Type}
\EndHaskellToken

\BeginHaskellToken	{MonadFix}				{Ord}	{monad-fix}
  \BeginHaskellAlt
    \HaskellToken	{\_M}					{Ord}	{«M»}
    \HaskellToken	{,}					{Void}	{}
    \HaskellToken	{Monad}					{Void}	{}
    \HaskellToken	{\_M}					{Void}	{}
  \EndHaskellAlt
  \HaskellArgument							{Type}
\EndHaskellToken

\NoHaskellTokenDescription{Eq}
\NoHaskellTokenDescription{Ord}
\NoHaskellTokenDescription{Num}
\NoHaskellTokenDescription{Enum}
\NoHaskellTokenDescription{Show}
\NoHaskellTokenDescription{Monad}
\NoHaskellTokenDescription{MonadFix}

\IndexClass		{Eq}						{eq \tstar}
\IndexClass		{Ord}						{ord \tstar}
\IndexClass		{Num}						{num \tstar}
\IndexClass		{Show}						{show \tstar}
\IndexClass		{Enum}						{enum \tstar}
  
The standard prelude also supports a number of type classes, but only seven of these are of any interest to us. The “"Eq"” class provides the two standard structural equality operators “"=="” and “"/="”, while “"Ord"” implements total ordering of values, as exemplified by the four operators “"<"”, “">"”, “"<="” and “">="”. Further, the “"Num"” class introduces the four arithmetic operators “"+"”, “"-"”, “"*"” and “\!"/"” common to both integers and rational numbers, while “"Show"” describes types whose values can be formatted into human-readable strings. Finally, “"Enum"” represents enumerated types such as “"Bool"\,”, all of which can be happily confused with integer values by the virtue of two predefined functions “fromEnum” and “toEnum”. For conciseness, in this work both of these functions are treated as implicit coercions, whose names are hidden from the presentation. Their type signatures are introduced as follows:

\BeginHaskellToken	{fromEnumeration}			{Void}	{}
  \HaskellContext							{(Enum _T)}
  \HaskellArgument							{_T}
  \HaskellResult							{Integer}
\EndHaskellToken

\BeginHaskellToken	{toEnumeration}				{Void}	{}
  \HaskellContext							{(Enum _T)}
  \HaskellArgument							{Integer}
  \HaskellResult							{_T}
\EndHaskellToken

\IndexTerm		{fromEnumeration}				{ T (integer) ((enum T))}
\IndexTerm		{toEnumeration}					{ integer (T) ((enum T))}

\begin{code}
fromEnumeration {-[*]-} 	:: (Enum _T) => _T -> Integer
toEnumeration {-[*]-} 		:: (Enum _T) => Integer -> _T
\end{code}

\set{capture}		{hidden}
\begin{code}
fromEnumeration{-sem-}(x) = toInteger (fromEnum x)
toEnumeration{-sem-}(x) | inRange (toInteger (minBound::Int)) (<=) (x) (<=) (toInteger (maxBound::Int)) = toEnum (fromInteger x)
\end{code}

Most often, these implicit conversions are applied to the values of boolean quantities, whereby the “"True"” and “"False"” constructors are rendered isomorphic to the integers 1 and 0, respectively. At any rate, if both enumeration coercions represent total functions, then the notation “"succ{--}(n)"” is equal to the following value "n+1" in the underlying enumeration sequence:

\BeginHaskellToken	{fromBool}				{Void}	{}
  \HaskellArgument							{Bool}
\EndHaskellToken

\BeginHaskellToken	{toBool}				{Void}	{}
  \HaskellArgument							{Integer}
\EndHaskellToken

\set{capture}{hidden}
\begin{code}
toBool {-[*]-} :: Integer -> Bool

toBool {-sem-}(0) 		= False
toBool {-sem-}(1) 		= True

fromBool {-[*]-} :: Bool -> Integer

fromBool {-sem-}(False) 	= 0
fromBool {-sem-}(True) 		= 1
\end{code}

\BeginHaskellToken	{succ}					{Open}	{succ}
  \HaskellContext							{(Enum _T)}
  \HaskellArgument							{_T}
  \HaskellResult							{_T}
\EndHaskellToken

\IndexTerm		{succ}						{succ T (T) ((enum T))}

\begin{haskell}
succ {-[*]-} :: (Enum _T) => _T -> _T

succ {-sem-}(n) = toEnumeration (fromEnumeration n + 1)
\end{haskell}

In similar vain, in this work we also confuse integer and rational numbers, allowing the earlier to be used freely whenever a rational number is actually expected by the compiler. This implicit promotions of integers into the realm of fractions can be formalised by a coercion function of the following form:

\BeginHaskellToken	{toRational}				{Void}	{}
  \HaskellArgument							{Integer}
  \HaskellResult							{Rational}
\EndHaskellToken

\IndexTerm		{toRational}					{ integer (rational)}

\begin{haskell}
toRational {-[*]-} :: Integer -> Rational

toRational {-sem-}(x) = x % 1
\end{haskell}

assuming that the division operator “\!"\%"” described later in Section \ref{Additional/Arithmetic} always produces a rational number. The reverse demotion of rationals into the realm of integers is also possible, provided that the rational's value does not contain a fractional part. Formally:

\BeginHaskellToken	{discardFraction}			{Void}	{}
  \HaskellArgument							{Rational}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{discardFraction}				{ rational (integer)}

\begin{code}
discardFraction {-[*]-} :: Rational -> Integer

discardFraction {-sem-}(x) | (toRational(truncate (x)) == x) = truncate (x)
\end{code}

where the truncation operation “"truncate (x)"” is also defined later in Section \ref{Additional/Arithmetic}.

\end{section}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Additional/Combinators}
\set{title}{Type Combinators}
\begin{section}

In Haskell, type-valued functions are represented by polymorphic data types. Perhaps the simplest of such constructions is epitomised by the standard “Maybe” type. Since, in this work, this type combinator is used almost exclusively for depiction of optional parse tree components, both of its constructors “Just” and “Nothing” are almost always suppressed from the presentation. In particular, an application of the standard Haskell type combinator “Maybe” to "_T", is represented by the concise notation “"_T{-!-}{-!-}{-sub-}opt"\!”, formatted in line with the conventions adopted by the C standardisation committee [ANSI89]. In those contexts where an omission of the data constructor “Nothing” could lead to a confusion, the constructor is written as “"Nothing"”. Accordingly, the Haskell definition of “Maybe” can be presented as follows:

\BeginHaskellToken	{Maybe}					{Ord}	{}
  \HaskellCommand						{1}	{#1\Opt}
  \HaskellArgument							{Type}
  \HaskellResult							{Type}
\EndHaskellToken

\BeginHaskellToken	{Just}					{Void}	{}
  \HaskellContext							{forall _T}
  \HaskellArgument							{_T}
  \HaskellResult							{Maybe _T}
\EndHaskellToken

\NewHaskellToken	{Nothing}				{Ord}	{\varepsilon}
\NewHaskellResult	{Nothing}					{forall _T => Maybe _T}

\IndexType		{Maybe}						{opt \tstar (\tstar)}
\IndexCons		{Just}						{ T (T_opt) ((\forall T))}
\IndexCons		{Nothing}					{\varepsilon (T_opt) ((\forall T))}

\begin{syntax}
\begin{haskell}
data Maybe {-[-}_T{-]-} =
	Just _T
	| Nothing
\end{haskell}
\end{syntax}

For clarity, all variables of a “Maybe” type are generally assigned names that are decorated with the “«opt»” subscript, such as “"x_opt"\!” and “"identifier_opt"\!”. To complete the annihilation of the “Just” construction from our presentation, we also introduce the following coercion function, which is always undefined on “Nothing” values:

\BeginHaskellToken	{unJust}				{Void}	{}
  \HaskellContext							{forall _T}
  \HaskellArgument							{Maybe _T}
  \HaskellResult							{_T}
\EndHaskellToken

\IndexTerm		{unJust}					{ T_opt (T) ((\forall T))}

\begin{code}
unJust {-[*]-} :: Maybe _T -> _T

unJust {-sem-}(Just x) = x
\end{code}

In Haskell, stateful constructs are often represented as «monadic computations», whose structure is described by members of the standard classes “"Monad"” and “"MonadFix"”. In this work, such monads appear solely in the translation of C entities, whereby they are used to identify and reject any invalid inputs to the compiler. To this end, the notation “"require $ (_P)"” represents a monadic term that always evaluates to the Haskell unit value “"()"” lifted into some monad "_M", provided that the supplied boolean predicate "_P" is true. Formally:

\BeginHaskellToken	{require}				{Ord}	{require}
  \HaskellContext							{(Monad _M)}
  \HaskellArgument							{Bool}
  \HaskellResult							{_M{--}()}
\EndHaskellToken

\IndexClass		{Monad}						{monad \tstar}
\IndexClass		{MonadFix}					{monad_fix \tstar}

\IndexTerm		{require}					{require bool (M()) ((monad M))}

\begin{code}
require {-[*]-} :: (Monad _M) => Bool -> _M{--}()

require {-sem-}(True)	= return ()
require {-sem-}(False)	= fail "Semantic error"
\end{code}

which relies on two additional standard monadic combinators “return” and “reject” that are provided by every Haskell monad. In the actual Haskell prelude, the \hbox{“reject”} combinator actually appears under the name “fail”:

\BeginHaskellToken	{return}				{Ord}	{return}
  \HaskellContext							{forall _T => (Monad _M)}
  \HaskellArgument							{_T}
  \HaskellResult							{_M{--}(_T)}
\EndHaskellToken

\BeginHaskellToken	{fail}					{Ord}	{reject}
  \HaskellContext							{forall _T => (Monad _M)}
  \HaskellCommand						{1}	{}
  \BeginHaskellParens						{Void}	{}
  \EndHaskellParens						{Void}	{}
  \HaskellResult							{_M{--}(_T)}
\EndHaskellToken

\IndexTerm		{return}					{return T (M(T)) ((monad M \forall T))}
\IndexTerm		{fail}						{reject (M(T)) ((monad M \forall T))}

\begin{haskell}
return {-[*]-} 	:: (Monad _M) => _T -> _M{--}(_T)
fail 		:: (Monad _M) => _M{--}(_T)
\end{haskell}

The final monadic operation “"<|>"” combines two or more computations into a list of the form “"_M1 <|> _M2 <|> .. <|> _M{-sub-}n"\!”. It evaluates to the first of these terms which delivers a well-formed Haskell value. Formally:

\NewMacro		{AltSymbol}					{\if\StartOfHaskellLine
									   \setbox\TmpBox\hbox{\HaskellKeyword{=}}\hbox to \wd\TmpBox {\HaskellKeyword{\hfil\Vert\hfil}}\else
									   \HaskellKeyword{\Vert}\fi}

\BeginHaskellToken	{{-prefix-<|>-}}			{Void}	{}
  \HaskellContext							{forall _T => (Monad _M)}
  \HaskellArgument							{Maybe _T}
  \HaskellToken		{{-<|>-}}				{Bin}	{\AltSymbol}
  \HaskellArgument							{Maybe _T}
  \HaskellResult							{_M{--}(_T)}
\EndHaskellToken

\NewHaskellToken	{{-<|>-}}				{Bin}	{\AltSymbol}
\NewHaskellToken	{<|>}					{Bin}	{\AltSymbol}

\IndexTerm		{{-prefix-<|>-}}				{|| T_opt T_opt (M(T)) ((monad M \forall T))}

\begin{haskell}
{-[*]-} 		    <|> {-[*]-} :: (Monad _M) => Maybe _T -> Maybe _T -> _M{--}(_T)

{-sem-}({-hfil-}Just x	 )  <|> {-sem-}({-hfil-}_M2	 ) = return $ (x)
{-sem-}({-hfil-}Nothing	 )  <|> {-sem-}({-hfil-}Just y	 ) = return $ (y)
{-sem-}({-hfil-}Nothing	 )  <|> {-sem-}({-hfil-}Nothing	 ) = fail ("Semantic error")
\end{haskell}

\set{capture}		{hidden}
\begin{code}
(<|>) {-[*]-} {-[*]-} :: (Monad _M) => Maybe _T -> Maybe _T -> _M{--}(_T)

(Just x) <|> (_M2) 		= return (x)
(Nothing) <|> (Just y)		= return (y)
(Nothing) <|> (Nothing)		= fail "Semantic error"			
\end{code}

observing that the standard “Maybe” type already belongs to the “"Monad"” class.

\BeginHaskellToken	{{-prefix-,-type}}			{Ord}	{}
  \HaskellCommand						{2}	{#1,~#2}
  \HaskellArgument							{Type}
  \HaskellArgument							{Type}
  \HaskellResult							{Type}
\EndHaskellToken

\BeginHaskellToken	{{-prefix-:-type}}			{Ord}	{}
  \HaskellCommand						{2}	{#1~:~#2}
  \HaskellArgument							{Type}
  \HaskellArgument							{Type}
  \HaskellResult							{Type}
\EndHaskellToken

\BeginHaskellToken	{{-prefix->-type}}			{Ord}	{}
  \HaskellCommand						{2}	{#1~\triangleright~#2}
  \HaskellArgument							{Type}
  \HaskellArgument							{Type}
  \HaskellResult							{Type}
\EndHaskellToken

\BeginHaskellToken	{{-prefix-,-term}}			{Ord}	{}
  \HaskellContext							{forall _T, _U}
  \HaskellCommand						{2}	{#1,~#2}
  \HaskellArgument							{_T}
  \HaskellArgument							{_U}
  \HaskellResult							{(_T, _U)}
\EndHaskellToken

\BeginHaskellToken	{{-prefix-:-term}}			{Ord}	{}
  \HaskellContext							{forall _T, _U}
  \HaskellCommand						{2}	{#1~:~#2}
  \HaskellArgument							{_T}
  \HaskellArgument							{_U}
  \HaskellResult							{(_T, _U)}
\EndHaskellToken

\BeginHaskellToken	{{-prefix->-term}}			{Ord}	{}
  \HaskellContext							{forall _T, _U}
  \HaskellCommand						{2}	{#1~\triangleright~#2}
  \HaskellArgument							{_T}
  \HaskellArgument							{_U}
  \HaskellResult							{(_T, _U)}
\EndHaskellToken

\BeginHaskellToken	{{->-}}					{Rel}	{\triangleright}
  \HaskellToken		{,}					{Void}	{}
\EndHaskellToken

\BeginHaskellToken	{{-:-}}					{Op}	{:}
  \HaskellToken		{,}					{Void}	{}
\EndHaskellToken

\IndexType		{{-prefix-,-type}}				{, \tstar \tstar (\tstar)}
\IndexType		{{-prefix-:-type}}				{: \tstar \tstar (\tstar)}
\IndexType		{{-prefix->-type}}				{\triangleright \tstar \tstar (\tstar)}

\IndexCons		{{-prefix-,-term}}				{, T U (T U) ((\forall T U))}
\IndexCons		{{-prefix-:-term}}				{: T U (T U) ((\forall T U))}
\IndexCons		{{-prefix->-term}}				{\triangleright T U (T U) ((\forall T U))}

Last but not least, no treatment of type combinators would be complete without a mention of «tuples», or «n»-products of predetermined Haskell types. In this work, a collection of «n» Haskell values "x1 :: _T1, x2 :: _T2 .. x_n :: _T_n" is usually written as a term  "(x1, x2 .. x_n)" of the type "(_T1, _T2 .. _T_n)", although the surrounding parentheses may be omitted whenever they are deemed visually redundant by the expression's contents. For clarity, the “","” separator in a tuple expression or type is often replaced with the symbol “\,"{-:-},"\,” or “\,"{->-},"\!”. In other words, both of the terms "(_A {-:-}, _B)" and "(_A {->-}, _B)" should be always considered as semantically identical to "(_A, _B)", distinguished from the later form only in the interest of presentation. 

\end{section}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Additional/Logical}
\set{title}{Logical Operations}
\begin{section}

\BeginHaskellToken	{{-prefix-\&\&-}}			{Void}	{}
  \HaskellArgument							{Bool}
  \HaskellToken		{{-land-}}				{Bin}	{\land}
  \HaskellArgument							{Bool}
  \HaskellResult							{Bool}
\EndHaskellToken

\BeginHaskellToken	{{-prefix-||-}}				{Void}	{}
  \HaskellArgument							{Bool}
  \HaskellToken		{{-lor-}}				{Bin}	{\lor}
  \HaskellArgument							{Bool}
  \HaskellResult							{Bool}
\EndHaskellToken

\BeginHaskellToken	{not}					{Open}	{\lnot}
  \HaskellArgument							{Bool}
  \HaskellResult							{Bool}
\EndHaskellToken

\NewHaskellToken	{\&\&}					{Rel}	{\land}
\NewHaskellToken	{||}					{Rel}	{\lor}

The three Haskell operators “\&\&”, “||” and “not” are typeset in this work as the well-known logical notations “"&&"”, “"||"” and “"not"”, respectively. In the standard library, they assume the following Haskell signatures:

\IndexTerm		{{-prefix-\&\&-}}				{\land bool bool (bool)}
\IndexTerm		{{-prefix-||-}}					{\lor bool bool (bool)}
\IndexTerm		{not}						{\lnot bool (bool)}

\begin{haskell}
{-[*]-} && {-[*]-}, {-[*]-} || {-[*]-} :: Bool -> Bool -> Bool
not {-[*]-} :: Bool -> Bool
\end{haskell}

It is important to observe that the standard definitions of these functions are strict only in their first boolean argument.

For convenience, the “"&&"” and “"||"” operators may be also applied to an entire list of operands using the standard Haskell functions “and” and “or”. In the presentation, these two list operations are always typeset as “"and"” and “"or"”, respectively and their Haskell signatures are defined as follows:

\BeginHaskellToken	{and}					{Open}	{\hbox{\raise1.6ex\hbox{\bigwedge}}}
  \HaskellArgument							{[Bool]}
  \HaskellResult							{Bool}
\EndHaskellToken

\BeginHaskellToken	{or}					{Open}	{\hbox{\raise1.6ex\hbox{\bigvee}}}
  \HaskellArgument							{[Bool]}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{and}						{\bigwedge bools (bool)}
\IndexTerm		{or}						{\bigvee bools (bool)}

\begin{haskell}
and {-[*]-}, or {-[*]-} :: [Bool] -> Bool
\end{haskell}

where the notation “"[_T]"” represents the standard Haskell polymorphic list type. Both of these operations are particularly useful when combined with the list comprehension notation. For example, the expression “"and [ {-,-} _P{--}(x_k) | x_k <- x_set {-,,-} ]"” evaluates to “"True"” if and only if every element of the list "x_set" satisfies the predicate function "_P".

\end{section}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Additional/Relational}
\set{title}{Relational Operations}
\begin{section}

\NewHaskellToken	{==}					{Rel}	{\eq}
\NewHaskellToken	{/=}					{Rel}	{\ne}
\NewHaskellToken	{<}					{Rel}	{\lt}
\NewHaskellToken	{>}					{Rel}	{\gt}
\NewHaskellToken	{<=}					{Rel}	{\le}
\NewHaskellToken	{>=}					{Rel}	{\ge}

Boolean values may be also constructed using the two Haskell operators “==” and “/=”, typeset as “"=="” and “"/="” in our presentation. In this work, these operators are always assumed to represent the structural identity relation that can be derived implicitly for every non-functional type "_T" using an appropriate “deriving” clause. Intuitively, these operators have the following Haskell signatures:

\BeginHaskellToken	{{-prefix-==-}}				{Void}	{}
  \HaskellContext							{(Eq _T)}
  \HaskellArgument							{_T}
  \HaskellToken		{{-eq-}}				{Rel}	{\eq}
  \HaskellArgument							{_T}
  \HaskellResult							{Bool}
\EndHaskellToken

\BeginHaskellToken	{{-prefix-/=-}}				{Void}	{}
  \HaskellContext							{(Eq _T)}
  \HaskellArgument							{_T{-sub-}Eq}
  \HaskellToken		{{-ne-}}				{Rel}	{\ne}
  \HaskellArgument							{_T{-sub-}Eq}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{{-prefix-==-}}					{\eq T T (bool) ((Eq T))}
\IndexTerm		{{-prefix-/=-}}					{\ne T T (bool) ((Eq T))}

\begin{haskell}
{-[*]-} == {-[*]-}, {-[*]-} /= {-[*]-} :: (Eq _T) => _T -> _T -> Bool
\end{haskell}

A pair of values whose type is subject to an appropriate total ordering may be also compared using the standard Haskell operators “<”, “<=”, “>” and “>=”, predictably typeset as “"<"”, “"<="”, “">"” and “">="”, respectively:

\BeginHaskellToken	{{-prefix-<-}}				{Void}	{}
  \HaskellContext							{(Ord _T)}
  \HaskellArgument							{_T}
  \HaskellToken		{{-lt-}}				{Rel}	{\lt}
  \HaskellArgument							{_T}
  \HaskellResult							{Bool}
\EndHaskellToken

\BeginHaskellToken	{{-prefix-<=-}}				{Void}	{}
  \HaskellContext							{(Ord _T)}
  \HaskellArgument							{_T}
  \HaskellToken		{{-le-}}				{Rel}	{\le}
  \HaskellArgument							{_T}
  \HaskellResult							{Bool}
\EndHaskellToken

\BeginHaskellToken	{{-prefix->-}}				{Void}	{}
  \HaskellContext							{(Ord _T)}
  \HaskellArgument							{_T}
  \HaskellToken		{{-gt-}}				{Rel}	{\gt}
  \HaskellArgument							{_T}
  \HaskellResult							{Bool}
\EndHaskellToken

\BeginHaskellToken	{{-prefix->=-}}				{Void}	{}
  \HaskellContext							{(Ord _T)}
  \HaskellArgument							{_T}
  \HaskellToken		{{-ge-}}				{Rel}	{\ge}
  \HaskellArgument							{_T}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{{-prefix-<-}}					{\lt T T (bool) ((Ord T))}
\IndexTerm		{{-prefix-<=-}}					{\le T T (bool) ((Ord T))}
\IndexTerm		{{-prefix->-}}					{\gt T T (bool) ((Ord T))}
\IndexTerm		{{-prefix->=-}}					{\ge T T (bool) ((Ord T))}

\begin{haskell}
{-[*]-} < {-[*]-}, {-[*]-} <= {-[*]-}, {-[*]-} > {-[*]-}, {-[*]-} >= {-[*]-} :: (Ord _T) => _T -> _T -> Bool
\end{haskell}

Given a pair of ordered values "(x, y)", the lesser and greater of its members may be retrieved using the respective notations “"minOfPair{--}(x, y)"” and “"maxOfPair{--}(x, y)"”. Both of these functions are provided in the standard Haskell library with the following types:

\BeginHaskellToken	{minOfPair}				{Open}	{min}
  \HaskellContext							{(Ord _T)}
  \HaskellArgument							{(_T, _T)}
  \HaskellResult							{_T}
\EndHaskellToken

\BeginHaskellToken	{maxOfPair}				{Open}	{max}
  \HaskellContext							{(Ord _T)}
  \HaskellArgument							{(_T, _T)}
  \HaskellResult							{_T}
\EndHaskellToken

\IndexTerm		{minOfPair}					{min T T (T) ((Ord T))}
\IndexTerm		{maxOfPair}					{max T T (T) ((Ord T))}

\begin{code}
minOfPair {-[*]-}, maxOfPair {-[*]-} :: (Ord _T) => (_T, _T) -> _T
\end{code}

\set{capture}{hidden}
\begin{code}
minOfPair {-sem-}(x, y) = min x y
maxOfPair {-sem-}(x, y) = max x y
\end{code}

As with the “"&&"” and “"||"” operators, “"minOfPair"” and “"maxOfPair"” can be also applied to entire lists of values. The relevant functions are known in the standard Haskell library under the names of “minimum” and “maximum”, but, for clarity, we will typeset them simply as follows:

\BeginHaskellToken	{minimum}				{Open}	{min}
  \HaskellContext							{(Ord _T)}
  \HaskellArgument							{[_T]}
  \HaskellResult							{_T}
\EndHaskellToken

\BeginHaskellToken	{maximum}				{Open}	{max}
  \HaskellContext							{(Ord _T)}
  \HaskellArgument							{[_T]}
  \HaskellResult							{_T}
\EndHaskellToken

\IndexTerm		{minimum}					{min Ts (T) ((Ord T))}
\IndexTerm		{maximum}					{max Ts (T) ((Ord T))}

\begin{haskell}
minimum {-[*]-}, maximum {-[*]-} :: (Ord _T) => [_T] -> _T
\end{haskell}

It should be observed that both of these constructions have undefined semantics when applied to an empty list of values.

Comparisons of numeric quantities often give rise to mathematical notations of the form “"inRange (x) (<) (y) (<) (z)"”. In this work, such constructs are given a formal mandate by the following Haskell definitions:

\BeginHaskellToken	{inRange}				{Void}	{}
  \HaskellArgument							{_T}
  \HaskellArgument							{_T -> _U -> Bool}
  \HaskellArgument							{_U}
  \HaskellArgument							{_U -> _V -> Bool}
  \HaskellArgument							{_V}
  \HaskellResult							{Bool}
\EndHaskellToken
\IndexTerm		{inRange}					{ T T U bool U V U bool V (bool)}

\BeginHaskellToken	{inRange3}				{Void}	{}
  \HaskellArgument							{_T}
  \HaskellArgument							{_T -> _U -> Bool}
  \HaskellArgument							{_U}
  \HaskellArgument							{_U -> _V -> Bool}
  \HaskellArgument							{_V}
  \HaskellArgument							{_V -> _W -> Bool}
  \HaskellArgument							{_W}
  \HaskellResult							{Bool}
\EndHaskellToken

\BeginHaskellToken	{inRange4}				{Void}	{}
  \HaskellArgument							{_T}
  \HaskellArgument							{_T -> _U -> Bool}
  \HaskellArgument							{_U}
  \HaskellArgument							{_U -> _V -> Bool}
  \HaskellArgument							{_V}
  \HaskellArgument							{_V -> _W -> Bool}
  \HaskellArgument							{_W}
  \HaskellArgument							{_W -> _X -> Bool}
  \HaskellArgument							{_X}
  \HaskellResult							{Bool}
\EndHaskellToken

\LocalHaskellToken	{`op1`}					{Rel}	{«op\sub{1}»}
\LocalHaskellToken	{`op2`}					{Rel}	{«op\sub{2}»}
\LocalHaskellToken	{`op3`}					{Rel}	{«op\sub{3}»}
\LocalHaskellToken	{`op4`}					{Rel}	{«op\sub{4}»}

\set{continued}		{yes}
\begin{code}
inRange {-[*]-} {-[*]-} {-[*]-} {-[*]-} {-[*]-} :: _T -> (_T -> _U -> Bool) -> _U -> (_U -> _V -> Bool) -> _V -> Bool

inRange ({-sem-}(x)) (op1) ({-sem-}(y)) (op2) ({-sem-}(z)) = (x `op1` y && y `op2` z)
\end{code}

\set{capture}{hidden}
\begin{code}
inRange3 {-[*]-} {-[*]-} {-[*]-} {-[*]-} {-[*]-} {-[*]-} {-[*]-} ::
	_T -> (_T -> _U -> Bool) -> _U -> (_U -> _V -> Bool) -> _V -> (_V -> _W -> Bool) -> _W -> Bool

inRange3 (w) (op1) (x) (op2) (y) (op3) (z) = (w `op1` x && x `op2` y && y `op3` z)

inRange4 {-[*]-} {-[*]-} {-[*]-} {-[*]-} {-[*]-} {-[*]-} {-[*]-} {-[*]-} {-[*]-} ::
	_T -> (_T -> _U -> Bool) -> _U -> (_U -> _V -> Bool) -> _V -> (_V -> _W -> Bool) -> _W -> (_W -> _X -> Bool) -> _X -> Bool

inRange4 (v) (op1) (w) (op2) (x) (op3) (y) (op4) (z) = (v `op1` w && w `op2` x && x `op3` y && y `op4` z)
\end{code}
\end{section}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Additional/Arithmetic}
\set{title}{Arithmetic Operations}
\begin{section}

\NewHaskellToken	{+}					{Bin}	{\plus}
\NewHaskellToken	{-}					{Bin}	{\minus}
\NewHaskellToken	{*}					{Bin}	{\times}
\NewHaskellToken	{/}					{Op}	{\slash}
\NewHaskellToken	{\%}					{Op}	{\slash}

\BeginHaskellToken	{{-prefix-+-}}				{Void}	{}
  \HaskellContext							{(Num _T)}
  \HaskellArgument							{_T}
  \HaskellToken		{{-plus-}}				{Bin}	{\plus}
  \HaskellArgument							{_T}
  \HaskellResult							{_T}
\EndHaskellToken

\BeginHaskellToken	{{-prefix---}}				{Void}	{}
  \HaskellContext							{(Num _T)}
  \HaskellArgument							{_T}
  \HaskellToken		{{-minus-}}				{Bin}	{\minus}
  \HaskellArgument							{_T}
  \HaskellResult							{_T}
\EndHaskellToken

\BeginHaskellToken	{{-prefix-*-}}				{Void}	{}
  \HaskellContext							{(Num _T)}
  \HaskellArgument							{_T}
  \HaskellToken		{{-times-}}				{Bin}	{\times}
  \HaskellArgument							{_T}
  \HaskellResult							{_T}
\EndHaskellToken

\BeginHaskellToken	{{-prefix-/-}}				{Void}	{}
  \HaskellContext							{(Num _T)}
  \HaskellArgument							{_T}
  \HaskellToken		{{-slash-}}				{Bin}	{\slash}
  \HaskellArgument							{_T}
  \HaskellResult							{_T}
\EndHaskellToken

\BeginHaskellToken	{negate}				{Open}	{\minus}
  \HaskellContext							{(Num _T)}
  \HaskellArgument							{_T}
  \HaskellResult							{_T}
\EndHaskellToken

Predictably, the four standard binary arithmetic operators “+”, “-”, “*” and “/\,” are always typeset as the familiar mathematical symbols “"+"”, “"-"”, “"*"” and “\!"/"”. Further, the unary operator “negate” is also written as “"negate"”. Each of these five operators may be applied to either integer or rational operands, but, in both cases, it always reflects the true arithmetic result of the corresponding mathematical construction:

\IndexTerm		{{-prefix-+-}}				{\plus T T (T) ((num T))}
\IndexTerm		{{-prefix---}}				{\minus T T (T) ((num T))}
\IndexTerm		{negate}				{\minus T (T) ((num T))}
\IndexTerm		{{-prefix-*-}}				{\times T T (T) ((num T))}
\IndexTerm		{{-prefix-/-}}				{\slash T T (T) ((num T))}

\begin{haskell}
{-[*]-} + {-[*]-}, {-[*]-} - {-[*]-}, {-[*]-} * {-[*]-} :: (Num _T) => _T -> _T -> _T
{-[*]-} / {-[*]-} :: (Num _T) => _T -> _T -> Rational
negate{-,-}{-[*]-} :: (Num _T) => _T -> _T
\end{haskell}

The reader should observe that, in this work, the division operator “\!"/"” always produces a rational number, even when applied to integer arguments. To convert the resulting fraction back into an integer, one of the following four rounding operations must be applied explicitly to its value:

\BeginHaskellToken	{round}					{Open}	{round}
  \HaskellArgument							{Rational}
  \HaskellResult							{Integer}
\EndHaskellToken

\BeginHaskellToken	{truncate}				{Void}	{}
  \BeginHaskellParens						{Open}	{\hbox{\hbox to 0pt {\lceil\hss}\lfloor}}
    \RawHaskellArgument							{Rational}
  \EndHaskellParens						{Close}	{\hbox{\hbox to 0pt {\rceil\hss}\rfloor}}
  \HaskellResult							{Integer}
\EndHaskellToken

\BeginHaskellToken	{ceiling}				{Void}	{}
  \BeginHaskellParens						{Open}	{\lceil}
    \RawHaskellArgument							{Rational}
  \EndHaskellParens						{Close}	{\rceil}
  \HaskellResult							{Integer}
\EndHaskellToken

\BeginHaskellToken	{floor}					{Void}	{}
  \BeginHaskellParens						{Open}	{\lfloor}
    \RawHaskellArgument							{Rational}
  \EndHaskellParens						{Close}	{\rfloor}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{round}						{round rational (integer)}
\IndexTerm		{truncate}					{\lceil\lfloor\rfloor\rceil rational (integer)}
\IndexTerm		{ceiling}					{\lceil\rceil rational (integer)}
\IndexTerm		{floor}						{\lfloor\rfloor rational (integer)}

\begin{haskell}
round {-[*]-}		:: Rational -> Integer
ceiling({-[*]-})	:: Rational -> Integer
truncate ({-[*]-})	:: Rational -> Integer
floor({-[*]-})		:: Rational -> Integer
\end{haskell}

All four of these operations are implemented for us in the standard Haskell library. The “"round"” function converts its operand to the nearest integer value, while “"truncate (x)"”,  “"ceiling(x)"” and “"floor(x)"” round "x" towards zero, infinity and negative infinity, respectively. On the other hand, the superficially similar notation “"abs(x)"” has nothing to do with rounding at all. Instead, it returns the absolute value of a numeric quantity. Once again, it is already provided for us in the standard Haskell library with the following type signature:

\BeginHaskellToken	{abs}					{Void}	{}
  \HaskellContext							{(Num _T)}
  \BeginHaskellParens						{Open}	{\vert\,}
    \RawHaskellArgument							{_T}
  \EndHaskellParens						{Close}	{\,\vert}
  \HaskellResult							{_T}
\EndHaskellToken

\IndexTerm		{abs}						{\vert\vert T (T) ((num T))}

\begin{haskell}
abs ({-[*]-}) :: (Num _T) => _T -> _T
\end{haskell}

\BeginHaskellToken	{sum}					{Open}	{\hbox{\raise1.6ex\hbox{\sum}}}
  \HaskellContext							{(Num _T)}
  \HaskellArgument							{[_T]}
  \HaskellResult							{_T}
\EndHaskellToken

\BeginHaskellToken	{product}				{Open}	{\hbox{\raise1.6ex\hbox{\prod}}}
  \HaskellContext							{(Num _T)}
  \HaskellArgument							{[_T]}
  \HaskellResult							{_T}
\EndHaskellToken

Mathematicians often like to apply the "+" and "*" operators to entire lists of values. In the standard Haskell library, the relevant functions are written as “sum” and “product” but, in this work, they are typeset using the familiar symbols “"sum"” and “"product"”, giving rise to the following declarations of their types:

\IndexTerm		{sum}						{\sum Ts (T) ((num T))}
\IndexTerm		{product}					{\prod Ts (T) ((num T))}

\begin{haskell}
sum {-[*]-}, product {-[*]-} :: (Num _T) => [_T] -> _T
\end{haskell}

Occasionally, when the two operands of the “"*"” operator are represented by short variable names or parenthesised expressions, the operator symbol is omitted from the presentation for conciseness. No ambiguity is generally possible, since the resulting presentation essentially resembles the syntax of a function application, which is of course meaningless on numeric quantities. Formally, the notation is enacted by the following unusual Haskell definition:

\BeginHaskellToken	{{-prefix-\#*\#-}}			{Void}	{}
  \HaskellContext							{(Num _T)}
  \HaskellArgument							{_T}
  \HaskellArgument							{_T}
  \HaskellResult							{_T}
\EndHaskellToken

\IndexTerm		{{-prefix-\#*\#-}}				{ T T (T) ((num T))}

\begin{haskell}
{-[*]-} 	{-[*]-} :: (Num _T) => _T -> _T -> _T

{-sem-}(x) 	{-sem-}(y) = x * y
\end{haskell}

\NewHaskellToken	{\#*\#}					{Op}	{}

\set{capture}{hidden}
\begin{code}
(#*#) :: (Num _T) => _T -> _T -> _T

x #*# y = x * y
\end{code}

The standard library also supports a number of exponentiation operators. In this work, all of these are presented using the same familiar mathematical formatting, which is exemplified by the following Haskell type signature:

\VoidFunction		{\^}
\NewHaskellModifier	{\^}						{\sup}
\VoidFunction		{\^\^}
\NewHaskellModifier	{\^\^}						{\sup}

\BeginHaskellToken	{{-prefix-\^-}}				{Ord}	{}
  \HaskellContext							{(Num _T)}
  \HaskellCommand						{2}	{#1\sup{#2}}
  \HaskellArgument							{_T}
  \HaskellArgument							{_T}
  \HaskellResult							{_T}
\EndHaskellToken

\IndexTerm		{{-prefix-\^-}}					{ T T (T) ((Num T)) superscript}

\begin{haskell}
{-[*]-} ^ {-[*]-} :: (Num _T) => _T -> _T -> _T
\end{haskell}

\BeginHaskellToken	{ceilingLogBase2}			{Void}	{}
  \BeginHaskellParens						{Open}	{\lceil log\sub{2}}
    \RawHaskellArgument							{Rational}
  \EndHaskellParens						{Close}	{\rceil}
  \HaskellResult							{Integer}
\EndHaskellToken

\BeginHaskellToken	{floorLogBase2}				{Void}	{}
  \BeginHaskellParens						{Open}	{\lfloor log\sub{2}}
    \RawHaskellArgument							{Rational}
  \EndHaskellParens						{Close}	{\rfloor}
  \HaskellResult							{Integer}
\EndHaskellToken

\BeginHaskellToken	{logBase}				{Open}	{log}
  \HaskellModifier							{\sub}
  \HaskellArgument							{Integer}
  \HaskellArgument							{Rational}
  \HaskellResult							{Rational}
\EndHaskellToken

While, in our compiler, we do not require a complete mathematical implementation of the dual “log” function, in a number of places we rely on its integer approximations “"ceilingLogBase2((x))"” and “"floorLogBase2((x))"”. Fortunately, both of these can be obtained easily enough using the following recursive algorithms:

\IndexTerm		{ceilingLogBase2}				{log2\lceil\rceil rational (integer)}
\IndexTerm		{floorLogBase2}					{log2\lfloor\rfloor rational (integer)}

\begin{code}
ceilingLogBase2 {- ({-[*]-}) -}, floorLogBase2 {- ({-[*]-}) -} :: Rational -> Integer

ceilingLogBase2 ({-sem-}(x))	| (x <= 1%2)				= ceilingLogBase2 ((x * 2)) - 1
				| (inRange (1%2) (<) (x) (<=) (1))	= 0
				| (x > 1)				= ceilingLogBase2 ((x / 2)) + 1

floorLogBase2 ({-sem-}(x))	| (x < 1)				= floorLogBase2 ((x * 2)) - 1
				| (inRange (1) (<=) (x) (<) (2))	= 0
				| (x >= 2)				= floorLogBase2 ((x / 2)) + 1
\end{code}

For integers only, we must also provide three additional functions “"gcdOfPair"”, “"lcmOfPair"” and “"`mod`"” borrowed from number theory. The first two construct the greatest common divisor and the least common multiple of two integers, as obtained using the well-known Euclid's algorithm. In this work, their type signature are represented as follows:

\BeginHaskellToken	{gcdOfPair}				{Open}	{gcd}
  \HaskellArgument							{(Integer, Integer)}
  \HaskellResult							{Integer}
\EndHaskellToken

\BeginHaskellToken	{lcmOfPair}				{Open}	{lcm}
  \HaskellArgument							{(Integer, Integer)}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{gcdOfPair}					{gcd integer integer (integer)}
\IndexTerm		{lcmOfPair}					{lcm integer integer (integer)}

\begin{code}
gcdOfPair {-[*]-}, lcmOfPair {-[*]-} :: (Integer, Integer) -> Integer
\end{code}

\set{capture}{hidden}
\begin{code}
gcdOfPair{-sem-}(x, y) = gcd x y
lcmOfPair{-sem-}(x, y) = lcm x y
\end{code}

The third, “"`mod`"”, is used to implement proper modulo arithmetic. Given an arbitrary integer "x" and a positive integer "y", “"x `mod` y"” returns the least non-negative integer that is congruent to "x" in arithmetic modulo "y". An efficient implementation of this function is readily available in the standard Haskell library and comes with the following type signature:

\BeginHaskellToken	{mod}					{Void}	{}
  \HaskellArgument							{Integer}
  \HaskellToken		{{-mod-}}				{Ord}	{mod}
  \HaskellArgument							{Integer}
  \HaskellResult							{Integer}
\EndHaskellToken

\NewHaskellToken	{`mod`}					{Ord}	{mod}

\IndexTerm		{mod}						{mod integer integer (integer)}

\begin{haskell}
{-[*]-} `mod` {-[*]-} :: Integer -> Integer -> Integer
\end{haskell}

In Appendix \ref{C/Implementation}, we also find it necessary to apply the “"lcmOfPair"” operator to an entire list of values. Using the built-in Haskell function “foldl” described in \hbox{Section \ref{Additional/Functions},} this construction can be implemented as follows:

\BeginHaskellToken	{lcmOfList}				{Open}	{lcm}
  \HaskellArgument							{[Integer]}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{lcmOfList}					{lcm integers (integer)}

\begin{code}
lcmOfList {-[*]-} :: [Integer] -> Integer

lcmOfList {-sem-}(x_set {-,-}) = foldl (lcm) 1 x_set
\end{code}

A careful use of the “"`mod`"” operator also permits us to extract individual bits of data from a given integer value "x". In particular, the "i"th bit in the two's complement representation of such an integer can be obtained using the following formula:

\BeginHaskellToken	{bit}					{Void}	{}
  \HaskellArgument							{Integer}
  \HaskellArgument							{Integer}
  \HaskellResult							{Integer}
\EndHaskellToken

\NewHaskellToken	{`bit`}					{Op}	{}

\IndexTerm		{bit}						{ integer integer (integer)}

\begin{code}
bit {-[*]-} 	{-[*]-} :: Integer -> Integer -> Integer

bit {-sem-}(x) 	{-sem-}(i {-,-}) = floor (x % 2^i) `mod` 2
\end{code}

Finally, an arbitrary subset of bits extracted in this fashion can be summed up backed into an integer value, giving rise to the following useful notation:

\BeginHaskellToken	{bits}					{Void}	{}
  \HaskellArgument							{Integer}
  \HaskellArgument							{[Integer]}
  \HaskellResult							{Integer}
\EndHaskellToken

\NewHaskellToken	{`bits`}				{Op}	{}

\IndexTerm		{bits}						{ integer integers (integer)}

\begin{code}
bits {-[*]-} 	 {-[*]-} :: Integer -> [Integer] -> Integer

bits {-sem-}(x)  {-sem-}(i_set {-,,-}) = sum [ x`bit`(k) * 2^(k - minimum{--}(i_set)) | k <- i_set {-,,-}]
\end{code}

This completes the entire suite of arithmetic functions required for a satisfactory implementation of our C compiler. Although many of the above definitions are clearly sub-optimal, they are nevertheless correct. Readers are encouraged to improve upon them and to apply the theorem proving capabilities from Chapter \ref{Haskell} in order to establish a semantic neutrality of any such improvements.

\end{section}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Additional/Functions}
\set{title}{Function Combinators}
\begin{section}

Of course, numbers are not the only entities manipulated by our compiler. In particular, our implementation often operates on Haskell functions themselves. Perhaps the most useful of all such «function combinators» is the composition operator “"."”, whose well-known implementation is included in the standard Haskell library as follows:

\BeginHaskellToken	{{-prefix-.-}}				{Ord}	{}
  \HaskellContext							{forall _T _U _V}
  \HaskellCommand						{2}	{#1~\circle~#2}
  \HaskellArgument							{(_U -> _V)}
  \HaskellArgument							{(_T -> _U)}
  \HaskellResult							{(_T -> _V)}
\EndHaskellToken

\NewHaskellToken	{.}					{Bin}	{\circle}

\IndexTerm		{{-prefix-.-}}					{\circle U V T U (T V) ((\forall T U V))}

\begin{haskell}
{-[*]-} 		. {-[*]-} :: (_U -> _V) -> (_T -> _U) -> (_T -> _V)

{-sem-}( {-,,,-} f) 	. {-sem-}(g) = \ x -> f{--}(g{--}(x))
\end{haskell}

Intuitively, given a pair of unary functions "f" and "g", "f . g" represents a new unary function that begins by applying "g" to its operand and produces the value obtained from an application of "f" to the result of "g". Such function composition may be also applied naturally to an entire list of functions of the Haskell type "_T -> _T":

\BeginHaskellToken	{composeList}				{Open}	{\odot}
  \HaskellContext							{forall _T}
  \HaskellArgument							{[_T -> _T]}
  \HaskellResult							{(_T -> _T)}
\EndHaskellToken

\IndexTerm		{composeList}					{\odot T T (T T) ((\forall T))}

\begin{code}
composeList {-[*]-} :: [_T -> _T] -> (_T -> _T)

composeList {-sem-}({-,,-} f_set {-,,,-} ) = foldl (.) (id) f_set
\end{code}

\BeginHaskellToken	{id}					{Ord}	{id}
  \HaskellContext							{forall _T}
  \HaskellArgument							{_T}
  \HaskellResult							{_T}
\EndHaskellToken

\IndexTerm		{id}						{id T (T) ((\forall T))}

in which “"id"” represents the following trivial identity function:

\begin{haskell}
id {--} {-[*]-} :: _T -> _T

id {--} {-sem-}(x) = x
\end{haskell}

and “"foldl f y x_set"\,\,” depicts the «left fold combinator», which, beginning with the value "y", systematically applies "f{--}(y, x_k)" to all "x_k @@ x_set":

\BeginHaskellToken	{foldl}					{Ord}	{fold}
  \HaskellContext							{forall _T _U}
  \HaskellArgument							{(_T -> _U -> _T)}
  \HaskellArgument							{_T}
  \HaskellArgument							{[_U]}
  \HaskellResult							{_T}
\EndHaskellToken

\IndexTerm		{foldl}						{fold T U T T Us (T) ((\forall T U))}

\begin{haskell}
foldl {-[*]-} {-[*]-} {-[*]-} :: (_T -> _U -> _T) -> _T -> [_U] -> _T

foldl {-sem-}({-,,,-}f) {-sem-}(y) {-sem-}({-empty-}[])		= y
foldl {-sem-}({-,,,-}f) {-sem-}(y) {-sem-}(x:x_set)		= foldl f (f{--}(y, x)) x_set
\end{haskell}

This combinator is, of course, available as part of the standard Haskell library and serves the important rôle of combining list values into a single result. However, in this work, it is seldom used directly, other than through its implicit application in most list operations such as "sum", "product" and "composeList".

\end{section}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Additional/Lists}
\set{title}{List Operations}
\begin{section}

\BeginHaskellToken	{{-prefix-:-}}				{Void}	{}
  \HaskellContext							{forall _T}
  \HaskellCommand						{2}	{#1\WhiteSpace:\WhiteSpace#2}
  \HaskellArgument							{_T}
  \HaskellArgument							{[_T]}
  \HaskellResult							{[_T]}
\EndHaskellToken

\NewHaskellToken	{:}					{Op}	{:}

\BeginHaskellToken	{{-empty-}}				{Ord}	{\emptyset}
  \HaskellContext							{forall _T}
  \BeginHaskellList						{Void}	{}
  \EndHaskellList						{Void}	{}
  \HaskellResult							{[_T]}
\EndHaskellToken

\IndexCons		{{-prefix-:-}}					{: T Ts (Ts) ((\forall T))}
\IndexCons		{{-empty-}}					{\emptyset (Ts) ((\forall T))}

The compiler implementation presented in this work relies heavily on the standard polymorphic list type “"[_T]"”. Individual values of this type include the empty list "{-empty-}[]", together with nodes of the form “"x{-,,-}:{-,,-}x_set"\,\,”. Most often, however, list values are represented by an explicit syntax such as "[1, 2, 3]", "[1 .. 3]" or "[ {-,,,-} f{--}(x_k) | x_k <- x_set {-,-}]". The later form represents a list obtained by applying the function "f" to every element of some other list "x_set" and all of these notations are supported natively by the Haskell programming language. Further, to make many list comprehension expressions such as "[x * k | {-raw-}(x_k) <- x_set {-,-}]" clearer, we will write the standard Haskell function “zip” as an infix operator “"`zipList`"”. Intuitively, "a_set1 `zipList` a_set2" converts a pair of lists into a single list of pairs, in which each pair is assembled from the corresponding elements of its two operands, after truncating both lists to the length of the shorter argument. Its type signature assumes the following form:

\BeginHaskellToken	{zipList}				{Ord}	{}
  \HaskellContext							{forall _T, _U}
  \HaskellCommand						{2}	{#1~\vert~#2}  
  \HaskellArgument							{[_T]}
  \HaskellArgument							{[_U]}
  \HaskellResult							{[_T, _U]}
\EndHaskellToken

\NewHaskellToken	{`zipList`}				{Op}	{\,\vert\,}

\IndexTerm		{zipList}					{\vert Ts Us (Ts Us) ((\forall T U))}

\begin{code}
zipList {-[*]-} {-[*]-} :: [_T] -> [_U] -> [{-raw-}(_T, _U)]
\end{code}

\set{capture}{hidden}
\begin{code}
zipList a b = zip a b
\end{code}

The actual length of a finite Haskell list can be obtained using the standard function “"lengthOfList"”, which is introduced with the following type signature:

\BeginHaskellToken	{lengthOfList}				{Open}	{length}
  \HaskellContext							{forall _T}
  \HaskellArgument							{[_T]}
  \HaskellResult							{integer}
\EndHaskellToken

\IndexTerm		{lengthOfList}					{length Ts (integer) ((\forall T))}

\begin{code}
lengthOfList {-[*]-} :: [_T] -> Integer
\end{code}

\set{capture}{hidden}
\begin{code}
lengthOfList [] 	= 0
lengthOfList (x:xs)	= 1 + lengthOfList xs
\end{code}

\NewHaskellToken	{++}					{Bin}	{\concat}

Further, a concatenation of two lists is represented by the familiar notation “"x_set++y_set"\,\,”:

\BeginHaskellToken	{{-prefix-++-}}				{Ord}	{}
  \HaskellContext							{forall _T}
  \HaskellCommand						{2}	{#1~\concat~#2}
  \HaskellArgument							{[_T]}
  \HaskellArgument							{[_T]}
  \HaskellResult							{[_T]}
\EndHaskellToken

\IndexTerm		{{-prefix-++-}}					{\concat Ts Ts (Ts) ((\forall T))}

\begin{haskell}
{-[*]-} ++ {-[*]-} :: [_T] -> [_T] -> [_T]
\end{haskell}

The dual operation of deconstructing a list into two sub-lists is most commonly performed with the help of standard Haskell functions “"head"”, “"tail"”, “"takeFromList"” and “"dropFromList"”. In particular, given a non-empty list "x_set", "head{--}(x_set)" and "tail{--}(x_set)" return the list's first element and the remainder of the list, respectively,
while "last{--}(x_set)" and "init{--}(x_set)" return the final element and all elements preceding it. Further, "reverse{--}(x_set)" conveniently reverses the ordering of all elements in "x_set", so that "head{--}(reverse{--}(x_set))" is always equal to "last{--}(x_set)", unless "x_set" represents an empty list:

\BeginHaskellToken	{head}					{Open}	{head}
  \HaskellContext							{forall _T}
  \HaskellArgument							{[_T]}
  \HaskellResult							{_T}
\EndHaskellToken

\BeginHaskellToken	{tail}					{Open}	{tail}
  \HaskellContext							{forall _T}
  \HaskellArgument							{[_T]}
  \HaskellResult							{[_T]}
\EndHaskellToken

\BeginHaskellToken	{last}					{Open}	{last}
  \HaskellContext							{forall _T}
  \HaskellArgument							{[_T]}
  \HaskellResult							{_T}
\EndHaskellToken

\BeginHaskellToken	{init}					{Open}	{init}
  \HaskellContext							{forall _T}
  \HaskellArgument							{[_T]}
  \HaskellResult							{[_T]}
\EndHaskellToken

\BeginHaskellToken	{reverse}				{Open}	{reverse}
  \HaskellContext							{forall _T}
  \HaskellArgument							{[_T]}
  \HaskellResult							{[_T]}
\EndHaskellToken

\IndexTerm		{head}						{head Ts (T) ((\forall T))}
\IndexTerm		{tail}						{tail Ts (Ts) ((\forall T))}
\IndexTerm		{last}						{last Ts (T) ((\forall T))}
\IndexTerm		{init}						{init Ts (Ts) ((\forall T))}
\IndexTerm		{reverse}					{reverse Ts (Ts) ((\forall T))}
	
\begin{haskell}
head {-[*]-}, last {-[*]-} :: [_T] -> _T
tail {-[*]-}, init {-[*]-}, reverse {-[*]-} :: [_T] -> [_T]
\end{haskell}

None of these five functions is ever defined on an empty list "{-empty-}[]".

Finally, given an arbitrary list "x_set" and an integer "n", "takeFromList{--}(n, x_set)" retrieves the first "n" elements of "x_set", while "dropFromList{--}(n, x_set)" discards these elements, returning the remainder of "x_set". If "n" is 0 or negative, “"takeFromList"” will return an empty list while “"dropFromList"” will return its operand unchanged. Conversely, if "n" is greater than the length of "x_set", “"takeFromList"” will return "x_set" unchanged and “"dropFromList"” will return an empty list. The Haskell signatures of both functions are given as follows:

\BeginHaskellToken	{takeFromList}				{Open}	{take}
  \HaskellContext							{forall _T}
  \HaskellArgument							{(Integer, [_T])}
  \HaskellResult							{[_T]}
\EndHaskellToken

\BeginHaskellToken	{dropFromList}				{Open}	{drop}
  \HaskellContext							{forall _T}
  \HaskellArgument							{(Integer, [_T])}
  \HaskellResult							{[_T]}
\EndHaskellToken

\IndexTerm		{takeFromList}					{take integer Ts (Ts) ((\forall T))}
\IndexTerm		{dropFromList}					{drop integer Ts (Ts) ((\forall T))}

\begin{code}
takeFromList {-[*]-}, dropFromList {-[*]-} :: (Integer, [_T]) -> [_T]
\end{code}

\set{capture}{hidden}
\begin{code}
takeFromList (n, x:xs) | (n > 0) 	= x : takeFromList (n - 1, xs)
takeFromList (n, xs) 			= []

dropFromList (n, x:xs) | (n > 0) 	= dropFromList (n - 1, xs)
dropFromList (n, xs) 			= xs
\end{code}

Finally, in Section \ref{Lambda/Terms}, we rely on one more list combinator “"permutation{--}(k, x_set)"”, which returns the "k"th permutation of the list "x_set":

\BeginHaskellToken	{permutation}				{Open}	{\ZapfChancery{P}}
  \HaskellContext							{forall _T}
  \HaskellArgument							{(Integer, [_T])}
  \HaskellResult							{[_T]}
\EndHaskellToken

\IndexTerm		{permutation}					{P Ts (Ts)}

\set{continued}		{yes}
\begin{haskell}
permutation {-[*]-} :: (Integer, [_T]) -> [_T]
\end{haskell}

\BeginHaskellToken	{coerceList}				{Void}	{}
  \HaskellCommand						{1}	{}
  \HaskellArgument							{_T -> _U}
  \HaskellArgument							{[_T]}
  \HaskellResult							{[_U]}
\EndHaskellToken

\set{capture}		{hidden}
\begin{code}
coerceList :: (_T -> _U) -> [_T] -> [_U]

coerceList (f) (x_set) = [ f{--}(x_k) | x_k <- x_set ]
\end{code}

\end{section}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Additional/Sets}
\set{title}{Finite Sets}
\begin{section}

\BeginHaskellToken	{SetT}					{Ord}	{}
  \HaskellCommand						{1}	{\lbrace#1\rbrace}
  \HaskellArgument							{Type}
  \HaskellResult							{Type}
\EndHaskellToken

\NewHaskellToken	{emptySet}				{Ord}	{\emptyset}
\NewHaskellResult	{emptySet}					{(Ord _T) => SetT _T}

\IndexType		{SetT}						{\lbrace\rbrace \tstar (\tstar)}
\IndexTerm		{emptySet}					{\emptyset (Tss) ((ord T))}

\set{capture}{hidden}
\begin{code}
type SetT _T = Data.Set.Set _T

emptySet :: SetT _T
emptySet = Data.Set.empty
\end{code}

No treatment of algebraic data types would be complete without providing a convenient polymorphic implementation of «sets». Fortunately, the standard Haskell library comes complete with a very capable implementation of this data structure, in the form of the standard “Data.Set” module. In this work, the set of «n» ordered Haskell expressions \hbox{"x1, x2 .. x_n :: _T"} is represented by the familiar notation “"listToSet [x1, x2, x_n]"” and its type is written simply as “"SetT _T"”. In this work, I also recognise the notion of set comprehensions, writing them as expression of the form "listToSet [1 .. 7]" and even "listToSet [{-,,-}f{--}(x_k) | x_k <- setToList x_set]". Formally, such notation can be given a formal mandate by confusing the concepts of sets and lists whenever the context of the discussion provides sufficient disambiguation of the term, which gives rise to the following pair of implicit coercion functions:

\BeginHaskellToken	{listToSet}				{Void}	{}
  \HaskellContext							{(Ord _T)}
  \BeginHaskellAlt
    \BeginHaskellList						{Open}	{\lbrace}
    \EndHaskellList						{Close}	{\rbrace}
  \EndHaskellAlt
  \HaskellArgument							{[_T]}
  \HaskellResult							{SetT _T}
\EndHaskellToken

\BeginHaskellToken	{setToList}				{Void}	{}
  \HaskellContext							{(Ord _T)}
  \HaskellArgument							{SetT _T}
  \HaskellResult							{[_T]}
\EndHaskellToken

\IndexTerm		{listToSet}					{ Ts (Tss) ((ord T))}
\IndexTerm		{setToList}					{ Tss (Ts) ((ord T))}

\begin{code}
listToSet {-[*]-} :: (Ord _T) => 	[_T] 		-> SetT _T
setToList {-[*]-} :: {- (Ord _T) => -}	SetT _T 	-> [_T]
\end{code}

\set{capture}{hidden}
\begin{code}
listToSet (x_set) = Data.Set.fromList x_set
setToList (_A) = Data.Set.toList _A
\end{code}

The «cardinality» of a set, or the number of distinct elements stored within it, is represented by standard mathematical notation "cardinality(x_set)":

\BeginHaskellToken	{cardinality}				{Void}	{}
  \HaskellContext							{(Ord _T)}
  \BeginHaskellParens						{Open}	{\vert\,}
    \RawHaskellArgument							{SetT _T}
  \EndHaskellParens						{Close}	{\,\vert}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{cardinality}					{\vert\vert Tss (integer) ((ord T))}

\begin{code}
cardinality {- ({-[*]-}) -} :: {- (Ord _T) => -} SetT _T -> Integer
\end{code}

\set{capture}{hidden}
\begin{code}
cardinality (_A) = toInteger (Data.Set.size _A)
\end{code}

The Haskell library also provides an efficient direct access to the smallest and greatest element of a set of ordered values using the following pair of functions:

\BeginHaskellToken	{minimumOfSet}				{Open}	{min}
  \HaskellContext							{(Ord _T)}
  \HaskellArgument							{SetT (_T{-sub-}Ord)}
  \HaskellResult							{_T}
\EndHaskellToken

\BeginHaskellToken	{maximumOfSet}				{Open}	{max}
  \HaskellContext							{(Ord _T)}
  \HaskellArgument							{SetT (_T{-sub-}Ord)}
  \HaskellResult							{_T}
\EndHaskellToken

\IndexTerm		{minimumOfSet}					{min Tss (T) ((ord T))}
\IndexTerm		{maximumOfSet}					{max Tss (T) ((ord T))}

\begin{code}
minimumOfSet {-[*]-}, maximumOfSet {-[*]-} :: {- (Ord _T) => -} SetT _T -> _T
\end{code}

\set{capture}{hidden}
\begin{code}
minimumOfSet (x) = Data.Set.findMin x
maximumOfSet (x) = Data.Set.findMax x
\end{code}

Most often, however, we are merely concerned with the question of set membership. Given a value "x" and a set "x_set", the Haskell expression “"x @@ x_set"\,\,” returns “"True"” if "x" can be found in the supplied set and “"False"” otherwise. This common operator (and its dual “"/@"”) are readily available in the standard Haskell library:

\BeginHaskellToken	{{-prefix-@@-}}				{Ord}	{}
  \HaskellContext							{(Ord _T)}
  \HaskellCommand						{2}	{#1~\in~#2}
  \HaskellArgument							{_T}
  \HaskellArgument							{SetT _T}
  \HaskellResult							{Bool}
\EndHaskellToken

\BeginHaskellToken	{{-prefix-/@-}}				{Ord}	{}
  \HaskellContext							{(Ord _T)}
  \HaskellCommand						{2}	{#1~\notin~#2}
  \HaskellArgument							{_T}
  \HaskellArgument							{SetT _T}
  \HaskellResult							{Bool}
\EndHaskellToken

\NewHaskellToken	{@@}					{Rel}	{\in}
\NewHaskellToken	{/@}					{Rel}	{\notin}

\IndexTerm		{{-prefix-@@-}}					{\in T Tss (bool) ((ord T))}
\IndexTerm		{{-prefix-/@-}}					{\notin T Tss (bool) ((ord T))}

\begin{haskell}
{-[*]-} @@ {-[*]-}, {-[*]-} /@ {-[*]-} :: (Ord _T) => _T -> SetT _T -> Bool
\end{haskell}

\set{capture}{hidden}
\begin{code}
(@@), (/@) :: Ord _T => _T -> SetT _T -> Bool

(x) @@ (_A)  	= Data.Set.member x _A
(x) /@ (_A)  	= not (Data.Set.member x _A)
\end{code}

Similarly, the subset operator “"`subseteq`"” and its strict variant “"`subset`"” have the following Haskell declarations:

\BeginHaskellToken	{subset}				{Void}	{}
  \HaskellContext							{(Ord _T)}
  \HaskellArgument							{SetT _T}
  \HaskellToken		{{-subset-}}				{Rel}	{\subset}
  \HaskellArgument							{SetT _T}
  \HaskellResult							{Bool}
\EndHaskellToken

\BeginHaskellToken	{subseteq}				{Void}	{}
  \HaskellContext							{(Ord _T)}
  \HaskellArgument							{SetT _T}
  \HaskellToken		{{-subseteq-}}				{Rel}	{\subseteq}
  \HaskellArgument							{SetT _T}
  \HaskellResult							{Bool}
\EndHaskellToken

\NewHaskellToken	{{-subset-}}				{Rel}	{\subset}
\NewHaskellToken	{{-subseteq-}}				{Rel}	{\subseteq}

\NewHaskellToken	{`subset`}				{Rel}	{\subset}
\NewHaskellToken	{`subseteq`}				{Rel}	{\subseteq}

\IndexTerm		{subset}					{\subset Tss Tss (bool) ((ord T))}
\IndexTerm		{subseteq}					{\subseteq Tss Tss (bool) ((ord T))}

\begin{code}
subset {-[*]-} {-subset-} {-[*]-}, subseteq {-[*]-} {-subseteq-} {-[*]-} :: (Ord _T) => SetT _T -> SetT _T -> Bool
\end{code}

\set{capture}{hidden}
\begin{code}
subset		(_A) {-subset-} (_B) = Data.Set.isProperSubsetOf _A _B
subseteq	(_A) {-subseteq-} (_B) = Data.Set.isSubsetOf _A _B
\end{code}

Further, a union, intersection and difference of two sets can be obtained using the standard mathematical notations “"x_set1 `union` x_set2"\!”,  “"x_set1 `intersection` x_set2"\!” and “"x_set1 {-!-} `difference` x_set2"\!”, respectively. They are implemented by the following Haskell functions:

\BeginHaskellToken	{union}					{Ord}	{}
  \HaskellContext							{(Ord _T)}
  \HaskellCommand						{2}	{#1~\cup~#2}
  \HaskellArgument							{SetT _T}
  \HaskellArgument							{SetT _T}
  \HaskellResult							{SetT _T}
\EndHaskellToken

\BeginHaskellToken	{intersection}				{Ord}	{}
  \HaskellContext							{(Ord _T)}
  \HaskellCommand						{2}	{#1~\cap~#2}
  \HaskellArgument							{SetT _T}
  \HaskellArgument							{SetT _T}
  \HaskellResult							{SetT _T}
\EndHaskellToken

\BeginHaskellToken	{difference}				{Ord}	{}
  \HaskellContext							{(Ord _T)}
  \HaskellCommand						{2}	{#1~\setminus~#2}
  \HaskellArgument							{SetT _T}
  \HaskellArgument							{SetT _T}
  \HaskellResult							{SetT _T}
\EndHaskellToken

\NewHaskellToken	{`union`}				{Bin}	{\cup}
\NewHaskellToken	{`intersection`}			{Bin}	{\cap}
\NewHaskellToken	{`difference`}				{Op}	{\setminus}

\IndexTerm		{union}						{\cup Tss Tss (Tss) ((ord T))}
\IndexTerm		{intersection}					{\cap Tss Tss (Tss) ((ord T))}
\IndexTerm		{difference}					{\setminus Tss Tss (Tss) ((ord T))}

\begin{code}
union {-[*]-} {-[*]-}, intersection {-[*]-} {-[*]-}, difference {-[*]-} {-[*]-} :: (Ord _T) => SetT _T -> SetT _T -> SetT _T
\end{code}

\set{capture}{hidden}
\begin{code}
(_A) 	`union` 	(_B) 	= Data.Set.union _A _B
(_A) 	`intersection` 	(_B) 	= Data.Set.intersection _A _B
(_A)	`difference` 	(_B) 	= Data.Set.difference _A _B

\end{code}

Similarly, the cross-product of two sets can be obtained using the following standard notation “"x_set1 `crossProduct` x_set2"\!”:

\BeginHaskellToken	{crossProduct}				{Ord}	{}
  \HaskellContext							{(Ord _T, Ord _U)}
  \HaskellCommand						{2}	{#1~\times~#2}
  \HaskellArgument							{SetT _T}
  \HaskellArgument							{SetT _U}
  \HaskellResult							{SetT (_T, _U)}
\EndHaskellToken

\NewHaskellToken	{`crossProduct`}			{Bin}	{\times}

\IndexTerm		{crossProduct}					{\times Tss Uss (Tss Uss) ((ord T ord U))}

\begin{haskell}
crossProduct {-[*]-} {-[*]-} :: (Ord _T, Ord _U) => SetT _T -> SetT _U -> SetT (_T, _U)
\end{haskell}

\set{capture}{hidden}
\begin{code}
crossProduct (_A) (_B) = listToSet [ (a, b) | a <- setToList _A, b <- setToList _B ]
\end{code}

Last but not least, the “"`union`"” and “"`intersection`"” operators may be also applied wholesale to an entire list of sets:

\BeginHaskellToken	{unions}				{Open}	{\hbox{\raise1.7ex\vtop{\offinterlineskip\vskip0pt\bigcup}}}
  \HaskellContext							{(Ord _T)}
  \HaskellArgument							{[SetT _T]}
  \HaskellResult							{SetT _T}
\EndHaskellToken

\BeginHaskellToken	{intersections}				{Open}	{\hbox{\raise1.7ex\vtop{\offinterlineskip\vskip0pt\bigcap}}}
  \HaskellContext							{(Ord _T)}
  \HaskellArgument							{[SetT _T]}
  \HaskellResult							{SetT _T}
\EndHaskellToken

\IndexTerm		{unions}					{\bigcap Tss (Tss) ((ord T))}
\IndexTerm		{unions}					{\bigcup Tss (Tss) ((ord T))}

\begin{haskell}
unions {-[*]-}, intersections {-[*]-} :: (Ord _T) => [SetT _T] -> SetT _T
\end{haskell}

\set{capture}{hidden}
\begin{code}
unions (_A_set) = foldl (union) (emptySet) _A_set
intersections (_A_set) = foldl (intersection) (unions _A_set) _A_set
\end{code}

Highly optimised implementations of the above set operations are readily available in the standard Haskell library. However, the reader should observe that most of the set functions described in this section are defined only for types that are a member of the Haskell class “"Ord"”. When no sensible total ordering is available for a type whose sets are constructed by our compiler implementation, the reader should assume that an adequate instance of the “"Ord"” class has been constructed implicitly using the standard instance derivation mechanism supported by the language.

\end{section}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Additional/Maps}
\set{title}{Finite Maps}
\begin{section}

\BeginHaskellToken	{MapT}					{Ord}	{}
  \HaskellCommand						{2}	{#1~\mapsto~#2}
  \HaskellArgument							{Type}
  \HaskellArgument							{Type}
  \HaskellResult							{Type}
\EndHaskellToken

\NewHaskellToken	{emptyMap}				{Ord}	{\emptyset}
\NewHaskellResult	{emptyMap}					{forall _U => (Ord _T) => (MapT _T _U)}

\IndexType		{MapT}						{\mapsto \tstar \tstar (\tstar)}
\IndexTerm		{emptyMap}					{\emptyset (TUss) ((ord T \forall U))}

\set{capture}{hidden}
\begin{code}
type MapT _T _U = Data.Map.Map _T _U

emptyMap :: MapT _T _U
emptyMap = Data.Map.empty
\end{code}

A «finite map» is a unary function defined over a finite domain of distinct values in such a way that it can be «extended» with additional definitions during execution of the program. The standard Haskell library conveniently provides us with an efficient implementation of such constructs for ordered domain types, in the form of the “Data.Map” module. In this work, the type of a map from an ordered domain type "_T" to "_U" is written as “"MapT _T _U"”. The most primitive value of such a type is the «empty map» "emptyMap", whose domain is guaranteed to be empty. Other map values may be constructed using the familiar set comprehension notation such as "listToMap [x1:y1, x2:y2, x3:y3]" and "listToMap [ {-,,,-} f{--}(x_k, y_k) | x_k:y_k <- _M]", in which every value "x_k" represents an element of the resulting map's domain. In the resulting map, each "x_k" is bound to the corresponding value "y_k". Similarly to the earlier set type, these notations are mandated by the following pair of implicit coercions between finite maps and lists of tuples:

\BeginHaskellToken	{listToMap}				{Void}	{}
  \HaskellContext							{forall _U => (Ord _T)}
  \BeginHaskellAlt
    \BeginHaskellList						{Open}	{\lbrace}
    \EndHaskellList						{Close}	{\rbrace}
  \EndHaskellAlt
  \HaskellArgument							{[_T, _U]}
  \HaskellResult							{(MapT _T _U)}
\EndHaskellToken

\BeginHaskellToken	{mapToList}				{Void}	{}
  \HaskellContext							{forall _U => (Ord _T)}
  \HaskellArgument							{(MapT _T _U)}
  \HaskellResult							{[_T, _U]}
\EndHaskellToken

\IndexTerm		{listToMap}					{ TUs (TUsss) ((ord T \forall U))}
\IndexTerm		{mapToList}					{ TUssss (TUs) ((ord T \forall U))}

\begin{code}
listToMap {-[*]-} :: (Ord _T) => 	[{-raw-}(_T, _U)] -> (MapT _T _U)
mapToList {-[*]-} :: {- (Ord _T) => -} 	(MapT _T _U) -> [{-raw-}(_T, _U)]
\end{code}

\set{capture}{hidden}
\begin{code}
listToMap (x_set) = Data.Map.fromList (x_set)
mapToList (_M) = Data.Map.toList _M
\end{code}

The cardinality or size of a finite map is depicted by the standard mathematical notation "mapCardinality(_M)", with a type similar to the corresponding set operation:

\BeginHaskellToken	{mapCardinality}			{Void}	{}
  \HaskellContext							{forall _U => (Ord _T)}
  \BeginHaskellParens						{Open}	{\vert\,}
    \RawHaskellArgument							{(MapT _T _U)}
  \EndHaskellParens						{Close}	{\,\vert}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{mapCardinality}				{\vert\vert TUsss (integer) ((ord T \forall U))}

\begin{code}
mapCardinality {- ({-[*]-}) -} :: {- Ord _T => -} (MapT _T _U) -> Integer
\end{code}

\set{capture}{hidden}
\begin{code}
mapCardinality (_M) = toInteger (Data.Map.size _M)
\end{code}

A set of all elements present in the domain or codomain of a given finite map "_M" is represented by the respective notations “"dom{--}(_M)"” and “"codom{--}(_M)"”. A native implementation of both constructions is provided in the standard library as the following pair of Haskell functions:

\BeginHaskellToken	{dom}					{Open}	{dom}
  \HaskellContext							{forall _U => (Ord _T)}
  \HaskellArgument							{(MapT _T _U)}
  \HaskellResult							{SetT _T}
\EndHaskellToken

\BeginHaskellToken	{codom}					{Open}	{codom}
  \HaskellContext							{forall _T => (Ord _U)}
  \HaskellArgument							{(MapT _T _U)}
  \HaskellResult							{SetT _U}
\EndHaskellToken

\IndexTerm		{dom}						{dom TUsss (Tss) ((ord T \forall U))}
\IndexTerm		{codom}						{codom TUsss (Uss) ((ord U \forall T))}

\begin{haskell}
dom 	{-[*]-} :: (Ord {-hfill-}_T  ) => (MapT _T _U) -> listToSet [  {-hfil-}_T  ]
codom 	{-[*]-} :: (Ord {-hfill-}_U  ) => (MapT _T _U) -> listToSet [  {-hfil-}_U  ]
\end{haskell}

\set{capture}{hidden}
\begin{code}
dom 	{-[*]-} :: (Ord {-hfill-}_T  ) => (MapT _T _U) -> SetT (  {-hfil-}_T  )
codom 	{-[*]-} :: (Ord {-hfill-}_U  ) => (MapT _T _U) -> SetT (  {-hfil-}_U  )

dom (_M) = Data.Map.keysSet _M
codom (_M) = listToSet (Data.Map.elems _M)
\end{code}

In practice, finite maps are generally considered to represent partial functions. In this work, an application of a map "m_set" to a value "x" from the map's domain is typeset as like an ordinary function application "m_set{--}(x)". In the real life of Haskell, such map applications are executed by a function with the following type signature:

\BeginHaskellToken	{{-prefix-\#-}}				{Void}	{}
  \HaskellContext							{forall _U => (Ord _T)}
  \HaskellArgument							{(MapT _T _U)}
  \HaskellArgument							{_T}
  \HaskellResult							{_U}
\EndHaskellToken

\NewHaskellToken	{\#}					{Op}	{}

\IndexTerm		{{-prefix-\#-}}					{ TUsss T (U) ((ord T \forall U))}

\begin{haskell}
{-[*]-} {-[*]-} :: (Ord _T) => (MapT _T _U) -> _T -> _U
\end{haskell}

\set{capture}{hidden}
\begin{code}
(#) :: (Ord _T) => (MapT _T _U) -> _T -> _U
_M#(x) = unJust (Data.Map.lookup (x) (_M))
\end{code}

To complete the partial function illusion, we can also define the standard composition operator “"`composeMap`"” on finite maps as follows:

\BeginHaskellToken	{composeMap}				{Ord}	{}
  \HaskellContext							{forall _V => (Ord _T, Ord _U)}
  \HaskellCommand						{2}	{#1~\circle~#2}
  \HaskellArgument							{(MapT _U _V)}
  \HaskellArgument							{(MapT _T _U)}
  \HaskellResult							{(MapT _T _V)}
\EndHaskellToken

\NewHaskellToken	{`composeMap`}				{Bin}	{\circle}

\IndexTerm		{composeMap}					{\circle UVsss TUsss (TVsss) ((ord T ord U \forall V))}

\begin{code}
composeMap {-[*]-} {-[*]-} :: (Ord _T, Ord _U) => (MapT _U _V) -> (MapT _T _U) -> (MapT _T _V)

composeMap ({-sem-}(_M)) ({-sem-}(_N)) = listToMap [ {-raw-}({-,-} x {-,,-} {-:-}, {-,,-} _M#(y)) | (x {-,,-} {-:-}, {-,,-}y) <- mapToList _N, y @@ dom{--}(_M) ]
\end{code}

Given two or more finite maps with disjoint domain sets, a new map that consists of all the bindings found in any of the supplied operands can be constructed with one of the following familiar union operators:

\BeginHaskellToken	{mapUnion}				{Ord}	{}
  \HaskellContext							{forall _U => (Ord _T)}
  \HaskellCommand						{2}	{#1~\cup~#2}
  \HaskellArgument							{(MapT _T _U)}
  \HaskellArgument							{(MapT _T _U)}
  \HaskellResult							{(MapT _T _U)}
\EndHaskellToken

\NewHaskellToken	{`mapUnion`}				{Bin}	{\cup}

\IndexTerm		{mapUnion}					{\cup TUsss TUsss (TUsss) ((ord T \forall U))}

\begin{code}
mapUnion {-[*]-} {-[*]-} :: (Ord _T) => (MapT _T _U) -> (MapT _T _U) -> (MapT _T _U)
\end{code}

\set{capture}{hidden}
\begin{code}
mapUnion ((_A)) ((_B))	| (dom{--}(_A) `intersection` dom{--}(_B) == emptySet) = Data.Map.union _A _B
\end{code}

\BeginHaskellToken	{mapUnions}				{Open}	{\hbox{\raise1.6ex\vtop{\offinterlineskip\vskip0pt\bigcup}}}
  \HaskellContext							{forall _U => (Ord _T)}
  \HaskellArgument							{[MapT _T _U]}
  \HaskellResult							{(MapT _T _U)}
\EndHaskellToken

\IndexTerm		{mapUnions}					{\bigcup TUsss (TUsss) ((ord T \forall U))}

\begin{code}
mapUnions {-[*]-} :: (Ord _T) => [MapT _T _U] -> (MapT _T _U)
\end{code}

\set{capture}{hidden}
\begin{code}
mapUnions (_L) = foldl (mapUnion) (emptyMap) _L
\end{code}

Both operations are explicitly left undefined on arguments with overlapping domains. When combining such overlapping finite maps, a different kind of a binary map operator, known as a «map extension» or a «right-biased union», is required. In this work, an extension of a given map "m_set1" by "m_set2" is represented by the notation “"m_set1 // m_set2"\!”. Formally, it is implemented in Haskell as follows:

\BeginHaskellToken	{{-prefix-//-}}				{Void}	{}
  \HaskellContext							{forall _U => (Ord _T)}
  \HaskellCommand						{2}	{#1~\slash~#2}
  \HaskellArgument							{(MapT _T _U)}
  \HaskellArgument							{(MapT _T _U)}
  \HaskellResult							{(MapT _T _U)}
\EndHaskellToken

\NewHaskellToken	{//}					{Op}	{\slash}

\IndexTerm		{{-prefix-//-}}					{\slash TUsss TUsss (TUsss) ((ord T \forall U))}

\begin{haskell}
{-prefix-//-} {-[*]-} {-[*]-} :: (Ord _T) => (MapT _T _U) -> (MapT _T _U) -> (MapT _T _U)

{-prefix-//-} ({-sem-}(_M)) ({-sem-}(_N)) = (_M `mapSetDifference` dom{--}(_N)) `mapUnion` _N
\end{haskell}

\set{capture}{hidden}
\begin{code}
(//) :: Ord _T => (MapT _T _U) -> (MapT _T _U) -> (MapT _T _U)
_M // _N = Data.Map.unionWith (\ x1 x2 -> x2) (_M) (_N)
\end{code}

In other words, "_M//_N" maps every element "x @@ dom{--}(_N)" to "_N{--}(x)" and every element "y /@ dom{--}(_N)" to "_M{--}(y)". Intuitively, the new bindings introduced by the second operand of “\!"//"” override any existing mappings from "_M". This popular map operation is critical to a successful implementation of most symbol tables encountered in our compiler.

A careful reader will observe that, in the above definition, the map difference operator “"`mapSetDifference`"\!” was applied with a set, rather than a map as its second argument. In fact, it is often convenient to define a similar asymmetric map-set intersection operator “"`mapSetIntersection`"”. Both of these functions are implemented in the standard library with the following obvious type signatures:

\BeginHaskellToken	{mapSetIntersection}			{Void}	{}
  \HaskellContext							{forall _U => (Ord _T)}
  \HaskellArgument							{(MapT _T _U)}
  \HaskellToken		{{-cap-}}				{Bin}	{\cap}
  \HaskellArgument							{SetT _T}
  \HaskellResult							{(MapT _T _U)}
\EndHaskellToken

\BeginHaskellToken	{mapSetDifference}			{Void}	{}
  \HaskellContext							{forall _U => (Ord _T)}
  \HaskellArgument							{(MapT _T _U)}
  \HaskellToken		{{-setminus-}}				{Bin}	{\setminus}
  \HaskellArgument							{SetT _T}
  \HaskellResult							{(MapT _T _U)}
\EndHaskellToken

\NewHaskellToken	{`mapSetIntersection`}			{Bin}	{\cap}
\NewHaskellToken	{`mapSetDifference`}			{Op}	{\setminus}

\IndexTerm		{mapSetIntersection}				{\cap TUsss Tss (TUsss) ((ord T \forall U))}
\IndexTerm		{mapSetDifference}				{\setminus TUsss Tss (TUsss) ((ord T \forall U))}

\begin{haskell}
{-[*]-} `mapSetIntersection` {-[*]-}, {-[*]-} `mapSetDifference` {-[*]-} :: (Ord _T) => (MapT _T _U) -> SetT _T -> (MapT _T _U)
\end{haskell}

\set{capture}{hidden}
\begin{code}
mapSetIntersection {-[*]-} {-cap-} {-[*]-}, mapSetDifference {-[*]-} {-setminus-} {-[*]-} :: (Ord _T) => (MapT _T _U) -> SetT _T -> (MapT _T _U)
mapSetIntersection 	((  _A  )) 	((  _B  ))	= Data.Map.filterWithKey (\ x y -> x @@ _B) _A
mapSetDifference	((  _A  )) 	((  _B  ))	= Data.Map.filterWithKey (\ x y -> x /@ _B) _A
\end{code}

As for finite sets, a complete implementation of every data type and function described in this section is readily available in the standard Haskell module “Data.Map”. Since the art of purely functional data type constructions is not the primary subject of this work, I refer an interested reader to the existing relevant literature for a further discussion of the topic [Adams93].

\end{section}
\end{appendix}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%