\set{capture}{hidden}
\begin{code}

module C (
	module C.Variables,
	module C.Syntax,
	module C.Types,
	module C.Scopes,
	module C.Constants,
	module C.Expressions,
	module C.Declarations,
	module C.Statements,
	module C.Definitions,
	module C.Implementation,
) where

import C.Syntax
import C.Variables
import C.Types
import C.Scopes
import C.Constants
import C.Expressions
import C.Declarations
import C.Statements
import C.Definitions
import C.Implementation

\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C}
\set{title}		{The C Programming Language}
\set{citation}		{Philosophy is a battle against the bewitchment\break
               		 of our intelligence by means of language.}
\set{citation author}	{Ludwig Wittgenstein}
\begin{chapter}

\vfill

The C programming language was originally developed by Kernighan and Ritchie at Bell Labs in 1970s, as part of their work on the UNIX operating system for the \hbox{PDP-7} architecture [Kernighan78]. It provides a quintessential imperative design environment in the ALGOL tradition, whose wide availability, simplicity and access to low-level programming features has rendered it quickly into the favourite tool of the UNIX community from the 1980s. Since its introduction, the user base of C has been increasing steadily across the whole spectrum of the software development industry and, today, it arguably remains the single most popular programming language in existence. To meet the growing demand for portability, C has been ratified into an international standard ANSI/ISO 9899:1990 and further refined in 2000 by the same regulatory group [ANSI89, ANSI99].

In recent years, C saw a growing deployment in software written for embedded devices and, with these developments, a pressing need for a formal verification of C programs. Such verification necessarily involves a mathematically rigorous definition of the language. In the past, numerous authors have undertaken the task of devising a formal semantic model of C, with the most recent, successful and complete treatments given by Norrish and Papaspyrou [Norrish97, Papaspyrou98]. Nevertheless, to the best of my knowledge, none of these models have been ever incorporated into a fully featured compiler for the language and, accordingly, in this chapter I revisit the task of formalising Standard C by presenting a complete and detailed translation of its programs into Etude, the variant of lambda calculus defined in Chapter \ref{Lambda}. To make the work more specific, I focus the project on the original version of the Standard described in the ANSI/ISO 9899:1990 document [ANSI89]. The formal definition presented in this chapter covers all aspects of C programs with the sole exception of their lexical and syntactic analysis, which are omitted in recognition of the already-extensive treatment given to these subjects in literature [Aho86, Appel98ML]. Coupled with the partial Etude semantics presented earlier in Chapter \ref{Lambda}, the translation formalises precisely the set of requirements described in the C Standard [ANSI89]. Further, in combination with the mapping of Etude programs to executable machine instructions presented in Chapter \ref{MMIX}, it also forms a prototype compiler implementation, that, although inefficient as given (preferring clarity of presentation over optimality of the generated code), could be easily improved into an industrial-quality development environment, while ensuring that the result always remains provably equivalent to its present canonical definition.

Throughout this chapter, I carefully avoid any over-specification of those aspects of the source language that the C Standard leaves either unspecified or open to further specification by individual compiler vendors. Following the principles discussed earlier in Section \ref{Lambda/Params}, all such linguistic features are described without a concrete implementation in terms of explicit «language parameters», whose minimal semantic requirements are captured in appropriate algebraic theorems. For example, in Section \ref{C/Types/Representation}, I formalise the precise representation of C types in terms of a function "formatOfType", together with a set of 42 theorems that describe the minimal constraints on its definition which must be satisfied by every instance of the language. A typical implementation of "formatOfType" is given separately in Appendix \ref{C/Implementation}, together with definitions of all other implementation-specific language parameters that are introduced throughout this chapter.

The reader should also keep in mind that the resulting formal semantics of C rely fundamentally on the Etude program representation given in Chapter \ref{Lambda}, and especially on the generic fragment of its semantics presented in Sections \ref{Lambda/Atoms}, \ref{Lambda/State} and \ref{Lambda/Terms}. In particular, it should be emphasised that many aspects of C are rendered implementation-defined, unspecified or undefined indirectly through translation into Etude programs with corresponding properties. For example, all C expressions to whom the C Standard does not attribute a predetermined evaluation order are translated into Etude "LET" group bindings, for which theorems "wfLetListTermJ" and "eqvLetListTermJ" from Section \ref{Lambda/Terms} provide only a very rudimentary set of operational guarantees. The minimal documentation requirements mandated by the C Standard for all such indirectly-defined aspects of the language are presented in Section \ref{Lambda/Arch}, in the form of a concrete operational semantics for Etude on the MMIX instruction set architecture.

\InputFile		{C/Overview}
\InputFile		{C/Notation}
\InputFile		{C/Variables}
\InputFile		{C/Syntax}
\InputFile		{C/Types}
\InputFile		{C/Scopes}
\InputFile		{C/Constants}
\InputFile		{C/Expressions}
\InputFile		{C/Declarations}
\InputFile		{C/Statements}
\InputFile		{C/Definitions}
\InputFile		{C/Example}

\end{chapter}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%