\set{capture}{hidden}
\begin{code}

module C.Overview (
	simple_example
) where

import Additional
import Lambda

x, r, i, t, t1, t2, t3 :: String

x = "x"
r = "r"
i = "i"
t = "t"
t1 = "t1"
t2 = "t2"
t3 = "t3"
s1 = "s2"
s2 = "s2"

fac_, fac1_, fac2_, fac'_, fac1'_, fac2'_ :: String
fac_ = "fac"
fac1_ = "fac1"
fac2_ = "fac2"
fac'_ = "fac'"
fac1'_ = "fac1'"
fac2'_ = "fac2'"

simple_example = MODULE (RET [])
	{-EXPORT-} (listToMap [ {-raw-}("fac" {-=-}, fac'_) ])
	{-WHERE-}  (listToMap [ {-raw-}(fac'_ {-=-}, FUNI fac'), {-raw-}(fac1'_ {-=-}, FUNI fac1'), {-raw-}(fac2'_ {-=-}, FUNI fac2') ])

\end{code}

\NewHaskellToken	{fac1\_}				{Ord}	{«fac\sub{1}»}
\NewHaskellToken	{fac2\_}				{Ord}	{«fac\sub{2}»}
\NewHaskellToken	{fac1'\_}				{Ord}	{«fac\vprime\sub{2}»}
\NewHaskellToken	{fac2'\_}				{Ord}	{«fac\vprime\sub{2}»}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Overview}
\set{title}		{Overview}
\begin{section}

Under the formal semantic model of C presented in this chapter, every well-formed translation unit is represented naturally by an Etude module. Global and static variables are mapped to mutable object items in the constructed program, while all C functions and statement labels are rendered as Etude functions. To illustrate the intended behaviour of this translation, let us consider the following simple C program that computes the factorial of an integer "x":

\begin{C}
int fac(int x) {
	int r, i;
	r  = 1;
	i  = 1;
	while (i <= x) {
		r  *=  i;
		i  +=  1;
	}
	return r;
}
\end{C}

This program could be mapped into the following set of three Etude functions:

\begin{code}
fac	{-hv-}= FUN [x] {-DOT-} $ {-quad-} ---
			LET [	{-raw-}[r]  =: {-hide-}RET {-raw-}[IMM (1) (ZG`FMT`standardEncoding)]] $
			LET [	{-raw-}[i]  =: {-hide-}RET {-raw-}[IMM (1) (ZG`FMT`standardEncoding)]] $
			APP (VAR fac1_) [VAR x {-hv-}, VAR r {-hv-}, VAR i]

fac1	{-hv-}= FUN [x, r, i] {-DOT-} $	{-quad-} ---
			IFF (OP2 (VAR i) (CLE) (ZG`FMT`standardEncoding) (VAR x)) {-THEN-} (
			{-quad-}	LET [	{-raw-}[r]  =: {-hide-}RET {-raw-}[OP2 (VAR r)  (MUL)  (ZG`FMT`standardEncoding) (VAR i)]] $
			     		LET [	{-raw-}[i]  =: {-hide-}RET {-raw-}[OP2 (VAR i)  (ADD)  (ZG`FMT`standardEncoding) (IMM (1) (ZG`FMT`standardEncoding))]] $
			     		APP (VAR fac1_) [VAR x {-hv-}, VAR r {-hv-}, VAR i])
			{-ELSE-} (
			{-quad-}	APP (VAR fac2_) [VAR x {-hv-}, VAR r {-hv-}, VAR i])

fac2	{-hv-}= FUN [x, r, i] {-DOT-} $	{-quad-} ---
			RET [VAR r]
\end{code}

Under this mapping, every C function and statement label (including the implicit “\CKeyword{continue}” and “\CKeyword{break}” labels defined by every iteration statement) is represented by an Etude function which acccepts as its parameters the list of all local C variables that are visible within its scope. Jumps are translated into Etude tail calls and function calls are translated into monadic bindings of appropriate Etude application expressions.

While this natural translation works for the current elementary example, in most cases Etude variables are not a very good match for their C counterparts, since they cannot be accessed indirectly through pointer references in the way that C variables can. In a more accurate translation, every C variable should denote a mutable Etude object whose content corresponds to the variable's value. Accordingly, the translation defined in this chapter produces a representation of “\C{fac}” akin to the following \hbox{Etude program:}

\begin{code}
fac'	{-hv-}= FUN [x]	{-DOT-} $ {-quad-} ---
			LET [	{-raw-}[r]   =: NEW  (listToSet {-raw-}[{-raw-}(0 {-hv-}, ZG`FMT`standardEncoding {-hide-}, {-hide-}emptySet)]) ] $
			LET [	{-raw-}[i]   =: NEW  (listToSet {-raw-}[{-raw-}(0 {-hv-}, ZG`FMT`standardEncoding {-hide-}, {-hide-}emptySet)]) ] $
			LET [	{-hv-}[]     =: SET  (VAR r {-hide-},{-hide-}emptySet  ) (ZG`FMT`standardEncoding) {-TO-} (IMM (1) (ZG`FMT`standardEncoding)) ] $
			LET [	{-hv-}[]     =: SET  (VAR i {-hide-},{-hide-}emptySet  ) (ZG`FMT`standardEncoding) {-TO-} (IMM (1) (ZG`FMT`standardEncoding)) ] $
			APP (VAR fac1'_) [VAR x {-hv-}, VAR r {-hv-}, VAR i]

fac1'	{-hv-}= FUN [x, r, i] {-DOT-} $	{-quad-} ---
			LET [	{-raw-}[t1]  =: GET  (VAR i {-hide-},{-hide-}emptySet  ) (ZG`FMT`standardEncoding) {-hv-},
				{-raw-}[t2]  =: GET  (VAR x {-hide-},{-hide-}emptySet  ) (ZG`FMT`standardEncoding) ] $
			IFF (OP2 (VAR t1) (CLE) (ZG`FMT`standardEncoding) (VAR t2)) {-THEN-} (
			{-quad-}	 LET [  {-raw-}[t1]  =: GET  (VAR r {-hide-},{-hide-}emptySet  ) (ZG`FMT`standardEncoding) {-hv-},
				 		{-raw-}[t2]  =: GET  (VAR i {-hide-},{-hide-}emptySet  ) (ZG`FMT`standardEncoding) ] $
					 LET [	{-hv-}[]     =: SET  (VAR r {-hide-},{-hide-}emptySet  ) (ZG`FMT`standardEncoding) --
					 					{-TO-} (OP2 (VAR t1) (MUL) (ZG`FMT`standardEncoding) (VAR t2)) ] $
					 LET [	{-raw-}[t1]  =: GET  (VAR i {-hide-},{-hide-}emptySet  ) (ZG`FMT`standardEncoding) ] $
					 LET [	{-hv-}[]     =: SET  (VAR i {-hide-},{-hide-}emptySet  ) (ZG`FMT`standardEncoding) --
					 					{-TO-} (OP2 (VAR t1) (ADD) (ZG`FMT`standardEncoding) (IMM (1) (ZG`FMT`standardEncoding))) ] $
	     				 APP (VAR fac1'_) [VAR x {-hv-}, VAR r {-hv-}, VAR i])
			{-ELSE-} (
			{-quad-}	 APP (VAR fac2'_) [VAR x {-hv-}, VAR r {-hv-}, VAR i])

fac2'	{-hv-}= FUN [x, r, i] {-DOT-} $ {-quad-} ---
			LET [	{-raw-}[t1]  =: GET  (VAR r {-hide-},{-hide-}emptySet  ) (ZG`FMT`standardEncoding) ] $
			LET [	{-hv-}[]     =: DEL  (listToSet {-raw-}[{-raw-}(0, ZG`FMT`standardEncoding {-hide-}, {-hide-}emptySet)]) ] $
			LET [	{-hv-}[]     =: DEL  (listToSet {-raw-}[{-raw-}(0, ZG`FMT`standardEncoding {-hide-}, {-hide-}emptySet)]) ] $
			RET [VAR t1]
\end{code}

Under this mapping, every C variable is represented by a memory-resident object with an envelope derived from that variable's C type. For example, in the above code fragment, "r" and "i" represent integer variables, so that all of the corresponding envelopes contain a single tuple of the form “"{-hv-}(0 {-hv-}, ZG`FMT`standardEncoding)"”, which binds the envelope offset 0 to the standard integer format “"ZG`FMT`standardEncoding"” with an empty set of access attributes. All such objects are deallocated at the end of their C scope by appropriate “"DEL"” terms, invoked in the reverse order of the objects' introduction. Further, since the C Standard does not prescribe a specific order of evaluation for most subexpressions in the program, the corresponding Etude terms are always evaluated within a single Etude group, such as that used to bind the temporary variables "t1" and "t2" in the above example.

Although this version of the program is arguably less pleasant than the earlier translation, it carries the benefit of a uniform treatment of all C expressions and scopes. Since the Etude term equivalence relation defined in Chapter \ref{Lambda} considers only the sequence of actual system calls invoked during a program's evaluation and not the resulting memory image, in the context of an entire program both of the above representations of “\C{fac}” may, in principle, be deemed synonymous, which a smarter implementation of the compiler could easily utilise to produce an even more efficient representation of that function, either immediately during its translation into Etude terms, or else through a set of generic term transformations applied to the constructed Etude modules in an optimising back end of the compiler. The linear approach to compiler design specifically promotes such performance improvements through its formulation of the linear \hbox{correctness criteria.}

\end{section}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
