\set{capture}{hidden}
\begin{code}

{-# LANGUAGE RecursiveDo #-}

module C.Statements (
	meaningOfJump,
	meaningOfControllingExpression,
	meaningOfStatementListOpt,
) where

import Control.Monad.Fix
import Additional
import C.Syntax
import C.Variables
import C.Types
import C.Scopes
import C.Constants
import C.Expressions
import C.Declarations
import {-# SOURCE #-} C.Implementation
import Lambda

\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Statements}
\set{title}		{Statements}
\begin{section}

\BeginHaskellToken	{meaningOfControllingExpression}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{ctrl}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, TermT VarT)}
\EndHaskellToken

\set{capture}		{meaningOfControllingExpression}
\begin{code}
meaningOfControllingExpression {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, ExpressionT) -> _M{--}(S, S, D, TermT VarT)

meaningOfControllingExpression {-sem-}(_S, _T, _D, _I {->-}, e) = mdo
	(_S', _T', _D', _V, t, tau) <- meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, e)
	require	$ (isScalarType{--}(t))
	return $ (_S', _T', _D', {-sem-}({-sem-}(scopeConstructor{--}(_V)) `composeTerms` --
					 LET [{-raw-}[TVar 1] =: tau] $ --
				 	 {-sem-}(scopeDestructor{--}(_V)) `completeTerm` --
					 RET [OP2 (VAR (TVar 1)) (CNE) ({-sem-}(formatOfType{--}(t))) (IMM (0) ({-sem-}(formatOfType{--}(t))))]))
\end{code}

Most of the computational structure of C programs is captured in the syntax of entities known as «statements». Statements represent both the most interesting and, semantically, the most complex aspect of the C programming language, since they expose a number of fundamental differences between the designs of typical imperative languages such as C and the declarative paradigms of lambda calculi. It is a precise formal specification of statement semantics to which we will now turn our attention.

In particular, every well-formed C statement may be classified into one of six general forms, giving rise to the following Haskell definitions of their concrete syntax:

\IndexType		{StatementT}					{statement (\tstar)}
\IndexCons		{S1}						{ labelled_statement (statement)}
\IndexCons		{S2}						{ compound_statement (statement)}
\IndexCons		{S3}						{ expression_statement (statement)}
\IndexCons		{S4}						{ selection_statement (statement)}
\IndexCons		{S5}						{ iteration_statement (statement)}
\IndexCons		{S6}						{ jump_statement (statement)}

\IndexType		{LabelledStatementT}				{labelled_statement (\tstar)}
\IndexCons		{LabelStatement}				{: identifier statement (labelled_statement)}
\IndexCons		{CaseStatement}					{case: constant_expression statement (labelled_statement)}
\IndexCons		{DefaultStatement}				{default: statement (labelled_statement)}

\IndexType		{CompoundStatementT}				{compound_statement (\tstar)}
\IndexCons		{CompoundStatement}				{{} declaration_list_opt statement_list_opt (compound_statement)}

\IndexType		{ExpressionStatementT}				{expression_statement (\tstar)}
\IndexCons		{ExpressionStatement}				{; expression_opt (expression_statement)}

\IndexType		{SelectionStatementT}				{selection_statement (\tstar)}
\IndexCons		{IfStatement}					{if() expression statement (selection_statement)}
\IndexCons		{IfElseStatement}				{if_else() expression statement statement (selection_statement)}
\IndexCons		{SwitchStatement}				{switch() expression statement (selection_statement)}

\IndexType		{IterationStatementT}				{iteration_statement (\tstar)}
\IndexCons		{WhileStatement}				{while() expression statement (iteration_statement)}
\IndexCons		{DoWhileStatement}				{do_while(); statement expression (iteration_statement)}
\IndexCons		{ForStatement}					{for(;;) expression_opt expression_opt expression_opt statement (iteration_statement)}

\IndexType		{JumpStatementT}				{jump_statement (\tstar)}
\IndexCons		{GotoStatement}					{goto; identifier (jump_statement)}
\IndexCons		{BreakStatement}				{break; (jump_statement)}
\IndexCons		{ContinueStatement}				{continue; (jump_statement)}
\IndexCons		{ReturnStatement}				{return; expression_opt (jump_statement)}

\begin{syntax}
\InsertHaskell		{StatementT, \par, LabelledStatementT, \par, CompoundStatementT, \par, ExpressionStatementT, \par, 
			 SelectionStatementT, \par, IterationStatementT, \par, JumpStatementT}
\end{syntax}

Due to the peculiar sequential nature and scoping rules adopted by these entities, a representation of their meanings in Etude is rather less straight forward than the one featured in our earlier semantic treatment of C expressions from Section \ref{C/Expressions}. Fundamentally, every group of consecutive C statements that begins with an optional «statement label» and ends in an explicit or implicit «jump» construct depicts a logical entity known as a «basic block» in the standard programming language nomenclature. Intuitively, such blocks describe a group of machine instructions that are always executed in a predetermined purely-sequential manner by the program. For example, the following list of C statements defines three basic blocks: \hbox{“"LabelStatement (_L1 {-!-}) {-:-} {-,,,,-} ExpressionStatement (s1) {-;-} {-,,,,-} GotoStatement (_L2) {-;-}"\!”,} \hbox{“"LabelStatement (_L2 {-!-}) {-:-} {-,,,,-} ExpressionStatement (s2) {-;-} {-,,,,-} ExpressionStatement (s3) {-;-} {-,,,,-} GotoStatement (_L1 {-!-}) {-;-}"\!”} and \hbox{“"LabelStatement (_L3 {-!-}) {-:-} {-,,,,-} ExpressionStatement (s4) {-;-} {-,,,,-} ReturnStatement () {-;-}"\!”:}

\begin{C}
L1:	s1;
L2:	s2;
	s3;
	goto L1;
L3:	s4;
	return;
\end{C}

In Etude, every such basic block is represented by a separate lambda abstraction, whose parameter list includes all of the local variables visible to the corresponding set of statements in a C program, although, strictly speaking, such functions represent «extended basic blocks» under conventional meaning of the term, since, in general they may include multiple exit points or jumps. The set of all basic blocks derived from a particular C function is represented in the following translation by a finite map "_B", which associates the Etude variable of each label with its local variable set "_V". Further, during translation of all C statements, a finite map "_L" maps all label identifiers visible in their «function scope» to the corresponding Etude variables. More so, a finite map "_C" contains the mapping of all visible “"CaseStatement"” labels to their associated integer values and, finally, a set "_J" maintains the collection of all label names actually utilised by some “"GotoStatement"” statement within the function. Formally, these four structures assume the following Haskell types:

\IndexType		{B}						{B (\tstar)}
\IndexType		{L}						{L (\tstar)}
\IndexType		{C}						{C (\tstar)}
\IndexType		{J}						{J (\tstar)}

\begin{syntax}
\InsertHaskell		{B, L, C, J}
\end{syntax}

This somewhat elaborate context of C statements reflects the fact that, in C, constructs such as “\CKeyword{goto}” may transcend the usual scoping structure of declarative programs. For example, in the following nonsensical code fragment:

\begin{C}
L1:	int x = 1;
	s1;
	{
		int x = 2;
		s2;
	L2:	if (x) {
			goto L1;
		}
		s3;
	}
	s4;
	goto L2;
\end{C}

the two declarations of "x" introduce a pair of semantically-distinct objects, each allocated conceptually at the beginning of its respective scope. Only the first "x" is visible to the statements "s1" and "s4", while "s2" and "s3" see only the second. However, upon execution of the jump “"GotoStatement (_L1 {-!-}) {-;-}"\!” and at the end of the following statement "s3", the first "x" reappears in the current scope, so that its Etude counterpart must, somehow, be retained throughout the entire program fragment, despite the preceding intermittent invisibility of the corresponding C identifier. To complicate the matters further, the second "x" is actually allocated twice in the above list of statements, first by the declaration “"IntSpecifier x SimpleAssignment ExpressionStatement (2) {-;-}"\!” and then again by the jump “"GotoStatement (_L2 {-!-}) {-;-}"\!”, which transcends the usual scoping structure and brings the second "x" into scope without executing its proper declaration.

Accordingly, every control transfer to a basic block, whether represented explicitly in the concrete syntax of a C program by one of the jump statements discussed later in Section \ref{C/Statements/Jump}, or implied by a selection or iteration statement from Sections \ref{C/Statements/Selection} and \ref{C/Statements/Iteration}, is modelled uniformly by a tail call to the corresponding Etude function, as constructed by the following Haskell derivation of its meaning:

\BeginHaskellToken	{meaningOfJump}				{Open}	{\ZapfChancery{J}}
  \HaskellArgument							{(B, V {->-}, VarT)}
  \HaskellResult							{TermT VarT}
\EndHaskellToken

\IndexTerm		{meaningOfJump}					{J var B V (term)}

\begin{code}
meaningOfJump {-[*]-} :: (B, V {->-}, VarT) -> TermT VarT

meaningOfJump {-sem-}(_B, _V {->-}, nu) ---
			= {-sem-}({-sem-}(scopeDestructor{--}(_V `mapSetDifference` dom{--}(_B#(nu)))) `composeTerms` --
				  {-sem-}(scopeConstructor{--}(_B#(nu) `mapSetDifference` dom{--}(_V))) `completeTerm` --
				  APP (VAR (nu)) (coerceList (VAR) (setToList {-sem-}(dom{--}(_B#(nu))))))
\end{code}

so that the notation “\!"meaningOfJump{--}(_B, _V {->-} nu)"” depicts a jump to a C label associated with the Etude variable "nu", provided that "_V" represents the set of all local C variables visible at the beginning of that label, with every such variable mapped to the Etude envelope of its respective C type, and the finite map "_B" binds the Etude variables associated with all labels defined in the surrounding C function to their individual local variable sets. 

Intuitively, the resulting Etude term begins by purging from the current address space any local variables in "_V" that do not exist in the scope of the targeted label "nu", using the scope destruction operation "scopeDestructor{--}(_V `mapSetDifference` dom{--}(_B#(nu)))" described earlier in Section \ref{C/Declarations/Initialisers/Dynamic}. Next, all local definitions visible to "nu" but missing from the caller's scope are introduced using the scope constructor "scopeConstructor{--}(_B#(nu) `mapSetDifference` dom{--}(_V))", effectively recreating the C scope current at the point where the label denoted by "nu" was actually introduced into the program. Finally, the actual tail call or jump operation is modelled by a function application with a list of arguments that are lexically identical to the sorted list of all the local variables visible at the point of "nu"'s definition, since, under the translation described in this chapter, the meaning of each locally-defined C object is always depicted by a predetermined Etude variable whose atomic value remains invariant throughout the entire execution of a given C function.

Accordingly, the meanings of all C statements are always derived in a semantic context that consists of thirteen individual components "_S", "_T", "_D", "_I", "_B", "_L", "_V", "t_r", "t_s", "nu_r", "nu_c", "nu_b" and "tau_kappa", where "_S", "_T", "_D" and "_I" represent the usual current variable and tag scopes, set of item definitions and scope index. In addition, the two finite maps "_B" and "_L" describe the complete set of all basic blocks and label identifiers of the surrounding C function, retaining the same invariant values throughout the translation of a particular function's body. Finally, the remaining seven components "_V", "t_r", "t_s", "nu_r", "nu_c", "nu_b" and "tau_kappa", further refine the statement's semantic context as follows:

\begin{list}

•	"_V" represents the set of all local Etude variables visible to the statement under translation, with each such variable mapped to the envelope of its declared C type.

•	"t_r" depicts the «returned type» of the surrounding C function. If this type is not “"VoidType"\,”, then the first variable from the associated "_V" set is always bound to a memory-resident object destined to hold the function's returned value.

•	"t_s" specifies the optional «selection type» of the controlling expression "e" in the innermost surrounding statement of the form “"SwitchStatement (e) {-,,-} s"\,”. It is used in determination of the precise numeric values assigned to any “"CaseStatement"” labels encountered during translation. This component is omitted from the statement's semantic context during analysis of entities that appear outside of any “\CKeyword{switch}” statements, whereby its value is represented by the symbol “"Nothing"” described in Appendix \ref{Additional}.

•	"nu_r" represents the «return label» variable, bound to an implicit basic block that is targeted by all “"ReturnStatement"” statements within the current C function.

•	"nu_c" specifies the optional «continuation label», bound to the basic block targeted by any “"ContinueStatement"” statement forms within the current innermost iteration statement. If no such statement exists, "nu_c" is given the value of “"Nothing"”.

•	Similarly, "nu_b" specifies the optional «break label», bound to the basic block targeted by any “"BreakStatement"” statement forms within the current innermost “"SwitchStatement"” or iteration statement, once again defaulting to “"Nothing"” if no such statement exists in the program.

•	Finally, "tau_kappa" represents the «continuation term», intuitively equal to the combined denotation of all C statements that, in the program's lexical syntax, follow the one currently under translation. Its presence reflects the fact that the structure of a C statement alone is generally insufficient to determine the basic block to which the entity belongs. Consider, for example, the following program fragment:

\begin{C}
L1:	s1;
	{
		s2;
	L2:	s3;
	}
	s4;
\end{C}

which, conceptually, defines two basic blocks \hbox{“"LabelStatement (_L1 {-!-}) {-:-} {-,,,,-} ExpressionStatement (s1) {-;-} {-,,,,-} ExpressionStatement (s2) {-;-}"\!”} and \hbox{“"LabelStatement (_L2 {-!-}) {-:-} {-,,,,-} ExpressionStatement (s3) {-;-} {-,,,,-} ExpressionStatement (s4) {-;-}"\!”.} Unfortunately, there is no simple way for "s4" to known that it belongs to the label "_L2", or for the compound statement \hbox{“"ExpressionStatement (s2) {-;-} {-,,,,-} LabelStatement (_L2 {-!-}) {-:-} {-,,,,-} ExpressionStatement (s3) {-;-}"\!”} to discover that it spans two basic blocks, neither of which is actually complete within that statement. In order to resolve all such complications in a uniform manner, the denotation of every C statement is systematically composed with its respective continuation term and shoved into the set of item definitions "_D" as soon as the boundary of a complete basic block to which the statement belongs has been discovered by the compiler. Specifically, in the above example, the continuation terms of "s1", "s2" and "s3" represent the denotations of "s2", \hbox{“"GotoStatement (s3) {-;-}"\!”} and "s4", respectively.

\end{list}

The actual denotations derived from all C statements consist of the usual updated current scopes "_S" and "_T", the set of item definitions "_D", as well as five statement-specific components "_B'", "_L'", "_C'", "_J'" and "tau", which, intuitively, fulfil the following individual rôles:

\begin{list}

•	The set of «new basic block definitions» "_B'" represents a strict subset of the basic block definitions "_B" from the statement's context, consisting precisely of those basic block definitions that have been introduced into the program by the statement under semantic scrutiny. Intuitively, a union of all such sets "_B_s'" for all statements "s" appearing within the lexical syntax of a given C function represents the complete value of the context component "_B" supplied to every one of these statements, in recognition of the fact that, in C, a jump to a given basic block often precedes that block's actual appearance within the program's concrete syntax.

•	Similarly, the set of «new named label definitions» "_L'" represents the subset of "_L" contributed to the surrounding C function by the statement being scrutinised. Each label is mapped in "_L'" to the Etude variable of the corresponding basic block.

•	Further, the set of «case label definitions» "_C'" maps all case labels introduced by the current statement to the Etude variables of their respective basic blocks. Every such label is represented in the domain of "_C'" either by its concrete integer value, or else by the symbol “"Nothing"” for labels introduced by the “"DefaultStatement"” statement form described later in Section \ref{C/Statements/Labelled}.

•	To facilitate correct diagnostics of “"GotoStatement"” statements, the set of all label identifiers that are actually accessed by some explicit jump within the currently-examined program fragment is returned in the statement's denotation as the component "_J'". The reader should observe that the value of "_J'" is only ever examined later in Section \ref{C/Definitions/Functions}, once the entire C function has been processed, since, until that time, the complete set of all defined label cannot be known to the compiler.

•	Finally, the Etude term "tau" represents the actual computation performed by the current statement, which, as already mentioned earlier, implicitly incorporates its continuation term "tau_kappa". Further, as shown in Section \ref{C/Statements/Labelled}, the meanings of all labelled statement forms such as “"LabelStatement (x) {-:-} {--} s1"\!”, “"CaseStatement (e {-!-}) {-:-} {--} s2"\!” and “"DefaultStatement {-:-} {--} s3"\!” are defined as implicit jumps to their respective basic blocks. The actual Etude terms derived from "s1", "s2" and "s3" are always incorporated into the bodies of the corresponding Etude functions and are never returned as part of the actual term denotations of their surrounding labelled statement entities.

\end{list}

Formally, these denotations are derived by the following Haskell construction:

\BeginHaskellToken	{meaningOfStatementListOpt}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{s}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, B, L, V, I, TypeT, Maybe TypeT, VarT, Maybe VarT, Maybe VarT, TermT VarT {->-}, Maybe StatementListT)}
  \HaskellResult							{_M{--}(S, S, D, B, L, C, J, TermT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfStatementListOpt}			{D_S statement_list_opt}

\begin{code}
meaningOfStatementListOpt {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, B, L, V, I, TypeT, Maybe TypeT, VarT, Maybe VarT, Maybe VarT, TermT VarT {->-}, Maybe StatementListT) ->
	_M{--}(S, S, D, B, L, C, J, TermT VarT)
\end{code}

observing that this definition is naturally extended to encompass meanings of entire statement lists. For the sake of presentation, the actual definition of "meaningOfStatementListOpt" is presented separately for each statement form in the following seven sections.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Statements/Labelled}
\set{title}		{Labelled Statements}
\begin{subsection}

In C, all constructs of the form “"LabelStatement (x) {-:-} {-,,-} s"”, “"CaseStatement (e) {-:-} {-,,-} s"” and  “"DefaultStatement {-:-} {-,,-} s"” are known collectively as «labelled statements». Every such entity consists of an arbitrary «nested statement» "s", further associated with an annotation known as a «label», which may be fashioned from an identifier "x", a construct of the form “"CaseStatement (e)"\,”, or the “"DefaultStatement"” keyword. Conceptually, every labelled statement introduces a new basic block into the program, which, in the eventual Etude module, is represented by a function of the form “"FUN (nu_set) {-DOT-} (tau)"”, whose parameter list "nu_set" is formed from the domain of the statement's set of local variables "_V" and whose body is equal to the denotation of the nested statement "s". As already mentioned earlier, the denotation of "s" already incorporates its continuation term "tau_kappa" and, accordingly, constitutes a complete body of that basic block.

The new function item is always contributed to the set of item definitions "_D" and returned as part of the statement's denotation. Further, the corresponding unique Etude variable "uvar{--}(_D)" is added to the set of new basic block definitions "_B'", whereby it is bound to the statement's set of local variables "_V". The actual Etude term derived from the statement represents a simple jump to the new basic block, as depicted by the Haskell construction “\!"meaningOfJump{--}(_B, _V {->-}, uvar{--}(_D))"” defined earlier.

Further, if the statement's label consists entirely of a single identifier "x", then "x" must not appear among the label names "_L_s" derived from its nested statement "s". In the resulting denotation, the identifier "x" is added to "_L_s", with a binding to the newly-introduced basic block variable "uvar{--}(_D)". In Haskell:

\begin{code}
meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 (S1 (LabelStatement (x) {-:-} (s {-,-}))))) = mdo
	(_D') <- {-hide-}return $ _D `mapUnion` listToMap [ {-raw-}(uvar{--}(_D) {-:-}, Just {-sem-}(FUNI (FUN (setToList {-sem-}(dom{--}(_V))) {-DOT-} (tau_s)))) ]
	(_S_s, _T_s, _D_s, _B_s, _L_s, _C_s, _J_s, tau_s) <- meaningOfStatementListOpt --
		{--}(_S, _T, _D', _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 s))
	require $ (x /@ dom{--}(_L_s))
	return $ (_S_s, --
		  _T_s, --
		  _D_s, --
		  _B_s `mapUnion` listToMap [ {-raw-}(uvar{--}(_D) {-:-}, _V) ], --
		  _L_s `mapUnion` listToMap [ {-raw-}(x {-:-}, uvar{--}(_D)) ], --
		  _C_s, --
		  _J_s, --
		  meaningOfJump{--}(_B, _V {->-}, uvar{--}(_D)))
\end{code}

On the other hand, in a labelled statement of the form “"CaseStatement (e) {-:-} {-,,-} s"\,”, the C expression "e" must always represent an integral constant, whose numeric value "n" is converted into the type of the surrounding switch statement "t_s" and, in the resulting set of case definitions "_C'", mapped to the newly-introduced basic block variable "uvar{--}(_D)". Every such statement must be always embedded within some “"SwitchStatement"” construct, which, in the following Haskell definition, is asserted by requiring the selection type "t_s" to have some concrete value other than “"Nothing"”. As for identifier labels, the converted value of "n" must not appear in the set "_C_s" that has been derived from the construct's nested statement "s":

\begin{code}
meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 (S1 (CaseStatement (e) {-:-} (s {-,-}))))) = mdo
	require $ (t_s /= Nothing)
	(_D') <- {-hide-}return $ _D `mapUnion` listToMap [ {-raw-}(uvar{--}(_D) {-:-}, Just {-sem-}(FUNI (FUN (setToList {-sem-}(dom{--}(_V))) {-DOT-} (tau_s)))) ]
	(_S_e,   _T_e,  _D_e,  t, n) <- meaningOfIntegralConstantExpression --
		{--}(_S, _T, _D', _I {->-}, E1 (E2 e))
	(_S_s,   _T_s,  _D_s,  _B_s, _L_s, _C_s, _J_s, tau_s) <- meaningOfStatementListOpt --
		{--}(_S_e, _T_e, _D_e, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 s))
	(n') <- valueOfAtom (formatOfType{--}(unJust t_s)) --
		{-sem-}(OP1 (CNV {-sem-}(formatOfType{--}(unJust t_s))) ({-sem-}(formatOfType{--}(t))) {--} {-hv-}(IMM (toRational n) ({-sem-}(formatOfType{--}(t)))))
	require $ (Just (discardFraction n') /@ dom{--}(_C_s))
	return $ (_S_s, --
		  _T_s, --
		  _D_s, --
		  _B_s `mapUnion` listToMap [ {-raw-}(uvar{--}(_D) {-:-}, _V) ], --
		  _L_s, --
		  _C_s `mapUnion` listToMap [ {-raw-}(Just (discardFraction n') {-:-}, uvar{--}(_D)) ], --
		  _J_s, --
		  meaningOfJump{--}(_B, _V {->-}, uvar{--}(_D)))
\end{code}

Finally, every construction of the form “"DefaultStatement {-:-} {-,,-} s"\,” must also appear within the context of an enclosing “"SwitchStatement"” statement. In the resulting set of case labels "_C'", the new basic block is represented by the distinguished case label “"Nothing"”, which, once again, cannot appear among the labels of its nested statement "s". Formally:

\set{continued}		{yes}
\begin{code}
meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 (S1 (DefaultStatement {-:-} (s {-,-}))))) = mdo
	require $ (t_s /= Nothing)
	(_D') <- {-hide-}return $ _D `mapUnion` listToMap [ {-raw-}(uvar{--}(_D) {-:-}, Just {-sem-}(FUNI (FUN (setToList {-sem-}(dom{--}(_V))) {-DOT-} (tau_s)))) ]
	(_S_s, _T_s, _D_s, _B_s, _L_s, _C_s, _J_s, tau_s) <- meaningOfStatementListOpt --
		{--}(_S, _T, _D', _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 s))
	require $ (Nothing /@ dom{--}(_C_s))
	return $ (_S_s, --
		  _T_s, --
		  _D_s, --
		  _B_s `mapUnion` listToMap [ {-raw-}(uvar{--}(_D) {-:-}, _V) ], --
		  _L_s, --
		  _C_s `mapUnion` listToMap [ {-raw-}(Nothing {-:-}, uvar{--}(_D)) ], --
		  _J_s, --
		  meaningOfJump{--}(_B, _V {->-}, uvar{--}(_D)))
\end{code}
\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Statements/Compound}
\set{title}		{Compound Statements or Blocks}
\begin{subsection}

Multiple C statements are often combined into a single «compound statement», which, in the concrete syntax of the language, is represented by an optional list of declarations followed by a list of zero or more statements, with the entire construct enclosed in braces. Every such compound statement “\!"CompoundStatement (dl_opt) (sl_opt)"\!” introduces a new current scope into the program, whose index "_I" is one greater than that of its surrounding semantic context. Further, the context of the statement list "sl_opt" is extended with the set of local variable definitions "_V_d" introduced by any non-static declarations from "dl_opt". The resulting Etude term begins by introducing into the program's address space all of these newly-defined local variables, as depicted by the the scope constructor "scopeConstructor{--}(_V_d)". Next, the underlying local objects are populated with their desired initial contents by evaluating the initialiser term "tau_d" derived from the declaration list "dl_opt". Finally, the actual computational meaning "tau_s" of the supplied statement list "sl_opt" is evaluated. An appropriate scope destructor "scopeDestructor{--}(_V_d)" is also prefixed to the continuation term of "sl_opt", in order to ensure that all of the Etude objects local to the new scope are purged from the program's address space upon completion of the statement's execution. Formally, this denotation is represented in Haskell as follows:

\begin{code}
meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 (S2 (CompoundStatement (dl_opt) (sl_opt))))) = mdo
	(_S_d,  _T_d,  _D_d,  _V_d, tau_d) <- meaningOfDeclarationListOpt{--}(_S, _T, _D, _I + 1 {->-}, dl_opt)
	(_S_s,  _T_s,  _D_s,  _B_s, _L_s, _C_s, _J_s, tau_s)
	{-quad-}<- meaningOfStatementListOpt{--}(_S_d, _T_d, _D_d, _B, _L, _V `mapUnion` _V_d, _I + 1, t_r, t_s, nu_r, nu_c, nu_b, --
			{-sem-}({-sem-}(scopeDestructor{--}(_V_d)) `completeTerm` tau_kappa) {->-}, sl_opt)
	return $ (_S, --
		  _T, --
		  _D_s, --
		  _B_s, --
		  _L_s, --
		  _C_s, --
		  _J_s, --
		  {-sem-}({-sem-}(scopeConstructor{--}(_V_d)) `completeTerm` LET [{-hv-}[] =: tau_d] (tau_s)))
\end{code}

The reader should observe that the new variable and tag scopes do not persist past the end of the construct, so that the resulting denotation includes the original values of its prevailing current scopes "_S" and "_T", instead of those derived from the statement \hbox{list "sl_opt".}

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Statements/Expression}
\set{title}		{Expression and Null Statements}
\begin{subsection}

Any C expression can be also turned into an «expression statement» by appending a semicolon to its concrete syntax. Further, the actual expression component can be omitted entirely from such constructs, in which case the entity is said to represent the «null statement» form “"ExpressionStatement () {-;-}"\!”. Regardless of its precise syntax, every expression statement denotes a simple Etude term of the form \hbox{“"scopeConstructor{--}(_V_e) `composeTerms` LET [{-hv-}[] =: tau_e] $ scopeDestructor{--}(_V_e) `completeTerm` tau_kappa"\!”,} in which "tau_e" represents the void denotation of its optional expression component. Intuitively, this denotation evaluates that void denotation, after introducing any required temporary variables into the program's address space, which are promptly discarded before proceeding with execution of the continuation term "tau_kappa". Formally:

\begin{code}
meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 (S3 (ExpressionStatement (e_opt))))) = mdo
	(_S_e, _T_e, _D_e, _V_e, tau_e) <- meaningOfExpressionOpt{--}(_S, _T, _D, _I {->-}, e_opt)
	return $ (_S_e, --
		  _T_e, --
		  _D_e, --
		  emptyMap, --
		  emptyMap, --
		  emptyMap, --
		  emptySet, --
		  {-sem-}({-sem-}(scopeConstructor{--}(_V_e)) `composeTerms` --
			  LET [{-hv-}[] =: tau_e] $ --
			  {-sem-}(scopeDestructor{--}(_V_e)) `completeTerm` --
			  tau_kappa))
\end{code}

In other words, expression statements never introduce any new labels or local variables into the program, so that the "_B'", "_L'", "_C" and "_J" components of their denotations always represent the empty set "emptySet". The fact that the continuation term usually pertains to an expression which is only encountered much later in the translation process is immaterial, since Haskell's lazy evaluation mechanism is perfectly capable of alluding to events that are yet to come. In particular, the continuation term is always incorporated, but never actually constructed until the C statement from which it is to be derived has been encountered in the program, deemed well-formed and processed by the compiler.

This simple definition underpins the entire expressive power of the continuation term approach to the specification of C statements pursued in this work. In particular, given that C expressions are the sole means provided by the language for representation of all non-trivial computations in a program, the above inclusion of the continuation term "tau_kappa" in their denotations ensures that every other form of a C statement always denotes an Etude term which implicitly incorporates the remainder of a basic block to which the statement belongs. As we have already seen in Section \ref{C/Statements/Labelled}, this basic block is contributed to the constructed Etude program by the translation of its introducing label and, thereafter, it is always referenced solely through tail calls (or jumps) to the Etude variable associated with that label.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Statements/Selection}
\set{title}		{Selection Statements}
\begin{subsection}

The language recognises three kinds of «selection statements», often referred to as the “\CKeyword{if}\,”, “\CKeyword{if}-\C{else}” and “\CKeyword{switch}” statement forms. Conceptually, all three variants of a selection statement represent a fork in the program's control flow graph, whereby only some subset of the statement's nested components are evaluated as determined dynamically during the construct's execution.

Upon completion of its evaluation, every selection statement always proceeds with the reduction of its continuation term "tau_kappa". In order to avoid replication of "tau_kappa" on every possible execution path through the program, all of the following denotations introduce into the statement's set of item definitions "_D'" a new basic block "uvar{--}(_D')" with the body of “"FUN (nu_set) {-DOT-} (tau_kappa)"\!”, in which "nu_set" includes all of the local variables visible at the beginning of the construct. During translation the entity's nested statements, their continuation terms are systematically replaced with a simple jump to this new basic block, as depicted by the standard notation “\!"meaningOfJump{--}(_B, _V {->-}, uvar{--}(_D'))"” described earlier in this section.

The parenthesised C expression which appears in the syntax of every selection statement and most of the iteration statement forms described later in Section \ref{C/Statements/Iteration} is known as the entity's «controlling expression». Formally, such expressions must always represent values of a scalar C type. Their denotations are evaluated in a scope delimited by the usual constructor and destructor terms "scopeConstructor{--}(_V_e)" and "scopeDestructor{--}(_V_e)", where "_V_e" is the set of temporary variables introduced by the expression. They always deliver the result obtained from comparison of the expression's value for inequality with the constant “"integerConstantZero"” of an appropriate C type, as depicted by the following translation of these entities:

\IndexTerm		{meaningOfControllingExpression}		{D_CTRL expression}
\InsertHaskell		{meaningOfControllingExpression}

In particular, every selection statement of the form “"IfStatement (e) (s)"” represents the Etude term “"LET [{-raw-}[TVar 1] =: tau_e] $ IFF (VAR (TVar 1)) {-THEN-} (tau_s) {-ELSE-} (tau_kappa')"”, in which "tau_e" and "tau_s" depict the respective denotations of the controlling expression "e" and the nested statement "s", while "uvar{--}(_D_s)" denotes a unique basic block variable bound to the construct's continuation term "tau_kappa". In other words, such statements evaluate "s" if and only if "e" has a non-zero value. Formally:

\begin{code}
meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 (S4 (IfStatement (e) (s))))) = mdo
	(tau_kappa') <- {-hide-}return $ meaningOfJump{--}(_B, _V {->-}, uvar{--}(_D_s))
	(_S_e, _T_e, _D_e, tau_e) <- meaningOfControllingExpression{--}(_S, _T, _D, _I {->-}, e)
	(_S_s, _T_s, _D_s, _B_s, _L_s, _C_s, _J_s, tau_s) --
		<- meaningOfStatementListOpt{--}(_S_e, _T_e, _D_e, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa' {->-}, Just (S0 s)) {-!-} {-!-} {-!-} {-!-} {-!-}
	return $  (_S_s, --
		   _T_s, --
		   _D_s `mapUnion` listToMap [ {-raw-}(uvar{--}(_D_s) {-:-}, Just {-sem-}(FUNI (FUN (setToList {-sem-}(dom{--}(_V))) {-DOT-} (tau_kappa)))) ], --
		   _B_s `mapUnion` listToMap [ {-raw-}(uvar{--}(_D_s) {-:-}, _V) ], --
		   _L_s, --
		   _C_s, --
		   _J_s,
		  {-quad-}{-sem-}(LET [{-raw-}[TVar 1] =: tau_e] $ --
			 	 IFF (VAR (TVar 1)) {-THEN-} --
					(tau_s) --
				 {-ELSE-} --
					(tau_kappa')))
\end{code}

Similarly, a selection statement of the form “"IfElseStatement (e) (s1) {-else-} (s2)"\!” evaluates either of the two nested statements "s1" or "s2", whenever its controlling expression "e" has a non-zero or zero value, respectively. Formally, the resulting denotation represents an Etude term of the form “"LET [{-raw-}[TVar 1] =: tau_e] $ IFF (VAR (TVar 1)) {-THEN-} (tau1) {-ELSE-} (tau2)"”, in which "tau_e", "tau1" and "tau2" depict the respective denotations of "e", "s1" and "s2". Similarly to the earlier “\CKeyword{if}\,” statement form, "tau_kappa" is saved in the body of a new basic block "uvar{--}(_D2)" and the continuation terms of both nested statements are replaced with a jump to that block. \hbox{In Haskell:}

\begin{code}
meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 (S4 (IfElseStatement (e) (s1) {-else-} (s2))))) = mdo
	(tau_kappa') <- {-hide-}return $ meaningOfJump{--}(_B, _V {->-}, uvar{--}(_D2))
	(_S_e,   _T_e,  _D_e,  tau_e) <- meaningOfControllingExpression{--}(_S, _T, _D, _I {->-}, e)
	(_S1,    _T1,   _D1,   _B1, _L1, _C1, _J1, tau1) --
		<- meaningOfStatementListOpt{--}(_S_e,  _T_e,  _D_e,  _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa' {->-}, Just (S0 s1)) {-!-} {-!-} {-!-} {-!-} {-!-}
	(_S2,    _T2,   _D2,   _B2, _L2, _C2, _J2, tau2) --
		<- meaningOfStatementListOpt{--}(_S1,   _T1,   _D1,   _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa' {->-}, Just (S0 s2)) {-!-} {-!-} {-!-} {-!-} {-!-}
	require $ (dom{--}(_L1) `intersection` dom{--}(_L2) == emptySet && dom{--}(_C1) `intersection` dom{--}(_C2) == emptySet)
	return $  (_S2, --
		   _T2, --
		   _D2 `mapUnion` listToMap [ {-raw-}(uvar{--}(_D2) {-:-}, Just {-sem-}(FUNI (FUN (setToList {-sem-}(dom{--}(_V))) {-DOT-} (tau_kappa)))) ],
		  {-quad-}_B1 `mapUnion` _B2 `mapUnion` listToMap [ {-raw-}(uvar{--}(_D2) {-:-}, _V) ], --
		   _L1 `mapUnion` _L2, --
		   _C1 `mapUnion` _C2, --
		   _J1 `union` _J2,
		  {-quad-}{-sem-}(LET [{-raw-}[TVar 1] =: tau_e] $ --
			  	  IFF (VAR (TVar 1)) {-THEN-} --
					(tau1) --
				  {-ELSE-} --
					(tau2)))
\end{code}

The reader should observe that the two nested statements appearing in such constructs must derive non-overlapping sets "_L'" and "_C'", in order to ensure that all statement and case labels always remain unique within their respective scopes.

On the other hand, selection statements of the form “"SwitchStatement (e) (s)"” have a rather different semantic interpretation. In every such construct, the controlling expression "e" must assume an integral type that, after integral promotion, will serve as the selection type "t_s" throughout translation of the nested statement "s". The entire construct represents a «multi-way jump», which, in Etude, can be modelled by a term of the following form:

\begin{haskell}
	{--}{-raw-}	(IFF (OP2 (VAR (TVar 1)) (CEQ) (phi) (n1   )) {-THEN-} (meaningOfJump{--}{-sem-}(_B, _V {->-}, nu1	))
	{-ELSE-}	(IFF (OP2 (VAR (TVar 1)) (CEQ) (phi) (n2   )) {-THEN-} (meaningOfJump{--}{-sem-}(_B, _V {->-}, nu2	)
			{-quad-}{-vdots-})
	{-ELSE-}	(IFF (OP2 (VAR (TVar 1)) (CEQ) (phi) (n_k  )) {-THEN-} (meaningOfJump{--}{-sem-}(_B, _V {->-}, nu_k	))
	{-ELSE-}	(meaningOfJump{--}{-sem-}(_B, _V {->-}, nu_d)))))
\end{haskell}

in which "n1..n_k" are the integer values of all “"CaseStatement"” labels introduced by the nested statement "s", "nu1..nu_k" represent the Etude variables bound to these labels' respective basic blocks and "nu_d" depicts a designated basic block bound to the special “"Nothing"” label introduced by the statement's “"DefaultStatement"” label, or else the basic block of its continuation statement "tau_kappa" if no such label is present in the syntax of "s". The Etude variable "nu_d" also doubles as the break label within "s", so that all “"BreakStatement {-;-}"\!” statements included within the syntax of "s" represent a jump to that basic block under their later interpretation in Section \ref{C/Statements/Jump}. The return and continuation labels "nu_r" and "nu_c" are not affected by these constructs. Formally, the denotation of every “"SwitchStatement"” statement is represented by the following Haskell translation:

\begin{code}
meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 (S4 (SwitchStatement (e) (s))))) = mdo
	(tau_kappa'  ) <- {-hide-}return $ meaningOfJump{--}(_B, _V {->-}, uvar{--}(_D_s))
	(tau_fun     ) <- {-hide-}return $ {-sem-}(composeList{--}[ {-sem-}(IFF (OP2 (VAR (TVar 2)) (CEQ) ({-sem-}(formatOfType{--}(integralPromotedType{--}(t_e)))) --
										(IMM (toRational n_k) ({-sem-}(formatOfType{--}(integralPromotedType{--}(t_e)))))) {-THEN-} --
									({-sem-}(meaningOfJump{--}(_B, _V {->-}, nu_k))) {-ELSE-}) --
								| {-raw-}(Just n_k {-:-}, nu_k) <- mapToList _C_s ])
	(_S_e,   _T_e,   _D_e,   _V_e, t_e, tau_e) <- meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, e)
	(_S_s,   _T_s,   _D_s,   _B_s, _L_s, _C_s, _J_s, tau_s)
	{-quad-}<- meaningOfStatementListOpt{--}(_S_e, _T_e, _D_e, _B, _L, _V, _I, t_r, --
							Just (integralPromotedType{--}(t_e)), nu_r, nu_c, Just (uvar{--}(_D_s)), tau_kappa' {->-}, Just (S0 s))
	require $ (isIntegralType{--}(t_e))
	return $  (_S_s, --
		   _T_s, --
		   _D_s `mapUnion` listToMap [ {-raw-}(uvar{--}(_D_s) {-:-}, Just {-sem-}(FUNI (FUN (setToList {-sem-}(dom{--}(_V))) {-DOT-} (tau_kappa)))) ], --
		   _B_s `mapUnion` listToMap [ {-raw-}(uvar{--}(_D_s) {-:-}, _V) ], --
		   _L_s, --
		   emptyMap, --
		   _J_s,
		  {-quad-}{-sem-}( ---
		  	{-sem-}(scopeConstructor{--}(_V_e)) `composeTerms`
			LET [{-raw-}[TVar 1] =: tau_e] $
			LET [{-raw-}[TVar 2] =: {-hide-}RET {-raw-}[OP1 (CNV {-sem-}(formatOfType{--}(integralPromotedType{--}(t_e)))) ({-sem-}(formatOfType{--}(t_e))) {--} --
			  				{-hv-}(VAR (TVar 1))]] $
			{-sem-}(scopeDestructor{--}(_V_e)) `composeTerms`
			tau_fun `completeTerm`
			meaningOfJump{--}(_B, _V {->-}, (listToMap [ {-raw-}(Nothing {-:-}, uvar{--}(_D_s)) ] // _C_s)#(Nothing))))
\end{code}

Observe that the case labels of "s" remain confined within their innermost surrounding “"SwitchStatement"” statement, so that the resulting set "_C'" is always emptied by the above definition, regardless of the precise form assumed by a given “"SwitchStatement"” statement.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Statements/Iteration}
\set{title}		{Iteration Statements}
\begin{subsection}

The language also defines three forms of «iteration statements» or «loops», commonly known as the “\CKeyword{while}”, “\CKeyword{do}-\CKeyword{while}” and “\CKeyword{for}” statement forms, respectively. Every such construct evaluates its nested statement repeatedly for as long as its controlling expression retains a non-zero value. Each iteration statement introduces at least two new basic blocks into the program, associated with the implicit continuation and break labels "nu_c" and "nu_b" that serve as a target for all occurrences of the jump statement forms “"ContinueStatement {-;-}"\!” and “"BreakStatement {-;-}"\!” within its syntax, as described later in Section \ref{C/Statements/Jump}.

In particular, if "L1" and "L2" are two otherwise-unused label identifiers and if “\!"{-...-}{--}"\!” represents the remainder of the current basic block, then an iteration statement of the form \hbox{“"WhileStatement (e) {-,,-} s"\,”} evaluates its nested statement "s" zero or more times, corresponding loosely to the following control flow structure, except that no new scope is introduced for its nested statement "s":

\begin{C}
L1:	if (e) {
		s;
		goto L1;
	}
	else {
		goto L2;
	}
L2:	...
\end{C}

Any occurrences of the “"ContinueStatement {-;-}"\!” and “"BreakStatement {-;-}"\!” statement forms within the above code fragment represent jumps to the respective labels "L1" and "L2",  which, in the following Haskell translation, is modelled by setting the continuation and break labels "nu_c" and "nu_b" to the globally-unique Etude variables "uvar{--}(_D)" and "uvar{--}(_D_s)" associated with these labels during analysis of the loop body "s". The selection type "t_s" is not affected by these constructs. Formally:

\begin{code}
meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 (S5 (WhileStatement (e) (s) {-,-})))) = mdo
	(nu_c'		) <- {-hide-}return $ uvar{--}(_D)
	(nu_b'		) <- {-hide-}return $ uvar{--}(_D_s)	
	(tau_ell	) <- {-hide-}return $ {-sem-}(LET [{-raw-}[TVar 1] =: tau_e] $ IFF (VAR (TVar 1)) {-THEN-} (tau_s) {-ELSE-} ({-sem-}(meaningOfJump{--}(_B, _V {->-}, nu_b'))))
	(tau_ell'	) <- {-hide-}return $ meaningOfJump{--}(_B, _V {->-}, nu_c')
	(_S_e,   _T_e,   _D_e, tau_e) <- meaningOfControllingExpression --
		{--}(_S, _T, _D `mapUnion` listToMap [ {-raw-}(nu_c' {-:-}, Just {-sem-}(FUNI (FUN (setToList {-sem-}(dom{--}(_V))) {-DOT-} (tau_ell)))) ], _I {->-}, e)
	(_S_s,   _T_s,   _D_s, _B_s, _L_s, _C_s, _J_s, tau_s) <- meaningOfStatementListOpt --
		{--}(_S_e, _T_e, _D_e, _B, _L, _V, _I, t_r, t_s, nu_r, Just nu_c', Just nu_b', tau_ell' {->-}, Just (S0 s))
	return $ (_S_s, --
		  _T_s, --
		  _D_s `mapUnion` listToMap [ {-raw-}(nu_b' {-:-}, Just {-sem-}(FUNI (FUN (setToList {-sem-}(dom{--}(_V))) {-DOT-} (tau_kappa)))) ], --
		  _B_s `mapUnion` listToMap [ {-raw-}(nu_c' {-:-}, _V), {-raw-}(nu_b' {-:-}, _V) ], --
		  _L_s, --
		  _C_s, --
		  _J_s, --
		  tau_ell')
\end{code}

Similarly, the iteration statement “"DoWhileStatement ({-,,,-}{-,-}s{-,,,-}{-,,,-}) {-while-} (e) {-!-} {-!-} ExpressionStatement () {-;-}"\!” evaluates "s" one or more times until the controlling expression "e" attains the value of “"integerConstantZero"”. In other words, the control structure of these statements can be described roughly by the following code fragment:

\begin{C}
L1:	s;
L2:	if (e) {
		goto L1;
	}
	else {
		goto L3;
	}
L3:	...
\end{C}

where, once again, the three identifiers "L1", "L2" and "L3" represent otherwise-unused basic block variables, with "L1" and "L3" also doubling as the continuation and break labels during translation of the loop body "s", as required for a proper translation of any \hbox{“"ContinueStatement {-;-}"\!”} and \hbox{“"BreakStatement {-;-}"\!”} statements appearing within it:

\begin{code}
meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 (S5 (DoWhileStatement (s) {-while-} (e) {-;-})))) = mdo
	(nu_i		) <- {-hide-}return $ uvar{--}(_D)
	(nu_c'		) <- {-hide-}return $ uvar{--}(_D_s)	
	(nu_b'		) <- {-hide-}return $ uvar{--}(_D_e)
	(tau_ell	) <- {-hide-}return $ {-sem-}(LET [{-raw-}[TVar 1] =: tau_e] $ IFF (VAR (TVar 1)) {-THEN-} ({-sem-}(meaningOfJump{--}(_B, _V {->-}, nu_i))) --
													  {-ELSE-} ({-sem-}(meaningOfJump{--}(_B, _V {->-}, nu_b'))))
	(tau_ell'	) <- {-hide-}return $ meaningOfJump{--}(_B, _V {->-}, nu_c')
	(_S_s, _T_s, _D_s, _B_s, _L_s, _C_s, _J_s, tau_s)
		<- meaningOfStatementListOpt ---
			 {--}(_S, _T, _D `mapUnion` listToMap [ {-raw-}(nu_i {-:-}, Just {-sem-}(FUNI (FUN (setToList {-sem-}(dom{--}(_V))) {-DOT-} (tau_s)))) ], --
			     _B, _L, _V, _I, t_r, t_s, nu_r, Just nu_c', Just nu_b', tau_ell' {->-}, Just (S0 s))
	(_S_e, _T_e, _D_e, tau_e) <- meaningOfControllingExpression --
			 {--}(_S_s, _T_s, _D_s `mapUnion` listToMap [ {-raw-}(nu_c' {-:-}, Just {-sem-}(FUNI (FUN (setToList {-sem-}(dom{--}(_V))) {-DOT-} (tau_ell)))) ], _I {->-}, e)
	return $  (_S_e, --
		   _T_e, --
		   _D_e `mapUnion` listToMap [ {-raw-}(nu_b' {-:-}, Just {-sem-}(FUNI (FUN (setToList {-sem-}(dom{--}(_V))) {-DOT-} (tau_kappa)))) ],
		  {-quad-}_B_s `mapUnion` listToMap [ {-raw-}(nu_i {-:-}, _V), {-raw-}(nu_c' {-:-}, _V),  {-raw-}(nu_b' {-:-}, _V) ], --
		   _L_s, --
		   _C_s, --
		   _J_s, --
		   meaningOfJump{--}(_B, _V {->-}, nu_i))
\end{code}

Finally, every iteration statement of the form “"ForStatement ((e1_opt), ({-,,,-}e2), ({-,,,-}e3_opt)) {-,,-} s"\,” always begins with an evaluation of the optional expression "e1_opt", which, similarly to the earlier “\CKeyword{while}” statement forms, is followed by zero or more iterations over the loop body “"ExpressionStatement (s) {-;-} {-,,-} ExpressionStatement ({-!-}e3_opt) {-;-}"\!”, subject to the controlling expression "e2" retaining a non-zero value. Like “\CKeyword{do}-\CKeyword{while}” statements, every “\CKeyword{for}” construct introduces three new basic blocks into the program and arranges them into the following control flow structure:

\begin{C}
___	e1_opt;
L1:	if (e2) {
		s;
		goto L2;
	}
	else {
		goto L3;
	}
L2:	e3_opt;
	goto L1;
L3:	...
\end{C}

in which the continuation and break labels are represented by the later unique identifiers "L2" and "L3". Formally, the meanings of all “\CKeyword{for}” statements are described in Haskell by the following translation semantics:

\begin{code}
meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, --
 Just (S0 (S5 (ForStatement ((e1_opt), (Just e2), (e3_opt)) {-,,-} s {-,-})))) = mdo
	(nu_i		) <- {-hide-}return $ uvar{--}(_D1)
  	(nu_c'		) <- {-hide-}return $ uvar{--}(_D2)	
  	(nu_b'		) <- {-hide-}return $ uvar{--}(_D_s)	
	(tau_ell	) <- {-hide-}return $ {-sem-}(LET [{-raw-}[TVar 1] =: tau2] $ IFF (VAR (TVar 1)) {-THEN-} (tau_s) {-ELSE-} ({-sem-}(meaningOfJump{--}(_B, _V {->-}, nu_b'))))
	(tau_ell'	) <- {-hide-}return $ meaningOfJump{--}(_B, _V {->-}, nu_c')
	(tau_c		) <- {-hide-}return $ {-sem-}({-sem-}(scopeConstructor{--}(_V3)) `composeTerms` --
						      LET [{-hv-}[] =: tau3] $ --
						      {-sem-}(scopeDestructor{--}(_V3)) `completeTerm` --
						      meaningOfJump{--}(_B, _V {->-}, nu_i))
	(_S1, _T1, _D1, _V1, tau1) <- meaningOfExpressionOpt{--}(_S, _T, _D, _I {->-}, e1_opt)
	(_S2, _T2, _D2, tau2) <- meaningOfControllingExpression --
		{--}(_S1, _T1, _D1 `mapUnion` listToMap [ {-raw-}(nu_i {-:-}, Just {-sem-}(FUNI (FUN (setToList {-sem-}(dom{--}(_V))) {-DOT-} (tau_ell)))) ], _I {->-}, e2)
	(_S3, _T3, _D3, _V3, tau3) <- meaningOfExpressionOpt --
		{--}(_S2, _T2, _D2 // listToMap [ {-raw-}(nu_c' {-:-}, Just {-sem-}(FUNI (FUN (setToList {-sem-}(dom{--}(_V))) {-DOT-} (tau_c)))) ], _I {->-}, e3_opt)
	(_S_s, _T_s, _D_s, _B_s, _L_s, _C_s, _J_s, tau_s) <- meaningOfStatementListOpt --
		{--}(_S3, _T3, _D3, _B, _L, _V, _I, t_r, t_s, nu_r, Just nu_c', Just nu_b', tau_ell' {->-}, Just (S0 s))
	return $  (_S_s, --
		   _T_s, --
		   _D_s `mapUnion` listToMap [ {-raw-}(nu_b' {-:-}, Just {-sem-}(FUNI (FUN (setToList {-sem-}(dom{--}(_V))) {-DOT-} (tau_kappa)))) ], --
		   _B_s `mapUnion` listToMap [ {-raw-}(nu_i {-:-}, _V), {-raw-}(nu_c' {-:-}, _V),  {-raw-}(nu_b' {-:-}, _V) ], --
		   _L_s, --
		   _C_s, --
		   _J_s,
		  {-quad-}{-sem-}({-sem-}(scopeConstructor{--}(_V1)) `composeTerms` --
		   	   LET [{-hv-}[] =: tau1] $ --
			   {-sem-}(scopeDestructor{--}(_V1)) `completeTerm` --
			   meaningOfJump{--}(_B, _V {->-}, nu_i)))
\end{code}

Finally, if the controlling expression "e2" is omitted from the syntax of a given “\CKeyword{for}” statement, then the entity behaves as if an implicit controlling expression with a constant non-zero value was inserted into the program by compiler:

\begin{code}
meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, --
 Just (S0 (S5 (ForStatement ((e1_opt), ({-hide-}Nothing), (e3_opt)) {-,,-} s{-,-}))))
	= meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 (S5 (ForStatement ((e1_opt), --
				  	(Just (E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (integerConstantOne)))))))))))))))))), (e3_opt)) {-,,-} s {-,-}))))
\end{code}

A close inspection of the control-flow graph constructed by the above translation of iteration statements reveals that the nested statement "s" in the “\C{do}-\C{while}” constructs is always evaluated at least once for every appearance of such an entity on the program's execution path. In all other iteration statements, however, evaluation of "s" is subject to the controlling expression "e" having a non-zero value at the beginning of the construct. As we shall soon discover in Section \ref{C/Statements/Jump}, in all three cases, the ordinary sequence of events may be also interrupted by explicit jumps to the continuation and break labels introduced by these entities, as represented in the concrete syntax of the language by the two jump statement forms “"ContinueStatement {-;-}"\!” and “"BreakStatement {-;-}"\!” that are specially reserved for that purpose.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Statements/Jump}
\set{title}		{Jump Statements}
\begin{subsection}

Finally, the class of «jump statements» includes four simple syntactic forms known respectively as the “\CKeyword{goto}”, “\CKeyword{continue}”, “\CKeyword{break}” and “\CKeyword{return}” statements. The first three of these have trivial translations into Etude, with every statement of the form “"GotoStatement (x) {-;-}"\!” representing a jump to a named label bound in the surrounding function scope "_L" to the specified identifier "x". For the purposes of later error diagnostics, "x" is also included in the returned set of referenced labels "_J'". On the other hand, every C statement of the form “"ContinueStatement {-;-}"\!”,  “"BreakStatement {-;-}"\!” and “"ReturnStatement () {-;-}"\!” represents an explicit jump to the corresponding continuation, break or return label "nu_c", "nu_b" or "nu_r". Such jumps are well-formed only if that label is actually included in the statement's context, i.e., if it does not have the omitted value “"Nothing"”. Formally:

\begin{code}
meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 (S6 (GotoStatement (x) {-;-})))) = mdo
	return $ (_S, _T, _D, emptyMap, emptyMap, emptyMap, listToSet [x], meaningOfJump{--}(_B, _V {->-}, _L#(x)))

meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 (S6 (ContinueStatement {-;-})))) = mdo
	require $ (nu_c /= Nothing)
	return $ (_S, _T, _D, emptyMap, emptyMap, emptyMap, emptySet, meaningOfJump{--}(_B, _V {->-}, unJust nu_c))

meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 (S6 (BreakStatement {-;-})))) = mdo
	require $ (nu_b /= Nothing)
	return $ (_S, _T, _D, emptyMap, emptyMap, emptyMap, emptySet, meaningOfJump{--}(_B, _V {->-}, unJust nu_b))
	
meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 (S6 (ReturnStatement ({-hide-}Nothing) {-;-})))) = mdo
	return $ (_S, _T, _D, emptyMap, emptyMap, emptyMap, emptySet, meaningOfJump{--}(_B, _V {->-}, nu_r))
\end{code}

On the other hand, a “"ReturnStatement (e) {-;-}"\!” statement whose syntax includes some C expression "e" is only admissible within C functions whose returned type "t_r" represents an object other than an array. Such statements precede the jump to "nu_r" with an assignment of "e"'s value to a temporary l-value bound to the least entry in the function's local variable set "_V". In particular, the supplied C expression "e" represents a dynamic initialiser for that temporary l-value, whose denotation \hbox{"meaningOfDynamicInitialiser{-sem-}(_S, _T, _D, _I, t_r, VAR (minimumOfSet{--}(dom{--}(_V))) {->-}, ExpressionInitialiser (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (NestedExpression (e))))))))))))))))))"} was described earlier in Section \ref{C/Declarations/Initialisers/Dynamic}. \hbox{In Haskell:}

\begin{code}
meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (S0 (S6 (ReturnStatement (Just e) {-;-})))) = mdo
	(_S_e, _T_e, _D_e, t_e, tau_e) <- meaningOfDynamicInitialiser{-sem-}(_S, _T, _D, _I, t_r, VAR (minimumOfSet{--}(dom{--}(_V))) {->-}, --
		ExpressionInitialiser (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (NestedExpression (e))))))))))))))))))
	require $ (isObjectType{--}(t_r) && not (isArrayType{--}(t_r)))
	return $ (_S_e, _T_e, _D_e, emptyMap, emptyMap, emptyMap, emptySet, {-sem-}(LET [{-hv-}[] =: tau_e] ({-sem-}(meaningOfJump{--}(_B, _V {->-}, nu_r)))))
\end{code}

A careful reader will observe that none of the above five labelled statement forms ever mentions the continuation term "tau_kappa" within their denotations. In fact, any computations that follow an unconditional jump in a C function can never be evaluated during ordinary execution of a well-formed C program, so that, in effect, the remainder of any basic block which includes such a jump statement is deemed «unreachable» and, as such, it is discarded by the above translations of these constructs.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Statements/Lists}
\set{title}		{Lists of Statements}
\begin{subsection}

To complete our formalisation of C statements, we must also specify the translation semantics of entire statement lists. Formally, the concrete syntax of such lists is depicted by the following BNF grammar:

\IndexType		{StatementListT}				{statement_list (\tstar)}
\IndexCons		{S0}						{statement (statement_list)}
\IndexCons		{Sn}						{statement_list statement (statement_list)}

\begin{syntax}
\InsertHaskell		{StatementListT}
\end{syntax}

Although, in this work, the denotations of all C statements are described by a single function "meaningOfStatementListOpt" that always operates on an entire statement list, in reality, sequences of multiple statemnets can only ever appear within the compound statement forms described earlier in Section \ref{C/Statements/Compound}. In all other contexts, such lists are formed by an implicit coercion of a singular statement form, whose presence, however, is always hidden in the presentation for the sake of conciseness.

Intuitively, a list of the form “"s1 `Sn` s2"\!” always evaluates both of its statements in the order of their appearance within the program. Formally, its denotation is equal to the meaning "tau1" of the first statement "s1", taken in the context of a continuation term "tau2" derived from the following statement "s2", which, in turn, is supplied the continuation term "tau_kappa" of the entire construction, provided that the sets of new label entities introduced by each statement in the list remain disjoint from each other. In Haskell:

\begin{code}
meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Just (sl `Sn` s)) = mdo
     (_S1, _T1, _D1, _B1, _L1, _C1, _J1, tau1) <- meaningOfStatementListOpt{--}(_S,   _T,   _D,   _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau2	    {->-}, Just sl) --
		{-!-}{-!-}{-!-}{-!-}{-!-}
     (_S2, _T2, _D2, _B2, _L2, _C2, _J2, tau2) <- meaningOfStatementListOpt{--}(_S1,  _T1,  _D1,  _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa  {->-}, Just (S0 s)) --
		{-!-}{-!-}{-!-}{-!-}{-!-}
     require $ (dom{--}(_L1) `intersection` dom{--}(_L2) == emptySet && dom{--}(_C1) `intersection` dom{--}(_C2) == emptySet)
     return $ (_S2, _T2, _D2, _B1 `mapUnion` _B2, _L1 `mapUnion` _L2, _C1 `mapUnion` _C2, _J1 `union` _J2, tau1)

meaningOfStatementListOpt {-sem-}(_S, _T, _D, _B, _L, _V, _I, t_r, t_s, nu_r, nu_c, nu_b, tau_kappa {->-}, Nothing) --
  = return $ (_S, _T, _D, emptyMap, emptyMap, emptyMap, emptySet, tau_kappa)
\end{code}

The reader should observe that, under the above translation, an empty statement list denotes simply its own continuation term "tau_kappa".

\end{subsection}
\end{section}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%