\set{capture}{hidden}
\begin{code}

module C.Types (
	basicIntegralTypes,
	isEnumType,
	isCompoundType,
	typeOfMember,
	nameOfMember,
	offsetOfMember,
	setOfMembers,
	tagOfType,
	suOfType,
	membersOfType,
	isBitFieldType,
	widthOfType,
	offsetOfType,
	isSignedType,
	isUnsignedType,
	isCharType,
	isFloatingType,
	isPointerType,
	isArrayType,
	lengthOfType,
	isObjectType,
	isFunctionType,
	prototypeOfType,
	isValidPrototype,
	isVoidType,
	isIncompleteType,
	isIntegralType,
	isArithmeticType,
	isScalarType,
	baseOfType,
	tqOfType,
	unqualifiedType,
	qualifiedType,
	requalifiedType,
	requalifiedMembers,
	recTqOfType,
	attrOfQualifier,
	glbOfType,
	lubOfType,
	envelopeOfType,
	attrOfType,
	sizeOfType,
	integralPromotedType,
	pointerPromotedType,
	argumentPromotedType,
	commonArithmeticType,
	areCompatibleTypes,
	areCompatiblePrototypesOpt,
	compositeType,
	compositeMembers,
	compositePrototypeOpt,
) where

import Additional
import C.Variables
import C.Syntax
import {-# SOURCE #-} C.Implementation
import Lambda

infixl 6 `qualifiedType`, `requalifiedType`, `commonArithmeticType`, `compositeType`
infix  4 `areCompatibleTypes`

\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Types}
\set{title}		{Types}
\begin{section}

During translation, many C entities such as function declarations and members of a structure or union object are annotated with various auxiliary information used to provide further information about the entity's semantics. In standard C nomenclature, these annotations are known as «types», although they bear little resemblance to their type-theoretic namesakes. In the actual source files, such types are usually depicted using the “"TypeNameT"” syntax defined later in Section \ref{C/Declarations/TypeNames}. However, for historical reasons, this syntax is unsuitable for a direct deployment during translation and, accordingly, most C compilers resort to an abstracted denotation of these entities during analysis of C programs. In the following presentation, these abstract entities are always referred to as «C types» in order to distinguish them from their Haskell cousins.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{TypeT}
\set{title}		{Abstract Syntax}
\begin{subsection}

The language defines fifteen distinct C type forms, described collectively by the following Haskell constructions:

\IndexType		{TypeT}						{type (\tstar)}
\IndexCons		{Signed}					{signed integer_specifier (type)}
\IndexCons		{Unsigned}					{unsigned integer_specifier (type)}
\IndexCons		{PlainCharType}					{char (type)}
\IndexCons		{PlainIntType}					{int (type)}
\IndexCons		{EnumType}					{enum type var (type)}
\IndexCons		{FloatType}					{float (type)}
\IndexCons		{DoubleType}					{double (type)}
\IndexCons		{LongDoubleType}				{long_double (type)}
\IndexCons		{PointerType}					{* type (type)}
\IndexCons		{CompoundType}					{ struct_or_union var members (type)}
\IndexCons		{ArrayType}					{[] type integer_opt (type)}
\IndexCons		{FunctionType}					{() type prototype_opt (type)}
\IndexCons		{BitFieldType}					{:@ type integer integer (type)}
\IndexCons		{VoidType}					{void (type)}
\IndexCons		{Qualified}					{type_qualifier type (type)}

\IndexType		{TypesT}					{types (\tstar)}

\IndexType		{IntegerSpecifierT}				{integer_specifier (\tstar)}
\IndexCons		{CharType}					{char (integer_specifier)}
\IndexCons		{ShortType}					{short (integer_specifier)}
\IndexCons		{IntType}					{int (integer_specifier)}
\IndexCons		{LongType}					{long (integer_specifier)}

\IndexType		{MemberT}					{member (\tstar)}
\IndexCons		{Member}					{@ type identifier_opt integer (member)}

\IndexType		{MembersT}					{members (\tstar)}

\IndexType		{PrototypeT}					{prototype (\tstar)}
\IndexCons		{Types}						{ types (prototype)}
\IndexCons		{Ellipsis}					{... types (prototype)}

\begin{syntax}
\InsertHaskell		{TypeT, \par, TypesT, \par, IntegerSpecifierT, \par, MemberT, \par, MembersT, \par, PrototypeT}
\end{syntax}

where the two Haskell types “"StructOrUnionT"” and “"TypeQualifierT"” used in the above definition are discussed separately in Sections \ref{C/Declarations/StructAndUnionSpecifiers} and\ref{C/Declarations/TypeQualifiers} later in this chapter. 

As described later in Section \ref{C/Declarations}, in the concrete syntax of the language most of these types are usually depicted by sequences of C tokens similar to those found in their abstract names given above. In particular, structure, union and enumeration types of the form “"CompoundType (Struct) (nu)"”, “"CompoundType (Union) (nu)"” and “"EnumType (t) (nu)"” are introduced into C programs using the “"StructOrUnionSpecifierT"” and “"EnumSpecifierT"” entities defined in Sections \ref{C/Declarations/StructAndUnionSpecifiers} and \ref{C/Declarations/EnumerationSpecifiers}. On the other hand, pointer, array and function types are represented by the concrete syntax of C «declarators» discussed in Section \ref{C/Declarations/Declarators}, whose enormous complexity is the main reason for introduction of the above abstract structures into the present formal specification of the language. Finally,
«bit field types» of the form “"BitFieldType (t) {-:-} (n) {-@-} (m)"” can be introduced only as part of a larger structure or union declaration as described in Section \ref{C/Declarations/StructAndUnionSpecifiers}. Within such bit fields, the “"PlainIntType"” type assumes a slightly unusual semantics, whereas, in every other context, it is precisely equivalent to “"Signed IntType"”. 

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Types/Corresponding}
\set{title}		{Corresponding Types}
\begin{subsection}

Many type properties are left unspecified in the C Standard, with an intention that a suitable choice of their values should be made in consideration of various architecture-specific factors. In the following presentation, such properties of C types are usually described using Haskell signatures and theorems instead of complete definitions. Nevertheless, most of the partially-specified type forms are guaranteed to share certain properties with some other predetermined entity known as that form's «corresponding type». Five such designated C types are introduced by the following Haskell signature:

\begin{haskell}
charType, intType, wideCharType, ptrDiffType, sizeType :: TypeT
\end{haskell}

Informally, “"charType"” and “"intType"” depict the corresponding types of their “"PlainCharType"” and “"PlainIntType"” variants, with “"intType"” used only during analysis of the bit field type form \hbox{“"BitFieldType (PlainIntType) {-:-} (m) {-@-} (n)"”.} Both of these types are always guaranteed to be equal to a “"Signed"” or “"Unsigned"” version of “"CharType"” and “"IntType"”, respectively:

\NewHaskellToken	{wfCharTypeJ}				{Ord}	{\TKeyword{wf}\sub{\CSyntax{char\_t}}}
\NewHaskellToken	{wfIntTypeJ}				{Ord}	{\TKeyword{wf}\sub{\CSyntax{int\_t}}}

\NewHaskellDescription	{wfCharTypeJ}					{Admissiblity of the “\haskell{charType}” type}
\NewHaskellDescription	{wfIntTypeJ}					{Admissiblity of the “\haskell{intType}” type}

\IndexTheorem		{wfCharTypeJ}					{WF_char_t Admissiblity of the char_t type}
\IndexTheorem		{wfIntTypeJ}					{WF_int_t Admissiblity of the int_t type}

\begin{haskell}
wfCharTypeJ		:: {-sem-}(charType 		@@ listToSet [Signed CharType, 	Unsigned CharType])
wfIntTypeJ		:: {-sem-}(intType		@@ listToSet [Signed IntType,	Unsigned IntType])
\end{haskell}

Of the remaining three types, “"wideCharType"” is used solely in formalisation of certain «initialiser» forms described in Section \ref{C/Declarations/Initialisers}, while “"ptrDiffType"” and “"sizeType"” are required to capture various aspects of pointer arithmetic discussed throughout Section \ref{C/Expressions}, without committing to its details in the generic specification of the language. Each of these types is guaranteed to be equal to one of the ten «basic integral types», as captured by the following three theorems:

\NewHaskellToken	{wfWideCharTypeJ}			{Ord}	{\TKeyword{wf}\sub{\CSyntax{wchar\_t}}}
\NewHaskellToken	{wfPtrDiffTypeJ}			{Ord}	{\TKeyword{wf}\sub{\CSyntax{ptrdiff\_t}}}
\NewHaskellToken	{wfSizeTypeJ}				{Ord}	{\TKeyword{wf}\sub{\CSyntax{size\_t}}}

\NewHaskellDescription	{wfWideCharTypeJ}				{Admissiblity of the “\haskell{wideCharType}” type}
\NewHaskellDescription	{wfPtrDiffTypeJ}				{Admissiblity of the “\haskell{ptrDiffType}” type}
\NewHaskellDescription	{wfSizeTypeJ}					{Admissiblity of the “\haskell{sizeType}” type}

\IndexTheorem		{wfWideCharTypeJ}				{WF_wchar_t Admissiblity of the wchar_t type}
\IndexTheorem		{wfPtrDiffTypeJ}				{WF_ptrdiff_t Admissiblity of the ptrdiff_t type}
\IndexTheorem		{wfSizeTypeJ}					{WF_size_t Admissiblity of the size_t type}

\begin{haskell}
wfWideCharTypeJ		:: {-sem-}(wideCharType		@@ basicIntegralTypes {-,-})
wfPtrDiffTypeJ		:: {-sem-}(ptrDiffType		@@ basicIntegralTypes {-,-})
wfSizeTypeJ		:: {-sem-}(sizeType		@@ basicIntegralTypes {-,-})
\end{haskell}

where “"basicIntegralTypes"\,” represents the complete set of all basic integral type forms. Formally, this set consists of the eight “"Signed"\,” and “"Unsigned"\,” type variants together with their plain “"CharType"” and “"IntType"” cousins, as specified by the following Haskell definition:

\NewHaskellToken	{basicIntegralTypes}			{Ord}	{«BIT»}
\NewHaskellResult	{basicIntegralTypes}				{SetT TypeT}
\IndexTerm		{basicIntegralTypes}				{BIT (types)}

\set{capture}		{hidden}
\begin{code}
basicIntegralTypes :: SetT TypeT
\end{code}

\begin{code}
basicIntegralTypes = listToSet [
     Signed CharType, Signed ShortType, Signed IntType, Signed LongType,
     Unsigned CharType, Unsigned ShortType, Unsigned IntType, Unsigned LongType,
     PlainCharType, PlainIntType
{--}]
\end{code}

An example of the most popular implementation of the five corresponding types described above, also suitable for the MMIX architecture from Chapter \ref{MMIX}, is presented separately in Appendix \ref{C/Implementation}.

The reader should also observe that every enumeration type form “"EnumType (t) (nu)"” corresponds to another basic integral type "t". That type, however, is always specified explicitly by every instance of such an entity, since, in some C compilers, its choice may vary between enumerations. For a further discussion of this issue, an interested reader is referred to the definition of enumeration specifiers presented later in \hbox{Section \ref{C/Declarations/EnumerationSpecifiers}.}

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Types/Properties}
\set{title}		{Properties of Types}
\begin{subsection}

For convenience, the above syntactic forms of C types are grouped into a number of «type families», every one of which is identified by an appropriate Haskell predicate function. In particular, types of the form “"EnumType (t) (nu)"”, and all “"ConstQualifier"” and “"VolatileQualifier"”-qualified versions thereof, are known as «enumeration types». Such types are meaningful only if "t" represents one of the ten basic integral types described previously in Section \ref{C/Types/Corresponding} and if no type qualifier is applied to the enumeration more than once in its syntax. Formally, this group of types is identified by the following predicate “"isEnumType"”:

\BeginHaskellToken	{isEnumType}				{Open}	{ET}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isEnumType}					{ET type (bool)}

\begin{code}
isEnumType	{-[*]-} :: TypeT -> Bool

isEnumType	{-sem-}(EnumType (t) (nu))				= t @@ basicIntegralTypes
isEnumType	{-sem-}(Qualified (q) (t))				= q /@ tqOfType{--}(t) && isEnumType{--}(t)
isEnumType	{-sem-}(other)						= False
\end{code}

in which the notation “"q /@ tqOfType{--}(t)"” is discussed later in Section \ref{C/Types/Qualifiers}.

Similarly, the family of «bit field types» is identified by the predicate “"isBitFieldType"”. It consists of all qualified or unqualified versions of the type forms “"BitFieldType (Signed IntType) {-:-} (m) {-@-} (n)"”, “"BitFieldType (Unsigned IntType) {-:-} (m) {-@-} (n)"” and “"BitFieldType (PlainIntType) {-:-} (m) {-@-} (n)"”, provided that both "m" and "n" represent non-negative integers and "m" has a value no greater than "byteWidth * sizeOfType{--}(t) - n", where "byteWidth" is the byte width parameters from Section \ref{Lambda/Atoms} and "sizeOfType{--}(t)" represents the «size» of the C type "t", as described later in Section \ref{C/Types/Storage}. Formally:

\BeginHaskellToken	{isBitFieldType}			{Open}	{BF}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isBitFieldType}				{BF type (bool)}

\begin{code}
isBitFieldType	{-[*]-} :: TypeT -> Bool

isBitFieldType 	{-sem-}(BitFieldType (t) {-:-} (m) {-@-} (n))		= 	t @@ listToSet [Signed IntType, Unsigned IntType, PlainIntType] &&
										n >= 0 && inRange (0) (<=) (m) (<=) (byteWidth * sizeOfType{--}(t) - n)
isBitFieldType	{-sem-}(Qualified (q) (t))				= 	q /@ tqOfType{--}(t) && isBitFieldType{--}(t)
isBitFieldType	{-sem-}(other)						= 	False
\end{code}

In such types, the integers "m" and "n" are known as the «bit width» and «offset», respectively. For convenience, they can be retrieved from an arbitrary bit field type using the following pair of Haskell functions:

\BeginHaskellToken	{widthOfType}				{Open}	{\ZapfChancery{W}}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Integer}
\EndHaskellToken

\BeginHaskellToken	{offsetOfType}				{Open}	{\ZapfChancery{O}}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{widthOfType}					{W type (integer)}
\IndexTerm		{offsetOfType}					{O type (integer)}

\begin{code}
widthOfType {-[*]-}, offsetOfType {-[*]-} :: TypeT -> Integer

widthOfType	{-sem-}(BitFieldType (t) {-:-} (m) {-@-} (n)) 		= m
widthOfType	{-sem-}(Qualified (q) (t))				= widthOfType{--}(t)

offsetOfType	{-sem-}(BitFieldType (t) {-:-} (m) {-@-} (n)) 		= n
offsetOfType	{-sem-}(Qualified (q) (t))				= offsetOfType{--}(t)
\end{code}

Intuitively, the offset of a bit field type determines its location within a larger object described by the associated base type, while its width represents the number of actual bits occupied by the object.

Collectively, the four “"Signed"\,” type forms, “"PlainIntType"”, “"BitFieldType (Signed IntType) {-:-} (m) {-@-} (n)"” and all qualified versions thereof are known as «signed integral types». Further, the plain “"PlainCharType"” type, “"BitFieldType (PlainIntType) {-:-} (m) {-@-} (n)"” and enumeration types “"EnumType (t) (nu)"” are included in this family if their corresponding types “"charType"”, “"BitFieldType (intType) {-:-} (m) {-@-} (n)"” and "t" are themselves signed. In Haskell, these criteria can be captured concisely by the following Haskell \hbox{predicate “"isSignedType"”:}

\BeginHaskellToken	{isSignedType}				{Open}	{ST}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isSignedType}					{ST type (bool)}

\begin{code}
isSignedType	{-[*]-} :: TypeT -> Bool

isSignedType	{-sem-}(Signed CharType)					= True
isSignedType	{-sem-}(Signed ShortType)					= True
isSignedType	{-sem-}(Signed IntType)						= True
isSignedType	{-sem-}(Signed LongType)					= True
isSignedType 	{-sem-}(PlainIntType)						= True
isSignedType 	{-sem-}(PlainCharType)						= isSignedType{-sem-}(charType)
isSignedType 	{-sem-}(EnumType (t) (nu))					= isEnumType{-sem-}(EnumType (t) (nu)) && isSignedType{--}(t)
isSignedType 	{-sem-}(BitFieldType (Signed IntType) {-:-} (m) {-@-} (n))	= isBitFieldType{-sem-}(BitFieldType (Signed IntType) {-:-} (m) {-@-} (n))
isSignedType 	{-sem-}(BitFieldType (PlainIntType) {-:-} (m) {-@-} (n))	= isSignedType{-sem-}(BitFieldType (intType) {-:-} (m) {-@-} (n))
isSignedType	{-sem-}(Qualified (q) (t))					= q /@ tqOfType{--}(t) && isSignedType{--}(t)
isSignedType	{-sem-}(other)							= False
\end{code}

Similarly, the family of «unsigned integral types», identified with the predicate “"isUnsignedType"”, is defined trivially as follows:

\BeginHaskellToken	{isUnsignedType}			{Open}	{UT}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isUnsignedType}				{UT type (bool)}

\begin{code}
isUnsignedType	{-[*]-} :: TypeT -> Bool

isUnsignedType	{-sem-}(Unsigned CharType)					= True
isUnsignedType	{-sem-}(Unsigned ShortType)					= True
isUnsignedType	{-sem-}(Unsigned IntType)					= True
isUnsignedType	{-sem-}(Unsigned LongType)					= True
isUnsignedType 	{-sem-}(PlainCharType)						= isUnsignedType{-sem-}(charType)
isUnsignedType 	{-sem-}(EnumType (t) (nu))					= isEnumType{-sem-}(EnumType (t) (nu)) && isUnsignedType{--}(t)
isUnsignedType 	{-sem-}(BitFieldType (Unsigned IntType) {-:-} (m) {-@-} (n))	= isBitFieldType{-sem-}(BitFieldType (Signed IntType) {-:-} (m) {-@-} (n))
isUnsignedType 	{-sem-}(BitFieldType (PlainIntType) {-:-} (m) {-@-} (n))	= isUnsignedType{-sem-}(BitFieldType (intType) {-:-} (m) {-@-} (n))
isUnsignedType	{-sem-}(Qualified (q) (t))					= q /@ tqOfType{--}(t) && isUnsignedType{--}(t)
isUnsignedType	{-sem-}(other)							= False
\end{code}

Further, the three basic types “"PlainCharType"”, “"Signed CharType"” and “"Unsigned CharType"” are often referred to as «character types». They are identified by the following Haskell predicate, observing that the wide character type “"wideCharType"” is excluded from this type family, even though, in a sense, it is also used to depict character values:

\BeginHaskellToken	{isCharType}				{Open}	{CHR}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isCharType}					{CHR type (bool)}

\begin{code}
isCharType		{-[*]-} :: TypeT -> Bool

isCharType		{-sem-}(PlainCharType)				= True
isCharType		{-sem-}(Signed CharType)			= True
isCharType		{-sem-}(Unsigned CharType)			= True
isCharType		{-sem-}(Qualified (q) (t))			= q /@ tqOfType{--}(t) && isCharType{--}(t)
isCharType		{-sem-}(other)					= False
\end{code}

More so, the family of «floating types» consists of the three syntactic forms “"FloatType"”, “"DoubleType"” and “"LongDoubleType"”, together with all properly-qualified versions thereof. Intuitively, these types describe «floating point numbers», usually of the kind defined in the «IEEE Standard for Binary Floating-Point Arithmetic» [IEEE754]. Formally, they are identified as follows:

\BeginHaskellToken	{isFloatingType}			{Open}	{FLT}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isFloatingType}				{FLT type (bool)}

\begin{code}
isFloatingType 		{-[*]-} :: TypeT -> Bool

isFloatingType		{-sem-}(FloatType)				= True
isFloatingType		{-sem-}(DoubleType)				= True
isFloatingType		{-sem-}(LongDoubleType)				= True
isFloatingType		{-sem-}(Qualified (q) (t))			= q /@ tqOfType{--}(t) && isFloatingType{--}(t)
isFloatingType		{-sem-}(other)					= False
\end{code}
 
In C, qualified and unqualified types of the form “"PointerType (t) {-*-}"” are known as «pointer types». In this work, they are associated with the type family predicate “"isPointerType"”. The type "t" from which such pointers are derived must represent a valid object, function or incomplete type, as depicted by the “"isObjectType"”, “"isFunctionType"” and “"isIncompleteType"” predicates defined shortly:

\BeginHaskellToken	{isPointerType}				{Open}	{PTR}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isPointerType}					{PTR type (bool)}

\begin{code}
isPointerType	{-[*]-} :: TypeT -> Bool

isPointerType	{-sem-}(PointerType (t) {-*-})				= isObjectType{--}(t) || isFunctionType{--}(t) || isIncompleteType{--}(t)
isPointerType	{-sem-}(Qualified (q) (t))				= q /@ tqOfType{--}(t) && isPointerType{--}(t)
isPointerType	{-sem-}(other)						= False
\end{code}

Collectively, all signed and unsigned types are known simply as «integral types», which, in combination with the floating type family, also form the set of all «arithmetic types» supported by the C language. More so, arithmetic and pointer types constitute the family of «scalar types» which, intuitively, characterise all C variables whose values are translated into simple Etude atoms. Formally, these three auxiliary type families are depicted by the following predicates “"isIntegralType"”, “"isArithmeticType"” and “"isScalarType"”:

\BeginHaskellToken	{isIntegralType}			{Open}	{INT}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\BeginHaskellToken	{isArithmeticType}			{Open}	{AT}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\BeginHaskellToken	{isScalarType}				{Open}	{SCR}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isIntegralType}				{INT type (bool)}
\IndexTerm		{isArithmeticType}				{AT type (bool)}
\IndexTerm		{isScalarType}					{SCR type (bool)}

\begin{code}
isIntegralType		{-[*]-}, isArithmeticType {-[*]-}, isScalarType {-[*]-} :: TypeT -> Bool

isIntegralType		{-sem-}(t) = isSignedType		{--}(t) || isUnsignedType		{--}(t)
isArithmeticType	{-sem-}(t) = isIntegralType		{--}(t) || isFloatingType		{--}(t)
isScalarType		{-sem-}(t) = isArithmeticType		{--}(t) || isPointerType		{--}(t)
\end{code}

Further, the family of «structure and union types» consists of all correctly-qualified and unqualified versions of constructs with the abstract syntax of “"CompoundType (Struct) (nu) (m_set)"” and “"CompoundType (Union) (nu) (m_set)"”, in which "m_set" represents a list of «structure or union members». Formally, such types are identified by the Haskell predicate “"isCompoundType"”, which can be defined \hbox{as follows:}

\BeginHaskellToken	{isCompoundType}			{Open}	{SU}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isCompoundType}				{SU type (bool)}

\begin{code}
isCompoundType	{-[*]-} :: TypeT -> Bool

isCompoundType	{-sem-}(CompoundType (Struct)  (nu) (m_set))	= True
isCompoundType	{-sem-}(CompoundType (Union)   (nu) (m_set))	= True
isCompoundType	{-sem-}(Qualified (q) (t))			= q /@ tqOfType{--}(t) && isCompoundType{--}(t)
isCompoundType	{-sem-}(other)					= False
\end{code}

As exposed by the earlier BNF grammar, every member of such a type consists of a «member type», an optional identifier and a non-negative integer known as the member's «offset», which, informally, specifies the precise location of the member within an entire structure or union as a byte offset into the corresponding Etude object. The three components of every member entity can be extracted individually from its abstract syntax using the following trio of auxiliary Haskell definitions:

\BeginHaskellToken	{typeOfMember}				{Open}	{\ZapfChancery{T}}
  \HaskellArgument							{MemberT}
  \HaskellResult							{TypeT}
\EndHaskellToken

\BeginHaskellToken	{nameOfMember}				{Open}	{\ZapfChancery{N}}
  \HaskellArgument							{MemberT}
  \HaskellResult							{Maybe IdentifierT}
\EndHaskellToken

\BeginHaskellToken	{offsetOfMember}			{Open}	{\ZapfChancery{O}}
  \HaskellArgument							{MemberT}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{typeOfMember}					{T member (type)}
\IndexTerm		{nameOfMember}					{N member (identifier_opt)}
\IndexTerm		{offsetOfMember}				{O member (integer)}

\begin{code}
typeOfMember	{-[*]-} :: MemberT -> TypeT				-- type
nameOfMember	{-[*]-} :: MemberT -> Maybe IdentifierT			-- name
offsetOfMember	{-[*]-} :: MemberT -> Integer				-- offset

typeOfMember	{-sem-}(Member (t) (x_opt) {-@-} (n))	= t
nameOfMember	{-sem-}(Member (t) (x_opt) {-@-} (n))	= x_opt
offsetOfMember	{-sem-}(Member (t) (x_opt) {-@-} (n))	= n
\end{code}

In practice, every member list constitutes a unique name space for its member names and, accordingly, such lists are usually viewed as sets of members indexed by their associated identifiers. This natural interpretation can be formalised in Haskell using the following implicit coercion function:

\BeginHaskellToken	{setOfMembers}				{Void}	{}
  \HaskellArgument							{MembersT}
  \HaskellResult							{(MapT IdentifierT MemberT)}
\EndHaskellToken

\IndexTerm		{setOfMembers}					{ members (identifiers member)}

\begin{code}
setOfMembers {-[*]-} :: MembersT -> (MapT IdentifierT MemberT)

setOfMembers {-sem-}(m_set) = listToMap [ {-raw-}(unJust (nameOfMember{--}(m_k)) {-:-}, m_k) | m_k <- m_set, nameOfMember{--}(m_k) /= Nothing ]
\end{code}

In other words, the name space introduced by a given structure or union type consists of all named members of that structure or union, with each identifier appearing in such a member mapped to the member entity itself. Anonymous members without an identifier are explicitly excluded from this name space. Observe that the above conversion function is treated as an implicit coercion whose name is always omitted from the presentation, since its existence can be inferred trivially from the context of the surrounding Haskell expression. 

The Etude variable associated with every enumeration, structure and union type is known as that type's «tag» and may be obtained from its abstract syntax using the notation “"tagOfType{--}(t)"”, whose obvious implementation can be captured in Haskell \hbox{as follows:}

\BeginHaskellToken	{tagOfType}				{Open}	{tag}
  \HaskellArgument							{TypeT}
  \HaskellResult							{VarT}
\EndHaskellToken

\IndexTerm		{tagOfType}					{tag type (\nu)}

\begin{code}
tagOfType 	{-[*]-} :: TypeT -> VarT

tagOfType 	{-sem-}(EnumType (t) (nu))				= nu
tagOfType 	{-sem-}(CompoundType (struct_or_union) (nu) (m_set))	= nu
tagOfType 	{-sem-}(Qualified (q) (t))				= tagOfType{--}(t)
\end{code}

Such tags are used solely to formalise the equivalence relation for structure, union and enumeration types. In this work, they are associated with Etude variables only in order to simplify generation of globally-unique type tags in Sections \ref{C/Scopes} and \ref{C/Declarations}.

Similarly, the “"struct_or_union"” and member list components of a structure or union type can be extracted from its abstract syntax using the following simple constructions:

\BeginHaskellToken	{suOfType}				{Open}	{su}
  \HaskellArgument							{TypeT}
  \HaskellResult							{StructOrUnionT}
\EndHaskellToken

\BeginHaskellToken	{membersOfType}				{Open}	{«\macron m»}
  \HaskellArgument							{TypeT}
  \HaskellResult							{MembersT}
\EndHaskellToken

\IndexTerm		{suOfType}					{su type (struct_or_union)}
\IndexTerm		{membersOfType}					{m- type (members)}

\begin{code}
suOfType 	{-[*]-} :: TypeT -> StructOrUnionT

suOfType 	{-sem-}(CompoundType (struct_or_union) (nu) (m_set))	= struct_or_union
suOfType 	{-sem-}(Qualified (q) (t))				= suOfType{--}(t)
{--}
membersOfType 	{-[*]-} :: TypeT -> MembersT

membersOfType 	{-sem-}(CompoundType (struct_or_union) (nu) (m_set))	= m_set
membersOfType 	{-sem-}(Qualified (q) (t))				= membersOfType{--}(t)
\end{code}

Unqualified types of the form “"ArrayType (t) (n_opt)"” are known as «array types». In all cases, such types must be derived from a valid member of the object type family, as identified by the “"isObjectType"” predicate described shortly, together with an optional number "n", that, when specified, must represent a strictly-positive integer no greater than "lubOfType{--}(ptrDiffType) / toRational (sizeOfType{--}(t))" in magnitude. Formally:

\BeginHaskellToken	{isArrayType}				{Open}	{ARR}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isArrayType}					{ARR type (bool)}

\begin{code}
isArrayType	{-[*]-} :: TypeT -> Bool

isArrayType	{-sem-}(ArrayType (t) (n_opt))		= isObjectType{--}(t) && --
							  (n_opt == Nothing || --
							   inRange (0) (<) (toRational (unJust n_opt)) (<=) (lubOfType{--}(ptrDiffType) / toRational (sizeOfType{--}(t))))
isArrayType	{-sem-}(other)				= False
\end{code}

using the “"sizeOfType{--}(t)"” and “"lubOfType{--}(t)"” notations discussed separately in Sections \ref{C/Types/Storage} and \ref{C/Types/Representation}. The optional integer component of an array type is known as the array's «length». Intuitively, it specifies the number of discrete objects stored in that array. For convenience, the length of every array type can be determined by the following function:

\BeginHaskellToken	{lengthOfType}				{Open}	{length}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{lengthOfType}					{length type (integer)}

\begin{code}
lengthOfType 	{-[*]-} :: TypeT -> Maybe Integer

lengthOfType 	{-sem-}(ArrayType (t) (n_opt)) 		= n_opt
\end{code}

Collectively, all of the above types with the exception of bit fields, arrays of unspecified length and structure or union types with an empty member list form an important family of «object types», which, intuitively, can be used to describe C variables with predictable storage requirements. In Haskell, this family can be identified as follows:

\BeginHaskellToken	{isObjectType}				{Open}	{OBJ}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isObjectType}					{OBJ type (bool)}

\begin{code}
isObjectType	{-[*]-} :: TypeT -> Bool

isObjectType 	{-sem-}(t)	=	unqualifiedType{--}(t) @@ basicIntegralTypes ||
					isFloatingType{--}(t) || isEnumType{--}(t) || isPointerType{--}(t) ||
					(isArrayType{--}(t) && lengthOfType{--}(t) /= Nothing) ||
					(isCompoundType{--}(t) && membersOfType{--}(t) /= {-empty-}[])
\end{code}

More so, types of the form “"FunctionType (t) (prototype_opt)"\!” represent «function types», which are never qualified and must be derived from an object or incomplete type other than an array. In Haskell, these types are identified by the following predicate:

\BeginHaskellToken	{isFunctionType}			{Open}	{FUN}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isFunctionType}				{FUN type (bool)}

\begin{code}
isFunctionType	{-[*]-} :: TypeT -> Bool

isFunctionType	{-sem-}(FunctionType (t) (p_opt)) 	= (isObjectType{--}(t) || isIncompleteType{--}(t)) && not (isArrayType{--}(t)) && isValidPrototype{--}(p_opt)
isFunctionType	{-sem-}(other)				= False
\end{code}

and their optional prototype "p_opt" can be obtained by the following function:

\BeginHaskellToken	{prototypeOfType}			{Open}	{prot}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Maybe PrototypeT}
\EndHaskellToken

\IndexTerm		{prototypeOfType}				{prot type (prototype_opt)}

\begin{code}
prototypeOfType {-[*]-} :: TypeT -> Maybe PrototypeT

prototypeOfType	{-sem-}(FunctionType (t) (p_opt))	= p_opt
\end{code}

Informally, the prototype of a C function characterises any argument values expected by every call to that function as described in Section \ref{C/Expressions}. An empty prototype indicates that no information about the arguments' types is available to the compiler, while a prototype of the form “"Ellipsis (t_set) {-...-}"\!” signals that the function may accept additional arguments beyond those specified by its prototype's type list. When present, a prototype without the “\!"{-...-}{--}"\!” suffix must be formed only from valid object, incomplete and function types, while prototypes with the “\!"{-...-}{--}"\!”, are considered to be well-formed if they specify a non-empty list of types and if the corresponding prototype without that suffix is also deemed valid. Formally:

\BeginHaskellToken	{isValidPrototype}			{Open}	{WF}
  \HaskellArgument							{Maybe PrototypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isValidPrototype}				{WF prototype_opt (bool)}

\begin{code}
isValidPrototype {-[*]-} :: (Maybe PrototypeT) -> Bool

isValidPrototype {-sem-}(Nothing)				= True
isValidPrototype {-sem-}(Just (Types (t_set)))			= and [ isObjectType{--}(t_k) || isFunctionType{--}(t_k) || isIncompleteType{--}(t_k) | t_k <- t_set {-,-} ] --
									&& t_set /= {-empty-}[]
isValidPrototype {-sem-}(Just (Ellipsis (t_set) {-...-}))	= isValidPrototype{--}(Just (Types (t_set))) && t_set /= {-empty-}[]
\end{code}

Although a well-formed C program may never define a function with an incomplete parameter type, a careful perusal of the C standard reveals that nonsensical constructs such as “"FunctionType (PlainIntType) (VoidType, VoidType, VoidType)"\!” are still valid within “"SizeOf"” expressions and cast operations. Accordingly, they are supported in this work, although virtually no other C compiler provides a correct implementations of these types.

The special type “"VoidType"\,” is known as the «void type». All of its qualified and unqualified variants are identified collectively by the following predicate “"isVoidType"”:

\BeginHaskellToken	{isVoidType}				{Open}	{VT}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isVoidType}					{VT type (bool)}

\begin{code}
isVoidType	{-[*]-} :: TypeT -> Bool

isVoidType	{-sem-}(VoidType)					= True
isVoidType	{-sem-}(Qualified (q) (t))				= q /@ tqOfType{--}(t) && isVoidType{--}(t)
isVoidType	{-sem-}(other)						= False
\end{code}

The “"VoidType"\,” type represents a distinguished member of the «incomplete type family», which, intuitively, describes objects with unknown storage requirements. Other examples of incomplete types include arrays of unspecified length and structures or unions with empty member lists. Formally, these rather unusual entities are identified by the collective predicate “"isIncompleteType"”:

\BeginHaskellToken	{isIncompleteType}			{Open}	{INC}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isIncompleteType}				{INC type (bool)}

\begin{code}
isIncompleteType {-[*]-} :: TypeT -> Bool

isIncompleteType {-sem-}(t) = isVoidType{--}(t) || (isArrayType{--}(t) && lengthOfType{--}(t) == Nothing) || (isCompoundType{--}(t) && membersOfType{--}(t) == {-empty-}[])
\end{code}

A reader will observer that the structure of many C type forms defined at the beginning of this section is «derived» from some other type entity, known as that type's «base». In particular, the base of an enumeration or bit field type defines that type's corresponding basic integral derivation, while the base of a pointer type represents the type of an object addressed by the pointer. Similarly, the base of an array type describes the array's individual elements, while, for functions, it characterises the entity's returned value. For convenience, the notion of the base type is captured collectively by the following Haskell definition:

\BeginHaskellToken	{baseOfType}				{Open}	{\ZapfChancery{B}}
  \HaskellArgument							{TypeT}
  \HaskellResult							{TypeT}
\EndHaskellToken

\IndexTerm		{baseOfType}					{B type (type)}

\begin{code}
baseOfType {-[*]-} :: TypeT -> TypeT

baseOfType {-sem-}(EnumType (t) (nu)) 					= t
baseOfType {-sem-}(PointerType (t) {-*-})				= t
baseOfType {-sem-}(ArrayType (t) (n_opt)) 				= t
baseOfType {-sem-}(FunctionType (t) (p_opt)) 				= t
baseOfType {-sem-}(BitFieldType (t) {-:-} (m) {-@-} (n)) 		= t
baseOfType {-sem-}(Qualified (q) (t))					= baseOfType{--}(t)
\end{code}

For clarity, in the C Standard the base type of a pointer, array or function type is usually referred to as the entity's «referenced», «element» or «returned type», respectively.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Types/Qualifiers}
\set{title}		{Type Qualifiers}
\begin{subsection}

As already hinted in the previous section, most C types can be «qualified» with one or both of the keywords “"ConstQualifier"” and “"VolatileQualifier"”. Intuitively, a type whose qualification includes the “"ConstQualifier"” keyword is said to be «constant-qualified». Such types describe objects whose values remain immutable throughout execution of a well-formed C program. Similarly, a type that includes the “"VolatileQualifier"” qualifier is said to be «volatile-qualified». Intuitively, these types describe objects such as memory-mapped hardware device registers, whose values may be modified by means outside of the program's explicit control. The complete set of all such qualifiers is derived from a type's syntax as follows:

\BeginHaskellToken	{tqOfType}				{Open}	{tq}
  \HaskellArgument							{TypeT}
  \HaskellResult							{SetT TypeQualifierT}
\EndHaskellToken

\IndexTerm		{tqOfType}					{tq type (type_qualifiers)}

\begin{code}
tqOfType {-[*]-} :: TypeT -> SetT TypeQualifierT

tqOfType {-sem-}(Qualified (q) (t))	= listToSet [q] `union` tqOfType{--}(t)
tqOfType {-sem-}(other)			= emptySet
\end{code}

It should be pointed out that, since the above function returns a set of type qualifiers rather than a list, the precise order in which individual qualifiers appear within the syntax of a C type does not affect that type's core meaning, so that both of the syntactic forms \hbox{“"Qualified (ConstQualifier) (Qualified (VolatileQualifier) (t))"\,”} and \hbox{“"Qualified (VolatileQualifier) (Qualified (ConstQualifier) (t))"\,”} remain semantically indistinguishable everywhere within a C program.

Informally, every C type corresponds to a unique «unqualified type», obtained by stripping any type qualifier keywords from its syntax. Formally, this type is depicted by the notation “"unqualifiedType{--}(t)"”, which can be implemented in Haskell as follows:

\BeginHaskellToken	{unqualifiedType}			{Open}	{unq}
  \HaskellArgument							{TypeT}
  \HaskellResult							{TypeT}
\EndHaskellToken

\IndexTerm		{unqualifiedType}				{unq type (type)}

\begin{code}
unqualifiedType {-[*]-} :: TypeT -> TypeT

unqualifiedType {-sem-}(Qualified (q) (t))	= unqualifiedType{--}(t)
unqualifiedType {-sem-}(other)			= other
\end{code}

Finally, any type may be qualified with an arbitrary list of type qualifiers "q_set", which, in this work, is paraphrased as a list concatenation operation “"q_set `qualifiedType` t"\,”. When this notation is used to qualify an array type of the form “"ArrayType (t) (n_opt)"”, the result is an unqualified type of the same length, whose element type describes the qualified version of the original array's elements. Otherwise, the qualified type is obtained by composing the supplied list of type qualifiers "q_set" into a single function of the Haskell type “"TypeT -> TypeT"” using the list composition operator “"composeList"” described separately in Appendix \ref{Additional} and applying that function to the original type "t". Formally:

\BeginHaskellToken	{qualifiedType}				{Void}	{}
  \HaskellArgument							{[TypeQualifierT]}
  \HaskellToken		{{-++-}}				{Bin}	{\concat}
  \HaskellArgument							{TypeT}
  \HaskellResult							{TypeT}
\EndHaskellToken

\NewHaskellToken	{{-++-}}				{Bin}	{\concat}
\NewHaskellToken	{`qualifiedType`}			{Bin}	{\concat}

\IndexTerm		{qualifiedType}					{++ type_qualifiers type (type)}

\begin{code}
qualifiedType {-[*]-} 		{-++-} {-[*]-} :: [TypeQualifierT] -> TypeT -> TypeT

qualifiedType {-sem-}(q_set) 	{-++-} {-sem-}(ArrayType (t) (n_opt))	= {-sem-}(ArrayType ({-sem-}(q_set `qualifiedType` t)) (n_opt))
qualifiedType {-sem-}(q_set) 	{-++-} {-sem-}(t)			= (composeList (coerceList (Qualified) (q_set))){--}(t)
\end{code}

In certain contexts, it is also useful to «requalify» an already-qualified C type, in which case a union of the supplied type qualifiers "q_set" with the existing qualification of "t" is obtained and applied to the unqualified version of "t" using the above “"`qualifiedType`"” operator. In this work, such construction is paraphrased as the following set operation:

\BeginHaskellToken	{requalifiedType}			{Void}	{}
  \HaskellArgument							{SetT TypeQualifierT}
  \HaskellToken		{{-cup-}}				{Bin}	{\cup}
  \HaskellArgument							{TypeT}
  \HaskellResult							{TypeT}
\EndHaskellToken

\NewHaskellToken	{{-cup-}}				{Bin}	{\cup}
\NewHaskellToken	{`requalifiedType`}			{Bin}	{\cup}

\IndexTerm		{requalifiedType}				{\cup type_qualifiers type (type)}

\begin{code}
requalifiedType {-[*]-} 	{-cup-} {-[*]-} :: SetT TypeQualifierT -> TypeT -> TypeT

requalifiedType {-sem-}(q_set) 	{-cup-} {-sem-}(ArrayType (t) (n_opt))	= {-sem-}(ArrayType ({-sem-}(q_set `requalifiedType` t)) (n_opt))
requalifiedType {-sem-}(q_set) 	{-cup-} {-sem-}(t)			= setToList ((q_set `union` tqOfType{--}(t))) `qualifiedType` (unqualifiedType{--}(t))
\end{code}

Finally, in Section \ref{C/Expressions/Values}, requalification can be also applied to the list of members of a structure or union type, as implemented by the following simple list composition:

\BeginHaskellToken	{requalifiedMembers}			{Void}	{}
  \HaskellArgument							{SetT TypeQualifierT}
  \HaskellToken		{{-cup-}}				{Bin}	{\cup}
  \HaskellArgument							{MembersT}
  \HaskellResult							{MembersT}
\EndHaskellToken

\NewHaskellToken	{`requalifiedMembers`}			{Bin}	{\cup}

\IndexTerm		{requalifiedMembers}				{\cup type_qualifiers members (members)}

\begin{code}
requalifiedMembers {-[*]-} {-cup-} {-[*]-} :: SetT TypeQualifierT -> MembersT -> MembersT

requalifiedMembers {-sem-}(q_set) {-cup-} {-sem-}(m_set) ---
	= [ {-sem-}(Member ({-sem-}(q_set `requalifiedType` t_k)) (x_k) {-@-} (n_k)) | {-sem-}(Member (t_k) (x_k) {-@-} (n_k)) <- m_set ]
\end{code}

In certain contexts, we must also consider the «recursive qualification» of a structure or union type "t", which extends the set "tqOfType{--}(t)" with any qualifiers hidden among the construct's individual members, together with the qualification of any array element types found in its syntax. Formally, the recursive qualification of a C type is defined by induction over the entity's structure as follows:

\BeginHaskellToken	{recTqOfType}				{Open}	{rtq}
  \HaskellArgument							{TypeT}
  \HaskellResult							{SetT TypeQualifierT}
\EndHaskellToken

\IndexTerm		{recTqOfType}					{rtq type (type_qualifiers)}

\begin{code}
recTqOfType {-[*]-} :: TypeT -> SetT TypeQualifierT

recTqOfType {-sem-}(t) ---
    | isCompoundType{--}(t)		= tqOfType{--}(t) `union` unions [ recTqOfType{--}(typeOfMember{--}(m_k)) | m_k <- membersOfType{--}(t) ]
    | isArrayType{--}(t)		= tqOfType{--}(t) `union` recTqOfType{--}(baseOfType{--}(t))
    | otherwise				= tqOfType{--}(t)
\end{code}

Observe that the result of this construction is well-formed only if the operand does not represent a function type, or else if the specified set of type qualifiers is empty, since function types cannot be qualified in a well-formed C program.

The two “"ConstQualifier"” and “"VolatileQualifier"” type qualifiers correspond naturally to the Etude access attributes “"CM"” and “"VM"”, a fact that is easily exploited by the following Haskell mapping of these constructs:

\BeginHaskellToken	{attrOfQualifier}			{Open}	{«\mu»}
  \HaskellArgument							{TypeQualifierT}
  \HaskellResult							{AttrT}
\EndHaskellToken

\IndexTerm		{attrOfQualifier}				{\mu type_qualifier (attr)}

\begin{code}
attrOfQualifier {-[*]-} :: TypeQualifierT -> AttrT

attrOfQualifier {-sem-}(ConstQualifier)		= {-sem-}(CM)
attrOfQualifier {-sem-}(VolatileQualifier)	= {-sem-}(VM)
\end{code}

In Section \ref{C/Types/Storage}, this correspondence is utilised to construct Etude envelopes suitable for depiction of C variable objects in an address space of the translated program.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Types/Representation}
\set{title}		{Representation of Values}
\begin{subsection}

As discussed in Chapter \ref{Lambda}, many unspecified aspects of the various binary encoding schemes used in Etude for representation of its numeric quantities is captured by the notion of an atomic «format». Likewise, every C object and bit field type is naturally associated with one of these formats by an unspecified function "formatOfType{--}(t)", as exposed by the following Haskell signature:

\ReindexTerm		{formatOfType}

\begin{haskell}
formatOfType {-[*]-} :: TypeT -> FormatT
\end{haskell}

% In the generic fragment of C semantics, this functions is described indirectly through a set of theorems rather than a concrete Haskell definition. 

\NewHaskellToken	{wfCReprJ}				{Ord}	{\TKeyword{repr}}
\NewHaskellDescription	{wfCReprJ}					{Properties of C formats}
\IndexTheorem		{wfCReprJ}					{REPR Properties of C formats}

In particular, under every implementation of the C language, the format of all signed, unsigned and floating types belongs to the “"ZG"”, “"NG"” and “"RG"” genre, respectively:

\begin{haskell}
wfCReprJ1    	:: forall t => {-sem-}(isSignedType    {--}(t)) -> (WfJ{-sem-}(formatOfType{--}(t)), {-sem-}(genreOfFormat{--}(formatOfType{--}(t)) == {-sem-}({-hfil-}ZG  )))
wfCReprJ2  	:: forall t => {-sem-}(isUnsignedType  {--}(t)) -> (WfJ{-sem-}(formatOfType{--}(t)), {-sem-}(genreOfFormat{--}(formatOfType{--}(t)) == {-sem-}({-hfil-}NG  )))
wfCReprJ3	:: forall t => {-sem-}(isFloatingType  {--}(t)) -> (WfJ{-sem-}(formatOfType{--}(t)), {-sem-}(genreOfFormat{--}(formatOfType{--}(t)) == {-sem-}({-hfil-}RG  )))
\end{haskell}

Further, the format of all function types and all pointers to such types belongs to the Etude function genre “"FG"”, while all structure, union and array types, together with any pointers to an object or incomplete type are formatted in the object genre “"OG"”:

\begin{haskell}
wfCReprJ4 :: forall t =>
	{-sem-}(isFunctionType{--}(t) || (isPointerType{--}(t) && isFunctionType{--}(baseOfType{--}(t)))) ->
	(WfJ{-sem-}(formatOfType{--}(t)), {-sem-}(genreOfFormat{--}(formatOfType{--}(t)) == {-sem-}(FG)))

wfCReprJ5 :: forall t =>
	{-sem-}(isCompoundType{--}(t) || isArrayType{--}(t) || (isPointerType{--}(t) && (isObjectType{--}(baseOfType{--}(t)) || isIncompleteType{--}(baseOfType{--}(t))))) ->
	(WfJ{-sem-}(formatOfType{--}(t)), {-sem-}(genreOfFormat{--}(formatOfType{--}(t)) == {-sem-}(OG)))
\end{haskell}

More so, on every C implementation, pointers to all character and void types must share the same format:

\begin{haskell}
wfCReprJ6 :: forall t1, t2 =>
	{-sem-}(isPointerType{--}(t1) && (isCharType{--}(baseOfType{--}(t1)) || isVoidType{--}(baseOfType{--}(t1)))) ->
	{-sem-}(isPointerType{--}(t2) && (isCharType{--}(baseOfType{--}(t2)) || isVoidType{--}(baseOfType{--}(t2)))) ->
	{-sem-}(formatOfType{--}(t1) == formatOfType{--}(t2))
\end{haskell}

Likewise, all pairs of «compatible object types» described later in Section \ref{C/Types/Compatibility} always assume a common representation:

\begin{haskell}
wfCReprJ7 :: forall t1, t2 => --
	{-sem-}(isObjectType{--}(t1) && isObjectType{--}(t2) && t1 `areCompatibleTypes` t2) -> --
	{-sem-}(formatOfType{--}(t1) == formatOfType{--}(t2))
\end{haskell}

The plain “"PlainCharType"” type must have the same format as “"charType"”, every well-formed array type must share its format with a pointer to its element type and all function types must have the same representation as a pointer to that function:

\begin{haskell}
wfCReprJ8 	:: {-sem-}(formatOfType{--}(PlainCharType) == formatOfType{--}(charType))
wfCReprJ9 	:: forall t => --
	{-sem-}(isArrayType{--}(t)) -> --
	{-sem-}(formatOfType{--}(t) == formatOfType{-sem-}(PointerType ({-sem-}(baseOfType{--}(t))) {-*-}))
wfCReprJ10 	:: forall t => --
	{-sem-}(isFunctionType{--}(t)) -> --
	{-sem-}(formatOfType{--}(t) == formatOfType{-sem-}(PointerType (t) {-*-}))
\end{haskell}

More so, a well-formed bit field type of the form “"BitFieldType (PlainIntType) {-:-} (m) {-@-} (n)"” must be represented identically to its corresponding type “"BitFieldType (intType) {-:-} (m) {-@-} (n)"”:

\begin{haskell}
wfCReprJ11 :: forall m, n => --
	{-sem-}(isBitFieldType{-sem-}(BitFieldType (PlainIntType) {-:-} (m) {-@-} (n))) -> --
	{-sem-}(formatOfType{-sem-}(BitFieldType (PlainIntType) {-:-} (m) {-@-} (n)) == formatOfType{-sem-}(BitFieldType (intType) {-:-} (m) {-@-} (n)))
\end{haskell}

Values of a qualified object or bit field type must have the same representation as those of the corresponding unqualified type:

\begin{haskell}
wfCReprJ12 :: forall _T, t => --
	{-sem-}(isObjectType{--}(t) || isBitFieldType{--}(t)) -> --
	{-sem-}(formatOfType{--}(t) == formatOfType{--}(unqualifiedType{--}(t)))
\end{haskell}

Last but not least, all well-formed pointers to every qualified and unqualified version of a given C type must share the same format:

\begin{haskell}
wfCReprJ13 :: forall t => --
	{-sem-}(isPointerType{--}(t)) --
	{-sem-}(formatOfType{--}(t) == formatOfType{-sem-}(PointerType ({-sem-}(unqualifiedType{--}(baseOfType{--}(t)))) {-*-}))
\end{haskell}

As described in Chapter \ref{Lambda}, every supported format "phi" is also associated with a pair of numeric quantities “"glbOfFormat{--}(phi)"” and “"lubOfFormat{--}(phi)"”, known as the format's «greatest lower bound» and «least upper bound», respectively. For integral formats, the set of all integers "n" in the range "glbOfFormat{--}(phi) <= n <= lubOfFormat{--}(phi)" is known as the format's «set of representable values». Inevitably, every arithmetic C type is also associated with such a set, which can be derived directly from the type itself using the following two Haskell constructions:

\BeginHaskellToken	{glbOfType}				{Open}	{glb}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Rational}
\EndHaskellToken

\BeginHaskellToken	{lubOfType}				{Open}	{lub}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Rational}
\EndHaskellToken

\IndexTerm		{glbOfType}					{glb type (type)}
\IndexTerm		{lubOfType}					{lub type (type)}

\begin{code}
glbOfType {-[*]-}, lubOfType {-[*]-} :: TypeT -> Rational

glbOfType	{-sem-}(t) = glbOfFormat{--}(formatOfType{--}(t))
lubOfType	{-sem-}(t) = lubOfFormat{--}(formatOfType{--}(t))
\end{code}

We have already discussed many of the constraints placed on Etude formats in Chapter \ref{Lambda}. However, the C programming language imposes a number of further requirements on the range of values associated with the individual integral C types. In particular, every basic integral type must be capable of representing at least the following range of values:

\NewHaskellToken	{intBoundsJ}				{Ord}	{\TKeyword{int}}
\NewHaskellDescription	{intBoundsJ}					{Minimum representation requirements for integral C types}
\IndexTheorem		{intBoundsJ}					{INT Minimum representation requirements for integral C types}

\begin{haskell}
intBoundsJ1	:: {-sem-}(glbOfType 	{-sem-}(Signed CharType) 	<=  {-hfill-} negate 127		)
intBoundsJ2	:: {-sem-}(glbOfType 	{-sem-}(Signed ShortType) 	<=  {-hfill-} negate 32767		)
intBoundsJ3	:: {-sem-}(glbOfType 	{-sem-}(Signed IntType) 	<=  {-hfill-} negate 32767		)
intBoundsJ4	:: {-sem-}(glbOfType 	{-sem-}(Signed LongType) 	<=  {-hfill-} negate 2147483647		)

intBoundsJ5	:: {-sem-}(lubOfType 	{-sem-}(Signed CharType) 	>=  {-hfill-} 127			)
intBoundsJ6	:: {-sem-}(lubOfType 	{-sem-}(Signed ShortType) 	>=  {-hfill-} 32767			)
intBoundsJ7	:: {-sem-}(lubOfType 	{-sem-}(Signed IntType) 	>=  {-hfill-} 32767			)
intBoundsJ8	:: {-sem-}(lubOfType 	{-sem-}(Signed LongType) 	>=  {-hfill-} 2147483647		)

intBoundsJ9	:: {-sem-}(lubOfType 	{-sem-}(Unsigned CharType) 	>=  {-hfill-} 255			)
intBoundsJ10	:: {-sem-}(lubOfType 	{-sem-}(Unsigned ShortType) 	>=  {-hfill-} 65535			)
intBoundsJ11	:: {-sem-}(lubOfType 	{-sem-}(Unsigned IntType) 	>=  {-hfill-} 65535			)
intBoundsJ12	:: {-sem-}(lubOfType 	{-sem-}(Unsigned LongType) 	>=  {-hfill-} 4294967295		)
\end{haskell}

Further, the four “"Signed"\,” and “"Unsigned"\,” versions of the “"CharType"”, “"ShortType"”, “"IntType"” and “"LongType"” types must be capable of encoding all values that are representable by all previous types in that list:

\begin{haskell}
intBoundsJ13	:: {-sem-}(glbOfType{-sem-}(Signed CharType) >= glbOfType{-sem-}(Signed ShortType)
{-quad-}{-quad-}{-quad-}{-quad-}{-quad-}{-quad-}{-quad-}	{-hfill-} >= glbOfType{-sem-}(Signed IntType) >= glbOfType{-sem-}(Signed LongType))		{-ord-}

intBoundsJ14	:: {-sem-}(lubOfType{-sem-}(Signed CharType) <= lubOfType{-sem-}(Signed ShortType)
								{-hfill-} <= lubOfType{-sem-}(Signed IntType) <= lubOfType{-sem-}(Signed LongType))		{-ord-}

intBoundsJ15	:: {-sem-}(lubOfType{-sem-}(Unsigned CharType) <= lubOfType{-sem-}(Unsigned ShortType)
								{-hfill-} <= lubOfType{-sem-}(Unsigned IntType) <= lubOfType{-sem-}(Unsigned LongType))		{-ord-}
\end{haskell}

Likewise, the set of all non-negative values representable by every “"Signed"\,” integral type form must be a subset of those representable by its “"Unsigned"\,” variant:

\begin{haskell}
intBoundsJ16	:: forall is => WfJ{--}(is) -> {-sem-}(lubOfType{-sem-}(Signed is {-,-}) <= lubOfType{-sem-}(Unsigned is {-,-}))
\end{haskell}

Finally, every well-formed bit field type must support a subset of the values representable by its base type:

\begin{haskell}
intBoundsJ17	:: forall t, m, n => --
	{-sem-}(isBitFieldType{-sem-}(BitFieldType (t) {-:-} (m) {-@-} (n))) -> --
	{-sem-}(glbOfType{--}(t) <= glbOfType{-sem-}(BitFieldType (t) {-:-} (m) {-@-} (n)) <= lubOfType{-sem-}(BitFieldType (t) {-:-} (m) {-@-} (n)) <= lubOfType{--}(t))
\end{haskell}

In Chapter \ref{Lambda}, formats from the “"RG"” genre are further associated with a set of four «floating point parameters» known as the «radix», «precision», «minimum exponent» and «maximum exponent», respectively. These properties can be naturally propagated to floating C types, observing, however, that the C Standard requires all floating C types to share a single radix value:

\NewHaskellToken	{floatBoundsJ}				{Ord}	{\TKeyword{flt}}
\NewHaskellDescription	{floatBoundsJ}					{Minimum representation requirements for floating C types}
\IndexTheorem		{floatBoundsJ}					{FLT Minimum representation requirements for floating C types}

\begin{haskell}
floatBoundsJ1 :: forall t1, t2 => --
	{-sem-}(isFloatingType{--}(t1) && isFloatingType{--}(t2)) -> --
	{-sem-}(radixOfFormat{--}(formatOfType{--}(t1)) == radixOfFormat{--}(formatOfType{--}(t2)))
\end{haskell}

Further, the range of values representable exactly by the “"FloatType"” type, as indicated by its precision and exponent bounds, must be a subset of those meaningful under the “"DoubleType"” representation and, for “"DoubleType"”, it must be a subset of “"LongDoubleType"”:

\begin{haskell}
floatBoundsJ2 	:: {-sem-}(precOfFormat{--}(formatOfType{-sem-}(FloatType)) ---
				<= precOfFormat{--}(formatOfType{-sem-}(DoubleType)) ---
					<= precOfFormat{--}(formatOfType{-sem-}(LongDoubleType)))
floatBoundsJ3 	:: {-sem-}(maxExpOfFormat{--}(formatOfType{-sem-}(FloatType)) ---
				<= maxExpOfFormat{--}(formatOfType{-sem-}(DoubleType)) ---
					<= maxExpOfFormat{--}(formatOfType{-sem-}(LongDoubleType)))
floatBoundsJ4 	:: {-sem-}(minExpOfFormat{--}(formatOfType{-sem-}(FloatType)) ---
				>= minExpOfFormat{--}(formatOfType{-sem-}(DoubleType)) ---
					>= minExpOfFormat{--}(formatOfType{-sem-}(LongDoubleType)))
\end{haskell}

Finally, each of these three types must be capable of representing at least the values described by the following choices of its four floating point parameters:

\begin{haskell}
floatBoundsJ5 	:: forall n :: Integer =>	{-sem-}((fromBool (b == 10^n)) + floor ((p - 1) * logBase(10){--}(b))		>=  {-hfill-} 6			{-,-})
floatBoundsJ6 	:: forall n :: Integer =>	{-sem-}((fromBool (b == 10^n)) + floor ((q - 1) * logBase(10){--}(b))		>=  {-hfill-} 10		{-,-})
floatBoundsJ7 	:: 				{-sem-}(ceiling (logBase(10){--}(b ^^ (_E_min - 1))) 				<=  {-hfill-} negate 37		{-,-})
floatBoundsJ8 	:: 				{-sem-}(floor (logBase(10){--}((1 - b ^^ (negate p)) * b ^^ _E_max)) 		>=  {-hfill-} 37		{-,-})
floatBoundsJ9 	::				{-sem-}((1 - b ^^ (negate p)) * b ^^ _E_max	 				>=  {-hfill-} 10		^(+{--}37))
floatBoundsJ10 	:: 				{-sem-}(b ^^ (1 - p)	 							<=  {-hfill-} 10		^(-{--}5))
floatBoundsJ11 	:: 				{-sem-}(b ^^ (1 - q)	 							<=  {-hfill-} 10		^(-{--}9))
floatBoundsJ12 	:: 				{-sem-}(b ^^ (_E_min - 1) 							<=  {-hfill-} 10		^(-{--}37))
\end{haskell}

where "b", "_E_min" and "_E_max" represent the radix and exponent bounds of the “"FloatType"” type, "p" is equal to that type's precision and "q" determines the precision of “"DoubleType"”:

\begin{haskell}
b	= radixOfFormat		{--}(formatOfType{-sem-}(FloatType))
_E_min	= minExpOfFormat	{--}(formatOfType{-sem-}(FloatType))
_E_max	= maxExpOfFormat	{--}(formatOfType{-sem-}(FloatType))
p	= precOfFormat		{--}(formatOfType{-sem-}(FloatType))
q	= precOfFormat		{--}(formatOfType{-sem-}(DoubleType))
\end{haskell}

For a more thorough treatment of the generic floating point arithmetic in Etude, an interested reader is referred to the earlier discussion of the topic in Section \ref{Lambda/Atoms}. A typical implementation of the "formatOfType{--}(t)" function, suitable for the MMIX architecture from Chapter \ref{MMIX}, is included in Appendix \ref{C/Implementation}.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Types/Storage}
\set{title}		{Storage Requirements}
\begin{subsection}

Ultimately, every object type defined in the C Standard is intended to characterise some collection of C «variables» or address space regions reserved for storage of mutable C values attributed with that type. In Etude, such regions are described concisely by the notion of an «envelope» from Section \ref{Lambda/State}, so that, in effect, every C type "t" describes an Etude envelope constructed naturally by the following function "envelopeOfType{--}(t)":

\BeginHaskellToken	{envelopeOfType}			{Open}	{«\superimpose{\raise.5ex\hbox{\,\macron{}}}{\xi}»}
  \HaskellArgument							{TypeT}
  \HaskellResult							{EnvelopeT}
\EndHaskellToken

\IndexTerm		{envelopeOfType}				{\xi- type (envelope)}

\begin{code}
envelopeOfType {-[*]-} :: TypeT -> EnvelopeT

envelopeOfType {-sem-}(t) ---
  | isVoidType		{--}(t) = emptySet
  | isBitFieldType	{--}(t) = listToSet {-sem-}[ {-hv-}(0 {-hv-}, {-sem-}(formatOfType{-sem-}(Unsigned IntType)) {-hv-}, {-sem-}(attrOfType{--}(t))) ]
  | isScalarType	{--}(t) = listToSet {-sem-}[ {-hv-}(0 {-hv-}, {-sem-}(formatOfType{--}(t)) {-hv-}, {-sem-}(attrOfType{--}(t))) ]
  | isArrayType		{--}(t) = unions [ envelopeOfType{--}(baseOfType{--}(t)) `shiftEnvelope` (k * sizeOfType{--}(baseOfType{--}(t))) --
  						| k <- [ 0 .. unJust (lengthOfType{--}(t)) - 1 ]]
  | isCompoundType	{--}(t) = unions [ envelopeOfType{--}(typeOfMember{--}(m_k)) `shiftEnvelope` offsetOfMember{--}(m_k) --
  						| m_k <- tqOfType{--}(t) `requalifiedMembers` membersOfType{--}(t) ]
\end{code}

In other words, every scalar type "t" describes a singular envelope of the form “"{-hv-}(0 {-hv-}, phi {-hv-}, mu_set)"”, where "phi" is either the format of the “"Unsigned IntType"” type if "t" represents a bit field, or else the format of "t" itself for all other scalar type forms. The set of access attributes "mu_set" associated with the type's envelope element is obtained from its recursive qualification using the following simple Haskell construction:

\BeginHaskellToken	{attrOfType}				{Open}	{«\superimpose{\hbox{\!\macron{}}}{\mu}»}
  \HaskellArgument							{TypeT}
  \HaskellResult							{SetT AttrT}
\EndHaskellToken

\IndexTerm		{attrOfType}					{\mu- type (attrs)}

\begin{code}
attrOfType {-[*]-} :: TypeT -> SetT AttrT

attrOfType {-sem-}(t) = listToSet [ attrOfQualifier{--}(mu_k) | mu_k <- setToList (recTqOfType{--}(t)) ]
\end{code}

On the other hand, the envelope of a complete structure or union type is defined as a union of the individual envelopes of its member types, each shifted by the member's offset within the object. Finally, the envelope of a complete array type (i.e., an array of some predetermined length "n") is formed from "n" copies of the envelope associated with the array's element type "t", each shifted by the offset "k * sizeOfType{--}(t)" for some "k" in the range "0 <= k < n". Formally, the size "sizeOfType{--}(t)" of every complete object type "t" is defined simply as the size of "t"'s envelope, as epitomised by the following trivial Haskell function:

\BeginHaskellToken	{sizeOfType}				{Open}	{\ZapfChancery{S}}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{sizeOfType}					{S type (integer)}

\begin{code}
sizeOfType {-[*]-} :: TypeT -> Integer

sizeOfType {-sem-}(t) = sizeOfEnvelope{--}(envelopeOfType{--}(t))
\end{code}

Since the definition of "envelopeOfType{--}(t)" rests on the implementation-defined mapping between C types and Etude formats, the size of most object types is left unspecified in the portable fragment of the C language. However, every implementation of a C compiler must ensure that the “"PlainCharType"” type has the predetermined size of 1 and that all “"Signed"\,” and “"Unsigned"\,” variants of the same integral type have identical storage requirements:

\NewHaskellToken	{sizeOfTypeJ}				{Ord}	{\TKeyword{size}}
\NewHaskellDescription	{sizeOfTypeJ}					{Minimum size requirements for C types}
\IndexTheorem		{sizeOfTypeJ}					{SIZE Minimum size requirements for C types}

\begin{haskell}
sizeOfTypeJ1 :: {-sem-}(sizeOfType{--}(PlainCharType) == 1)
sizeOfTypeJ2 :: forall is => WfJ{--}(is) -> {-sem-}(sizeOfType{-sem-}(Signed is {-,-}) == sizeOfType{-sem-}(Unsigned is {-,-}))
\end{haskell}

Further restrictions on C type sizes can be inferred indirectly from the requirements placed upon their corresponding Etude formats described earlier in Section \ref{C/Types/Representation}. For example, since all pointers to qualified and unqualified versions of compatible types must share a single binary representation, they must likewise agree in the storage requirements derived from their common Etude format.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Types/IntegralPromotion}
\set{title}		{Integral Promotion}
\begin{subsection}

Most arithmetic operations defined on integer values by the concrete syntax of C expressions described in Section \ref{C/Expressions} can be applied directly only to operands of the “"Signed"\,” and “"Unsigned"\,” versions of the “"IntType"” or “"LongType"” type. Such types are said to be «invariant» and are identified by the Haskell predicate “"isInvariantType"”. Specifically, the family of invariant types includes “"Unsigned IntType"”, the “"Signed"\,” and \hbox{“"Unsigned"\,”} versions of the “"LongType"” type, as well as all non-integral types. Formally:

\BeginHaskellToken	{isInvariantType}			{Open}	{INV}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isInvariantType}				{INV type (bool)}

\begin{code}
isInvariantType {-[*]-} :: TypeT -> Bool

isInvariantType {-sem-}(t) = unqualifiedType{--}(t) @@ listToSet [ Unsigned IntType, Signed LongType, Unsigned LongType ] || not (isIntegralType{--}(t))
\end{code}

In most cases, operands of every non-invariant C type are implicitly converted to the “"Signed"\,” or “"Unsigned"\,” version of the “"IntType"” type, with the “"Signed"\,” variant chosen whenever it is capable of representing every numeric value of the unconverted operand. This process, known as «integral promotion», also drops all type qualifiers associated with any invariably-typed operands, except that, for subtle technical reasons described later in Section \ref{C/Expressions/Values}, type qualification is always preserved for structure and union objects. Formally, this construction is modelled in Haskell as follows:

\BeginHaskellToken	{integralPromotedType}			{Open}	{ip}
  \HaskellArgument							{TypeT}
  \HaskellResult							{TypeT}
\EndHaskellToken

\IndexTerm		{integralPromotedType}				{ip type (type)}

\begin{code}
integralPromotedType {-[*]-} :: TypeT -> TypeT

integralPromotedType {-sem-}(t) ---
    | isCompoundType{--}(t)														= t
    | isInvariantType{--}(t)														= unqualifiedType{--}(t)
    | inRange3 (glbOfType{-sem-}(PlainIntType)) (<=) (glbOfType{--}(t)) (<=) (lubOfType{--}(t)) (<=) (lubOfType{-sem-}(PlainIntType))	= {-sem-}(Signed IntType)
    | otherwise																= {-sem-}(Unsigned IntType)
\end{code}

It should be observed that integral promotion never reduces the set of values representable by its operand. It is applied in C as a convenient means of culling the number of viable operator forms supported by the expression syntax described in Section \ref{C/Expressions}.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Types/PointerPromotion}
\set{title}		{Pointer Promotion}
\begin{subsection}

In most contexts, functions and arrays do not constitute true first-class objects in the C language. Instead, they are usually manipulated indirectly through pointer references, which can be constructed either explicitly by the programmer using the “"Address"” operator described in Section \ref{C/Expressions}, or implicitly by the compiler through the process known as «pointer promotion». Formally, pointer promotion converts every function type into a pointer to that function and every array into a pointer to its element type, leaving all other types unchanged. In Haskell, this operation can be modelled as follows:

\BeginHaskellToken	{pointerPromotedType}			{Open}	{pp}
  \HaskellArgument							{TypeT}
  \HaskellResult							{TypeT}
\EndHaskellToken

\IndexTerm		{pointerPromotedType}				{pp type (type)}

\begin{code}
pointerPromotedType {-[*]-} :: TypeT -> TypeT

pointerPromotedType {-sem-}(t) ---
  | isFunctionType	{--}(t)		= {-sem-}(PointerType (t) {-*-})
  | isArrayType		{--}(t)		= {-sem-}(PointerType ({-sem-}(baseOfType{--}(t))) {-*-})
  | isCompoundType	{--}(t)		= t
  | otherwise				= unqualifiedType{--}(t)
\end{code}

Like integral promotion, pointer promotion adjusts the types of C objects without affecting their Etude representations, in order to reduce the number of distinct expression forms supported by the language.

% Since, under the translation described in this work, every function and array already denotes the address of its corresponding Etude object, pointer promotion does not affect the value of an expression, but merely adjusts its C type as appropriate for almost all C operators that are described in Section \ref{C/Expressions}.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Types/ArgumentPromotion}
\set{title}		{Default Argument Promotion}
\begin{subsection}

The language also defines a similar conversion known as «default argument promotion», intended to normalise the types of arguments supplied to functions declared without an explicit prototype. Formally, default argument promotion converts the “"FloatType"” type into a “"DoubleType"” and, for all other argument types, returns the integral and pointer promoted version of its operand:

\BeginHaskellToken	{argumentPromotedType}			{Open}	{ap}
  \HaskellArgument							{TypeT}
  \HaskellResult							{TypeT}
\EndHaskellToken

\IndexTerm		{argumentPromotedType}				{ap type (type)}

\begin{code}
argumentPromotedType {-[*]-} :: TypeT -> TypeT

argumentPromotedType {-sem-}(t) ---
    | (unqualifiedType{--}(t) == {-sem-}(FloatType))		= {-sem-}(DoubleType)
    | otherwise							= integralPromotedType{--}(pointerPromotedType{--}(t))
\end{code}

The language always applies this promotion in absence of a precise information about the type of a given function argument. Informally, it makes it possible for such functions to be called with almost arbitrary numeric values, without regard for any differences in the specific storage requirements of the individual arithmetic types.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Types/UsualArithmeticConversions}
\set{title}		{Usual Arithmetic Conversions}
\begin{subsection}

When two C expressions are involved in a single binary operation, their C types are usually unified into a «common arithmetic type» by a process known as the «usual arithmetic conversion». In all cases, any type qualifiers are first discarded by the operation and integral promotion is applied to both operands. Then, if one of the resulting operands has the “"Signed LongType"” type, the other has the “"Unsigned IntType"” type and if the set of values representable by “"Unsigned IntType"” is not a subset of those representable by “"Signed LongType"”, then the entire operation assumes the “"Unsigned LongType"” type. Otherwise, the common arithmetic type is equal to the first type from the following list that matches one of the two operands after integral promotion:

\begin{haskell}
	LongDoubleType, DoubleType, FloatType,
	Unsigned LongType, Signed LongType, Unsigned IntType, Signed IntType
\end{haskell}

Observe that, after integral promotion, every arithmetic type is guaranteed to be converted into one of these seven type forms. Formally, the usual arithmetic conversion is represented by the type operator “"`commonArithmeticType`"” and is formulated in Haskell as follows:

\BeginHaskellToken	{commonArithmeticType}			{Void}	{}
  \HaskellArgument							{TypeT}
  \HaskellToken		{{-common-}}				{Bin}	{\common}
  \HaskellArgument							{TypeT}
  \HaskellResult							{TypeT}
\EndHaskellToken

\NewHaskellToken	{`commonArithmeticType`}		{Bin}	{\common}
\NewHaskellToken	{{-common-}}				{Bin}	{\common}

\IndexTerm		{commonArithmeticType}				{\sqcup\plus type type (type)}

\begin{code}
commonArithmeticType {-[*]-} {-common-} {-[*]-} :: TypeT -> TypeT -> TypeT

commonArithmeticType {-sem-}(t1) {-common-} {-sem-}(t2)

  | (  (integralPromotedType{--}(t1) == {-sem-}(Signed LongType) && integralPromotedType{--}(t2) == {-sem-}(Unsigned IntType))   ||
       (integralPromotedType{--}(t2) == {-sem-}(Signed LongType) && integralPromotedType{--}(t1) == {-sem-}(Unsigned IntType)))  &&
    lubOfType{-sem-}(Unsigned IntType) > lubOfType{-sem-}(Signed LongType)							= {-sem-}(Unsigned LongType)
  | integralPromotedType{--}(t1) == {-sem-}(LongDoubleType) 	|| integralPromotedType{--}(t2) == {-sem-}(LongDoubleType)	= {-sem-}(LongDoubleType)
  | integralPromotedType{--}(t1) == {-sem-}(DoubleType) 	|| integralPromotedType{--}(t2) == {-sem-}(DoubleType) 		= {-sem-}(DoubleType)
  | integralPromotedType{--}(t1) == {-sem-}(FloatType) 		|| integralPromotedType{--}(t2) == {-sem-}(FloatType) 		= {-sem-}(FloatType)
  | integralPromotedType{--}(t1) == {-sem-}(Unsigned LongType) 	|| integralPromotedType{--}(t2) == {-sem-}(Unsigned LongType)	= {-sem-}(Unsigned LongType)
  | integralPromotedType{--}(t1) == {-sem-}(Signed LongType) 	|| integralPromotedType{--}(t2) == {-sem-}(Signed LongType)	= {-sem-}(Signed LongType)
  | integralPromotedType{--}(t1) == {-sem-}(Unsigned IntType) 	|| integralPromotedType{--}(t2) == {-sem-}(Unsigned IntType)	= {-sem-}(Unsigned IntType)
  | integralPromotedType{--}(t1) == {-sem-}(Unsigned IntType) 	|| integralPromotedType{--}(t2) == {-sem-}(Unsigned IntType)	= {-sem-}(Unsigned IntType)
  | integralPromotedType{--}(t1) == {-sem-}(Signed IntType) 	|| integralPromotedType{--}(t2) == {-sem-}(Signed IntType)	= {-sem-}(Signed IntType)
  | otherwise															= t1 `compositeType` t2
\end{code}

The final rule in this definition proclaims the common arithmetic type of non-arithmetic operands equal to their «composite type», using an algorithm depicted by the notation “"t1 `compositeType` t2"\!” and described later in Section \ref{C/Types/Composition}. Although this extension does not follow the letter of the C Standard, it provides for a very concise formalisation of certain C expression forms described in Section \ref{C/Expressions}, while retaining the published semantics of all usual arithmetic conversions under their standard application to pairs of \hbox{arithmetic types.}

The usual arithmetic conversion affects the operands' actual values only if one of them represents a negative quantity of the “"Signed LongType"” type, the other has the “"Unsigned IntType"” type and if both of these are converted into “"Unsigned LongType"” by the above algorithm. In all other cases, only the types of the operands are adjusted by the above conversion and their actual values remain unchanged. 

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Types/Compatibility}
\set{title}		{Type Compatibility}
\begin{subsection}

In the course of translation, qualified types are usually compared using a relation known as «type compatibility». By definition, two types are said to be compatible if they have identical qualifications and if their unqualified variants satisfy one of the \hbox{following criteria:}

\begin{list}

•	both entities represent structurally-identical arithmetic types, or

•	one of the two operands represents an enumeration type and the other is compatible with that enumeration's corresponding type, or

•	both operands represent structure or union types with the same “"struct_or_union"” component and type tag, or

•	both represent pointers to compatible referenced types, or

•	both represent arrays of compatible element types and either both arrays have equal lengths, or else at least one of them does not include a length component, or

•	both constitute functions with compatible returned types and prototypes, or

•	both types represent identically-qualified versions of the “"VoidType"\,” type, or

•	one of the two operands has the “"PlainIntType"” form and the other is a “"Signed IntType"”.

\end{list}

In this work, the type compatibility relation is written as “"t1 `areCompatibleTypes` t2"\!”. Formally, the “"`areCompatibleTypes`"” operator is defined by the following protracted Haskell predicate:

\BeginHaskellToken	{areCompatibleTypes}			{Void}	{}
  \HaskellArgument							{TypeT}
  \HaskellToken		{{-approx-}}				{Rel}	{\approx}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\NewHaskellToken	{`areCompatibleTypes`}			{Rel}	{\approx}
\NewHaskellToken	{{-approx-}}				{Rel}	{\approx}

\IndexTerm		{areCompatibleTypes}				{\approx type type (bool)}

\begin{code}
areCompatibleTypes {-[*]-} {-approx-} {-[*]-} :: TypeT -> TypeT -> Bool

areCompatibleTypes {-sem-}(t1) {-approx-} {-sem-}(t2) ---
	= 	(tqOfType{--}(t1) == tqOfType{--}(t2)) &&
		(	(	isArithmeticType  {--}(t1) && unqualifiedType{--}(t1) == unqualifiedType{--}(t2)) ||
			(	isEnumType        {--}(t1) && baseOfType{--}(t1) `areCompatibleTypes` unqualifiedType{--}(t2)) ||
			(	isEnumType        {--}(t2) && baseOfType{--}(t2) `areCompatibleTypes` unqualifiedType{--}(t1)) ||
			(	isCompoundType    {--}(t1) && isCompoundType    {--}(t2) && suOfType{--}(t1) == suOfType{--}(t2) && tagOfType{--}(t1) == tagOfType{--}(t2)) ||
			(	isPointerType     {--}(t1) && isPointerType     {--}(t2) && baseOfType{--}(t1) `areCompatibleTypes` baseOfType{--}(t2)) ||
			(	isArrayType       {--}(t1) && isArrayType       {--}(t2) && baseOfType{--}(t1) `areCompatibleTypes` baseOfType{--}(t2) &&
				(lengthOfType{--}(t1) == lengthOfType{--}(t2) || lengthOfType{--}(t1) == Nothing || lengthOfType{--}(t2) == Nothing)) ||
			(	isFunctionType    {--}(t1) && isFunctionType    {--}(t2) && baseOfType{--}(t1) `areCompatibleTypes` baseOfType{--}(t2) && --
				prototypeOfType{--}(t1) `areCompatiblePrototypesOpt` prototypeOfType{--}(t2)) ||
			(	isVoidType        {--}(t1) && isVoidType        {--}(t2)) ||
			(	listToSet [ unqualifiedType{--}(t1), unqualifiedType{--}(t2) ] == listToSet [ {-sem-}(PlainIntType), {-sem-}(Signed IntType) ]))
\end{code}

where the compatibility relation for function prototypes is formalised as follows:

\BeginHaskellToken	{areCompatiblePrototypesOpt}		{Void}	{}
  \HaskellArgument							{Maybe PrototypeT}
  \HaskellToken		{{-approx-}}				{Rel}	{\approx}
  \HaskellArgument							{Maybe PrototypeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\NewHaskellToken	{`areCompatiblePrototypesOpt`}		{Rel}	{\approx}

\IndexTerm		{areCompatiblePrototypesOpt}			{\approx prototype_opt prototype_opt (bool)}

\begin{code}
areCompatiblePrototypesOpt {-[*]-} {-approx-} {-[*]-} :: Maybe PrototypeT -> Maybe PrototypeT -> Bool

areCompatiblePrototypesOpt {-sem-}({-hide-}Nothing) ---
	{-approx-} {-sem-}({-hide-}Nothing) ---
		= 	True
areCompatiblePrototypesOpt {-sem-}({-hide-}Nothing) ---
	{-approx-} {-sem-}(Just (Types t_set {-,,-})) ---
		= 	{-sem-}(Just (Types t_set {-,,-})) `areCompatiblePrototypesOpt` {-sem-}(Just (Types [ argumentPromotedType{--}(t_k) | t_k <- t_set {-,,-} ]))
areCompatiblePrototypesOpt {-sem-}(Just (Types t_set {-,,-})) ---
	{-approx-} {-sem-}({-hide-}Nothing) ---
		= 	{-sem-}(Just (Types t_set {-,,-})) `areCompatiblePrototypesOpt` {-sem-}(Just (Types [ argumentPromotedType{--}(t_k) | t_k <- t_set {-,,-} ]))
areCompatiblePrototypesOpt {-sem-}(Just (Types t_set1)) ---
	{-approx-} {-sem-}(Just (Types t_set2)) ---
		= 	lengthOfList{--}(t_set1) == lengthOfList{--}(t_set2) &&
			and [ unqualifiedType{--}(pointerPromotedType{--}(t_k1)) `areCompatibleTypes` unqualifiedType{--}(pointerPromotedType{--}(t_k2)) | (t_k1, t_k2) <- t_set1 `zipList` t_set2 ]
areCompatiblePrototypesOpt {-sem-}(Just (Ellipsis (t_set1))) ---
	{-approx-} {-sem-}(Just (Ellipsis (t_set2))) ---
		= 	Just (Types t_set1) `areCompatiblePrototypesOpt` Just (Types t_set2)
areCompatiblePrototypesOpt {-sem-}(other1) ---
	{-approx-} {-sem-}(other2) ---
		= 	False
\end{code}

In other words, two omitted function prototypes are always compatible with each other and, further, a non-empty prototype without the “\!"{-...-}{--}"\!” suffix is deemed compatible with an omitted prototype if and only if it is compatible with a prototype formed by applying default argument promotion to every element of its type list. On the other hand, a compatible pair of prototypes without the “\!"{-...-}{--}"\!” suffix must specify type lists of equal lengths, whose corresponding elements constitute qualified or unqualified versions of compatible types after pointer promotion. In all other cases, a pair of function prototypes is compatible if and only if both operands include the “\!"{-...-}{--}"\!” suffix and type lists that are pairwise-compatible with each other.

Two compatible types are considered to be “essentially the same” by the language and are generally interchangeable anywhere within a C program, unless one of these types represents an incomplete array type or a function without a prototype, in which case the under-specified variant of the type may remain inadmissible in certain semantic contexts described later in Sections \ref{C/Expressions} and \ref{C/Declarations}.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Types/Composition}
\set{title}		{Composite Types}
\begin{subsection}

A pair of compatible types may be used to derive a single «composite type», which is always guaranteed to be compatible with both of the original candidates for such a composition. Intuitively, the composition algorithm combines all information depicted by its two type operands. Formally, it is represented by the binary type operator “"`compositeType`"” and implemented by the following Haskell function:

\BeginHaskellToken	{compositeType}				{Void}	{}
  \HaskellArgument							{TypeT}
  \HaskellToken		{{-sqcup-}}				{Bin}	{\sqcup}
  \HaskellArgument							{TypeT}
  \HaskellResult							{TypeT}
\EndHaskellToken

\NewHaskellToken	{`compositeType`}			{Bin}	{\sqcup}
\NewHaskellToken	{{-sqcup-}}				{Bin}	{\sqcup}

\IndexTerm		{compositeType}					{\sqcup type type (type)}

\begin{code}
compositeType {-[*]-} {-sqcup-} {-[*]-} :: TypeT -> TypeT -> TypeT

compositeType {-sem-}(t1) {-sqcup-} {-sem-}(t2)

  | isArithmeticType  {--}(t1) && unqualifiedType{--}(t1) == unqualifiedType{--}(t2) ---
  	 = tqOfType{--}(t1) `union` tqOfType{--}(t2) `requalifiedType` unqualifiedType{--}(t1)
  | isEnumType        {--}(t1) && baseOfType{--}(t1) `areCompatibleTypes` unqualifiedType{--}(t2) ---
  	 = tqOfType{--}(t2) `requalifiedType` t1
  | isEnumType        {--}(t2) && baseOfType{--}(t2) `areCompatibleTypes` unqualifiedType{--}(t1) ---
  	 = tqOfType{--}(t1) `requalifiedType` t2
  | isCompoundType    {--}(t1) && isCompoundType    {--}(t2) && suOfType{--}(t1) == suOfType{--}(t2) && tagOfType{--}(t1) == tagOfType{--}(t2)
  	= tqOfType{--}(t1) `union` tqOfType{--}(t2) `requalifiedType` {-sem-}(CompoundType ({-sem-}(suOfType{--}(t1))) ({-sem-}(tagOfType{--}(t1))) --
  	   											({-sem-}(membersOfType{--}(t1) `compositeMembers` membersOfType{--}(t2))))
  | isPointerType     {--}(t1) && isPointerType     {--}(t2) ---
  	= tqOfType{--}(t1) `union` tqOfType{--}(t2) `requalifiedType` {-sem-}(PointerType ({-sem-}(baseOfType{--}(t1) `compositeType` baseOfType{--}(t2))) {-*-})
  | isPointerType     {--}(t1) && isIntegralType    {--}(t2) ---
  	= tqOfType{--}(t2) `requalifiedType` (t1)
  | isIntegralType    {--}(t1) && isPointerType     {--}(t2) ---
  	= tqOfType{--}(t1) `requalifiedType` (t2)
  | isArrayType       {--}(t1) && isArrayType       {--}(t2) ---
  	= tqOfType{--}(t1) `union` tqOfType{--}(t2) `requalifiedType` {-sem-}(ArrayType ({-sem-}(baseOfType{--}(t1) `compositeType` baseOfType{--}(t2))) --
  												({-sem-}(lengthOfType{--}(t1) `compositeLength` lengthOfType{--}(t2))))
  | isFunctionType    {--}(t1) && isFunctionType    {--}(t2) ---
  	= tqOfType{--}(t1) `union` tqOfType{--}(t2) `requalifiedType` {-sem-}(FunctionType ({-sem-}(baseOfType{--}(t1) `compositeType` baseOfType{--}(t2))) --
  	   											({-sem-}(prototypeOfType{--}(t1) `compositePrototypeOpt` prototypeOfType{--}(t2))))
  | isVoidType        {--}(t1) || isVoidType        {--}(t2) ---
  	= tqOfType{--}(t1) `union` tqOfType{--}(t2) `requalifiedType` {-sem-}(VoidType)
  | listToSet [ unqualifiedType{--}(t1), unqualifiedType{--}(t2) ] == listToSet [ {-sem-}(PlainIntType), {-sem-}(Signed IntType) ] --
  	= tqOfType{--}(t1) `union` tqOfType{--}(t2) `requalifiedType` {-sem-}(Signed IntType)
\end{code}

Intuitively, this construction merges two compatible types by obtaining a composition of their individual components. If one of the operands represents an enumeration type and the other is compatible with that enumeration's corresponding integral type, then their composition returns the enumeration operand unchanged. Similarly, if one of the two types represents a qualified or unqualified version of the plain “"PlainIntType"” type and the other is an explicit “"Signed IntType"”, then their composite type is, by definition, equal to the “"Signed IntType"” type. Further, in order to simplify formalisation of certain C expressions described later in Section \ref{C/Expressions}, the above definition extends the standard type composition rules as follows:

\begin{list}

•	If the two entities represent differently-qualified versions of otherwise compatible types, then their composite type is qualified with all the type qualifiers of both operands.

•	If one of the two entities represents a void type, then the result is always equal to an appropriately-qualified version of that type, regardless of the nature of the other operand.

•	Finally, if one of the two entities represents a pointer and the other is an integral type, then the result is equal to the pointer operand.

\end{list}

If, in a composition of two array types, one of the operands has an incomplete length, then the result always assumes the length of the other operand. Otherwise, the result has a length identical to that of both types. Formally:

\BeginHaskellToken	{compositeLength}			{Void}	{}
  \HaskellArgument							{Maybe Integer}
  \HaskellToken		{{-sqcup-}}				{Bin}	{\sqcup}
  \HaskellArgument							{Maybe Integer}
  \HaskellResult							{Maybe Integer}
\EndHaskellToken

\NewHaskellToken	{`compositeLength`}			{Bin}	{\sqcup}

\IndexTerm		{compositeLength}				{\sqcup integer_opt integer_opt (integer_opt)}

\begin{code}
compositeLength {-[*]-} {-sqcup-} {-[*]-} :: Maybe Integer -> Maybe Integer -> Maybe Integer

compositeLength {-sem-}({-hide-}Nothing) 	{-sqcup-} {-sem-}(n_opt)			= n_opt
compositeLength {-sem-}(n_opt) 			{-sqcup-} {-sem-}({-hide-}Nothing)		= n_opt
compositeLength {-sem-}(Just n1) 		{-sqcup-} {-sem-}(Just n2) | (n1 == n2)		= Just n1
\end{code}

Similarly, when two structure or union types are composed by the above algorithm and one of these has an empty member list, then the result has the member list of the other operand. Otherwise, both member lists must have identical lengths and their composition is performed pairwise on the corresponding list elements as follows:

\BeginHaskellToken	{compositeMembers}			{Void}	{}
  \HaskellArgument							{MembersT}
  \HaskellToken		{{-sqcup-}}				{Bin}	{\sqcup}
  \HaskellArgument							{MembersT}
  \HaskellResult							{MembersT}
\EndHaskellToken

\NewHaskellToken	{`compositeMembers`}			{Bin}	{\sqcup}

\IndexTerm		{compositeMembers}				{\sqcup members members (members)}

\begin{code}
compositeMembers {-[*]-} {-sqcup-} {-[*]-} :: MembersT -> MembersT -> MembersT

compositeMembers {-sem-}(m_set1) {-sqcup-} {-sem-}(m_set2) ---
  | m_set1 == {-empty-}[]					= m_set2
  | m_set2 == {-empty-}[]					= m_set1
  | lengthOfList{--}(m_set1) == lengthOfList{--}(m_set2)	= [ m_k1 `compositeMember` m_k2 | (m_k1, m_k2) <- m_set1 `zipList` m_set2 ]
\end{code}

In particular, a pair of compatible members must always assume identical names and offset values, with their respective types composed in the result:

\BeginHaskellToken	{compositeMember}			{Void}	{}
  \HaskellArgument							{MemberT}
  \HaskellToken		{{-sqcup-}}				{Bin}	{\sqcup}
  \HaskellArgument							{MemberT}
  \HaskellResult							{MemberT}
\EndHaskellToken

\NewHaskellToken	{`compositeMember`}			{Bin}	{\sqcup}

\IndexTerm		{compositeMember}				{\sqcup member member (member)}

\begin{code}
compositeMember {-[*]-} {-sqcup-} {-[*]-} :: MemberT -> MemberT -> MemberT

compositeMember {-sem-}(Member (t1) (x1) {-@-} (n1)) {-sqcup-} {-sem-}(Member (t2) (x2) {-@-} (n2)) ---
  | (x1 == x2 && n1 == n2) = {-sem-}(Member ({-sem-}(t1 `compositeType` t2)) (x1) {-@-} (n1))
\end{code}

Finally, the composition of two optional function prototypes produces an omitted prototype if and only if neither operand includes any prototype information. Otherwise, if only one of the operands has a prototype and that prototype does not include the “\!"{-...-}{--}"\!” suffix, then that prototype is preserved under composition. In all other cases, both \hbox{prototypes} must have identical lengths and agree in their use of the “\!"{-...-}{--}"\!” suffix, in which case their composition produces a similarly-structured prototype, whose elements are obtained by a pairwise composition of the corresponding elements in their type lists after pointer promotion. In Haskell:

\BeginHaskellToken	{compositePrototypeOpt}			{Void}	{}
  \HaskellArgument							{Maybe PrototypeT}
  \HaskellToken		{{-sqcup-}}				{Bin}	{\sqcup}
  \HaskellArgument							{Maybe PrototypeT}
  \HaskellResult							{Maybe PrototypeT}
\EndHaskellToken

\NewHaskellToken	{`compositePrototypeOpt`}		{Bin}	{\sqcup}

\IndexTerm		{compositePrototypeOpt}				{\sqcup prototype_opt prototype_opt (prototype_opt)}

\begin{code}
compositePrototypeOpt {-[*]-} {-sqcup-} {-[*]-} :: Maybe PrototypeT -> Maybe PrototypeT -> Maybe PrototypeT

compositePrototypeOpt {-sem-}({-hide-}Nothing) 				{-sqcup-} {-sem-}({-hide-}Nothing) 			= {-sem-}({-hide-}Nothing)
compositePrototypeOpt {-sem-}({-hide-}Nothing) 				{-sqcup-} {-sem-}(Just (Types t_set {-,,-}))		= Just (Types t_set)
compositePrototypeOpt {-sem-}(Just (Types t_set {-,,-}))		{-sqcup-} {-sem-}({-hide-}Nothing)			= Just (Types t_set)
compositePrototypeOpt {-sem-}(Just (Types t_set1)) 			{-sqcup-} {-sem-}(Just (Types t_set2)) ---
  | lengthOfList{--}(t_set1) == lengthOfList{--}(t_set2)									--
	= {-sem-}(Just (Types ([ pointerPromotedType{--}(t_k1) `compositeType` pointerPromotedType{--}(t_k2) | (t_k1, t_k2) <- t_set1 `zipList` t_set2 ])))
compositePrototypeOpt {-sem-}(Just (Ellipsis (t_set1) {-...-}))		{-sqcup-} {-sem-}(Just (Ellipsis (t_set2) {-...-})) ---
  | lengthOfList{--}(t_set1) == lengthOfList{--}(t_set2)									--
	= {-sem-}(Just (Ellipsis ([ pointerPromotedType{--}(t_k1) `compositeType` pointerPromotedType{--}(t_k2) | (t_k1, t_k2) <- t_set1 `zipList` t_set2 ])) {-...-})
\end{code}

In Sections \ref{C/Declarations} and \ref{C/Definitions}, composition is used to merge the typing information for multiple declarations of a single logical entity such as C function. Further, in Section \ref{C/Expressions}, type composition is used to compute the types of most C expressions constructed from multiple operands with distinct typing properties. Although the later use of this operation is not recognised by the C Standard, it allows us to refactor the semantics of most C expression forms into a small number of orthogonal rules, which proves invaluable to a successful formalisation of these entities in Section \ref{C/Expressions}.

\end{subsection}
\end{section}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%