\set{capture}{hidden}
\begin{code}

module C.Variables (
	IdentifierT,
	VarT (..)
) where

import Additional

instance Enum (VarT) where
	succ (IVar n)													= IVar (n + 1)
	succ (TVar n)													= TVar (n + 1)
	pred (IVar n) 		| (n > 1) 										= IVar (n - 1)
	pred (TVar n) 		| (n > 0) 										= TVar (n - 1)
	toEnum (n) 		| (n >= 0) 										= TVar (toInteger n)
	fromEnum (TVar n) 	| (inRange (toInteger (minBound :: Int)) (<=) (n) (<=) (toInteger (maxBound :: Int)))	= fromInteger n
	enumFrom (IVar n) 												= [ IVar i | i <- [ n .. ] ]
	enumFrom (TVar n) 												= [ TVar i | i <- [ n .. ] ]
	enumFromThen (IVar m) (IVar n) 											= [ IVar i | i <- [ m, n .. ] ]
	enumFromThen (TVar m) (TVar n) 											= [ TVar i | i <- [ m, n .. ] ]
	enumFromTo (IVar m) (IVar n) 											= [ IVar i | i <- [ m .. n ] ]
	enumFromTo (TVar m) (TVar n) 											= [ TVar i | i <- [ m .. n ] ]
	enumFromThenTo (IVar m) (IVar n) (IVar o)									= [ IVar i | i <- [ m, n .. o ] ]
	enumFromThenTo (TVar m) (TVar n) (TVar o)									= [ TVar i | i <- [ m, n .. o ] ]

\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Variables}
\set{title}		{Identifiers and Variables}
\begin{section}

In the concrete syntax of C, tokens known as «identifiers» are used to designate various semantic entities such as variables, functions and members of structure or union types. In the present formal definition of its semantics, these identifiers are represented simply as Haskell string values, in order to render them usable as names of exported Etude items. Nevertheless, to improve the clarity of our discussion, it is useful to define a distinct Haskell type intended solely for depiction of C identifiers. Formally, this simple type is introduced by the following Haskell synonym declaration:

\NewHaskellToken	{IdentifierT}				{Ord}	{«identifier»}
\NewHaskellResult	{IdentifierT}					{Type}
\NewVariable		{identifier}
\IndexType		{IdentifierT}					{identifier (\tstar)}

\BeginHaskellToken	{IdentifierT'}				{Void}	{}
  \HaskellArgument							{String}
  \HaskellResult							{IdentifierT}
\EndHaskellToken

\IndexCons		{IdentifierT'}					{ string (identifier)}

\begin{syntax}
\begin{code}
type IdentifierT =
	String
\end{code}
\end{syntax}

Although the discussion in Section \ref{C/Overview} suggests that names such as “\C{fac}”, “\C{x}” and “\C{i}\,” assigned by a programmer to the individual C entities are retained under their translation into Etude programs, in reality C identifiers are not a very good match for Etude variables. In particular, the C programming language has its own set of idiosyncratic rules that define visibility of names within the program, which are often orthogonal to the actual life spans of objects associated with these entities by the translation. While the match is perfect for global individuals such as C functions, in most cases, the entity designated by an identifier may persist long after the identifier itself ceased to be visible within the program and, in some cases, certain entities such as string literals are not associated with any C identifier at all. In recognition of these difficulties, all Etude variables that appear in the translated program are represented uniformly by the following Haskell data type “"VarT"”:

\NewHaskellToken	{VarT}					{Ord}	{«\nu»}
\NewHaskellResult	{VarT}						{Type}
\IndexType		{VarT}						{\nu (\tstar)}

\BeginHaskellToken	{EVar}					{Void}	{}
  \HaskellArgument							{IdentifierT}
  \HaskellResult							{VarT}
\EndHaskellToken

\BeginHaskellToken	{IVar}					{Ord}	{\SmallCaps{v}}
  \HaskellModifier							{\sub}
  \HaskellArgument							{Integer}
  \HaskellResult							{VarT}
\EndHaskellToken

\BeginHaskellToken	{TVar}					{Ord}	{\SmallCaps{t}}
  \HaskellModifier							{\sub}
  \HaskellArgument							{Integer}
  \HaskellResult							{VarT}
\EndHaskellToken

\IndexCons		{EVar}						{ identifier}
\IndexCons		{IVar}						{V integer}
\IndexCons		{TVar}						{T integer}

\begin{syntax}
\begin{code}
data VarT =
	EVar IdentifierT				-- external variables
	| IVar 	Integer					-- internal variables
	| TVar 	Integer deriving (Eq, Ord, Show)	-- temporary variable
\end{code}
\end{syntax}

The “«identifier»” form of these Etude variables is reserved solely for representation of C entities such as functions and named global objects whose declarations appear outside of any function definitions, while all other constructs are bound systematically to unique variable names of the form “"IVar n"\!”, in which the integer "n" represents a globally-unique index introduced by a simple algorithm described later in Section \ref{C/Scopes}. More so, temporary variables of the form “"TVar n"\!” are reserved for internal use within individual Etude terms constructed by the translation and, as a rule, will never appear free within a denotation of a complete C entity, thus ensuring that they are always available for an unrestricted use by any other closed term in the program.

\end{section}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
