%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{Lambda}
\set{title}		{Lambda Calculus}
\set{citation}		{Entia non sunt multiplicanda praeter necessitatem\break
               		 (Entities should not be multiplied beyond necessity)}
\set{citation author}	{William Ockham}
\begin{chapter}

\set{capture}{hidden}
\begin{code}

module Lambda (
	module Lambda.Syntax,
	module Lambda.Atoms,
	module Lambda.State,
	module Lambda.Terms,
	module Lambda.Modules,
	module Lambda.Simplifier,
	module Lambda.Arch,
) where

import Additional
import Lambda.Syntax
import Lambda.Atoms
import Lambda.State
import Lambda.Terms
import Lambda.Modules
import Lambda.Simplifier
import Lambda.Arch

\end{code}

\vfill

The entire feasibility of linear correctness rests on our ability to find a program representation suitable for simultaneous use as both an intermediate depiction of computations within the compiler and also as a semantic notation in the formal specification of its source and target languages. It is perhaps surprising that a such a construction exists at all, let alone that it is readily available in the form of the «untyped lambda calculus» [Church36, Church41], which, as will be demonstrated in Chapters \ref{C} and \ref{MMIX}, captures successfully the meanings of typical source and target programming languages and, further, carries the benefits of an extensive body of knowledge about its properties accumulated over the past seventy years since its introduction.

Lambda calculus was originally studied in the context of the foundational work on computability and the nature of mathematics. It made its debut in 1936 in a seminal paper by Alonzo Church, in which Church uses the calculus to provide the famous negative solution to the «Entscheidungs·problem» [Church36]. In an effort to ensure the system's consistency, Church extended the calculus with a «type system» in 1940 [Church40] and conducted an extensive study of its properties in his book on «Calculi of Lambda Conversion» published in the following year [Church41]. In 1943, Kleene demonstrated an equivalence between lambda calculus and the Turing machine [Turing36], a result that is now known as the Church-Turing thesis [Kleene43].

The calculus was first employed in a design of a practical programming language by John McCarthy as part of his research on artificial intelligence [McCarthy58]. Throughout the 1950's, McCarthy gradually developed LISP [McCarthy60]. This effort culminated in 1959 in the famous presentation of the «eval» function  which defined the operational semantics of LISP [McCarthy59] and, to McCarthy's uneasy surprise, could be implemented in LISP itself [Smith84]. The first such complete LISP system, executed entirely in LISP, was delivered in 1962 by Timothy Hart and Michael Levin [McCarthy62]. LISP was also famously used to implement the original FORTRAN-77 compiler for UNIX [Pitman79], which arguably constitutes the first practical encoding of an imperative language in a declarative system. More recently, the language has been reformulated as Scheme and standardised by the Institute of Electrical and Electronics Engineers [IEEE91]. In this new form, LISP has survived the test of time and remains in active use to this day.

The correspondence between lambda calculus and imperative programming languages was first established in 1963 by Landin, who made the observation for ALGOL [Landin63]. Unfortunately, Landin's result remained, for the most part, unexplored in the research community for over thirty years. In 1991, Cytron \etal. presented a highly influential algorithm for translation of imperative programs into their «static single assignment form» (SSA) [Cytron91], which was subsequently shown to be equivalent to lambda calculus by Appel [Appel98, Appel98ML]. In 1995, Kelsey established the formal equivalence between SSA and the «continuation passing style» of lambda calculus through bidirectional translation between the two forms [Kelsey95]. In our own prior work on the subject, we have presented a translation from SSA to the «administrative normal form» of lambda calculus and argued for the use of purely-functional representations in optimising compilers for imperative languages [Chakravarty03]. Moggi's work on monadic semantics of programming languages [Moggi91] and Danvy's research on the «monadic normal form» of programs [Hatcliff94, Danvy03] also contain material related to the subject. However, a direct transformation of a complete imperative programming language in common use into lambda calculus has not been attempted before and is a new contribution of the present work.

\InputFile {Lambda/Syntax}
\InputFile {Lambda/Pure}
\InputFile {Lambda/Extending}
\InputFile {Lambda/Params}
\InputFile {Lambda/Atoms}
\InputFile {Lambda/State}
\InputFile {Lambda/Terms}
\InputFile {Lambda/Modules}
\InputFile {Lambda/Simplifier}

\end{chapter}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%