\set{capture}{hidden}
\begin{code}

module Lambda.Atoms (
	fvOfAtom,
	fvOfAtoms,
	substAtom,
	substAtoms,
	genreOfFormat,
	encodingOfFormat,
	isImmediateAtom,
	renameAtom,
	renameAtoms,
) where

import Additional
import Lambda.Syntax
import {-# SOURCE #-} Lambda.Arch

infixr 9 `substAtom`, `substAtoms`, `renameAtom`, `renameAtoms`

renameAtom  {-[*]-} {-/-} {-[*]-} :: (Ord upsilon, Ord nu) => AtomT upsilon -> (MapT (upsilon) (nu)) -> AtomT nu

renameAtom {-sem-}(VAR upsilon) 			{-/-} {-sem-}(_S) = VAR ({-sem-}(_S#(upsilon)))
renameAtom {-sem-}(IMM (x) (phi))			{-/-} {-sem-}(_S) = {-sem-}(IMM (x) (phi))
renameAtom {-sem-}(OP1 (op) (phi) (alpha))		{-/-} {-sem-}(_S) = {-sem-}(OP1 (op) (phi) ({-sem-}(alpha `renameAtom` _S)))
renameAtom {-sem-}(OP2 (alpha1) (op) (phi) (alpha2))	{-/-} {-sem-}(_S) = {-sem-}(OP2 ({-sem-}(alpha1 `renameAtom` _S)) (op) (phi) ({-sem-}(alpha2 `renameAtom` _S)))

renameAtoms {-[*]-} {-/-} {-[*]-} :: (Ord upsilon, Ord nu) => AtomsT upsilon -> (MapT (upsilon) (nu)) -> AtomsT nu

renameAtoms {-sem-}(alpha_set)				{-/-} {-sem-}(_S) = [ alpha_k `renameAtom` _S | alpha_k <- alpha_set {-,-} ]
\end{code}

\BeginHaskellToken	{renameAtom}				{Void}	{}
  \HaskellContext							{(Ord upsilon, Ord nu)}
  \HaskellArgument							{AtomT upsilon}
  \HaskellToken		{{-/-}}					{Bin}	{\slash}
  \HaskellArgument							{(MapT (upsilon) (nu))}
  \HaskellResult							{AtomT nu}
\EndHaskellToken

\NewHaskellToken	{`renameAtom`}				{Op}	{\slash}

\BeginHaskellToken	{renameAtoms}				{Void}	{}
  \HaskellContext							{(Ord upsilon, Ord nu)}
  \HaskellArgument							{AtomsT upsilon}
  \HaskellToken		{{-/-}}					{Bin}	{\slash}
  \HaskellArgument							{(MapT (upsilon) (nu))}
  \HaskellResult							{AtomsT nu}
\EndHaskellToken

\NewHaskellToken	{`renameAtoms`}				{Op}	{\slash}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Lambda/Atoms}
\set{title}{Atoms and Their Formats}
\begin{section}

Perhaps the most pervasive of all language parameters are those which, collectively, abstract over the details of a specific arithmetic model adopted by a particular instruction set architecture. A correct level of abstraction in this model is as critical to a faithful specification of C as it is essential for definition of Etude's semantics itself. Accordingly, I begin the description of Etude with a detailed account of the facilities provided by the language for manipulation of numeric quantities, whose concrete implementation will be completed in Chapter \ref{MMIX} in a manner appropriate for the MMIX instruction set architecture.

In the pure lambda calculus from the previous section, the structure of lambda terms represented the only means of capturing the essence of all mathematical constructs. This is not so in Etude, in which all such objects are depicted by expression-like entities known as «atoms», intuitively intended to designate simple scalar object such as variables and concrete integer values. To improve readability of the following formalisation of the C language, Etude atoms also admit simple algebraic operations on these objects such as negation “"OP1 (NEG) (phi) {--}{-hv-}(alpha)"” and multiplication “"OP2 (alpha1) (MUL) (phi) (alpha2)"\!”. In particular, the abstract structure of all atoms is captured by the following set of four Haskell \hbox{data type definitions:}

\IndexType		{AtomT}						{atom \nu (\tstar)}
\IndexCons		{VAR}						{ \nu (atom)}
\IndexCons		{IMM}						{[] rational format (atom)}
\IndexCons		{OP1}						{ unary_op format atom (atom)}
\IndexCons		{OP2}						{ atom binary_op format atom (atom)}

\IndexType		{AtomsT}					{atoms \nu (\tstar)}

\IndexType		{UnaryOpT}					{unary_op (\tstar)}
\IndexCons		{NEG}						{\minus (unary_op)}
\IndexCons		{NOT}						{\sim (unary_op)}
\IndexCons		{CNV}						{ format (unary_op)}

\IndexType		{BinaryOpT}					{binary_op (\tstar)}
\IndexCons		{ADD}						{\plus (binary_op)}
\IndexCons		{SUB}						{\minus (binary_op)}
\IndexCons		{MUL}						{\times (binary_op)}
\IndexCons		{DIV}						{\div (binary_op)}
\IndexCons		{REM}						{\rem (binary_op)}
\IndexCons		{CEQ}						{\eq (binary_op)}
\IndexCons		{CNE}						{\ne (binary_op)}
\IndexCons		{CLT}						{\lt (binary_op)}
\IndexCons		{CGT}						{\gt (binary_op)}
\IndexCons		{CLE}						{\le (binary_op)}
\IndexCons		{CGE}						{\ge (binary_op)}
\IndexCons		{AND}						{\bitand (binary_op)}
\IndexCons		{XOR}						{\bitxor (binary_op)}
\IndexCons		{IOR}						{\bitor (binary_op)}
\IndexCons		{SHL}						{\ll (binary_op)}
\IndexCons		{SHR}						{\gg (binary_op)}

\begin{syntax}
\InsertHaskell		{AtomT, \par, AtomsT, \par, UnaryOpT, \par, BinaryOpT}
\end{syntax}

In other words, the syntax of atoms includes variables, rational numbers and a handful of predefined arithmetic operations such as “"OP1 (NEG) (phi) {--}{-hv-}(alpha)"” and “"OP2 (alpha1) (MUL) (phi) (alpha2)"\!”, in which "alpha", "alpha1" and "alpha2" represent some other arbitrary atomic expressions. The precise meanings of all atomic operators described by the Haskell data types “"UnaryOpT"” and “"BinaryOpT"”, as well as the purpose of their "FormatT" operand "phi" are discussed later in this section.

The simplest of all atomic forms are the «variable atoms», depicted by the variable's identifier as in “"nu"”. Unlike the pure lambda calculus from Section \ref{Lambda/Pure}, Etude does not prescribe a predetermined implementation of such terms, allowing us to represent variables by values of an arbitrary Haskell type that is a member of the standard classes “"Eq"”, “"Ord"” and “"Enum"” described separately in Appendix \ref{Additional}. Given one such type "nu", the type of Etude atoms in which all variables are represented by well-formed values of "nu" is written as “"AtomT nu"\!”. Individual atom expressions of this type are generally depicted by the Greek letter “"alpha"” or a decorated version thereof, such as “"alpha'"\,”, “"alpha_k"\!”, etc. Similarly, operator values of the “"UnaryOpT"” and “"BinaryOpT"” types are generally represented by the variable “"op"”, while “"FormatT"” entities are, for conciseness, depicted by the \hbox{Greek letter “"phi"”.}

Since the syntax of atoms does not include a binding construct such as “"PureLambda (nu) {-dot-} tau"\,”, every variable appearing anywhere in an atom "alpha" is automatically included in its set of free variables "fvOfAtom{--}(alpha)", which can be constructed trivially by the following induction over the atom's lexical structure:

\BeginHaskellToken	{fvOfAtom}				{Open}	{FV}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{AtomT nu}
  \HaskellResult							{SetT nu}
\EndHaskellToken

\BeginHaskellToken	{fvOfAtoms}				{Open}	{FV}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{AtomsT nu}
  \HaskellResult							{SetT nu}
\EndHaskellToken

\IndexTerm		{fvOfAtom}					{FV atom (\nu s)}
\IndexTerm		{fvOfAtoms}					{FV atoms (\nu s)}

\begin{code}
fvOfAtom {-[*]-} :: (Ord nu) => AtomT nu -> SetT nu

fvOfAtom {-sem-}(VAR nu)					= listToSet [nu]
fvOfAtom {-sem-}(IMM (x) (phi))					= emptySet
fvOfAtom {-sem-}(OP1 (op) (phi) {--} {-hv-} (alpha))		= fvOfAtom{--}(alpha)
fvOfAtom {-sem-}(OP2 (alpha1) (op) (phi) (alpha2))		= fvOfAtom{--}(alpha1) `union` fvOfAtom{--}(alpha2)

{-{ and also, for entire lists of atoms: }-}

fvOfAtoms {-[*]-} :: (Ord nu) => AtomsT nu -> SetT nu

fvOfAtoms {-sem-}(alpha_set)					= unions [ fvOfAtom{--}(alpha_k) | alpha_k <- alpha_set {-,-} ]
\end{code}

The later definition uses the Haskell list comprehension notation “"[ {-,,,-} f{--}(x_k) | x_k <- x_set {-,,-} ]"” in order to apply the “"fvOfAtom"” function to every element of the supplied atomic list "alpha_set" and, subsequently, the list union operator “"unions"” described in Appendix \ref{Additional/Sets} to collapse the resulting list into a single set.

In general, variable atoms are meaningless in isolation, but may be closed by the simple process of variable substitution defined by the following Haskell constructions:

\BeginHaskellToken	{substAtom}				{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{AtomT nu}
  \HaskellToken		{{-/-}}					{Bin}	{\slash}
  \HaskellArgument							{(MapT (nu) (AtomT nu))}
  \HaskellResult							{AtomT nu}
\EndHaskellToken

\NewHaskellToken	{{-/-}}					{Op}	{\slash}
\NewHaskellToken	{`substAtom`}				{Op}	{\slash}

\IndexTerm		{substAtom}					{\slash atom (\nu s_atom) (atom)}

\begin{code}
substAtom  {-[*]-} {-/-} {-[*]-} :: (Ord nu) => AtomT nu -> (MapT (nu) (AtomT nu)) -> AtomT nu
\end{code}

such that “"VAR nu `substAtom` _S"\,” replaces the variable "nu" with its binding in the finite map "_S" whenever such binding exists, leaving all other atoms "nu' /@ dom{--}(_S)" unchanged:

\begin{code}
substAtom {-sem-}(VAR nu) {-/-} {-sem-}(_S)		| nu @@ dom{--}(_S)	= _S#(nu)
							| otherwise		= {-sem-}(VAR nu)
\end{code}

For all other atomic forms, the substitution is simply applied recursively to every constituent of the construct:

\begin{code}
substAtom {-sem-}(IMM (x) (phi))			{-/-} {-sem-}(_S) = {-sem-}(IMM (x) (phi))
substAtom {-sem-}(OP1 (op) (phi) {--} {-hv-} (alpha))	{-/-} {-sem-}(_S) = {-sem-}(OP1 (op) (phi) {--} {-sem-}(alpha`substAtom`_S))
substAtom {-sem-}(OP2 (alpha1) (op) (phi) (alpha2))	{-/-} {-sem-}(_S) = {-sem-}(OP2 ({-sem-}(alpha1`substAtom`_S)) (op) (phi) ({-sem-}(alpha2`substAtom`_S)))
\end{code}

In the same vein, a substitution of an entire list of atoms can be described by the following list comprehension:

\BeginHaskellToken	{substAtoms}				{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{AtomsT nu}
  \HaskellToken		{{-/-}}					{Bin}	{\slash}
  \HaskellArgument							{(MapT (nu) (AtomsT nu))}
  \HaskellResult							{AtomsT nu}
\EndHaskellToken

\NewHaskellToken	{`substAtoms`}				{Op}	{\slash}

\IndexTerm		{substAtoms}					{\slash atoms (\nu s_atom) (atoms)}

\begin{code}
substAtoms {-[*]-} {-/-} {-[*]-} :: (Ord nu) => AtomsT nu -> (MapT (nu) (AtomT nu)) -> AtomsT nu

substAtoms {-sem-}(alpha_set)				{-/-} {-sem-}(_S) = [ alpha_k`substAtom`_S | alpha_k <- alpha_set {-,-} ]
\end{code}

The second most common atomic form, written as “"IMM (x) (phi)"\!\!”, represents quantities with a predefined numeric value depicted by the rational constant "x". Precisely what values of "x" should be permitted in a well-formed atomic construct is, unfortunately, a controversial topic since, after over 50 years of development, designers of instruction set architectures are yet to reach a universal agreement on the precise mapping between the infinite set of all rational numbers known to mathematics and the finite resources available to physical computational machines. In the generic fragment of Etude semantics, this mapping between atomic and numeric values is known as a «format», so called because, under typical Etude implementations, it is directly related to the encoding of numeric values into bit patterns manipulated by binary computational hardware. The reader should take care to note that the concept of a format is vastly different from that of an expression «type» in the type-theoretic sense of the term. In fact, formats do little to mitigate Etude's untyped nature and, under most modern instruction sets architectures, only a handful of different representations of data is supported directly by the underlying computational hardware (the MMIX architecture described in Chapter \ref{MMIX} supports only one.) The format annotations found in atoms of the form “"IMM (x) (phi)"\!” are used only to guide the precise binary encoding of the rational quantity "x" and those associated with all other atomic forms serve merely as a convenient lexical means of grouping the plethora of otherwise-distinct arithmetic operations into a small number of related categories.

Formally, every format is represented by a pair of the form “"gamma`FMT`epsilon"”, in which the two components "gamma" and "epsilon" are known, respectively, as the format's «genre» and «encoding»:

\IndexType		{FormatT}					{format (\tstar)}
\IndexCons		{FMT}						{. genre encoding (format)}

\IndexType		{GenreT}					{genre (\tstar)}
\IndexCons		{NG}						{N (genre)}
\IndexCons		{ZG}						{Z (genre)}
\IndexCons		{RG}						{R (genre)}
\IndexCons		{FG}						{F (genre)}
\IndexCons		{OG}						{O (genre)}

\begin{syntax}
\InsertHaskell		{FormatT, \par, GenreT}
\end{syntax}

For convenience, these two components may be extracted from a given format with the help of the following Haskell definitions:

\BeginHaskellToken	{genreOfFormat}				{Open}	{«\gamma»}
  \HaskellArgument							{FormatT}
  \HaskellResult							{GenreT}
\EndHaskellToken

\BeginHaskellToken	{encodingOfFormat}			{Open}	{«\epsilon»}
  \HaskellArgument							{FormatT}
  \HaskellResult							{EncodingT}
\EndHaskellToken

\IndexTerm		{genreOfFormat}					{\gamma format}
\IndexTerm		{encodingOfFormat}				{\epsilon format}

\begin{code}
genreOfFormat 		{-[*]-} :: FormatT -> GenreT
genreOfFormat		{-sem-}(gamma `FMT` epsilon)			= gamma

encodingOfFormat	{-[*]-} :: FormatT -> EncodingT
encodingOfFormat	{-sem-}(gamma `FMT` epsilon)			= epsilon
\end{code}

Genres are used to subdivide the set of all available formats into six different families according to their various semantic properties. In particular, the “"NG"” genre represents a family of formats capable of representing non-negative integers or «natural numbers», while the “"ZG"” formats interpret bit patterns as signed integers taken from a suitable range of values as determined by the format's associated encoding component. Formats in these two genres are known as «natural» and «integer formats», respectively. The “"RG"” genre describes a small set of «rational formats» that enable Etude programs to manipulate a more general class of «rational numbers» represented in one of the popular «floating point encodings» such as the «IEEE 754 Standard for Binary Floating-Point Arithmetic» [IEEE754]. The remaining two genres “"FG"” and “"OG"” characterise the highly-specialised «function» and «object formats», collectively known as the «pointer format» family. Their individual rôles are discussed later in the current chapter. Collectively, formats from the “"NG"” and “"ZG"” genres are known as «integral formats» and those from “"NG"”, “"ZG"” or “"RG"” genres are often referred to as «arithmetic formats» in this work.

The genre also determines the set of encoding values permitted in a format, although, once again, most of the rules that determine the precise relationship between genres and encodings is left unspecified in the generic fragment of the Etude language. Accordingly, in this chapter, the well-formedness property “"WfJ {-[*]-} :: FormatT -> Prop"” is described only indirectly through a small set of simple constraints that, collectively, outline the minimal algebraic properties of all well-formed Etude formats.

In particular, every Etude implementation is guaranteed to support at least one format from every genre, whose encoding is represented by some unspecified common value “"standardEncoding"” and, further, for every well-formed integer format “"ZG`FMT`epsilon"”, the corresponding natural format “"NG`FMT`epsilon"” is also guaranteed to be well-formed. Formally:

\ReindexTheorem		{wfStandardFormatJ}
\ReindexTheorem		{wfIntegralFormatJ}

\begin{haskell}
wfStandardFormatJ 	:: forall gamma 	=> WfJ{--}(gamma) -> WfJ{-sem-}(gamma`FMT`standardEncoding)
wfIntegralFormatJ	:: forall epsilon	=> WfJ{-sem-}(ZG`FMT`epsilon) -> WfJ{-sem-}(NG`FMT`epsilon)
\end{haskell}

Such distinguished encoding “"standardEncoding"” is known as the «standard encoding» of the underlying instruction set architecture and is represented formally by an implementation-defined Haskell definition with the following type signature:

\ReindexTerm		{standardEncoding}

\begin{haskell}
standardEncoding :: EncodingT
\end{haskell}

Every well-formed format "phi" is further associated with four numeric properties known as its «size», «width», «greatest lower bound» and «least upper bound», represented by the following Haskell constructions:

\ReindexTerm		{sizeOfFormat}
\ReindexTerm		{widthOfFormat}
\ReindexTerm		{glbOfFormat}
\ReindexTerm		{lubOfFormat}

\begin{haskell}
sizeOfFormat	{-[*]-}		:: FormatT -> Integer		-- size
widthOfFormat	{-[*]-} 	:: FormatT -> Integer		-- width
glbOfFormat  	{-[*]-}		:: FormatT -> Rational		-- greatest lower bound
lubOfFormat  	{-[*]-} 	:: FormatT -> Rational		-- least upper bound
\end{haskell}

Intuitively, a format's size "sizeOfFormat{--}(phi)" represents the amount of space (in bytes) occupied by values encoded under that format in memory, its width "widthOfFormat{--}(phi)" represents the number of bits utilised in the underlying binary representation of data and the lower and upper bounds depict the smallest and greatest numeric quantities that are representable precisely under that format. Formally, these properties are subject to the following universal constraints on every Etude implementation:

\ReindexTheorem		{sizeOfValidFormatJ}
\ReindexTheorem		{widthOfValidFormatJ}
\ReindexTheorem		{glbOfValidFormatJ}
\ReindexTheorem		{lubOfValidFormatJ}

\begin{haskell}
sizeOfValidFormatJ 	:: forall phi => WfJ{--}(phi) -> {-sem-}(sizeOfFormat{--}(phi) > 0)
widthOfValidFormatJ 	:: forall phi => WfJ{--}(phi) -> {-sem-}(0 < widthOfFormat{--}(phi) <= byteWidth * sizeOfFormat{--}(phi))
glbOfValidFormatJ 	:: forall phi => WfJ{--}(phi) -> {-sem-}(glbOfFormat	{--}(phi) <= 0)
lubOfValidFormatJ 	:: forall phi => WfJ{--}(phi) -> {-sem-}(lubOfFormat	{--}(phi) > 0)
\end{haskell}

In other words, the greatest lower bound of every Etude format must always have a negative or zero value, while its size, width and least upper bound are guaranteed to represent positive integer quantities, with the width no greater than the number of bytes required for representation of that format's value in a pure binary notation, i.e., the product of its size with the architecture's «byte width» parameter. In the remainder of this work, the architecture's byte width is represented universally by the Greek letter “"byteWidth"”, as stipulated by the following Haskell type signature:

\ReindexTerm		{byteWidth}

\begin{haskell}
byteWidth :: Integer
\end{haskell}

A further two constraints are imposed on the bounds of all well-formed natural formats as follows:

\ReindexTheorem		{glbOfNaturalFormatJ}
\ReindexTheorem		{lubOfNaturalFormatJ}

\begin{haskell}
glbOfNaturalFormatJ 	:: forall epsilon => WfJ{-sem-}(NG`FMT`epsilon)	-> {-sem-}(glbOfFormat	{-sem-}(NG`FMT`epsilon) == 0)
lubOfNaturalFormatJ 	:: forall epsilon => WfJ{-sem-}(NG`FMT`epsilon)	-> {-sem-}(lubOfFormat	{-sem-}(NG`FMT`epsilon) == 2^(widthOfFormat{-sem-}(NG`FMT`epsilon)) - 1)
\end{haskell}

From theorem "widthOfValidFormatJ", it should be clear that the byte width must always assume a strictly-positive integer value. Collectively, the above theorems guarantee that, on every Etude implementation, all natural numbers are always represented using «pure binary notation» as required by the ISO C Standard [ANSI89]. On the other hand, integer formats have a much more relaxed set of properties that are captured as follows:

\ReindexTheorem		{sizeOfIntegerFormatJ}
\ReindexTheorem		{glbOfIntegerFormatJ}
\ReindexTheorem		{lubOfIntegerFormatJ}

\begin{haskell}
sizeOfIntegerFormatJ 	:: forall epsilon => WfJ{-sem-}(ZG`FMT`epsilon)	-> {-sem-}(sizeOfFormat{-sem-}(ZG`FMT`epsilon) == sizeOfFormat{-sem-}(NG`FMT`epsilon))
glbOfIntegerFormatJ 	:: forall epsilon => WfJ{-sem-}(ZG`FMT`epsilon)	-> {-sem-}(glbOfFormat	{-sem-}(ZG`FMT`epsilon) <= negate (lubOfFormat{-sem-}(ZG`FMT`epsilon)))
lubOfIntegerFormatJ 	:: forall epsilon => WfJ{-sem-}(ZG`FMT`epsilon)	-> {-sem-}(lubOfFormat	{-sem-}(ZG`FMT`epsilon) < lubOfFormat{-sem-}(NG`FMT`epsilon))
\end{haskell}

The range of quantities representable under the rational arithmetic formats in the “"RG"” genre is somewhat more difficult to characterise. On almost all modern architectures, these formats are associated with the numeric encodings defined by the «IEEE 754 Standard for Binary Floating Point Arithmetic» [IEEE754], although other similar representations of floating point numbers can be also found in common use. In general, every such representation is characterised completely by four integer quantities known as the format's «radix», «precision», «minimum exponent» and «maximum exponent», respectively. Formally, these language parameters are captured for every well-formed rational format “"RG`FMT`epsilon"” by the following four Haskell functions: 

\ReindexTerm		{radixOfFormat}
\ReindexTerm		{precOfFormat}
\ReindexTerm		{minExpOfFormat}
\ReindexTerm		{maxExpOfFormat}

\begin{haskell}
radixOfFormat	 {-[*]-} 	:: FormatT -> Integer		-- radix
precOfFormat	 {-[*]-} 	:: FormatT -> Integer		-- precision
minExpOfFormat	  {-[*]-} 	:: FormatT -> Integer		-- minimum exponent
maxExpOfFormat	  {-[*]-} 	:: FormatT -> Integer		-- maximum exponent
\end{haskell}

In general, these four properties are only ever defined for well-formed rational formats, whereby they are subject to the following value constraints:

\ReindexTheorem		{radixOfFormatJ}
\ReindexTheorem		{precOfFormatJ}
\ReindexTheorem		{minExpOfFormatJ}
\ReindexTheorem		{maxExpOfFormatJ}

\begin{haskell}
radixOfFormatJ		:: forall epsilon => WfJ{-sem-}(RG`FMT`epsilon) -> {-sem-}(radixOfFormat	{-sem-}(RG`FMT`epsilon) > 1)
precOfFormatJ		:: forall epsilon => WfJ{-sem-}(RG`FMT`epsilon) -> {-sem-}(precOfFormat		{-sem-}(RG`FMT`epsilon) > 0)
minExpOfFormatJ		:: forall epsilon => WfJ{-sem-}(RG`FMT`epsilon) -> {-sem-}(minExpOfFormat	 {-sem-}(RG`FMT`epsilon) < 0)
maxExpOfFormatJ		:: forall epsilon => WfJ{-sem-}(RG`FMT`epsilon) -> {-sem-}(maxExpOfFormat	 {-sem-}(RG`FMT`epsilon) > 0)
\end{haskell}

Intuitively, a given rational formal "phi" is capable of representing precisely every number of the form "(negate 1)^s * m * radixOfFormat{--}(phi)^(e - precOfFormat{--}(phi))", where "s", "m" and "e" are non-negative integers such that "0 <= s <= 1", "minExpOfFormat{--}(phi) <= e <= maxExpOfFormat{--}(phi)" and "0 <= m < radixOfFormat{--}(phi)^(precOfFormat{--}(phi))". Formally:

\ReindexTheorem		{glbOfRationalFormatJ}
\ReindexTheorem		{lubOfRationalFormatJ}

\begin{haskell}
glbOfRationalFormatJ	:: forall epsilon => ---
	WfJ{-sem-}(RG`FMT`epsilon) -> {-sem-}(glbOfFormat  {-sem-}(RG`FMT`epsilon) == negate (radixOfFormat{-sem-}(RG`FMT`epsilon)^(precOfFormat{-sem-}(RG`FMT`epsilon)) - 1) --
												* radixOfFormat{-sem-}(RG`FMT`epsilon)^(maxExpOfFormat{-sem-}(RG`FMT`epsilon) --
											 						- precOfFormat{-sem-}(RG`FMT`epsilon)))
lubOfRationalFormatJ	:: forall epsilon => ---
	WfJ{-sem-}(RG`FMT`epsilon) -> {-sem-}(lubOfFormat  {-sem-}(RG`FMT`epsilon) == negate (glbOfFormat{-sem-}(RG`FMT`epsilon)))
\end{haskell}

But back to Etude atoms. In this chapter, their meaning is represented by an algebraic semantics formed from the usual well-formedness property “"WfJ {-[*]-} :: AtomT nu -> Prop"” and an atom equivalence relation “"{-[*]-} `EqvJ` {-[*]-} :: AtomT nu -> AtomT nu -> Prop"”. In Chapter \ref{MMIX}, both properties are defined precisely in terms of the operational behaviour of Etude atoms and the two semantic formulations are shown to be equivalent by proving all of the theorems outlined in the remainder of this section.

In general, well-formedness can be asserted only for closed atoms, so that variables may never appear in operands of the “"WfJ"” and “"`EqvJ`"” properties. A constant atom of the form “"IMM (x) (phi)"\!”, however, has a well-defined meaning provided that the rational number "x" is equal to 0 or else it is representable precisely under the corresponding format "phi":

\ReindexTheorem		{wfZeroAtomJ}
\ReindexTheorem		{wfImmIgAtomJ}
\ReindexTheorem		{wfImmRgAtomJ}

\begin{haskell}
wfZeroAtomJ 	:: forall phi => --
	WfJ{--}(phi) -> --
	WfJ{-sem-}(IMM (0) (phi))

wfImmIgAtomJ 	:: forall n :: Integer, phi =>
	WfJ{--}(phi) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG] && glbOfFormat{--}(phi) <= n <= lubOfFormat{--}(phi)) ->
	WfJ{-sem-}(IMM (x) (phi))
	
wfImmRgAtomJ 	:: forall s, m, e :: Integer, phi =>
 	WfJ{--}(phi) ->
 	{-sem-}(genreOfFormat{--}(phi) == {-sem-}(RG) && --
		0 <= s <= 1 && --
		0 <= m < radixOfFormat{--}(phi)^(precOfFormat{--}(phi)) && --
		minExpOfFormat{--}(phi) <= e <= maxExpOfFormat{--}(phi)) ->
	WfJ{-sem-}(IMM ({-sem-}((negate 1)^s * m * radixOfFormat{--}(phi)^(e - precOfFormat{--}(phi)))) (phi))
\end{haskell}

The last of these theorems defines the minimal set of atoms guaranteed to be representable under a rational format from the “"RG"” genre. Intuitively, this set includes at least the «normalised floating point numbers» of the form "(negate 1)^s * m * radixOfFormat{--}(phi)^(e - precOfFormat{--}(phi))", where "s", "m" and "e" are arbitrary integers such that "0 <= s <= 1", "0 <= m < radixOfFormat{--}(phi)^(precOfFormat{--}(phi))" and "minExpOfFormat{--}(phi) <= e <= maxExpOfFormat{--}(phi)".

Otherwise, in all arithmetic atoms of the form “"OP1 (op) (phi) {--} {-hv-}(IMM (x) (phi))"” or “"OP2 (IMM (x) (phi)) (op) (phi) (IMM (y) (phi))"\!”, every operand “"IMM (x) (phi)"\!” and “"IMM (y) (phi)"\!” must represent a well-formed atom. More so, the operations are guaranteed to be meaningful only if their true arithmetic result falls within the numeric bounds established by "phi", or else if "phi" belongs to the natural genre “"NG"”. If "op" represents one of the five arithmetic operators “"ADD{-sub-}phi"\!”, “"SUB{-sub-}phi"\!”, “"MUL{-sub-}phi"\!”, “"DIV{-sub-}phi"\!” or “"REM{-sub-}phi"\!”, then these constraints are captured formally by the following five theorems:

\ReindexTheorem		{wfNegAtomJ}
\ReindexTheorem		{wfAddAtomJ}
\ReindexTheorem		{wfSubAtomJ}
\ReindexTheorem		{wfMulAtomJ}
\ReindexTheorem		{wfDivAtomJ}

\begin{haskell}
wfNegAtomJ :: forall x, phi =>
	WfJ{-sem-}(IMM (x) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) == {-sem-}(NG) || --
		(genreOfFormat{--}(phi) @@ listToSet [ZG, RG] && glbOfFormat{--}(phi) <= negate x <= lubOfFormat{--}(phi))) ->
	WfJ{-sem-}(OP1 (NEG) (phi) {--} {-raw-}(IMM (x) (phi)))

wfAddAtomJ :: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) == {-sem-}(NG) || --
		(genreOfFormat{--}(phi) @@ listToSet [ZG, RG] && glbOfFormat{--}(phi) <= x + y <= lubOfFormat{--}(phi))) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (ADD) (phi) (IMM (y) (phi)))
			
wfSubAtomJ :: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) == {-sem-}(NG) || --
		(genreOfFormat{--}(phi) @@ listToSet [ZG, RG] && glbOfFormat{--}(phi) <= x - y <= lubOfFormat{--}(phi))) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (SUB) (phi) (IMM (y) (phi)))

wfMulAtomJ :: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) == {-sem-}(NG) || --
		(genreOfFormat{--}(phi) @@ listToSet [ZG, RG] && glbOfFormat{--}(phi) <= x * y <= lubOfFormat{--}(phi))) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (MUL) (phi) (IMM (y) (phi)))

wfDivAtomJ :: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG, RG] && glbOfFormat{--}(phi) <= x / y <= lubOfFormat{--}(phi)) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (DIV) (phi) (IMM (y) (phi)))
\end{haskell}

More so, in a «remainder operation» of the form “"OP2 (IMM (x) (phi)) (REM) (phi) (IMM (y) (phi))"\!”, "phi" must always represent an integral format with "glbOfFormat{--}(phi) <= x / y <= lubOfFormat{--}(phi)":

\ReindexTheorem		{wfRemAtomJ}

\begin{haskell}
wfRemAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG, RG] && glbOfFormat{--}(phi) <= x / y <= lubOfFormat{--}(phi)) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (REM) (phi) (IMM (y) (phi)))
\end{haskell}

Similarly, in the «bit operations» \hbox{“"OP2 (alpha1) (AND) (phi) (alpha2)"\!”,} \hbox{“"OP2 (alpha1) (XOR) (phi) (alpha2)"\!”,} \hbox{“"OP2 (alpha1) (IOR) ({-!-}{-!-}phi) (alpha2)"\!”,} \hbox{“"OP2 (alpha1) (SHL) (phi) (alpha2)"\!”} and \hbox{“"OP2 (alpha1) (SHR) (phi) (alpha2)"\!”,} "phi" must belong to an integral genre and both operand atoms must have non-negative values. For the «bit shift operators» “"SHL{-sub-}phi"\!” and “"SHR{-sub-}phi"\!”, the value of "alpha2" must be smaller than the width of "phi" and, further, for the «right bit shift» “"SHR{-sub-}phi"\!”, the product of "alpha1" and "2^alpha2" must also fall within the range of values representable by that format. In Haskell:

\ReindexTheorem		{wfBitAtomJ}
\ReindexTheorem		{wfShlAtomJ}
\ReindexTheorem		{wfShrAtomJ}

\begin{haskell}
wfBitAtomJ	:: forall x, y, phi, op =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(x >= 0 && y >= 0 && op @@ listToSet [ AND, XOR, IOR ] && genreOfFormat{--}(phi) @@ listToSet [NG, ZG]) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (op) (phi) (IMM (y) (phi)))

wfShlAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(x >= 0 && 0 <= y < widthOfFormat{--}(phi) && (genreOfFormat{--}(phi) == {-sem-}(NG) || (genreOfFormat{--}(phi) == {-sem-}(ZG) && x * 2^y <= lubOfFormat{--}(phi)))) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (SHL) (phi) (IMM (y) (phi)))

wfShrAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(x >= 0 && 0 <= y < widthOfFormat{--}(phi) && genreOfFormat{--}(phi) @@ listToSet [ NG, ZG ]) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (SHR) (phi) (IMM (y) (phi)))
\end{haskell}

Finally, the remaining «bit complement operator» “"NOT{-sub-}phi"\!” is guaranteed to be applicable only under natural formats from the “"NG"” genre:

\ReindexTheorem		{wfNotAtomJ}

\begin{haskell}
wfNotAtomJ	:: forall x, phi => ---
	WfJ{-sem-}(IMM (x) (phi)) -> --
	{-sem-}(genreOfFormat{--}(phi) == {-sem-}(NG)) -> --
	WfJ{-sem-}(OP1 (NOT) (phi) {--} {-raw-}(IMM (x) (phi)))
\end{haskell}

The addition operator “"ADD{-sub-}phi"\!” may be also applied under a well-formed format from the object genre “"OG"”, provided that its second operand is well-formed under the standard integer format “"ZG`FMT`standardEncoding"”. However, the precise construction of all such atomic forms depends heavily on the underlying memory architecture and their generic semantics are discussed separately in Sections \ref{Lambda/State} and \ref{Lambda/Terms} later in this chapter.

The two «equality operators» “"CEQ{-sub-}phi"\!” and “"CNE{-sub-}phi"\!” are well-formed for all pairs of meaningful operand values, while the «relational operators» “"CLT{-sub-}phi"\!”, “"CGT{-sub-}phi"\!”, “"CLE{-sub-}phi"\!” and “"CGE{-sub-}phi"\!” are meaningful only under the arithmetic and object (but not functional) formats:

\ReindexTheorem		{wfEqlAtomJ}
\ReindexTheorem		{wfRelAtomJ}

\begin{haskell}
wfEqlAtomJ	:: forall x, y, phi, op =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(op @@ listToSet [ CEQ, CNE ]) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (op) (phi) (IMM (y) (phi)))

wfRelAtomJ	:: forall x, y, phi, op =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(op @@ listToSet [ CLT, CGT, CLE, CGE ] && genreOfFormat{--}(phi) /= {-sem-}(FG)) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (op) (phi) (IMM (y) (phi)))
\end{haskell}

Finally, a «conversion operation» of the form “"OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(IMM (x) (phi))"” is well-formed whenever both of the formats "phi" and "phi'" belong to some arithmetic genre and the numeric value of "x" falls within the bounds of the target format "phi'", or else if an integral format "phi" is converted into a natural format "phi'", or when the operation converts between natural, integral, function and object representations of the constant "0". Formally:

\ReindexTheorem		{wfCnvNgAtomJ}
\ReindexTheorem		{wfCnvAgAtomJ}
\ReindexTheorem		{wfCnvPgAtomJ}

\begin{haskell}
wfCnvNgAtomJ	:: forall x, phi, phi' =>
	WfJ{-sem-}(IMM (x) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG] && genreOfFormat{--}(phi') == {-sem-}(NG)) ->
	WfJ{-sem-}(OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(IMM (x) (phi)))
	
wfCnvAgAtomJ	:: forall x, phi, phi' =>
	WfJ{-sem-}(IMM (x) (phi)) ->
	{-sem-}(listToSet [genreOfFormat{--}(phi), genreOfFormat{--}(phi')] `subset` listToSet [NG, ZG, RG] && --
		glbOfFormat{--}(phi') <= x <= lubOfFormat{--}(phi')) ->
	WfJ{-sem-}(OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(IMM (x) (phi)))
	
wfCnvPgAtomJ	:: forall phi, phi' =>
	WfJ{--}(phi) -> WfJ{--}(phi') ->
	{-sem-}(genreOfFormat{--}(phi) /@ {-sem-}(RG) && genreOfFormat{--}(phi) /@ {-sem-}(RG)) ->
	WfJ{-sem-}(OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(IMM (0) (phi)))
\end{haskell}

No other form of atoms need be well-formed on ever Etude implementation, except that, in order to reason about more complex inductively-defined atomic structures, one additional theorem is required:

\ReindexTheorem		{wfEqvAtomJ}

\begin{haskell}
wfEqvAtomJ :: forall alpha1, alpha2 => WfJ{--}(alpha1) -> (alpha1 `EqvJ` alpha2) -> WfJ{--}(alpha2)
\end{haskell}

Intuitively, this states that well-formedness of a particular atom "alpha" is always sufficient to establish well-formedness of all other atoms "alpha'" belonging to the same equivalence class. In the generic algebraic semantics of Etude, these equivalence classes are defined by the binary Haskell property “"`EqvJ`"”, which, like, “"WfJ"”, may have various implementation-specific aspects and, accordingly, is only characterised through a set of appropriate constraint theorems in this chapter. For the MMIX instruction set architecture, its complete formulation is given separately in Chapter \ref{MMIX}.

Since “"`EqvJ`"” should represent an equivalence relation, it must always satisfy the three equivalence laws of «reflexivity», «symmetry» and «transitivity». Formally:

\ReindexTheorem		{eqvReflAtomJ}
\ReindexTheorem		{eqvSymmAtomJ}
\ReindexTheorem		{eqvTransAtomJ}

\begin{haskell}
eqvReflAtomJ	:: forall alpha -> (alpha `EqvJ` alpha)
eqvSymmAtomJ	:: forall alpha1, alpha2 => (alpha1 `EqvJ` alpha2) -> (alpha2 `EqvJ` alpha1)
eqvTransAtomJ	:: forall alpha1, alpha2, alpha3 => (alpha1 `EqvJ` alpha2) -> (alpha2 `EqvJ` alpha3) -> (alpha1 `EqvJ` alpha3)
\end{haskell}

More so, applying a unary or binary arithmetic operator to pairwise-equivalent arguments always produces a pair of equivalent Etude atoms as depicted by the following two «compatibility» laws:

\ReindexTheorem		{eqvUnaryAtomJ}
\ReindexTheorem		{eqvBinaryAtomJ}

\begin{haskell}
eqvUnaryAtomJ	:: forall alpha1, alpha2, phi, op => --
	(alpha1 `EqvJ` alpha2) -> --
	{-sem-}(OP1 (op) (phi) {--} {-hv-}(alpha1)) `EqvJ` {-sem-}(OP1 (op) (phi) {--} {-hv-}(alpha2))
eqvBinaryAtomJ	:: forall alpha11, alpha21, alpha12, alpha22, phi, op =>
	(alpha11 `EqvJ` alpha12) -> --
	(alpha21 `EqvJ` alpha22) -> --
	{-sem-}(OP2 (alpha11) (op) (phi) (alpha21)) `EqvJ` {-sem-}(OP2 (alpha12) (op) (phi) (alpha22))
\end{haskell}

The remaining theorems in this section capture the precise set of properties guaranteed for arithmetic operations by the C Standard [ANSI89]. First of all, every well-formed arithmetic operation applied under a natural format “"NG`FMT`epsilon"” is equivalent to an atom of the form “"IMM (x) (NG`FMT`epsilon)"\!\!”, where "x" represents the true mathematical result of the operation taken modulo "lubOfFormat{-sem-}(NG`FMT`epsilon) + 1". Formally:

\ReindexTheorem		{eqvAddNgAtomJ}
\ReindexTheorem		{eqvSubNgAtomJ}
\ReindexTheorem		{eqvMulNgAtomJ}
\ReindexTheorem		{eqvNegNgAtomJ}
\ReindexTheorem		{eqvNotNgAtomJ}
\ReindexTheorem		{eqvCnvNgAtomJ}

\begin{haskell}
eqvAddNgAtomJ	:: forall x, y, epsilon =>
	WfJ{-sem-}(IMM (x) (NG`FMT`epsilon)) -> --
	WfJ{-sem-}(IMM (y) (NG`FMT`epsilon)) ->
	{-sem-}(OP2 (IMM (x) (NG`FMT`epsilon)) (ADD) (NG`FMT`epsilon) (IMM (y) (NG`FMT`epsilon))) --
		`EqvJ` {-sem-}(IMM ({-sem-}((x + y) `mod` (lubOfFormat{-sem-}(NG`FMT`epsilon) + 1))) (NG`FMT`epsilon))

eqvSubNgAtomJ	:: forall x, y, epsilon =>
	WfJ{-sem-}(IMM (x) (NG`FMT`epsilon)) -> --
	WfJ{-sem-}(IMM (y) (NG`FMT`epsilon)) ->
	{-sem-}(OP2 (IMM (x) (NG`FMT`epsilon)) (SUB) (NG`FMT`epsilon) (IMM (y) (NG`FMT`epsilon))) --
		`EqvJ` {-sem-}(IMM ({-sem-}((x - y) `mod` (lubOfFormat{-sem-}(NG`FMT`epsilon) + 1))) (NG`FMT`epsilon))

eqvMulNgAtomJ	:: forall x, y, epsilon =>
	WfJ{-sem-}(IMM (x) (NG`FMT`epsilon)) -> --
	WfJ{-sem-}(IMM (y) (NG`FMT`epsilon)) ->
	{-sem-}(OP2 (IMM (x) (NG`FMT`epsilon)) (MUL) (NG`FMT`epsilon) (IMM (y) (NG`FMT`epsilon))) --
		`EqvJ` {-sem-}(IMM ({-sem-}((x * y) `mod` (lubOfFormat{-sem-}(NG`FMT`epsilon) + 1))) (NG`FMT`epsilon))

eqvNegNgAtomJ	:: forall x, epsilon =>
	WfJ{-sem-}(IMM (x) (NG`FMT`epsilon)) -> 
	{-sem-}(OP1 (NEG) (NG`FMT`epsilon) {--} {-hv-}(IMM (x) (NG`FMT`epsilon))) --
		`EqvJ` {-sem-}(IMM ({-sem-}(negate x `mod` (lubOfFormat{-sem-}(NG`FMT`epsilon) + 1))) (NG`FMT`epsilon))

eqvNotNgAtomJ	:: forall x, epsilon =>
	WfJ{-sem-}(IMM (x) (NG`FMT`epsilon)) ->
	{-sem-}(OP1 (NOT) (NG`FMT`epsilon) {--} {-hv-}(IMM (x) (NG`FMT`epsilon))) --
		`EqvJ` {-sem-}(IMM ({-sem-}(lubOfFormat{-sem-}(NG`FMT`epsilon) - x)) (NG`FMT`epsilon))

eqvCnvNgAtomJ	:: forall x, phi, epsilon =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(NG`FMT`epsilon) -> --
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG]) ->
	{-sem-}(OP1 (CNV (NG`FMT`epsilon)) (phi) {--} {-hv-}(IMM (x) (phi))) --
		`EqvJ` {-sem-}(IMM ({-sem-}(x `mod` (lubOfFormat{-sem-}(NG`FMT`epsilon) + 1))) (NG`FMT`epsilon))
\end{haskell}

The last two theorems in the above list describe the meanings of «bit complement» and «natural conversion» operations, respectively. Intuitively, a bit complement of the form “"OP1 (NOT) (phi) {--} {-hv-}(IMM (x) (phi))"” inverts the binary representation of "x", producing the value of “"lubOfFormat{--}(phi) - x"” for all natural formats "phi", while the conversion atom “"OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(IMM (x) (phi))"” transforms an arbitrary signed or unsigned integer "x" into the natural number “"x `mod` (lubOfFormat{--}(phi') - 1)"”.

On the other hand, under an integer or rational format "phi", these operations always approximate their true arithmetic result "x" within some predetermined margin of error "ulpOfAtom(phi){--}(x)", provided that this result falls within the numeric bounds established by the format. In particular, if the true arithmetic result "x" of such an operation is approximated by an atom of the form “"IMM (x') ({-!-}phi)"\!”, then we can always be sure that "abs (x' - x) < ulpOfAtom(phi){--}(x)". Formally, the upper bounds imposed on this approximation error is known as the «unit of least precision». For formats from the “"ZG"” genre, this value is always equal to "1", so that all arithmetic operations over integer values are always exact in Etude. Under rational formats, however, the only guarantee provided by the generic fragments of the language is that the operation's result differs from its true arithmetic meaning by less than "radixOfFormat{--}(phi) ^ (floor (logBase (radixOfFormat{--}(phi)) (abs (x))) - precOfFormat{--}(phi) + 1)" or "radixOfFormat{--}(phi) ^ (minExpOfFormat{--}(phi) - 1)" in magnitude, whichever is greater. Formally:

\BeginHaskellToken	{ulpOfAtom}				{Open}	{ulp}
  \HaskellModifier							{\sub}
  \HaskellArgument							{FormatT}
  \HaskellArgument							{Rational}
  \HaskellResult							{Rational}
\EndHaskellToken

\IndexTerm		{ulpOfAtom}					{ulp format rational (rational)}

\begin{haskell}
ulpOfAtom {-[*]-} {-[*]-} :: FormatT -> Rational -> Rational

ulpOfAtom (phi) {-sem-}(x) ---
  | genreOfFormat{--}(phi) == {-sem-}(ZG) ---
  	= 1
  | genreOfFormat{--}(phi) == {-sem-}(RG) && abs (x) < radixOfFormat{--}(phi) ^ (minExpOfFormat{--}(phi) - 1) ---
  	= radixOfFormat{--}(phi) ^ (minExpOfFormat{--}(phi) - 1)
  | genreOfFormat{--}(phi) == {-sem-}(RG) && abs (x) >= radixOfFormat{--}(phi) ^ (minExpOfFormat{--}(phi) - 1) ---
	= radixOfFormat{--}(phi) ^ (floor (logBase (radixOfFormat{--}(phi)) (abs (x))) - precOfFormat{--}(phi) + 1)
\end{haskell}

Using this auxiliary definition, the meanings of all arithmetic operations over signed integers and rational numbers can be formalised as follows:

\ReindexTheorem		{eqvAddAgAtomJ}
\ReindexTheorem		{eqvSubAgAtomJ}
\ReindexTheorem		{eqvMulAgAtomJ}
\ReindexTheorem		{eqvDivAgAtomJ}
\ReindexTheorem		{eqvNegAgAtomJ}

\begin{haskell}
eqvAddAgAtomJ	:: forall x, y, z, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [ ZG, RG ] && glbOfFormat{--}(phi) <= x + y <= lubOfFormat{--}(phi)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (ADD) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM (z) (phi)) ->
	{-sem-}(abs (z - (x + y)) < ulpOfAtom(phi){--}(x + y))

eqvSubAgAtomJ	:: forall x, y, z, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [ ZG, RG ] && glbOfFormat{--}(phi) <= x - y <= lubOfFormat{--}(phi)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (SUB) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM (z) (phi)) ->
	{-sem-}(abs (z - (x - y)) < ulpOfAtom(phi){--}(x - y))

eqvMulAgAtomJ	:: forall x, y, z, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [ ZG, RG ] && glbOfFormat{--}(phi) <= x * y <= lubOfFormat{--}(phi)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (MUL) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM (z) (phi)) ->
	{-sem-}(abs (z - (x * y)) < ulpOfAtom(phi){--}(x * y))

eqvDivAgAtomJ	:: forall x, y, z, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [ ZG, RG ] && y /= 0 && glbOfFormat{--}(phi) <= x / y <= lubOfFormat{--}(phi)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (DIV) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM (z) (phi)) ->
	{-sem-}(abs (z - (x / y)) < ulpOfAtom(phi){--}(x / y))

eqvNegAgAtomJ	:: forall x, z, phi =>
	WfJ{-sem-}(IMM (x) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [ ZG, RG ] && glbOfFormat{--}(phi) <= negate x <= lubOfFormat{--}(phi)) ->
	{-sem-}(OP1 (NEG) (phi) {--} {-hv-}(IMM (x) (phi))) `EqvJ` {-sem-}(IMM (z) (phi)) ->
	{-sem-}(abs (z - (negate x)) < ulpOfAtom(phi){--}(negate x))
\end{haskell}

When applied under an integral format, the two Etude operators “"DIV{-sub-}phi"\!” and “"REM{-sub-}phi"\!” are further constrained to produce the quotient and remainder from the division of their operands' values, so that, if both of the operands involved in an atom of the form “"OP2 (IMM (x) (phi)) (DIV) (phi) (IMM (y) (phi))"\!” have non-negative values, then the result is always equivalent to the integral portion of the fraction "x / y". Further, for every triple of integers "x", "y" and "z" such that “"OP2 (IMM (x) (phi)) (DIV) (phi) (IMM (y) (phi)) `EqvJ` IMM (z) (phi)"\!”, the corresponding «remainder operation» of the form “"OP2 (IMM (x) (phi)) (REM) (phi) (IMM (y) (phi))"\!” is guaranteed to be equivalent to “"IMM (x - y * z) (phi)"\!”, provided that this result falls within the bounds of the underlying integral format "phi". Formally:

\ReindexTheorem		{eqvDivIgAtomJ}
\ReindexTheorem		{eqvRemIgAtomJ}

\begin{haskell}
eqvDivIgAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG] && x >= 0 && y > 0) ->
	{-sem-}(OP2 (IMM (x) (phi)) (DIV) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM ({-sem-}(floor (x / y))) (phi))

eqvRemIgAtomJ	:: forall x, y, z, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG] && y /= 0 && glbOfFormat{--}(phi) <= x - y*z <= lubOfFormat{--}(phi)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (DIV) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM (z) (phi)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (REM) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM ({-sem-}(x - y*z)) (phi))
\end{haskell}

Besides the earlier natural conversions, Etude also supports translations between arbitrary arithmetic formats, as well as conversions between an integral format and one from the functional or object genre. In particular, if “"IMM (x) (phi)"\!” represents a well-formed atom from the arithmetic genre “"NG"”, “"ZG"” or “"RG"” with "glbOfFormat{--}(phi') <= x <= lubOfFormat{--}(phi')", then a conversion operation of the form “"OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(IMM (x) (phi))"” is always equivalent to some other atomic form “"IMM (z) (phi')"\!”, in which "z" approximates "x" to within one unit of the target format's least precision "ulpOfAtom(phi'){--}(x)". Further, if "phi'" belongs to an integral genre, then "z" is guaranteed to be equal to the integral part of "x" as depicted by the notation “"truncate (x)"” defined separately in Appendix \ref{Additional}. Finally, if "x" is equal to 0 and both of the two formats involved belong to a non-rational genre, then the resulting atom is simply equivalent to “"IMM (0) (phi')"\!”. Formally:

\ReindexTheorem		{eqvCnvRgAtomJ}
\ReindexTheorem		{eqvCnvIgAtomJ}
\ReindexTheorem		{eqvCnvPgAtomJ}

\begin{haskell}
eqvCnvRgAtomJ	:: forall x, z, phi, phi' =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(phi') ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG, RG] && --
		genreOfFormat{--}(phi') == {-sem-}(RG) && --
		glbOfFormat{--}(phi') <= x <= lubOfFormat{--}(phi')) ->
	{-sem-}(OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(IMM (x) (phi))) `EqvJ` {-sem-}(IMM (z) (phi')) ->
	{-sem-}(abs (z - x) < ulpOfAtom(phi'){--}(x))

eqvCnvIgAtomJ	:: forall x, phi, phi' =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(phi') ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG, RG] && --
		genreOfFormat{--}(phi') @@ listToSet [NG, ZG] && --
		glbOfFormat{--}(phi') <= truncate (x) <= lubOfFormat{--}(phi')) ->
	{-sem-}(OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(IMM (x) (phi))) `EqvJ` {-sem-}(IMM ({-sem-}(truncate (x))) (phi'))

eqvCnvPgAtomJ	:: forall phi, phi' =>
	WfJ{--}(phi) -> WfJ{--}(phi') ->
	{-sem-}(genreOfFormat{--}(phi) /= {-sem-}(RG) && genreOfFormat{--}(phi') /= {-sem-}(RG)) ->
	{-sem-}(OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(IMM (0) (phi))) `EqvJ` {-sem-}(IMM (0) (phi'))
\end{haskell}

The three «bitwise operators» “"AND{-sub-}phi"\!\!”, “"XOR{-sub-}phi"\!\!” and “"IOR{-!-}{-!-}{-sub-}phi"\!” have a universally-defined meaning only if both of their operands have non-negative integer values. When applied to a pair of such well-formed operands under an appropriate integral format, these operations are equivalent to performing, respectively, the boolean operation “"&&"”, “"/="” and “"||"” on the pairwise-corresponding bits in the binary representation of their two arguments. In Haskell, these requirements can be captured rather inefficiently as follows:

\ReindexTheorem		{eqvAndAtomJ}
\ReindexTheorem		{eqvXorAtomJ}
\ReindexTheorem		{eqvIorAtomJ}

\begin{haskell}
eqvAndAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG] && x >= 0 && y >= 0) ->
	{-sem-}(OP2 (IMM (x) (phi)) (AND) (phi) (IMM (y) (phi))) --
		`EqvJ` {-sem-}(IMM ({-sem-}(sum [ (x`bit`(k) && y`bit`(k)) * 2^i | k <- [0..widthOfFormat{--}(phi) - 1]])) (phi))

eqvXorAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG] && x >= 0 && y >= 0) ->
	{-sem-}(OP2 (IMM (x) (phi)) (XOR) (phi) (IMM (y) (phi))) --
		`EqvJ` {-sem-}(IMM ({-sem-}(sum [ (x`bit`(k) /= y`bit`(k)) * 2^i | k <- [0..widthOfFormat{--}(phi) - 1]])) (phi))

eqvIorAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG] && x >= 0 && y >= 0) ->
	{-sem-}(OP2 (IMM (x) (phi)) (IOR) (phi) (IMM (y) (phi))) --
		`EqvJ` {-sem-}(IMM ({-sem-}(sum [ (x`bit`(k) || y`bit`(k)) * 2^i | k <- [0..widthOfFormat{--}(phi) - 1]])) (phi))
\end{haskell}

using the bit index notation “"x`bit`(k)"\!” and list summation operator “"Sigma"” whose formal definition is given in Appendix \ref{Additional}. Observe that only the least-significant "widthOfFormat{--}(phi)" bits of "x" and "y" are ever manipulated by these operations and that the result always represents a non-negative integer in the range of the respective format, provided that the same holds for each of the two operands to the construction.

The remaining two bit operations “"OP2 (IMM (x) (phi)) (SHL) (phi) (IMM (y) (phi))"\!” and “"OP2 (IMM (x) (phi)) (SHL) (phi) (IMM (y) (phi))"\!” are used to shift the bit pattern encoded in their first operand "x" left or right by the number of bits designated by the second argument "y", effectively multiplying or dividing "x" by the integer quantity "2^y" and rounding the resulting quotient towards \minus\infty. Both operations are only guaranteed to be defined if "x" has a non-negative value, "y" is both non-negative and less than the width of "phi" and, for the left shift operator “"SHL{-sub-}phi"\!”, "phi" belongs to a natural genre “"NG"”. As expected for natural formats, the arithmetic is performed modulo "lubOfFormat{--}(phi) + 1", although this point is mute for the “"SHR{-sub-}phi"\!” operator, whose result "truncate(x/2^y)" is always smaller than "x" in magnitude. Formally, these provisions are captured by the following pair of theorems:

\ReindexTheorem		{eqvShlNgAtomJ}
\ReindexTheorem		{eqvShlZgAtomJ}
\ReindexTheorem		{eqvShrAtomJ}

\begin{haskell}
eqvShlNgAtomJ	:: forall x, y, epsilon =>
	WfJ{-sem-}(IMM (x) (NG`FMT`epsilon)) -> --
	WfJ{-sem-}(IMM (y) (NG`FMT`epsilon)) ->
	{-sem-}(0 <= y < widthOfFormat{-sem-}(NG`FMT`epsilon)) ->
	{-sem-}(OP2 (IMM (x) (NG`FMT`epsilon)) (SHL) (NG`FMT`epsilon) (IMM (y) (NG`FMT`epsilon))) --
		`EqvJ` {-sem-}(IMM ({-sem-}((x * 2^y) `mod` (lubOfFormat{-sem-}(NG`FMT`epsilon) + 1))) (NG`FMT`epsilon))

eqvShlZgAtomJ	:: forall x, y, epsilon =>
	WfJ{-sem-}(IMM (x) (ZG`FMT`epsilon)) -> --
	WfJ{-sem-}(IMM (y) (ZG`FMT`epsilon)) ->
	{-sem-}(0 <= x * 2^y <= lubOfFormat{--}(ZG`FMT`epsilon) && 0 <= y < widthOfFormat{-sem-}(ZG`FMT`epsilon)) ->
	{-sem-}(OP2 (IMM (x) (ZG`FMT`epsilon)) (SHL) (ZG`FMT`epsilon) (IMM (y) (ZG`FMT`epsilon))) --
		`EqvJ` {-sem-}(IMM ({-sem-}(x * 2^y)) (ZG`FMT`epsilon))

eqvShrAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG] && x >= 0 && 0 <= y < widthOfFormat{--}(phi)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (SHR) (phi) (IMM (y) (phi))) --
		`EqvJ` {-sem-}(IMM ({-sem-}(floor (x / 2^y))) (phi))
\end{haskell}

Similarly, the operations “"OP2 (IMM (x) (phi)) (CLT) (phi) (IMM (y) (phi))"\!”, “"OP2 (IMM (x) (phi)) (CGT) (phi) (IMM (y) (phi))"\!”, “"OP2 (IMM (x) (phi)) (CLE) (phi) (IMM (y) (phi))"\!” and “"OP2 (IMM (x) (phi)) (CGE) (phi) (IMM (y) (phi))"\!” are always equivalent to “"IMM (z) (ZG`FMT`standardEncoding)"\!”, where "z" is equal to 1 whenever the corresponding relation holds for its two operands and 0 otherwise, provided that "phi" represents a well-formed format from any genre other than “"FG"”:

\ReindexTheorem		{eqvCltAtomJ}
\ReindexTheorem		{eqvCgtAtomJ}
\ReindexTheorem		{eqvCleAtomJ}
\ReindexTheorem		{eqvCgeAtomJ}

\begin{haskell}
eqvCltAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) /= {-sem-}(FG)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (CLT) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM ({-sem-}(x < y)) (ZG`FMT`standardEncoding))

eqvCgtAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) /= {-sem-}(FG)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (CGT) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM ({-sem-}(x > y)) (ZG`FMT`standardEncoding))

eqvCleAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) /= {-sem-}(FG)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (CLE) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM ({-sem-}(x <= y)) (ZG`FMT`standardEncoding))

eqvCgeAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) /= {-sem-}(FG)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (CGE) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM ({-sem-}(x >= y)) (ZG`FMT`standardEncoding))
\end{haskell}

Little can be said in the generic fragment of Etude semantics about the meanings of atoms constructed under object formats from the “"OG"” genre. Nevertheless, every implementation of the language is required to preserve the following four simple equivalence relations:

\ReindexTheorem		{eqvAddZeroAtomJ}
\ReindexTheorem		{eqvAddOgAtomJ}
\ReindexTheorem		{eqvSubOgAtomJ}
\ReindexTheorem		{eqvCnvOgAtomJ}

\begin{haskell}
eqvAddZeroAtomJ :: forall x, epsilon =>
	WfJ{-sem-}(IMM (x) (OG`FMT`epsilon)) ->
	{-sem-}(IMM (x) (OG`FMT`epsilon)) `EqvJ` {-sem-}(OP2 (IMM (x) (OG`FMT`epsilon)) (ADD) (OG`FMT`epsilon) (IMM (0) (ZG`FMT`standardEncoding)))
	
eqvAddOgAtomJ :: forall x, m, n, epsilon =>
	WfJ{-sem-}(OP2 (IMM (x) (OG`FMT`epsilon)) (ADD) (OG`FMT`epsilon) (IMM (m) (ZG`FMT`standardEncoding))) -> --
	WfJ{-sem-}(OP2 (IMM (m) (ZG`FMT`standardEncoding)) (ADD) (ZG`FMT`standardEncoding) (IMM (n) (ZG`FMT`standardEncoding))) ->
	{-sem-}(OP2 ({-hv-}(OP2 (IMM (x) (OG`FMT`epsilon)) (ADD) (OG`FMT`epsilon) (IMM (n) (ZG`FMT`standardEncoding)))) --
			(ADD) (OG`FMT`epsilon) (IMM (m) (ZG`FMT`standardEncoding))) `EqvJ` --
	{-sem-}(OP2 (IMM (x) (OG`FMT`epsilon)) (ADD) (OG`FMT`epsilon) --
			({-hv-}(OP2 (IMM (m) (ZG`FMT`standardEncoding)) (ADD) (ZG`FMT`standardEncoding) (IMM (n) (ZG`FMT`standardEncoding)))))

eqvSubOgAtomJ :: forall x, m, n, epsilon =>
	WfJ{-sem-}(OP2 (IMM (x) (OG`FMT`epsilon)) (ADD) (OG`FMT`epsilon) (IMM (m) (ZG`FMT`standardEncoding))) -> --
	WfJ{-sem-}(OP2 (IMM (x) (OG`FMT`epsilon)) (ADD) (OG`FMT`epsilon) (IMM (n) (ZG`FMT`standardEncoding))) -> --
	WfJ{-sem-}(OP2 (IMM (m) (ZG`FMT`standardEncoding)) (SUB) (ZG`FMT`standardEncoding) (IMM (n) (ZG`FMT`standardEncoding))) ->
	{-sem-}(OP2 ({-hv-}(OP2 (IMM (x) (OG`FMT`epsilon)) (ADD) (OG`FMT`epsilon) (IMM (m) (ZG`FMT`standardEncoding)))) --
			(SUB) (OG`FMT`epsilon) ({-hv-}(OP2 (IMM (x) (OG`FMT`epsilon)) (ADD) (OG`FMT`epsilon) (IMM (n) (ZG`FMT`standardEncoding))))) `EqvJ` --
	{-sem-}(OP2 (IMM (m) (ZG`FMT`standardEncoding)) (SUB) (ZG`FMT`standardEncoding) (IMM (n) (ZG`FMT`standardEncoding)))

eqvCnvOgAtomJ :: forall x, epsilon =>
	WfJ{-sem-}(IMM (x) (OG`FMT`epsilon)) ->
	{-sem-}(IMM (x) (OG`FMT`epsilon)) `EqvJ` {-sem-}(OP1 (CNV (OG`FMT`epsilon)) (OG`FMT`standardEncoding) {--} --
								{-hv-}(OP1 (CNV (OG`FMT`standardEncoding)) (OG`FMT`epsilon) {--} {-hv-}(IMM (x) (OG`FMT`epsilon))))
\end{haskell}

Intuitively, these state that every valid atomic form “"IMM (x) (OG`FMT`epsilon)"\!” can be also interpreted as an object sum “"OP2 (IMM (y) (OG`FMT`epsilon)) (ADD) (OG`FMT`epsilon) (IMM (n) (ZG`FMT`standardEncoding)"\!”, in which "y" and "n" are known as the atom's «base» and «offset», respectively. Under this interpretation, every constant object atom “"IMM (x) (OG`FMT`epsilon)"\!” has the base "x" with an implicit offset of 0. Further, the sum of two object atoms with the same base is always equivalent to an atom with the same base and the sum of the operands' individual offsets. More so, the difference of two atoms with equal base components is always equal to the difference of their offset values, provided that, in all cases, every intermediate step of these computations represents a well-formed Etude entity. Finally, every well-formed object atom retains its meaning after round-trip conversion through the standard object format “"OG`FMT`standardEncoding"”. A motivation for such interpretation of these constructs can be found in Section \ref{Lambda/State} later in this chapter.

Last but not least, the portable fragment of Etude's atomic semantics always assigns a very specific meaning to the four comparison operators “"CEQ{-sub-}phi"\!”, “"CNE{-sub-}phi"\!”, “"CLT{-sub-}phi"\!”, “"CGT{-sub-}phi"\!”, “"CLE{-sub-}phi"\!” and “"CGE{-sub-}phi"\!”. In particular, for all well-formed values of “"IMM (x) (phi)"\!” and “"IMM (y) (phi)"\!”, the atom “"OP2 (IMM (x) (phi)) (CEQ) (phi) (IMM (y) (phi))"\!” is always equivalent to “"IMM (z) (ZG`FMT`standardEncoding)"\!”, where "z" has the value of 1 if "x" and "y" have identical numeric values and 0 otherwise. Conversely, under the same circumstances, the “"CNE{-sub-}phi"\!” operator delivers 0 if its operands have equal values and 1 otherwise. Both constraints can be expressed concisely by the following pair of theorems:

\ReindexTheorem		{eqvCeqAtomJ}
\ReindexTheorem		{eqvCneAtomJ}

\begin{haskell}
eqvCeqAtomJ	:: forall x, y, phi => ---
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) -> --
	{-sem-}(OP2 (IMM (x) (phi)) (CEQ) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM ({-sem-}(x == y)) (ZG`FMT`standardEncoding))
eqvCneAtomJ	:: forall x, y, phi => ---
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) -> --
	{-sem-}(OP2 (IMM (x) (phi)) (CNE) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM ({-sem-}(x /= y)) (ZG`FMT`standardEncoding))
\end{haskell}

An astute reader will rightly observe that, in most of the above constraint theorems, well-formed atoms of the form “"IMM (x) (phi)"” are singled out as particularly interesting. In this work, such atoms are said to represent «immediate constants» and are identified formally by the following Haskell predicate function:

\BeginHaskellToken	{isImmediateAtom}			{Open}	{IMM}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{AtomT nu}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isImmediateAtom}				{IMM atom (bool)}

\begin{code}
isImmediateAtom {-[*]-} :: {- (Ord nu) => -} AtomT nu -> Bool

isImmediateAtom {-sem-}(IMM (x) (phi))	= True
isImmediateAtom {-sem-}(other)		= False
\end{code}

On all sensible Etude implementations, every well-formed atom "alpha" is always equivalent to some immediate constant "alpha'" that can be viewed as the atom's fully-reduced normal form. In this work, this constant is represented by the notation “"evalAtom{--}(alpha)"”, using the «evaluation function» "evalAtom" with the following Haskell signature:

\ReindexTerm		{evalAtom}

\begin{haskell}
evalAtom {-[*]-} :: (Ord nu) => AtomT nu -> AtomT nu
\end{haskell}

and the obvious trio of semantic properties:

\ReindexTheorem		{wfEvalAtomJ}
\ReindexTheorem		{immEvalAtomJ}
\ReindexTheorem		{eqvEvalAtomJ}

\begin{haskell}
wfEvalAtomJ 	:: forall alpha => WfJ{--}(alpha) -> WfJ{-sem-}(evalAtom{--}(alpha))
immEvalAtomJ 	:: forall alpha => WfJ{--}(alpha) -> {-sem-}(isImmediateAtom{--}(evalAtom{--}(alpha)))
eqvEvalAtomJ 	:: forall alpha1, alpha2 => WfJ{--}(alpha1) -> WfJ{--}(alpha2) -> {-sem-}(evalAtom{--}(alpha1) == evalAtom{--}(alpha2)) -> (alpha1 `EqvJ` alpha2)
\end{haskell}

Intuitively, "evalAtom" represents the «operational semantics» of Etude atoms, which must be defined individually by every complete Etude implementation. In Chapter \ref{MMIX}, one such definition is provided for the MMIX-specific incarnation of the language. In contrast, the above constraints on the atomic well-formedness and equivalence properties constitutes the «algebraic semantics» of the portable Etude fragment, whose consistency with the operational formulation is asserted by appropriate proofs of these properties given in \hbox{Section \ref{Lambda/Arch}.}

Although the above algebraic semantics may, at first sight, appear somewhat arbitrary, a closer scrutiny will quickly reveal that they describe precisely the minimal set of requirements imposed on arithmetic operations by the ISO C Standard [ANSI89], whose accurate formalisation is the ultimate purpose of the Etude program representation. If we were to apply the same methodology in a description of some other programming language, these algebraic semantics would, in all likelihood require a substantial reworking, so that, in its present format, the calculus remains unsuitable for a broader deployment in other compiler verification projects.

An astute reader may also observe that the resulting arithmetic model is incomplete, in that it readily supports a rather unhelpful implementation of Etude that places all atoms in the same single equivalence class. However, Chapter \ref{MMIX} bears witness to the fact that at least one sensible operational model consistent with the above constraints can be formulated for the language. Since all Etude programs that aspire for complete portability must conform to every admissible operational interpretation of their syntax, they must, by definition, also retain their validity on MMIX. Accordingly, the generic Etude algebra presented in this section remains sufficient for a successful depiction of the meaning of all programs conforming to the C Standard, while its rudimentary nature readily explains the difficulty of constructing such programs within the Spartan framework of ANSI C.

\end{section}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
