\set{capture}{hidden}
\begin{code}

module Lambda.Pure (
	PureTermT (..), PureVarT, pureFreeVariables, pureSubstitution, pureEvalA, pureEvalN, pureFix
) where

import Additional

infixl 7 `PureApp`

type PureVarT = String

x = "x"
y = "y"
z = "z"
f = "f"
g = "g"
p = "p"
q = "q"
m = "m"
n = "n"

\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Lambda/Pure}
\set{title}{Lambda Calculus}
\begin{section}

In its simplest form, lambda calculus consists of only three syntactic constructs: «variables», «applications» and «lambda abstractions». This form of the calculus, known as the «pure lambda calculus», can be described using the following BNF grammar:

\NewHaskellToken	{PureTermT}				{Ord}	{«pure-term»}
\NewHaskellResult	{PureTermT}					{Type}
\NewVariable		{pure_term}

\BeginHaskellToken	{PureVar}				{Void}	{}
  \HaskellArgument							{PureVarT}
  \HaskellResult							{PureTermT}
\EndHaskellToken

\BeginHaskellToken	{PureApp}				{Void}	{}
  \HaskellArgument							{PureTermT}
  \HaskellArgument							{PureTermT}
  \HaskellResult							{PureTermT}
\EndHaskellToken

\NewHaskellToken	{`PureApp`}				{Void}	{}

\BeginHaskellToken	{PureLambda}				{Open}	{\sslambda\WhiteSpace}
  \HaskellArgument							{PureVarT}
  \HaskellToken		{{-dot-}}				{Op}	{\WhiteSpace\Helvetica{.}\WhiteSpace}
  \HaskellArgument							{PureTermT}
  \HaskellResult							{PureTermT}
\EndHaskellToken

\NewHaskellToken	{PureVarT}				{Ord}	{«pure-variable»}
\NewHaskellResult	{PureVarT}					{Type}
\NewVariable		{pure_var}

\IndexType		{PureTermT}					{pure_term (\tstar)}
\IndexCons		{PureVar}					{ pure_variable (pure_term)}
\IndexCons		{PureApp}					{ pure_term pure_term (pure_term)}
\IndexCons		{PureLambda}					{\lambda . pure_variable pure_term (pure_term)}
\IndexType		{PureVarT}					{pure_variable (\tstar)}

\begin{syntax}
\begin{code}
data PureTermT =
	PureVar (PureVarT)						-- variables
	| PureTermT `PureApp` PureTermT					-- application expressions
	| PureLambda (PureVarT) {-dot-} (PureTermT) deriving (Show)	-- lambda abstractions
\end{code}
\end{syntax}

As discussed in Chapter \ref{Haskell}, the grammar presented above is actually implemented as a definition of the Haskell data type “PureTermT” with constructors “PureVar” and “PureApp” whose names are suppressed from the presentation for the sake of clarity and “PureLambda”, which is typeset simply as “"PureLambda"\!”. The reader is advised to refer to Chapter \ref{Haskell} for a further discussion of the precise rules governing typesetting of all Haskell material in this work and Appendix \ref{Index} for an exhaustive list of all Haskell types, data constructors and functions mentioned in the presentation, cross-indexed with the page numbers of their respective definitions.

We shall refer to all expressions of this kind as «lambda expressions» or «terms». Simple lambda expressions consisting of a single identifier are called «variables». Terms of the form “"PureLambda(x){-dot-}(tau)"\,” are known as «lambda abstractions». Intuitively, they represent a function or a procedure with a single formal parameter "x" and a body "tau" which computes the value returned by the function and may contain references to "x". Functions of multiple parameters may be constructed using nested lambda abstractions. For example, the term “"PureLambda(x){-dot-}(PureLambda(y){-dot-}(tau))"\,” represents a function of two variables "x" and "y". Finally, terms of the form “"tau1{-,-}tau2"\!” are known as «application expressions» or just «applications». In such expressions, the first term (which represents an actual function) is said to be «applied» to the second one, which depicts the value of that function's operand. Intuitively, an application of the form “"tau1{-,-}tau2"\!” represents a call to the function "tau1" with the argument value specified by "tau2". In particular, a term of the form “"{-hv-}(PureLambda(x){-dot-}(tau1)){--}tau2"\!” is considered equivalent to the result of replacing every occurrence of "x" in "tau1" with the expression "tau2". For the sake of exposition, application expressions are taken to be right-associative and have a higher precedence than lambda abstractions. In other words, “"tau1{-,-}tau2{-,-}tau3"\!” is always taken to mean “"{-hv-}(tau1{-,-}tau2){--}tau3"\!” rather than “"tau1{--}{-hv-}(tau2{-,-}tau3)"” and “"PureLambda(x){-dot-}(tau1{-,-}tau2)"\!” is taken to mean “"PureLambda(x){-dot-}({-hv-}(tau1{-,-}tau2))"” rather than “"{-hv-}(PureLambda(x){-dot-}(tau1)){--}tau2"\!”.

Variables, lambda abstractions and application expressions are the only three syntactic elements of the pure lambda calculus; it is a remarkable consequence of the Church-Turing thesis [Kleene43] that all interesting programs (and many others that are not interesting at all) may be constructed from these three symbolic patterns alone. The set of all computations that are expressible in lambda calculus is precisely that of «computable functions», that is, functions that may be evaluated by a physical device.

It should be observed that not every conceivable function is computable. For example, in 1936 Church and Turing have independently shown that the «halting problem», which poses the question of whether any given function terminates, can never be solved in its full generality [Church36, Turing36]. Many unsolvable problems, are, however, «partially computable», in the sense that it is possible to devise a function that either returns the desired property or does not terminate. In the later case, it is usually possible to further categorise the argument values leading to such non-termination. For example, it is quite possible to devise a function that, given an arbitrary lambda expression, returns “1” whenever that expression terminates and fails to terminate itself for all non-terminating input expressions. We will often find it necessary to work with such partially-computable functions when designing a compiler. In particular, every attempt to express a semantics of a lambda calculus as a function that reduces its terms to their values, is by the very nature of computability, doomed to result in a partially-computable construction.

The minimality of lambda calculus is certainly very appealing, since, with only three language constructs, it renders itself readily to detailed scrutiny. Although, in this work, I will refrain from defining a complete formal semantic model of the pure calculus presented above, such semantics are readily available in literature [Barendregt84] and structures similar to “"PureTermT"” continue to prevail as the authoritative representation of computability. Accordingly, a suitable variant of lambda calculus seems to be a highly attractive choice for an intermediate program representation in a linearly-correct compiler. In the remainder of this chapter, I first present a few examples of useful lambda constructions, followed by two simple variants of their operational meanings. The actual Etude program representation utilised in our compiler verification project is then defined in Sections \ref{Lambda/Atoms}--\ref{Lambda/Modules}, complete with a formal specification of a suitable algebraic semantics for the language. Although Etude represents a substantial departure from the simplicity of its pure cousin, it is the purity and deeply-rooted minimality of Church's invention which makes it possible to verify an entire C compiler within the space of a single manuscript.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Lambda/Pure/Examples}
\set{title}{Examples}
\begin{subsection}

Since, in lambda calculus, abstractions and their applications are the only building blocks available for assembly of the plethora of common programming idioms that occur routinely throughout computer science, every such idiom must, somehow, be formulated as an appropriate function, whose arguments are themselves functions. This minimality may be appealing to a mathematician but can easily prove intimidating to a programmer accustomed to dealing with more intuitive operational constructs such as integers, conditionals and loops.

Fortunately, all such constructs may be readily formulated in lambda calculus as «combinators», or functions that modify other functions, so that the user is not burdened with inventing  a new lambda abstraction for every programming task required in the course of a software design project. As the first example of such combinators, let us consider the two familiar boolean constants “true” and “false”, which, in the pure lambda calculus, are usually defined as follows:

\NewHaskellToken	{pureTrue}				{Ord}	{pure-true}
\NewHaskellToken	{pureFalse}				{Ord}	{pure-false}

\NewHaskellResult	{pureTrue}					{PureTermT}
\NewHaskellResult	{pureFalse}					{PureTermT}

\IndexTerm		{pureTrue}					{pure_true (pure_term)}
\IndexTerm		{pureFalse}					{pure_false (pure_term)}

\begin{code}
pureTrue	= PureLambda (x) {-dot-} (PureLambda (y) {-dot-} (PureVar x))
pureFalse	= PureLambda (x) {-dot-} (PureLambda (y) {-dot-} (PureVar y))
\end{code}

Intuitively, both “"pureTrue"” and “"pureFalse"” represent «binary functions» of two arguments "x" and "y", such that “"pureTrue"” always returns the first argument "x" while “"pureFalse"” returns the second of its operands "y". In this way, the two constants allow us to select between two arbitrary lambda expressions, capturing the notion of «choice» central to the concept of a boolean constant. Accordingly, the familiar conditional construct “"if p then x else y"” may be defined in the pure lambda calculus as the following «ternary function» of three arguments:

\NewHaskellToken	{pureIf}				{Ord}	{pure-if}
\NewHaskellResult	{pureIf}					{PureTermT}
\IndexTerm		{pureIf}					{pure_if (pure_term)}

\begin{code}
pureIf = PureLambda (p) {-dot-} (PureLambda (x) {-dot-} (PureLambda (y) {-dot-} (PureVar p `PureApp` PureVar x `PureApp` PureVar y)))
\end{code}

The expression “"pureIf `PureApp` p `PureApp` f1 `PureApp` f2"” evaluates to "f1" if "p" is equal to “"pureTrue"” and "f2" if "p" is “"pureFalse"”, by applying its "x" and "y" operands to the condition argument "p". To further convince ourselves that “"pureTrue"” and “"pureFalse"” do, in fact, represent boolean constants, let us define the three common boolean operations “"not"”, “"&&"” and “"||"”:

\NewHaskellToken	{pureNot}				{Ord}	{pure-not}
\NewHaskellToken	{pureAnd}				{Ord}	{pure-and}
\NewHaskellToken	{pureOr}				{Ord}	{pure-or}

\NewHaskellResult	{pureNot}					{PureTermT}
\NewHaskellResult	{pureAnd}					{PureTermT}
\NewHaskellResult	{pureOr}					{PureTermT}

\IndexTerm		{pureNot}					{pure_not (pure_term)}
\IndexTerm		{pureAnd}					{pure_and (pure_term)}
\IndexTerm		{pureOr}					{pure_or (pure_term)}

\begin{code}
pureNot		= PureLambda (p) {-dot-} (PureLambda (x) {-dot-} (PureLambda (y) {-dot-} (PureVar p `PureApp` PureVar y `PureApp` PureVar x)))
pureAnd		= PureLambda (p) {-dot-} (PureLambda (q) {-dot-} (PureLambda (x) {-dot-} (PureLambda (y) {-dot-} (PureVar p `PureApp` --
			{-hv-}(PureVar q `PureApp` PureVar x `PureApp` PureVar y) `PureApp` PureVar y))))
pureOr		= PureLambda (p) {-dot-} (PureLambda (q) {-dot-} (PureLambda (x) {-dot-} (PureLambda (y) {-dot-} (PureVar p `PureApp` PureVar x `PureApp` --
			{-hv-}(PureVar q `PureApp` PureVar x `PureApp` PureVar y)))))
\end{code}

An application of the “"pureNot"” combinator to a boolean constant (or binary function) "p" constructs a new binary function which applies "x" and "y" to "p" in reverse order, effectively inverting its selection process. “"pureAnd p q"\,” constructs a binary function that uses "q" to choose between "x" and "y" and then applies "p" to choose between the result of "q" and "y" as the second alternative, effectively implementing the decision process \hbox{“"if p then (if q then x else y) else y"\,”} which, as every programmer knows, captures the notion of a logical conjunction. Similarly, “"pureOr p q"\,” implements the decision process “"if p then x else (if q then x else y)"” representing logical disjunction of "p" and "q".

The constants “"pureTrue"” and “"pureFalse"” were easy to define due to the finite nature of boolean algebra. A more interesting example of combinators is introduced by the concept of natural numbers 0, 1, 2 and so on. Readers may be surprised that such infinite sets can be expressed in the pure lambda calculus, well, naturally:

\NewHaskellToken	{pureZero}				{Ord}	{pure-zero}
\NewHaskellToken	{pureOne}				{Ord}	{pure-one}
\NewHaskellToken	{pureTwo}				{Ord}	{pure-two}
\NewHaskellToken	{pureThree}				{Ord}	{pure-three}
\NewHaskellToken	{pureFour}				{Ord}	{pure-four}

\NewHaskellResult	{pureZero}					{PureTermT}
\NewHaskellResult	{pureOne}					{PureTermT}
\NewHaskellResult	{pureTwo}					{PureTermT}
\NewHaskellResult	{pureThree}					{PureTermT}
\NewHaskellResult	{pureFour}					{PureTermT}

\IndexTerm		{pureZero}					{pure_zero (pure_term)}
\IndexTerm		{pureOne}					{pure_one (pure_term)}
\IndexTerm		{pureTwo}					{pure_two (pure_term)}
\IndexTerm		{pureThree}					{pure_three (pure_term)}
\IndexTerm		{pureFour}					{pure_four (pure_term)}

\begin{code}
pureZero	= PureLambda (f) {-dot-} (PureLambda (x) {-dot-} (PureVar (x)))
pureOne		= PureLambda (f) {-dot-} (PureLambda (x) {-dot-} (PureVar (f) `PureApp` PureVar (x)))
pureTwo		= PureLambda (f) {-dot-} (PureLambda (x) {-dot-} (PureVar (f) `PureApp` {-hv-}({-,,,-}PureVar (f) `PureApp` PureVar (x))))
pureThree	= PureLambda (f) {-dot-} (PureLambda (x) {-dot-} (PureVar (f) `PureApp` {-hv-}({-,,,-}PureVar (f) `PureApp` {-hv-}(PureVar (f) `PureApp` PureVar (x)))))
pureFour	= PureLambda (f) {-dot-} (PureLambda (x) {-dot-} (PureVar (f) `PureApp` {-hv-}({-,,,-}PureVar (f) `PureApp` {-hv-}(PureVar (f) `PureApp` --
																{-hv-}(PureVar (f) `PureApp` PureVar (x))))))
\end{code}

and so on, until every number required by the program has been defined. These lambda expressions are known as «Church numerals»; the number of "f"s in the body of each numeral encodes its numeric value. Just as the boolean constants “"pureTrue"” and “"pureFalse"” captured the primitive notion of choice, every Church numeral “pure-«n»” encodes the primitive notion of «counting» successive applications of a function to its argument. Such chained application of the result of one function "g" directly to another function "f" is known as «function composition», commonly written as “"f.g"”. Because composition can be considered as the multiplication operator of the world of lambda abstractions, the composition “"f.f"\,” of a function "f" with itself is traditionally written as "f^2", the composition “"f.f.f"\,” as "f^3" and so on. Using this notation, every Church numeral "n" can be formulated as a combinator which constructs the "n"-fold composition of its argument function "f^n". To see why Church numerals represent natural numbers, let us try to define the common arithmetic operations on these objects:

\NewHaskellToken	{pureAdd}				{Ord}	{pure-add}
\NewHaskellToken	{pureMul}				{Ord}	{pure-mul}
\NewHaskellToken	{pureExp}				{Ord}	{pure-exp}

\NewHaskellResult	{pureAdd}					{PureTermT}
\NewHaskellResult	{pureMul}					{PureTermT}
\NewHaskellResult	{pureExp}					{PureTermT}

\IndexTerm		{pureAdd}					{pure_add (pure_term)}
\IndexTerm		{pureMul}					{pure_mul (pure_term)}
\IndexTerm		{pureExp}					{pure_exp (pure_term)}

\begin{code}
pureAdd		= PureLambda (m) {-dot-} (PureLambda (n) {-dot-} (PureLambda (f) {-dot-} (PureLambda (x) {-dot-} --
			(PureVar (m) `PureApp` PureVar (f) `PureApp` {-hv-}(PureVar (n) `PureApp` PureVar (f) `PureApp` PureVar (x))))))
pureMul		= PureLambda (m) {-dot-} (PureLambda (n) {-dot-} (PureLambda (f) {-dot-} (PureVar (m) `PureApp` {-hv-}(PureVar (n) `PureApp` PureVar (f)))))
pureExp		= PureLambda (m) {-dot-} (PureLambda (n) {-dot-} (PureVar (n) `PureApp` PureVar (m)))
\end{code}

The lambda abstractions “"pureAdd"”, “"pureMul"” and “"pureExp"” implement the arithmetic operations "m + n", "m * n" and "m^n", respectively. Notice how the composition of Church numerals implements multiplication of the corresponding natural numbers, making our definitions of “"pureMul"” and “"pureExp"” trivial. Although superficially more complicated, the above definition of “"pureAdd"” is also quite intuitive: we simply concatenate the two application sequences by inserting the body of "n" at the end of the chain in "m".

Composite objects such as pairs are also representable as lambda combinators. For example, the notion of a pair of lambda terms is conveniently captured as follows:

\NewHaskellToken	{purePair}			{Ord}	{pure-pair}
\NewHaskellToken	{pureFirst}			{Ord}	{pure-first}
\NewHaskellToken	{pureSecond}			{Ord}	{pure-second}

\NewHaskellResult	{purePair}				{PureTermT}
\NewHaskellResult	{pureFirst}				{PureTermT}
\NewHaskellResult	{pureSecond}				{PureTermT}

\IndexTerm		{purePair}				{pure_pair (pure_term)}
\IndexTerm		{pureFirst}				{pure_first (pure_term)}
\IndexTerm		{pureSecond}				{pure_second (pure_term)}

\begin{code}
purePair	= PureLambda (x) {-dot-} (PureLambda (y) {-dot-} (PureLambda (f) {-dot-} ({-,,,-}PureVar f `PureApp` PureVar x `PureApp` PureVar y)))
pureFirst	= PureLambda (z) {-dot-} (PureVar z `PureApp` {-hv-}(PureLambda (x) {-dot-} (PureLambda (y) {-dot-} ({-,-}PureVar x))	))
pureSecond	= PureLambda (z) {-dot-} (PureVar z `PureApp` {-hv-}(PureLambda (x) {-dot-} (PureLambda (y) {-dot-} (PureVar y))	))
\end{code}

The expression “"purePair `PureApp` tau1 `PureApp` tau2"\!” produces a combinator that, when, applied to a binary function "f", makes the two individual components of the pair available to "f" as its two operand values. The expressions “"pureFirst `PureApp` tau"\,” and “"pureSecond `PureApp` tau"\,” extract these component from the pair "tau" by applying it to a binary function which returns its first or second parameter, respectively.

Many more common and powerful combinators may be defined using the similar approach of capturing the essence of the underlying concept. However, by now, I hope that the reader has gained sufficient intuition for the behaviour of lambda abstractions, so that, without further ado, we can proceed to formalising the general meaning of lambda expressions.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Lambda/Pure/FV}
\set{title}{Free Variables}
\begin{subsection}

A lambda abstraction of the form “"PureLambda(x){-dot-}(tau)"\,” is said to «bind» all occurrences of the variable "x" in "tau". If a variable is not bound by any lambda abstraction within the expression in which it appears, it is known as a «free variable» and the corresponding expression is said to be an «open expression». Conversely, an expression that does not contain any free variables is said to be «closed». The set of all variables appearing free in a lambda expression can be constructed as follows:

\BeginHaskellToken	{pureFreeVariables}			{Open}	{FV}
  \HaskellArgument							{PureTermT}
  \HaskellResult							{SetT PureVarT}
\EndHaskellToken

\IndexTerm		{pureFreeVariables}				{FV pure_term (pure_vars)}

\begin{code}
pureFreeVariables {-[*]-} :: PureTermT -> SetT PureVarT

pureFreeVariables {-sem-}(PureVar x)				= listToSet[x]
pureFreeVariables {-sem-}(PureLambda (x) {-dot-} (tau))		= pureFreeVariables{--}(tau) `difference` listToSet[x]
pureFreeVariables {-sem-}(tau1 `PureApp` tau2)			= pureFreeVariables{--}(tau1) `union` pureFreeVariables{--}(tau2)
\end{code}

where the notation “"SetT PureVarT"” appearing in the signature of “"pureFreeVariables"” represents the type of all identifier sets. The notation “"listToSet[x]"” constructs a singleton set consisting of "x" alone, while the operators “\,"`union`"\,” and “"`difference`"\!” return the union and difference of two sets, respectively. All of these definitions are taken directly from the standard Haskell libraries discussed in Appendix \ref{Additional} and are typeset through an appropriate application of the formatting rules described in Chapter \ref{Haskell}.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Lambda/Pure/Substitution}
\set{title}{Capture-Avoiding Substitution}
\begin{subsection}

Before turning our attention to a semantic study of the pure lambda calculus, we must also formalise the notion of a «capture-avoiding substitution». Intuitively, a substitution of the form “"tau1 `pureSubstitution` (x {-,-} {-:-}, tau2)"” systematically replaces all free occurrences of the variable "x" in "tau1" with the term "tau2". Note that any bound occurrences of "x" in "tau2" are not replaced, so that the result of substituting "tau" for "x" in the expression “"{-hv-}(PureLambda(x){-dot-}({-,-}x)){-,-}x"\,” is “"{-hv-}(PureLambda(x){-dot-}({-,-}x)){--}tau"\,” rather than “"{-hv-}(PureLambda(x){-dot-}(tau)){--}tau"\,”. Capture-avoiding substitution can be defined in Haskell as a function with the following type signature:

\BeginHaskellToken	{pureSubstitution}			{Void}	{}
  \HaskellArgument							{PureTermT}
  \HaskellToken		{{-/-}}					{Bin}	{\slash}
  \BeginHaskellParens						{Open}	{\lbrace}
    \RawHaskellArgument							{(PureVarT{-,,-}{-:-},{-,,-}PureTermT)}
  \EndHaskellParens						{Close}	{\rbrace}
  \HaskellResult							{PureTermT}
\EndHaskellToken

\BeginHaskellToken	{`pureSubstitution`}			{Op}	{\slash}
  \BeginHaskellParens						{Open}	{\lbrace}
  \EndHaskellParens						{Close}	{\rbrace}
\EndHaskellToken

\IndexTerm		{pureSubstitution}				{/ pure_term pure_variable pure_term (pure_term)}

\begin{code}
pureSubstitution {- ({-[*]-}) {-/-} ({-[*]-}) -} :: PureTermT -> (PureVarT {-,,-} {-:-}, {-,,-}PureTermT) -> PureTermT
\end{code}

Barring certain cumbersome technicalities, a typical implementation of this function assumes the following inductive form:

\begin{code}
pureSubstitution ({-sem-}(PureVar x))				{-/-} ({-sem-}(x' {-:-}, tau'))	 | (x == x')				= tau'
												 | (x /= x')				= {-sem-}(PureVar x)
pureSubstitution ({-sem-}(PureLambda (x) {-dot-}(tau)))		{-/-} ({-sem-}(x' {-:-}, tau'))	 | (x == x')				= {-sem-}(PureLambda (y) {-dot-} (tau'))
												 | (x /@ pureFreeVariables{--}(tau'))	--
	= {-sem-}(PureLambda (x) {-dot-} ({-sem-}(tau `pureSubstitution` (x' {-:-}, tau'))))
pureSubstitution ({-sem-}(tau1 `PureApp` tau2))			{-/-} ({-sem-}(x' {-:-}, tau'))						--
	= {-sem-}({-sem-}(tau1 `pureSubstitution` (x' {-:-}, tau')) `PureApp` {-sem-}(tau2 `pureSubstitution` (x' {-:-}, tau')))
\end{code}

in which the notation “"listToSet [a {-:-}, b]"” stands simply for a fancy typesetting of an ordinary Haskell pair “"(a, b)"”. Appendix \ref{Additional} introduces a number of such presentation forms, whose only purpose is to improve exposition of certain important constructs such as variable substitutions, in an attempt to bridge the gap between established mathematical notations and their rendition in the Haskell language. In the actual source code, all of these constructs materialise simply as pairs of ordinary parentheses symbols “"()"”.

A careful reader will observe that the above formulation of substitution represents a partial function, undefined whenever the expression being substituted contains a free occurrence of a variable bound within the expression being modified. Since the terms being substituted into an expression are usually closed, this restriction is usually irrelevant in practice and the above definition of capture-avoiding substitution proves sufficient for the purpose of the present discussion. Nevertheless, the extended Etude calculus defined later in this chapter goes to some lengths in order to lift the restriction for the actual intermediate program representation deployed in the design of our C compiler.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Lambda/Pure/Operational}
\set{title}{Operational Semantics}
\begin{subsection}

A lambda expression of the form “"{-hv-}(PureLambda (x) {-dot-} (tau1)){--}tau2"\!” is known as a «beta redex». It represents an application of the term "tau2" to the function “"PureLambda (x) {-dot-} (tau1)"\!”. Such expressions are equivalent to the result of evaluating the function, formalised by the substitution of "tau2" for every free occurrence of "x" in "tau1". This form of substitution corresponds closely to the intuitive notion of calling a function in an imperative programming language, since it renders an application of a function to a particular argument value equivalent to an instance of the function's body with all free occurrences of the formal parameter instantiated to the supplied expression. The process of replacing a beta redex with the corresponding substituted expression is known as «beta reduction». If we apply beta reduction systematically to eliminate all redexes from an expression, we arrive at the expression's «beta-normal form» which may be viewed as a outcome of «evaluating» that expression. It is a remarkable result known as the «Church-Rosser property» that every lambda expression has at most one beta-normal form. This theorem, proven for the pure lambda calculus in 1936 by Church and Rosser [Rosser36], allows us to view the beta-normal form of an expression as its «value» and renders beta reduction an attractive basis for an operational interpretation of the pure lambda calculus.

\NewHaskellToken	{pureEval}				{Open}	{\ZapfChancery{E}}

We can formulate an operational semantics of the pure lambda calculus as a function "pureEval" which takes lambda expressions to their fully-evaluated beta-normal forms. Observe that multiple implementations of "pureEval" are possible, corresponding to the different viable orders in which beta redexes can be eliminated from some lambda expressions. Perhaps the most obvious evaluation strategy, known as an «applicative order evaluation», is to follow the intuitive understanding of the function call mechanism, according to which arguments are always substituted into function bodies in their fully-reduced beta-normal forms. In Haskell, this can be specified as follows:

\BeginHaskellToken	{pureEvalA}				{Open}	{\ZapfChancery{E}\sub{\LKeyword{a}}}
  \HaskellArgument							{PureTermT}
  \HaskellResult							{PureTermT}
\EndHaskellToken
\NewHaskellToken	{pureEvalA'}				{Open}	{\ZapfChancery{E}\vprime\sub{\LKeyword{a}}}

\IndexTerm		{pureEvalA}					{Ea pure_term (pure_term)}

\begin{code}
pureEvalA {-[*]-}, pureEvalA' {-[*]-} :: PureTermT -> PureTermT

pureEvalA   {-sem-}(PureVar x)		 				= {-sem-}(PureVar x)
pureEvalA   {-sem-}(PureLambda(x){-dot-}(tau))				= {-sem-}(PureLambda(x){-dot-}({-sem-}(pureEvalA{--}(tau))))
pureEvalA   {-sem-}(tau1`PureApp`tau2)					= {-sem-}({-sem-}(pureEvalA'{--}(pureEvalA{--}(tau1))) `PureApp` {-sem-}(pureEvalA{--}(tau2)))
{-{

where the auxiliary function "pureEvalA'" performs the actual beta reduction of an applicative term formed from a pair of lambda expressions already reduced into their respective normal forms:

}-}
pureEvalA'  {-sem-}({-hv-}(PureLambda(x){-dot-}(tau1))`PureApp`tau2)	= pureEvalA{--}(tau1 `pureSubstitution` (x {-:-}, tau2))
pureEvalA'  {-sem-}(tau1`PureApp`tau2)					= {-sem-}(tau1 `PureApp` tau2)
\end{code}

It can be shown that, for all closed lambda expressions "tau", if "pureEvalA{--}(tau)" terminates, then it produces a beta-normal form, although the proof of this simple theorem is rather involved in practice, due to various technicalities mentioned in our earlier discussion of the capture avoiding substitution operator from Section \ref{Lambda/Pure/Substitution}. Accordingly, I leave verification of this theorem as an exercise for a keen reader.

However, it should be noted that "pureEvalA" cannot construct the beta-normal form of every possible lambda expression. Consider, for example, the expression “"{-hv-}(PureLambda(x){-dot-}({-,-}x)){--}{-hv-}(PureLambda(x){-dot-}({-,-}x))"”. No matter how hard we try, we cannot eliminate all beta redexes from this expression, since reducing one redex through beta conversion merely gives rise to another one in the reduced expression. In fact, this expression remains unchanged by beta reduction! Such lambda terms are said to be «diverging» and are distinguished by the equivalence of “"tau1{-,-}tau2"\!” with the expression "tau1" itself for every argument value "tau2". They are usually represented collectively by the symbol “\!"undefined"\!”. It would be tempting to disregard diverging expressions as irrelevant and forbid them from ever appearing in lambda terms, but, unfortunately, it turns out that this cannot be done without unduly limiting the computational power of the calculus. In particular, observe that diverging expressions may be passed safely as arguments to some other expressions; for example, the beta-normal form of “"{-hv-}(PureLambda(x){-dot-}(y)){--}tau"” is equivalent to the variable "y" for all lambda terms "tau", including the diverging ones. Unfortunately, "pureEvalA" fails to construct the beta-normal form of “"{-hv-}(PureLambda(x){-dot-}(y)){--}{-hv-}({-hv-}(PureLambda(x){-dot-}({-,-}x)){--}{-hv-}(PureLambda(x){-dot-}({-,-}x)))"”, by attempting to obtain the non-existent beta-normal form of the term “"{-hv-}(PureLambda(x){-dot-}({-,-}x)){--}{-hv-}(PureLambda(x){-dot-}({-,-}x))"”. We say that such evaluation functions «diverge» when applied to diverging lambda terms.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Lambda/Pure/Normal}
\set{title}{Normal Order Evaluation}
\begin{subsection}

Fortunately, as shown by Rosser [Rosser36], it is possible to reformulate "pureEval" in such a way that it always finds the beta-normal form of every lambda expression for which the beta-normal form exists. Unsurprisingly, this strategy order is known as the «normal order evaluation» and it can be modelled in Haskell as the following definition:

\BeginHaskellToken	{pureEvalN}				{Open}	{\ZapfChancery{E}\sub{\,\LKeyword{n}}}
  \HaskellArgument							{PureTermT}
  \HaskellResult							{PureTermT}
\EndHaskellToken

\NewHaskellToken	{pureEvalN'}				{Open}	{\ZapfChancery{E}\vprime\sub{\,\LKeyword{n}}}

\IndexTerm		{pureEvalN}					{En pure_term (pure_term)}

\begin{code}
pureEvalN {-[*]-}, pureEvalN' {-[*]-} :: PureTermT -> PureTermT
\end{code}

such that:

\begin{code}
pureEvalN	{-sem-}(PureVar x)						= {-sem-}(PureVar x)
pureEvalN	{-sem-}(PureLambda(x){-dot-}(tau))				= {-sem-}(PureLambda(x){-dot-}({-sem-}(pureEvalN{--}(tau))))
pureEvalN	{-sem-}(tau1`PureApp`tau2)					= {-sem-}({-sem-}(pureEvalN'(pureEvalN{--}(tau1))) `PureApp` tau2)

pureEvalN'	{-sem-}({-hv-}(PureLambda(x){-dot-}(tau1))`PureApp`tau2)	= pureEvalN{--}(tau1 `pureSubstitution` (x {-:-}, tau2))
pureEvalN'	{-sem-}(tau1`PureApp`tau2)					= {-sem-}(tau1 `PureApp` {-sem-}(pureEvalN{--}(tau2)))
\end{code}

Intuitively, this evaluation strategy always reduces the «head» of an application expression before reducing the corresponding function argument. Note that, under the normal order evaluation, the argument expressions are only evaluated if they are actually referenced within the corresponding function bodies. However, normal order evaluation is not entirely about performance. One consequence of admitting this evaluation strategy in a formulation of an operational semantics for the pure lambda calculus is an ability to handle some important cases of diverging expressions. Perhaps the best known of all such constructs is the «fixed point combinator», sometimes called the «Y combinator» following the nomenclature of Curry. This combinator can be defined in the pure lambda calculus as the following expression “"pureFix"”:

\NewHaskellToken	{pureFix}				{Ord}	{pure-fix}
\NewHaskellResult	{pureFix}					{PureTermT}
\IndexTerm		{pureFix}					{pure_fix (pure_term)}

\begin{code}
pureFix = PureLambda (f) {-dot-} ({-hv-}(PureLambda (x) {-dot-} (PureVar f `PureApp` {-hv-}(PureVar x `PureApp` PureVar x))) --
			`PureApp` {-hv-}(PureLambda (x) {-dot-} (PureVar f `PureApp` {-hv-}(PureVar x `PureApp` PureVar x))))
\end{code}

The “"pureFix"” function was initially discovered by Moses Schönfinkel [Schonfinkel24, BauerMengelberg67] and rediscovered by Haskell Curry after Schönfinkel's work was culled by Stalin's imperialism and the outbreak of World War II [Curry58, Curry72]. It takes a single argument "f" (which we assume to be a lambda abstraction) and constructs a new function that applies "f" to a somewhat-unintelligible argument with a structure similar to the diverging expression “"{-hv-}(PureLambda(x){-dot-}({-,-}x)){--}{-hv-}(PureLambda(x){-dot-}({-,-}x))"” mentioned earlier, except that, in the fixed point combinator, the body "x" is replaced with an application expression “"f{--}{-hv-}(x{-,,-}x)"”. If we try to reduce the body of the constructed function “"pureFix f"”, we will end up with the original expression applied to "f" itself. In other words, “"pureFix"” calls "f" with a an infinite sequence of applications “"f{--}{-hv-}({-,,,-}f{--}{-hv-}({-,,,-}f{--}{-hv-}({-,,,-}f{--}{-hv-}(..))))"”, or, put differently, “"pureFix f"\,” is equivalent to “"f{--}{-hv-}(pureFix f)"\,”, since adding another "f" to an already-infinite sequence of "f"s is no different from the original infinite sequence.

At this point, the reader may wonder why such bizarre expression could ever be useful. We note that, since the argument of "f" is equivalent to “"pureFix f"”, the fixed point operator provides a means for the body of "f" to refer to itself. In other words, “"pureFix"” allows us to define self-referencing or «recursive» lambda expressions. As mentioned earlier, such expressions may have normal forms (or «converge») if, eventually, "f" is reduced to an abstraction that does not refer to its parameter. Extensive experience with lambda calculus has shown that recursive expressions are of an immense practical importance and that they are required by virtually every useful program ever written.

Nevertheless, for pragmatic reasons applicative order evaluation plays an important rôle in the study of an operational semantics of the pure lambda calculus. Since many common lambda abstractions such as “"PureLambda(x){-dot-}({-hv-}({-,,,-}f{-,-}x{-,,-}x))"” contain multiple references to their parameters, applying "pureEvalN" to such lambda terms, which substitutes an unreduced argument into the function body, results in a separate evaluation of that argument for its every occurrence within the lexical syntax of the program. As terms grow larger, the amount of additional work mandated by normal order evaluation grows exponentially with their complexity. Such operational semantics of the lambda calculus may insinuate that a perfectly-sensible lambda expression “"PureLambda(x){-dot-}({-hv-}({-,,,-}f{-,-}x{-,,-}x))"” is prohibitively expensive to apply in a real program, while in fact its actual evaluation on a real computational system can be efficient and pleasant. In the interest of relevance, we cannot afford for our evaluation model to disperse such misinformation and, accordingly, we should endeavour to reflect the computational reality of the language within its specification with a greater degree of immediacy. The reader is assured that such formulation will be presented in Section \ref{Lambda/Terms} for the full-featured variant of the calculus deployed as the intermediate program representation in this work.

\end{subsection}
\end{section}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%