\BeginHaskellToken	{SVAR}		{Ord}	{\LKeyword{v}}
  \HaskellModifier				{\sub}
  \HaskellArgument				{Integer}
\EndHaskellToken

\set{capture}{hidden}
\begin{code}
module Lambda.Simplifier (
	renameModule,
	simplifyModule,
) where

import Additional
import Lambda.Syntax
import Lambda.Atoms hiding (renameAtom, renameAtoms)
import Lambda.Terms
import Lambda.Modules hiding (renameData)


-- RENAMED VARIABLE TYPE --

newtype SVarT = SVAR Integer deriving (Eq, Ord, Show)


-- RENAMING --

renameAtom :: (Ord nu) => (MapT (nu) (upsilon) {->-}, AtomT nu) -> (AtomT upsilon)

renameAtom {-sem-}(_S {->-}, VAR nu)						= VAR (_S#(nu))
renameAtom {-sem-}(_S {->-}, IMM (x) (phi))					= IMM (x) (phi)
renameAtom {-sem-}(_S {->-}, OP1 (op) (phi) (alpha))				= OP1 (op) (phi) (renameAtom{--}(_S, alpha))
renameAtom {-sem-}(_S {->-}, OP2 (alpha1) (op) (phi) (alpha2))			= OP2 (renameAtom{--}(_S, alpha1)) (op) (phi) (renameAtom{--}(_S, alpha2))


renameAtoms :: (Ord nu) => (MapT (nu) (upsilon) {->-}, AtomsT nu) -> (AtomsT upsilon)

renameAtoms {-sem-}(_S {->-}, alpha_set) 					= [ renameAtom{--}(_S {->-}, alpha_k) | alpha_k <- alpha_set ]


renameTerm :: (Ord nu) => ([upsilon], MapT (nu) (upsilon) {->-}, TermT nu) -> ([upsilon], TermT upsilon)

renameTerm {-sem-}(_G, _S {->-}, RET (alpha_set))				= (_G, RET (renameAtoms{--}(_S {->-}, alpha_set)))
renameTerm {-sem-}(_G, _S {->-}, APP (alpha) (alpha_set))			= (_G, APP (renameAtom{--}(_S {->-}, alpha)) (renameAtoms{--}(_S {->-}, alpha_set)))
renameTerm {-sem-}(_G, _S {->-}, SYS (pi) (alpha_set))				= (_G, SYS (pi) (renameAtoms{--}(_S {->-}, alpha_set)))
renameTerm {-sem-}(_G, _S {->-}, IFF (alpha) {-THEN-} (tau1) {-ELSE-} (tau2))	= let 	(_G1, tau1') 	= renameTerm{--}(_G,   _S   {->-}, tau1)
											(_G2, tau2') 	= renameTerm{--}(_G1,  _S   {->-}, tau2)
										  in 	(_G2, IFF (renameAtom{--}(_S {->-}, alpha)) {-THEN-} (tau1') {-ELSE-} (tau2'))
renameTerm {-sem-}(_G, _S {->-}, NEW (xi_set))					= (_G, NEW (xi_set))
renameTerm {-sem-}(_G, _S {->-}, DEL (xi_set))					= (_G, DEL (xi_set))
renameTerm {-sem-}(_G, _S {->-}, GET (alpha, mu_set) (phi))			= (_G, GET (renameAtom{--}(_S {->-}, alpha), mu_set) (phi))
renameTerm {-sem-}(_G, _S {->-}, SET (alpha1, mu_set) (phi) {-TO-} (alpha2))	= (_G, SET (renameAtom{--}(_S {->-}, alpha1), mu_set) (phi) {-TO-} (renameAtom{--}(_S {->-}, alpha2)))
renameTerm {-sem-}(_G, _S {->-}, INI (alpha1, mu_set) (phi) {-TO-} (alpha2))	= (_G, INI (renameAtom{--}(_S {->-}, alpha1), mu_set) (phi) {-TO-} (renameAtom{--}(_S {->-}, alpha2)))

renameTerm {-sem-}(_G, _S {->-}, LET [] (tau))					= renameTerm{--}(_G, _S {->-}, tau)
renameTerm {-sem-}(_G, _S {->-}, LET [(nu_set {-=-}, tau1)] (tau2))		= let	(_G1, tau1')	= renameTerm{--}(_G,   _S   {->-}, tau1)
											(_G2, tau2')	= renameTerm{--}(_G',  _S'  {->-}, tau2)
											(nu_set')	= takeFromList{--}(lengthOfList{--}(nu_set), _G1)
											(_G')		= dropFromList(lengthOfList{--}(nu_set), _G1)
											(_S')		= _S // listToMap (nu_set `zipList` _G1)
										  in	(_G2, LET [nu_set' =: tau1'] (tau2'))
renameTerm {-sem-}(_G, _S {->-}, LET (beta_set) (tau))				= renameTerm{--}(_G, _S {->-}, LET [head{--}(beta_set)] $ LET (tail{--}(beta_set)) $ tau)


renameItems :: (Ord nu, Ord upsilon) => ([upsilon], MapT (nu) (upsilon) {->-}, [{-raw-}(nu {-=-}, ItemT nu)]) -> ([upsilon], ItemsT upsilon)

renameItems {-sem-}(_G, _S {->-}, {-raw-}[(nu {-=-}, FUNI (FUN (nu_set) {-DOT-} (tau)))]) ---
										= let 	(nu_set')	= takeFromList{--}(lengthOfList{--}(nu_set), _G)
											(_G')		= dropFromList(lengthOfList{--}(nu_set), _G)
											(_S')		= _S // listToMap (nu_set `zipList` _G)
											(_G'', tau')	= renameTerm{--}(_G', _S' {->-}, tau)
										  in	(_G'', listToMap [ _S#(nu) =: FUNI (FUN (nu_set') {-DOT-} (tau')) ])
renameItems {-sem-}(_G, _S {->-}, {-raw-}[(nu {-=-}, OBJI (gamma_set) {-OF-} (xi_set))]) ---
										= (_G, listToMap [ _S#(nu) =: OBJI (renameData{--}(_S {->-}, gamma_set)) {-OF-} (xi_set) ])
renameItems {-sem-}(_G, _S {->-}, {-raw-}[(nu {-=-}, IMPI x)])			= (_G, listToMap [ _S#(nu) =: IMPI x ])

renameItems {-sem-}(_G, _S {->-}, {-empty-}[])					= (_G, emptyMap)
renameItems {-sem-}(_G, _S {->-}, iota_set)					= let	(_G1, iota_set1') = renameItems{--}(_G,   _S  {->-}, [head{--}(iota_set)])
											(_G2, iota_set2') = renameItems{--}(_G1,  _S  {->-}, tail{--}(iota_set))
										  in	(_G2, iota_set1' `mapUnion` iota_set2')


renameData :: (Ord nu, Ord upsilon) => (MapT (nu) (upsilon) {->-}, DataT nu) -> DataT upsilon

renameData {-sem-}(_S {->-}, delta_set) ---
	= listToSet [ (n_k, phi_k, mu_set_k, renameAtom{--}(_S {->-}, alpha_k)) | (n_k, phi_k, mu_set_k, alpha_k) <- setToList delta_set ]


renameModule :: (Ord nu, Ord upsilon) => ([upsilon], ModuleT nu) -> ModuleT upsilon

renameModule {-sem-}(_G {->-}, MODULE (tau) {-EXPORT-} (xi_set) {-WHERE-} (iota_set)) ---
	= let	(_G')			= dropFromList{--}(mapCardinality (iota_set), _G)
		(_S)			= listToMap (setToList (dom{--}(iota_set)) `zipList` _G)
		(_G1, tau')		= renameTerm{--}(_G', _S {->-}, tau)
		(_G2, iota_set') 	= renameItems{--}(_G1, _S {->-}, mapToList iota_set)
		(xi_set')		= listToMap [ {-sem-}(x_k =: _S#(nu_k)) | {-sem-}(x_k {-=-}, nu_k) <- mapToList xi_set ]
	  in	(MODULE (tau') {-EXPORT-} (xi_set') {-WHERE-} (iota_set'))

			
-- JUMP PROPAGATOR --

propagateTerm :: (Ord nu) => (ItemsT nu {->-}, TermT nu) -> TermT nu

propagateTerm {-sem-}(iota_set {->-}, APP (VAR nu) (alpha_set)) | nu @@ dom{--}(iota_set) --
	= case iota_set#(nu) of
		{-sem-}(FUNI (FUN (nu_set') {-DOT-} (LET (beta_set) (tau))))		-> APP (VAR nu) (alpha_set)
		{-sem-}(FUNI (FUN (nu_set') {-DOT-} (tau)))				-> tau `substTerm` listToMap (nu_set' `zipList` alpha_set)
		{-sem-}(other)								-> APP (VAR nu) (alpha_set)

propagateTerm {-sem-}(iota_set {->-}, RET (alpha_set))					= RET (alpha_set)
propagateTerm {-sem-}(iota_set {->-}, APP (alpha) (alpha_set)) 				= APP (alpha) (alpha_set)
propagateTerm {-sem-}(iota_set {->-}, SYS (pi) (alpha_set))				= SYS (pi) (alpha_set)
propagateTerm {-sem-}(iota_set {->-}, IFF (alpha) {-THEN-} (tau1) {-ELSE-} (tau2))	= IFF (alpha) 	{-THEN-} (propagateTerm{--}(iota_set {->-}, tau1)) --
												   	{-ELSE-} (propagateTerm{--}(iota_set {->-}, tau2))
propagateTerm {-sem-}(iota_set {->-}, NEW (xi_set))					= NEW (xi_set)
propagateTerm {-sem-}(iota_set {->-}, DEL (xi_set))					= DEL (xi_set)
propagateTerm {-sem-}(iota_set {->-}, GET (alpha, mu_set) (phi))			= GET (alpha, mu_set) (phi)
propagateTerm {-sem-}(iota_set {->-}, SET (alpha1, mu_set) (phi) {-TO-} (alpha2))	= SET (alpha1, mu_set) (phi) {-TO-} (alpha2)
propagateTerm {-sem-}(iota_set {->-}, INI (alpha1, mu_set) (phi) {-TO-} (alpha2))	= INI (alpha1, mu_set) (phi) {-TO-} (alpha2)

propagateTerm {-sem-}(iota_set {->-}, LET [] (tau))					= propagateTerm{--}(iota_set {->-}, tau)
propagateTerm {-sem-}(iota_set {->-}, LET [(nu_set {-=-}, tau1)] (tau2))		= LET [nu_set =: propagateTerm{--}(iota_set {->-}, tau1)] $
											  propagateTerm{--}(iota_set {->-}, tau2)
propagateTerm {-sem-}(iota_set {->-}, LET (beta_set) (tau))				= propagateTerm{--}(iota_set {->-}, LET [head{--}(beta_set)] $  LET (tail{--}(beta_set)) $ tau)


propagateItem :: (Ord nu) => (ItemsT nu {->-}, ItemT nu) -> ItemT nu

propagateItem {-sem-}(iota_set {->-}, FUNI (FUN (nu_set) {-DOT-} (tau)))		= FUNI (FUN (nu_set) {-DOT-} (propagateTerm{--}(iota_set {->-}, tau)))
propagateItem {-sem-}(iota_set {->-}, OBJI (gamma_set) {-OF-} (xi_set))			= OBJI (gamma_set) {-OF-} (xi_set)
propagateItem {-sem-}(iota_set {->-}, IMPI x)						= IMPI x


propagateItems :: (Ord nu) => ItemsT nu -> ItemsT nu

propagateItems {-sem-}(iota_set) ---
	= listToMap [ (nu_k =: propagateItem{--}(iota_set {->-}, iota_k)) | (nu_k {-=-}, iota_k) <- mapToList iota_set ]

propagatedJumps :: (Ord nu) => ItemsT nu -> (MapT (nu) (AtomT nu))

propagatedJumps {-sem-}(iota_set) ---
	= listToMap [ (nu1 {-:-}, VAR nu2) | (nu1 {-=-}, FUNI (FUN (nu_set) {-DOT-} (APP (VAR nu2) (alpha_set)))) <- mapToList iota_set,
			alpha_set == coerceList VAR nu_set && nu2 /@ listToSet nu_set ]


-- OBJECT REUSER --

reuseTerm :: (Ord nu) => ([nu] {->-}, TermT nu) -> TermT nu

reuseTerm {-sem-}(nu_set' {->-}, LET [([nu] {-=-}, NEW (xi_set))] (tau))	= LET [[nu] =: NEW (xi_set)] $ reuseTerm{--}(nu:nu_set' {->-}, tau)
reuseTerm {-sem-}(nu_set' {->-}, LET [([] {-=-}, DEL (xi_set))] (tau)) ---
  | nextNew{--}(tau) == Just xi_set						= reuseNew{--}(nu_set' {->-}, tau)
  | otherwise									= LET [[] =: DEL (xi_set)] $ reuseTerm{--}(tail{--}(nu_set') {->-}, tau)
  
reuseTerm {-sem-}(nu_set' {->-}, LET [(nu_set {-=-}, tau1)] (tau2))		= LET [nu_set =: tau1] $ reuseTerm{--}(nu_set' {->-}, tau2)
reuseTerm {-sem-}(nu_set' {->-}, LET (beta_set) (tau))				= error "Denormalised LET form"

reuseTerm {-sem-}(nu_set' {->-}, IFF (alpha) {-THEN-} (tau1) {-ELSE-} (tau2))	= IFF (alpha) 	{-THEN-} (reuseTerm{--}(nu_set' {->-}, tau1)) --
												{-ELSE-} (reuseTerm{--}(nu_set' {->-}, tau2))
reuseTerm {-sem-}(nu_set' {->-}, tau)						= tau


nextNew :: (Ord nu) => TermT nu -> Maybe EnvelopeT

nextNew {-sem-}(LET [([nu] {-=-}, NEW (xi_set))] (tau))				= Just xi_set
nextNew {-sem-}(LET [([] {-=-}, DEL (xi_set))] (tau))				= Nothing
nextNew {-sem-}(LET (beta_set) (tau))						= nextNew{--}(tau)
nextNew {-sem-}(tau)								= Nothing


reuseNew :: (Ord nu) => ([nu] {->-}, TermT nu) -> TermT nu

reuseNew {-sem-}(nu_set' {->-}, LET [([nu] {-=-}, NEW (xi_set))] (tau))		= reuseTerm{--}(nu_set' {->-}, tau `substTerm` listToMap [ (nu {-:-}, VAR (head{--}(nu_set'))) ])
reuseNew {-sem-}(nu_set' {->-}, LET (beta_set) (tau))				= LET (beta_set) $ reuseNew{--}(nu_set' {->-}, tau)


reuseItem :: (Ord nu) => ItemT nu -> ItemT nu

reuseItem {-sem-}(FUNI (FUN (nu_set) {-DOT-} (tau)))				= FUNI (FUN (nu_set) {-DOT-} (reuseTerm{--}({-empty-}[] {->-}, tau)))
reuseItem {-sem-}(OBJI (gamma_set) {-OF-} (xi_set))				= OBJI (gamma_set) {-OF-} (xi_set)
reuseItem {-sem-}(IMPI x)							= IMPI x


reuseItems :: (Ord nu) => ItemsT nu -> ItemsT nu

reuseItems {-sem-}(iota_set) = listToMap [ nu_k =: reuseItem{--}(iota_k) | (nu_k, iota_k) <- mapToList iota_set ]


-- TODO: INLINER --

countAtom :: (Ord nu) => AtomT nu -> MapT (nu) (Integer)

countAtom {-sem-}(alpha) = listToMap [ (nu_k, 7) | nu_k <- setToList (fvOfAtom{--}(alpha)) ]


countAtoms :: (Ord nu) => AtomsT nu -> MapT (nu) (Integer)

countAtoms {-sem-}(alpha_set) = listToMap [ (nu_k, 7) | nu_k <- setToList (fvOfAtoms{--}(alpha_set)) ]


countTerm :: (Ord nu) => TermT nu -> MapT (nu) (Integer)

countTerm {-sem-}(RET (alpha_set))					= countAtoms{--}(alpha_set)
countTerm {-sem-}(APP (VAR nu) (alpha_set))				= listToMap [ (nu, 1) ]  `multiUnion` countAtoms{--}(alpha_set)
countTerm {-sem-}(APP (alpha) (alpha_set))				= countAtom{--}(alpha) `multiUnion` countAtoms{--}(alpha_set)
countTerm {-sem-}(SYS (pi) (alpha_set))					= countAtoms{--}(alpha_set)
countTerm {-sem-}(IFF (alpha) {-THEN-} (tau1) {-ELSE-} (tau2))		= countAtom{--}(alpha) `multiUnion` countTerm{--}(tau1) `multiUnion` countTerm{--}(tau2)
countTerm {-sem-}(NEW (xi_set))						= emptyMap
countTerm {-sem-}(DEL (xi_set))						= emptyMap
countTerm {-sem-}(GET (alpha, mu_set) (phi))				= countAtom{--}(alpha)
countTerm {-sem-}(SET (alpha1, mu_set) (phi) {-TO-} (alpha2))		= countAtoms{--}[alpha1, alpha2]
countTerm {-sem-}(INI (alpha1, mu_set) (phi) {-TO-} (alpha2))		= countAtoms{--}[alpha1, alpha2]

countTerm {-sem-}(LET [(nu_set {-=-}, tau1)] (tau2))			= countTerm{--}(tau1) `multiUnion` countTerm{--}(tau2) `multiUnion` listToMap [ (nu_k, 7) | nu_k <- nu_set ]
countTerm {-sem-}(LET (beta_set) (tau))					= error "Denormalised LET form"


countItem :: (Ord nu) => ItemT nu -> MapT (nu) (Integer)

countItem {-sem-}(FUNI (FUN (nu_set) {-DOT-} (tau)))			= countTerm{--}(tau) `multiUnion` listToMap [ (nu_k, 7) | nu_k <- nu_set ]
countItem {-sem-}(OBJI (gamma_set) {-OF-} (xi_set))			= listToMap [ (nu_k, 7) | nu_k <- setToList (fvOfItem{--}(OBJI (gamma_set) {-OF-} (xi_set))) ]
countItem {-sem-}(IMPI x)						= emptyMap


countItems :: (Ord nu) => ItemsT nu -> MapT (nu) (Integer)

countItems {-sem-}(iota_set) = foldl (multiUnion) (emptyMap) [ countItem{--}(iota_k) `multiUnion` (listToMap [ (nu_k, 7) ] `mapSetIntersection` fvOfItem{--}(iota_k)) --
									| (nu_k, iota_k) <- mapToList iota_set ]


multiUnion :: (Ord nu) => MapT (nu) (Integer) -> MapT (nu) (Integer) -> MapT (nu) (Integer)

multiUnion {-sem-}(a_set) {-sem-}(b_set) = (a_set `mapSetDifference` c_set) `mapUnion` (b_set `mapSetDifference` c_set) `mapUnion` listToMap [ (nu_k, 7) | nu_k <- setToList c_set ]
 where c_set = dom{--}(a_set) `intersection` dom{--}(b_set)


-- SIMPLIFIER --

simplifyAtom :: (Ord nu) => AtomT nu -> AtomT nu

simplifyAtom {-sem-}(VAR nu)						= VAR nu
simplifyAtom {-sem-}(IMM (x) (phi))					= IMM (x) (phi)
simplifyAtom {-sem-}(OP1 (op) (phi) (alpha))
  | op == {-sem-}(CNV phi)						= simplifyAtom{--}(alpha)
  | otherwise								= OP1 (op) (phi) (simplifyAtom{--}(alpha))
simplifyAtom {-sem-}(OP2 (alpha1) (op) (phi) (alpha2))			= OP2 (simplifyAtom{--}(alpha1)) (op) (phi) (simplifyAtom{--}(alpha2))


simplifyAtoms :: (Ord nu) => AtomsT nu -> AtomsT nu

simplifyAtoms {-sem-}(alpha_set) 					= [ simplifyAtom{--}(alpha_k) | alpha_k <- alpha_set ]


simplifyTerm :: (Ord nu) => TermT nu -> TermT nu

simplifyTerm {-sem-}(RET (alpha_set))					= RET (simplifyAtoms{--}(alpha_set))
simplifyTerm {-sem-}(APP (alpha) (alpha_set))				= APP (simplifyAtom{--}(alpha)) (simplifyAtoms{--}(alpha_set))
simplifyTerm {-sem-}(SYS (pi) (alpha_set))				= SYS (pi) (simplifyAtoms{--}(alpha_set))
simplifyTerm {-sem-}(IFF (alpha) {-THEN-} (tau1) {-ELSE-} (tau2))	= IFF (simplifyAtom{--}(alpha)) {-THEN-} (simplifyTerm{--}(tau1)) {-ELSE-} (simplifyTerm{--}(tau2))
simplifyTerm {-sem-}(NEW (xi_set))					= NEW (xi_set)
simplifyTerm {-sem-}(DEL (xi_set))					= DEL (xi_set)
simplifyTerm {-sem-}(GET (alpha, mu_set) (phi))				= GET (simplifyAtom{--}(alpha), mu_set) (phi)
simplifyTerm {-sem-}(SET (alpha1, mu_set) (phi) {-TO-} (alpha2))	= SET (simplifyAtom{--}(alpha1), mu_set) (phi) {-TO-} (simplifyAtom{--}(alpha2))
simplifyTerm {-sem-}(INI (alpha1, mu_set) (phi) {-TO-} (alpha2))	= INI (simplifyAtom{--}(alpha1), mu_set) (phi) {-TO-} (simplifyAtom{--}(alpha2))

simplifyTerm {-sem-}(LET [] (tau))					= simplifyTerm{--}(tau)
simplifyTerm {-sem-}(LET [(nu_set {-=-}, tau1)] (tau2))			= case simplifyTerm{--}(tau1) of
	{-sem-}(RET (alpha_set'))					-> simplifyTerm{--}(tau2 `substTerm` (listToMap (nu_set `zipList` alpha_set')))
	{-sem-}(NEW (xi_set')) | listToSet nu_set `intersection` fvOfTerm{--}(tau2) == emptySet ---
									-> simplifyTerm'{--}(xi_set' {->-}, simplifyTerm{--}(tau2))
	{-sem-}(GET (alpha', mu_set') (phi')) | mu_set' == emptySet && listToSet nu_set `intersection` fvOfTerm{--}(tau2) == emptySet ---
									-> simplifyTerm{--}(tau2)
	{-sem-}(LET (beta_set') (tau1'))				-> simplifyTerm{--}(LET (beta_set') $ LET [nu_set =: tau1'] $ tau2)
	{-sem-}(tau1')							-> LET [nu_set =: tau1'] (simplifyTerm{--}(tau2))
simplifyTerm {-sem-}(LET (beta_set) (tau))				= simplifyTerm{--}(LET [head{--}(beta_set)] $  LET (tail{--}(beta_set)) $ tau)


simplifyTerm' :: (Ord nu) => (EnvelopeT {->-}, TermT nu) -> TermT nu

simplifyTerm' {-sem-}(xi_set' {->-}, LET [([] {-=-}, DEL (xi_set))] (tau))	| xi_set == xi_set'	= tau
simplifyTerm' {-sem-}(xi_set' {->-}, LET (beta_set) (tau))						= LET (beta_set) (simplifyTerm'{--}(xi_set' {->-}, tau))
simplifyTerm' {-sem-}(xi_set' {->-}, DEL (xi_set)) 				| xi_set == xi_set'	= RET []
simplifyTerm' {-sem-}(xi_set' {->-}, tau)								= tau


simplifyItem :: (Ord nu) => ItemT nu -> ItemT nu

simplifyItem {-sem-}(FUNI (FUN (nu_set) {-DOT-} (tau)))			= FUNI (FUN (nu_set) {-DOT-} (simplifyTerm{--}(tau)))
simplifyItem {-sem-}(OBJI (gamma_set) {-OF-} (xi_set))			= OBJI (simplifyData{--}(gamma_set)) {-OF-} (xi_set)
simplifyItem {-sem-}(IMPI x)						= IMPI x


simplifyItems :: (Ord nu) => ItemsT nu -> ItemsT nu

simplifyItems {-sem-}(iota_set)						= listToMap [ (nu_k =: simplifyItem{--}(iota_k)) | (nu_k {-=-}, iota_k) <- mapToList iota_set ]


simplifyData :: (Ord nu) => DataT nu -> DataT nu

simplifyData {-sem-}(delta_set) ---
	= listToSet [ (n_k, phi_k, mu_set_k, simplifyAtom{--}(alpha_k)) | (n_k, phi_k, mu_set_k, alpha_k) <- setToList delta_set ]


simplifyModule :: (Ord nu) => ModuleT nu -> ModuleT SVarT

simplifyModule {-sem-}(m)
  | (m1 == m2)	= m2
  | otherwise	= simplifyModule{--}(m2)

 where	{-sem-}(m1@(MODULE (tau) {-EXPORT-} (xi_set) {-WHERE-} (iota_set)))	= renameModule{--}(_G {->-}, m)
 	{-raw-}(m2)								= MODULE (tau') {-EXPORT-} (xi_set') {-WHERE-} (iota_set'')
 	{-raw-}(tau')								= propagateTerm{--}(iota_set'' {->-}, simplifyTerm{--}(tau))
 	{-raw-}(xi_set')							= listToMap [ (x_k, nu_k') | (x_k, nu_k) <- mapToList xi_set, VAR nu_k' <- [_S#(nu_k)] ]
 	{-raw-}(iota_set')							= propagateItems{--}(reuseItems{--}(simplifyItems{--}(iota_set)))
 	{-raw-}(iota_set'')							= iota_set' `mapSetIntersection` --
 											(fvOfTerm{--}(tau') `union` fvOfExports{--}(xi_set') `union` fvOfItems{--}(iota_set'))
 	{-raw-}(_G)								= [ SVAR k | k <- [ 1 .. ] ]
 	{-raw-}(_S)								= listToMap [ (nu_k, VAR nu_k) | (x_k, nu_k) <- mapToList xi_set ] // propagatedJumps{--}(iota_set')
\end{code}
