\set{capture}{hidden}
\begin{code}
module Lambda.Syntax (
	showIndent,
	AtomT (..),
	AtomsT,
	UnaryOpT (..),
	BinaryOpT (..),
	FormatT (..),
	GenreT (..),
	EncodingT,
	TermT (..),
	PrimT,
	BindingsT,
	BindingT,
	ParametersT,
	bindTerm,
	(=:),
	AttrT (..),
	FEnvT,
	FunctionT (..),
	OEnvT,
	A,
	M,
	StackT,
	StackFrameT,
	EnvelopeT,
	EnvelopeElementT,
	ModuleT (..),
	ModulesT (..),
	ExportsT,
	ItemsT,
	ItemT (..),
	DataT,
) where

import Additional

infixr 0 =:
infix 9 `APP`, `SYS`

instance Show FormatT where
	showsPrec p (gamma`FMT`epsilon) 			= showParen (p > 0) (shows gamma . showString "`FMT`" . shows epsilon)

instance Show nu => Show (AtomT nu) where
	showsPrec p (VAR nu)					= showParen (p > 10) (showString "VAR " . showsPrec 11 nu)
	showsPrec p (IMM (x) (phi))
	  | x == toRational (truncate (x))			= showParen (p > 10) (showString "IMM (" . showsPrec 0 (truncate (x)::Integer) . showString ") " . showsPrec 11 phi)
	  | otherwise						= showParen (p > 10) (showString "IMM (" . showsPrec 0 x . showString ") " . showsPrec 11 phi)
	showsPrec 100 (alpha)					= showString "({-hv-}(" . showsPrec 0 alpha . showString "))"
	showsPrec p (OP1 (op) (phi) (alpha))			= showParen (p > 10) (showString "OP1 (" . showsPrec 0 op . showString ") " . 
								  showsPrec 11 phi . showString " {--} {-hv-}" . showsPrec 100 alpha)
	showsPrec p (OP2 (alpha1) (op) (phi) (alpha2))		= showParen (p > 10) (showString "OP2 " . showsPrec 100 alpha1 . showString " (" . showsPrec 0 op . showString ") " .
								  showsPrec 11 phi . showChar ' ' . showsPrec 100 alpha2)
	 
	showList (alpha_set)					= showString "[" . showList' alpha_set . showString "]"
	  where
	    showList' []					= id
	    showList' [alpha]					= showsPrec 0 alpha
	    showList' (alpha:alpha_set)				= showsPrec 0 alpha . showString " {-hv-}, " . showList' alpha_set

instance Show nu => Show (TermT nu) where
	showsPrec _ tau		= showTerm 0 0 tau

instance Show nu => Show (FunctionT nu) where
	showsPrec _ f						= showFunction f

instance Show nu => Show (ModuleT nu) where
	showsPrec _ (MODULE tau xi_set iota_set)		= showString "MODULE ( ---\n" .
								  showIndent 1 0 . showTerm 1 0 tau . showString ")\n" .
								  showString "{-EXPORT-} " . showExports 11 xi_set . showChar '\n' .
								  showString "{-WHERE-} " . showItems 11 iota_set

instance Show nu => Show (ItemT nu) where
	showsPrec _ iota					= showItem iota

showTerm :: Show nu => Int -> Int -> TermT nu -> ShowS
showTerm n m (RET (alpha_set))					= showString "RET\t" . showOffset m . showList alpha_set
showTerm n m (APP (alpha) (alpha_set))				= showString "APP " . showsPrec 100 alpha . showChar ' ' . showList alpha_set
showTerm n m (SYS (pi) (alpha_set))				= showString "SYS (" . showsPrec 0 pi . showString ") " . showList alpha_set
showTerm n m (IFF (alpha) {-THEN-} (tau1) {-ELSE-} (tau2))	= showString "IFF " . showsPrec 100 alpha . showString " {-THEN-} (\n" .
								  showIndent n m . showString "{-Q-}\t " . showTerm (n + 1) (1) tau1 . showString ")\n" .
								  showIndent n m . showString "{-ELSE-} (\n" .
								  showIndent n m . showString "{-Q-}\t " . showTerm (n + 1) (1) tau2 . showString ")"
showTerm n m (NEW (xi_set))					= showString "NEW (" . showEnvelope xi_set . showChar ')'
showTerm n m (DEL (xi_set))					= showString "DEL (" . showEnvelope xi_set . showChar ')'
showTerm n m (GET (alpha, mu_set) (phi))
  | mu_set == emptySet						= showString "GET   (" . showsPrec 0 alpha . showString " {-hide-}, {-hide-}emptySet) " . 
  												showsPrec 11 phi
  | otherwise							= showString "GET   (" . showsPrec 0 alpha . showString " {-hv-}, " . showAttr 0 mu_set . showString ") " .
												showsPrec 11 phi
showTerm n m (SET (alpha1, mu_set) (phi) {-TO-} (alpha2))
  | mu_set == emptySet						= showString "SET   (" . showsPrec 0 alpha1 . showString " {-hide-}, {-hide-}emptySet) " . 
												showsPrec 11 phi . showString " {-TO-} " . showsPrec 100 alpha2
  | otherwise							= showString "SET   (" . showsPrec 0 alpha1 . showString " {-hv-}, " . showAttr 0 mu_set . showString ") " . 
												showsPrec 11 phi . showString " {-TO-} " . showsPrec 100 alpha2
showTerm n m (INI (alpha1, mu_set) (phi) {-TO-} (alpha2))	
  | mu_set == emptySet						= showString "INI   (" . showsPrec 0 alpha1 . showString " {-hide-}, {-hide-}emptySet) " .
								  				showsPrec 11 phi . showString " {-TO-} " . showsPrec 100 alpha2
  | otherwise							= showString "INI   (" . showsPrec 0 alpha1 . showString " {-hv-}, " . showAttr 0 mu_set . showString ") " . 
								  				showsPrec 11 phi . showString " {-TO-} " . showsPrec 100 alpha2
showTerm n m (LET (beta_set) (tau))				= showString "LET[\t" .  showOffset m . showBindings (n + 1) m beta_set . showString "] $\n" .
								  showIndent n m . showTerm n m tau

showBindings :: Show nu => Int -> Int -> BindingsT nu -> ShowS
showBindings n m []						= id
showBindings n m [beta]						= showBinding n m beta
showBindings n m (beta:beta_set)				= showBinding n m beta . showString " {-hv-},\n" . showIndent n m . showBindings n m beta_set

showBinding :: Show nu => Int -> Int -> BindingT nu -> ShowS
showBinding n m ([nu], RET [alpha])				= showParameters [nu] . showString " ---\n" .
								  showIndent n m . showString "  =:\t" . showOffset m . showString "{--}{-hide-}RET {-raw-}" . showList [alpha]
showBinding n m (nu_set, RET (alpha_set))			= showParameters nu_set . showString " ---\n" .
								  showIndent n m . showString "  =:\t" . showOffset m . showString "{--}{-hide-}RET {-hv-}" . showList alpha_set
showBinding n m (nu_set, tau)					= showParameters nu_set . showString " ---\n" .
								  showIndent n m . showString "  =:\t" . showOffset m . showTerm (n + 1) m tau

showParameters :: Show nu => ParametersT nu -> ShowS
showParameters [nu]						= showString "{-raw-}[" . showsPrec 0 nu . showString "]"
showParameters (nu_set)						= showString "{-hv-}[" . showParameters' nu_set . showString "]"

showParameters' :: Show nu => ParametersT nu -> ShowS
showParameters' []						= id
showParameters' [nu]						= showsPrec 0 nu
showParameters' (nu:nu_set)					= showsPrec 0 nu . showString " {-hv-}, " . showParameters' nu_set

showAttr :: Int -> SetT AttrT -> ShowS
showAttr p mu_set
  | mu_set == emptySet						= showParen (p > 10) (showString "emptySet")
  | otherwise							= showParen (p > 10) (showString "listToSet {-raw-}[" . showAttr' (setToList mu_set) . showString "]")

showAttr' :: [AttrT] -> ShowS
showAttr' [mu]							= showsPrec 0 mu
showAttr' (mu:mu_set)						= showsPrec 0 mu . showString " {-hv-}, " . showAttr' mu_set

showEnvelope :: EnvelopeT -> ShowS
showEnvelope xi_set
  | xi_set == emptySet						= showString "{-hide-}emptySet"
  | otherwise							= showString "listToSet {-raw-}[" . showEnvelope' 1 (setToList xi_set) . showString "]"

showEnvelope' :: Int -> [(Integer, FormatT, SetT AttrT)] -> ShowS
showEnvelope' 0 [(n, phi, mu_set)]
  | mu_set == emptySet						= showChar '(' . shows n . showString " {-hv-}, " . shows phi . showString " {-hide-}, {-hide-}emptySet)"
  | otherwise							= showChar '(' . shows n . showString " {-hv-}, " . shows phi . showString " {-hv-}, " . showAttr 0 mu_set . showChar ')'
showEnvelope' 1 [xi]						= showString "{-raw-}" . showEnvelope' 0 [xi]
showEnvelope' 2 [xi]						= showString "{-hv-}" . showEnvelope' 0 [xi]
showEnvelope' _ (xi:xi_set)					= showEnvelope' 2 [xi] . showString " {-hv-}, " . showEnvelope' 2 xi_set

showFunction :: Show nu => FunctionT nu -> ShowS
showFunction (FUN (nu_set) {-DOT-} tau)				= showString "FUN [" . showParameters' nu_set . showString "] {-DOT-} $\n" .
								  showIndent 2 0 . showTerm 2 0 tau

showExports :: Show nu => Int -> ExportsT nu -> ShowS
showExports p mu_set
  | mapCardinality (mu_set) == 0				= showParen (p > 10) (showString "{-hide-}emptyMap")
  | otherwise							= showParen (p > 10) (showString "listToMap {-raw-}[ ---\n" . 
  								  showIndent 1 0 . showExports' (mapToList mu_set) . showString "]")

showExports' :: Show nu => [(String, nu)] -> ShowS
showExports' [(x, nu)]						= shows x . showString " =: " . shows nu
showExports' (xi:xi_set)					= showExports' [xi] . showString " {-hide-},\n" . 
								  showIndent 1 0 . showExports' xi_set

showItems :: Show nu => Int -> ItemsT nu -> ShowS
showItems p iota_set
  | mapCardinality (iota_set) == 0				= showParen (p > 10) (showString "{-hide-}emptyMap")
  | otherwise							= showParen (p > 10) (showString "listToMap {-raw-}[ ---\n" . 
  								  showIndent 1 0 . showItems' (mapToList iota_set) . showString "]")

showItems' :: Show nu => [(nu, ItemT nu)] -> ShowS
showItems' [(nu, iota)]						= shows nu . showString " =: " . showItem iota
showItems' (iota:iota_set)					= showItems' [iota] . showString " {-hide-},\n\n" . 
								  showIndent 1 0 . showItems' iota_set

showItem :: Show nu => ItemT nu -> ShowS
showItem (OBJI (delta_set) {-OF-} (xi_set))			= showString "OBJI (" . showData delta_set . showString ") {-OF-} (" . showEnvelope xi_set . showChar ')'
showItem (FUNI f)						= showString "FUNI $ " . showFunction f
showItem (IMPI x)						= showString "IMPI " . shows x

showData :: Show nu => DataT nu -> ShowS
showData delta_set
  | cardinality (delta_set) == 0				= showString "{-hide-}emptySet"
  | otherwise							= showString "listToSet {-raw-}[" . showData' 1 (setToList delta_set) . showString "]"

showData' :: Show nu => Int -> [(Integer, FormatT, SetT AttrT, AtomT nu)] -> ShowS
showData' 0 [(n, phi, mu_set, alpha)]
  | mu_set == emptySet						= showChar '(' . shows n . showString " {-hv-}, " . shows phi . showString " {-hide-}, {-hide-}emptySet {-hv-}, " .
								  shows alpha . showChar ')'
  | otherwise							= showChar '(' . shows n . showString " {-hv-}, " . shows phi . showString " {-hv-}, " .
								  showAttr 0 mu_set . showString " {-hv-}, " . shows alpha . showChar ')'
showData' 1 [delta]						= showString "{-raw-}" . showData' 0 [delta]
showData' 2 [delta]						= showString "{-hv-}" . showData' 0 [delta]
showData' _ (delta:delta_set)					= showData' 2 [delta] . showString " {-hv-}, " . showData' 2 delta_set

showIndent :: Int -> Int -> ShowS
showIndent n m = showString (take n (repeat '\t')) . showOffset m

showOffset :: Int -> ShowS
showOffset m = showString (take m (repeat ' '))

\end{code}

\begin{syntax}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\BeginHaskellToken	{AtomT}					{Ord}	{«atom»}
  \HaskellContext							{(Ord nu)}
  \HaskellCommand						{1}	{\sub{#1}}
  \HaskellArgument							{nu}
  \HaskellResult							{Type}
\EndHaskellToken 
\NewVariable		{atom}

\BeginHaskellToken	{VAR}					{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{nu}
  \HaskellResult							{AtomT nu}
\EndHaskellToken

\BeginHaskellToken	{IMM}					{Open}	{\LKeyword{\#}\WhiteSpace}
  \HaskellArgument							{Rational}
  \HaskellModifier							{\WhiteSpace\sub}
  \HaskellArgument							{FormatT}
  \HaskellResult							{AtomT nu}
\EndHaskellToken

\BeginHaskellToken	{OP1}					{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{UnaryOpT}
  \HaskellModifier							{\WhiteSpace\sub}
  \HaskellArgument							{FormatT}
  \HaskellArgument							{AtomT nu}
  \HaskellResult							{AtomT nu}
\EndHaskellToken

\BeginHaskellToken	{OP2}					{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{AtomT nu}
  \HaskellArgument							{BinaryOpT}
  \HaskellModifier							{\WhiteSpace\sub}
  \HaskellArgument							{FormatT}
  \HaskellArgument							{AtomT nu}
  \HaskellResult							{AtomT nu}
\EndHaskellToken

\set{capture}		{AtomT}
\begin{code}
data AtomT {-[-} nu {-]-} =
	VAR nu										-- variable atoms
	| IMM (Rational) (FormatT)							-- constant atoms
	| OP1 (UnaryOpT) (FormatT) (AtomT nu)						-- unary operations
	| OP2 (AtomT nu) (BinaryOpT) (FormatT) (AtomT nu) deriving (Eq, Ord)		-- binary operations
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\BeginHaskellToken	{AtomsT}				{Ord}	{«atoms»}
  \HaskellContext							{(Ord nu)}
  \HaskellCommand						{1}	{\sub{#1}}
  \HaskellArgument							{nu}
  \HaskellResult							{Type}
\EndHaskellToken 
\NewVariable		{atoms}

\set{capture}		{AtomsT}
\begin{code}
type AtomsT {-[-} nu {-]-} =								-- lists of atoms
	[AtomT nu]
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\NewHaskellToken	{UnaryOpT}				{Ord}	{«unary-op»}
\NewHaskellResult	{UnaryOpT}					{Type}
\NewVariable		{unary_op}

\NewHaskellToken	{NEG}					{Ord}	{\minus}
\NewHaskellToken	{NOT}					{Ord}	{\sim}

\NewHaskellResult	{NEG}						{UnaryOpT}
\NewHaskellResult	{NOT}						{UnaryOpT}

\BeginHaskellToken	{CNV}					{Void}	{}
  \HaskellArgument							{FormatT}
  \HaskellResult							{UnaryOpT}
\EndHaskellToken

\set{capture}		{UnaryOpT}
\begin{code}
data UnaryOpT {-one-of-} =
	NEG {-alt-}| NOT {-alt-}| CNV (FormatT) deriving (Eq, Ord, Show)
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\NewHaskellToken	{BinaryOpT}				{Ord}	{«binary-op»}
\NewHaskellResult	{BinaryOpT}					{Type}
\NewVariable		{binary_op}

\NewHaskellToken	{ADD}					{Ord}	{\plus}
\NewHaskellToken	{SUB}					{Ord}	{\minus}
\NewHaskellToken	{MUL}					{Ord}	{\times}
\NewHaskellToken	{DIV}					{Ord}	{\div}
\NewHaskellToken	{REM}					{Ord}	{\rem}
\NewHaskellToken	{AND}					{Ord}	{\bitand}
\NewHaskellToken	{XOR}					{Ord}	{\bitxor}
\NewHaskellToken	{IOR}					{Ord}	{\bitor}
\NewHaskellToken	{SHL}					{Ord}	{\ll}
\NewHaskellToken	{SHR}					{Ord}	{\gg}
\NewHaskellToken	{CEQ}					{Ord}	{\eq}
\NewHaskellToken	{CNE}					{Ord}	{\ne}
\NewHaskellToken	{CLT}					{Ord}	{\lt}
\NewHaskellToken	{CGT}					{Ord}	{\gt}
\NewHaskellToken	{CLE}					{Ord}	{\le}
\NewHaskellToken	{CGE}					{Ord}	{\ge}

\NewHaskellResult	{ADD}						{BinaryOpT}
\NewHaskellResult	{SUB}						{BinaryOpT}
\NewHaskellResult	{MUL}						{BinaryOpT}
\NewHaskellResult	{DIV}						{BinaryOpT}
\NewHaskellResult	{REM}						{BinaryOpT}
\NewHaskellResult	{AND}						{BinaryOpT}
\NewHaskellResult	{XOR}						{BinaryOpT}
\NewHaskellResult	{IOR}						{BinaryOpT}
\NewHaskellResult	{SHL}						{BinaryOpT}
\NewHaskellResult	{SHR}						{BinaryOpT}
\NewHaskellResult	{CEQ}						{BinaryOpT}
\NewHaskellResult	{CNE}						{BinaryOpT}
\NewHaskellResult	{CLT}						{BinaryOpT}
\NewHaskellResult	{CGT}						{BinaryOpT}
\NewHaskellResult	{CLE}						{BinaryOpT}
\NewHaskellResult	{CGE}						{BinaryOpT}

\set{capture}		{BinaryOpT}
\begin{code}
data BinaryOpT {-one-of-} =
	ADD {-alt-}| SUB {-alt-}| MUL {-alt-}| DIV {-alt-}| REM {-alt-}| --
	AND {-alt-}| XOR {-alt-}| IOR {-alt-}| SHL {-alt-}| SHR {-alt-}| --
	CEQ {-alt-}| CNE {-alt-}| CLT {-alt-}| CGT {-alt-}| CLE {-alt-}| CGE deriving (Eq, Ord, Enum, Show)
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\NewHaskellToken	{FormatT}				{Ord}	{«format»}
\NewHaskellResult	{FormatT}					{Type}
\NewVariable		{format}

\BeginHaskellToken	{FMT}					{Void}	{}
  \HaskellArgument							{GenreT}
  \HaskellToken		{{-DOT-}}				{Ord}	{\LKeyword{.}}
  \HaskellArgument							{EncodingT}
  \HaskellResult							{FormatT}
\EndHaskellToken

\NewHaskellToken	{`FMT`}					{Op}	{\LKeyword{.}}

\set{capture}		{FormatT}
\begin{code}
data FormatT =
	FMT (GenreT) {-DOT-} (EncodingT) deriving (Eq, Ord)
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\NewHaskellToken	{GenreT}				{Ord}	{«genre»}
\NewHaskellResult	{GenreT}					{Type}
\NewVariable		{genre}

\NewHaskellToken	{NG}					{Ord}	{\LKeyword{n}}
\NewHaskellToken	{ZG}					{Ord}	{\LKeyword{z}}
\NewHaskellToken	{RG}					{Ord}	{\LKeyword{r}}
\NewHaskellToken	{FG}					{Ord}	{\LKeyword{f}}
\NewHaskellToken	{OG}					{Ord}	{\LKeyword{o}}

\NewHaskellResult	{NG}						{GenreT}
\NewHaskellResult	{ZG}						{GenreT}
\NewHaskellResult	{RG}						{GenreT}
\NewHaskellResult	{FG}						{GenreT}
\NewHaskellResult	{OG}						{GenreT}

\set{capture}		{GenreT}
\begin{code}
data GenreT {-one-of-} =
	NG {-alt-}| ZG {-alt-}| RG {-alt-}| FG {-alt-}| OG deriving (Eq, Ord, Enum, Show)
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\NewHaskellToken	{EncodingT}				{Ord}	{«encoding»}
\NewHaskellResult	{EncodingT}					{Type}
\NewVariable		{encoding}

\BeginHaskellToken	{EncodingT'}				{Void}	{}
  \HaskellArgument							{Integer}
  \HaskellResult							{EncodingT}
\EndHaskellToken

\set{capture}		{EncodingT}
\begin{code}
type EncodingT =
	Integer
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\BeginHaskellToken	{TermT}					{Ord}	{«term»}
  \HaskellContext							{(Ord nu)}
  \HaskellCommand						{1}	{\sub{#1}}
  \HaskellArgument							{nu}
  \HaskellResult							{Type}
\EndHaskellToken 
\NewVariable		{term}

\BeginHaskellToken	{RET}					{Ord}	{\LKeyword{ret}}
  \HaskellContext							{(Ord nu)}
  \BeginHaskellAlt
    \BeginHaskellList						{Open}	{\WhiteSpace\LKeyword{(}\WhiteSpace}
    \EndHaskellList						{Close}	{\WhiteSpace\LKeyword{)}}
  \EndHaskellAlt
  \BeginHaskellParens						{Open}	{\LKeyword{(}\WhiteSpace}
    \RawHaskellArgument							{AtomsT nu}
  \EndHaskellParens						{Close}	{\WhiteSpace\LKeyword{)}}
  \HaskellResult							{TermT nu}
\EndHaskellToken

\BeginHaskellToken	{APP}					{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{AtomT nu}
  \BeginHaskellAlt
    \BeginHaskellList						{Op}	{\WhiteSpace\LKeyword{(}\WhiteSpace}
    \EndHaskellList						{Close}	{\WhiteSpace\LKeyword{)}}
  \EndHaskellAlt
  \BeginHaskellParens						{Op}	{\LKeyword{(}\WhiteSpace}
    \RawHaskellArgument							{AtomsT nu}
  \EndHaskellParens						{Close}	{\WhiteSpace\LKeyword{)}}
  \HaskellResult							{TermT nu}
\EndHaskellToken

\BeginHaskellToken	{`APP`}					{Void}	{}
  \BeginHaskellAlt
    \BeginHaskellList						{Op}	{\WhiteSpace\LKeyword{(}\WhiteSpace}
    \EndHaskellList						{Close}	{\WhiteSpace\LKeyword{)}}
  \EndHaskellAlt
  \BeginHaskellParens						{Op}	{\WhiteSpace\LKeyword{(}\WhiteSpace}
  \EndHaskellParens						{Close}	{\WhiteSpace\LKeyword{)}}
\EndHaskellToken

\BeginHaskellToken	{SYS}					{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{PrimT}
  \BeginHaskellAlt
    \BeginHaskellList						{Op}	{\WhiteSpace\LKeyword{(}\WhiteSpace}
    \EndHaskellList						{Close}	{\WhiteSpace\LKeyword{)}}
  \EndHaskellAlt
  \BeginHaskellParens						{Op}	{\LKeyword{(}\WhiteSpace}
    \RawHaskellArgument							{AtomsT nu}
  \EndHaskellParens						{Close}	{\WhiteSpace\LKeyword{)}}
  \HaskellResult							{TermT nu}
\EndHaskellToken

\BeginHaskellToken	{`SYS`}					{Void}	{}
  \BeginHaskellAlt
    \BeginHaskellList						{Op}	{\WhiteSpace\LKeyword{(}\WhiteSpace}
    \EndHaskellList						{Close}	{\WhiteSpace\LKeyword{)}}
  \EndHaskellAlt
  \BeginHaskellParens						{Op}	{\WhiteSpace\LKeyword{(}\WhiteSpace}
  \EndHaskellParens						{Close}	{\WhiteSpace\LKeyword{)}}
\EndHaskellToken

\BeginHaskellToken	{IFF}					{Ord}	{\LKeyword{if}}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{AtomT nu}
  \HaskellToken		{{-THEN-}}				{Ord}	{\LKeyword{then}}
  \HaskellArgument							{TermT nu}
  \HaskellToken		{{-ELSE-}}				{Ord}	{\LKeyword{else}}
  \HaskellArgument							{TermT nu}
  \HaskellResult							{TermT nu}
\EndHaskellToken

\BeginHaskellToken	{NEW}					{Ord}	{\LKeyword{new}}
  \HaskellContext							{(Ord nu)}
  \BeginHaskellParens						{Open}	{\LKeyword{(}\WhiteSpace}
    \RawHaskellArgument							{EnvelopeT}
  \EndHaskellParens						{Close}	{\WhiteSpace\LKeyword{)}}
  \HaskellResult							{TermT nu}
\EndHaskellToken

\BeginHaskellToken	{DEL}					{Ord}	{\LKeyword{del}}
  \HaskellContext							{(Ord nu)}
  \BeginHaskellParens						{Open}	{\LKeyword{(}\WhiteSpace}
    \RawHaskellArgument							{EnvelopeT}
  \EndHaskellParens						{Close}	{\WhiteSpace\LKeyword{)}}
  \HaskellResult							{TermT nu}
\EndHaskellToken

\BeginHaskellToken	{GET}					{Ord}	{\LKeyword{get}}
  \HaskellContext							{(Ord nu)}
  \BeginHaskellParens						{Open}	{\LKeyword{\[}\WhiteSpace}
    \RawHaskellArgument							{AtomT nu {-hv-}, SetT AttrT}
  \EndHaskellParens						{Close}	{\WhiteSpace\LKeyword{\]}}
  \HaskellModifier							{\WhiteSpace\sub}
  \HaskellArgument							{FormatT}
  \HaskellResult							{TermT nu}
\EndHaskellToken

\BeginHaskellToken	{SET}					{Ord}	{\LKeyword{set}}
  \HaskellContext							{(Ord nu)}
  \BeginHaskellParens						{Open}	{\LKeyword{\[}\WhiteSpace}
    \RawHaskellArgument							{AtomT nu {-hv-}, SetT AttrT}
  \EndHaskellParens						{Close}	{\WhiteSpace\LKeyword{\]}}
  \HaskellModifier							{\WhiteSpace\sub}
  \HaskellArgument							{FormatT}
  \HaskellToken		{{-TO-}}				{Ord}	{\LKeyword{to}}
  \HaskellArgument							{AtomT nu}
  \HaskellResult							{TermT nu}
\EndHaskellToken

\BeginHaskellToken	{INI}					{Ord}	{\LKeyword{set\hbox to 0pt {\hss\sub{i}\!}}}
  \HaskellContext							{(Ord nu)}
  \BeginHaskellParens						{Open}	{\LKeyword{\[}\WhiteSpace}
    \RawHaskellArgument							{AtomT nu {-hv-}, SetT AttrT}
  \EndHaskellParens						{Close}	{\WhiteSpace\LKeyword{\]}}
  \HaskellModifier							{\WhiteSpace\sub}
  \HaskellArgument							{FormatT}
  \HaskellToken		{{-TO-}}				{Ord}	{\LKeyword{to}}
  \HaskellArgument							{AtomT nu}
  \HaskellResult							{TermT nu}
\EndHaskellToken

\BeginHaskellToken	{LET}					{Ord}	{\LKeyword{let}}
  \HaskellContext							{(Ord nu)}
  \BeginHaskellAlt
    \BeginHaskellList						{Void}	{}
    \EndHaskellList						{Close}	{\WhiteSpace\LKeyword{;}}
    \BeginHaskellParens						{Void}	{}
    \EndHaskellParens						{Void}	{}
  \EndHaskellAlt
  \BeginHaskellParens						{Void}	{}
    \RawHaskellArgument							{BindingsT nu}
  \EndHaskellParens						{Close}	{\WhiteSpace\LKeyword{;}}
  \HaskellArgument							{TermT nu}
  \HaskellResult							{TermT nu}
\EndHaskellToken

\set{capture}		{TermT}
\begin{code}
data TermT {-[-} nu {-]-} =
	RET (AtomsT nu)									-- lifted atoms
	| APP (AtomT nu) (AtomsT nu)							-- function application
	| SYS (PrimT) (AtomsT nu)							-- primitive operation
	| IFF (AtomT nu) {-THEN-} (TermT nu) {-ELSE-} (TermT nu)			-- conditional expressions
	| NEW  (EnvelopeT)								-- object introduction
	| DEL  (EnvelopeT)								-- object destruction
	| GET  (AtomT nu {-hv-}, SetT AttrT) (FormatT)					-- object inspection
	| SET  (AtomT nu {-hv-}, SetT AttrT) (FormatT) {-TO-} (AtomT nu)		-- object adjustment
	| INI  (AtomT nu {-hv-}, SetT AttrT) (FormatT) {-TO-} (AtomT nu)		-- object initialisation
	| LET  (BindingsT nu) (TermT nu) deriving (Eq, Ord)				-- monadic bindings
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\NewHaskellToken	{PrimT}					{Ord}	{«prim»}
\NewHaskellResult	{PrimT}						{Type}
\NewVariable		{prim}

\BeginHaskellToken	{PrimT'}				{Void}	{}
  \HaskellArgument							{Integer}
  \HaskellResult							{PrimT}
\EndHaskellToken

\set{capture}		{PrimT}
\begin{code}
type PrimT =
	Integer
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\BeginHaskellToken	{BindingsT}				{Ord}	{«bindings»}
  \HaskellContext							{(Ord nu)}
  \HaskellCommand						{1}	{\sub{#1}}
  \HaskellArgument							{nu}
  \HaskellResult							{Type}
\EndHaskellToken 
\NewVariable		{bindings}

\set{capture}		{BindingsT}
\begin{code}
type BindingsT {-[-} nu {-]-} =
	[BindingT nu]									-- set of term bindings
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\BeginHaskellToken	{BindingT}				{Ord}	{«binding»}
  \HaskellContext							{(Ord nu)}
  \HaskellCommand						{1}	{\sub{#1}}
  \HaskellArgument							{nu}
  \HaskellResult							{Type}
\EndHaskellToken 
\NewVariable		{binding}

\BeginHaskellToken	{{-bindTerm-}}				{Void}	{}
  \HaskellToken		{,}					{Ord}	{\LKeyword{=}}
\EndHaskellToken

\BeginHaskellToken	{{-=-}}					{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellToken		{,}					{Ord}	{\LKeyword{=}}
  \HaskellResult							{BindingT nu}
\EndHaskellToken

\set{capture}		{BindingT}
\begin{code}
type BindingT {-[-} nu {-]-} =
	{-raw-}(ParametersT nu {-=-}, TermT nu)						-- individual term binding
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\BeginHaskellToken	{ParametersT}				{Ord}	{«parameters»}
  \HaskellContext							{(Ord nu)}
  \HaskellCommand						{1}	{\sub{#1}}
  \HaskellArgument							{nu}
  \HaskellResult							{Type}
\EndHaskellToken 
\NewVariable		{parameters}

\set{capture}		{ParametersT}
\begin{code}
type ParametersT {-[-} nu {-]-} =
	[nu]
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\NewHaskellToken	{=:}					{Ord}	{\LKeyword{=}}

\set{capture}		{bindTerm}
\begin{code}
bindTerm, (=:) :: _T -> _U -> (_T, _U)

bindTerm _T _U = (_T, _U)
_T =: _U = (_T, _U)
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\NewHaskellToken	{AttrT}					{Ord}	{«mode»}
\NewHaskellResult	{AttrT}						{Type}
\NewVariable		{mode}

\NewHaskellToken	{CM}					{Ord}	{\LKeyword{c}}
\NewHaskellToken	{VM}					{Ord}	{\LKeyword{v}}

\NewHaskellResult	{CM}						{AttrT}
\NewHaskellResult	{VM}						{AttrT}

\set{capture}		{AttrT}
\begin{code}
data AttrT {-one-of-} =
	CM {-alt-}| VM deriving (Eq, Ord, Enum, Show)
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\BeginHaskellToken	{FEnvT}					{Ord}	{«f-env»}
  \HaskellContext							{(Ord nu)}
  \HaskellCommand						{1}	{\sub{#1}}
  \HaskellArgument							{nu}
  \HaskellResult							{Type}
\EndHaskellToken 
\NewVariable		{f_env}

\set{capture}		{FEnvT}
\begin{code}
type FEnvT {-[-} nu {-]-} =
	MapT (Integer) (FunctionT nu)
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\BeginHaskellToken	{FunctionT}				{Ord}	{«function»}
  \HaskellContext							{(Ord nu)}
  \HaskellCommand						{1}	{\sub{#1}}
  \HaskellArgument							{nu}
  \HaskellResult							{Type}
\EndHaskellToken 
\NewVariable		{function}

\BeginHaskellToken	{FUN}					{Open}	{\sslambda\WhiteSpace}
  \HaskellContext							{(Ord nu)}
  \BeginHaskellAlt
    \BeginHaskellList						{Void}	{}
    \EndHaskellList						{Void}	{}
    \HaskellToken	{{-DOT-}}				{Op}	{\WhiteSpace\LKeyword{.}\WhiteSpace}
    \BeginHaskellParens						{Void}	{}
    \EndHaskellParens						{Void}	{}
  \EndHaskellAlt
  \HaskellArgument							{ParametersT nu}
  \HaskellToken		{{-DOT-}}				{Op}	{\WhiteSpace\LKeyword{.}\WhiteSpace}
  \HaskellArgument							{TermT nu}
  \HaskellResult							{FunctionT nu}
\EndHaskellToken

\NewHaskellToken	{{-DOT-}}				{Op}	{\LKeyword{.}}

\set{capture}		{FunctionT}
\begin{code}
data FunctionT {-[-} nu {-]-} =
	FUN (ParametersT nu) {-DOT-} (TermT nu) deriving (Eq, Ord)
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\NewHaskellToken	{OEnvT}					{Ord}	{«o-env»}
\NewHaskellResult	{OEnvT}						{Type}
\NewVariable		{o_env}

\NewHaskellToken	{M}					{Ord}	{«M»}
\NewHaskellResult	{M}						{Type}

\NewHaskellToken	{A}					{Ord}	{«A»}
\NewHaskellResult	{A}						{Type}

\set{capture}		{OEnvT}
\begin{code}
type OEnvT = ---
	{-raw-}(M, A, EnvelopeT, EnvelopeT, StackT, Integer, Integer, Integer)
type M = ---
	MapT Integer Integer						
type A = ---
	MapT Integer Bool						
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\NewHaskellToken	{StackT}				{Ord}	{«stack»}
\NewHaskellResult	{StackT}					{Type}

\set{capture}		{StackT}
\begin{code}
type StackT =
	[StackFrameT]
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\NewHaskellToken	{StackFrameT}				{Ord}	{«stack-frame»}
\NewHaskellResult	{StackFrameT}					{Type}

\set{capture}		{StackFrameT}
\begin{code}
type StackFrameT =
	{-hv-}(Integer {-hv-}, EnvelopeT)
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\NewHaskellToken	{EnvelopeT}				{Ord}	{«envelope»}
\NewHaskellResult	{EnvelopeT}					{Type}
\NewVariable		{envelope}

\set{capture}		{EnvelopeT}
\begin{code}
type EnvelopeT =
	SetT EnvelopeElementT
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\NewHaskellToken	{EnvelopeElementT}			{Ord}	{«envelope-element»}
\NewHaskellResult	{EnvelopeElementT}				{Type}
\NewVariable		{envelope_element}

\set{capture}		{EnvelopeElementT}
\begin{code}
type EnvelopeElementT =
	{-hv-}(Integer {-hv-}, FormatT {-hv-}, SetT AttrT)
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\BeginHaskellToken	{ModuleT}				{Ord}	{«module»}
  \HaskellContext							{(Ord nu)}
  \HaskellCommand						{1}	{\sub{#1}}
  \HaskellArgument							{nu}
  \HaskellResult							{Type}
\EndHaskellToken 
\NewVariable		{module}

\BeginHaskellToken	{MODULE}				{Ord}	{\LKeyword{module}}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{TermT nu}
  \HaskellToken		{{-EXPORT-}}				{Ord}	{\LKeyword{export}}
  \HaskellArgument							{ExportsT nu}
  \HaskellToken		{{-WHERE-}}				{Ord}	{\LKeyword{where}}
  \HaskellArgument							{ItemsT nu}
  \HaskellResult							{ModuleT nu}
\EndHaskellToken

\set{capture}		{ModuleT}
\begin{code}
data ModuleT {-[-} nu {-]-} =
	MODULE (TermT nu) {-EXPORT-} (ExportsT nu) {-WHERE-} (ItemsT nu) deriving (Eq, Ord)
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\BeginHaskellToken	{ModulesT}				{Ord}	{«modules»}
  \HaskellContext							{(Ord nu)}
  \HaskellCommand						{1}	{\sub{#1}}
  \HaskellArgument							{nu}
  \HaskellResult							{Type}
\EndHaskellToken 
\NewVariable		{modules}

\set{capture}		{ModulesT}
\begin{code}
data ModulesT {-[-} nu {-]-} =
	MapT (ModuleT nu) ((MapT (nu) (AtomT nu)))
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\BeginHaskellToken	{ExportsT}				{Ord}	{«exports»}
  \HaskellContext							{(Ord nu)}
  \HaskellCommand						{1}	{\sub{#1}}
  \HaskellArgument							{nu}
  \HaskellResult							{Type}
\EndHaskellToken 
\NewVariable		{exports}

\set{capture}		{ExportsT}
\begin{code}
type ExportsT {-[-} nu {-]-} =
	MapT (String) (nu)
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\BeginHaskellToken	{ItemsT}				{Ord}	{«items»}
  \HaskellContext							{(Ord nu)}
  \HaskellCommand						{1}	{\sub{#1}}
  \HaskellArgument							{nu}
  \HaskellResult							{Type}
\EndHaskellToken 
\NewVariable		{items}

\set{capture}		{ItemsT}
\begin{code}
type ItemsT {-[-} nu {-]-} =
	MapT (nu) (ItemT nu)
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\BeginHaskellToken	{ItemT}					{Ord}	{«item»}
  \HaskellContext							{(Ord nu)}
  \HaskellCommand						{1}	{\sub{#1}}
  \HaskellArgument							{nu}
  \HaskellResult							{Type}
\EndHaskellToken 
\NewVariable		{item}

\BeginHaskellToken	{FUNI}					{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{FunctionT nu}
  \HaskellResult							{ItemT nu}
\EndHaskellToken

\BeginHaskellToken	{OBJI}					{Ord}	{\LKeyword{obj}}
  \HaskellContext							{(Ord nu)}
  \BeginHaskellParens						{Open}	{\LKeyword{(}\WhiteSpace}
    \RawHaskellArgument							{DataT nu}
  \EndHaskellParens						{Close}	{\WhiteSpace\LKeyword{)}}
  \HaskellToken		{{-OF-}}				{Ord}	{\LKeyword{of}}
  \BeginHaskellParens						{Open}	{\LKeyword{(}\WhiteSpace}
    \RawHaskellArgument							{EnvelopeT}
  \EndHaskellParens						{Close}	{\WhiteSpace\LKeyword{)}}
  \HaskellResult							{ItemT nu}
\EndHaskellToken

\BeginHaskellToken	{IMPI}					{Ord}	{\LKeyword{imp}}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{String}
  \HaskellResult							{ItemT nu}
\EndHaskellToken

\set{capture}		{ItemT}
\begin{code}
data ItemT {-[-} nu {-]-} =
	FUNI (FunctionT nu)
	| OBJI (DataT nu) {-OF-} (EnvelopeT)
	| IMPI (String) deriving (Eq, Ord)
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\BeginHaskellToken	{DataT}					{Ord}	{«data»}
  \HaskellContext							{(Ord nu)}
  \HaskellCommand						{1}	{\sub{#1}}
  \HaskellArgument							{nu}
  \HaskellResult							{Type}
\EndHaskellToken 
\NewVariable		{data}

\set{capture}		{DataT}
\begin{code}
type DataT {-[-} nu {-]-} =
	SetT (Integer {-hv-}, FormatT {-hv-}, SetT AttrT {-hv-}, AtomT nu)
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\end{syntax}
