\set{label}{MMIX}
\set{title}{Generating Code}
\set{citation}{Errors using inadequate data are much less\break
		than those using no data at all.}
\set{citation author}{Charles Babbage (1792--1871)}
\set{xcitation}{There's an old story about the person who wished\break
		his computer were as easy to use as his telephone.\break
		That wish has come true, since I no longer know\break
		how to use my telephone.}
\set{xcitation author}{Bjarne Stroustrup}
\begin{chapter}

\set{capture}{hidden}
\begin{code}

module MMIX (
	module MMIX.Syntax,
	module MMIX.Semantics,
	module MMIX.Lambda
) where

import Additional
import MMIX.Syntax
import MMIX.Semantics
import MMIX.Lambda

\end{code}

\vskip -12pt
\vfill

Ultimately, the elementary purpose of every compiler is a representation of its input programs in the designated target language, a rôle that has been traditionally assigned to a portion of its implementation known as a «back end». It is this final stage of the entire translation process to which I will now turn the reader's attention.

A satisfactory formal verification of a compiler's back end poses some non-trivial challenges beyond those already encountered in the previous two chapters. Usually, the rôle of the target language is assumed by an «instruction set architecture» of some computational hardware such as ARM, MIPS, SPARC or x86. In these languages, programs are represented universally by sequences of primitive entities known as «instructions», every one of which is encoded into a bit pattern known variously as a «byte» or a «word», depending on its precise size and semantics. Accordingly, the structure of typical target programs differs significantly from their rendition under languages such as C or Etude, which offer a vastly disparate abstraction of variables, objects and control flow structures. These fundamental differences already pose a substantial challenge for the design of a robust translation system, but even if we could somehow overcome all such difficulties, the task of establishing correctness for a compiler's back end remains complicated by a number of further issues.

First, of all, in the final stage of translation, we cannot readily assume the approach from Chapter \ref{C}, whereby the source language was defined through a mapping of its syntactic constructs onto the compiler's intermediate program representation. It would be futile to attempt a definition of Etude in terms of any given instruction set architecture, since most users would shiver at the thought of their programs having incomparable meanings on different computers. It would be equally risky to attack the problem using conventional extensional techniques, given that a formal establishment of an equivalence between the disparate semantic models of these two computational paradigms would require a precise definition of the targeted system, which few architectures are prepared to provide in the real world. In the interest of relevance, it would be vastly preferable to retain the view of the compiler's target as a formal language rather than an algebra, refraining from a direct operational description of the behaviours exhibited by its machine instructions and, instead, formulating their semantic interpretation through a translation into some other well-understood calculus of computable functions, similar in spirit to the one described in Chapter \ref{C} for our source language C.

As demonstrated earlier in Chapter \ref{Design}, if the translation semantics of both the source and the target language can be rendered as a pair of total functions "rho" and "psi" that share a common codomain in the form of the compiler's intermediate program representation, then a correctness of the entire translation system rests on a single equivalence judgement \hbox{"rho . psi_hat . psi == rho"}, known as «the principle of linear correctness», in which "psi_hat" represents the actual code generation function of the translation system. For the present compiler, this judgement is established formally in Section \ref{MMIX/Lambda}, together with a formal definition of the translation function "psi_hat". The actual semantics "psi" of our target architecture are described separately in Section \ref{MMIX/Semantics}, while a suitable rendition of "rho" has been already provided in Chapter \ref{C}, in the form of the function "meaningOfTranslationUnit", which maps all valid C programs to their Etude counterparts.

In order to sidestep any religious debates on the relative merits of various popular instruction set architectures such as SPARC or x86, in this chapter I choose to focus on a translation of Etude programs into the MMIX architecture, which was designed for the specific purpose of presenting low-level assembly programs [Knuth05]. The instruction set of MMIX provides a very realistic sample of the typical computational facilities supported by its industrial counterparts and, in principle, the architecture could be easily rendered into an efficient hardware implementation. Nevertheless, the unparalleled uniformity of all MMIX instruction forms greatly simplify the following discussion, which permits us to focus on the actual academic issues at hand instead of the peculiarities of some given computational system.

\InputFile	{MMIX/Syntax}
\InputFile	{MMIX/Semantics}
\InputFile	{Lambda/Arch}
\InputFile	{MMIX/Lambda}
\InputFile	{MMIX/Example}

\end{chapter}
