\set{capture}{hidden}
\begin{code}

module MMIX.Example (
	simple_compiled_example,
	compiled_example,
	optimised_compiled_example
) where

import Lambda
import MMIX
import C.Overview
import C.Example

simple_compiled_example = compileModule simple_example
compiled_example = compileModule (translated_example)
optimised_compiled_example = compileModule (simplifyModule translated_example)

swap 	= "iswap"
sort 	= "sort"
a	= "a"
b	= "b"
t	= "t"
i	= "i"
j	= "j"
p	= "p"
l	= "l"
r	= "r"

\end{code}

\LocalHaskellToken	{swap}					{Ord}	{{\SetFont{Courier}{medium}{italic}swap}}
\LocalHaskellToken	{sort}					{Ord}	{{\SetFont{Courier}{medium}{italic}sort}}
\LocalHaskellToken	{a}					{Ord}	{{\SetFont{Courier}{medium}{italic}a}}
\LocalHaskellToken	{b}					{Ord}	{{\SetFont{Courier}{medium}{italic}b}}
\LocalHaskellToken	{t}					{Ord}	{{\SetFont{Courier}{medium}{italic}t}}
\LocalHaskellToken	{i}					{Ord}	{{\SetFont{Courier}{medium}{italic}i}}
\LocalHaskellToken	{j}					{Ord}	{{\SetFont{Courier}{medium}{italic}j}}
\LocalHaskellToken	{p}					{Ord}	{{\SetFont{Courier}{medium}{italic}p}}
\LocalHaskellToken	{l}					{Ord}	{{\SetFont{Courier}{medium}{italic}l}}
\LocalHaskellToken	{r}					{Ord}	{{\SetFont{Courier}{medium}{italic}r}}

\FormatLocalCSymbol	{swap}						{{\SetFont{Courier}{medium}{italic}swap}}
\FormatLocalCSymbol	{sort}						{{\SetFont{Courier}{medium}{italic}sort}}
\FormatLocalCSymbol	{a}						{{\SetFont{Courier}{medium}{italic}a}}
\FormatLocalCSymbol	{b}						{{\SetFont{Courier}{medium}{italic}b}}
\FormatLocalCSymbol	{t}						{{\SetFont{Courier}{medium}{italic}t}}
\FormatLocalCSymbol	{i}						{{\SetFont{Courier}{medium}{italic}i}}
\FormatLocalCSymbol	{j}						{{\SetFont{Courier}{medium}{italic}j}}
\FormatLocalCSymbol	{p}						{{\SetFont{Courier}{medium}{italic}p}}
\FormatLocalCSymbol	{l}						{{\SetFont{Courier}{medium}{italic}l}}
\FormatLocalCSymbol	{r}						{{\SetFont{Courier}{medium}{italic}r}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{MMIX/Example}
\set{title}		{Example Translation}
\begin{section}

As an illustrative example of our compiler in action, let us present the MMIX rendition of the C translation unit discussed earlier in Section \ref{C/Example}, which, as the reader may recall, provided a textbook implementation of the quick sort algorithm. The resulting MMIX module exports two external names "swap" and "sort", which are bound to the text symbols "SYM 1" and "SYM 2", respectively. The data segment is empty, while the text segment defines eight distinct symbols, of which "SYM 0" represents the entry point into the entire program:

\let\RaggedBottom\true

\begin{haskell}
MMixLET ({-hide-}listToMap {-raw-}[ ---
  {-raw-}(SYM 1 {-,,-} {-:-}, {-,,-} ---
		swap) {-hide-},
  {-raw-}(SYM 2 {-,,-} {-:-}, {-,,-} ---
		sort)])
{-TEXT-} (..)
{-DATA-} ({-hide-}listToMap {-hide-}[])
{-IN-} {-raw-}(SYM 0)

{-{ First, symbol "SYM 0" represents the trivial initialiser term of the two C declarations found in the original source file: }-}

    {-raw-}(SYM 0 {-,,-} {-:-}, {-,,-} {-raw-}[ ---
		MMixInstr (MMixGOi)	(_Rt) {-,-}	(_Rk) {-,-}	(0) {-hide-}Nothing]) {-hide-},

{-{ The following section "SYM 1" implements the actual body of the C function "swap": }-}

    {-raw-}(SYM 1 {-,,-} {-:-}, {-,,-} {-raw-}[ ---
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, {-quad-} {-quad-} {-quad-} {-quad-} ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 249) {-,-}	(_Rs) {-,-}	(_Rs) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 248) {-,-}	(R 0) {-,-}	(R 0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDOi)	(R 248) {-,-}	(R 248) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDTi)	(R 248) {-,-}	(R 248) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTTUi)	(R 248) {-,-}	(R 249) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 248) {-,-}	(R 0) {-,-}	(R 0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDOi)	(R 248) {-,-}	(R 248) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 247) {-,-}	(R 1) {-,-}	(R 1) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDOi)	(R 247) {-,-}	(R 247) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDTi)	(R 247) {-,-}	(R 247) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTTUi)	(R 247) {-,-}	(R 248) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 248) {-,-}	(R 247) {-,-}	(R 247) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 248) {-,-}	(R 1) {-,-}	(R 1) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDOi)	(R 248) {-,-}	(R 248) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDTi)	(R 249) {-,-}	(R 249) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTTUi)	(R 249) {-,-}	(R 248) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(R 2) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 3)){-hide-}, ---
			MMixInstr (MMixINCML)	(R 2) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 3)){-hide-},
		MMixInstr (MMixINCMH)	(R 2) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 3)){-hide-}, ---
			MMixInstr (MMixINCH)	(R 2) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 3)){-hide-},
		MMixInstr (MMixGOi)	(_Rt) {-,-}	(R 2) {-,-}	(0) {-hide-}Nothing]) {-hide-},

{-{ Next, "SYM 2" provides an MMIX rendition of "sort" itself: }-}

  {-raw-}(SYM 2 {-,,-} {-:-}, {-,,-} {-raw-}[ ---
		MMixInstr (MMixOR)	(R 249) {-,-}	(R 2) {-,-}	(R 2) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDTi)	(R 249) {-,-}	(R 249) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 248) {-,-}	(R 1) {-,-}	(R 1) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDTi)	(R 248) {-,-}	(R 248) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(R 247) {-,-}	(0) {-,-}	(1) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(R 248) {-,-}	(R 248) {-,-}	(R 247) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(R 247) {-,-}	(0) {-,-}	(32) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSLU)	(R 248) {-,-}	(R 248) {-,-}	(R 247) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(R 247) {-,-}	(0) {-,-}	(32) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSR)	(R 248) {-,-}	(R 248) {-,-}	(R 247) {-hide-}Nothing{-hide-},
		MMixInstr (MMixCMP)	(R 249) {-,-}	(R 249) {-,-}	(R 248) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixZSPi)	(R 249) {-,-}	(R 249) {-,-}	(1) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(R 248) {-,-}	(0) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixCMP)	(R 249) {-,-}	(R 249) {-,-}	(R 248) {-hide-}Nothing{-hide-},
		MMixInstr (MMixZSNZi)	(R 249) {-,-}	(R 249) {-,-}	(1) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixBZ)	(R 249) {-,-}	(0) {-,-}	(38) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 249) {-,-}	(_Rs) {-,-}	(_Rs) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 248) {-,-}	(_Rs) {-,-}	(_Rs) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 247) {-,-}	(_Rs) {-,-}	(_Rs) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 246) {-,-}	(R 0) {-,-}	(R 0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 245) {-,-}	(R 1) {-,-}	(R 1) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDTi)	(R 245) {-,-}	(R 245) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(R 244) {-,-}	(0) {-,-}	(4) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixMULU)	(R 245) {-,-}	(R 245) {-,-}	(R 244) {-hide-}Nothing{-hide-},
		MMixInstr (MMixADDU)	(R 246) {-,-}	(R 246) {-,-}	(R 245) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDTi)	(R 246) {-,-}	(R 246) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTTUi)	(R 246) {-,-}	(R 249) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 246) {-,-}	(R 1) {-,-}	(R 1) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDTi)	(R 246) {-,-}	(R 246) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(R 245) {-,-}	(0) {-,-}	(1) {-hide-}Nothing{-hide-},
		MMixInstr (MMixADDU)	(R 246) {-,-}	(R 246) {-,-}	(R 245) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(R 245) {-,-}	(0) {-,-}	(32) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSLU)	(R 246) {-,-}	(R 246) {-,-}	(R 245) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(R 245) {-,-}	(0) {-,-}	(32) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSR)	(R 246) {-,-}	(R 246) {-,-}	(R 245) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTTUi)	(R 246) {-,-}	(R 248) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 246) {-,-}	(R 2) {-,-}	(R 2) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDTi)	(R 246) {-,-}	(R 246) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTTUi)	(R 246) {-,-}	(R 247) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 3) {-,-}	(R 249) {-,-}	(R 249) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 4) {-,-}	(R 248) {-,-}	(R 248) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 5) {-,-}	(R 247) {-,-}	(R 247) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 4)){-hide-}, ---
			MMixInstr (MMixINCML)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 4)){-hide-},
		MMixInstr (MMixINCMH)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 4)){-hide-}, ---
			MMixInstr (MMixINCH)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 4)){-hide-},
		MMixInstr (MMixGOi)	(_Rt) {-,-}	(R 6) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(R 3) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 7)){-hide-},
		MMixInstr (MMixINCML)	(R 3) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 7)){-hide-}, ---
			MMixInstr (MMixINCMH)	(R 3) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 7)){-hide-},
		MMixInstr (MMixINCH)	(R 3) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 7)){-hide-}, ---
			MMixInstr (MMixGOi)	(_Rt) {-,-}	(R 3) {-,-}	(0) {-hide-}Nothing]) {-hide-},

{-{ The function "SYM 3" implements the trivial exit block of "swap": }-}

    {-raw-}(SYM 3 {-,,-} {-:-}, {-,,-} {-raw-}[ ---
		MMixInstr (MMixGOi)	(_Rt) {-,-}	(_Rk) {-,-}	(0) {-hide-}Nothing]) {-hide-},

{-{ while "SYM 4" depicts the loop body at the heart of the entire algorithm: }-}

    {-raw-}(SYM 4 {-,,-} {-:-}, {-,,-} {-raw-}[ ---
		MMixInstr (MMixOR)	(R 249) {-,-}	(R 4) {-,-}	(R 4) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDTi)	(R 249) {-,-}	(R 249) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 248) {-,-}	(R 5) {-,-}	(R 5) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDTi)	(R 248) {-,-}	(R 248) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixCMP)	(R 249) {-,-}	(R 249) {-,-}	(R 248) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixZSNi)	(R 249) {-,-}	(R 249) {-,-}	(1) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(R 248) {-,-}	(0) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixCMP)	(R 249) {-,-}	(R 249) {-,-}	(R 248) {-hide-}Nothing{-hide-},
		MMixInstr (MMixZSNZi)	(R 249) {-,-}	(R 249) {-,-}	(1) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixBZ)	(R 249) {-,-}	(0) {-,-}	(118) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 249) {-,-}	(R 0) {-,-}	(R 0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 248) {-,-}	(R 4) {-,-}	(R 4) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDTi)	(R 248) {-,-}	(R 248) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(R 247) {-,-}	(0) {-,-}	(4) {-hide-}Nothing{-hide-},
		MMixInstr (MMixMULU)	(R 248) {-,-}	(R 248) {-,-}	(R 247) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(R 249) {-,-}	(R 249) {-,-}	(R 248) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDTi)	(R 249) {-,-}	(R 249) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 248) {-,-}	(R 3) {-,-}	(R 3) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDTi)	(R 248) {-,-}	(R 248) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixCMP)	(R 249) {-,-}	(R 249) {-,-}	(R 248) {-hide-}Nothing{-hide-},
		MMixInstr (MMixZSNPi)	(R 249) {-,-}	(R 249) {-,-}	(1) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(R 248) {-,-}	(0) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixCMP)	(R 249) {-,-}	(R 249) {-,-}	(R 248) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixZSNZi)	(R 249) {-,-}	(R 249) {-,-}	(1) {-hide-}Nothing{-hide-},
		MMixInstr (MMixBZ)	(R 249) {-,-}	(0) {-,-}	(16) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 249) {-,-}	(R 4) {-,-}	(R 4) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(R 248) {-,-}	(0) {-,-}	(1) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDTi)	(R 247) {-,-}	(R 249) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixADDU)	(R 248) {-,-}	(R 247) {-,-}	(R 248) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(R 247) {-,-}	(0) {-,-}	(32) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSLU)	(R 248) {-,-}	(R 248) {-,-}	(R 247) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(R 247) {-,-}	(0) {-,-}	(32) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSR)	(R 248) {-,-}	(R 248) {-,-}	(R 247) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTTUi)	(R 248) {-,-}	(R 249) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 249) {-,-}	(R 248) {-,-}	(R 248) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 5)){-hide-},
		MMixInstr (MMixINCML)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 5)){-hide-}, ---
			MMixInstr (MMixINCMH)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 5)){-hide-},
		MMixInstr (MMixINCH)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 5)){-hide-}, ---
			MMixInstr (MMixGOi)	(_Rt) {-,-}	(R 6) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 249) {-,-}	(_Rs) {-,-}	(_Rs) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 248) {-,-}	(_Rs) {-,-}	(_Rs) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(R 247) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 1)){-hide-}, ---
			MMixInstr (MMixINCML)	(R 247) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 1)){-hide-},
		MMixInstr (MMixINCMH)	(R 247) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 1)){-hide-}, ---
			MMixInstr (MMixINCH)	(R 247) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 1)){-hide-},
		MMixInstr (MMixOR)	(R 246) {-,-}	(R 0) {-,-}	(R 0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 245) {-,-}	(R 4) {-,-}	(R 4) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDTi)	(R 245) {-,-}	(R 245) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(R 244) {-,-}	(0) {-,-}	(4) {-hide-}Nothing{-hide-},
		MMixInstr (MMixMULU)	(R 245) {-,-}	(R 245) {-,-}	(R 244) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(R 246) {-,-}	(R 246) {-,-}	(R 245) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTOUi)	(R 246) {-,-}	(R 249) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 246) {-,-}	(R 0) {-,-}	(R 0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 245) {-,-}	(R 5) {-,-}	(R 5) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(R 244) {-,-}	(0) {-,-}	(1) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDTi)	(R 243) {-,-}	(R 245) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(R 244) {-,-}	(R 243) {-,-}	(R 244) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(R 243) {-,-}	(0) {-,-}	(32) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSLU)	(R 244) {-,-}	(R 244) {-,-}	(R 243) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(R 243) {-,-}	(0) {-,-}	(32) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSR)	(R 244) {-,-}	(R 244) {-,-}	(R 243) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTTUi)	(R 244) {-,-}	(R 245) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 245) {-,-}	(R 244) {-,-}	(R 244) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(R 244) {-,-}	(0) {-,-}	(4) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixMULU)	(R 245) {-,-}	(R 245) {-,-}	(R 244) {-hide-}Nothing{-hide-},
		MMixInstr (MMixADDU)	(R 246) {-,-}	(R 246) {-,-}	(R 245) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTOUi)	(R 246) {-,-}	(R 248) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTOUi)	(R 0) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTOUi)	(R 1) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTOUi)	(R 2) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTOUi)	(R 3) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTOUi)	(R 4) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTOUi)	(R 5) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTOUi)	(_Rk) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 0) {-,-}	(R 249) {-,-}	(R 249) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 1) {-,-}	(R 248) {-,-}	(R 248) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 2) {-,-}	(R 247) {-,-}	(R 247) {-hide-}Nothing{-hide-},
		MMixInstr (MMixGOi)	(_Rk) {-,-}	(R 2) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDOUi)	(_Rk) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDOUi)	(R 5) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDOUi)	(R 4) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDOUi)	(R 3) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDOUi)	(R 2) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDOUi)	(R 1) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDOUi)	(R 0) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 5)){-hide-}, ---
			MMixInstr (MMixINCML)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 5)){-hide-},
		MMixInstr (MMixINCMH)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 5)){-hide-}, ---
			MMixInstr (MMixINCH)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 5)){-hide-},
		MMixInstr (MMixGOi)	(_Rt) {-,-}	(R 6) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 6)){-hide-},
		MMixInstr (MMixINCML)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 6)){-hide-}, ---
			MMixInstr (MMixINCMH)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 6)){-hide-},
		MMixInstr (MMixINCH)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 6)){-hide-}, ---
			MMixInstr (MMixGOi)	(_Rt) {-,-}	(R 6) {-,-}	(0) {-hide-}Nothing]) {-hide-},

{-{ The next function "SYM 5" facilitiates an exit from the inner “\C{if}” statement: }-}

    {-raw-}(SYM 5 {-,,-} {-:-}, {-,,-} {-raw-}[ ---
		MMixInstr (MMixSETL)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 4)){-hide-}, ---
			MMixInstr (MMixINCML)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 4)){-hide-},
		MMixInstr (MMixINCMH)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 4)){-hide-}, ---
			MMixInstr (MMixINCH)	(R 6) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 4)){-hide-},
		MMixInstr (MMixGOi)	(_Rt) {-,-}	(R 6) {-,-}	(0) {-hide-}Nothing]) {-hide-},

{-{ and "SYM 6" provides a rather lengthy rendition of the remaining computations performed within the loop: }-}

    {-raw-}(SYM 6 {-,,-} {-:-}, {-,,-} {-raw-}[ ---
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 249) {-,-}	(_Rs) {-,-}	(_Rs) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 248) {-,-}	(_Rs) {-,-}	(_Rs) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(R 247) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 1)){-hide-}, ---
			MMixInstr (MMixINCML)	(R 247) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 1)){-hide-},
		MMixInstr (MMixINCMH)	(R 247) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 1)){-hide-}, ---
			MMixInstr (MMixINCH)	(R 247) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 1)){-hide-},
		MMixInstr (MMixOR)	(R 246) {-,-}	(R 0) {-,-}	(R 0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 245) {-,-}	(R 4) {-,-}	(R 4) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(R 244) {-,-}	(0) {-,-}	(1) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDTi)	(R 243) {-,-}	(R 245) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSUBU)	(R 244) {-,-}	(R 243) {-,-}	(R 244) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(R 243) {-,-}	(0) {-,-}	(32) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSLU)	(R 244) {-,-}	(R 244) {-,-}	(R 243) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(R 243) {-,-}	(0) {-,-}	(32) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSR)	(R 244) {-,-}	(R 244) {-,-}	(R 243) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTTUi)	(R 244) {-,-}	(R 245) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 245) {-,-}	(R 244) {-,-}	(R 244) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(R 244) {-,-}	(0) {-,-}	(4) {-hide-}Nothing{-hide-},
		MMixInstr (MMixMULU)	(R 245) {-,-}	(R 245) {-,-}	(R 244) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(R 246) {-,-}	(R 246) {-,-}	(R 245) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTOUi)	(R 246) {-,-}	(R 249) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 246) {-,-}	(R 0) {-,-}	(R 0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 245) {-,-}	(R 1) {-,-}	(R 1) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDTi)	(R 245) {-,-}	(R 245) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(R 244) {-,-}	(0) {-,-}	(4) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixMULU)	(R 245) {-,-}	(R 245) {-,-}	(R 244) {-hide-}Nothing{-hide-},
		MMixInstr (MMixADDU)	(R 246) {-,-}	(R 246) {-,-}	(R 245) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTOUi)	(R 246) {-,-}	(R 248) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTOUi)	(R 0) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTOUi)	(R 1) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTOUi)	(R 2) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTOUi)	(R 4) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTOUi)	(R 5) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTOUi)	(_Rk) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 0) {-,-}	(R 249) {-,-}	(R 249) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(1) {-,-}	(R 248) {-,-}	(R 248) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 2) {-,-}	(R 247) {-,-}	(R 247) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixGOi)	(_Rk) {-,-}	(R 2) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDOUi)	(_Rk) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDOUi)	(R 5) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDOUi)	(R 4) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDOUi)	(R 2) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDOUi)	(R 1) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDOUi)	(R 0) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 249) {-,-}	(_Rs) {-,-}	(_Rs) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 248) {-,-}	(_Rs) {-,-}	(_Rs) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 247) {-,-}	(_Rs) {-,-}	(_Rs) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(R 246) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 2)){-hide-},
		MMixInstr (MMixINCML)	(R 246) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 2)){-hide-}, ---
			MMixInstr (MMixINCMH)	(R 246) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 2)){-hide-},
		MMixInstr (MMixINCH)	(R 246) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 2)){-hide-}, ---
			MMixInstr (MMixOR)	(R 245) {-,-}	(R 0) {-,-}	(R 0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTOUi)	(R 245) {-,-}	(R 249) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 245) {-,-}	(R 1) {-,-}	(R 1) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDTi)	(R 245) {-,-}	(R 245) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTTUi)	(R 245) {-,-}	(R 248) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 245) {-,-}	(R 4) {-,-}	(R 4) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDTi)	(R 245) {-,-}	(R 245) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTTUi)	(R 245) {-,-}	(R 247) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTOUi)	(R 0) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTOUi)	(R 1) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTOUi)	(R 2) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTOUi)	(R 5) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTOUi)	(_Rk) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 0) {-,-}	(R 249) {-,-}	(R 249) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 1) {-,-}	(R 248) {-,-}	(R 248) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 2) {-,-}	(R 247) {-,-}	(R 247) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 3) {-,-}	(R 246) {-,-}	(R 246) {-hide-}Nothing{-hide-},
		MMixInstr (MMixGOi)	(_Rk) {-,-}	(R 3) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDOUi)	(_Rk) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDOUi)	(R 5) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDOUi)	(R 2) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDOUi)	(R 1) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDOUi)	(R 0) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 249) {-,-}	(_Rs) {-,-}	(_Rs) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 248) {-,-}	(_Rs) {-,-}	(_Rs) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 247) {-,-}	(_Rs) {-,-}	(_Rs) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(R 246) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 2)){-hide-},
		MMixInstr (MMixINCML)	(R 246) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 2)){-hide-}, ---
			MMixInstr (MMixINCMH)	(R 246) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 2)){-hide-},
		MMixInstr (MMixINCH)	(R 246) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 2)){-hide-}, ---
			MMixInstr (MMixOR)	(R 245) {-,-}	(R 0) {-,-}	(R 0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTOUi)	(R 245) {-,-}	(R 249) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 245) {-,-}	(R 5) {-,-}	(R 5) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDTi)	(R 245) {-,-}	(R 245) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTTUi)	(R 245) {-,-}	(R 248) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 245) {-,-}	(R 1) {-,-}	(R 1) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDTi)	(R 245) {-,-}	(R 245) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTTUi)	(R 245) {-,-}	(R 247) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTOUi)	(R 0) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTOUi)	(R 1) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSTOUi)	(R 2) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSUBU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSTOUi)	(_Rk) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 0) {-,-}	(R 249) {-,-}	(R 249) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 1) {-,-}	(R 248) {-,-}	(R 248) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixOR)	(R 2) {-,-}	(R 247) {-,-}	(R 247) {-hide-}Nothing{-hide-},
		MMixInstr (MMixOR)	(R 3) {-,-}	(R 246) {-,-}	(R 246) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixGOi)	(_Rk) {-,-}	(R 3) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDOUi)	(_Rk) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDOUi)	(R 2) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixLDOUi)	(R 1) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-},
		MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixLDOUi)	(R 0) {-,-}	(_Rs) {-,-}	(0) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(_Rt) {-,-}	(0) {-,-}	(8) {-hide-}Nothing{-hide-}, ---
			MMixInstr (MMixADDU)	(_Rs) {-,-}	(_Rs) {-,-}	(_Rt) {-hide-}Nothing{-hide-},
		MMixInstr (MMixSETL)	(R 3) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 7)){-hide-}, ---
			MMixInstr (MMixINCML)	(R 3) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 7)){-hide-},
		MMixInstr (MMixINCMH)	(R 3) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 7)){-hide-}, ---
			MMixInstr (MMixINCH)	(R 3) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 7)){-hide-},
		MMixInstr (MMixGOi)	(_Rt) {-,-}	(R 3) {-,-}	(0) {-hide-}Nothing]) {-hide-},

{-{

\break

Finally, symbols "SYM 7" and "SYM 8" describe the exit blocks of the outer “\C{if}” statement and the "sort" function itself:

}-}

    {-raw-}(SYM 7 {-,,-} {-:-}, {-,,-} {-raw-}[ ---
		MMixInstr (MMixSETL)	(R 3) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 8)){-hide-}, ---
			MMixInstr (MMixINCML)	(R 3) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 8)){-hide-},
		MMixInstr (MMixINCMH)	(R 3) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 8)){-hide-}, ---
			MMixInstr (MMixINCH)	(R 3) {-,-}	(0) {-,-}	(0) {-@-}	(Just (SYM 8)){-hide-},
		MMixInstr (MMixGOi)	(_Rt) {-,-}	(R 3) {-,-}	(0) {-hide-}Nothing]) {-hide-},

    {-raw-}(SYM 8 {-,,-} {-:-}, {-,,-} {-raw-}[ ---
		MMixInstr (MMixGOi)	(_Rt) {-,-}	(_Rk) {-,-}	(0) {-hide-}Nothing])
\end{haskell}

The reader will undoubtedly observe the naïveté of the above translation, whereby constants such as "8" are repeatedly placed into scratch registers during allocation of objects on the program's data stack. Given a smarter register allocation algorithm and a more elaborate constant synthesis strategy, the above program could be easily reduced into 20-odd individual MMIX operations.  Nevertheless, I chose to present the above unoptimised version of the program in order to expose the actual behaviour of our compiler in its present form that was described throughout the last three chapters.

\end{section}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%