\set{capture}{hidden}
\begin{code}

module MMIX.Lambda (
	compileModule
) where

import Additional
import MMIX.Syntax
import MMIX.Semantics
import Lambda

infix 4 `isSubformatOf`

\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{MMIX/Lambda}
\set{title}		{Code Generation}
\begin{section}

As described in Chapter \ref{Design}, the ultimate validity of our entire compiler rests on a single property of «linear correctness». In Section \ref{Design/Linear}, this property was formulated as the judgement \hbox{"rho.psi_hat.psi == rho",} in which "rho" and "psi" represent the translation semantics of C and MMIX. In Sections \ref{C/Definitions} and \ref{MMIX/Semantics}, these two functions were implemented by the respective Haskell definitions "meaningOfTranslationUnit" and "meaningOfMMixModule". We are now ready to define the final piece of the entire puzzle "psi_hat", in a form of the reverse mapping of Etude modules into their MMIX counterparts. Formally, "psi_hat" is depicted by the following algorithm "compileModule":

\BeginHaskellToken	{compileModule}				{Open}	{\ZapfChancery{C}}
  \HaskellContext							{(Ord upsilon)}
  \HaskellArgument							{ModuleT upsilon}
  \HaskellResult							{MMixModuleT}
\EndHaskellToken

\IndexTerm		{compileModule}					{C module (MMIX_module)}

\begin{code}
compileModule {-[*]-} :: (Ord upsilon) => ModuleT upsilon -> MMixModuleT

compileModule {-sem-}(MODULE (tau) {-EXPORT-} (xi_set) {-WHERE-} (iota_set) {-,,-})
{--}			= 		{-sem-}(MMixLET (_S) {-TEXT-} (_T) {-DATA-} (_D) {-IN-} (SYM 0))
 where	_S		=		listToMap [ {-raw-}(_Sigma#(upsilon_k) {-,,-} {-:-}, {-,,-} x_k) --
 							| {-sem-}(x_k {-=-}, upsilon_k) <- mapToList xi_set ] --
 			`mapUnion` listToMap [ {-raw-}(_Sigma#(upsilon_k) {-,,-} {-:-}, {-,,-} x_k) --
 							| {-sem-}(upsilon_k {-=-}, IMPI (x_k)) <- mapToList iota_set ]
 	_T		=		listToMap [ {-raw-}(_Sigma#(upsilon_k) {-,,-} {-:-}, {-,,-} compileFunction{-sem-}(_Sigma' {->-}, FUN (upsilon_set_k) {-DOT-} (tau_k))) --
 							| {-sem-}(upsilon_k {-=-}, FUNI (FUN (upsilon_set_k) {-DOT-} (tau_k))) <- mapToList iota_set ] --
 			`mapUnion` listToMap [ {-raw-}(SYM 0 {-,,-} {-:-}, {-,,-} compileFunction{-sem-}(_Sigma' {->-}, FUN [] {-DOT-} (tau))) ]
 	_D		=		listToMap [ {-raw-}(_Sigma#(upsilon_k) {-,,-} {-:-}, {-,,-} {-sem-}(OBJI ({-sem-}(delta_k `renameData` _Sigma')) {-OF-} (xi_set_k))) --
 							| {-sem-}(upsilon_k {-=-}, OBJI (delta_k) {-OF-} (xi_set_k)) <- mapToList iota_set ]
 	_Sigma		= 		listToMap (setToList (dom{--}(iota_set)) `zipList` [ SYM 1 .. ])
 	_Sigma'		= 		listToMap [ {-raw-}(upsilon_k {-,,-} {-:-}, {-,,-} {-sem-}(sigma_k `SR` 0)) --
 							| {-raw-}(upsilon_k {-,,-} {-:-}, {-,,-} sigma_k) <- mapToList _Sigma ]
\end{code}

In other words, an Etude module of the form “"MODULE (tau) {-EXPORT-} (xi_set) {-WHERE-} (iota_set)"\,\,” is compiled into the symbolic MMIX construct “"MMixLET (_S) {-TEXT-} (_T) {-DATA-} (_D) {-IN-} (SYM 0)"”. The supplied set of item definitions "iota_set" is split into the text segment "_T", data segment "_D" and symbol table "_S" that are formed, respectively, from all function, data and imported items found in "iota_set". Further, the symbol table is also extended with bindings of any symbols that are made available to the remainder of the entire program through the source module's export list "xi_set". The precise translation of Etude functions, depicted in the above definition by the algorithm "compileFunction", is discussed later in Section \ref{MMIX/Lambda/Functions}.

\set{title}		{Linear correctness}
\begin{theorem}

Collectively, the three algorithms "meaningOfTranslationUnit", "meaningOfMMixModule" and "compileModule" satisfy the linear correctness property \hbox{"rho.psi_hat.psi == rho".} In particular, for every set of translation units "tu_set" that forms an autonomous C program, their collective image produced by the system's linking algorithm "linkModules" is equivalent to the term obtained by the same algorithm from the collection of meanings assigned to their respective MMIX renditions. Formally:

\NewHaskellToken	{lcMMixJ}				{Ord}	{\TKeyword{lc}}
\NewHaskellDescription	{lcMMixJ}					{Linear correctness of the C compiler}
\IndexTheorem		{lcMMixJ}					{LC Linear correctness of the C compiler}

\begin{haskell}
lcMMixJ :: forall tu_set => --
	linkModules{--}[ meaningOfTranslationUnit{--}(tu_k) | tu_k <- tu_set ] --
			`EqvJ` linkModules{--}[ meaningOfMMixModule{--}(compileModule{--}(meaningOfTranslationUnit{--}(tu_k))) | tu_k <- tu_set ]
\end{haskell}
\begin{proof}

Recall from Section \ref{Lambda/Modules} that "linkModules" produces a single Etude term "tau", together with a context "(_Lambda, _Delta)" that, collectively, describes the initial address space of the entire program. Accordingly, the above theorem follows directly from a similar result derived for individual terms in Section \ref{MMIX/Lambda/Functions} below. \QED

\end{proof}
\end{theorem}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{MMIX/Lambda/Registers}
\set{title}		{Variables and Registers}
\begin{subsection}

In the above definition of the module compilation algorithm "compileModule", the reader will doubtless notice the two finite maps "_Sigma" and "_Sigma'". In a nutshell, these maps are used to rename any input variables found in the source module into their appropriate MMIX counterparts that, in the resulting program, are depicted by the respective symbols "SYM 0", "SYM 1", "SYM 2" and so on. The designated symbol "SYM 0" is reserved for the module's initialiser term, while all other MMIX variables are bound systematically to any global Etude item definitions found in the supplied list "iota_set". For data items, the renaming process is represented by the standard Etude substitution function "delta_k `renameData` _Sigma" that was defined earlier in Section \ref{Lambda/Modules}. For all other Etude entities, however, variable renaming is integrated directly into the compilation process, whereby it facilitates implementation of an important translation stage known as «register allocation».

Intuitively, register allocation converts all Etude variables found in the source program into appropriate general purpose MMIX registers. Although our target architecture provides an abundance of register resources, in principle, it is always possible to define Etude programs with more variables than those supported directly by the hardware. To this end, in the following translation, only the first 250 Etude variables are assigned to concrete hardware registers "R 0 .. R 249", with all remaining entities bound to «pseudo-registers» "R 256", "R 257" and so on. As will be revealed shortly, in the translated program the values of all such pseudo-registers are stored in memory-resident objects that are set aside for that specific purpose by our compiler.

For conciseness, the sequence of all MMIX registers available for depiction of Etude variables is represented by the notation "nu_set_G". Further, the list of all variables assigned by our compiler to the individual parameters of every Etude function is depicted by the symbol "nu_set_P". Formally, both "nu_set_G" and "nu_set_P" are defined in Haskell as the following infinite list structures:

\NewHaskellToken	{nu\_set\_P}				{Ord}	{«\macron \nu\sub{\SmallCaps{p}}»}
\NewHaskellToken	{nu\_set\_G}				{Ord}	{«\macron \nu\sub{\SmallCaps{g}}»}

\NewHaskellToken	{nu'set'P}				{Ord}	{«\macron \nu\sub{\SmallCaps{p}}»}
\NewHaskellToken	{nu'set'G}				{Ord}	{«\macron \nu\sub{\SmallCaps{g}}»}

\NewHaskellResult	{nu'set'P}					{[MMixVarT]}
\NewHaskellResult	{nu'set'G}					{[MMixVarT]}

\IndexTerm		{nu'set'P}					{\nu-P (MMIX_vars)}
\IndexTerm		{nu'set'G}					{\nu-G (MMIX_vars)}

\begin{code}
nu_set_G, nu_set_P :: [MMixVarT] 

nu_set_G 	= [  R 249,	R 248 	.. R 0 		] ++ [ R 256 .. ]
nu_set_P 	= [  R 0, 	R 1 	.. R 249 	] ++ [ R 256 .. ]
\end{code}

A careful reader will observe that the order in which true MMIX registers appear in these lists is inverted within "nu_set_G". In Section \ref{MMIX/Lambda/Functions}, this simple technique serves to reduce the possibility of a single variable being associated with both a function argument and a temporary entity within that function's caller. Under the naïve register allocation algorithm defined in this section, such conflicts would result in a significant performance degradation of the generated MMIX programs, but, given the multitude of registers provided by our target architecture, they are unlikely to occur in practice.

The remaining six general purpose hardware register "R 250 .. R 255" are reserved by our code generator for various specific purposes described throughout the remainder of this section. For clarity, in the following presentation they are always depicted by the following symbolic names:

\NewHaskellToken	{\_Rs}					{Ord}	{\MKeyword{\$S}}
\NewHaskellToken	{\_Rk}					{Ord}	{\MKeyword{\$K}}
\NewHaskellToken	{\_Rt}					{Ord}	{\MKeyword{\$T}}
\NewHaskellToken	{\_Ru}					{Ord}	{\MKeyword{\$U}}
\NewHaskellToken	{\_Rv}					{Ord}	{\MKeyword{\$V}}
\NewHaskellToken	{\_Rw}					{Ord}	{\MKeyword{\$W}}

\NewHaskellToken	{xxRs}					{Ord}	{\MKeyword{\$S}}
\NewHaskellToken	{xxRk}					{Ord}	{\MKeyword{\$K}}
\NewHaskellToken	{xxRt}					{Ord}	{\MKeyword{\$T}}
\NewHaskellToken	{xxRu}					{Ord}	{\MKeyword{\$U}}
\NewHaskellToken	{xxRv}					{Ord}	{\MKeyword{\$V}}
\NewHaskellToken	{xxRw}					{Ord}	{\MKeyword{\$W}}

\NewHaskellResult	{xxRs}						{MMixVarT}
\NewHaskellResult	{xxRk}						{MMixVarT}
\NewHaskellResult	{xxRt}						{MMixVarT}
\NewHaskellResult	{xxRu}						{MMixVarT}
\NewHaskellResult	{xxRv}						{MMixVarT}
\NewHaskellResult	{xxRw}						{MMixVarT}

\IndexTerm		{xxRs}						{$S (MMIX_var)}
\IndexTerm		{xxRk}						{$K (MMIX_var)}
\IndexTerm		{xxRt}						{$T (MMIX_var)}
\IndexTerm		{xxRu}						{$U (MMIX_var)}
\IndexTerm		{xxRv}						{$V (MMIX_var)}
\IndexTerm		{xxRw}						{$W (MMIX_var)}

\begin{code}
_Rs, _Rk, _Rt, _Ru, _Rv, _Rw :: MMixVarT

_Rs	= R 255								-- stack pointer
_Rk	= R 254								-- continuation pointer
_Rt	= R 253								-- temporary variable T
_Ru	= R 252								-- temporary variable U	
_Rv	= R 251								-- temporary variable V
_Rw	= R 250								-- temporary variable W
\end{code}

Intuitively, the two register "_Rs" and "_Rk" (or "R 255" and "R 254") are dedicated for storage of the current stack pointer and the continuation address of the Etude function under execution. The following translation always ensures that "_Rs" invariably echoes the current value of the object environment component "sigma_c", in order to relieve the processor from maintaining that component explicitly within its hardware state. Further "_Rk" will always contain the address of the current function's caller, or, more specifically, the precise location of the instruction that the program intends to execute upon return from the presently evaluated function. The remaining four registers "_Rt", "_Ru", "_Rv" and "_Rw" provide our compiler with a temporary scratch space that is required by our register allocator for a transparent implementation of any pseudo-register variables.

In particular, the process of allocating a given MMIX variable "nu" to an appropriate general purpose hardware register can be depicted in Haskell as follows:

\BeginHaskellToken	{allocReg}				{Open}	{\ZapfChancery{A}\sub{«\nu»}}
  \HaskellArgument							{(MMixVarT {->-}, MMixVarT)}
  \HaskellResult							{MMixVarT}
\EndHaskellToken

\IndexTerm		{allocReg}					{A MMIX_var MMIX_var (MMIX_var)}

\begin{code}
allocReg {-[*]-} :: (MMixVarT {->-}, MMixVarT) -> MMixVarT

allocReg {-sem-}(R _N {->-}, nu)	| inRange (R 0) (<=) (nu) (<=) (R 255)	= nu
					| otherwise				= R _N
\end{code}

If the specified Etude variable already corresponds to one of the hardware registers "R 0 .. R 255", then "allocReg{--}(R _N {->-}, nu)" always allocates "nu" directly to that register. Otherwise, the construction returns the «backup register» "R _N", which, in all cases, must be supplied by the surrounding algorithm with a value disjoint from any other such registers already allocated for one of the Etude variables that remain free within the remainder of the program under translation.

For convenience, we also provide an implicit coercion function, which maps all hardware-provided registers to their concrete numeric encodings, suitable for use within the operand fields of an appropriate MMIX instruction form. Specifically:

\BeginHaskellToken	{fromReg}				{Void}	{}
  \HaskellArgument							{MMixVarT}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{fromReg}					{ MIX_var (integer)}

\begin{code}
fromReg {-[*]-} :: MMixVarT -> Integer

fromReg {-sem-}(R _N)	= _N
fromReg {-sem-}(RR)	= 6
\end{code}

Finally, the notation “"loadReg{--}(R _D {->-}, nu)"” represents a «variable load operation», which, in the translated program, can be utilised to place the value of an arbitrary MMIX variable into the specified general purpose hardware register "R _D". In all cases, "loadReg{--}(R _D {->-}, nu)" returns a sequence of zero or more MMIX instructions that achieve the desired effect as part of their execution. In particular, if "nu" and "R _D" represent the same hardware register, then the resulting sequence will be empty, since no actual movement of data is required. Otherwise, if "nu" represents a hardware register, then the construction delivers a single «register copy operation», depicted by an MMIX instruction of the form \hbox{“"MMixInstr (MMixOR) (fromReg (R _D)) {-,-} ({-!-}fromReg (R _S)) {-,-} ({-!-}fromReg (R _S))"\,”} or \hbox{“"MMixInstr (MMixGET) (fromReg (R _D)) {-,-} ({-!-}0) {-,-} ({-!-}fromReg RR)"”} for all general purpose registers and the remainder register, respectively. If, on the other hand, the source variable assumes a symbolic form “"sigma `SR` delta"”, then the symbol's numeric value is placed in the target register using a sequence of four MMIX instructions “"MMixSETL"”, “"MMixINCML"”, “"MMixINCMH"” and “"MMixINCH"\,”, which were dedicated to the specific task of constant synthesis earlier in Section \ref{MMIX/Semantics}. Finally, if "nu" represents one of the pseudo-registers "R _S" in which "_S >= 256", then the variable's value is stored in a memory-resident object found at the predetermined offset "8 * _S" into the dedicated MMIX data segment “"LOC"”. In the resulting instruction sequence, the content of that object is placed into "R _D" by synthesising the target address “"LOC`SR`8{--}_S"\,” into the dedicated temporary register "_Rv" and fetching the required word of data using an appropriate MMIX instruction “"MMixLDOUi"\!”. Formally:

\BeginHaskellToken	{loadReg}				{Open}	{load}
  \HaskellArgument							{(MMixVarT {->-}, MMixVarT)}
  \HaskellResult							{[MMixInstrT]}
\EndHaskellToken

\IndexTerm		{loadReg}					{load MMIX_var MMIX_var (MMIX_instrs)}

\begin{code}
loadReg {-[*]-} :: (MMixVarT {->-}, MMixVarT) -> [MMixInstrT]

loadReg {-sem-}(R _D {->-}, R _S) ---
  | R _S == R _D	= {-empty-}[]
  | R _S <= R 255	= {-sem-}[ 	MMixInstr (MMixOR) (fromReg (R _D)) {-,-} (fromReg (R _S)) {-,-} (fromReg (R _S)) {-hide-}Nothing {-,-} {-,-} ]
  | R _S >= R 256	= loadReg{-sem-}(_Rv {->-}, LOC `SR` 8 #*# _S {-,-} ) ++ {-sem-}[MMixInstr (MMixLDOUi) (fromReg (R _D)) {-,-} (fromReg _Rv) {-,-} (0) {-hide-}Nothing ]
loadReg {-sem-}(R _D {->-}, RR) ---
			= {-sem-}[	MMixInstr (MMixGET)	(fromReg (R _D)) {-,-} 	(0) {-,-} (fromReg RR) {-hide-}Nothing ]
loadReg {-sem-}(R _D {->-}, sigma `SR` delta) ---
			= {-sem-}[	MMixInstr (MMixSETL)	(fromReg (R _D)) {-,-} 	({-sem-}(delta `bits` [8..15])) {-,-}	({-sem-}(delta `bits` [0..7])) ---
						{-@-} (Just sigma) {-hide-},
					MMixInstr (MMixINCML)	(fromReg (R _D)) {-,-} 	({-sem-}(delta `bits` [24..31])) {-,-}	({-sem-}(delta `bits` [16..23])) ---
						{-@-} (Just sigma) {-hide-},
					MMixInstr (MMixINCMH)	(fromReg (R _D)) {-,-} 	({-sem-}(delta `bits` [40..47])) {-,-}	({-sem-}(delta `bits` [32..39])) ---
						{-@-} (Just sigma) {-hide-},
					MMixInstr (MMixINCH)	(fromReg (R _D)) {-,-} 	({-sem-}(delta `bits` [56..63])) {-,-}	({-sem-}(delta `bits` [48..55])) ---
						{-@-} (Just sigma) ]
\end{code}

Further, the dual operation “"storeReg{--}(nu {->-}, R _S)"” binds the specified MMIX variable "nu" to the current value of a given general purpose hardware register "R _S". Specifically, if "nu == R _S", or if it represents a general purpose hardware register, then the construction behaves identically to the earlier “"loadReg"” operation with the same arguments. On the other hand, if "nu" refers to some pseudo-register "R _D" for "_D >= 256", then the corresponding memory-resident object “"LOC`SR`8{--}_D"\,” is updated with the value of "R _S" using an appropriate “"MMixSTOUi"\!” instruction form. Finally, an update of the "RR" register can be performed using the MMIX instruction “\,"MMixPUT"\,” as follows:

\BeginHaskellToken	{storeReg}				{Open}	{store}
  \HaskellArgument							{(MMixVarT {->-}, MMixVarT)}
  \HaskellResult							{[MMixInstrT]}
\EndHaskellToken

\IndexTerm		{storeReg}					{store MMIX_var MMIX_var (MMIX_instrs)}

\begin{code}
storeReg {-[*]-} :: (MMixVarT {->-}, MMixVarT) -> [MMixInstrT]
\end{code}

\begin{code}
storeReg {-sem-}(R _D {->-}, R _S) ---
  | R _D == R _S	= {-empty-}[]
  | R _D <= R 255	= {-sem-}[MMixInstr (MMixOR) (fromReg (R _D)) {-,-} (fromReg (R _S)) {-,-} (fromReg (R _S)) {-hide-}Nothing {-,-} ]
  | R _D >= R 256	= loadReg{-sem-}(_Rv {->-}, LOC `SR` 8 #*# _D) ++ {-sem-}[MMixInstr (MMixSTOUi) (fromReg (R _S)) {-,-} (fromReg _Rv) {-,-} (0) {-hide-}Nothing ]
storeReg {-sem-}(RR {->-}, R _S) ---
			= {-sem-}[MMixInstr (MMixPUT) (fromReg RR) {-,-} (0) {-,-} (fromReg (R _S)) {-hide-}Nothing {-,-} ]
\end{code}

The reader should observe that “"storeReg"” is left undefined whenever its target operand represents a symbolic MMIX variable, since symbol bindings cannot be redefined in a well-formed Etude program.

Finally, two additional constructs facilitate direct movement of information between pairs of MMIX variables or between two sets of such variables. Formally, "compileCopy{--}(nu_D {->-}, nu_S)" begins by loading the value of "nu_S" into an appropriate general purpose hardware register "R _R", set to the scratch register "_Rw" if "nu_S" does not already have the appropriate form. The resulting value is then placed in the targeted variable "nu_D" using the earlier “"storeReg"” combinator. In Haskell:

\BeginHaskellToken	{compileCopy}				{Open}	{\ZapfChancery{C}\!\sub{«\nu»}}
  \HaskellArgument							{(MMixVarT {->-}, MMixVarT)}
  \HaskellResult							{[MMixInstrT]}
\EndHaskellToken

\IndexTerm		{compileCopy}					{C_\nu MMIX_var MMIX_var (MMIX_instrs)}

\begin{code}
compileCopy {-[*]-} :: (MMixVarT {->-}, MMixVarT) -> [MMixInstrT]
\end{code}

\begin{code}
compileCopy {-sem-}(nu_D {->-}, nu_S) ---
			= loadReg{--}(R _R {->-}, nu_S) ++ storeReg{--}(nu_D {->-}, R _R)
 where	{-sem-}(R _R)	= allocReg{--}(allocReg{--}(_Rw {->-}, nu_D) {->-}, nu_S)
\end{code}

Further, "compileCopies{--}(nu_set_D {->-}, nu_set_S)" makes it possible to shuffle data between two potentially overlapping sets of MMIX variables. Formally, this construction is implemented as follows:

\BeginHaskellToken	{compileCopies}				{Open}	{\ZapfChancery{C}\!\sub{«\macron\nu»}}
  \HaskellArgument							{([MMixVarT] {->-}, [MMixVarT])}
  \HaskellResult							{[MMixInstrT]}
\EndHaskellToken

\IndexTerm		{compileCopies}					{C_\nu- MMIX_vars MMIX_vars (MMIX_instrs)}

\begin{code}
compileCopies {-[*]-} :: ([MMixVarT] {->-}, [MMixVarT]) -> [MMixInstrT]

compileCopies {-sem-}(nu_set_D {->-}, nu_set_S)
  | nu_set_D == {-empty-}[] || nu_set_S == {-empty-}[] ---
  				= {-sem-}[]
  | otherwise			= compileCopy{--}(nu_D' {->-}, nu_D) ++ --
  				  compileCopy{--}(nu_D {->-}, nu_S) ++ --
  				  compileCopies{--}(tail{--}(nu_set_D) {->-}, tail{--}(nu_set_S'))
  
 where	nu_S		= head{--}(nu_set_S)
 	nu_D		= head{--}(nu_set_D)
 	nu_D'		= head{--}[ nu_k | nu_k <- {-raw-}[nu_D] ++ nu_set_G, nu_k /@ listToSet (tail{--}(nu_set_S)) ]
 	nu_set_S'	= [ _Sigma#(nu_k) | nu_k <- nu_set_S ]
	_Sigma		= listToMap ((nu_set_S `zipList` nu_set_S)) // listToMap [ {-raw-}(nu_D {-:-}, nu_D') ]
\end{code}

In summary, "compileCopies{--}(nu_set_D {->-}, nu_set_S)" simply binds every variable "nu_D @@ nu_set_D" to the corresponding value "nu_S @@ nu_set_S". However, in order to preserve the semantics of any still unprocessed variables in "nu_set_S", the present value of "nu_D" is first saved in some temporary scratch space "nu_D'", taken to represent the first unallocated variable from the list "[nu_D] ++ nu_set_G" that is not mentioned in the remainder of the source list "nu_set_S". Once "nu_S" has been assigned its new value, "compileCopies" proceeds with any remaining entries in "nu_set_S", after renaming any occurrences of "nu_D" in that list to its new binding "nu_D'".

In order to simplify presentation of the following linear correctness theorems, let us introduce one additional auxiliary definition, which presents the meaning of an entire instruction section that was produced by our compiler as an appropriate function environment structure "_Lambda". Formally:

\BeginHaskellToken	{meaningsOfMMixInstr}			{Open}	{\ZapfChancery{D}\sub{\MKeyword{\macron i}}}
  \HaskellArgument							{(MapT MMixVarT Integer, MMixVarT {->-}, MMixSectionT)}
  \HaskellResult							{FEnvT MMixVarT}
\EndHaskellToken

\IndexTerm		{meaningsOfMMixInstr}				{D_I- MMIX_instrs (f_env)}

\begin{code}
meaningsOfMMixInstr {-[*]-} :: (MapT MMixVarT Integer, MMixVarT {->-}, MMixSectionT) -> FEnvT MMixVarT

meaningsOfMMixInstr {-sem-}(_Gamma, nu {->-}, iota_set {-,,-}) ---
	= listToMap [ (_Gamma#(nu) + 4#*#k {-,,-}{-:-},{-,,-} meaningOfMMixInstr{--}(nu `shiftMMixVar` 4#*#k {->-}, iota_k)) | (iota_k, k) <- iota_set `zipList` [ 0 .. ] ]
\end{code}

\set{title}		{Preservation of atom semantics under compilation}
\begin{theorem}

Let "nu_S" and "nu_D" represent a pair of MMIX variables involved in a given copy operation and let "_Gamma" represent the mapping of all relevant MMIX variables to their respective atomic values. Further, let "_Lambda'" represent a function environment updated with bindings of "n" successive instruction addresses, beginning at some Etude function variable "nu", to the respective "n" instructions generated from the compilation of “"compileCopy{--}(nu_D {->-}, nu_S)"”. If "nu_S" represents a pseudo-register "R _N", then let "_Delta'" represent an object environment that is likewise updated with a binding of the corresponding address "LOC`SR`8{--}_N" to "_Gamma#(R _N)". Then, every application of "nu" to a the standard parameter list "pregs" under a substitution by "_Gamma" is always equivalent to an application of the function "nu `shiftMMixVar` 4{--}n" to the same parameter list under a substitution by an updated finite map "_Gamma // listToMap [ _Gamma#(nu_D) {-,,-}{-:-},{-,,-} nu_S ]". Formally:

\NewHaskellToken	{copyPreservationJ}			{Ord}	{\TKeyword{LC\sub{«\nu»}}}
\NewHaskellDescription	{copyPreservationJ}				{Preservation of variable semantics under compilation}
\IndexTheorem		{copyPreservationJ}				{LC_\alpha Preservation of atom semantics under compilation}

\begin{haskell}
copyPreservationJ :: forall _Gamma, _Gamma', _Lambda, _Lambda', _Delta, _Delta', nu, nu_S, nu_D =>
	WfJ{--}(_Lambda) -> WfJ{--}(_Delta) ->
	{-sem-}(_Gamma'	  = _Gamma 	// 		listToMap [ _Gamma#(nu_D) {-,,-}{-:-},{-,,-} nu_S ]) ->
	{-sem-}(_Lambda'  = _Lambda 	// 		meaningsOfMMixInstr{--}(_Gamma, nu, compileCopy{--}(nu_D {->-}, nu_S))) ->
	{-sem-}(_Delta'   = _Delta 	`setInOEnv` 	listToMap [ _Gamma#(LOC`SR`8{--}k) {-,,-}{-:-},{-,,-} _Gamma#(R k)) | {-sem-}(R k) <- [ nu_S ], k >= 256 ]) ->
	{-sem-}(_Lambda', _Delta' {->-}, {-sem-}(APP ({-sem-}_Gamma#(nu)) (pregs)) `substAtom` _Gamma) `EqvJ` --
	{-sem-}(_Lambda, _Delta {->-}, {-sem-}(APP ({-sem-}(_Gamma#(nu `shiftMMixVar` lengthOfList{--}(compileCopy{--}(nu_D {->-}, nu_S))))) (pregs)) `substTerm` _Gamma')
\end{haskell}
\begin{proof}

If "nu_S == nu_D", then "compileCopy" produces an empty instruction sequence. Accordingly, \hbox{"nu `shiftMMixVar` lengthOfList{--}(compileCopy{--}(nu_D {->-}, nu_S)) = nu `shiftMMixVar` 0 = nu"} and \hbox{"_Gamma' = _Gamma // listToMap [ nu_D {-,,-}{-:-},{-,,-} nu_S ] = _Gamma // listToMap [ nu_D {-,,-}{-:-},{-,,-} nu_D ] = _Gamma",} so that the above theorem holds trivially in such cases.

Otherwise, if both "nu_S" and "nu_D" represent general purpose hardware registers, then the single “"MMixInstr (MMixOR) (nu_R) {-,-} (alpha) {-,-} (alpha)"” instruction produced by the compiler is, by the definition of "meaningOfMMixInstr", equivalent to “"FUN (pregs) {-DOT-} $ LET [{-raw-}[nu_R] =: {-hide-}RET {-raw-}[OP2 (alpha) (IOR) (NG`FMT`64) (alpha)]] $ APP (VAR ({-sem-}(_Gamma#(nu) `shiftMMixVar` 4))) (regs))"”, which can be easily simplified into “"FUN (pregs) {-DOT-} $ LET [{-raw-}[nu_R] =: alpha] $ APP (VAR ({-sem-}(_Gamma#(nu) `shiftMMixVar` 4))) (regs))"”, given that, under the earlier operational interpretation of the “"IOR{-!-}{-!-}{-sub-}phi"\!” operator in Section \ref{Lambda/Arch/Atoms}, we can easily establish the identity "alpha IOR{-!-}{-!-}{-sub-}phi alpha `EqvJ` alpha" for all Etude atoms "alpha". Accordingly, an application of "_Gamma#(nu)" to "pregs" is reducible by "evalTerm" into “"APP (VAR ({-sem-}(sigma `shiftMMixVar` 4))) (regs)"” as required.

Finally, if "alpha" depicts one of the MMIX pseudo-registers, then the resulting sequence of constant synthesis instructions can be likewise demonstrated to be equivalent to the register's corresponding address "LOC`SR`n", after substituting each function's body into its caller in accordance with the operational model of Etude terms that was described in Section \ref{Lambda/Arch/Terms} and applying a simple arithmetic reasoning to the resulting sum of constant operands in order to arrive at the required conclusion. Accordingly, the "copyPreservationJ"\!'s assumption about the bindings of such address locations in the extended environment "_Delta'" provides us with all the information required to complete the proof. \QED

\end{proof}
\end{theorem}
\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{MMIX/Lambda/Atoms}
\set{title}		{Atoms}
\begin{subsection}

\BeginHaskellToken	{compileAtomU}				{Open}	{\ZapfChancery{C}\sub{\SmallCaps{u}}}
  \HaskellContext							{(Ord upsilon)}
  \WideHaskellArgument							{(MapT (upsilon) (MMixVarT), SetT MMixVarT, MMixVarT, MMixOpcodeT, Integer {->-}, AtomT upsilon)}
  \HaskellResult							{[MMixInstrT]}
\EndHaskellToken

\set{capture}		{compileAtomU}
\begin{code}
compileAtomU {-[*]-} :: (Ord upsilon) =>
	(MapT (upsilon) (MMixVarT), SetT MMixVarT, MMixVarT, MMixOpcodeT, Integer {->-}, AtomT upsilon) ->
	[MMixInstrT]

compileAtomU {-sem-}(_Sigma, nu_set_K, nu_R, _OP, _Y {->-}, alpha) ---
		= 	compileAtom {--}(_Sigma, nu_set_K, R _A {->-}, alpha) ++
	  	  	{-sem-}[MMixInstr (_OP) (fromReg (R _R)) {-,-} (_Y) {-,-} (fromReg (R _A)) {-hide-}Nothing ] ++
  		  	compileCopy {--}(nu_R {->-}, R _R)

 where	 {-sem-}(R _A)	 = allocReg{--}(_Rt {->-}, allocAtom{--}(_Sigma, nu_set_K {->-}, alpha))
 	 {-sem-}(R _R)	 = allocReg{--}(_Rt {->-}, nu_R)
\end{code}

\BeginHaskellToken	{compileAtomA}				{Open}	{\ZapfChancery{C}\sub{\SmallCaps{a}}}
  \HaskellContext							{(Ord upsilon)}
  \HaskellArgument							{(MapT (upsilon) (MMixVarT), SetT MMixVarT, MMixVarT, MMixOpcodeT {->-}, AtomT upsilon, AtomT upsilon)}
  \HaskellResult							{[MMixInstrT]}
\EndHaskellToken

\set{capture}		{compileAtomA}
\begin{code}
compileAtomA {-[*]-} :: (Ord upsilon) =>
	(MapT (upsilon) (MMixVarT), SetT MMixVarT, MMixVarT, MMixOpcodeT {->-}, AtomT upsilon, AtomT upsilon) ->
	[MMixInstrT]

compileAtomA {-sem-}(_Sigma, nu_set_K, nu_R, _OP {->-}, alpha1, alpha2) ---
				= 	compileAtom{--}(_Sigma, nu_set_KA, nu_A {->-}, alpha1) ++
					compileAtom{--}(_Sigma, nu_set_KB, R _B {->-}, alpha2) ++
					compileCopy{--}(R _A {->-}, nu_A) ++
					{-sem-}[ MMixInstr (_OP) (fromReg (R _R)) {-,-} (fromReg (R _A)) {-,-} (fromReg (R _B)) {-hide-}Nothing ] ++
					compileCopy{--}(nu_R {->-}, R _R)
 where	{-sem-}(R _A)		 = allocReg{--}(_Rt {->-}, nu_A)
 	{-sem-}(R _B)		 = allocReg{--}(_Ru {->-}, nu_B)
 	{-sem-}(R _R)		 = allocReg{--}(_Rt {->-}, nu_R)
	{-raw-}(nu_A)		 = allocAtom{--}(_Sigma, nu_set_KA {->-}, alpha1)
 	{-raw-}(nu_B)		 = allocAtom{--}(_Sigma, nu_set_KB {->-}, alpha2)
 	{-raw-}(nu_set_KA)	 = nu_set_K `union` listToSet [ _Sigma#(upsilon_k) | upsilon_k <- setToList (fvOfAtom{--}(alpha2)) ]
 	{-raw-}(nu_set_KB)	 = nu_set_K `union` listToSet [ nu_A ]
\end{code}

\BeginHaskellToken	{compileAtomB}				{Open}	{\ZapfChancery{C}\sub{\SmallCaps{b}}}
  \HaskellContext							{(Ord upsilon)}
  \HaskellArgument							{(MapT (upsilon) (MMixVarT), SetT MMixVarT, MMixVarT, MMixOpcodeT {->-}, AtomT upsilon, AtomT upsilon)}
  \HaskellResult							{[MMixInstrT]}
\EndHaskellToken

\set{capture}		{compileAtomB}
\begin{code}
compileAtomB {-[*]-} :: (Ord upsilon) =>
	(MapT (upsilon) (MMixVarT), SetT MMixVarT, MMixVarT, MMixOpcodeT {->-}, AtomT upsilon, AtomT upsilon) ->
	[MMixInstrT]

compileAtomB {-sem-}(_Sigma, nu_set_K, nu_R, _OP {->-}, alpha1, alpha2) ---
				= 	compileAtom{--}(_Sigma, nu_set_KA, nu_A {->-}, alpha1) ++
					compileAtom{--}(_Sigma, nu_set_KB, R _B {->-}, alpha2) ++
					compileCopy{--}(R _A {->-}, nu_A) ++
					{-sem-}[ MMixInstr (_OP) (fromReg _Rt) {-,-} (fromReg (R _A)) {-,-} (fromReg (R _B)) {-hide-}Nothing ] ++
					compileCopy{--}(nu_R {->-}, RR)
 where	{-sem-}(R _A)		 = allocReg{--}(_Rt {->-}, nu_A)
 	{-sem-}(R _B)		 = allocReg{--}(_Ru {->-}, nu_B)
	{-raw-}(nu_A)		 = allocAtom{--}(_Sigma, nu_set_KA {->-}, alpha1)
 	{-raw-}(nu_B)		 = allocAtom{--}(_Sigma, nu_set_KB {->-}, alpha2)
 	{-raw-}(nu_set_KA)	 = nu_set_K `union` listToSet [ _Sigma#(upsilon_k) | upsilon_k <- setToList (fvOfAtom{--}(alpha2)) ]
 	{-raw-}(nu_set_KB)	 = nu_set_K `union` listToSet [ nu_A ]
\end{code}

\BeginHaskellToken	{compileAtomC}				{Open}	{\ZapfChancery{C}\sub{\SmallCaps{c}}}
  \HaskellContext							{(Ord upsilon)}
  \WideHaskellArgument							{(MapT (upsilon) (MMixVarT), SetT MMixVarT, MMixVarT, MMixOpcodeT, MMixOpcodeT {->-}, 
  										AtomT upsilon, AtomT upsilon)}
  \HaskellResult							{[MMixInstrT]}
\EndHaskellToken

\set{capture}		{compileAtomC}
\begin{code}
compileAtomC {-[*]-} :: (Ord upsilon) =>
	(MapT (upsilon) (MMixVarT), SetT MMixVarT, MMixVarT,
	{-quad-} MMixOpcodeT, MMixOpcodeT {->-}, AtomT upsilon, AtomT upsilon) ->
	[MMixInstrT]

compileAtomC {-sem-}(_Sigma, nu_set_K, nu_R, _OP1, _OP2 {->-}, alpha1, alpha2) ---
				= 	compileAtom{--}(_Sigma, nu_set_KA, nu_A {->-}, alpha1) ++
					compileAtom{--}(_Sigma, nu_set_KB, R _B {->-}, alpha2) ++
					compileCopy{--}(R _A {->-}, nu_A) ++
					{-sem-}[ MMixInstr (_OP1) 	(fromReg (R _R)) {-,-} 	(fromReg (R _A)) {-,-} 	(fromReg (R _B)) {-hide-}Nothing ] ++
					{-sem-}[ MMixInstr (_OP2)	(fromReg (R _R)) {-,-} 	(fromReg (R _R)) {-,-} 	(1) {-hide-}Nothing ] ++
					compileCopy{--}(nu_R {->-}, R _R) 
 where	{-sem-}(R _A)		 = allocReg{--}(_Rt {->-}, nu_A)
 	{-sem-}(R _B)		 = allocReg{--}(_Ru {->-}, nu_B)
 	{-sem-}(R _R)		 = allocReg{--}(_Rt {->-}, nu_R)
	{-raw-}(nu_A)		 = allocAtom{--}(_Sigma, nu_set_KA {->-}, alpha1)
 	{-raw-}(nu_B)		 = allocAtom{--}(_Sigma, nu_set_KB {->-}, alpha2)
 	{-raw-}(nu_set_KA)	 = nu_set_K `union` listToSet [ _Sigma#(upsilon_k) | upsilon_k <- setToList (fvOfAtom{--}(alpha2)) ]
 	{-raw-}(nu_set_KB)	 = nu_set_K `union` listToSet [ nu_A ]
\end{code}

We are now ready to proceed with the actual translation of Etude atoms into appropriate MMIX instruction sequences. A proof of this translation's correctness is included at the end of the section, once we have completed the compiler's design for all valid atomic forms.

To begin with, we observe that, under all sequential program representations such as that provided by the MMIX instruction set architectures, every intermediate result of an arithmetic computation must be bound to a variable before its value can be utilised in the remainder of the program. Accordingly, we commence our translation of Etude into MMIX by describing a register allocation algorithm that selects an appropriate MMIX register for every well-formed atomic expression. A naïve but sufficient implementation of this algorithm can be defined as follows:

\BeginHaskellToken	{allocAtom}				{Open}	{\ZapfChancery{A}\!\sub{«\alpha»}}
  \HaskellContext							{(Ord upsilon)}
  \HaskellArgument							{(MapT (upsilon) (MMixVarT), SetT MMixVarT {->-}, AtomT upsilon)}
  \HaskellResult							{MMixVarT}
\EndHaskellToken

\IndexTerm		{allocAtom}					{A_\alpha atom MMIX_vars MMIX_var (MMIX_var)}

\begin{code}
allocAtom {-[*]-} :: (Ord upsilon) => (MapT (upsilon) (MMixVarT), SetT MMixVarT {->-}, AtomT upsilon) -> MMixVarT

allocAtom {-sem-}(_Sigma, nu_set_K {->-}, VAR upsilon)									= _Sigma#(upsilon)
allocAtom {-sem-}(_Sigma, nu_set_K {->-}, OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(alpha)) | (phi `isSubformatOf` phi')	= allocAtom{--}(_Sigma, nu_set_K {->-}, alpha)
allocAtom {-sem-}(_Sigma, nu_set_K {->-}, alpha)									= head{--}[ nu_k | nu_k <- nu_set_G, nu_k /@ nu_set_K ]
\end{code}

In addition to the usual variable substitution mapping "_Sigma", the above function also accepts a set of «live variables» "nu_set_K" that, intuitively, must be preserved for other uses later in the surrounding program. Accordingly, "allocAtom{--}(_Sigma, nu_set_K {->-}, alpha)" allocates the specified Etude atom "alpha" to the first variable "nu_k @@ nu_set_G" whose name does not appear in "nu_set_K". However, if "alpha" represents a simple Etude variable "upsilon", then "upsilon" is always associated with the MMIX register "_Sigma#(upsilon)" that was previously allocated for that variable. Further, as a special case, any operation of the form “"OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(alpha)"” that, under the earlier semantics of Etude atoms defined in Section \ref{Lambda/Arch/Atoms}, is reducible to "alpha" itself, is allocated to the same register as its operand "alpha", since such redundant conversions appear frequently in programs generated by automatic translations from other sources. Formally, the “"`isSubformatOf`"” operator used above for identification of these atomic forms is defined as follows:

\BeginHaskellToken	{isSubformatOf}				{Void}	{}
  \HaskellArgument							{FormatT}
  \HaskellToken		{{-subseteq-}}				{Rel}	{\subseteq}
  \HaskellArgument							{FormatT}
  \HaskellResult							{Bool}
\EndHaskellToken

\NewHaskellToken	{`isSubformatOf`}			{Rel}	{\subseteq}

\IndexTerm		{isSubformatOf}					{\subseteq format format (bool)}

\begin{code}
isSubformatOf {-[*]-} {-subseteq-} {-[*]-} :: FormatT -> FormatT -> Bool

isSubformatOf {-sem-}(phi1) {-subseteq-} {-sem-}(phi2) ---
	=	(genreOfFormat{--}(phi1) == genreOfFormat{--}(phi2) && encodingOfFormat{--}(phi1) <= encodingOfFormat{--}(phi2)) ||
		(genreOfFormat{--}(phi1) == {-sem-}(NG) && genreOfFormat{--}(phi2) == {-sem-}(ZG) && encodingOfFormat{--}(phi1) < encodingOfFormat{--}(phi2)) ||
		(genreOfFormat{--}(phi1) /= {-sem-}(RG) && genreOfFormat{--}(phi2) @@ listToSet [NG, ZG] && encodingOfFormat{--}(phi2) >= 64) ||
		(genreOfFormat{--}(phi1) /= {-sem-}(RG) && genreOfFormat{--}(phi2) @@ listToSet [FG, OG])
\end{code}

The actual sequence of MMIX instructions used to represent a given atom "alpha" in the translated program is then obtained by the following algorithm "compileAtom":

\BeginHaskellToken	{compileAtom}				{Open}	{\ZapfChancery{C}\!\sub{«\alpha»}}
  \HaskellContext							{(Ord upsilon)}
  \HaskellArgument							{(MapT (upsilon) (MMixVarT), SetT MMixVarT, MMixVarT {->-}, AtomT upsilon)}
  \HaskellResult							{[MMixInstrT]}
\EndHaskellToken

\IndexTerm		{compileAtom}					{C_\alpha atom MMIX_var MMIX_vars (MMIX_instrs)}

\begin{code}
compileAtom {-[*]-} :: (Ord upsilon) => (MapT (upsilon) (MMixVarT), SetT MMixVarT, MMixVarT {->-}, AtomT upsilon) -> [MMixInstrT]
\end{code}

In every construction of the form “"compileAtom{--}(_Sigma, nu_set_K, nu_R {->-}, alpha)"”, the «result variable» "nu_R" should represent an MMIX register which has been previously allocated for the atom by \hbox{"allocAtom{--}(_Sigma, nu_set_K {->-}, alpha)".} If "alpha" is formed entirely from a single Etude variable "upsilon", or if it depicts some atomic constant “"IMM (x) (phi)"\!”, then the translation proceeds as follows:

\begin{code}
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, VAR upsilon) ---
			=	compileCopy{--}(nu_R {->-}, _Sigma#(upsilon))
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, IMM (x) (phi)) ---
  | (n < 2^16)		=	{-sem-}[	MMixInstr (MMixSETL)	(fromReg (R _R)) {-,-} ({-sem-}(discardFraction n `bits` [8..15])) {-,-} ---
												({-sem-}(discardFraction n `bits` [0..7])) {-hide-}Nothing ] ++
				compileCopy{--}(nu_R {->-}, R _R)	
  | (n < 2^32)		=	{-sem-}[	MMixInstr (MMixSETL)	(fromReg (R _R)) {-,-} ({-sem-}(discardFraction n `bits` [8..15])) {-,-} ---
												({-sem-}(discardFraction n `bits` [0..7])) {-hide-}Nothing {-hide-},
						MMixInstr (MMixINCML)	(fromReg (R _R)) {-,-} ({-sem-}(discardFraction n `bits` [24..31])) {-,-} ---
												({-sem-}(discardFraction n `bits` [16..23])) {-hide-}Nothing ] ++
				compileCopy{--}(nu_R {->-}, R _R)	
  | (n < 2^48)		=	{-sem-}[	MMixInstr (MMixSETL)	(fromReg (R _R)) {-,-} ({-sem-}(discardFraction n `bits` [8..15])) {-,-} ---
												({-sem-}(discardFraction n `bits` [0..7])) {-hide-}Nothing {-hide-},
						MMixInstr (MMixINCML)	(fromReg (R _R)) {-,-} ({-sem-}(discardFraction n `bits` [24..31])) {-,-} ---
												({-sem-}(discardFraction n `bits` [16..23])) {-hide-}Nothing {-hide-},
						MMixInstr (MMixINCMH)	(fromReg (R _R)) {-,-} ({-sem-}(discardFraction n `bits` [40..47])) {-,-} ---
												({-sem-}(discardFraction n `bits` [32..39])) {-hide-}Nothing ] ++
				compileCopy{--}(nu_R {->-}, R _R)	
  | otherwise		=	{-sem-}[	MMixInstr (MMixSETL)	(fromReg (R _R)) {-,-} ({-sem-}(discardFraction n `bits` [8..15])) {-,-} ---
												({-sem-}(discardFraction n `bits` [0..7])) {-hide-}Nothing {-hide-},
						MMixInstr (MMixINCML)	(fromReg (R _R)) {-,-} ({-sem-}(discardFraction n `bits` [24..31])) {-,-} ---
												({-sem-}(discardFraction n `bits` [16..23])) {-hide-}Nothing {-hide-},
						MMixInstr (MMixINCMH)	(fromReg (R _R)) {-,-} ({-sem-}(discardFraction n `bits` [40..47])) {-,-} ---
												({-sem-}(discardFraction n `bits` [32..39])) {-hide-}Nothing {-hide-},
						MMixInstr (MMixINCH)	(fromReg (R _R)) {-,-} ({-sem-}(discardFraction n `bits` [56..63])) {-,-} ---
												({-sem-}(discardFraction n `bits` [48..55])) {-hide-}Nothing ] ++
				compileCopy{--}(nu_R {->-}, R _R)
 where	{-sem-}(IMM (n) (NG`FMT`64)) ---
 		=  evalAtom{-sem-}(IMM (x) (phi) {-hide-}:: {-hide-}AtomT {-hide-}Integer)
 	{-sem-}(R _R) ---
 		=  allocReg{--}(_Rt {->-}, nu_R)
\end{code}

Intuitively, the above construction attempts to find the shortest sequence of MMIX instructions that are sufficient to bind "nu_R" to the atom's semantic interpretation. The reader should observe that, for many constants of the form “"IMM (n) (phi)"\!”, an even shorter sequence could be obtained using various MMIX instruction forms that were excluded from the discussion in Section \ref{MMIX/Semantics}. However, for conciseness, in this chapter I present only the simplest sufficient implementation of the MMIX code generator.

Next, we dispense with the translation of the following five simple atomic forms, whose meanings were defined in Section \ref{Lambda/Arch/Atoms} in terms of some other Etude constructs:

\begin{code}
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP1 (op) (phi) {--} {-hv-}(alpha)) ---
  | genreOfFormat{--}(phi)   @@ listToSet [ FG, OG ] {-ord-}	= compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP1 (op) (ZG`FMT`64) {--} {-hv-}(alpha))
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP1 (CNV phi') (phi) {--} {-hv-}(alpha)) ---
  | genreOfFormat{--}(phi')  @@ listToSet [ FG, OG ]		= compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP1 (CNV (ZG`FMT`64)) (phi) {--} {-hv-}(alpha))
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) (op) (phi) (alpha2)) ---
  | genreOfFormat{--}(phi)   @@ listToSet [ FG, OG ]		= compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) (op) (ZG`FMT`64) (alpha2))
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP1 (NEG) (phi) {--} {-hv-}(alpha)) ---
  | genreOfFormat{--}(phi) == {-sem-}(RG) 			= compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 ({-sem-}(minusZero)) (SUB) (phi) (alpha))
  | genreOfFormat{--}(phi) /= {-sem-}(RG)  			= compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 ({-sem-}(plusZero)) (SUB) (phi) (alpha))
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP1 (NOT) (phi) {--} {-hv-}(alpha)) ---
								= compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (IMM ({-sem-}(2^64 - 1)) (NG`FMT`64)) (SUB) (phi) (alpha))
\end{code}

Similarly, an Etude conversion of the form “"OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(alpha)"”, in which "phi `isSubformatOf` phi'", was defined in Section \ref{Lambda/Arch/Atoms} to be synonymous with its atomic operand "alpha". On the other hand, most other conversions that involve the rational format “"RG`FMT`64"” must be performed in the resulting program using an appropriate application of the MMIX opcode “\,"MMixFIXU"\,”, “\,"MMixFIX"\,”, “\,"MMixFLOTU"\,” or “\,"MMixFLOT"\,” as follows:

\begin{code}
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(alpha)) ---
	| phi `isSubformatOf` phi' ---
  		= 	compileAtom {--}(_Sigma, nu_set_K, nu_R {->-}, alpha)
	| phi' == {-sem-}({-hfil-}NG  `FMT`64) 		&& genreOfFormat{--}(phi) == {-sem-}({-hfil-}RG  ) ---
		= 	compileAtomU{--}(_Sigma, nu_set_K, R _A, MMixFIXU, 		0 {->-}, alpha)
	| phi' == {-sem-}({-hfil-}ZG  `FMT`64) 		&& genreOfFormat{--}(phi) == {-sem-}({-hfil-}RG  ) ---
		= 	compileAtomU{--}(_Sigma, nu_set_K, R _A, MMixFIX, 		0 {->-}, alpha)
	| genreOfFormat{--}(phi') == {-sem-}(RG) 	&& genreOfFormat{--}(phi) == {-sem-}({-hfil-}NG  ) ---
		= 	compileAtomU{--}(_Sigma, nu_set_K, R _A, MMixFLOTU, 		0 {->-}, alpha)
	| genreOfFormat{--}(phi') == {-sem-}(RG) 	&& genreOfFormat{--}(phi) == {-sem-}({-hfil-}ZG  ) ---
		= 	compileAtomU{--}(_Sigma, nu_set_K, R _A, MMixFLOT, 		0 {->-}, alpha)

 where	 {-sem-}(R _A)	 = allocReg{--}(_Rt {->-}, allocAtom{--}(_Sigma, nu_set_K {->-}, alpha))
 	 {-sem-}(R _R)	 = allocReg{--}(_Rt {->-}, nu_R)
\end{code}

In all of these cases, an MMIX rendition of the operand atom is first obtained using a recursive application of the algorithm "compileAtom", binding that operand's value to a temporary MMIX variable chosen for the atom by the earlier definitions of "allocAtom" and "allocReg". The scratch register "_Rt" is used to hold both the value of "alpha" and the result of the entire operation if either of these happens to be allocated by "allocAtom" to a pseudo-register. Once the desired result has been computed by the selected instruction, its output is copied into the specified MMIX variable "nu_R", whereby it becomes available to the rest of the translated program. Formally:

\IndexTerm		{compileAtomU}					{C_A atom atom MMIX_opcode MMIX_var MMIX_vars (MMIX_instrs)}
\InsertHaskell		{compileAtomU}

All other unary Etude operations of the form “"OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(alpha)"” describe conversions of a rational number into some non-standard integral format, or else a conversion between a pair of such integral formats. In the first case, the operation is enacted in the translated program through an intermediate conversion of "alpha" into the corresponding standard format “"NG`FMT`64"” or “"ZG`FMT`64"”, as depicted by the Etude term \hbox{“"OP1 (CNV phi' {-!-}) (phi'') {--} {-hv-}(OP1 (CNV phi'' {-!-}{-!-}) (phi) {--} {-hv-}(alpha))"”,} in which "phi''" represents a standard format of the same genre as "phi'". In the later case, we observe that, by the virtue of their semantic interpretation in Section \ref{Lambda/Arch/Atoms}, every such conversion is essentially equivalent to \hbox{"alpha `mod` 2^(widthOfFormat{--}(phi'))",} which can be also rephrased as \hbox{"floor (((alpha * 2^(64 - widthOfFormat{--}(phi'))) `mod` 2^64) / 2^(64 - widthOfFormat{--}(phi')))".} The later variant of this expression can be easily compiled into an efficient MMIX instruction sequence through the following intermediate translation into a pattern of Etude bit shift operations:

\begin{code}
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(alpha))
	| (gamma' @@ listToSet [NG, ZG] && gamma == {-sem-}(RG)) --
		= compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, --
					OP1 (CNV phi' {-!-}) (phi'') {--} {-hv-}(OP1 (CNV phi'' {-!-}{-!-}) (phi) {--} {-hv-}(alpha)))
	| otherwise --
		= compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, --
					OP2 ({-hv-}(OP2 (alpha) (SHL) (NG`FMT`64) (IMM (toRational {-sem-}(64 - epsilon')) (NG`FMT`64)))) --
		  						(SHR) (phi'') (IMM (toRational {-sem-}(64 - epsilon')) (NG`FMT`64)))

 where	 {-sem-}(gamma`FMT`epsilon)    	= phi
 	 {-sem-}(gamma'`FMT`epsilon')  	= phi'
 	 {-raw-}(phi'')			= {-sem-}(gamma'`FMT`64)
\end{code}

All of the remaining atomic forms that are defined by Etude pertain to applications of its various binary operators. Each of these operators is naturally associated with a predetermined MMIX opcode as captured by the following translation:

\begin{code}
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}ADD  ) (RG`FMT`epsilon)  	(alpha2)) --
	= compileAtomA{--}(_Sigma, nu_set_K, nu_R, MMixFADD  	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}SUB  ) (RG`FMT`epsilon)  	(alpha2)) --
	= compileAtomA{--}(_Sigma, nu_set_K, nu_R, MMixFSUB  	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}MUL  ) (RG`FMT`epsilon)  	(alpha2)) --
	= compileAtomA{--}(_Sigma, nu_set_K, nu_R, MMixFMUL  	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}DIV  ) (RG`FMT`epsilon)  	(alpha2)) --
	= compileAtomA{--}(_Sigma, nu_set_K, nu_R, MMixFDIV  	{->-}, alpha1, alpha2)

compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}ADD  ) (NG`FMT`64)  		(alpha2)) --
	= compileAtomA{--}(_Sigma, nu_set_K, nu_R, MMixADDU 	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}SUB  ) (NG`FMT`64)  		(alpha2)) --
	= compileAtomA{--}(_Sigma, nu_set_K, nu_R, MMixSUBU 	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}MUL  ) (NG`FMT`64)  		(alpha2)) --
	= compileAtomA{--}(_Sigma, nu_set_K, nu_R, MMixMULU 	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}DIV  ) (NG`FMT`epsilon) 	(alpha2)) --
	= compileAtomA{--}(_Sigma, nu_set_K, nu_R, MMixDIVU  	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}DIV  ) (ZG`FMT`epsilon) 	(alpha2)) --
	= compileAtomA{--}(_Sigma, nu_set_K, nu_R, MMixDIV  	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}SHL  ) (NG`FMT`64)	 	(alpha2)) --
	= compileAtomA{--}(_Sigma, nu_set_K, nu_R, MMixSLU  	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}AND  ) (phi) 			(alpha2)) --
	= compileAtomA{--}(_Sigma, nu_set_K, nu_R, MMixAND  	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}XOR  ) (phi) 			(alpha2)) --
	= compileAtomA{--}(_Sigma, nu_set_K, nu_R, MMixXOR  	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}IOR  ) (phi) 			(alpha2)) --
	= compileAtomA{--}(_Sigma, nu_set_K, nu_R, MMixOR  	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}SHR  ) (NG`FMT`epsilon)		(alpha2)) --
	= compileAtomA{--}(_Sigma, nu_set_K, nu_R, MMixSRU  	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}SHR  ) (ZG`FMT`epsilon)		(alpha2)) --
	= compileAtomA{--}(_Sigma, nu_set_K, nu_R, MMixSR  	{->-}, alpha1, alpha2)
\end{code}

All of these atomic forms are invariably compiled into almost identical sequences of MMIX instructions, which are distinguished only in their choice of a specific opcode "_OP" for enactment of the desired computational effect. In particular, the resulting program always beings by evaluating the first operand "alpha1" and binding its result to an appropriate MMIX variable "nu_A" that has been designated for the purpose by "allocAtom". In order to preserve the precise semantics of the following operand "alpha2", all Etude variables that appear free within "alpha2" are added to the set of live variables "nu_set_KA" during translation of "alpha1". Next, "alpha2" is likewise evaluated and bound to some general purpose hardware register "R _B", equal to the atom's allotted variable "nu_B" if "nu_B" is implemented in hardware, or else to the scratch register "R _U" if all such hardware registers have been already assigned to other rôles by the program. Within "alpha2", the set of live variables is extended with "nu_A", in order to preserve that variable's value during the atom's evaluation. Subsequently, the compiler ensures that the value of "nu_A" is available in a hardware register, using the scratch space of "R _T" if necessary. Finally, the specified MMIX instruction "_OP" is applied to the values of the two operands and the result of the entire construction is copied into the desired MMIX variable "nu_R", mediating it through "R _T" if deemed necessary by the register allocator. Formally:

\IndexTerm		{compileAtomA}					{C_A atom atom MMIX_opcode MMIX_var MMIX_vars (MMIX_instrs)}
\InsertHaskell		{compileAtomA}

Similarly, the translation of signed and unsigned integer remainder operations is enacted using an appropriate MMIX division instruction “\,"MMixDIVU"\,” or “\,"MMixDIV"\,”:

\begin{code}
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}REM  ) (NG`FMT`epsilon) 	(alpha2)) --
	= compileAtomB{--}(_Sigma, nu_set_K, nu_R, MMixDIVU  	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}REM  ) (ZG`FMT`epsilon) 	(alpha2)) --
	= compileAtomB{--}(_Sigma, nu_set_K, nu_R, MMixDIV 	{->-}, alpha1, alpha2)
\end{code}

The precise translation of these two atomic forms is identical to the earlier treatment of other binary arithmetic operations, except that the ultimate result of the entire construct can be found in the special purpose register "RR" rather than its general purpose counterpart "R _R" that was featured in the definition of "compileAtomA":

\IndexTerm		{compileAtomB}					{C_B atom atom MMIX_opcode MMIX_var MMIX_vars (MMIX_instrs)}
\InsertHaskell		{compileAtomB}

On the other hand, every application of the Etude comparison operator “"CEQ{-sub-}phi"\!”, “"CNE{-sub-}phi"\!”, “"CLT{-sub-}phi"\!”, “"CGT{-sub-}phi"\!”, “"CLE{-sub-}phi"\!” or “"CGE{-sub-}phi"\!” must be translated into a sequence of two MMIX instructions. First, the actual comparison is performed using the operation “\,"MMixFCMP"”, “"MMixCMPU"\,” or “"MMixCMP"”, as appropriate for the underlying Etude format "phi". Next, the value of "negate 1", "0" or "1" that each of these instruction forms delivers is converted into the required boolean constant "1" or "0" using one of the six conditional copy instructions “\,"MMixZSZi"\!”, “\,"MMixZSNZi"\!”, “\,"MMixZSNi"\!”, “\,"MMixZSPi"\!”, “\,"MMixZSNPi"\!” or “\,"MMixZSNNi"\!”, as required to achieve the overall effect of the specified relational operation. In particular:

\begin{code}
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}CEQ  ) (RG`FMT`epsilon) 	(alpha2)) ---
	= compileAtomC{--}(_Sigma, nu_set_K, nu_R, MMixFCMP, 	MMixZSZi	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}CNE  ) (RG`FMT`epsilon) 	(alpha2)) ---
	= compileAtomC{--}(_Sigma, nu_set_K, nu_R, MMixFCMP, 	MMixZSNZi	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}CLT  ) (RG`FMT`epsilon) 	(alpha2)) ---
	= compileAtomC{--}(_Sigma, nu_set_K, nu_R, MMixFCMP, 	MMixZSNi	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}CGT  ) (RG`FMT`epsilon) 	(alpha2)) ---
	= compileAtomC{--}(_Sigma, nu_set_K, nu_R, MMixFCMP, 	MMixZSPi	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}CLE  ) (RG`FMT`epsilon) 	(alpha2)) ---
	= compileAtomC{--}(_Sigma, nu_set_K, nu_R, MMixFCMP, 	MMixZSNPi	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}CGE  ) (RG`FMT`epsilon) 	(alpha2)) ---
	= compileAtomC{--}(_Sigma, nu_set_K, nu_R, MMixFCMP, 	MMixZSNNi	{->-}, alpha1, alpha2)

compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}CEQ  ) (NG`FMT`epsilon) 	(alpha2)) ---
	= compileAtomC{--}(_Sigma, nu_set_K, nu_R, MMixCMPU, 	MMixZSZi	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}CNE  ) (NG`FMT`epsilon) 	(alpha2)) ---
	= compileAtomC{--}(_Sigma, nu_set_K, nu_R, MMixCMPU, 	MMixZSNZi	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}CLT  ) (NG`FMT`epsilon) 	(alpha2)) ---
	= compileAtomC{--}(_Sigma, nu_set_K, nu_R, MMixCMPU, 	MMixZSNi	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}CGT  ) (NG`FMT`epsilon) 	(alpha2)) ---
	= compileAtomC{--}(_Sigma, nu_set_K, nu_R, MMixCMPU, 	MMixZSPi	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}CLE  ) (NG`FMT`epsilon) 	(alpha2)) ---
	= compileAtomC{--}(_Sigma, nu_set_K, nu_R, MMixCMPU, 	MMixZSNPi	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}CGE  ) (NG`FMT`epsilon) 	(alpha2)) ---
	= compileAtomC{--}(_Sigma, nu_set_K, nu_R, MMixCMPU, 	MMixZSNNi	{->-}, alpha1, alpha2)

compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}CEQ  ) (ZG`FMT`epsilon) 	(alpha2)) ---
	= compileAtomC{--}(_Sigma, nu_set_K, nu_R, MMixCMP, 	MMixZSZi	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}CNE  ) (ZG`FMT`epsilon) 	(alpha2)) ---
	= compileAtomC{--}(_Sigma, nu_set_K, nu_R, MMixCMP, 	MMixZSNZi	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}CLT  ) (ZG`FMT`epsilon) 	(alpha2)) ---
	= compileAtomC{--}(_Sigma, nu_set_K, nu_R, MMixCMP, 	MMixZSNi	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}CGT  ) (ZG`FMT`epsilon) 	(alpha2)) ---
	= compileAtomC{--}(_Sigma, nu_set_K, nu_R, MMixCMP, 	MMixZSPi	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}CLE  ) (ZG`FMT`epsilon) 	(alpha2)) ---
	= compileAtomC{--}(_Sigma, nu_set_K, nu_R, MMixCMP, 	MMixZSNPi	{->-}, alpha1, alpha2)
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) ({-hfil-}CGE  ) (ZG`FMT`epsilon) 	(alpha2)) ---
	= compileAtomC{--}(_Sigma, nu_set_K, nu_R, MMixCMP, 	MMixZSNNi	{->-}, alpha1, alpha2)
\end{code}

where the actual compilation of these atoms is performed by the following Haskell construction "compileAtomC", whose structure follows the same general principles as its earlier cousins "compileAtomA" and "compileAtomB":

\IndexTerm		{compileAtomC}					{C_A atom atom MMIX_opcode MMIX_var MMIX_vars (MMIX_instrs)}
\InsertHaskell		{compileAtomC}

Finally, the four Etude operators “"ADD{-sub-}phi"\!”, “"SUB{-sub-}phi"\!”, “"MUL{-sub-}phi"\!” and “"SHL{-sub-}phi"\!” can be also applied under non-standard integral formats. Observing that all of these operators implement pure modulo arithmetic, their MMIX representation can be obtained trivially by evaluating the operation under the standard natural format “"NG`FMT`64"” and converting the resulting value into the desired precision of "phi":

\begin{code}
compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP2 (alpha1) (op) (phi) (alpha2)) ---
  | op @@ listToSet [ ADD, SUB, MUL, SHL ] && genreOfFormat{--}(phi) @@ listToSet [ NG, ZG ]
  	= compileAtom {-sem-}(_Sigma, nu_set_K, nu_R {->-}, OP1 (CNV phi) (NG`FMT`64) {--} {-hv-}(OP2 (alpha1) (op) (NG`FMT`64) (alpha2)))
\end{code}

Last but not least, a list of Etude atoms can be compiled into a sequence of semantically equivalent MMIX instructions by the following recursive algorithm "compileAtoms":

\BeginHaskellToken	{compileAtoms}				{Open}	{\ZapfChancery{C}\!\sub{«\macron\alpha»}}
  \HaskellContext							{(Ord upsilon)}
  \HaskellArgument							{(MapT (upsilon) (MMixVarT), SetT MMixVarT, [MMixVarT] {->-}, AtomsT upsilon)}
  \HaskellResult							{[MMixInstrT]}
\EndHaskellToken

\IndexTerm		{compileAtoms}					{C_\alpha- atoms MMIX_vars MMIX_vars (MMIX_instrs)}

\begin{code}
compileAtoms {-[*]-} :: (Ord upsilon) =>
	(MapT (upsilon) (MMixVarT), SetT MMixVarT, [MMixVarT] {->-}, AtomsT upsilon) ->
	[MMixInstrT]
\end{code}

Specifically, \hbox{"compileAtoms{--}(_Sigma, nu_set_K, nu_set_R {->-}, alpha_set {-,-})"} successively compiles each of the supplied atoms "alpha_k" in "alpha_set", binding their respective results to the corresponding MMIX variables from "nu_set_R". During translation of each "alpha_k", the existing value of the targeted register "nu_R" is saved in a temporary MMIX variable "nu_R'" if "nu_R" appears free within the remainder of the list, in which case the binding of the corresponding Etude variable "upsilon_k" is also updated in the substitution function "_Sigma". The chosen temporary variable is also added to the atom's set of live register "nu_set_K", except that the actual target of the assignment "nu_R" is never included in "nu_set_K", in order to allow subsequent atoms in the list to reuse its preallocated value. Formally:

\begin{code}
compileAtoms {-sem-}(_Sigma, nu_set_K, nu_set_R {->-}, alpha_set {-,-}) ---
  | alpha_set /= {-empty-}[]	= 	compileCopy{--}(nu_R' {->-}, nu_R) ++
  				  	compileAtom{--}(_Sigma, nu_set_K' `union` listToSet [nu_R'] `difference` listToSet [nu_R] , nu_R {->-}, head{--}(alpha_set)) ++
  				  	compileAtoms{--}(_Sigma', nu_set_K, tail{--}(nu_set_R) {->-}, tail{--}(alpha_set))
  | otherwise			=	{-sem-}[]
 where	nu_R		= head{--}(nu_set_R)
 	nu_R'		= head{--}[ nu_k | nu_k <- {-raw-}[nu_R] ++ nu_set_G, nu_k /@ nu_set_K' ]
 	nu_set_K'	= nu_set_K `union` listToSet [ _Sigma#(upsilon_k) | upsilon_k <- setToList (fvOfAtoms{--}(tail{--}(alpha_set))) ]
	_Sigma'		= _Sigma // listToMap [ {-raw-}(upsilon_k {-:-}, nu_R') | upsilon_k <- setToList (fvOfAtoms{--}(tail{--}(alpha_set))), _Sigma#(upsilon_k) == nu_R ]
\end{code}

\set{title}		{Preservation of atom semantics under compilation}
\begin{theorem}

Let "_Sigma" be a variable renaming function and "_Gamma" represent the mapping of all relevant MMIX variables to their respective atomic values. Let "alpha" be an Etude atom that is closed and well-formed under the substitution by "_Gamma . _Sigma". Further, let "_Lambda'" represent a function environment updated with bindings of "n" successive instruction addresses that begin at some Etude function variable "nu" to the respective "n" instructions generated from the compilation of "alpha" and let "_Delta'" represent an object environment that is likewise updated with bindings of any symbolic addresses "LOC`SR`8{--}k" to the current values of all pseudo-registers "R k" which appear free within "alpha". Then, every application of "nu" to a the standard parameter list "pregs" under a substitution by "_Gamma" is always equivalent to an application of the function "nu `shiftMMixVar` 4{--}n" to the same parameter list under a substitution by an updated finite map "_Gamma // listToMap [ _Gamma#(_Sigma#(upsilon)) {-,,-}{-:-},{-,,-} alpha ]". Formally:

\NewHaskellToken	{atomPreservationJ}			{Ord}	{\TKeyword{LC\sub{«\alpha»}}}
\NewHaskellDescription	{atomPreservationJ}				{Preservation of atom semantics under compilation}
\IndexTheorem		{atomPreservationJ}				{LC_\alpha Preservation of atom semantics under compilation}

\begin{haskell}
atomPreservationJ :: forall _Sigma, _Gamma, _Gamma', _Lambda, _Lambda', _Delta, _Delta', nu, upsilon, alpha, nu_set_K, upsilon =>
	WfJ{--}(_Lambda) -> WfJ{--}(_Delta) -> WfJ{--}(alpha `substAtom` (_Gamma . _Sigma)) ->
	{-sem-}(_Gamma'	  = _Gamma 	// 		listToMap [ _Gamma#(_Sigma#(upsilon)) {-,,-}{-:-},{-,,-} alpha ]) ->
	{-sem-}(_Lambda'  = _Lambda 	// 		meaningsOfMMixInstr{--}(_Gamma, nu {->-}, compileAtom{--}(_Sigma, nu_set_K, _Sigma#(upsilon) {->-}, alpha))) ->
	{-sem-}(_Delta'   = _Delta 	`setInOEnv` 	listToMap [ (_Gamma#(LOC`SR`8{--}k) {-,,-}{-:-},{-,,-} _Gamma#(R k)) --
									| {-sem-}(R k) <- [ _Sigma#(upsilon_k) | upsilon_k <- fvOfAtom{--}(alpha) ], k >= 256 ]) ->
	{-sem-}(_Lambda', _Delta' {->-}, {-sem-}(APP ({-sem-}_Gamma#(nu)) (pregs)) `substAtom` _Gamma) `EqvJ` --
	{-sem-}(_Lambda, _Delta {->-}, --
		{-sem-}(APP ({-sem-}(_Gamma#(nu `shiftMMixVar` lengthOfList{--}(compileAtom{--}(_Sigma, nu_set_K, _Sigma#(upsilon) {->-}, alpha))))) (pregs)) `substTerm` _Gamma')
\end{haskell}
\begin{proof}

If "alpha" represents a single Etude register or a constant, then the above theorem follows directly from the earlier result "copyPreservationJ", which provides us with a convenient base case for the subsequent induction proof. Otherwise, if the atom represents one of the fifteen binary operations compiled through the construction "compileAtomA", then:

\begin{haskell}
compileAtom{--}(_Sigma, nu_set_K, _Sigma#(upsilon) {->-}, alpha) ---
	=	compileAtom{--}(_Sigma, nu_set_KA, nu_A {->-}, alpha1) ++
		compileAtom{--}(_Sigma, nu_set_KB, R _B {->-}, alpha2) ++
		compileCopy{--}(R _A {->-}, nu_A) ++
		{-sem-}[ MMixInstr (_OP) (fromReg (R _R)) {-,-} (fromReg (R _A)) {-,-} (fromReg (R _B)) {-hide-}Nothing ] ++
		compileCopy{--}(nu_R {->-}, R _R))
\end{haskell}

If we now apply "atomPreservationJ" inductively to the first two constructs in the above term and, further, utilise "copyPreservationJ" to similarly eliminate the third construct, then the resulting instruction sequence can be simplified into “"{-sem-}[ MMixInstr (_OP) (fromReg (R _R)) {-,-} (fromReg (R _A)) {-,-} (fromReg (R _B)) {-hide-}Nothing ] ++ compileCopy{--}(nu_R {->-}, R _R)"”, to be taken in the context of "_Gamma" updated with the set of bindings "listToMap [ nu_A {-:-}, alpha1, R _A {-:-}, alpha1, R _B {-:-}, alpha2 ]". Observing that the above instruction sequence would be translated by "meaningsOfMMixInstr" into an Etude function with the body of “"LET [{-raw-}[R _R] =: {-hide-}RET {-raw-}[OP2 (VAR (R _A)) (ADD) (RG`FMT`64)  (VAR (R _B))]] .. APP (VAR ({-sem-}(nu `shiftMMixVar` n))) (regs)"”, an application of this term to "pregs" under "pregs // listToMap [ nu_A {-:-}, alpha1, R _A {-:-}, alpha1, R _B {-:-}, alpha2 ]" is clearly equivalent to the term's body “"APP (VAR ({-sem-}(nu `shiftMMixVar` n))) (regs)"”, which matches the right-hand side of the desired equivalence form. \QED

\end{proof}
\end{theorem}
\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{MMIX/Lambda/Functions}
\set{title}		{Functions and Terms}
\begin{subsection}

\BeginHaskellToken	{saveRegs}				{Open}	{save}
  \HaskellArgument							{[MMixVarT]}
  \HaskellResult							{[MMixInstrT]}
\EndHaskellToken

\set{capture}		{saveRegs}
\begin{code}
saveRegs {-[*]-} :: [MMixVarT] -> [MMixInstrT]

saveRegs {-sem-}({-empty-}[]) ---
			=	{-sem-}[]
saveRegs {-sem-}(nu_set) ---
			= 	loadReg{--}(R _R {->-}, head{--}(nu_set)) ++
  				{-sem-}[	MMixInstr (MMixSETL)	(fromReg _Rt) {-,-}	(0) {-,-} 		(8) 		{-hide-}Nothing {-hide-},
  						MMixInstr (MMixSUBU) 	(fromReg _Rs) {-,-} 	(fromReg _Rs) {-,-} 	(fromReg _Rt) 	{-hide-}Nothing {-hide-},
  						MMixInstr (MMixSTOUi) 	(fromReg (R _R)) {-,-} 	(fromReg _Rs) {-,-} 	(0) 		{-hide-}Nothing ] ++
  				saveRegs{--}(tail{--}(nu_set))

 where	{-sem-}(R _R)	 = allocReg{--}(_Rt {->-}, head{--}(nu_set))
\end{code}

\BeginHaskellToken	{restoreRegs}				{Open}	{restore}
  \HaskellArgument							{[MMixVarT]}
  \HaskellResult							{[MMixInstrT]}
\EndHaskellToken

\set{capture}		{restoreRegs}
\begin{code}
restoreRegs {-[*]-} :: [MMixVarT] -> [MMixInstrT]

restoreRegs {-sem-}({-empty-}[]) ---
			=	{-sem-}[]
restoreRegs {-sem-}(nu_set) ---
			=	restoreRegs{--}(tail{--}(nu_set)) ++
  				{-sem-}[	MMixInstr (MMixLDOUi) 	(fromReg (R _R)) {-,-}	(fromReg _Rs) {-,-} 	(0) 		{-hide-}Nothing {-hide-},
  				  		MMixInstr (MMixSETL)	(fromReg _Rt) {-,-}	(0) {-,-} 		(8) 		{-hide-}Nothing {-hide-},
  						MMixInstr (MMixADDU) 	(fromReg _Rs) {-,-}	(fromReg _Rs) {-,-} 	(fromReg _Rt) 	{-hide-}Nothing ] ++
  				storeReg{--}(head{--}(nu_set) {->-}, R _R)

 where	{-sem-}(R _R)	 = allocReg{--}(_Rt {->-}, head{--}(nu_set))
\end{code}

\BeginHaskellToken	{compileLoad}				{Open}	{\ZapfChancery{C}\sub{\SmallCaps{ld}}}
  \HaskellContext							{(Ord upsilon)}
  \HaskellArgument							{(MapT (upsilon) (MMixVarT), SetT MMixVarT, MMixVarT, MMixOpcodeT {->-}, AtomT upsilon)}
  \HaskellResult							{[MMixInstrT]}
\EndHaskellToken

\set{capture}		{compileLoad}
\begin{code}
compileLoad {-[*]-} :: (Ord upsilon) =>
	(MapT (upsilon) (MMixVarT), SetT MMixVarT, MMixVarT, MMixOpcodeT {->-}, AtomT upsilon) ->
	[MMixInstrT]

compileLoad {-sem-}(_Sigma, nu_set_K, nu_R, _OP {->-}, alpha) ---
				=	compileAtom{--}(_Sigma, nu_set_K, R _A {->-}, alpha) ++
					{-sem-}[ MMixInstr (_OP) (fromReg (R _R)) {-,-} (fromReg (R _A)) {-,-} (0) {-hide-}Nothing ] ++
					compileCopy{--}(nu_R {->-}, R _R)

 where	{-sem-}(R _A)	= allocReg{--}(_Rt {->-}, allocAtom{--}(_Sigma, nu_set_K {->-}, alpha))
 	{-sem-}(R _R)	= allocReg{--}(_Rt {->-}, nu_R)
\end{code}

\BeginHaskellToken	{compileStore}				{Open}	{\ZapfChancery{C}\sub{\SmallCaps{st}}}
  \HaskellContext							{(Ord upsilon)}
  \HaskellArgument							{(MapT (upsilon) (MMixVarT), SetT MMixVarT, MMixVarT, MMixOpcodeT {->-}, AtomT upsilon, AtomT upsilon)}
  \HaskellResult							{[MMixInstrT]}
\EndHaskellToken

\set{capture}		{compileStore}
\begin{code}
compileStore {-[*]-} :: (Ord upsilon) =>
	(MapT (upsilon) (MMixVarT), SetT MMixVarT, MMixOpcodeT {->-}, AtomT upsilon, AtomT upsilon) ->
	[MMixInstrT]

compileStore {-sem-}(_Sigma, nu_set_K, _OP {->-}, alpha1, alpha2) ---
				= 	compileAtom{--}(_Sigma, nu_set_KA,  nu_A  {->-}, alpha1) ++
					compileAtom{--}(_Sigma, nu_set_KB,  R _B  {->-}, alpha2) ++
					compileCopy{--}(R _A {->-}, nu_A) ++
					{-sem-}[ MMixInstr (_OP) (fromReg (R _B)) {-,-} (fromReg (R _A)) {-,-} (0) {-hide-}Nothing ]
 where	{-sem-}(R _A)		=	allocReg{--}(_Rt {->-}, nu_A  )
 	{-sem-}(R _B)		=	allocReg{--}(_Ru {->-}, nu_B  )
	{-raw-}(nu_A)		=	allocAtom{--}(_Sigma, nu_set_KA  {->-}, alpha1)
 	{-raw-}(nu_B)		=	allocAtom{--}(_Sigma, nu_set_KB  {->-}, alpha2)
 	{-raw-}(nu_set_KA)	=	nu_set_K `union` listToSet [ _Sigma#(upsilon_k) | upsilon_k <- setToList (fvOfAtom{--}(alpha2)) ]
 	{-raw-}(nu_set_KB)	=	nu_set_K `union` listToSet [ nu_A ]
\end{code}

\BeginHaskellToken	{compileJump}				{Open}	{\ZapfChancery{C}\sub{\SmallCaps{j}}}
  \HaskellArgument							{Integer}
  \HaskellResult							{[MMixInstrT]}
\EndHaskellToken

\set{capture}		{compileJump}
\begin{code}
compileJump {-[*]-} :: Integer -> [MMixInstrT]

compileJump {-sem-}(delta) ---
  | (delta < 2^32)	= {-sem-}[	MMixInstr (MMixGETA) 	(fromReg _Rt) {-,-} ({-sem-}(delta `bits` [10..15])) {-,-} ({-sem-}(delta `bits` [2..9])) {-hide-}Nothing {-hide-},
		  			MMixInstr (MMixINCML) 	(fromReg _Rt) {-,-} ({-sem-}(delta `bits` [24..31])) {-,-} ({-sem-}(delta `bits` [16..23])) {-hide-}Nothing {-hide-},
					MMixInstr (MMixGOi) 	(fromReg _Rt) {-,-} (fromReg _Rt) {-,-} (12) {-hide-}Nothing ]
  | (delta < 2^48)	= {-sem-}[	MMixInstr (MMixGETA) 	(fromReg _Rt) {-,-} ({-sem-}(delta `bits` [10..15])) {-,-} ({-sem-}(delta `bits` [2..9])) {-hide-}Nothing {-hide-},
		  			MMixInstr (MMixINCML) 	(fromReg _Rt) {-,-} ({-sem-}(delta `bits` [24..31])) {-,-} ({-sem-}(delta `bits` [16..23])) {-hide-}Nothing {-hide-},
		  			MMixInstr (MMixINCMH) 	(fromReg _Rt) {-,-} ({-sem-}(delta `bits` [40..47])) {-,-} ({-sem-}(delta `bits` [32..39])) {-hide-}Nothing {-hide-},
					MMixInstr (MMixGOi) 	(fromReg _Rt) {-,-} (fromReg _Rt) {-,-} (16) {-hide-}Nothing ]
  | otherwise		= {-sem-}[	MMixInstr (MMixGETA) 	(fromReg _Rt) {-,-} ({-sem-}(delta `bits` [10..15])) {-,-} ({-sem-}(delta `bits` [2..9])) {-hide-}Nothing {-hide-},
		  			MMixInstr (MMixINCML) 	(fromReg _Rt) {-,-} ({-sem-}(delta `bits` [24..31])) {-,-} ({-sem-}(delta `bits` [16..23])) {-hide-}Nothing {-hide-},
		  			MMixInstr (MMixINCMH) 	(fromReg _Rt) {-,-} ({-sem-}(delta `bits` [40..47])) {-,-} ({-sem-}(delta `bits` [32..39])) {-hide-}Nothing {-hide-},
		  			MMixInstr (MMixINCH) 	(fromReg _Rt) {-,-} ({-sem-}(delta `bits` [56..63])) {-,-} ({-sem-}(delta `bits` [48..55])) {-hide-}Nothing {-hide-},
					MMixInstr (MMixGOi) 	(fromReg _Rt) {-,-} (fromReg _Rt) {-,-} (20) {-hide-}Nothing ]
\end{code}

Armed with the above atom compiler, we are now ready to proceed with the translation of actual Etude functions. As already mentioned at the beginning of this section, every Etude construct of the form “"FUN (upsilon_set) {-DOT-} (tau)"” materialises in the eventual MMIX program as a collection of one or more MMIX instructions, which are generated from the function's body using the term compiler "compileTail" that will be described shortly. In particular, at the beginning of every such entity, the supplied substitution function "_Sigma" is updated with bindings of all Etude variables "upsilon_set" to their respective MMIX analogues found in the standard parameter list "nu_set_P" as defined earlier in Section \ref{MMIX/Lambda/Registers}. Formally:

\BeginHaskellToken	{compileFunction}			{Open}	{\ZapfChancery{C}\!\sub{«\lambda»}}
  \HaskellContext							{(Ord upsilon)}
  \HaskellArgument							{(MapT (upsilon) (MMixVarT) {->-}, FunctionT upsilon)}
  \HaskellResult							{[MMixInstrT]}
\EndHaskellToken

\IndexTerm		{compileFunction}				{C_\lambda function (MMIX_instrs)}

\begin{code}
compileFunction {-[*]-} :: (Ord upsilon) => (MapT (upsilon) (MMixVarT) {->-}, FunctionT upsilon) -> [MMixInstrT]

compileFunction {-sem-}(_Sigma {->-}, FUN (upsilon_set) {-DOT-} (tau)) = compileTail{--}(_Sigma // listToMap (upsilon_set `zipList` nu_set_P) {->-}, tau)
\end{code}

In the following translation, a term which appears outside of any “"LET"” bindings is known as a «tail term». In the eventual MMIX program, the concrete rendition of all such terms is obtained by the following Haskell function "compileTail":

\BeginHaskellToken	{compileTail}				{Open}	{\ZapfChancery{C}\!\sub{«\kappa»}}
  \HaskellContext							{(Ord upsilon)}
  \HaskellArgument							{(MapT (upsilon) (MMixVarT) {->-}, TermT upsilon)}
  \HaskellResult							{[MMixInstrT]}
\EndHaskellToken

\IndexTerm		{compileTail}					{C_\kappa term (MMIX_instrs)}

\begin{code}
compileTail {-[*]-} :: (Ord upsilon) => (MapT (upsilon) (MMixVarT) {->-}, TermT upsilon) -> [MMixInstrT]
\end{code}

In particular, every tail term ends in a jump to the function's continuation address, which, by construction, is always made available to the program as a value of the designated variable "_Rk". Under the following translation, such jumps are always depicted by the MMIX instruction \hbox{“"MMixInstr (MMixGOi) (R _X) {-,-} (R _Y) {-,-} (0) {-hide-}Nothing"”,} which, as described earlier in Section \ref{MMIX/Semantics}, transfers control to the location within the program's address space specified by the general purpose register "R _Y", after saving the address of the following instruction in "R _X". Since, in tail terms, this later address is of no use to the function's caller, we will always place it in the scratch variable "_Rt", whereby it will be promptly discarded by some following instruction which finds a better use for that register.

The actual results of the entire Etude function are always placed in the successive registers "R 0", "R 1", "R 2" and so on, as represented by the MMIX variable list "nu_set_P". Accordingly, a simple tail term of the form “"RET (alpha_set)"” is compiled directly into an MMIX representation of the supplied atoms "alpha_set", whose respective values are placed into the corresponding variables from "nu_set_P". On the other hand, the computational effect of all “"NEW"”, “"DEL"”, “"GET"”, “"SET"” and “"INI"” term forms must be obtained using another algorithm "compileTerm" that is defined later in this section. Their individual translations are then completed with the same operation \hbox{“"MMixInstr (MMixGOi) (fromReg _Rt) {-,-} (fromReg _Rk) {-,-} (0) {-hide-}Nothing"”}. In Haskell:

\begin{code}
compileTail {-sem-}(_Sigma {->-}, RET (alpha_set)) ---
			= compileTerm{-sem-}(_Sigma, emptySet, takeFromList{--}(lengthOfList{--}(alpha_set), nu_set_P) {-,,-} {->-}, {-,,-} RET (alpha_set)) ++ --
			  {-sem-}[ MMixInstr (MMixGOi) (fromReg _Rt) {-,-} ({--}fromReg _Rk) {-,-} ({--}0) {-hide-}Nothing ] {-!-}{-!-}{-!-}{-!-}{-!-}{-!-}
compileTail {-sem-}(_Sigma {->-}, NEW (xi_set)) ---
			= compileTerm{-sem-}(_Sigma, emptySet, {-raw-}[R 0] 	{->-}, NEW  (xi_set)) ++ --
			  {-sem-}[ MMixInstr (MMixGOi) (fromReg _Rt) {-,-} (fromReg _Rk) {-,-} (0) {-hide-}Nothing ]
compileTail {-sem-}(_Sigma {->-}, DEL (xi_set)) ---
			= compileTerm{-sem-}(_Sigma, emptySet, {-empty-}[]	{->-}, DEL  (xi_set)) ++ --
			  {-sem-}[ MMixInstr (MMixGOi) (fromReg _Rt) {-,-} (fromReg _Rk) {-,-} (0) {-hide-}Nothing ]
compileTail {-sem-}(_Sigma {->-}, GET (alpha {-hv-}, mu_set) (phi)) ---
			= compileTerm{-sem-}(_Sigma, emptySet, {-raw-}[R 0]	{->-}, GET (alpha {-hv-}, mu_set) (phi)) ++ --
			  {-sem-}[ MMixInstr (MMixGOi) (fromReg _Rt) {-,-} (fromReg _Rk) {-,-} (0) {-hide-}Nothing ]
compileTail {-sem-}(_Sigma {->-}, SET (alpha1 {-hv-}, mu_set) (phi) {-TO-} (alpha2)) ---
			= compileTerm{-sem-}(_Sigma, emptySet, {-empty-}[]	{->-}, SET (alpha1 {-hv-}, mu_set) (phi) {-TO-} (alpha2)) ++ --
			  {-sem-}[ MMixInstr (MMixGOi) (fromReg _Rt) {-,-} (fromReg _Rk) {-,-} (0) {-hide-}Nothing ]
compileTail {-sem-}(_Sigma {->-}, INI (alpha1 {-hv-}, mu_set) (phi) {-TO-} (alpha2)) ---
			= compileTerm{-sem-}(_Sigma, emptySet, {-empty-}[]	{->-}, INI (alpha1 {-hv-}, mu_set) (phi) {-TO-} (alpha2)) ++ --
			  {-sem-}[ MMixInstr (MMixGOi) (fromReg _Rt) {-,-} (fromReg _Rk) {-,-} (0) {-hide-}Nothing ]
\end{code}

The core meaning of an Etude tail call operation “"APP (alpha) (alpha_set)"” is likewise represented by a similar “"MMixGOi"\!” instruction. However, in this case, the targeted Etude function is to be found at a memory location depicted by the construct's initial operand "alpha". Accordingly, we begin by compiling all of the supplied atoms and allocating them to the successive variables form "nu_set_P", where the eventual program expects to find their respective values. Next, we ensure that the actual address of the target instruction has been placed in a real hardware register (say, "R _R"), after which the “"MMixGOi"\!” instruction can be applied directly to that register as follows:

\begin{code}
compileTail {-sem-}(_Sigma {->-}, APP (alpha) (alpha_set)) ---
				= 	compileAtoms{--}(_Sigma, emptySet, nu_set_R {->-}, alpha_set ++ [alpha]) ++ --
				  	compileCopy{--}(R _R {->-}, last{--}(nu_set_R)) ++ --
				  	{-sem-}[MMixInstr (MMixGOi) (fromReg _Rt) {-,-} (fromReg (R _R)) {-,-} (0) {-hide-}Nothing ]
 where	{-sem-}(R _R)		 = allocReg{--}(_Rt {->-}, last{--}(nu_set_R))
	{-raw-}(nu_set_R)	 = takeFromList{--}(lengthOfList{--}(alpha_set) + 1, nu_set_P)
\end{code}

A suitable MMIX rendition of a system call operation “"SYS (pi) (alpha_set)"” is obtained similarly, except that, instead of an actual memory address, the jump materialises as an MMIX “\,"MMixTRAP"\,” instruction, whose three operands "_X", "_Y" and "_Z" are used to encode the 24-bit system call number "pi". Once control has been relinquished by the operating system, the program proceeds with a jump to the prevailing continuation address "_Rk":

\begin{code}
compileTail {-sem-}(_Sigma {->-}, SYS (pi) (alpha_set)) ---
				= 	compileAtoms{--}(_Sigma, emptySet, takeFromList{--}(lengthOfList{--}(alpha_set), nu_set_P) {->-}, alpha_set) ++
				  	{-sem-}[MMixInstr (MMixTRAP) ({-sem-}(pi `bits` [16..23])) {-,-} --
				  					({-sem-}(pi `bits` [8..15])) {-,-} ({-sem-}(pi `bits` [0..7])) {-hide-}Nothing {-sc-},
			  		MMixInstr (MMixGOi) (fromReg _Rt) {-,-} (fromReg _Rk) {-,-} (0) {-hide-}Nothing]
\end{code}

On the other hand, every conditional Etude term of the form “"IFF (alpha) {-THEN-} (tau1) {-ELSE-} (tau2)"\!” must choose between one of the two available targets "tau1" or "tau2". Either way, both of these terms are compiled into their respective instruction sequences "iota_set1" and "iota_set2". If the first of these sequences is short enough (specifically, if it contains fewer than "2^16" instructions), then the branch target can be encoded verbatim within the "_Y" and "_Z" operands of an MMIX “\,"MMixBZ"\,” operation. Otherwise, the desired target address must be synthesised indirectly, which, in the following translation, is performed by the Haskell function "compileJump". Formally:

\begin{code}
compileTail {-sem-}(_Sigma {->-}, IFF (alpha) {-THEN-} (tau1) {-ELSE-} (tau2)) ---
  | (delta < 2^18) ---
  	 =	compileAtom{--}(_Sigma, nu_set_K, R _R {->-}, alpha) ++
		{-sem-}[MMixInstr (MMixBZ) (fromReg (R _R)) {-,-} ({-sem-}(delta `bits` [10..17])) {-,-} ({-sem-}(delta `bits` [2..9])) {-hide-}Nothing ] ++
		iota_set1 ++ --
		iota_set2
  | otherwise ---
  	 =	compileAtom{--}(_Sigma, nu_set_K, R _R {->-}, alpha) ++
		{-sem-}[MMixInstr (MMixBZ) (fromReg (R _R)) {-,-} (0) {-,-} (3) {-hide-}Nothing] ++
		compileJump{--}(4 #*# (lengthOfList{--}(iota_set2) + 1)) ++
  	 	iota_set2 ++ --
  	 	iota_set1
 where	{-sem-}(R _R)		= allocReg{--}(_Rt {->-}, allocAtom{--}(_Sigma, nu_set_K {->-}, alpha))
 	{-raw-}(nu_set_K)	= listToSet [ _Sigma#(upsilon_k) | upsilon_k <- setToList (fvOfTerm{--}(tau1) `union` fvOfTerm{--}(tau2)) ]
	{-raw-}(iota_set1)	= compileTail{--}(_Sigma {->-}, tau1)
	{-raw-}(iota_set2)	= compileTail{--}(_Sigma {->-}, tau2)
	{-raw-}(delta)		= 4 #*# (lengthOfList{--}(iota_set1) + 1)
\end{code}

In order to compile a jump to such a remote address that is found "n" bytes away from its source, "compileJump{--}(n)" begins by obtaining the location of the current instruction, which, on MMIX, can be achieved easily enough using the “"MMixGETA"” opcode. Conveniently, “"MMixGETA"” also allows us to increment that address by the 16 least significant bits of "n". The remaining bits must be added to the resulting value using the familiar sequence of “"MMixINCML"”, “"MMixINCMH"\,” and “"MMixINCH"\,” operations. Once the entire address has been prepared in "_Rt", the program proceeds with the unconditional control transfer to that location, enacted in the usual manner by an appropriate “"MMixGOi"\!” instruction. The "_Z" operand of “"MMixGOi"\!”, which, in the semantic translation from Section \ref{MMIX/Semantics}, was always added to the supplied target register value, is used to perform the final adjustment of the synthesised address by the number of instructions introduced internally \hbox{within "compileJump" itself:}

\IndexTerm		{compileJump}					{C_J integer (MMIX_instrs)}
\InsertHaskell		{compileJump}

Finally, a tail term of the form “"LET [{-raw-}(upsilon_set {-=-}, tau1)] (tau2)"\!” is depicted by a sequence of MMIX instructions that represent the inner Etude term "tau1", followed by the rendition of the tail's body "tau2". Any results of "tau1" are bound to fresh MMIX variables which otherwise cannot appear free within "tau2". Predictably, during compilation of the actual “"LET"” body "tau2", all of these variables are substituted for their respective Etude equivalents from "upsilon_set", which, in Haskell, can be represented concisely as follows:

\begin{code}
compileTail {-sem-}(_Sigma {->-}, LET [{-raw-}(upsilon_set {-=-}, tau1)] (tau2)) ---
				= compileTerm{--}(_Sigma, nu_set_K, takeFromList{--}(lengthOfList{--}(upsilon_set), nu_set) {->-}, tau1) ++ --
				  compileTail{--}(_Sigma // listToMap ((upsilon_set `zipList` nu_set)) {->-}, tau2)
 where 	nu_set_K		= listToSet [ _Sigma#(upsilon_k) | upsilon_k <- setToList (fvOfTerm{--}(tau2) `difference` listToSet upsilon_set) ]
 	nu_set			= [ nu_k | nu_k <- nu_set_G, nu_k /@ nu_set_K ]
\end{code}

Last but not least, groups of “"LET"” bindings are compiled naturally through their standard semantic translation into monadic sequences of singular binding forms:

\begin{code}
compileTail {-sem-}(_Sigma {->-}, LET [] (tau))		= compileTail {-sem-}(_Sigma {->-}, tau)
compileTail {-sem-}(_Sigma {->-}, LET (beta_set) (tau))	= compileTail {-sem-}(_Sigma {->-}, LET [{-sem-}(head{--}(beta_set))] (LET ({-sem-}(tail{--}(beta_set))) (tau)))
\end{code}

Within a “"LET"” binding, the above compilation algorithm must be adjusted slightly, in order to ensure that all Etude variables that are referenced in the remainder of the program retain their original semantic significance under the translation, and that the actual results delivered by the term are saved in the MMIX equivalents of the Etude variables to which these results have been assigned by the programmer. To this end, the translation of such terms in enacted by one final algorithm "compileTerm" in our compiler:

\BeginHaskellToken	{compileTerm}				{Open}	{\ZapfChancery{C}\!\sub{«\tau»}}
  \HaskellContext							{(Ord upsilon)}
  \HaskellArgument							{(MapT (upsilon) (MMixVarT), SetT MMixVarT, [MMixVarT] {->-}, TermT upsilon)}
  \HaskellResult							{[MMixInstrT]}
\EndHaskellToken

\IndexTerm		{compileTerm}					{C_\tau term (MMIX_instrs)}

\begin{code}
compileTerm {-[*]-} :: (Ord upsilon) => (MapT (upsilon) (MMixVarT), SetT MMixVarT, [MMixVarT] {->-}, TermT upsilon) -> [MMixInstrT]
\end{code}

In the present context, a trivial “"RET (alpha_set)"” term simply compiles its atoms into the supplied list of MMIX variables "nu_set_R" as follows:

\begin{code}
compileTerm {-sem-}(_Sigma, nu_set_K, nu_set_R {->-}, RET (alpha_set)) ---
				=	compileAtoms{--}(_Sigma, nu_set_K, nu_set_R {->-}, alpha_set)
\end{code}

Further, an application term of the form “"APP (alpha) (alpha_set)"” proceeds similarly to its earlier tail variant, except that, prior to evaluating any of the Etude atoms found within the construct's syntax, the continuation variable "_Rk" and any other values "nu_set_K" that remain live somewhere in the remainder of the program must be placed into some temporary storage facilities, since any of these variables may be clobbered by the targeted function. As expected, the “"MMixGOi"\!” instruction introduced by the term is followed with a binding operation "compileCopies", which copies any delivered results to the respective registers from "nu_set_R". Once this has been achieved, any preserved variables are promptly restored to their original values. Formally:

\begin{code}
compileTerm {-sem-}(_Sigma, nu_set_K, nu_set_R {->-}, APP (alpha) (alpha_set)) ---
				= 	saveRegs{--}[ R _K | R _K <- setToList (nu_set_K `union` listToSet [_Rk]) ] ++
					compileAtoms{--}(_Sigma, emptySet, nu_set_A {->-}, alpha_set ++ [alpha]) ++
					compileCopy{--}(R _R {->-}, last{--}(nu_set_A)) ++
					{-sem-}[MMixInstr (MMixGOi) (fromReg _Rk) {-,-} (fromReg (R _R)) {-,-} (0) {-hide-}Nothing ] ++
				  	compileCopies{--}(nu_set_R {->-}, takeFromList{--}(lengthOfList{--}(nu_set_R), nu_set_P)) ++
				  	restoreRegs{--}[ R _K | R _K <- setToList (nu_set_K `union` listToSet [_Rk]) ]

 where	{-sem-}(R _R)		 = allocReg{--}(_Rt {->-}, last{--}(nu_set_A))
	{-raw-}(nu_set_A)	 = takeFromList{--}(lengthOfList{--}(alpha_set) + 1, nu_set_P)
\end{code}

A convenient scratch space can be found for storage of any variables during an execution of a nested MMIX function call in the form of the data stack "stackOfOEnv{--}(_Delta)". As already mentioned earlier in Section \ref{MMIX/Lambda/Registers}, throughout the entire execution of all MMIX programs produced by our compiler, the current value of the stack pointer "sigma_c" can be found in the designated register "_Rs". Recalling from Section \ref{Lambda/Arch/State} that, on the present instruction set architecture, the data stack always grows downwards towards lower address values and that, under the MMIX semantics of Etude atoms from Section \ref{Lambda/Arch/Atoms}, the value of every register is always representable under the 64-bit natural format “"NG`FMT`64"”, the process of pushing the values of an arbitrary MMIX variable set "nu_set" onto the data stack can be modelled by the following recursive algorithm:

\IndexTerm		{saveRegs}					{save MMIX_vars (MMIX_instrs)}
\InsertHaskell		{saveRegs}

A dual operation, which reverts these registers to their original values and restores the earlier configuration of the stack pointer "sigma_c", can be also implemented without further difficulties. The reader should observe, however, that during restoration of variables from the data stack, their values must be retrieved in the reverse order of these variables' earlier allocation by “"saveRegs"”, so that the following algorithm “"restoreRegs"” processes the tail of the supplied register list before proceeding with restoration of the initial entry \hbox{in that sequence:}

\IndexTerm		{restoreRegs}					{restore MMIX_vars (MMIX_instrs)}
\InsertHaskell		{restoreRegs}

Similarly to application expressions, in the context of a nested term form, an MMIX rendition of an Etude system call operation must be likewise surrounded in an appropriate pair of “"saveRegs"” and “"restoreRegs"” constructs:

\begin{code}
compileTerm {-sem-}(_Sigma, nu_set_K, nu_set_R {->-}, SYS (pi) (alpha_set)) ---
				= 	saveRegs{--}[ R _K | R _K <- setToList nu_set_K ] ++
					compileAtoms{--}(_Sigma, emptySet, nu_set_P {->-}, alpha_set) ++
					{-sem-}[MMixInstr (MMixTRAP) ({-sem-}(pi `bits` [16..23])) {-,-} --
									({-sem-}(pi `bits` [8..15])) {-,-} ({-sem-}(pi `bits` [0..7])) {-hide-}Nothing ] ++
				  	compileCopies{--}(nu_set_R {->-}, takeFromList{--}(lengthOfList{--}(nu_set_R), nu_set_P)) ++
				  	restoreRegs{--}[ R _K | R _K <- setToList nu_set_K ]
\end{code}

Further, all conditional terms materialise in the resulting MMIX program almost identically to their earlier tail variants, except that, in the present scenario, we must ensure that execution proceeds at the following address within the surrounding function. To this end, one of the instruction sequences "iota_set1" and "iota_set2" derived from the respective nested terms "tau1" and "tau2" must be always amended with an additional jump past the entire block generated from the complete construct:

\begin{code}
compileTerm {-sem-}(_Sigma, nu_set_K, nu_set_R {->-}, IFF (alpha) {-THEN-} (tau1) {-ELSE-} (tau2))

	| (delta < 2^18) ---
  		=	compileAtom{--}(_Sigma, nu_set_K', R _R {->-}, alpha) ++
			{-sem-}[MMixInstr (MMixBZ) (fromReg (R _R)) {-,-} ({-sem-}(delta `bits` [10..17])) {-,-} ({-sem-}(delta `bits` [2..9])) {-hide-}Nothing ] ++
			iota_set1 ++ --
			iota_set1' ++ --
			iota_set2

	| otherwise ---
	  	 =	compileAtom{--}(_Sigma, nu_set_K', R _R {->-}, alpha) ++
			{-sem-}[MMixInstr (MMixBZ) (fromReg (R _R)) {-,-} (0) {-,-} (3) {-hide-}Nothing] ++
			compileJump{--}(4 #*# (lengthOfList{--}(iota_set2) + lengthOfList{--}(iota_set2') + 1)) ++
  		 	iota_set2 ++ --
  		 	iota_set2' ++ --
  		 	iota_set1

 where	{-sem-}(R _R)		= 	allocReg{--}(_Rt {->-}, allocAtom{--}(_Sigma, nu_set_K' {->-}, alpha))
 	{-raw-}(nu_set_K')	= 	nu_set_K `union` listToSet [ _Sigma#(upsilon_k) | upsilon_k <- setToList (fvOfTerm{--}(tau1) `union` fvOfTerm{--}(tau2)) ]
	{-raw-}(iota_set1)	= 	compileTerm{--}(_Sigma, nu_set_K, nu_set_R {->-}, tau1)
	{-raw-}(iota_set1')	= 	compileJump{--}(4 #*# (lengthOfList{--}(iota_set2) + 1))
	{-raw-}(iota_set2)	= 	compileTerm{--}(_Sigma, nu_set_K, nu_set_R {->-}, tau2)
	{-raw-}(iota_set2')	= 	compileJump{--}(4 #*# (lengthOfList{--}(iota_set1) + 1))
	{-raw-}(delta)		= 	4 #*# (lengthOfList{--}(iota_set1) + lengthOfList{--}(iota_set1') + 1)
\end{code}

Next, we must tackle the two object environment operations “"NEW (xi_set)"” and “"DEL (xi_set)"”. As described earlier in Section \ref{Lambda/Arch/State}, the entire effect of these term forms can be captured by rounding the size of the specified envelope "xi_set" to the nearest multiple of 8 and subtracting or adding the resulting integer to the existing value of the stack pointer "sigma_c" which, in the translated program, is depicted simply by the general purpose hardware register "_Rs". Accordingly:

\begin{code}
compileTerm {-sem-}(_Sigma, nu_set_K, {-raw-}[nu_R] {->-}, NEW (xi_set)) ---
				=	compileAtom{-sem-}(_Sigma, nu_set_K, _Rt {->-}, IMM (toRational {-sem-}(ceiling (sizeOfEnvelope{--}(xi_set) % 8) * 8)) (NG`FMT`64)) ++
					{-sem-}[ MMixInstr (MMixSUBU) 	(fromReg _Rs) {-,-} (fromReg _Rs) {-,-} (fromReg _Rt) {-hide-}Nothing ] ++
					compileCopy{--}(nu_R {->-}, _Rs)
compileTerm {-sem-}(_Sigma, nu_set_K, {-empty-}[] {->-}, DEL (xi_set)) ---
				=	compileAtom{-sem-}(_Sigma, nu_set_K, _Rt {->-}, IMM (toRational {-sem-}(ceiling (sizeOfEnvelope{--}(xi_set) % 8) * 8)) (NG`FMT`64)) ++
					{-sem-}[ MMixInstr (MMixADDU) 	(fromReg _Rs) {-,-} (fromReg _Rs) {-,-} (fromReg _Rt) {-hide-}Nothing ]
\end{code}

On the other hand, every memory inspection operation “"GET (alpha {-hv-}, mu_set) (phi)"” is translated into yet another common structure, with each variant of this term form differing only in the precise choice of an MMIX opcode, selected in accordance with the specified Etude format "phi" to perform the actual movement of data from the program's memory image. In particular, the three distinguished natural formats “"NG`FMT`8"”, “"NG`FMT`16"” and “"NG`FMT`32"” are implemented directly by the MMIX instruction forms “\,"MMixLDBUi"\!”, “\,"MMixLDWUi"\!” and “\,"MMixLDTUi"\!”, while their signed integer cousins “"ZG`FMT`8"”, “"ZG`FMT`16"” and “"ZG`FMT`32"” correspond to “\,"MMixLDBi"\!”, “\,"MMixLDWi"\!” and “\,"MMixLDTi"\!”, respectively. In every other case, the object's content is fetched invariably using the 64-bit load operation “\,"MMixLDOUi"\!”:

\begin{code}
compileTerm {-sem-}(_Sigma, nu_set_K, {-raw-}[nu_R] {->-}, GET (alpha {-hv-}, mu_set) (NG`FMT`8)) 	= compileLoad{--}(_Sigma, nu_set_K, nu_R, MMixLDBUi 	{->-}, alpha)
compileTerm {-sem-}(_Sigma, nu_set_K, {-raw-}[nu_R] {->-}, GET (alpha {-hv-}, mu_set) (NG`FMT`16)) 	= compileLoad{--}(_Sigma, nu_set_K, nu_R, MMixLDWUi 	{->-}, alpha)
compileTerm {-sem-}(_Sigma, nu_set_K, {-raw-}[nu_R] {->-}, GET (alpha {-hv-}, mu_set) (NG`FMT`32)) 	= compileLoad{--}(_Sigma, nu_set_K, nu_R, MMixLDTUi 	{->-}, alpha)
compileTerm {-sem-}(_Sigma, nu_set_K, {-raw-}[nu_R] {->-}, GET (alpha {-hv-}, mu_set) (ZG`FMT`8))	= compileLoad{--}(_Sigma, nu_set_K, nu_R, MMixLDBi 	{->-}, alpha)
compileTerm {-sem-}(_Sigma, nu_set_K, {-raw-}[nu_R] {->-}, GET (alpha {-hv-}, mu_set) (ZG`FMT`16))	= compileLoad{--}(_Sigma, nu_set_K, nu_R, MMixLDWi 	{->-}, alpha)
compileTerm {-sem-}(_Sigma, nu_set_K, {-raw-}[nu_R] {->-}, GET (alpha {-hv-}, mu_set) (ZG`FMT`32))	= compileLoad{--}(_Sigma, nu_set_K, nu_R, MMixLDTi 	{->-}, alpha)
compileTerm {-sem-}(_Sigma, nu_set_K, {-raw-}[nu_R] {->-}, GET (alpha {-hv-}, mu_set) (phi)) 		= compileLoad{--}(_Sigma, nu_set_K, nu_R, MMixLDOUi	{->-}, alpha)
\end{code}

All such load operations are depicted in the translated program by a sequence of MMIX instructions which beings with the compiled representation of the supplied address atom "alpha", followed by the actual load operation “"MMixInstr (_OP) (R _R) {-,-} (R _A) {-,-} (0)"” and ends in a copy operation that moves the fetched value "R _R" into its rightful place "nu_R":

\IndexTerm		{compileLoad}					{C_LD atom MMIX_opcode MMIX_var MMIX_vars (MMIX_instrs)}
\InsertHaskell		{compileLoad}

The partial MMIX instruction set that was formalised in Section \ref{MMIX/Semantics} provides only four unsigned variants of the dual object update operations “"MMixSTBUi"\!”, “"MMixSTWUi"\!”, “"MMixSTTUi"\!” and “"MMixSTOUi"\!”. Accordingly, all updates to the content of a signed memory-resident object must be first replaced with a semantically equivalent unsigned term form, observing that, either way, MMIX discards any insignificant bits of the object's value, so that both Etude formats result in identical program behaviours. Once this adjustment has been enacted, the compiler proceeds with the term's translation as follows:

\begin{code}
compileTerm {-sem-}(_Sigma, nu_set_K, {-empty-}[] {->-}, SET (alpha1 {-hv-}, mu_set) (ZG`FMT`epsilon) 	{-TO-} (alpha2)) ---
	= compileTerm {-sem-}(_Sigma, nu_set_K, {-empty-}[] {->-}, SET (alpha1 {-hv-}, mu_set) (NG`FMT`epsilon) {-TO-} (alpha2))
compileTerm {-sem-}(_Sigma, nu_set_K, {-empty-}[] {->-}, SET (alpha1 {-hv-}, mu_set) (NG`FMT`8) 	{-TO-} (alpha2)) --
	= compileStore{--}(_Sigma, nu_set_K, MMixSTBUi  {->-}, alpha1, alpha2)
compileTerm {-sem-}(_Sigma, nu_set_K, {-empty-}[] {->-}, SET (alpha1 {-hv-}, mu_set) (NG`FMT`16) 	{-TO-} (alpha2)) --
	= compileStore{--}(_Sigma, nu_set_K, MMixSTWUi  {->-}, alpha1, alpha2)
compileTerm {-sem-}(_Sigma, nu_set_K, {-empty-}[] {->-}, SET (alpha1 {-hv-}, mu_set) (NG`FMT`32) 	{-TO-} (alpha2)) --
	= compileStore{--}(_Sigma, nu_set_K, MMixSTTUi  {->-}, alpha1, alpha2)
compileTerm {-sem-}(_Sigma, nu_set_K, {-empty-}[] {->-}, SET (alpha1 {-hv-}, mu_set) (phi)	 	{-TO-} (alpha2)) --
	= compileStore{--}(_Sigma, nu_set_K, MMixSTOUi  {->-}, alpha1, alpha2)
\end{code}

Further, the reader should recall that, on MMIX, every “"INI"” variant of an object update operation assumes a semantic meaning that is indistinguishable from the corresponding “"SET"” form of the term:

\begin{code}
compileTerm {-sem-}(_Sigma, nu_set_K, nu_set_R {->-}, INI (alpha1 {-hv-}, mu_set) (phi) {-TO-} (alpha2)) --
	= compileTerm {-sem-}(_Sigma, nu_set_K, nu_set_R {->-}, SET (alpha1 {-hv-}, mu_set) (phi) {-TO-} (alpha2))
\end{code}

In all cases, the resulting instruction sequence is prepared from the “"SET"” operation in the following manner:

\IndexTerm		{compileStore}					{C_ST atom atom MMIX_opcode MMIX_var MMIX_vars (MMIX_instrs)}
\InsertHaskell		{compileStore}

Finally, in the context of a nested Etude term, a binding “"LET"” construct of the form \hbox{“"LET [{-raw-}(upsilon_set {-=-}, tau1)] (tau2)"\!”} is compiled in an essentially the same way as its earlier tail variant, except that, during translation of "tau1", the set of preserved variables "nu_set_K" is extended with any entities that are found free within the following body term "tau2". Formally:

\begin{code}
compileTerm {-sem-}(_Sigma, nu_set_K, nu_set_R {->-}, LET [{-raw-}(upsilon_set {-=-}, tau1)] (tau2)) ---
				= 	compileTerm{--}(_Sigma, nu_set_K', takeFromList{--}(lengthOfList{--}(upsilon_set), nu_set) {->-}, tau1) ++
				  	compileTerm{--}(_Sigma // listToMap ((upsilon_set `zipList` nu_set)), nu_set_K, nu_set_R {->-}, tau2)

 where 	nu_set_K'	= nu_set_K `union` listToSet [ _Sigma#(upsilon_k) | upsilon_k <- setToList (fvOfTerm{--}(tau2) `difference` listToSet upsilon_set) ]
 	nu_set		= [ nu_k | nu_k <- nu_set_G, nu_k /@ nu_set_K' ]

compileTerm {-sem-}(_Sigma, nu_set_K, nu_set_R {->-}, LET [] (tau)) ---
  = compileTerm {-sem-}(_Sigma, nu_set_K, nu_set_R {->-}, tau)
compileTerm {-sem-}(_Sigma, nu_set_K, nu_set_R {->-}, LET (beta_set) (tau)) ---
  = compileTerm {-sem-}(_Sigma, nu_set_K, nu_set_R {->-}, LET [{-sem-}(head{--}(beta_set))] (LET ({-sem-}(tail{--}(beta_set))) (tau)))
\end{code}

\set{title}		{Preservation of term semantics under compilation}
\begin{theorem}

Let "_Sigma" be a variable renaming function and "_Gamma" represent the mapping of all relevant MMIX variables to their respective atomic values. Let "tau" be an Etude term that is closed and well-formed under the substitution by "_Gamma . _Sigma". Further, let "_Lambda'" represent a function environment updated with bindings of "n" successive instruction addresses that begin at some Etude function variable "nu" to the respective "n" instructions generated from the compilation of "tau" and let "_Delta'" represent an object environment that is likewise updated with bindings of any symbolic addresses "LOC`SR`8{--}k" to the current values of all pseudo-registers "R k" which appear free within "tau". Then, taken in the context of "_Lambda" and "_Delta", a term of the form “"LET (upsilon_set =: tau) {-sem-}(APP (_Rk) (pregs) `substAtom` _Gamma)"” is equivalent to the function application term “"APP (nu) (pregs)"” under substitution by "_Gamma". Formally:

\NewHaskellToken	{termPreservationJ}			{Ord}	{\TKeyword{LC\sub{«\tau»}}}
\NewHaskellDescription	{termPreservationJ}				{Preservation of term semantics under compilation}
\IndexTheorem		{termPreservationJ}				{LC_\alpha Preservation of term semantics under compilation}

\begin{haskell}
termPreservationJ :: forall _Sigma, _Gamma, _Gamma', _Lambda, _Lambda', _Delta, _Delta', nu, upsilon, alpha, nu_set_K, upsilon =>
	WfJ{--}(_Lambda) -> WfJ{--}(_Delta) -> WfJ{--}(tau `substAtom` (_Gamma . _Sigma)) ->
	{-sem-}(_Lambda'  = _Lambda 	// 		meaningsOfMMixInstr{--}(_Gamma, nu {->-}, --
								compileTerm{--}(_Sigma, nu_set_K, [ _Sigma#(upsilon_k) | upsilon_k <- upsilon_set ] {->-}, alpha))) ->
	{-sem-}(_Delta'   = _Delta 	`setInOEnv` 	listToMap [ (_Gamma#(LOC`SR`8{--}k) {-,,-}{-:-},{-,,-} _Gamma#(R k)) --
									| {-sem-}(R k) <- [ _Sigma#(upsilon_k) | upsilon_k <- fvOfTerm{--}(tau) ], k >= 256 ]) ->
	{-sem-}(_Lambda, _Delta {->-}, {-sem-}(LET (upsilon_set =: tau) {-sem-}(APP (_Rk) (pregs) `substAtom` _Gamma))) `EqvJ` --
	{-sem-}(_Lambda', _Delta' {->-}, {-sem-}(APP ({-sem-}(_Gamma#(_Sigma#(nu)))) (pregs)) `substAtom` _Gamma)
\end{haskell}
\begin{proof}

For all “"RET"”, “"NEW"”, “"DEL"”, “"GET"”, “"SET"” and “"INI"” term forms of "tau", “"LET (upsilon_set =: tau) (APP (_Rk) (pregs)"” is always reducible under "evalTerm" into a direct call to "_Rk", provided that all variables found in both "upsilon_set" and "pregs" are replaced in "_Rk"'s argument list with their actual atomic values delivered by "tau", since none of these term forms ever affects the evaluation state. From theorems "copyPreservationJ" and "atomPreservationJ", we also have an equivalence between such calls and their compiled representations, observing that the MMIX rendition of every such term is precisely identical to that obtained from the delivered atomic values. Accordingly, the desired equivalence relation “"LET (upsilon_set =: tau) (APP (_Rk) (pregs)) `EqvJ` APP (nu) (pregs)"” holds as required for all of these term forms.

If "tau" represents a function application term, then both “"LET (upsilon_set =: tau) (APP (_Rk) (pregs)"” and “"APP (nu) (pregs)"” are compiled into identical MMIX instruction sequences, save for an occasional copy operation that can be amortised into "_Gamma" using the earlier equivalence result provided by theorem "copyPreservationJ". Accordingly, for all such term forms, theorem "termPreservationJ" holds trivially by the virtue of a definitional equality between the respective function environments.

Finally, the induction step, which pertains to the preservation of nested “"LET"” bindings, follows directly from the equivalence between \hbox{“"LET [{-raw-}(upsilon_set2 {-=-}, {-hv-}(LET [{-raw-}(upsilon_set1 {-=-}, tau1)] (tau2)))] (tau3)"\!”} and its flattened variant \hbox{“"LET [{-raw-}(upsilon_set1 {-=-}, tau1)] {-hv-}(LET [{-raw-}(upsilon_set2 {-=-}, tau2)] (tau3))"”.} Accordingly, we can begin by establishing theorem "termPreservationJ" inductively for the inner “"LET"” term and, subsequently, utilising the resulting equivalence in order to establish the result for the entire construct.

The remaining two cases, which pertain to the system calls and conditional “"IFF"” term forms, are no more difficult to scrutinise. However, a detailed analysis of their respective evaluation sequences is somewhat more elaborate, so that I leave the fulfilment of the associated proof obligations as an exercise for the reader. \QED

\end{proof}
\end{theorem}

With this last set of Haskell definitions, we have now completed our implementation of a verified compiler for the C programming language and can proceed directly with an implementation and subsequent verification of individual C programs, using their semantic model defined in the previous three chapters. While space and time constraints prohibited me from presenting the full details of many lesser aspects of the described compiler implementation, or from elaborating fully on the required proof obligations, these omissions should in no way distract from the magnitude of our present success.

\end{subsection}
\end{section}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
