\set{capture}{hidden}
\begin{code}

module MMIX.Syntax (
	MMixVarT (..),
	MMixVarsT,
	MMixModuleT (..),
	MMixSymbolsT,
	MMixTextT,
	MMixSectionT,
	MMixDataT,
	MMixInstrT (..),
	MMixOpcodeT (..),
	MMixSymbolT (..),
) where

import Additional
import Lambda.Syntax

instance Enum (MMixSymbolT) where
	succ (SYM n)													= SYM (n + 1)
	pred (SYM n) 		| (n > 1) 										= SYM (n - 1)
	toEnum (n) 		| (n >= 0) 										= SYM (toInteger n)
	fromEnum (SYM n) 	| (inRange (toInteger (minBound :: Int)) (<=) (n) (<=) (toInteger (maxBound :: Int)))	= fromInteger n
	enumFrom (SYM n) 												= [ SYM i | i <- [ n .. ] ]
	enumFromThen (SYM m) (SYM n) 											= [ SYM i | i <- [ m, n .. ] ]
	enumFromTo (SYM m) (SYM n) 											= [ SYM i | i <- [ m .. n ] ]
	enumFromThenTo (SYM m) (SYM n) (SYM o)										= [ SYM i | i <- [ m, n .. o ] ]

instance Enum (MMixVarT) where
	toEnum (n) 		| (inRange (0) (<=) (n) (<=) (255))							= R (toInteger n)
	fromEnum (R n) 													= fromInteger n

	succ (R n)		| (n < 255)										= R (n + 1)
	succ (SR sigma n)												= SR sigma (n + 1)

	pred (R n) 		| (n > 0) 										= R (n - 1)
	pred (SR sigma n) 	| (n > 0) 										= SR sigma (n - 1)

	enumFrom (R n) 													= [ R i | i <- [ n .. ] ]
	enumFrom (SR sigma n) 												= [ SR sigma i | i <- [ n .. ] ]

	enumFromThen (R m) (R n) 											= [ R i | i <- [ m, n .. ] ]
	enumFromThen (SR sigma m) (SR sigma' n) | sigma == sigma' 							= [ SR sigma i | i <- [ m, n .. ] ]

	enumFromTo (R m) (R n) 												= [ R i | i <- [ m .. n ] ]
	enumFromTo (SR sigma m) (SR sigma' n) | sigma == sigma'								= [ SR sigma i | i <- [ m .. n ] ]

	enumFromThenTo (R m) (R n) (R o)										= [ R i | i <- [ m, n .. o ] ]
	enumFromThenTo (SR sigma m) (SR sigma' n) (SR sigma'' o) | inRange (sigma) (==) (sigma') (==) (sigma'')		= [ SR sigma i | i <- [ m, n .. o ] ]

instance Show (MMixInstrT) where

	showsPrec _ (MMixInstr (_OP) (_X) {-,-} (_Y) {-,-} (_Z) {-@-} (Nothing))
		= showString "MMixInstr (" . shows _OP . 
		  showString ")\t(" . shows _X .
		  showString ") {-,-}\t(" . shows _Y .
		  showString ") {-,-}\t(" . shows _Z .
		  showString ") {-hide-}Nothing"
	showsPrec _ (MMixInstr (_OP) (_X) {-,-} (_Y) {-,-} (_Z) {-@-} (Just sigma))
		= showString "MMixInstr (" . shows _OP . 
		  showString ")\t(" . shows _X .
		  showString ") {-,-}\t(" . shows _Y .
		  showString ") {-,-}\t(" . shows _Z .
		  showString ") {-@-}\t(Just (" . shows sigma .
		  showString "))"

	showList [] 		= showString "\t{-sem-}[]\n"
	showList (iota_set) 	= showString "\t{-sem-}[\t" . showList' iota_set . showString "]\n"
	 where
	   showList' []			= id
	   showList' [iota]		= shows iota
	   showList' (iota:iota_set)	= shows iota . showString "{-hide-},\n\t\t" . showList' iota_set

infix 5 `SR`

\end{code}

\BeginHaskellToken	{{-sc-}}				{Void}	{}
  \HaskellToken		{,}					{Punct}	{\MKeyword{;}}
\EndHaskellToken

\NewHaskellToken	{MMixVarT}				{Ord}	{«MMIX-var»}
\NewHaskellResult	{MMixVarT}					{Type}

\NewHaskellToken	{MMixVarsT}				{Ord}	{«MMIX-vars»}
\NewHaskellResult	{MMixVarsT}					{Type}

\BeginHaskellToken	{R}					{Open}	{\MKeyword{\$}\WhiteSpace}
  \HaskellArgument							{Integer}
  \HaskellResult							{MMixVarT}
\EndHaskellToken

\NewMacro		{MMixRR}					{rR}
\NewHaskellToken	{RR}					{Ord}	{\MKeyword{\MMixRR}}
\NewHaskellResult	{RR}						{MMixVarT}

\BeginHaskellToken	{SR}					{Void}	{}
  \HaskellArgument							{MMixSymbolT}
  \HaskellToken		{{-.-}}					{Op}	{\WhiteSpace\MKeyword{.}\WhiteSpace}
  \HaskellArgument							{Integer}
  \HaskellResult							{MMixVarT}
\EndHaskellToken

\NewHaskellToken	{`SR`}					{Op}	{\hbox{\!\MKeyword{.}\!}}

\set{capture}		{MMixVarT}
\begin{code}
data MMixVarT =
	{-ord-} R (Integer)
	| RR
	| SR (MMixSymbolT) {-.-} (Integer) deriving (Eq, Ord, Show)
\end{code}

\set{capture}		{MMixVarsT}
\begin{code}
type MMixVarsT =
	[ MMixVarT ]
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{MMIX/Syntax}
\set{title}		{The MMIX Architecture}
\begin{section}

The MMIX instruction set architecture describes a simple yet capable von Neumann machine, in which both programs and data are stored within the same 64-bit virtual address space. The upper half of that address space, i.e., the set of all memory locations with negative address values, is reserved for use by the operating system and remains inaccessible to all ordinary programs of the kind scrutinised in this chapter. The remaining 63 bits of every virtual address are subdivided as follows:

\bigskip
\begin{centered}
\image{*}{*}{Images/Addr.pdf}
\end{centered}
\smallskip

\vskip -3pt

The 2-bit field "seg" selects one of four "2^61"-byte «segments» numbered 0, 1, 2 and 3, which, by convention, are dedicated to the program's code, data, heap and hardware-maintained data stack, respectively. The 61-bit "offset" field is used to specify the precise location of every memory-resident object stored within one of these segments. In particular, an \hbox{"n"-byte} object stored in bytes "o, o+1 .. o+n-1" of a given segment "s" can be found under the address of "2^61 {-!-} s + o". The MMIX architecture utilises the big-endian addressing convention, so that the eight most significant bits of such an object are always placed at the address "2^61 {-!-} s + o".

Besides this «random-access memory», MMIX also provides 256 individual «general purpose registers» with names of the form "R 0", "R 1", "R 3" and so on. Every one of these registers is capable of storing a single «word» of data formed from a sequence of 64 bits. Depending on its context, such word may be interpreted variously as a natural number "n" in the range "0 <= n < 2^64", a signed integer "z" in the range "negate 2^63 <= z < 2^63", or else as a double precision floating point quantity of the form defined in the IEEE Standard for Binary Floating-Point Arithmetic [IEEE754]. Further, the architecture defines 32 «special purpose registers», although only one of these, namely the «remainder register» "RR", is relevant to a translation of ordinary C programs. Most of the other 31 special purpose registers are reserved for various uses by the operating system and are not discussed further in this work. The architecture follows in the RISC tradition of MIPS and SPARC, in that most of its arithmetic instructions are designed to operate only on values stored in general purpose registers, with all other data storage facilities made accessible only through appropriate «load» and «store» instructions, which facilitate movement of information between general purpose registers and their special purpose counterparts or the program's address space.

Every MMIX instruction is represented uniformly by a 32-bit unsigned integer value, whose binary encoding is organised into four 8-bit fields as follows:

\bigskip
\begin{centered}
\image{*}{*}{Images/Instr.pdf}
\end{centered}
\smallskip

\vskip -3pt

In particular, the "_OP" field stored in bits 24 to 31 is known as the instruction's «opcode». Intuitively, it selects one of the 256 possible MMIX instruction forms such as addition and multiplication, which happen to be assigned the respective opcode numbers 32 and 24. The remaining fields "_X", "_Y" and "_Z" contain up to three 8-bit operands for the instruction. In most cases, each of these operands selects one of the 256 general purpose registers "R 0 .. R 255" described earlier, which, in the following discussion, is represented symbolically by the notations “"R _X"\,”, “"R _Y"\,” and “"R _Z"\,” for the "_X", "_Y" and "_Z" operands, respectively. Typically, the register "R _X" is used to store the operation's actual result, while "R _Y" and "R _Z" represent two input values for the instruction. For some opcodes, however, one or more of these three fields may be interpreted directly as an «immediate operand», in which case its value is always taken to depict an unsigned numeric constant "n" in the range \hbox{"0 <= n <= 255".}

Although, in real-life, every MMIX program is ultimately represented as a binary object that describes its initial address space configuration and memory image, for the sake of exposition, the following discussion is restricted to an analysis of a simpler symbolic representation of such programs, whereby an abstract syntax of the individual object files, from which that memory image will be eventually assembled by the system's linker, is rendered as a symbolic «MMIX module». In particular, every such module is represented by a structure of the form \hbox{“\,"MMixLET (_S) {-TEXT-} (_T) {-DATA-} (_D) {-IN-} (sigma0)"\!”,} assembled from the «entry symbol» "sigma0" and three finite maps "_S", "_T" and "_D", known as the module's «symbol table», «text» and «data segments», respectively. For simplicity, in this work all data segments are represented directly by a set of Etude item bindings, so that, in the following discussion, we will not concern ourselves with the trivial but tedious details of their translation. On the other hand, the text segment of an MMIX module contains the actual instructions of the program, grouped together into sets known as «sections», that will be eventually inserted into the address space of a complete program at various locations chosen in an unspecified manner by the system's linking algorithm. Until then, the precise addresses of all segments are represented by abstract variables known as «symbols». Formally, every symbol is depicted either by some integer label “"SYM n"”, or else by the distinguished value “\,"LOC"”, reserved for a somewhat specialised purpose described later in Section \ref{MMIX/Lambda}.  The module's symbol table "_S" maps any of its symbols that are exported or imported by the module to appropriate Haskell string values that serve as these symbols' external names during the later linking process. In Haskell, the structure of all symbolic MMIX modules can be represented by the \hbox{following BNF grammar:}

\NewHaskellToken	{MMixModuleT}				{Ord}	{«MMIX-module»}
\NewHaskellToken	{MMixSymbolsT}				{Ord}	{«MMIX-symbols»}
\NewHaskellToken	{MMixTextT}				{Ord}	{«MMIX-text»}
\NewHaskellToken	{MMixSectionT}				{Ord}	{«MMIX-section»}
\NewHaskellToken	{MMixDataT}				{Ord}	{«MMIX-data»}

\NewHaskellResult	{MMixModuleT}					{Type}
\NewHaskellResult	{MMixSymbolsT}					{Type}
\NewHaskellResult	{MMixTextT}					{Type}
\NewHaskellResult	{MMixSectionT}					{Type}
\NewHaskellResult	{MMixDataT}					{Type}

\IndexType		{MMixModuleT}					{MMIX_module (\tstar)}
\IndexType		{MMixSymbolsT}					{MMIX_symbols (\tstar)}
\IndexType		{MMixTextT}					{MMIX_text (\tstar)}
\IndexType		{MMixSectionT}					{MMIX_section (\tstar)}
\IndexType		{MMixDataT}					{MMIX_data (\tstar)}

\NewHaskellToken	{MMixSymbolT}				{Ord}	{«MMIX-symbol»}
\NewHaskellResult	{MMixSymbolT}					{Type}
\IndexType		{MMixSymbolT}					{MMIX_symbol (\tstar)}

\BeginHaskellToken	{MMixLET}				{Ord}	{\MKeyword{let}}
  \HaskellArgument							{MMixSymbolsT}
  \HaskellToken		{{-TEXT-}}				{Ord}	{\MKeyword{text}}
  \HaskellArgument							{MMixTextT}
  \HaskellToken		{{-DATA-}}				{Ord}	{\MKeyword{data}}
  \HaskellArgument							{MMixDataT}
  \HaskellToken		{{-IN-}}				{Ord}	{\MKeyword{in}}
  \HaskellArgument							{MMixSymbolT}
  \HaskellResult							{MMixModuleT}
\EndHaskellToken

\BeginHaskellToken	{SYM}					{Open}	{\MKeyword{\#}\WhiteSpace}
  \HaskellArgument							{Integer}
  \HaskellResult							{MMixSymbolT}
\EndHaskellToken

\NewHaskellToken	{LOC}					{Ord}	{\MKeyword{loc}}
\NewHaskellResult	{LOC}						{MMixSymbolT}

\IndexCons		{SYM}						{\# integer (MMIX_symbol)}
\IndexCons		{LOC}						{LOC (MMIX_symbol)}

\begin{syntax}
\begin{code}
data MMixModuleT =
	MMixLET (MMixSymbolsT) {-TEXT-} (MMixTextT) {-DATA-} (MMixDataT) {-IN-} (MMixSymbolT) deriving (Eq, Ord, Show)
	
type MMixSymbolsT =
	MapT (MMixSymbolT) (String)
	
type MMixTextT =
	MapT (MMixSymbolT) (MMixSectionT)

type MMixSectionT =
	[ {-,-} MMixInstrT {-,-} ]

type MMixDataT =
	MapT (MMixSymbolT) (ItemT MMixVarT)

data MMixSymbolT =
	SYM Integer
	| LOC deriving (Eq, Ord, Show)
\end{code}
\end{syntax}

% The actual numeric value of a given symbol is never assigned any particular semantics, so that, in effect, such entities can be interpreted as abstract variables bound to the eventual values of their respective segment addresses and introduced into the program by a later linking stage of its compilation.

The actual content of every text section is depicted simply by a list of symbolic MMIX instructions. Every such instruction consists of its opcode, followed by the three operand fields "_X", "_Y" and "_Z" and an optional symbol. The entire construct is generally written as \hbox{“"MMixInstr (_OP) (_X) {-,-} (_Y) {-,-} (_Z) {-@-} (sigma_opt)"\!”,} except that the “\MKeyword{@}” keyword is usually omitted for conciseness whenever the instruction's syntax does not include a symbol component. The value of each operand field is depicted simply by a numeric representation of its binary encoding, so that the abstract syntax of all symbolic instructions can be captured in Haskell using the following data type definition:

\NewHaskellToken	{MMixInstrT}				{Ord}	{«MMIX-instr»}
\NewHaskellResult	{MMixInstrT}					{Type}
\IndexType		{MMixInstrT}					{MMIX_instr (\tstar)}

\BeginHaskellToken	{MMixInstr}				{Void}	{}
  \HaskellArgument							{MMixOpcodeT}
  \HaskellArgument							{Integer}
  \HaskellToken		{{-,-}}					{Close}	{\WhiteSpace\MKeyword{,}}
  \HaskellArgument							{Integer}
  \HaskellToken		{{-,-}}					{Close}	{\WhiteSpace\MKeyword{,}}
  \HaskellArgument							{Integer}
  \HaskellToken		{{-@-}}					{Ord}	{\MKeyword{@}}
  \HaskellArgument							{Maybe MMixSymbolT}
  \HaskellResult							{MMixInstrT}
\EndHaskellToken

\IndexCons		{MMixInstr}					{,, opcode integer integer integer MMIX_symbol_opt (MMIX_instr)}

\begin{syntax}
\begin{code}
data MMixInstrT =
	MMixInstr (MMixOpcodeT) (Integer) {-,-} (Integer) {-,-} (Integer) {-@-} (Maybe MMixSymbolT) deriving (Eq, Ord)
\end{code}
\end{syntax}

In a binary representation of MMIX modules, symbolic annotations are traditionally stored separately from the actual instruction stream as «relocation records». However, since their direct incorporation into the “"MMixInstrT"” type provides for a much more eloquent depiction of the architecture's semantics in Section \ref{MMIX/Semantics}, in this work I omit all details of such binary representations of MMIX instructions, with an understanding that their typical implementation could be easily formalised if deemed necessary at some future time.

Last but not least, values of the individual MMIX opcodes are always depicted in the symbolic representation by their standard «mnemonics» such as “"MMixADDU"\,” and “\,"MMixMULU"\,”. Formally, the complete list of all such MMIX mnemonics figures in the following definition of the Haskell type “"MMixOpcodeT"”:

\NewHaskellToken	{MMixOpcodeT}				{Ord}	{«MMIX-opcode»}
\NewHaskellResult	{MMixOpcodeT}					{Type}
\IndexType		{MMixOpcodeT}					{MMIX_opcode (\tstar)}

\NewHaskellToken	{MMixTRAP}				{Ord}	{\MKeyword{trap}}
\NewHaskellToken	{MMixFCMP}				{Ord}	{\MKeyword{fcmp}}
\NewHaskellToken	{MMixFUN}				{Ord}	{\MNonKeyword{fun}}
\NewHaskellToken	{MMixFEQL}				{Ord}	{\MNonKeyword{feql}}
\NewHaskellToken	{MMixFADD}				{Ord}	{\MKeyword{fadd}}
\NewHaskellToken	{MMixFIX}				{Ord}	{\MKeyword{fix}}
\NewHaskellToken	{MMixFSUB}				{Ord}	{\MKeyword{fsub}}
\NewHaskellToken	{MMixFIXU}				{Ord}	{\MKeyword{fixu}}

\NewHaskellResult	{MMixTRAP}					{MMixOpcodeT}
\NewHaskellResult	{MMixFCMP}					{MMixOpcodeT}
\NewHaskellResult	{MMixFADD}					{MMixOpcodeT}
\NewHaskellResult	{MMixFIX}					{MMixOpcodeT}
\NewHaskellResult	{MMixFSUB}					{MMixOpcodeT}
\NewHaskellResult	{MMixFIXU}					{MMixOpcodeT}

\IndexCons		{MMixTRAP}					{TRAP (MMIX_opcode)}
\IndexCons		{MMixFCMP}					{FCMP (MMIX_opcode)}
\IndexCons		{MMixFADD}					{FADD (MMIX_opcode)}
\IndexCons		{MMixFIX}					{FIX (MMIX_opcode)}
\IndexCons		{MMixFSUB}					{FSUB (MMIX_opcode)}
\IndexCons		{MMixFIXU}					{FIXU (MMIX_opcode)}

\NewHaskellToken	{MMixFLOT}				{Ord}	{\MKeyword{flot}}
\NewHaskellToken	{MMixFLOTi}				{Ord}	{\MNonKeyword{float\sub{i}}}
\NewHaskellToken	{MMixFLOTU}				{Ord}	{\MKeyword{flotu}}
\NewHaskellToken	{MMixFLOTUi}				{Ord}	{\MNonKeyword{flotu\sub{i}}}
\NewHaskellToken	{MMixSFLOT}				{Ord}	{\MNonKeyword{sflot}}
\NewHaskellToken	{MMixSFLOTi}				{Ord}	{\MNonKeyword{sflot\sub{i}}}
\NewHaskellToken	{MMixSFLOTU}				{Ord}	{\MNonKeyword{sflotu}}
\NewHaskellToken	{MMixSFLOTUi}				{Ord}	{\MNonKeyword{sflotu\sub{i}}}

\NewHaskellResult	{MMixFLOT}					{MMixOpcodeT}
\NewHaskellResult	{MMixFLOTU}					{MMixOpcodeT}

\IndexCons		{MMixFLOT}					{FLOT (MMIX_opcode)}
\IndexCons		{MMixFLOTU}					{FLOTU (MMIX_opcode)}

\NewHaskellToken	{MMixFMUL}				{Ord}	{\MKeyword{fmul}}
\NewHaskellToken	{MMixFCMPE}				{Ord}	{\MNonKeyword{fcmpe}}
\NewHaskellToken	{MMixFUNE}				{Ord}	{\MNonKeyword{fune}}
\NewHaskellToken	{MMixFEQLE}				{Ord}	{\MNonKeyword{feqle}}
\NewHaskellToken	{MMixFDIV}				{Ord}	{\MKeyword{fdiv}}
\NewHaskellToken	{MMixFSQRT}				{Ord}	{\MNonKeyword{fsqrt}}
\NewHaskellToken	{MMixFREM}				{Ord}	{\MNonKeyword{frem}}
\NewHaskellToken	{MMixFINT}				{Ord}	{\MNonKeyword{fint}}

\NewHaskellResult	{MMixFMUL}					{MMixOpcodeT}
\NewHaskellResult	{MMixFDIV}					{MMixOpcodeT}

\IndexCons		{MMixFMUL}					{FMUL (MMIX_opcode)}
\IndexCons		{MMixFDIV}					{FDIV (MMIX_opcode)}

\NewHaskellToken	{MMixMUL}				{Ord}	{\MNonKeyword{mul}}
\NewHaskellToken	{MMixMULi}				{Ord}	{\MNonKeyword{mul\sub{i}}}
\NewHaskellToken	{MMixMULU}				{Ord}	{\MKeyword{mulu}}
\NewHaskellToken	{MMixMULUi}				{Ord}	{\MNonKeyword{mulu\sub{i}}}
\NewHaskellToken	{MMixDIV}				{Ord}	{\MKeyword{div}}
\NewHaskellToken	{MMixDIVi}				{Ord}	{\MNonKeyword{div\sub{i}}}
\NewHaskellToken	{MMixDIVU}				{Ord}	{\MKeyword{divu}}
\NewHaskellToken	{MMixDIVUi}				{Ord}	{\MNonKeyword{divu\sub{i}}}

\NewHaskellResult	{MMixMULU}					{MMixOpcodeT}
\NewHaskellResult	{MMixDIV}					{MMixOpcodeT}
\NewHaskellResult	{MMixDIVU}					{MMixOpcodeT}

\IndexCons		{MMixMULU}					{MULU (MMIX_opcode)}
\IndexCons		{MMixDIV}					{DIV (MMIX_opcode)}
\IndexCons		{MMixDIVU}					{DIVU (MMIX_opcode)}

\NewHaskellToken	{MMixADD}				{Ord}	{\MNonKeyword{add}}
\NewHaskellToken	{MMixADDi}				{Ord}	{\MNonKeyword{add\sub{i}}}
\NewHaskellToken	{MMixADDU}				{Ord}	{\MKeyword{addu}}
\NewHaskellToken	{MMixADDUi}				{Ord}	{\MNonKeyword{addu\sub{i}}}
\NewHaskellToken	{MMixSUB}				{Ord}	{\MNonKeyword{sub}}
\NewHaskellToken	{MMixSUBi}				{Ord}	{\MNonKeyword{sub\sub{i}}}
\NewHaskellToken	{MMixSUBU}				{Ord}	{\MKeyword{subu}}
\NewHaskellToken	{MMixSUBUi}				{Ord}	{\MNonKeyword{subu\sub{i}}}

\NewHaskellResult	{MMixADDU}					{MMixOpcodeT}
\NewHaskellResult	{MMixSUBU}					{MMixOpcodeT}

\IndexCons		{MMixADDU}					{ADDU (MMIX_opcode)}
\IndexCons		{MMixSUBU}					{SUBU (MMIX_opcode)}

\NewHaskellToken	{MMix2ADDU}				{Ord}	{\MNonKeyword{{\AdjustFontSize{100}{2}}addu}}
\NewHaskellToken	{MMix2ADDUi}				{Ord}	{\MNonKeyword{{\AdjustFontSize{100}{2}}addu\sub{i}}}
\NewHaskellToken	{MMix4ADDU}				{Ord}	{\MNonKeyword{{\AdjustFontSize{100}{4}}addu}}
\NewHaskellToken	{MMix4ADDUi}				{Ord}	{\MNonKeyword{{\AdjustFontSize{100}{4}}addu\sub{i}}}
\NewHaskellToken	{MMix8ADDU}				{Ord}	{\MNonKeyword{{\AdjustFontSize{100}{8}}addu}}
\NewHaskellToken	{MMix8ADDUi}				{Ord}	{\MNonKeyword{{\AdjustFontSize{100}{8}}addu\sub{i}}}
\NewHaskellToken	{MMix16ADDU}				{Ord}	{\MNonKeyword{{\AdjustFontSize{100}{16}}addu}}
\NewHaskellToken	{MMix16ADDUi}				{Ord}	{\MNonKeyword{{\AdjustFontSize{100}{16}}addu\sub{i}}}

\NewHaskellToken	{MMixCMP}				{Ord}	{\MKeyword{cmp}}
\NewHaskellToken	{MMixCMPi}				{Ord}	{\MNonKeyword{cmp\sub{i}}}
\NewHaskellToken	{MMixCMPU}				{Ord}	{\MKeyword{cmpu}}
\NewHaskellToken	{MMixCMPUi}				{Ord}	{\MNonKeyword{cmpu\sub{i}}}
\NewHaskellToken	{MMixNEG}				{Ord}	{\MNonKeyword{neg}}
\NewHaskellToken	{MMixNEGi}				{Ord}	{\MNonKeyword{neg\sub{i}}}
\NewHaskellToken	{MMixNEGU}				{Ord}	{\MNonKeyword{negu}}
\NewHaskellToken	{MMixNEGUi}				{Ord}	{\MNonKeyword{negu\sub{i}}}

\NewHaskellResult	{MMixCMP}					{MMixOpcodeT}
\NewHaskellResult	{MMixCMPU}					{MMixOpcodeT}

\IndexCons		{MMixCMP}					{CMP (MMIX_opcode)}
\IndexCons		{MMixCMPU}					{CMPU (MMIX_opcode)}

\NewHaskellToken	{MMixSL}				{Ord}	{\MNonKeyword{sl}}
\NewHaskellToken	{MMixSLi}				{Ord}	{\MNonKeyword{sl\sub{i}}}
\NewHaskellToken	{MMixSLU}				{Ord}	{\MKeyword{slu}}
\NewHaskellToken	{MMixSLUi}				{Ord}	{\MNonKeyword{slu\sub{i}}}
\NewHaskellToken	{MMixSR}				{Ord}	{\MKeyword{sr}}
\NewHaskellToken	{MMixSRi}				{Ord}	{\MNonKeyword{sr\sub{i}}}
\NewHaskellToken	{MMixSRU}				{Ord}	{\MKeyword{sru}}
\NewHaskellToken	{MMixSRUi}				{Ord}	{\MNonKeyword{sru\sub{i}}}

\NewHaskellResult	{MMixSLU}					{MMixOpcodeT}
\NewHaskellResult	{MMixSR}					{MMixOpcodeT}
\NewHaskellResult	{MMixSRU}					{MMixOpcodeT}

\IndexCons		{MMixSLU}					{SL (MMIX_opcode)}
\IndexCons		{MMixSR}					{SR (MMIX_opcode)}
\IndexCons		{MMixSRU}					{SRU (MMIX_opcode)}

\NewHaskellToken	{MMixBN}				{Ord}	{\MNonKeyword{bn}}
\NewHaskellToken	{MMixBNb}				{Ord}	{\MNonKeyword{bn\sub{b}}}
\NewHaskellToken	{MMixBZ}				{Ord}	{\MKeyword{bz}}
\NewHaskellToken	{MMixBZb}				{Ord}	{\MNonKeyword{bz\sub{b}}}
\NewHaskellToken	{MMixBP}				{Ord}	{\MNonKeyword{bp}}
\NewHaskellToken	{MMixBPb}				{Ord}	{\MNonKeyword{bp\sub{b}}}
\NewHaskellToken	{MMixBOD}				{Ord}	{\MNonKeyword{bod}}
\NewHaskellToken	{MMixBODb}				{Ord}	{\MNonKeyword{bod\sub{b}}}

\NewHaskellResult	{MMixBZ}					{MMixOpcodeT}

\IndexCons		{MMixBZ}					{BZ (MMIX_opcode)}

\NewHaskellToken	{MMixBNN}				{Ord}	{\MNonKeyword{bnn}}
\NewHaskellToken	{MMixBNNb}				{Ord}	{\MNonKeyword{bnn\sub{b}}}
\NewHaskellToken	{MMixBNZ}				{Ord}	{\MNonKeyword{bnz}}
\NewHaskellToken	{MMixBNZb}				{Ord}	{\MNonKeyword{bnz\sub{b}}}
\NewHaskellToken	{MMixBNP}				{Ord}	{\MNonKeyword{bnp}}
\NewHaskellToken	{MMixBNPb}				{Ord}	{\MNonKeyword{bnp\sub{b}}}
\NewHaskellToken	{MMixBEV}				{Ord}	{\MNonKeyword{bev}}
\NewHaskellToken	{MMixBEVb}				{Ord}	{\MNonKeyword{bev\sub{b}}}

\NewHaskellToken	{MMixPBN}				{Ord}	{\MNonKeyword{pbn}}
\NewHaskellToken	{MMixPBNb}				{Ord}	{\MNonKeyword{pbn\sub{b}}}
\NewHaskellToken	{MMixPBZ}				{Ord}	{\MNonKeyword{pbz}}
\NewHaskellToken	{MMixPBZb}				{Ord}	{\MNonKeyword{pbz\sub{b}}}
\NewHaskellToken	{MMixPBP}				{Ord}	{\MNonKeyword{pbp}}
\NewHaskellToken	{MMixPBPb}				{Ord}	{\MNonKeyword{pbp\sub{b}}}
\NewHaskellToken	{MMixPBOD}				{Ord}	{\MNonKeyword{pbod}}
\NewHaskellToken	{MMixPBODb}				{Ord}	{\MNonKeyword{pbod\sub{b}}}

\NewHaskellToken	{MMixPBNN}				{Ord}	{\MNonKeyword{pbnn}}
\NewHaskellToken	{MMixPBNNb}				{Ord}	{\MNonKeyword{pbnn\sub{b}}}
\NewHaskellToken	{MMixPBNZ}				{Ord}	{\MNonKeyword{pbnz}}
\NewHaskellToken	{MMixPBNZb}				{Ord}	{\MNonKeyword{pbnz\sub{b}}}
\NewHaskellToken	{MMixPBNP}				{Ord}	{\MNonKeyword{pbnp}}
\NewHaskellToken	{MMixPBNPb}				{Ord}	{\MNonKeyword{pbnp\sub{b}}}
\NewHaskellToken	{MMixPBEV}				{Ord}	{\MNonKeyword{pbev}}
\NewHaskellToken	{MMixPBEVb}				{Ord}	{\MNonKeyword{pbev\sub{b}}}

\NewHaskellToken	{MMixCSN}				{Ord}	{\MNonKeyword{csn}}
\NewHaskellToken	{MMixCSNi}				{Ord}	{\MNonKeyword{csn\sub{i}}}
\NewHaskellToken	{MMixCSZ}				{Ord}	{\MNonKeyword{csz}}
\NewHaskellToken	{MMixCSZi}				{Ord}	{\MNonKeyword{csz\sub{i}}}
\NewHaskellToken	{MMixCSP}				{Ord}	{\MNonKeyword{csp}}
\NewHaskellToken	{MMixCSPi}				{Ord}	{\MNonKeyword{csp\sub{i}}}
\NewHaskellToken	{MMixCSOD}				{Ord}	{\MNonKeyword{csod}}
\NewHaskellToken	{MMixCSODi}				{Ord}	{\MNonKeyword{csod\sub{i}}}

\NewHaskellToken	{MMixCSNN}				{Ord}	{\MNonKeyword{csnn}}
\NewHaskellToken	{MMixCSNNi}				{Ord}	{\MNonKeyword{csnn\sub{i}}}
\NewHaskellToken	{MMixCSNZ}				{Ord}	{\MNonKeyword{csnz}}
\NewHaskellToken	{MMixCSNZi}				{Ord}	{\MNonKeyword{csnz\sub{i}}}
\NewHaskellToken	{MMixCSNP}				{Ord}	{\MNonKeyword{csnp}}
\NewHaskellToken	{MMixCSNPi}				{Ord}	{\MNonKeyword{csnp\sub{i}}}
\NewHaskellToken	{MMixCSEV}				{Ord}	{\MNonKeyword{csev}}
\NewHaskellToken	{MMixCSEVi}				{Ord}	{\MNonKeyword{csev\sub{i}}}

\NewHaskellToken	{MMixZSN}				{Ord}	{\MNonKeyword{zsn}}
\NewHaskellToken	{MMixZSNi}				{Ord}	{\MKeyword{zsn\sub{i}}}
\NewHaskellToken	{MMixZSZ}				{Ord}	{\MNonKeyword{zsz}}
\NewHaskellToken	{MMixZSZi}				{Ord}	{\MKeyword{zsz\sub{i}}}
\NewHaskellToken	{MMixZSP}				{Ord}	{\MNonKeyword{zsp}}
\NewHaskellToken	{MMixZSPi}				{Ord}	{\MKeyword{zsp\sub{i}}}
\NewHaskellToken	{MMixZSOD}				{Ord}	{\MNonKeyword{zsod}}
\NewHaskellToken	{MMixZSODi}				{Ord}	{\MNonKeyword{zsod\sub{i}}}

\NewHaskellResult	{MMixZSNi}					{MMixOpcodeT}
\NewHaskellResult	{MMixZSZi}					{MMixOpcodeT}
\NewHaskellResult	{MMixZSPi}					{MMixOpcodeT}

\IndexCons		{MMixZSNi}					{ZSNi (MMIX_opcode)}
\IndexCons		{MMixZSZi}					{ZSZi (MMIX_opcode)}
\IndexCons		{MMixZSPi}					{ZSPi (MMIX_opcode)}

\NewHaskellToken	{MMixZSNN}				{Ord}	{\MNonKeyword{zsnn}}
\NewHaskellToken	{MMixZSNNi}				{Ord}	{\MKeyword{zsnn\sub{i}}}
\NewHaskellToken	{MMixZSNZ}				{Ord}	{\MNonKeyword{zsnz}}
\NewHaskellToken	{MMixZSNZi}				{Ord}	{\MKeyword{zsnz\sub{i}}}
\NewHaskellToken	{MMixZSNP}				{Ord}	{\MNonKeyword{zsnp}}
\NewHaskellToken	{MMixZSNPi}				{Ord}	{\MKeyword{zsnp\sub{i}}}
\NewHaskellToken	{MMixZSEV}				{Ord}	{\MNonKeyword{zsev}}
\NewHaskellToken	{MMixZSEVi}				{Ord}	{\MNonKeyword{zsev\sub{i}}}

\NewHaskellResult	{MMixZSNNi}					{MMixOpcodeT}
\NewHaskellResult	{MMixZSNZi}					{MMixOpcodeT}
\NewHaskellResult	{MMixZSNPi}					{MMixOpcodeT}

\IndexCons		{MMixZSNNi}					{ZSNNi (MMIX_opcode)}
\IndexCons		{MMixZSNZi}					{ZSNZi (MMIX_opcode)}
\IndexCons		{MMixZSNPi}					{ZSNPi (MMIX_opcode)}

\NewHaskellToken	{MMixLDB}				{Ord}	{\MNonKeyword{ldb}}
\NewHaskellToken	{MMixLDBi}				{Ord}	{\MKeyword{ldb\sub{i}}}
\NewHaskellToken	{MMixLDBU}				{Ord}	{\MNonKeyword{ldbu}}
\NewHaskellToken	{MMixLDBUi}				{Ord}	{\MKeyword{ldbu\sub{i}}}
\NewHaskellToken	{MMixLDW}				{Ord}	{\MNonKeyword{ldw}}
\NewHaskellToken	{MMixLDWi}				{Ord}	{\MKeyword{ldw\sub{i}}}
\NewHaskellToken	{MMixLDWU}				{Ord}	{\MNonKeyword{ldwu}}
\NewHaskellToken	{MMixLDWUi}				{Ord}	{\MKeyword{ldwu\sub{i}}}

\NewHaskellResult	{MMixLDBi}					{MMixOpcodeT}
\NewHaskellResult	{MMixLDBUi}					{MMixOpcodeT}
\NewHaskellResult	{MMixLDWi}					{MMixOpcodeT}
\NewHaskellResult	{MMixLDWUi}					{MMixOpcodeT}

\IndexCons		{MMixLDBi}					{LDBi (MMIX_opcode)}
\IndexCons		{MMixLDBUi}					{LDBUi (MMIX_opcode)}
\IndexCons		{MMixLDWi}					{LDWi (MMIX_opcode)}
\IndexCons		{MMixLDWUi}					{LDWUi (MMIX_opcode)}

\NewHaskellToken	{MMixLDT}				{Ord}	{\MNonKeyword{ldt}}
\NewHaskellToken	{MMixLDTi}				{Ord}	{\MKeyword{ldt\sub{i}}}
\NewHaskellToken	{MMixLDTU}				{Ord}	{\MNonKeyword{ldtu}}
\NewHaskellToken	{MMixLDTUi}				{Ord}	{\MKeyword{ldtu\sub{i}}}
\NewHaskellToken	{MMixLDO}				{Ord}	{\MNonKeyword{ldo}}
\NewHaskellToken	{MMixLDOi}				{Ord}	{\MKeyword{ldo\sub{i}}}
\NewHaskellToken	{MMixLDOU}				{Ord}	{\MNonKeyword{ldou}}
\NewHaskellToken	{MMixLDOUi}				{Ord}	{\MKeyword{ldou\sub{i}}}

\NewHaskellResult	{MMixLDTi}					{MMixOpcodeT}
\NewHaskellResult	{MMixLDTUi}					{MMixOpcodeT}
\NewHaskellResult	{MMixLDOi}					{MMixOpcodeT}
\NewHaskellResult	{MMixLDOUi}					{MMixOpcodeT}

\IndexCons		{MMixLDTi}					{LDTi (MMIX_opcode)}
\IndexCons		{MMixLDTUi}					{LDTUi (MMIX_opcode)}
\IndexCons		{MMixLDOi}					{LDOi (MMIX_opcode)}
\IndexCons		{MMixLDOUi}					{LDOUi (MMIX_opcode)}

\NewHaskellToken	{MMixLDSF}				{Ord}	{\MNonKeyword{ldsf}}
\NewHaskellToken	{MMixLDSFi}				{Ord}	{\MNonKeyword{ldsf\sub{i}}}
\NewHaskellToken	{MMixLDHT}				{Ord}	{\MNonKeyword{ldht}}
\NewHaskellToken	{MMixLDHTi}				{Ord}	{\MNonKeyword{ldht\sub{i}}}
\NewHaskellToken	{MMixCSWAP}				{Ord}	{\MNonKeyword{cswap}}
\NewHaskellToken	{MMixCSWAPi}				{Ord}	{\MNonKeyword{cswap\sub{i}}}
\NewHaskellToken	{MMixLDUNC}				{Ord}	{\MNonKeyword{ldunc}}
\NewHaskellToken	{MMixLDUNCi}				{Ord}	{\MNonKeyword{ldunc\sub{i}}}

\NewHaskellToken	{MMixLDVTS}				{Ord}	{\MNonKeyword{ldvts}}
\NewHaskellToken	{MMixLDVTSi}				{Ord}	{\MNonKeyword{ldvts\sub{i}}}
\NewHaskellToken	{MMixPRELD}				{Ord}	{\MNonKeyword{preld}}
\NewHaskellToken	{MMixPRELDi}				{Ord}	{\MNonKeyword{preld\sub{i}}}
\NewHaskellToken	{MMixPREGO}				{Ord}	{\MNonKeyword{prego}}
\NewHaskellToken	{MMixPREGOi}				{Ord}	{\MNonKeyword{prego\sub{i}}}
\NewHaskellToken	{MMixGO}				{Ord}	{\MNonKeyword{go}}
\NewHaskellToken	{MMixGOi}				{Ord}	{\MKeyword{go\sub{i}}}

\NewHaskellResult	{MMixGOi}					{MMixOpcodeT}

\IndexCons		{MMixGOi}					{GOi (MMIX_opcode)}

\NewHaskellToken	{MMixSTB}				{Ord}	{\MNonKeyword{stb}}
\NewHaskellToken	{MMixSTBi}				{Ord}	{\MNonKeyword{stb\sub{i}}}
\NewHaskellToken	{MMixSTBU}				{Ord}	{\MNonKeyword{stbu}}
\NewHaskellToken	{MMixSTBUi}				{Ord}	{\MKeyword{stbu\sub{i}}}
\NewHaskellToken	{MMixSTW}				{Ord}	{\MNonKeyword{stw}}
\NewHaskellToken	{MMixSTWi}				{Ord}	{\MNonKeyword{stw\sub{i}}}
\NewHaskellToken	{MMixSTWU}				{Ord}	{\MNonKeyword{stwu}}
\NewHaskellToken	{MMixSTWUi}				{Ord}	{\MKeyword{stwu\sub{i}}}

\NewHaskellResult	{MMixSTBUi}					{MMixOpcodeT}
\NewHaskellResult	{MMixSTWUi}					{MMixOpcodeT}

\IndexCons		{MMixSTBUi}					{STBUi (MMIX_opcode)}
\IndexCons		{MMixSTWUi}					{STWUi (MMIX_opcode)}

\NewHaskellToken	{MMixSTT}				{Ord}	{\MNonKeyword{stt}}
\NewHaskellToken	{MMixSTTi}				{Ord}	{\MNonKeyword{stt\sub{i}}}
\NewHaskellToken	{MMixSTTU}				{Ord}	{\MNonKeyword{sttu}}
\NewHaskellToken	{MMixSTTUi}				{Ord}	{\MKeyword{sttu\sub{i}}}
\NewHaskellToken	{MMixSTO}				{Ord}	{\MNonKeyword{sto}}
\NewHaskellToken	{MMixSTOi}				{Ord}	{\MNonKeyword{sto\sub{i}}}
\NewHaskellToken	{MMixSTOU}				{Ord}	{\MNonKeyword{stou}}
\NewHaskellToken	{MMixSTOUi}				{Ord}	{\MKeyword{stou\sub{i}}}

\NewHaskellResult	{MMixSTTUi}					{MMixOpcodeT}
\NewHaskellResult	{MMixSTOUi}					{MMixOpcodeT}

\IndexCons		{MMixSTTUi}					{STTUi (MMIX_opcode)}
\IndexCons		{MMixSTOUi}					{STOUi (MMIX_opcode)}

\NewHaskellToken	{MMixSTSF}				{Ord}	{\MNonKeyword{stsf}}
\NewHaskellToken	{MMixSTSFi}				{Ord}	{\MNonKeyword{stsf\sub{i}}}
\NewHaskellToken	{MMixSTHT}				{Ord}	{\MNonKeyword{stht}}
\NewHaskellToken	{MMixSTHTi}				{Ord}	{\MNonKeyword{stht\sub{i}}}
\NewHaskellToken	{MMixSTCO}				{Ord}	{\MNonKeyword{stco}}
\NewHaskellToken	{MMixSTCOi}				{Ord}	{\MNonKeyword{stco\sub{i}}}
\NewHaskellToken	{MMixSTUNC}				{Ord}	{\MNonKeyword{stunc}}
\NewHaskellToken	{MMixSTUNCi}				{Ord}	{\MNonKeyword{stunc\sub{i}}}

\NewHaskellToken	{MMixSYNCD}				{Ord}	{\MNonKeyword{syncd}}
\NewHaskellToken	{MMixSYNCDi}				{Ord}	{\MNonKeyword{syncd\sub{i}}}
\NewHaskellToken	{MMixPREST}				{Ord}	{\MNonKeyword{prest}}
\NewHaskellToken	{MMixPRESTi}				{Ord}	{\MNonKeyword{prest\sub{i}}}
\NewHaskellToken	{MMixSYNCID}				{Ord}	{\MNonKeyword{syncid}}
\NewHaskellToken	{MMixSYNCIDi}				{Ord}	{\MNonKeyword{syncid\sub{i}}}
\NewHaskellToken	{MMixPUSHGO}				{Ord}	{\MNonKeyword{pushgo}}
\NewHaskellToken	{MMixPUSHGOi}				{Ord}	{\MNonKeyword{pushgo\sub{i}}}

\NewHaskellToken	{MMixOR}				{Ord}	{\MKeyword{or}}
\NewHaskellToken	{MMixORi}				{Ord}	{\MNonKeyword{or\sub{i}}}
\NewHaskellToken	{MMixORN}				{Ord}	{\MNonKeyword{orn}}
\NewHaskellToken	{MMixORNi}				{Ord}	{\MNonKeyword{orn\sub{i}}}
\NewHaskellToken	{MMixNOR}				{Ord}	{\MNonKeyword{nor}}
\NewHaskellToken	{MMixNORi}				{Ord}	{\MNonKeyword{nor\sub{i}}}
\NewHaskellToken	{MMixXOR}				{Ord}	{\MKeyword{xor}}
\NewHaskellToken	{MMixXORi}				{Ord}	{\MNonKeyword{xor\sub{i}}}

\NewHaskellResult	{MMixOR}					{MMixOpcodeT}
\NewHaskellResult	{MMixXOR}					{MMixOpcodeT}

\IndexCons		{MMixOR}					{OR (MMIX_opcode)}
\IndexCons		{MMixXOR}					{XOR (MMIX_opcode)}

\NewHaskellToken	{MMixAND}				{Ord}	{\MKeyword{and}}
\NewHaskellToken	{MMixANDi}				{Ord}	{\MNonKeyword{and\sub{i}}}
\NewHaskellToken	{MMixANDN}				{Ord}	{\MNonKeyword{andn}}
\NewHaskellToken	{MMixANDNi}				{Ord}	{\MNonKeyword{andn\sub{i}}}
\NewHaskellToken	{MMixNAND}				{Ord}	{\MNonKeyword{nand}}
\NewHaskellToken	{MMixNANDi}				{Ord}	{\MNonKeyword{nand\sub{i}}}
\NewHaskellToken	{MMixNXOR}				{Ord}	{\MNonKeyword{nxor}}
\NewHaskellToken	{MMixNXORi}				{Ord}	{\MNonKeyword{nxor\sub{i}}}

\NewHaskellResult	{MMixAND}					{MMixOpcodeT}

\IndexCons		{MMixAND}					{AND (MMIX_opcode)}

\NewHaskellToken	{MMixBDIF}				{Ord}	{\MNonKeyword{bdif}}
\NewHaskellToken	{MMixBDIFi}				{Ord}	{\MNonKeyword{bdif\sub{i}}}
\NewHaskellToken	{MMixWDIF}				{Ord}	{\MNonKeyword{wdif}}
\NewHaskellToken	{MMixWDIFi}				{Ord}	{\MNonKeyword{wdif\sub{i}}}
\NewHaskellToken	{MMixTDIF}				{Ord}	{\MNonKeyword{tdif}}
\NewHaskellToken	{MMixTDIFi}				{Ord}	{\MNonKeyword{tdif\sub{i}}}
\NewHaskellToken	{MMixODIF}				{Ord}	{\MNonKeyword{odif}}
\NewHaskellToken	{MMixODIFi}				{Ord}	{\MNonKeyword{odif\sub{i}}}

\NewHaskellToken	{MMixMUX}				{Ord}	{\MNonKeyword{mux}}
\NewHaskellToken	{MMixMUXi}				{Ord}	{\MNonKeyword{mux\sub{i}}}
\NewHaskellToken	{MMixSADD}				{Ord}	{\MNonKeyword{sadd}}
\NewHaskellToken	{MMixSADDi}				{Ord}	{\MNonKeyword{sadd\sub{i}}}
\NewHaskellToken	{MMixMOR}				{Ord}	{\MNonKeyword{mor}}
\NewHaskellToken	{MMixMORi}				{Ord}	{\MNonKeyword{mor\sub{i}}}
\NewHaskellToken	{MMixMXOR}				{Ord}	{\MNonKeyword{mxor}}
\NewHaskellToken	{MMixMXORi}				{Ord}	{\MNonKeyword{mxor\sub{i}}}

\NewHaskellToken	{MMixSETH}				{Ord}	{\MNonKeyword{seth}}
\NewHaskellToken	{MMixSETMH}				{Ord}	{\MNonKeyword{setmh}}
\NewHaskellToken	{MMixSETML}				{Ord}	{\MNonKeyword{setml}}
\NewHaskellToken	{MMixSETL}				{Ord}	{\MKeyword{setl}}
\NewHaskellToken	{MMixINCH}				{Ord}	{\MKeyword{inch}}
\NewHaskellToken	{MMixINCMH}				{Ord}	{\MKeyword{incmh}}
\NewHaskellToken	{MMixINCML}				{Ord}	{\MKeyword{incml}}
\NewHaskellToken	{MMixINCL}				{Ord}	{\MNonKeyword{incl}}

\NewHaskellResult	{MMixSETL}					{MMixOpcodeT}
\NewHaskellResult	{MMixINCH}					{MMixOpcodeT}
\NewHaskellResult	{MMixINCMH}					{MMixOpcodeT}
\NewHaskellResult	{MMixINCML}					{MMixOpcodeT}

\IndexCons		{MMixSETL}					{SETL (MMIX_opcode)}
\IndexCons		{MMixINCH}					{INCH (MMIX_opcode)}
\IndexCons		{MMixINCMH}					{INCMH (MMIX_opcode)}
\IndexCons		{MMixINCML}					{INCML (MMIX_opcode)}

\NewHaskellToken	{MMixORH}				{Ord}	{\MNonKeyword{orh}}
\NewHaskellToken	{MMixORMH}				{Ord}	{\MNonKeyword{ormh}}
\NewHaskellToken	{MMixORML}				{Ord}	{\MNonKeyword{orml}}
\NewHaskellToken	{MMixORL}				{Ord}	{\MNonKeyword{orl}}
\NewHaskellToken	{MMixANDNH}				{Ord}	{\MNonKeyword{andnh}}
\NewHaskellToken	{MMixANDNMH}				{Ord}	{\MNonKeyword{andnmh}}
\NewHaskellToken	{MMixANDNML}				{Ord}	{\MNonKeyword{andnml}}
\NewHaskellToken	{MMixANDNL}				{Ord}	{\MNonKeyword{andnl}}

\NewHaskellToken	{MMixJMP}				{Ord}	{\MNonKeyword{jmp}}
\NewHaskellToken	{MMixJMPb}				{Ord}	{\MNonKeyword{jmp\sub{b}}}
\NewHaskellToken	{MMixPUSHJ}				{Ord}	{\MNonKeyword{pushj}}
\NewHaskellToken	{MMixPUSHJb}				{Ord}	{\MNonKeyword{pushj\sub{b}}}
\NewHaskellToken	{MMixGETA}				{Ord}	{\MKeyword{geta}}
\NewHaskellToken	{MMixGETAb}				{Ord}	{\MNonKeyword{geta\sub{b}}}
\NewHaskellToken	{MMixPUT}				{Ord}	{\MKeyword{put}}
\NewHaskellToken	{MMixPUTi}				{Ord}	{\MNonKeyword{put\sub{i}}}

\NewHaskellResult	{MMixGETA}					{MMixOpcodeT}
\NewHaskellResult	{MMixPUT}					{MMixOpcodeT}

\IndexCons		{MMixGETA}					{GETA (MMIX_opcode)}
\IndexCons		{MMixPUT}					{PUT (MMIX_opcode)}

\NewHaskellToken	{MMixPOP}				{Ord}	{\MNonKeyword{pop}}
\NewHaskellToken	{MMixRESUME}				{Ord}	{\MNonKeyword{resume}}
\NewHaskellToken	{MMixSAVE}				{Ord}	{\MNonKeyword{save}}
\NewHaskellToken	{MMixUNSAVE}				{Ord}	{\MNonKeyword{unsave}}
\NewHaskellToken	{MMixSYNC}				{Ord}	{\MNonKeyword{sync}}
\NewHaskellToken	{MMixSWYM}				{Ord}	{\MNonKeyword{swym}}
\NewHaskellToken	{MMixGET}				{Ord}	{\MKeyword{get}}
\NewHaskellToken	{MMixTRIP}				{Ord}	{\MNonKeyword{trip}}

\NewHaskellResult	{MMixGET}					{MMixOpcodeT}

\IndexCons		{MMixGET}					{GET (MMIX_opcode)}

\NewHaskellToken	{MMixLDn}				{Ord}	{\MKeyword{ld}«n»}
\NewHaskellToken	{MMixLDnU}				{Ord}	{\MKeyword{ld}«n»\MKeyword{u}}
\NewHaskellToken	{MMixSTn}				{Ord}	{\MKeyword{st}«n»}
\NewHaskellToken	{MMixSTnU}				{Ord}	{\MKeyword{st}«n»\MKeyword{u}}

\NewHaskellToken	{{-@+-}}				{Open}	{\MKeyword{@+}}
\NewHaskellToken	{{-@--}}				{Open}	{\MKeyword{@-}}

\HaskellLeftSkip=24pt
\HaskellRightSkip=-24pt
\newdimen\MMixTab \MMixTab=\hsize \advance\MMixTab by -\HaskellLeftSkip \advance\MMixTab by -\HaskellTabWidth \divide\MMixTab by 8
\divide\HaskellTabWidth by 2
\advance\HaskellRightSkip by -1cm

\LocalHaskellToken	{{-mmix-tab-}}				{Op}	{\hbox to \MMixTab {}}
\LocalHaskellToken	{|}					{Void}	{}
\LocalHaskellToken	{data}					{Void}	{}
\begin{code}
data MMixOpcodeT {-one-of-} =

        MMixTRAP|              MMixFCMP|              MMixFUN|               MMixFEQL|              MMixFADD|              MMixFIX|               MMixFSUB|              MMixFIXU |
        MMixFLOT|              MMixFLOTi|             MMixFLOTU|             MMixFLOTUi|            MMixSFLOT|             MMixSFLOTi|            MMixSFLOTU|            MMixSFLOTUi |
        MMixFMUL|              MMixFCMPE|             MMixFUNE|              MMixFEQLE|             MMixFDIV|              MMixFSQRT|             MMixFREM|              MMixFINT |
        MMixMUL|               MMixMULi|              MMixMULU|              MMixMULUi|             MMixDIV|               MMixDIVi|              MMixDIVU|              MMixDIVUi |

        MMixADD|               MMixADDi|              MMixADDU|              MMixADDUi|             MMixSUB|               MMixSUBi|              MMixSUBU|              MMixSUBUi |
        MMix2ADDU|             MMix2ADDUi|            MMix4ADDU|             MMix4ADDUi|            MMix8ADDU|             MMix8ADDUi|            MMix16ADDU|            MMix16ADDUi |
        MMixCMP|               MMixCMPi|              MMixCMPU|              MMixCMPUi|             MMixNEG|               MMixNEGi|              MMixNEGU|              MMixNEGUi |
        MMixSL|                MMixSLi|               MMixSLU|               MMixSLUi|              MMixSR|                MMixSRi|               MMixSRU|               MMixSRUi |

        MMixBN|                MMixBNb|               MMixBZ|                MMixBZb|               MMixBP|                MMixBPb|               MMixBOD|               MMixBODb |
        MMixBNN|               MMixBNNb|              MMixBNZ|               MMixBNZb|              MMixBNP|               MMixBNPb|              MMixBEV|               MMixBEVb |
        MMixPBN|               MMixPBNb|              MMixPBZ|               MMixPBZb|              MMixPBP|               MMixPBPb|              MMixPBOD|              MMixPBODb |
        MMixPBNN|              MMixPBNNb|             MMixPBNZ|              MMixPBNZb|             MMixPBNP|              MMixPBNPb|             MMixPBEV|              MMixPBEVb |

        MMixCSN|               MMixCSNi|              MMixCSZ|               MMixCSZi|              MMixCSP|               MMixCSPi|              MMixCSOD|              MMixCSODi |
        MMixCSNN|              MMixCSNNi|             MMixCSNZ|              MMixCSNZi|             MMixCSNP|              MMixCSNPi|             MMixCSEV|              MMixCSEVi |
        MMixZSN|               MMixZSNi|              MMixZSZ|               MMixZSZi|              MMixZSP|               MMixZSPi|              MMixZSOD|              MMixZSODi |
        MMixZSNN|              MMixZSNNi|             MMixZSNZ|              MMixZSNZi|             MMixZSNP|              MMixZSNPi|             MMixZSEV|              MMixZSEVi |

        MMixLDB|               MMixLDBi|              MMixLDBU|              MMixLDBUi|             MMixLDW|               MMixLDWi|              MMixLDWU|              MMixLDWUi |
        MMixLDT|               MMixLDTi|              MMixLDTU|              MMixLDTUi|             MMixLDO|               MMixLDOi|              MMixLDOU|              MMixLDOUi |
        MMixLDSF|              MMixLDSFi|             MMixLDHT|              MMixLDHTi|             MMixCSWAP|             MMixCSWAPi|            MMixLDUNC|             MMixLDUNCi |
        MMixLDVTS|             MMixLDVTSi|            MMixPRELD|             MMixPRELDi|            MMixPREGO|             MMixPREGOi|            MMixGO|                MMixGOi |

        MMixSTB|               MMixSTBi|              MMixSTBU|              MMixSTBUi|             MMixSTW|               MMixSTWi|              MMixSTWU|              MMixSTWUi |
        MMixSTT|               MMixSTTi|              MMixSTTU|              MMixSTTUi|             MMixSTO|               MMixSTOi|              MMixSTOU|              MMixSTOUi |
        MMixSTSF|              MMixSTSFi|             MMixSTHT|              MMixSTHTi|             MMixSTCO|              MMixSTCOi|             MMixSTUNC|             MMixSTUNCi |
        MMixSYNCD|             MMixSYNCDi|            MMixPREST|             MMixPRESTi|            MMixSYNCID|            MMixSYNCIDi|           MMixPUSHGO|            MMixPUSHGOi |

        MMixOR|                MMixORi|               MMixORN|               MMixORNi|              MMixNOR|               MMixNORi|              MMixXOR|               MMixXORi |
        MMixAND|               MMixANDi|              MMixANDN|              MMixANDNi|             MMixNAND|              MMixNANDi|             MMixNXOR|              MMixNXORi |
        MMixBDIF|              MMixBDIFi|             MMixWDIF|              MMixWDIFi|             MMixTDIF|              MMixTDIFi|             MMixODIF|              MMixODIFi |
        MMixMUX|               MMixMUXi|              MMixSADD|              MMixSADDi|             MMixMOR|               MMixMORi|              MMixMXOR|              MMixMXORi |

        MMixSETH|              MMixSETMH|             MMixSETML|             MMixSETL|              MMixINCH|              MMixINCMH|             MMixINCML|             MMixINCL |
        MMixORH|               MMixORMH|              MMixORML|              MMixORL|               MMixANDNH|             MMixANDNMH|            MMixANDNML|            MMixANDNL |
        MMixJMP|               MMixJMPb|              MMixPUSHJ|             MMixPUSHJb|            MMixGETA|              MMixGETAb|             MMixPUT|               MMixPUTi |
        MMixPOP|               MMixRESUME|            MMixSAVE|              MMixUNSAVE|            MMixSYNC|              MMixSWYM|              MMixGET|               MMixTRIP --
        deriving (Eq, Ord, Enum, Show)
 	{-mmix-tab-}           {-mmix-tab-}           {-mmix-tab-}           {-mmix-tab-}           {-mmix-tab-}           {-mmix-tab-}           {-mmix-tab-}           {-mmix-tab-}
\end{code}

\vskip -12pt

In this definition, all opcodes are listed in the order of their actual numeric values, so that “\,"MMixTRAP"” represents opcode 0, “\,"MMixFCMP"” stands for the numeric value of 1 and so on, up to “\,\MKeyword{TRIP}”, which depicts the final opcode value 255. In order to focus the present discussion on its primary topic of compiler design, rather than the often esoteric semantics of the MMIX architecture, I restrict the present study to a small subset of all valid MMIX instructions. Specifically, only the 50 opcodes whose mnemonics are emphasised in the above definition of \hbox{“"MMixInstrT"”} are ever mentioned in the remainder of this chapter or assigned a precise semantic interpretation. Further, even though the architecture defines a very capable set of exception handling facilities, for simplicity I assume that all exceptions have been invariably disabled for the entire duration of a program's execution, so that any invalid arithmetic operations can be always assumed to deliver their predictable default values. More so, I take for granted that the processor has been configured permanently with the default IEEE rounding mode for all floating point operations, also computing any denormalised results to their maximum available precisions. The ensuing semantics of the selected 50 MMIX instruction forms are presented with meticulous detail in \hbox{the following section.}

\end{section}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%