#! /bin/perl

use strict;
use warnings;

my ($index_file, $order_file) = @ARGV;

my %order_entries = ( "\\" => '@000', '' => '@000' );
my $order_lineno = 0;
open my $order_fh, '<', $order_file;
while (my $order_line = <$order_fh>) {
	++$order_lineno;
	chomp $order_line;
   	$order_line =~ s/^\s+//;
   	$order_line =~ s/\s+$//;
   	next if $order_line eq '';
   	foreach my $order_key (split /\s+/, $order_line) {
   		print STDERR "$order_file:$order_lineno: duplicate collation token \"$order_key\".\n" if exists $order_entries{$order_key};
	   	$order_entries{$order_key} = sprintf('@%03X', $order_lineno);
	}
}
print STDERR "$order_file: The order file is longer than 4094 lines.\n" if $order_lineno > 0xFFF;

sub make_order_key {
	my ($file, $line, $char) = @_;
	$char =~ s/\s+$//;
	unless (exists $order_entries{$char}) {
		print STDERR "$file:$line: Undefined collation token \"$char\".\n";
		$order_entries{$char} = '@FFF';
	}
	return $order_entries{$char};
}

my %index_entries = ();
my $index_lineno = 0;
open my $index_fh, '<', $index_file;
while (my $index_line = <$index_fh>) {
   	++$index_lineno;
   	chomp $index_line;
   	$index_line =~ s/^\s+//;
   	$index_line =~ s/\s+$//;
   	next if $index_line eq '';
   	if ($index_line =~ m/^\\NewIndexEntry\s*\{(.*)\}\{----\}\{(.*)\}\{----\}\{(.*)\}$/) {
   		my ($label, $key, $page) = ($1, $2, $3);
   		$index_entries{$label} = {
   			key		=> '',
   			sort_key	=> '',
   			pages 		=> {},
   		} unless exists $index_entries{$label};
		$page = int($page);
		$index_entries{$label}->{pages}->{"$page"} = 'New';
		if ($index_entries{$label}->{key} eq '') {
	   		$index_entries{$label}->{key} = $key;
	   		my $sort_key = $key;
	   		$sort_key =~ s/([^\\\s]|\s+|\\[a-zA-Z]+\s*|\\[^a-zA-Z]?\s*)/make_order_key($index_file, $index_lineno, $1)/eg;
	   		$index_entries{$label}->{sort_key} = $sort_key;
   		}
	}
   	elsif ($index_line =~ m/^\\AddIndexEntry\s*\{(.*)\}\{----\}\{(.*)\}$/) {
   		my ($label, $page) = ($1, $2);
   		$index_entries{$label} = {
   			key		=> '',
   			sort_key	=> '',
   			pages 		=> {},
   		} unless exists $index_entries{$label};
		$page = int($page);
		$index_entries{$label}->{pages}->{"$page"} = 'Add' unless exists $index_entries{$label}->{pages}->{"$page"};
	}
   	else {
		print STDERR "$index_file:$index_lineno: Invalid index entry \"$index_line\".\n";
   	}
}

foreach my $label (sort { $index_entries{$a}->{sort_key} cmp $index_entries{$b}->{sort_key} } (keys %index_entries)) {
	my $key = $index_entries{$label}->{key};
	my @pages = map {
		my $page = $_;
		my $kind = $index_entries{$label}->{pages}->{"$page"};
		"\\${kind}IndexPage{$page}";
	} (sort { $a <=> $b } (keys %{$index_entries{$label}->{pages}}));
	my $pages = join ', ', @pages;
	print "\\IndexEntry{$label}{$pages}% $key\n";
}

exit 0;
