\set{capture}{hidden}
\begin{code}

{-# LANGUAGE RelaxedPolyRec #-}

module C.Expressions (
	integerConstantZero,
	integerConstantOne,
	meaningOfFunctionDesignator,
	meaningOfUndeclaredIdentifier,
	meaningOfLValue,
	meaningOfModifiableLValue,
	meaningOfAddressableLValue,
	contentOfLValue,
	bitFieldFormat,
	meaningOfArithmeticOperation,
	meaningOfIntegralOperation,
	meaningOfRelationalOperation,
	meaningOfEqualityOperation,
	meaningOfArithmeticAssignment,
	meaningOfIntegralAssignment,
	meaningOfValueExpression,
	meaningOfVoidExpression,
	meaningOfExpressionOpt,
	listOfArgumentExpressions,
	meaningOfArgumentExpression,
	meaningOfArgumentExpressions,
	meaningOfArgumentExpressionListOpt,
	meaningOfBinaryIntegralConstantExpression,
	meaningOfRelationalIntegralConstantExpression,
	meaningOfFloatingConstantExpression,
	meaningOfIntegralConstantExpression,
	meaningOfNullPointerConstant,
	isNullPointerConstant,
	meaningOfStaticInitialiserExpression,
	meaningOfArithmeticConstantExpression,
	meaningOfBinaryArithmeticConstantExpression,
	meaningOfIntegralArithmeticConstantExpression,
	meaningOfRelationalArithmeticConstantExpression,
	meaningOfAddressConstant,
	meaningOfConstantLValue,
) where

import Control.Monad.Fix
import Additional
import C.Syntax
import C.Variables
import C.Types
import C.Scopes
import C.Constants
import {-# SOURCE #-} C.Declarations
import {-# SOURCE #-} C.Implementation
import Lambda

integerConstantZero, integerConstantOne :: PrimaryExpressionT

integerConstantZero	= Constant (IntegerConstant (IntegerValue PlainIntType 0))
integerConstantOne	= Constant (IntegerConstant (IntegerValue PlainIntType 1))

\end{code}

\NewHaskellToken	{integerConstantZero}			{Ord}	{\CKeyword{0}}
\NewHaskellToken	{integerConstantOne}			{Ord}	{\CKeyword{1}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Expressions}
\set{title}		{Expressions}
\begin{section}

Most of the computational complexity associated with C programs is captured by entities known as «expressions». In the concrete syntax of the language, expressions can assume a number of different forms, most of which consist of an «operator» depicted by a lexical token such as “"`Sum`"” or “"`LogicalAnd`"”, together with one, two or three \hbox{«operand expressions».}

In order to capture the relative binding strengths of the various arithmetic operators provided by the language, the concrete syntax of C expression has been stratified into sixteen distinct «precedence levels», each associated with a unique non-terminal symbol in the BNF grammar and, consequently, a separate Haskell type in this presentation. In particular, every C expression form is uniquely classified as a «primary», «postfix», «unary», «cast», «multiplicative», «additive», «shift», «relational», «equality», «bitwise AND», «bitwise exclusive OR», «bitwise inclusive OR», «logical AND», «logical OR», «conditional», «assignment» or plain «expression». The precedence level of a C operator is used to disambiguate its application in a larger expression that also incorporates references to other operation forms. For example, since the “"`Sum`"” operator has a lower precedence than “"`Product`"”, the composite expression form “"e1 `Sum` e2 `Product` e3"\!” is interpreted as “"e1 `Sum` NestedExpression (e2 `Product` e3)"” rather than “"NestedExpression (e1 `Sum` e2) `Product` e3"\!”. Besides this simple syntactic rôle, precedence levels serve no other purpose in the language and do not influence a semantic interpretation of the individual expression forms in any other way.

\break

\noindent The syntax of all valid C expression forms is captured by the following set of nineteen Haskell data types:

\IndexType		{PrimaryExpressionT}				{primary_expression (\tstar)}
\IndexCons		{Variable}					{ identifier (primary_expression)}
\IndexCons		{Constant}					{ constant (primary_expression)}
\IndexCons		{StringLiteral}					{ string_literal (primary_expression)}
\IndexCons		{NestedExpression}				{() expression (primary_expression)}

\IndexType		{PostfixExpressionT}				{postfix_expression (\tstar)}
\IndexCons		{E16}						{ primary_expression (postfix_expression)}
\IndexCons		{ArraySubscript}				{[] postfix_expression expression (postfix_expression)}
\IndexCons		{FunctionCall}					{() postfix_expression argument_expression_list_opt (postfix_expression)}
\IndexCons		{DirectMember}					{. postfix_expression identifier (postfix_expression)}
\IndexCons		{IndirectMember}				{-> postfix_expression identifier (postfix_expression)}
\IndexCons		{PostIncrement}					{++ postfix_expression (postfix_expression)}
\IndexCons		{PostDecrement}					{-- postfix_expression (postfix_expression)}

\IndexType		{UnaryExpressionT}				{unary_expression (\tstar)}
\IndexCons		{E15}						{ postfix_expression (unary_expression)}
\IndexCons		{Increment}					{++ unary_expression (unary_expression)}
\IndexCons		{Decrement}					{-- unary_expression (unary_expression)}
\IndexCons		{Unary}						{ unary_operator unary_expression (unary_expression)}
\IndexCons		{SizeOf}					{sizeof unary_expression (unary_expression)}
\IndexCons		{SizeOfType}					{sizeof type_name (unary_expression)}

\IndexType		{UnaryOperatorT}				{unary_operator (\tstar)}
\IndexCons		{Address}					{\& (unary_operator)}
\IndexCons		{Indirection}					{* (unary_operator)}
\IndexCons		{Plus}						{+ (unary_operator)}
\IndexCons		{Minus}						{- (unary_operator)}
\IndexCons		{Not}						{\~ (unary_operator)}
\IndexCons		{LogicalNot}					{! (unary_operator)}

\IndexType		{CastExpressionT}				{cast_expression (\tstar)}
\IndexCons		{E14}						{ unary_expression (cast_expression)}
\IndexCons		{Cast}						{() unary_expression type_name (cast_expression)}

\IndexType		{MultiplicativeExpressionT}			{multiplicative_expression (\tstar)}
\IndexCons		{E13}						{ cast_expression (multiplicative_expression)}
\IndexCons		{Product}					{* multiplicative_expression cast_expression (multiplicative_expression)}
\IndexCons		{Quotient}					{/ multiplicative_expression cast_expression (multiplicative_expression)}
\IndexCons		{Remainder}					{\% multiplicative_expression cast_expression (multiplicative_expression)}

\IndexType		{AdditiveExpressionT}				{additive_expression (\tstar)}
\IndexCons		{E12}						{ multiplicative_expression (additive_expression)}
\IndexCons		{Sum}						{+ additive_expression multiplicative_expression (additive_expression)}
\IndexCons		{Difference}					{- additive_expression multiplicative_expression (additive_expression)}

\IndexType		{ShiftExpressionT}				{shift_expression (\tstar)}
\IndexCons		{E11}						{ additive_expression (shift_expression)}
\IndexCons		{LeftShift}					{<< shift_expression additive_expression (shift_expression)}
\IndexCons		{RightShift}					{>> shift_expression additive_expression (shift_expression)}

\IndexType		{RelationalExpressionT}				{relational_expression (\tstar)}
\IndexCons		{E10}						{ shift_expression (relational_expression)}
\IndexCons		{LessThan}					{< relational_expression shift_expression (relational_expression)}
\IndexCons		{GreaterThan}					{> relational_expression shift_expression (relational_expression)}
\IndexCons		{LessOrEqualTo}					{<= relational_expression shift_expression (relational_expression)}
\IndexCons		{GreaterOrEqualTo}				{>= relational_expression shift_expression (relational_expression)}

\IndexType		{EqualityExpressionT}				{equality_expression (\tstar)}
\IndexCons		{E9}						{ relational_expression (equality_expression)}
\IndexCons		{EqualTo}					{== equality_expression relational_expression (equality_expression)}
\IndexCons		{NotEqualTo}					{!= equality_expression relational_expression (equality_expression)}

\IndexType		{AndExpressionT}				{AND_expression (\tstar)}
\IndexCons		{E8}						{ equality_expression (AND_expression)}
\IndexCons		{And}						{& and_expression equality_expression (AND_expression)}

\IndexType		{ExclusiveOrExpressionT}			{exclusive_OR_expression (\tstar)}
\IndexCons		{E7}						{ AND_expression (exclusive_OR_expression)}
\IndexCons		{ExclusiveOr}					{^ exclusive_OR_expression AND_expression (exclusive_OR_expression)}

\IndexType		{InclusiveOrExpressionT}			{inclusive_OR_expression (\tstar)}
\IndexCons		{E6}						{ exclusive_OR_expression (inclusive_OR_expression)}
\IndexCons		{InclusiveOr}					{| inclusive_OR_expression exclusive_OR_expression (inclusive_OR_expression)}

\IndexType		{LogicalAndExpressionT}				{logical_AND_expression (\tstar)}
\IndexCons		{E5}						{ inclusive_OR_expression (logical_AND_expression)}
\IndexCons		{LogicalAnd}					{&& logical_AND_expression inclusive_OR_expression (logical_AND_expression)}

\IndexType		{LogicalOrExpressionT}				{logical_OR_expression (\tstar)}
\IndexCons		{E4}						{ logical_AND_expression (logical_OR_expression)}
\IndexCons		{LogicalOr}					{|| logical_OR_expression logical_AND_expression (logical_OR_expression)}

\IndexType		{ConditionalExpressionT}			{conditional_expression (\tstar)}
\IndexCons		{E3}						{ logical_OR_expression (conditional_expression)}
\IndexCons		{Conditional}					{?: logical_OR_expression expression conditional_expression (conditional_expression)}

\IndexType		{AssignmentExpressionT}				{assignment_expression (\tstar)}
\IndexCons		{E2}						{ conditional_expression (assignment_expression)}
\IndexCons		{Assignment}					{ unary_expression assignment_operator assignment_expression (assignment_expression)}

\IndexType		{AssignmentOperatorT}				{assignment_operator (\tstar)}
\IndexCons		{SimpleAssignment}				{= (assignment_operator)}
\IndexCons		{CompoundProduct}				{*= (assignment_operator)}
\IndexCons		{CompoundQuotient}				{/= (assignment_operator)}
\IndexCons		{CompoundRemainder}				{\%= (assignment_operator)}
\IndexCons		{CompoundSum}					{+= (assignment_operator)}
\IndexCons		{CompoundDifference}				{-= (assignment_operator)}
\IndexCons		{CompoundLeftShift}				{<<= (assignment_operator)}
\IndexCons		{CompoundRightShift}				{>>= (assignment_operator)}
\IndexCons		{CompoundAnd}					{\&= (assignment_operator)}
\IndexCons		{CompoundExclusiveOr}				{\^= (assignment_operator)}
\IndexCons		{CompoundInclusiveOr}				{|= (assignment_operator)}

\IndexType		{ExpressionT}					{expression (\tstar)}
\IndexCons		{E1}						{ assignment_expression (expression)}
\IndexCons		{Comma}						{, expression assignment_expression (expression)}

\begin{syntax}
\InsertHaskell		{PrimaryExpressionT, \par, PostfixExpressionT, \par, UnaryExpressionT, \par, UnaryOperatorT, \par, CastExpressionT, \par,
			 MultiplicativeExpressionT, \par, AdditiveExpressionT, \par, ShiftExpressionT, \par, RelationalExpressionT, \par, EqualityExpressionT, \par,
			 AndExpressionT, \par, ExclusiveOrExpressionT, \par, InclusiveOrExpressionT, \par, LogicalAndExpressionT, \par, LogicalOrExpressionT, \par,
			 ConditionalExpressionT, \par, AssignmentExpressionT, \par, AssignmentOperatorT, \par, ExpressionT}
\end{syntax}

The user should observe that every Haskell type used to describe a given expression precedence level explicitly includes all data constructors from levels higher than itself. For example, “"AdditiveExpressionT"” includes all constructors of the “"MultiplicativeExpressionT"” type. In the actual Haskell implementation, the initial data constructors of the “"MultiplicativeExpressionT"” and “"AdditiveExpressionT"” types are actually written as “MultiplicativeExpressionT (CastExpressionT)” and “AdditiveExpression (MultiplicativeExpressionT)”, respectively, although, for conciseness, their names are always hidden in the following presentation using a liberal application of appropriate implicit coercion functions. However, in most contexts, this segregation of C expressions into sixteen distinct precedence levels serves little useful purpose and it is convenient to think of all C expressions simply as values of the Haskell type \hbox{“"ExpressionT"”,} which implicitly includes all other expression forms. Since most functions presented in this section are defined only on plain “"ExpressionT"” arguments, the reader should always assume that every operand to these functions has been promoted into that type using an appropriate implicit coercion function, whose trivial definition and application have been excluded from the presentation in the interest of readability.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Expressions/Meaning}
\set{title}		{Meanings of Expressions}
\begin{subsection}

In the C language, the above expression syntax is utilised by a number of related, but conceptually distinct logical constructs known as «function designators», «l-values», «values», «void», «constant» and «static initialiser expressions». The formal denotations of these entities are scrutinised individually by five separate Haskell functions “"meaningOfFunctionDesignator"\!”, “"meaningOfLValue"\!”, “"meaningOfValueExpression"\!”, “"meaningOfVoidExpression"\!”, “"meaningOfIntegralConstantExpression"\!” and “"meaningOfStaticInitialiserExpression"\!” that are defined independently in Sections \ref{C/Expressions/FunctionDesignators}, \ref{C/Expressions/LValues}, \ref{C/Expressions/Values}, \ref{C/Expressions/VoidExpressions}, \ref{C/Expressions/Constants/Integral} and \ref{C/Expressions/Constants/Static} below.

In all cases, however, these denotations are derived in the context of a current variable and tag scopes "_S" and "_T", set of item definitions "_D" and the current scope index "_I". While no expression introduces any bindings into its current scope by itself, any type tag declarations found in the syntax of casts and “"SizeOf"\,” operations may extend the scope with new declarations of structure, union and enumeration types. As already mentioned, such declarations persist outside of the expression's own syntax and, accordingly must be always propagated throughout the translation in the order of a depth-first left-to-right traversal of the program's parse tree. Further, the translation of string literals and certain forms of function call operations may result in an introduction of new bindings into the set of variable definitions "_D", whose envelopes may be also recorded in the «temporary variable set» "_V", which, intuitively, collects all variables local to a given C expression. The precise purpose of this set is discussed later in Section \ref{C/Statements}. Formally, the structure of "_V" is defined as the following trivial Haskell type synonym:

\IndexType		{V}						{V (\tstar)}

\begin{syntax}
\InsertHaskell		{V}
\end{syntax}

Accordingly, a complete denotation of every C expression form consists of the updated scopes "_S'" and "_T'", item definition set "_D'", the set "_V" of all temporary variables introduced by the entity, as well as the expression's C type "t" and a monadic Etude term "tau", whose reduction produces the sequence of side effects and an atomic value corresponding to the expression's computational meaning, except that, for constant expressions, "tau" can be variously replaced by the entity's numeric or atomic value, as described later \hbox{in Section \ref{C/Expressions/Constants}.}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Expressions/FunctionDesignators}
\set{title}		{Function Designators}
\begin{subsubsection}

As suggested by its name, a «function designator» describes a reference to a C function. The set of all admissible function designator expressions consists precisely of those variables which, in the expression's current scope "_S", are bound to a well-formed l-value designator of a function type, together with every unary «indirection operation» of the form “"Unary (Indirection) (e)"” in which "e" represents a well-formed value of a function pointer, as well as all parenthesised versions of one of these two expression forms. The meaning of every such function designator "e" is described by an appropriate C function type and a monadic Etude term whose evaluation produces an atom from the function genre “"FG"”, with a value equal to the atomic encoding of the corresponding Etude function. Formally:

\BeginHaskellToken	{meaningOfFunctionDesignator}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{fd}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, V, TypeT, TermT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfFunctionDesignator}			{D_FD expression}

\begin{code}
meaningOfFunctionDesignator {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, ExpressionT) -> _M{--}(S, S, D, V, TypeT, TermT VarT)

meaningOfFunctionDesignator {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (Variable x))))))))))))))))) ---
  = 	do	require $ (x @@ dom{--}(_S) && --
			   isLValueLinkage{--}(linkageOfDesignator{--}(_S#(x))) && --
			   isFunctionType{--}(typeOfDesignator{--}(_S#(x))))
		return $ (_S, _T, _D, emptyMap, typeOfDesignator{--}(_S#(x)), {-sem-}(RET {-sem-}[VAR (variableOfLinkage{--}(linkageOfDesignator{--}(_S#(x))))]))
meaningOfFunctionDesignator {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (Indirection) (e)))))))))))))))) ---
  = 	do	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) ---
			<- meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
		require $ (isPointerType{--}(t_e) && isFunctionType{--}(baseOfType{--}(t_e)))
		return $ (_S_e, _T_e, _D_e, _V_e, baseOfType{--}(t_e), tau_e)
meaningOfFunctionDesignator {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (NestedExpression (e)))))))))))))))))) ---
  =	do	meaningOfFunctionDesignator {--}(_S, _T, _D, _I {->-}, e)
meaningOfFunctionDesignator {-sem-}(_S, _T, _D, _I {->-}, other) ---
  =	fail ("Semantic error")
\end{code}

Intuitively, expressions of the form “"Unary (Indirection) (e)"” allow C programs to convert a value denotation of any well-formed function pointer "e" into a function designator, although, in reality, this operation is rarely required in practice, since, as described later in Section \ref{C/Expressions/Values}, the compiler is usually able to perform it implicitly without any aid from the programmer.

In the context of a «function call operation» discussed later in Section \ref{C/Expressions/Values}, an undeclared identifier also behaves like a designator of a function with the type “"FunctionType (PlainIntType) ()"\!”. Formally, the denotation of such a degenerate designator is obtained by the following trivial derivation:

\BeginHaskellToken	{meaningOfUndeclaredIdentifier}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{und}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, V, TypeT, TermT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfUndeclaredIdentifier}			{D_UND expression}

\begin{code}
meaningOfUndeclaredIdentifier {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, ExpressionT) -> _M{--}(S, S, D, V, TypeT, TermT VarT)

meaningOfUndeclaredIdentifier {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (Variable x))))))))))))))))) = do
	require $ (x /@ dom{--}(_S))
	return $ (_S, _T, listToMap [ {-raw-}(EVar x {-:-}, Just {-sem-}(IMPI x)) ] // _D, emptyMap, --
		  {-sem-}(PointerType (FunctionType (PlainIntType) ({-hide-}Nothing))), --
		  {-sem-}(RET [VAR (EVar x)]))

meaningOfUndeclaredIdentifier {-sem-}(_S, _T, _D, _I {->-}, other) = fail ("Semantic error")
\end{code}

observing that all such identifiers are implicitly bound in the resulting set of item definitions "_D" to an imported Etude item “"IMPI x"”, provided that no previous binding of that identifier exists in the expression's context. In the above Haskell definition, this effect is achieved by extending a singular finite map “"listToMap [ {-raw-}(EVar x {-:-}, Just {-sem-}(IMPI x)) ]"” with all the bindings already present in "_D", using the map extension operator “\!"//"\,” described in Appendix \ref{Additional}. Since “"_A // _B"” always replaces any keys mapped in both of its operands "_A" and "_B" with their "_B" values, this useful idiom ensures that "_D" is affected by the operation only if it contains no binding of "EVar x" at the beginning of the construct.

\end{subsubsection}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Expressions/LValues}
\set{title}		{L-Values}
\begin{subsubsection}

In C, an «l-value» represents an actual memory-resident object whose content may be manipulated by the program. Its denotation always depicts an Etude term which delivers a single atom from a function or object genre, whose reduced value describes an actual location of the denoted C object in the program's address space. Formally, the meanings of all such expressions are characterised by the following Haskell function:

\BeginHaskellToken	{meaningOfLValue}			{Open}	{\ZapfChancery{D}\sub{\SmallCaps{lv}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, V, TypeT, TermT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfLValue}				{D_LV expression}

\begin{code}
meaningOfLValue {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, ExpressionT) -> _M{--}(S, S, D, V, TypeT, TermT VarT)
\end{code}

In particular, every variable name intended as a depiction of an l-value must be bound in its current scope to a designator with an l-value linkage and an object or incomplete type. Predictably, such identifiers denote the designator's type and a trivial monadic term that delivers the value of an Etude variable associated with that designator:

\begin{code}
meaningOfLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (Variable x))))))))))))))))) = do
	require	$ (x @@ dom{--}(_S) && --
  		   isLValueLinkage{--}(linkageOfDesignator{--}(_S#(x))) && --
  		   (isObjectType{--}(typeOfDesignator{--}(_S#(x))) || isIncompleteType{--}(typeOfDesignator{--}(_S#(x)))))
	return $ (_S, _T, _D, emptyMap, typeOfDesignator{--}(_S#(x)), {-sem-}(RET {-sem-}[VAR (variableOfLinkage{--}(linkageOfDesignator{--}(_S#(x))))]))
\end{code}

On the other hand, every well-formed string literal introduces a new unique variable "uvar{--}(_D)" into its set of variable definitions "_D", binding that variable to a new Etude object item whose data specification is obtained from the string literal's denotation. An entire \hbox{l-value} of this form has the string literal's type and a denotation of the trivial Etude term reducible to its new variable value:

\begin{code}
meaningOfLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (StringLiteral string_literal {-,-}))))))))))))))))) = do
	(t, delta_set {-,,-}) <- meaningOfStringLiteral{--}(string_literal)
	return $ (_S, _T, _D `mapUnion` listToMap [ {-raw-}(uvar{--}(_D) {-:-}, Just {-sem-}(OBJI (delta_set {-,,-} ) {-OF-} ({-sem-}(envelopeOfType{--}(t))))) ], --
		  emptyMap, t, {-sem-}(RET {-sem-}[VAR (uvar{--}(_D))]))
\end{code}

Every «direct member operation» of the form “"e {-!-} {-!-} `DirectMember` {-!-} x"” also constitutes a well-formed l-value, provided that its expression operand "e" is itself an l-value of a complete structure or union type and that "x" represents some named member "m" of that type. The entire operation has the member's type "typeOfMember{--}(m)" and a denotation that computes a sum of its offset "offsetOfMember{--}(m)" within the entire structure or union object with the member's l-value converted into the corresponding object format "objectFormat{--}(formatOfType{--}(typeOfMember{--}(m)))". Formally:

\begin{code}
meaningOfLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (e {-!-} {-!-} `DirectMember` {-!-} x)))))))))))))))) = do
	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <-	meaningOfLValue{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 e)))))))))))))))
	(m) <- {-hide-}return $ setOfMembers (membersOfType{--}(t_e))#(x)
 	require $ (isCompoundType{--}(t_e) && x @@ dom{--}(setOfMembers (membersOfType{--}(t_e))))
	return $ 	(_S_e, _T_e, _D_e, _V_e, typeOfMember{--}(m),
		 	{-quad-}{-sem-}( --
				LET [{-raw-}[TVar 1] =: tau_e] $ --
				RET [OP2 (OP1 (CNV {-sem-}(objectFormat{--}(formatOfType{--}(typeOfMember{--}(m))))) (OG`FMT`standardEncoding) {--} {-hv-}(VAR (TVar 1))) --
					(ADD) ({-sem-}(objectFormat{--}(formatOfType{--}(typeOfMember{--}(m))))) --
					(IMM (toRational {-sem-}(offsetOfMember{--}(m))) (ZG`FMT`standardEncoding))]))
\end{code}

Similarly, in every «indirect member operation» of the form “"e {-,-} `IndirectMember` x"”, the expression operand "e" must represent a well-formed value of a pointer to a structure or union type that includes "x" as a named member, with the entire construct assuming the member's type and a denotation that delivers the sum of the pointer's own value with the member's offset within the entire structure or union object. Formally:

\begin{code}
meaningOfLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (e {-,-} `IndirectMember` x)))))))))))))))) = do
	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 e)))))))))))))))
	(m) <- {-hide-}return $ setOfMembers (membersOfType{--}(baseOfType{--}(t_e)))#(x)
	require $ (isPointerType{--}(t_e) && isCompoundType{--}(baseOfType{--}(t_e)) && x @@ dom{--}(setOfMembers (membersOfType{--}(t_e))))
	return $	(_S_e, _T_e, _D_e, _V_e, typeOfMember{--}(m),
			{-quad-}{-sem-}( --
				LET [{-raw-}[TVar 1] =: tau_e] $ --
				RET [OP2 (OP1 (CNV {-sem-}(objectFormat{--}(formatOfType{--}(typeOfMember{--}(m))))) (OG`FMT`standardEncoding) {--} {-hv-}(VAR (TVar 1))) --
					(ADD) ({-sem-}(objectFormat{--}(formatOfType{--}(typeOfMember{--}(m))))) --
					(IMM (toRational {-sem-}(offsetOfMember{--}(m))) (ZG`FMT`standardEncoding))]))
\end{code}

Intuitively, such operations are equivalent to an expression of the form “\!"NestedExpression (Unary (Indirection) (e)) {-!-} {-!-} `DirectMember` x"”, in which the «indirection operation» “"Unary (Indirection) (e)"” converts a value "e" of a pointer type “"PointerType (t) {-*-}"” into an l-value of type "t" that designates an object located at the memory address depicted by the pointer. As described in Section \ref{C/Expressions/Values}, the denotation of every C pointer represents a monadic Etude term reducible precisely to such an address, so that its value is directly equivalent to that of the corresponding indirect l-value:

\begin{code}
meaningOfLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (Indirection) (e)))))))))))))))) = do
	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
	require $ (isPointerType{--}(t_e))
	return $ (_S_e, _T_e, _D_e, _V_e, baseOfType{--}(t_e), tau_e)
\end{code}

Last but not least, every parenthesised version of an l-value “\!"NestedExpression (e)"\!” has the same denotation as its constituent "e", while an «array subscript operations» with the syntax of “"ArraySubscript (e1) (e2)"\!” is semantically equivalent to “"Unary (Indirection) (NestedExpression (e1 `Sum` NestedExpression (e2) {-!-} {-!-}))"\!”, which completes the entire collection of all l-value forms supported by the language:

\begin{code}
meaningOfLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (NestedExpression (e)))))))))))))))))) ---
  = 	meaningOfLValue {-sem-}(_S, _T, _D, _I {->-}, e)
meaningOfLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (ArraySubscript (e1) (e2))))))))))))))))) ---
  = 	meaningOfLValue {-sem-}(_S, _T, _D, _I {->-}, ---
		E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (Indirection) --
			(E14 (E15 (E16 (NestedExpression --
				(E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (Sum --
					(E12 (E13 (E14 (E15 e1)))) --
					{-+-} --
					(E13 (E14 (E15 (E16 (NestedExpression (e2)))))))))))))))))))))))))))))))))))))
meaningOfLValue {-sem-}(_S, _T, _D, _I {->-}, other) ---
  =	fail ("Syntax Error")
\end{code}

An l-value is said to be «modifiable» if it represents an object whose value can be used as a target of an assignment operation. Formally, such l-values must have a complete object type other than an array and neither the object itself nor any of its members may include the “"ConstQualifier"” type qualifier. In all other respects, modifiable l-values behave like their ordinary l-value cousins, so that their meanings can be formalised concisely by the following construction:

\BeginHaskellToken	{meaningOfModifiableLValue}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{mlv}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, V, TypeT, TermT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfModifiableLValue}			{D_MLV expression}

\begin{code}
meaningOfModifiableLValue {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, ExpressionT) -> _M{--}(S, S, D, V, TypeT, TermT VarT)

meaningOfModifiableLValue {-sem-}(_S, _T, _D, _I {->-}, e) = do
	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfLValue{--}(_S, _T, _D, _I {->-}, e)
	require $ (isObjectType{--}(t_e) && not{--}(isArrayType{--}(t_e)) && {-sem-}(ConstQualifier) /@ recTqOfType{--}(t_e))
	return $ (_S_e, _T_e, _D_e, _V_e, t_e, tau_e)
\end{code}

Finally, a well-formed l-value is said to be «addressable» if it does not describe an object of a C bit field type, or one declared with a register linkage form. Formally, the denotations of such constructs are derived in Haskell as follows:

\BeginHaskellToken	{meaningOfAddressableLValue}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{alv}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, V, TypeT, TermT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfAddressableLValue}			{D_ALV expression}

\begin{code}
meaningOfAddressableLValue {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, ExpressionT) -> _M{--}(S, S, D, V, TypeT, TermT VarT)

meaningOfAddressableLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (Variable x))))))))))))))))) = do
	require	$	(x @@ dom{--}(_S) && --
  			 isLValueLinkage{--}(linkageOfDesignator{--}(_S#(x))) && --
  			 linkageOfDesignator{--}(_S#(x)) /= {-sem-}(RegisterLinkage {-sem-}(variableOfLinkage{--}(linkageOfDesignator{--}(_S#(x)))))) &&
  			(isObjectType{--}(typeOfDesignator{--}(_S#(x))) || isIncompleteType{--}(typeOfDesignator{--}(_S#(x)))) && --
  			 not (isBitFieldType{--}(typeOfDesignator{--}(_S#(x))))
	return $ (_S, _T, _D, emptyMap, typeOfDesignator{--}(_S#(x)), {-sem-}(RET {-sem-}[VAR (variableOfLinkage{--}(linkageOfDesignator{--}(_S#(x))))]))

meaningOfAddressableLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (StringLiteral string_literal {-,-}))))))))))))))))) = do
	(t, delta_set {-,,-}) <- meaningOfStringLiteral{--}(string_literal)
	return $ (_S, _T, _D `mapUnion` listToMap [ {-raw-}(uvar{--}(_D) {-:-}, Just {-sem-}(OBJI (delta_set) {-OF-} ({-sem-}(envelopeOfType{--}(t))))) ], --
		  emptyMap, t, {-sem-}(RET {-sem-}[VAR (uvar{--}(_D))]))

meaningOfAddressableLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (e {-!-} {-!-} `DirectMember` {-!-} x)))))))))))))))) = do
	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <-	meaningOfAddressableLValue{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 e)))))))))))))))
	(m) <- {-hide-}return $ setOfMembers (membersOfType{--}(t_e))#(x)
 	require $ (isCompoundType{--}(t_e) && x @@ dom{--}(setOfMembers (membersOfType{--}(t_e))))
	return $  (_S_e, _T_e, _D_e, _V_e, typeOfMember{--}(m),
		  {-quad-}{-sem-}( --
				LET [{-raw-}[TVar 1] =: tau_e] $ --
				RET [OP2 (OP1 (CNV {-sem-}(objectFormat{--}(formatOfType{--}(typeOfMember{--}(m))))) (OG`FMT`standardEncoding) {--} {-hv-}(VAR (TVar 1))) --
					(ADD) ({-sem-}(objectFormat{--}(formatOfType{--}(typeOfMember{--}(m))))) --
					(IMM (toRational {-sem-}(offsetOfMember{--}(m))) (ZG`FMT`standardEncoding))]))

meaningOfAddressableLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (e {-,-} `IndirectMember` x)))))))))))))))) = do
	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 e)))))))))))))))
	(m) <- {-hide-}return $ setOfMembers (membersOfType{--}(baseOfType{--}(t_e)))#(x)
	require $ (isPointerType{--}(t_e) && isCompoundType{--}(baseOfType{--}(t_e)) && x @@ dom{--}(setOfMembers (membersOfType{--}(t_e))))
	return $  (_S_e, _T_e, _D_e, _V_e, typeOfMember{--}(m),
		  {-quad-}{-sem-}( --
				LET [{-raw-}[TVar 1] =: tau_e] $ --
				RET [OP2 (OP1 (CNV {-sem-}(objectFormat{--}(formatOfType{--}(typeOfMember{--}(m))))) (OG`FMT`standardEncoding) {--} {-hv-}(VAR (TVar 1))) --
					(ADD) ({-sem-}(objectFormat{--}(formatOfType{--}(typeOfMember{--}(m))))) --
					(IMM (toRational {-sem-}(offsetOfMember{--}(m))) (ZG`FMT`standardEncoding))]))

meaningOfAddressableLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (Indirection) (e)))))))))))))))) = do
	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
	require $ (isPointerType{--}(t_e))
	return $ (_S_e, _T_e, _D_e, _V_e, baseOfType{--}(t_e), tau_e)

meaningOfAddressableLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (NestedExpression (e)))))))))))))))))) ---
  = meaningOfAddressableLValue {-sem-}(_S, _T, _D, _I {->-}, e)
meaningOfAddressableLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (ArraySubscript (e1) (e2))))))))))))))))) ---
  = meaningOfAddressableLValue {-sem-}(_S, _T, _D, _I {->-}, ---
		E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (Indirection) --
			(E14 (E15 (E16 (NestedExpression --
				(E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (Sum --
					(E12 (E13 (E14 (E15 e1)))) --
					{-+-} --
					(E13 (E14 (E15 (E16 (NestedExpression (e2)))))))))))))))))))))))))))))))))))))
meaningOfAddressableLValue {-sem-}(_S, _T, _D, _I {->-}, other) ---
  = fail ("Syntax Error")
\end{code}

Intuitively, such l-values describe well-formed operands of a unary «address operation» of the form “"Unary (Address) (e)"” but, in all other contexts, are treated identically to any other l-value form described earlier.

\end{subsubsection}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Expressions/Values}
\set{title}		{Values}
\begin{subsubsection}

\BeginHaskellToken	{contentOfLValue}			{Open}	{\ZapfChancery{V}}
  \HaskellArgument							{(TypeT {->-}, TermT VarT)}
  \HaskellResult							{TermT VarT}
\EndHaskellToken

\set{capture}		{contentOfLValue}
\begin{code}
contentOfLValue {-[*]-} :: (TypeT {->-}, TermT VarT) -> TermT VarT

contentOfLValue {-sem-}(t {->-}, tau) ---
    | isBitFieldType{--}(t)	= {-sem-}(LET [{-raw-}[TVar 1] =: tau] $ --
    					  LET [{-raw-}[TVar 2] =: GET (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t))) (bitFieldFormat)] $ --
    					  RET {-sem-}[unpackBitField{--}(t {->-}, VAR (TVar 2))])
    | isScalarType{--}(t)	= {-sem-}(LET [{-raw-}[TVar 1] =: tau] $ --
    					  GET (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t))) ({-sem-}(formatOfType{--}(t))))
    | otherwise			= tau
\end{code}

\NewHaskellToken	{bitFieldFormat}			{Ord}	{\Psi}
\NewHaskellResult	{bitFieldFormat}				{FormatT}

\set{capture}		{bitFieldFormat}
\begin{code}
bitFieldFormat :: FormatT
bitFieldFormat = formatOfType{-sem-}(Unsigned IntType)
\end{code}

\BeginHaskellToken	{meaningOfArithmeticOperation}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{ao}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, V, TypeT, TermT VarT)}
\EndHaskellToken

\set{capture}		{meaningOfArithmeticOperation}
\begin{code}
meaningOfArithmeticOperation {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT) -> _M{--}(S, S, D, V, TypeT, TermT VarT)

meaningOfArithmeticOperation {-sem-}(_S, _T, _D, _I, op {->-}, e1, e2) = do
	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfValueExpression{--}(_S,   _T,   _D,   _I {->-}, e1)
	(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression{--}(_S1,  _T1,  _D1,  _I {->-}, e2)
	require $ (isArithmeticType{--}(t1) && isArithmeticType{--}(t2))
	return $	(_S2, _T2, _D2, _V1 `mapUnion` _V2, t1 `commonArithmeticType` t2,
			{-quad-}{-sem-}( ---
				LET [{-raw-}[TVar 1] =: tau1, {-raw-}[TVar 2] =: tau2] $
			 	RET [OP2 ({-hv-}(OP1 (CNV ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2)))) ({-sem-}(formatOfType{--}(t1))) {--} {-hv-}(VAR (TVar 1)))) --
			 		(op) ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) --
			 		({-hv-}(OP1 (CNV ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2)))) ({-sem-}(formatOfType{--}(t2))) {--} {-hv-}(VAR (TVar 2))))]))
\end{code}

\BeginHaskellToken	{meaningOfIntegralOperation}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{io}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, V, TypeT, TermT VarT)}
\EndHaskellToken

\set{capture}		{meaningOfIntegralOperation}
\begin{code}
meaningOfIntegralOperation {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT) -> _M{--}(S, S, D, V, TypeT, TermT VarT)

meaningOfIntegralOperation {-sem-}(_S, _T, _D, _I, op {->-}, e1, e2) = do
	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfValueExpression{--}(_S,   _T,   _D,   _I {->-}, e1)
	(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression{--}(_S1,  _T1,  _D1,  _I {->-}, e2)
	require $ (isIntegralType{--}(t1) && isIntegralType{--}(t2))
	return $	(_S2, _T2, _D2, _V1 `mapUnion` _V2, t1 `commonArithmeticType` t2, 
			{-quad-}{-sem-}( ---
				LET [{-raw-}[TVar 1] =: tau1, {-raw-}[TVar 2] =: tau2] $
		 		RET [OP2 ({-hv-}(OP1 (CNV ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2)))) ({-sem-}(formatOfType{--}(t1))) {--} {-hv-}(VAR (TVar 1)))) --
		 	 		(op) ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) --
		 	 		({-hv-}(OP1 (CNV ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2)))) ({-sem-}(formatOfType{--}(t2))) {--} {-hv-}(VAR (TVar 2))))]))
\end{code}

\BeginHaskellToken	{meaningOfRelationalOperation}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{ro}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, V, TypeT, TermT VarT)}
\EndHaskellToken

\set{capture}		{meaningOfRelationalOperation}
\begin{code}
meaningOfRelationalOperation {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT) -> _M{--}(S, S, D, V, TypeT, TermT VarT)

meaningOfRelationalOperation {-sem-}(_S, _T, _D, _I, op {->-}, e1, e2) = do
  	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfValueExpression{--}(_S,   _T,   _D,   _I {->-}, e1)
	(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression{--}(_S1,  _T1,  _D1,  _I {->-}, e2)
	require $	(isArithmeticType{--}(t1) && isArithmeticType{--}(t2)) ||
			(  isPointerType{--}(t1) && (isObjectType{--}(baseOfType{--}(t1)) || isIncompleteType{--}(baseOfType{--}(t1))) &&
			   isPointerType{--}(t2) && (unqualifiedType{--}(baseOfType{--}(t1)) `areCompatibleTypes` unqualifiedType{--}(baseOfType{--}(t2))))
  	return $  (_S2, _T2, _D2, _V1 `mapUnion` _V2, {-sem-}(PlainIntType),
  		  {-quad-}{-sem-}( ---
  				LET [{-raw-}[TVar 1] =: tau1, {-raw-}[TVar 2] =: tau2] $
  				RET [OP2 ({-hv-}(OP1 (CNV ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2)))) ({-sem-}(formatOfType{--}(t1))) {--} {-hv-}(VAR (TVar 1)))) --
  				 	(op) ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) --
  				 	({-hv-}(OP1 (CNV ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2)))) ({-sem-}(formatOfType{--}(t2))) {--} {-hv-}(VAR (TVar 2))))]))
\end{code}

\BeginHaskellToken	{meaningOfEqualityOperation}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{eo}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, V, TypeT, TermT VarT)}
\EndHaskellToken

\set{capture}		{meaningOfEqualityOperation}
\begin{code}
meaningOfEqualityOperation {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT) -> _M{--}(S, S, D, V, TypeT, TermT VarT)

meaningOfEqualityOperation {-sem-}(_S, _T, _D, _I, op {->-}, e1, e2) = do
	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfValueExpression{--}(_S,   _T,   _D,   _I {->-}, e1)
	(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression{--}(_S1,  _T1,  _D1,  _I {->-}, e2)
	require $	(isArithmeticType{--}(t1) && isArithmeticType{--}(t2)) ||
			(isPointerType{--}(t1) && isPointerType{--}(t2) && --
				     (	unqualifiedType{--}(baseOfType{--}(t1)) `areCompatibleTypes` unqualifiedType{--}(baseOfType{--}(t2)) ||
		  			((isObjectType{--}(baseOfType{--}(t1)) || isIncompleteType{--}(baseOfType{--}(t1))) && isVoidType{--}(baseOfType{--}(t2))) ||
  					((isObjectType{--}(baseOfType{--}(t2)) || isIncompleteType{--}(baseOfType{--}(t2))) && isVoidType{--}(baseOfType{--}(t1))))) ||
  			(isPointerType{--}(t1) && isNullPointerConstant{--}(_S1, _T1, _D1, _I {->-}, e2)) ||
  			(isPointerType{--}(t2) && isNullPointerConstant{--}(_S, _T, _D, _I {->-}, e1))
	return $  (_S2, _T2, _D2, _V1 `mapUnion` _V2, {-sem-}(PlainIntType), 
		  {-quad-}{-sem-}( ---
				LET [{-raw-}[TVar 1] =: tau1, {-raw-}[TVar 2] =: tau2] $
			 	RET [OP2 ({-hv-}(OP1 (CNV ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2)))) ({-sem-}(formatOfType{--}(t1))) {--} {-hv-}(VAR (TVar 1)))) --
			 	 	(op) ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) --
  			 	 	({-hv-}(OP1 (CNV ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2)))) ({-sem-}(formatOfType{--}(t2))) {--} {-hv-}(VAR (TVar 2))))]))
\end{code}

\BeginHaskellToken	{meaningOfArithmeticAssignment}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{aa}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, V, TypeT, TermT VarT)}
\EndHaskellToken

\set{capture}		{meaningOfArithmeticAssignment}
\begin{code}
meaningOfArithmeticAssignment {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT) -> _M{--}(S, S, D, V, TypeT, TermT VarT)

meaningOfArithmeticAssignment {-sem-}(_S, _T, _D, _I, op {->-}, e1, e2)

	=	do	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfModifiableLValue	{--}(_S,   _T,   _D,   _I {->-}, e1)
	  		(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression	{--}(_S1,  _T1,  _D1,  _I {->-}, e2)
			require $ (isBitFieldType{--}(t1) && isArithmeticType{--}(t2))
			return $	(_S2, _T2, _D2, _V1 `mapUnion` _V2, t1,
					{-quad-}{-sem-}( ---
						LET [{-raw-}[TVar 1] 	=: tau1 {-hv-}, {-raw-}[TVar 2] =: tau2] $
					 	LET [{-raw-}[TVar 3] 	=: GET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t1))) (bitFieldFormat)] $
					 	LET [{-raw-}[TVar 4] 	=: RET  [OP2 ({-sem-}(unpackBitField{--}(t1 {->-}, VAR (TVar 3)))) --
				 							(op) ({-sem-}(formatOfType{--}(t1))) --
				 						({-hv-}(OP1 (CNV ({-sem-}(formatOfType{--}(t1)))) ({-sem-}(formatOfType{--}(t2))) {--} --
				 								{-hv-}(VAR (TVar 2))))]] $
			 	 		LET [{-hv-}[] 		=: SET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t1))) (bitFieldFormat) --
				 	 						{-TO-} ({-sem-}(packBitField{--}(t1 {->-}, VAR (TVar 3), VAR (TVar 4))))] $
			 	 		RET [VAR (TVar 4)]))

	<|>	do	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfModifiableLValue	{--}(_S,   _T,   _D,   _I {->-}, e1)
  			(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression	{--}(_S1,  _T1,  _D1,  _I {->-}, e2)
			require $ (isArithmeticType{--}(t1) && isArithmeticType{--}(t2))
			return $	(_S2, _T2, _D2, _V1 `mapUnion` _V2, t1,
					{-quad-}{-sem-}( ---
						LET [{-raw-}[TVar 1] 	=: tau1 {-hv-}, {-raw-}[TVar 2] =: tau2] $
					 	LET [{-raw-}[TVar 3] 	=: GET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t1))) ({-sem-}(formatOfType{--}(t1)))] $
					 	LET [{-raw-}[TVar 4] 	=: RET  [OP2 (VAR (TVar 3)) (op) ({-sem-}(formatOfType{--}(t1))) --
					 						({-hv-}(OP1 (CNV ({-sem-}(formatOfType{--}(t1)))) ({-sem-}(formatOfType{--}(t2))) {--} --
					 							{-hv-}(VAR (TVar 2))))]] $
				 	 	LET [{-hv-}[] 		=: SET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t1))) ({-sem-}(formatOfType{--}(t1))) --
				 	 						{-TO-} (VAR (TVar 4))] $	
				 	 	RET [VAR (TVar 4)]))
\end{code}

\BeginHaskellToken	{meaningOfIntegralAssignment}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{ia}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, V, TypeT, TermT VarT)}
\EndHaskellToken

\set{capture}		{meaningOfIntegralAssignment}
\begin{code}
meaningOfIntegralAssignment {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT) -> _M{--}(S, S, D, V, TypeT, TermT VarT)

meaningOfIntegralAssignment {-sem-}(_S, _T, _D, _I, op {->-}, e1, e2)

	=	do	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfModifiableLValue	{--}(_S,   _T,   _D,   _I {->-}, e1)
  			(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression	{--}(_S1,  _T1,  _D1,  _I {->-}, e2)
			require $ (isBitFieldType{--}(t1) && isArithmeticType{--}(t2))
			return $ 	(_S2, _T2, _D2, _V1 `mapUnion` _V2, t1,
					{-quad-}{-sem-}( ---
						LET [{-raw-}[TVar 1] 	=: tau1 {-hv-}, {-raw-}[TVar 2] =: tau2] $
					 	LET [{-raw-}[TVar 3] 	=: GET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t1))) (bitFieldFormat)] $
					 	LET [{-raw-}[TVar 4] 	=: RET  [OP2 ({-sem-}(unpackBitField{--}(t1 {->-}, VAR (TVar 3)))) --
					 						(op) ({-sem-}(formatOfType{--}(t1))) --
					 						({-hv-}(OP1 (CNV ({-sem-}(formatOfType{--}(t1)))) ({-sem-}(formatOfType{--}(t2))) {--} --
					 							{-hv-}(VAR (TVar 2))))]] $
				 	 	LET [{-hv-}[] 		=: SET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t1))) (bitFieldFormat) --
					 	 					{-TO-} ({-sem-}(packBitField{--}(t1 {->-}, VAR (TVar 3), VAR (TVar 4))))] $
				 	 	RET [VAR (TVar 4)]))

	<|>	do	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfModifiableLValue	{--}(_S,   _T,   _D,   _I {->-}, e1)
  			(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression	{--}(_S1,  _T1,  _D1,  _I {->-}, e2)
			require $ (isIntegralType{--}(t1) && isIntegralType{--}(t2))
			return $	(_S2, _T2, _D2, _V1 `mapUnion` _V2, t1,
					{-quad-}{-sem-}( ---
						LET [{-raw-}[TVar 1] 	=: tau1 {-hv-}, {-raw-}[TVar 2] =: tau2] $
					 	LET [{-raw-}[TVar 3] 	=: GET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t1))) ({-sem-}(formatOfType{--}(t1)))] $
					 	LET [{-raw-}[TVar 4] 	=: RET  [OP2 (VAR (TVar 3)) (op) ({-sem-}(formatOfType{--}(t1))) --
					 						({-hv-}(OP1 (CNV ({-sem-}(formatOfType{--}(t1)))) ({-sem-}(formatOfType{--}(t2))) {--} --
					 							{-hv-}(VAR (TVar 2))))]] $
				 	 	LET [{-hv-}[]		=: SET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t1))) ({-sem-}(formatOfType{--}(t1))) --
				 	 						{-TO-} (VAR (TVar 4))] $
				 	 	RET [VAR (TVar 4)]))
\end{code}

Besides function designators and l-values, most well-formed expressions belong to the semantic family of «values» which, intuitively, depict the actual Etude atoms resulting from such expressions' evaluation. In the translated C program, the meaning of a value with a scalar type is represented uniformly by a monadic Etude term that delivers an atom depicting the actual content of some memory-resident l-value, or else a result of an appropriate arithmetic operation on such atoms. On the other hand, values of a structure or union type are represented directly by their location within the program's address space, since the content of the corresponding object cannot, in general, be depicted by a single scalar quantity. Formally, the denotations of all well-formed C values are represented by the following Haskell construction:

\BeginHaskellToken	{meaningOfValueExpression}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{v}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, V, TypeT, TermT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfValueExpression}			{D_V expression}

\begin{code}
meaningOfValueExpression {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, ExpressionT) -> _M{--}(S, S, D, V, TypeT, TermT VarT)
\end{code}

In particular, every parenthesised version of a value expression “\!"NestedExpression (e)"\!” has the same meaning as its constituent "e", while a well-formed constant also constitutes a value of a C type derived from its lexical syntax, with a denotation that delivers an atomic rendition of the constant's numeric value under an appropriate Etude format:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (Constant constant))))))))))))))))) ---
  =	do	(t, x) <- meaningOfConstant {--}(_S {->-}, constant)
 		return $ (_S, _T, _D, emptyMap, t, {-sem-}(RET [IMM (x) ({-sem-}(formatOfType{--}(t)))]))
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (NestedExpression (e)))))))))))))))))) ---
  = 	do	meaningOfValueExpression {--}(_S, _T, _D, _I {->-}, e)
\end{code}

Further, every application of the “"SizeOf"\,” operator to a type name that depicts an object type, or to an l-value or value expression of such a type, denotes a trivial term of the implementation-defined “"sizeType"” type described earlier in Section \ref{C/Types/Corresponding}, whose evaluation delivers the size of the operand's type as an integer atom. The reader should observe that any C expressions appearing within a “"SizeOf"\,” operation are never evaluated by the program, so that all associated temporary variables "_V_e" introduced by these expressions' denotations are quietly discarded from the resulting set of item definitions "_D":

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (SizeOfType (type_name)))))))))))))))) = do
	(_S_t, _T_t, _D_t, t_t) <- meaningOfTypeName {--}(_S, _T, _D, _I {->-}, type_name)
	require $ (isObjectType{--}(t_t))
	return $ (_S_t, _T_t, _D_t, emptyMap, {-sem-}(sizeType), {-sem-}(RET [IMM (toRational {-sem-}(sizeOfType{--}(t_t))) ({-sem-}(formatOfType{-sem-}(sizeType)))]))

meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (SizeOf (e {-,,-})))))))))))))))) = do
	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfLValue{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e)))))))))))))) --
  						   <|> meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e))))))))))))))
	require $ (isObjectType{--}(t_e))
	return $ (_S_e, _T_e, _D `mapSetDifference` dom{--}(_V_e), emptyMap, {-sem-}(sizeType), --
		  {-sem-}(RET [IMM (toRational {-sem-}(sizeOfType{--}(t_e))) ({-sem-}(formatOfType{-sem-}(sizeType)))]))
\end{code}

in which the “"<|>"” symbol represents a generic monadic choice operator described in \hbox{Appendix \ref{Additional}.} Given a list of «n» monadic Haskell terms "_M1, _M2 .. _M{-sub-}n", the construction \hbox{“"_M1 <|> _M2 <|> .. <|> _M{-sub-}n"\!”} returns the first of its operands "_M{-sub-}i" which does not represent the monadic zero (or failure) action. In other words, in the above definition, the expression \hbox{“"meaningOfLValue{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e)))))))))))))) <|> meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e))))))))))))))"”} is equivalent to \hbox{"meaningOfLValue{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e))))))))))))))"} if "e" represents a well-formed l-value and to \hbox{"meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e))))))))))))))"} if it assumes a valid \hbox{value form.}

In a value context, a direct member operation of the form “"e {-!-} {-!-} `DirectMember`{-!-} x"” obtains a location of the member "x" within the memory-resident representation of the value operand "e" and delivers the current content of the underlying Etude object as a result of the entire construct. Formally, its meaning is derived similarly to the one of an l-value with an analogous structure, except that the value, rather than l-value denotation of the operand "e" is used in the construction. Formally:
 
\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (e {-!-} {-!-} `DirectMember` {-!-} x)))))))))))))))) = do
	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <-	meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 e)))))))))))))))
	({-hfil-}m    ) <- {-hide-}return $ setOfMembers (membersOfType{--}(t_e))#(x)
	({-hfil-}tau  ) <- {-hide-}return $ {-sem-}( ---
				LET [{-raw-}[TVar 1] =: tau_e] $ --
				RET [OP2 (OP1 (CNV {-sem-}(objectFormat{--}(formatOfType{--}(typeOfMember{--}(m))))) (OG`FMT`standardEncoding) {--} {-hv-}(VAR (TVar 1))) --
				 	(ADD) ({-sem-}(objectFormat{--}(formatOfType{--}(typeOfMember{--}(m))))) --
					(IMM (toRational {-sem-}(offsetOfMember{--}(m))) (ZG`FMT`standardEncoding))])
 	require $ (isCompoundType{--}(t_e) && x @@ dom{--}(setOfMembers (membersOfType{--}(t_e))))
	return $ (_S_e, _T_e, _D_e, _V_e, typeOfMember{--}(m), contentOfLValue{--}(typeOfMember{--}(m) {->-}, tau))
\end{code}

Once an address "tau" of the required memory-resident object has been obtained by the above definition, the actual value denotation of “"e {-!-} {-!-} `DirectMember`{-!-} x"” is obtained from it by constructing a new Etude term which retrieves a current content of that object from the program's address space. In this work, such terms are depicted by the notation “"contentOfLValue{--}(t {->-}, tau)"”, in which "tau" represents an Etude term which delivers an address of a C variable or object with the type "t". Specifically, if "t" represents a bit field type, then the constructed term retrieves the content of the bit field's «container object» using an Etude operation of the form “"GET (VAR (TVar 1) {-hv-}, mu_set) (bitFieldFormat)"\!”, in which the variable “"TVar 1"\!” is bound to the container's location, "mu_set" represents the set of all access attributes associated with "t" and "bitFieldFormat" depicts a designated Etude format assigned to all bit field containers. By definition, "bitFieldFormat" is always equal to the format of the “"Unsigned LongType"” type, as captured by the following Haskell function:

\IndexTerm		{bitFieldFormat}				{\Psi (format)}
\InsertHaskell		{bitFieldFormat}

The actual bit field value is then extracted from its container in an unspecified manner discussed separately in Appendix \ref{C/Implementation}. On the other hand, if "t" represents any other scalar type, then the Etude term constructed by "contentOfLValue" always retrieves an entire content of the specified memory-resident object and returns it as an atom of a format appropriate for "t". In all other cases, the constructed denotation is identical to "tau" itself, since, as already mentioned earlier in this section, all values of a function, structure, union or array type are depicted directly by the location of the corresponding Etude entity with the address space of a program. In Haskell, the "contentOfLValue" combinator can be implemented as follows:

\IndexTerm		{contentOfLValue}				{V term type (term)}
\InsertHaskell		{contentOfLValue}

The notation “"unpackBitField{--}(t {->-}, alpha)"” represents some unspecified Etude term that extracts the bit field "t" from its container value "alpha". The precise semantics of "unpackBitField" are left open to interpretation by the individual implementations of a C compiler but, on every system, the function is guaranteed to be accompanied by the reverse operation "packBitField", such that "unpackBitField{--}(t {->-}, packBitField{--}(t {->-}, alpha', alpha)) `EqvJ` alpha" for all bit field types "t", whenever "alpha" and "alpha'" represent Etude atoms that are well-formed under the formats of "t" and the base type of "t", respectively. Formally, these two functions are described by the following pair of Haskell \hbox{type signatures:}

\ReindexTerm		{packBitField}
\ReindexTerm		{unpackBitField}

\begin{haskell}
unpackBitField	{-[*]-} :: (TypeT {->-}, AtomT VarT) -> AtomT VarT
packBitField	{-[*]-} :: (TypeT {->-}, AtomT VarT, AtomT VarT) -> AtomT VarT
\end{haskell}

and the above-mentioned constraint on their definition is captured concisely by the following theorem:

\NewHaskellToken	{packBitFieldJ}				{Ord}	{\TKeyword{pack}}
\NewHaskellDescription	{packBitFieldJ}					{Representation of bit field values}
\IndexTheorem		{packBitFieldJ}					{PACK Representation of bit field values}

\begin{haskell}
packBitFieldJ :: forall x, y, t =>
	WfJ{-sem-}(IMM (x) ({-sem-}(formatOfType{--}(t)))) -> --
	WfJ{-sem-}(IMM (y) (bitFieldFormat)) -> --
	{-sem-}(isBitFieldType{--}(t)) ->
	{-sem-}(unpackBitField{--}(t {->-}, packBitField{--}(t {->-}, {-sem-}(IMM (y) ({-sem-}(bitFieldFormat))), {-sem-}(IMM (x) ({-sem-}(formatOfType{--}(t))))))) --
		`EqvJ` {-sem-}(IMM (x) ({-sem-}(formatOfType{--}(t))))
\end{haskell}

Intuitively, this theorem guarantees that every numeric quantity within the range of values representable by a given bit field type "t" can be inserted into a container atom of the bit field format "bitFieldFormat" and subsequently extracted from that container without affecting the bit field's original value. Accordingly, under typical implementations of the C language, adjacent bit field members of structure type are often “packed” into a single “"Unsigned LongType"” object in order to reduce the amount of space required for their storage within the program's address space.

In a «function call operation» of the form “"FunctionCall (e) (ael_opt)"\!”, the first operand "e" must represent either a valid pointer value whose referenced type describes a function returning an object type, or else a function designator formed from an undeclared identifier "x" as described earlier in Section \ref{C/Expressions/FunctionDesignators}. In both cases, the entire function call assumes the returned type of the corresponding function designator. Every such operation introduces a new temporary l-value "uvar{--}(_D_a)" of that returned type into the resulting set of local variable definitions "_V", which, intuitively, is used to store the result computed by the targeted C function. Its actual denotation evaluates, in an unspecified order, the value of "e" and any expressions in the optional argument list "ael_opt", then applies the function atom designated by "e" to the list of Etude variables derived from the supplied list of argument expressions and, finally, returns the content of the temporary l-value "uvar{--}(_D_a)" as the result of the entire operation. Formally, the complete translation of such constructs is defined in Haskell as follows:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (FunctionCall (e) (ael_opt))))))))))))))))) = do
	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 e))))))))))))))) --
  					<|> meaningOfUndeclaredIdentifier{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 e)))))))))))))))
	require $ (isPointerType{--}(t_e) && isFunctionType{--}(baseOfType{--}(t_e)) && isObjectType{--}(baseOfType{--}(baseOfType{--}(t_e))))
	(_S_a, _T_a, _D_a, _V_a, _W_a, beta_set_a) <- meaningOfArgumentExpressionListOpt{--}(_S_e, _T_e, _D_e, _I, prototypeOfType{--}(baseOfType{--}(t_e)) {->-}, ael_opt)
	return $	(_S_a, _T_a, --
			 _D_a `mapUnion` listToMap [ {-raw-}(uvar{--}(_D_a) {-:-}, Nothing) ], --
			 _V_e `mapUnion` _V_a `mapUnion` _W_a `mapUnion` listToMap [ {-raw-}(uvar{--}(_D_a) {-:-}, envelopeOfType{--}(baseOfType{--}(baseOfType{--}(t_e)))) ], --
			 baseOfType{--}(baseOfType{--}(t_e)),
			{-quad-}{-sem-}( ---
			 	LET ({-sem-}({-sem-}[{-raw-}[TVar 1] =: tau_e] ++ beta_set_a)) $
			  	LET [{-hv-}[] =: APP (VAR (TVar 1)) (coerceList (VAR) {-sem-}({-sem-}[uvar{--}(_D_a)] ++ setToList (dom{--}(_W_a))))] $
			  	{-sem-}(contentOfLValue{--}(baseOfType{--}(baseOfType{--}(t_e)) {->-}, RET {-sem-}[VAR (uvar{--}(_D_a))]))))
\end{code}

where the precise meaning of argument expression lists, represented above by the notation “"meaningOfArgumentExpressionListOpt{--}(_S, _T, _D, _I, p_opt {->-}, ael_opt)"”, is discussed separately in Section \ref{C/Expressions/FunctionArguments}.

Further, all values formed from a «cast operation» of the form “\!"Cast (type_name) {-!-} e"\,” convert the value of their expression operand into the C type denoted by the specified type name, using an appropriate Etude operation “"OP1 (CNV phi' {-!-}{-!-}) (phi) {--} {-hv-}(alpha)"”, in which "alpha" represents the atom delivered from evaluation of the value "e", "phi" is its Etude format and "phi'" is the format corresponding to the targeted C type. In all cases, both the type name operand and the specified expression value must represent scalar quantity. Formally:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (Cast (type_name) {-!-} e {-,-})))))))))))))) = do
	(_S_t,   _T_t,   _D_t,  t_t) <- meaningOfTypeName{--}(_S, _T, _D, _I {->-}, type_name)
	(_S_e,   _T_e,   _D_e,  _V_e, t_e, tau_e) <- meaningOfValueExpression{--}(_S_t, _T_t, _D_t, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
	require $ (isScalarType{--}(t_t) && isScalarType{--}(t_e))
	return $ (_S_e, _T_e, _D_e, _V_e, t_t, --
  		  {-sem-}(LET [{-raw-}[TVar 1] =: tau_e] $ RET [OP1 (CNV ({-sem-}(formatOfType{--}(t_t)))) ({-sem-}(formatOfType{--}(t_e))) {--} {-hv-}(VAR (TVar 1))]))
\end{code}

The reader should, however, recall from Chapter \ref{Lambda} that the Etude format conversion operator is only partially specified in the generic fragment of Etude's semantics and therefore, indirectly, also in standard C programs. At the very least, this operator is always capable of translating atoms between two arithmetic formats, in which case the object's numeric value is retained whenever possible. Further, conversions between objects and integral types, objects of two different kinds and pairs of distinct function encodings are also possible on every instruction set architecture, although the precise meanings of all such «pointer conversions» are always left open for further specification by the individual implementations of the language.

Most often, pointer values are introduced into a C program using an «address operation» of the form “"Unary (Address) (e)"”. In every such operation, the operand must represent a function designator or an addressable l-value. In all cases, the entire construct has the type of a pointer to its expression operand and a denotation identical to that of the function designator or l-value "e". Formally:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (Address) (e)))))))))))))))) = do
	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfFunctionDesignator {--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e))))))))))))) --
							<|> meaningOfAddressableLValue{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
	return $ (_S_e, _T_e, _D_e, _V_e, {-sem-}(PointerType (t_e) {-*-}), tau_e)
\end{code}

The reader should recall from Chapter \ref{Lambda} that all details of a binary encoding of object values are left unspecified in the generic fragment of Etude semantics, so that no portable C program can ever rely on any particular numeric properties of the value produced by the above “"Address"” operator.

Every unary C expression of the form “"Unary (Plus) (e)"” is equivalent to its value operand "e" converted into that operand's integral-promoted type. Similarly, “"Unary (Minus) (e)"” and “"Unary (Not) (e)"” deliver the negation and bit complement of "e" after integral promotion. All three of these operations assume the type of their converted operands, which must belong to the arithmetic family for “"Plus"” and “"Minus"”, or represent an integral type for “"Not"”. On the other hand, a «logical not operation» of the form “"Unary (LogicalNot) (e)"” always adopts the plain “"PlainIntType"” type and its operand "e" may represent an arbitrary scalar value, whose result is compared in the constructed denotation for equality with the integer constant “"integerConstantZero"”, using an application of the binary Etude operator “"CEQ{-sub-}phi"\!” under an appropriate format "phi". Formally, the denotations of all such operations are expressed in Haskell as follows:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (Plus) (e)))))))))))))))) = do
	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
	require $ (isArithmeticType{--}(t_e))
	return $ (_S_e, _T_e, _D_e, _V_e, integralPromotedType{--}(t_e), --
			{-sem-}(LET [{-raw-}[TVar 1] =: tau_e] $ --
  				RET [OP1 (CNV ({-sem-}(formatOfType{--}(integralPromotedType{--}(t_e))))) ({-sem-}(formatOfType{--}(t_e))) {--} {-hv-}(VAR (TVar 1))]))

meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (Minus) (e)))))))))))))))) = do
	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
  	require $ (isArithmeticType{--}(t_e))
  	return $ (_S_e, _T_e, _D_e, _V_e, integralPromotedType{--}(t_e), --
  			{-sem-}(LET [{-raw-}[TVar 1] =: tau_e] $ --
  				RET [OP1 (NEG) ({-sem-}(formatOfType{--}(integralPromotedType{--}(t_e)))) {--} --
  					 	{-hv-}(OP1 (CNV ({-sem-}(formatOfType{--}(integralPromotedType{--}(t_e))))) ({-sem-}(formatOfType{--}(t_e))) {--} --
  					 		{-hv-}(VAR (TVar 1)))]))

meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (Not) (e)))))))))))))))) = do
	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
	require $ (isIntegralType{--}(t_e))
	return $ (_S_e, _T_e, _D_e, _V_e, integralPromotedType{--}(t_e), --
			{-sem-}(LET [{-raw-}[TVar 1] =: tau_e] $ --
  				RET [OP1 (NOT) ({-sem-}(formatOfType{--}(integralPromotedType{--}(t_e)))) {--} --
  				 		{-hv-}(OP1 (CNV ({-sem-}(formatOfType{--}(integralPromotedType{--}(t_e))))) ({-sem-}(formatOfType{--}(t_e))) {--} --
  				 			{-hv-}(VAR (TVar 1)))]))

meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (LogicalNot) (e)))))))))))))))) = do
	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
	require $ (isScalarType{--}(t_e))
	return $ (_S_e, _T_e, _D_e, _V_e, {-sem-}(PlainIntType), --
  			{-sem-}(LET [{-raw-}[TVar 1] =: tau_e] $ --
  				RET [OP2 (VAR (TVar 1)) (CEQ) ({-sem-}(formatOfType{--}(t_e))) (IMM (0) ({-sem-}(formatOfType{--}(t_e))))]))
\end{code}

The two binary operators “"`Product`"” and “\!"`Quotient`"” form a family of C expressions known as the «binary arithmetic operations». A denotation of every such entity can be characterised completely by some binary Etude operator "op", such as “"MUL{-sub-}phi"\!” and “"DIV{-sub-}phi"\!” for “"`Product`"” and “\!"`Quotient`"”, respectively. Each binary arithmetic operation must be supplied with a pair of values of some arithmetic C types. It first computes the common arithmetic type of these operands using the usual arithmetic conversion algorithm described earlier in Section \ref{C/Types/UsualArithmeticConversions} and converts both operands' values into that type, which will be also assumed by the entire operation. In all cases, both values are evaluated within a single Etude group, thus leaving the relative order of their execution unspecified. The resulting Etude atoms are then  applied to the provided Etude operator “"op{-sub-}phi"\!”, where "phi" represents an Etude format derived from the C type of the entire operation. Formally:

\IndexTerm		{meaningOfArithmeticOperation}			{D_AO expression expression}
\InsertHaskell		{meaningOfArithmeticOperation}

In particular, the two C operators “"`Product`"” and “\!"`Quotient`"” are characterised by the respective binary arithmetic operations “"MUL{-sub-}phi"\!” and “"DIV{-sub-}phi"\!” as follows:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (e1 `Product` e2))))))))))))) ---
  = meaningOfArithmeticOperation{--}(_S, _T, _D, _I, {-sem-}(MUL)  {->-}, --
  					E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e1))))))))))), --
  					E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e2)))))))))))))
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (e1 `Quotient` e2))))))))))))) ---
  = meaningOfArithmeticOperation{--}(_S, _T, _D, _I, {-sem-}(DIV)  {->-}, --
  					E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e1))))))))))), --
  					E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e2)))))))))))))
\end{code}

Further, «addition» and «subtraction operations» of the form “"e1 `Sum` e2"\!” or “"e1 `Difference` e2"\!” represent binary arithmetic operations “"ADD{-sub-}phi"\!” and “"SUB{-sub-}phi"\!” if and only if both of their operands have arithmetic types. Otherwise, one of the two operands in an expression of the form “"e1 `Sum` e2"\!” must represent a pointer to an object type and the other must have an integral type, with the entire construct having the pointer operand's type and a denotation that evaluates both operands in an unspecified order, producing the sum of the pointer's value with the product of the integer operand's result converted into the standard integer format “"ZG`FMT`standardEncoding"” and multiplied by the size of the pointer's referenced type. Formally:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (e1 `Sum` e2))))))))))))

	=	meaningOfArithmeticOperation{--}(_S, _T, _D, _I, {-sem-}(ADD) {->-}, --
  							E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e1)))))))))), --
  							E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e2))))))))))))

	<|>	do	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfValueExpression{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e1)))))))))))
			(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression{--}(_S1,  _T1,  _D1,  _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e2))))))))))))
  			require $ (isPointerType{--}(t1) && isObjectType{--}(baseOfType{--}(t1)) && isIntegralType{--}(t2))
  			return $	(_S2, _T2, _D2, _V1 `mapUnion` _V2, t1, 
  					{-quad-}{-sem-}( ---
  					 	LET [{-raw-}[TVar 1] =: tau1, {-raw-}[TVar 2] =: tau2] $
  					 	RET [OP2 (VAR (TVar 1)) (ADD) ({-sem-}(formatOfType{--}(t1))) --
  					 		({-hv-}(OP2 (OP1 (CNV (ZG`FMT`standardEncoding)) ({-sem-}(formatOfType{--}(t2))) {--} {-hv-}(VAR (TVar 2))) --
  					 	 		(MUL) (ZG`FMT`standardEncoding) --
  				 		 			(IMM (toRational {-sem-}(sizeOfType{--}(baseOfType{--}(t1)))) (ZG`FMT`standardEncoding))))]))

	<|>	do	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfValueExpression{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e1)))))))))))
			(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression{--}(_S1,  _T1,  _D1,  _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e2))))))))))))
  			require $ (isPointerType{--}(t2) && isObjectType{--}(baseOfType{--}(t2)) && isIntegralType{--}(t1))
  			return $	(_S2, _T2, _D2, _V1 `mapUnion` _V2, t2,
  					{-quad-}{-sem-}( ---
  					 	LET [{-raw-}[TVar 1] =: tau1, {-raw-}[TVar 2] =: tau2] $
  					 	RET [OP2 (VAR (TVar 2)) (ADD) ({-sem-}(formatOfType{--}(t2))) --
  					 	 	({-hv-}(OP2 (OP1 (CNV (ZG`FMT`standardEncoding)) ({-sem-}(formatOfType{--}(t1))) {--} {-hv-}(VAR (TVar 1))) --
  				 		 		(MUL) (ZG`FMT`standardEncoding) --
  				 	 				(IMM (toRational {-sem-}(sizeOfType{--}(baseOfType{--}(t2)))) (ZG`FMT`standardEncoding))))]))
\end{code}

Similarly, if the first operand to a subtraction operation “"e1 `Difference` e2"\!” represents a pointer to an object type and the second has an integral type, then the entire expression has the pointer's type and a denotation that delivers the sum of the pointer's value with the product of the appropriately-converted integer operand multiplied by the negated size of the pointer's referenced type. More so, if both operands represent pointers to qualified or unqualified versions of compatible object types, then the entire operation has the implementation-defined “"ptrDiffType"” type described in Section \ref{C/Types/Corresponding} and denotes a difference of the pointers' values, divided by the size of their common referenced type. Formally:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (e1 `Difference` e2))))))))))))

    =	 meaningOfArithmeticOperation{--}(_S, _T, _D, _I, {-sem-}(SUB) {->-}, --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e1)))))))))), --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e2))))))))))))

    <|>	 do	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfValueExpression{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e1)))))))))))
		(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression{--}(_S1,  _T1,  _D1,  _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e2))))))))))))
  		require $ (isPointerType{--}(t1) && isObjectType{--}(baseOfType{--}(t1)) && isIntegralType{--}(t2))
  		return $ (_S2, _T2, _D2, _V1 `mapUnion` _V2, t1,
  		{-quad-}{-quad-}{-sem-}(	LET [{-raw-}[TVar 1] =: tau1, {-raw-}[TVar 2] =: tau2] $
  				 		RET [OP2 (VAR (TVar 1)) (ADD) ({-sem-}(formatOfType{--}(t1))) --
  				 	 		{-hv-}(OP2 (OP1 (CNV (ZG`FMT`standardEncoding)) ({-sem-}(formatOfType{--}(t2))) {--} {-hv-}(VAR (TVar 2))) --
  				 	 			(MUL) (ZG`FMT`standardEncoding) --
  				 	 				(IMM (toRational {-sem-}(negate (sizeOfType{--}(baseOfType{--}(t1))))) (ZG`FMT`standardEncoding)))]))

    <|>	 do	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfValueExpression{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e1)))))))))))
		(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression{--}(_S1,  _T1,  _D1,  _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e2))))))))))))
  		require $ (isPointerType{--}(t1) && isPointerType{--}(t2) && --
  			   isObjectType{--}(baseOfType{--}(t2)) && --
  			   unqualifiedType{--}(baseOfType{--}(t1)) `areCompatibleTypes` unqualifiedType{--}(baseOfType{--}(t2)))
  		return $ (_S2, _T2, _D2, _V1 `mapUnion` _V2, {-sem-}(ptrDiffType),
  		{-quad-}{-quad-}{-sem-}(	LET [{-raw-}[TVar 1] =: tau1, {-raw-}[TVar 2] =: tau2] $
  				 		RET [OP1 (CNV {-sem-}(formatOfType{-sem-}(ptrDiffType))) (ZG`FMT`standardEncoding) {--} --
  				 			{-hv-}(OP2 ({-hv-}(OP2 (VAR (TVar 1)) (SUB) ({-sem-}(formatOfType{--}(t1))) (VAR (TVar 2)))) --
		  				 		(DIV) (ZG`FMT`standardEncoding) (IMM (toRational --
		  				 			{-sem-}(sizeOfType{--}(baseOfType{--}(t1 `compositeType` t2)))) (ZG`FMT`standardEncoding)))])) {-!-} {-!-} {-!-}
\end{code}

Intuitively, if "p" is a pointer to the "n"-th element of some C array object and "i" is an integer, then the operations “"p `Sum` i"\;”, “"i `Sum` p"” and “"i `Difference` p"”  construct pointers to the element at the location "n + i" or "n - i" in the same array, provided that such an element exists. Further, if "p1" and "p2" represent two pointers into the same C array, then “"p1 `Difference` p2"\!” determines the number of elements in the array segment delimited by these two locations.

The next group of six expression forms, constructed from applications of the binary C operators “"`Remainder`"”, “"`LeftShift`"”, “"`RightShift`"”, “"`And`"”, “"`ExclusiveOr`"” and “\!"`InclusiveOr`"\!” to pairs of value expressions, represents the family of «binary integral operations». The semantics of such expressions differ from their earlier binary arithmetic cousins only in that their operands are restricted to integral, rather than arbitrary arithmetic types. The precise denotation of every well-formed binary integral operation is described completely by the following Haskell construction:

\IndexTerm		{meaningOfIntegralOperation}			{D_IO expression expression}
\InsertHaskell		{meaningOfIntegralOperation}

In particular, the six C operators “"`Remainder`"”, “"`LeftShift`"”, “"`RightShift`"”, “"`And`"”, “"`ExclusiveOr`"” and “\!"`InclusiveOr`"\!” represent binary integral operations characterised by the Etude operators “"REM{-sub-}phi"\!”, “"SHL{-sub-}phi"\!”, “"SHR{-sub-}phi"\!”, “"AND{-sub-}phi"\!”, “"XOR{-sub-}phi"\!” and “"IOR{-!-}{-sub-}phi"\!”, respectively:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (e1 {-hfil-}`Remainder` 	e2))))))))))))) ---
  = meaningOfIntegralOperation		{--}(_S, _T, _D, _I, {-sem-}({-hfil-}REM  )  {->-}, --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e1))))))))))), --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e2)))))))))))))
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (e1 {-hfil-}`LeftShift` 			e2))))))))))) ---
  = meaningOfIntegralOperation		{--}(_S, _T, _D, _I, {-sem-}({-hfil-}SHL  )  {->-}, --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 e1))))))))), --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e2)))))))))))
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (e1 {-hfil-}`RightShift` 		e2))))))))))) ---
  = meaningOfIntegralOperation		{--}(_S, _T, _D, _I, {-sem-}({-hfil-}SHR  )  {->-}, --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 e1))))))))), --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e2)))))))))))
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (e1 {-hfil-}`And` 					e2)))))))) ---
  = meaningOfIntegralOperation		{--}(_S, _T, _D, _I, {-sem-}({-hfil-}AND  )  {->-}, --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 e1)))))), --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 e2))))))))
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (e1 {-hfil-}`ExclusiveOr` 				e2))))))) ---
  = meaningOfIntegralOperation		{--}(_S, _T, _D, _I, {-sem-}({-hfil-}XOR  )  {->-}, --
  						E1 (E2 (E3 (E4 (E5 (E6 e1))))), --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 e2)))))))
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (e1 {-hfil-}`InclusiveOr` 					e2)))))) ---
  = meaningOfIntegralOperation		{--}(_S, _T, _D, _I, {-sem-}({-hfil-}IOR  )  {->-}, E1 (E2 (E3 (E4 (E5 e1)))), --
  						E1 (E2 (E3 (E4 (E5 (E6 e2))))))
\end{code}

For a further information about the resulting Etude constructions, the reader is referred to the earlier discussion of their properties in Section \ref{Lambda/Atoms}.

Similarly, each of the four «relational operations» “"e1 `LessThan` e2"\!”, “"e1 `GreaterThan` e2"\!”, “"e1 `LessOrEqualTo` e2"\!” and “"e1 `GreaterOrEqualTo` e2"\!” can be also described by one of the binary Etude operators “"CLT{-sub-}phi"\!”, “"CGT{-sub-}phi"\!”, “"CLE{-sub-}phi"\!” or “"CGE{-sub-}phi"\!”, using the following common derivation of its meaning:

\IndexTerm		{meaningOfRelationalOperation}			{D_RO expression expression}
\InsertHaskell		{meaningOfRelationalOperation}

Intuitively, in every such operation, either both of the operands must have arithmetic types, or else both operands must represent pointers to qualified or unqualified versions of compatible object or incomplete types. Either way, the result has the plain “"PlainIntType"” type and a denotation that applies the corresponding Etude operator "op" to the values of both operands, converted into their common arithmetic type. Specifically, the four C operators “"`LessThan`"”, “"`GreaterThan`"”, “"`LessOrEqualTo`"” and “"`GreaterOrEqualTo`"” are typified by their respective Etude equivalents “"CLT{-sub-}phi"\!”, “"CGT{-sub-}phi"\!”, “"CLE{-sub-}phi"\!” and “"CGE{-sub-}phi"\!” as follows:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (e1 {-hfil-}`LessThan` 			e2)))))))))) ---
  = meaningOfRelationalOperation	{--}(_S, _T, _D, _I, {-sem-}(CLT)  {->-}, --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 e1)))))))), --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 e2))))))))))
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (e1 {-hfil-}`GreaterThan` 			e2)))))))))) ---
  = meaningOfRelationalOperation	{--}(_S, _T, _D, _I, {-sem-}(CGT)  {->-}, --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 e1)))))))), --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 e2))))))))))
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (e1 {-hfil-}`LessOrEqualTo` 			e2)))))))))) ---
  = meaningOfRelationalOperation	{--}(_S, _T, _D, _I, {-sem-}(CLE)  {->-}, --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 e1)))))))), --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 e2))))))))))
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (e1 {-hfil-}`GreaterOrEqualTo` 		e2)))))))))) ---
  = meaningOfRelationalOperation	{--}(_S, _T, _D, _I, {-sem-}(CGE)  {->-}, --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 e1)))))))), --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 e2))))))))))
\end{code}

Further, a closely-related pair of «equality operations» “"e1 `EqualTo` e2"\!” and “"e1 `NotEqualTo` e2"\!” corresponds to the binary Etude operators “"CEQ{-sub-}phi"\!” and “"CNE{-sub-}phi"\!” as follows:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (e1 `EqualTo` e2))))))))) ---
  = meaningOfEqualityOperation		{--}(_S, _T, _D, _I, {-sem-}(CEQ)  {->-}, --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 e1))))))), --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 e2)))))))))
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (e1 `NotEqualTo` e2))))))))) ---
  = meaningOfEqualityOperation		{--}(_S, _T, _D, _I, {-sem-}(CNE)  {->-}, --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 e1))))))), --
  						E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 e2)))))))))
\end{code}

However, equality operators accept a broader range of operands then those supported by the earlier relational operation forms. In particular, they can be also applied in comparison of two function pointers, a pointer to an object against a “"VoidType"\,” pointer and for comparison of arbitrary pointer values against the integer constant “"integerConstantZero"”, optionally cast into the “"PointerType (VoidType) {-*-}"” type as described in Section \ref{C/Expressions/Constants/NullPointers}. Formally, every application of the C operator “"`EqualTo`"” and “"`NotEqualTo`"” must satisfy one of the following conditions:

\nobreak

\begin{list}

•	both operands must constitute values of arithmetic types, or

• 	both operands must represent pointers to qualified or unqualified versions of compatible types, or

•	one operand must represent a pointer to an object or incomplete type and the other must describe a pointer to a qualified or unqualified version of “"VoidType"\,”, or

•	one operand must be a pointer and the other must constitute a «null pointer constant», of the form described later in Section \ref{C/Expressions/Constants/NullPointers}.

\end{list}

In Haskell:

\IndexTerm		{meaningOfEqualityOperation}			{D_EO expression expression}
\InsertHaskell		{meaningOfEqualityOperation}

On the other hand, in all «logical operations» of the form “"e1 `LogicalAnd` e2"\!” and \hbox{“"e1 `LogicalOr` e2"\!”,} both operands may represent arbitrary scalar values and the entire construction always assumes the plain “"PlainIntType"” type. In particular, a «logical AND operation» of the form \hbox{“"e1 `LogicalAnd` e2"\!”} returns the integer constant “"integerConstantOne"\!” if both of its operands have non-zero values and “"integerConstantZero"” otherwise. Specifically, it begins by evaluating the first operand "e1" and comparing its atomic value for inequality with the constant “"integerConstantZero"” of an appropriate C type. If "e1" has a non-zero value, then the operation evaluates its second operand "e2", delivering the comparison of its atomic value against “"integerConstantZero"”. Otherwise, "e2" is never evaluated and the entire expression delivers the constant value of “"integerConstantZero"” as a result of the entire operation:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (e1 `LogicalAnd` e2))))) = do
	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfValueExpression{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 (E4 e1))))
	(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression{--}(_S1,  _T1,  _D1,  _I {->-}, E1 (E2 (E3 (E4 (E5 e2)))))
	require $ (isScalarType{--}(t1) && isScalarType{--}(t2))
	return $	(_S2, _T2, _D2, _V1 `mapUnion` _V2, {-sem-}(PlainIntType),
			{-quad-}{-sem-}( ---
  				LET [{-raw-}[TVar 1] =: tau1] $
  				IFF (OP2 (VAR (TVar 1)) (CNE) ({-sem-}(formatOfType{--}(t1))) (IMM (0) ({-sem-}(formatOfType{--}(t1))))) {-THEN-} (
				{-quad-} 	LET [{-raw-}[TVar 2] =: tau2] $
  				 	 	RET [OP2 (VAR (TVar 2)) (CNE) ({-sem-}(formatOfType{--}(t2))) (IMM (0) ({-sem-}(formatOfType{--}(t2))))])
  				{-ELSE-} (
  				{-quad-} 	RET [IMM (0) ({-sem-}(formatOfType{--}(t1)))])))
\end{code}

Similarly, a «logical OR operation» of the form “"e1 `LogicalOr` e2"\!” returns “"integerConstantOne"\!” if and only if at least one of its operands has a non-zero value. First, it compares the operand "e1" against “"integerConstantZero"” and always delivers “"integerConstantOne"\!” if "e1" has a non-zero value, without ever examining the denotation of its second operand "e2". Otherwise, the entire operation delivers the result of comparing "e2" against “"integerConstantOne"\!” as follows:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (e1 `LogicalOr` e2)))) = do
	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfValueExpression{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 e1)))
	(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression{--}(_S1,  _T1,  _D1,  _I {->-}, E1 (E2 (E3 (E4 e2))))
	require $ (isScalarType{--}(t1) && isScalarType{--}(t2))
	return $	(_S2, _T2, _D2, _V1 `mapUnion` _V2, {-sem-}(PlainIntType),
			{-quad-}{-sem-}( ---
				LET [{-raw-}[TVar 1] =: tau1] $
  				IFF (OP2 (VAR (TVar 1)) (CNE) ({-sem-}(formatOfType{--}(t1))) (IMM (0) ({-sem-}(formatOfType{--}(t1))))) {-THEN-} (
  				{-quad-}	RET [IMM (1) ({-sem-}(formatOfType{--}(t1)))])
  				{-ELSE-} (
  				{-quad-}	LET [{-raw-}[TVar 2] =: tau2] $
  				 	 	RET [OP2 (VAR (TVar 2)) (CNE) ({-sem-}(formatOfType{--}(t2))) (IMM (0) ({-sem-}(formatOfType{--}(t2))))])))
\end{code}

Finally, the meaning of all «conditional operations» of the form “"Conditional (e1) {-?-} (e2) {-:-} (e3)"\!” can be formalised in Haskell as follows:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (Conditional (e1) {-?-} (e2) {-:-} (e3)))) = do
	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfValueExpression{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 e1)))
	(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression{--}(_S1,  _T1,  _D1,  _I {->-}, e2)
	(_S3, _T3, _D3, _V3, t3, tau3) <- meaningOfValueExpression{--}(_S2,  _T2,  _D2,  _I {->-}, E1 (E2 e3))
	require $ 	(isScalarType{--}(t1)) &&
			(	(isArithmeticType{--}(t2) 	&& isArithmeticType{--}(t3)) ||
				(isCompoundType{--}(t2) 	&& unqualifiedType{--}(t2) `areCompatibleTypes` unqualifiedType{--}(t3)) ||
				(isPointerType{--}(t2) 		&& isPointerType{--}(t3) && --
					unqualifiedType{--}(baseOfType{--}(t2)) `areCompatibleTypes` unqualifiedType{--}(baseOfType{--}(t3))) ||
				(isPointerType{--}(t2) 		&& isNullPointerConstant{--}(_S2, _T2, _D2, _I {->-}, E1 (E2 e3))) ||
				(isPointerType{--}(t3) 		&& isNullPointerConstant{--}(_S1, _T1, _D1, _I {->-}, e2)) ||
				(isPointerType{--}(t2) 		&& isPointerType{--}(t3) && --
					(	((isObjectType{--}(baseOfType{--}(t2)) || isIncompleteType{--}(baseOfType{--}(t2))) && isVoidType{--}(baseOfType{--}(t3))) ||
					 	((isObjectType{--}(baseOfType{--}(t3)) || isIncompleteType{--}(baseOfType{--}(t3))) && isVoidType{--}(baseOfType{--}(t2))))))
	return $  (_S3, _T3, _D3, _V1 `mapUnion` _V2 `mapUnion` _V3, t2 `commonArithmeticType` t3,
		  {-quad-}{-sem-}( ---
				 LET [{-raw-}[TVar 1] =: tau1] $
				 IFF (OP2 (VAR (TVar 1)) (CNE) ({-sem-}(formatOfType{--}(t1))) (IMM (0) ({-sem-}(formatOfType{--}(t1))))) {-THEN-} (
				 {-quad-}	 LET [{-raw-}[TVar 2] =: tau2] $
						 RET [OP1 (CNV ({-sem-}(formatOfType{--}(t2 `commonArithmeticType` t3)))) ({-sem-}(formatOfType{--}(t2))) {--} {-hv-}(VAR (TVar 2))])
				 {-ELSE-} (
				 {-quad-}	 LET [{-raw-}[TVar 3] =: tau3] $
						 RET [OP1 (CNV ({-sem-}(formatOfType{--}(t2 `commonArithmeticType` t3)))) ({-sem-}(formatOfType{--}(t3))) {--} {-hv-}(VAR (TVar 3))])))
\end{code}

In other words, such constructs always assume the common arithmetic type of their second and third operand. Their denotations represent monadic Etude terms that evaluate "e1" and return the appropriately-converted denotation of either "e2" or "e3", whenever the first operand has a non-zero or zero value, respectively. The reader should observe that, in all cases, at most one of the two operands "e2" or "e3" is ever reduced on any given execution path through the program. At any rate, in every such conditional operation, the first operand "e1" must have a scalar type and the other two expressions must represent values that satisfy one of the following five conditions:

\nobreak

\begin{list}

•	both of these operands represent values of an arithmetic type, or

•	both of these operands represent qualified or unqualified versions of compatible structure or union object types, or

•	both represent pointers to qualified or unqualified versions of compatible types, or

•	either operand represents is a pointer and the other is a null pointer constant, or

•	one operand represents a pointer to an object or incomplete type and the other represents a pointer to a qualified or unqualified version of the “"VoidType"\,” type.

\end{list}

The denotation of a comma operation “"e1 {-!-} `Comma` {-!-} e2"\!” is rather simpler. Intuitively, in such expressions the first operand "e1" may represent an arbitrary function designator, l-value, value or void expression and the second operand must represent a value, whose type is also inherited by the entire construct. A denotation of every comma operation evaluates both of its arguments in sequence, discarding the result of "e1" and delivering that of the second operand "e2". Formally:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, e1 {-!-} `Comma` {-!-} e2) = do
	(_S1, _T1, _D1, _V1, tau1) <- meaningOfExpressionOpt{--}(_S, _T, _D, _I {->-}, Just e1)
	(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression{--}(_S1, _T1, _D1, _I {->-}, E1 e2)
	return $ (_S2, _T2, _D2, _V1 `mapUnion` _V2, t2, {-sem-}(LET [{-hv-}[] =: tau1] $ tau2))
\end{code}

All of the remaining value operators “"SimpleAssignment"”, “"CompoundProduct"”, “"CompoundQuotient"”, “"CompoundRemainder"”, “"CompoundSum"”, “"CompoundDifference"”, “"CompoundLeftShift"”, “"CompoundRightShift"”, “"CompoundAnd"”, “"CompoundExclusiveOr"”, “"CompoundInclusiveOr"”, “"Increment"” and “"Decrement"” introduce the set of fifteen «assignment operation» forms, intended to facilitate modification of values contained in memory-resident objects. The first operand to every assignment operation must always represent a modifiable l-value. The entire construct assumes that l-value's type, delivering the updated content of the underlying Etude object.

In particular, a «simple assignment operation» of the form “"Assignment (e1) (SimpleAssignment) (e2)"\!” replenishes the object designated by the l-value "e1" with an appropriately-converted value of "e2", provided that both expressions satisfy one of the following conditions:

\nobreak

\begin{list}

•	both operands have arithmetic types, or

•	"e1" has a structure or union type, whose unqualified version is compatible with the type of "e2" and both operands represent object types, or

•	both "e1" and "e2" represent pointers to qualified or unqualified versions of compatible types and the referenced type of "e2" is qualified with a subset of the qualifications associated with the type referenced by "e1", or

•	one of the two expressions represents a pointer to an object or incomplete type, the other is a pointer to the “"VoidType"\,” type and the referenced type of "e2" is qualified with a subset of the qualifications associated with the type referenced by "e1", or

•	"e1" has a pointer type and "e2" is a null pointer constant.

\end{list}

In all cases, the entire operation has the type of the first operand "e1". Its denotation evaluates, in an unspecified order, both the l-value "e1" and the value of "e2", with the later converted into the type of the entire operation. Next, the converted value of "e2" is placed into the memory-resident object designated by "e1" as follows:

\nobreak

\begin{list}

•	if the "e1" has a bit field type, then the existing value of the entire bit field container at that memory location is retrieved and replenished with the value of "e2", using the unspecified "packBitField" combinator described earlier in this section, with the result stored back into the same memory location. 

•	On the other hand, if "e2" has any other scalar type, then the object designated by its l-value is replenished with the value of "e2", converted into the first operand's type. 

•	Finally, if "e1" represents a structure or union l-value, then the entire structure or union object depicted by the denotation of "e2" is copies into that l-value using an unspecified function “"setObject"”, whose typical implementation is given in Appendix \ref{C/Implementation}.

\end{list}

In Haskell:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (Assignment (e1) (SimpleAssignment) (e2)))

  =	do	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfModifiableLValue	{--}(_S,   _T,   _D,   _I {->-}, ---
  											E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e1))))))))))))))
  		(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression	{--}(_S1,  _T1,  _D1,  _I {->-}, E1 e2)
		require $	(isBitFieldType{--}(t1) && isArithmeticType{--}(t2))
		return $  (_S2, _T2, _D2, _V1 `mapUnion` _V2, t1,
			  {-quad-}{-sem-}( ---
					LET [{-raw-}[TVar 1] 	=: tau1 {-hv-}, {-raw-}[TVar 2] =: tau2] $
				 	LET [{-raw-}[TVar 3] 	=: GET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t1))) (bitFieldFormat)] $
				 	LET [{-raw-}[TVar 4] 	=: RET  [OP1 (CNV ({-sem-}(formatOfType{--}(t1)))) ({-sem-}(formatOfType{--}(t2))) {--} {-hv-}(VAR (TVar 2))]] $
				 	LET [{-hv-}[]		=: SET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t1))) (bitFieldFormat) --
				 						{-TO-} ({-sem-}(packBitField{--}(t1 {->-}, VAR (TVar 3), VAR (TVar 4))))] $ --
				 								{-!-}{-!-}{-!-}{-!-}{-!-}
				 	RET [VAR (TVar 4)]))

  <|>	do	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfModifiableLValue	{--}(_S,   _T,   _D,   _I {->-}, ---
  											E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e1))))))))))))))
  		(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression	{--}(_S1,  _T1,  _D1,  _I {->-}, E1 e2)
		require $	(isArithmeticType{--}(t1)  && isArithmeticType{--}(t2)) ||
				(isPointerType{--}(t1) 	   && isPointerType{--}(t2) && --
					(	(unqualifiedType{--}(baseOfType{--}(t1)) `areCompatibleTypes` unqualifiedType{--}(baseOfType{--}(t2))) ||
						((isObjectType{--}(baseOfType{--}(t1)) || isIncompleteType{--}(baseOfType{--}(t1))) && isVoidType{--}(baseOfType{--}(t2))) ||
						((isObjectType{--}(baseOfType{--}(t2)) || isIncompleteType{--}(baseOfType{--}(t2))) && isVoidType{--}(baseOfType{--}(t1)))) &&
					(tqOfType{--}(baseOfType{--}(t2)) `subseteq` tqOfType{--}(baseOfType{--}(t1)))) || 
				(isPointerType{--}(t1) && isNullPointerConstant{--}(_S, _T, _D, _I {->-}, E1 e2))
		return $  (_S2, _T2, _D2, _V1 `mapUnion` _V2, t1,
			  {-quad-}{-sem-}( ---
					LET [{-raw-}[TVar 1]	=: tau1 {-hv-}, {-raw-}[TVar 2] =: tau2] $
				 	LET [{-raw-}[TVar 3]	=: RET [OP1 (CNV ({-sem-}(formatOfType{--}(t1)))) ({-sem-}(formatOfType{--}(t2))) {--} {-hv-}(VAR (TVar 2))]] $
				 	LET [{-hv-}[] 		=: SET (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t1))) ({-sem-}(formatOfType{--}(t1))) {-TO-} (VAR (TVar 3))] $
			 	 	RET [VAR (TVar 3)]))

  <|>	do	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfModifiableLValue	{--}(_S,   _T,   _D,   _I {->-}, ---
  											E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e1))))))))))))))
  		(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression	{--}(_S1,  _T1,  _D1,  _I {->-}, E1 e2)
		require $	(isCompoundType{--}(t1) && isObjectType{--}(t1) && unqualifiedType{--}(t1) `areCompatibleTypes` unqualifiedType{--}(t2))
		return $  (_S2, _T2, _D2, _V1 `mapUnion` _V2, t1,
			  {-quad-}{-sem-}( ---
					LET [{-raw-}[TVar 1] 	=: tau1 {-hv-}, {-raw-}[TVar 2] =: tau2] $
				 	LET [{-hv-}[]		=: {-sem-}(setObject ({-sem-}(t1 {->-}, VAR (TVar 1))) {-to-} ({-sem-}(t2 {->-}, VAR (TVar 2))))] $
			 	 	RET [VAR (TVar 1)]))
\end{code}

Intuitively, the construction “"setObject ((t1 {->-}, alpha1)) {-to-} ((t2 {->-}, alpha2))"” which appears in the above definition of structure and union assignments, duplicates the structure and content of an entire address space region described by the envelope of "t2" from the location given by the atom "alpha2" into a similarly-structured region beginning at the address "alpha1" and moulded into the envelope "envelopeOfType{--}(t1)". Since the precise semantics of this construction are left unspecified by the C Standard and, at any rate, cannot be captured exactly within the regime of the generic Etude fragment, its implementation is characterised in this chapter only by the following Haskell type signature:

\ReindexTerm		{setObject}

\begin{haskell}
setObject {-[*]-} {-to-} {-[*]-} :: (TypeT {->-}, AtomT VarT) -> (TypeT {->-}, AtomT VarT) -> TermT VarT
\end{haskell}

On every Etude implementation, the term “"setObject ((t1 {->-}, alpha1)) {-to-} ((t2 {->-}, alpha2))"” is always guaranteed to reduce into a trivial “"RET []"”, in the context of some appropriately-updated object environment "_Delta `copyOEnv` (sigma1, xi_set1, sigma2, xi_set2)", as long as that environment is well-formed, "alpha1" and "alpha2" represent meaningful object atoms of appropriate formats, the two address space regions have identical sizes and either do not overlap at all, or else the overlap is exact, "t1" depicts a recursively-unqualified type and, finally, "t2" does not mention the “"VolatileQualifier"” type qualifier:

\NewHaskellToken	{setObjectJ}				{Ord}	{\TKeyword{set}}
\NewHaskellDescription	{setObjectJ}					{Properties of the “\haskell{setObject (..) {-to-} (..)}” operation}
\IndexTheorem		{setObjectJ}					{SET Properties of the SET_TO operation}

\begin{haskell}
setObjectJ :: forall _Lambda, _Delta, t1, sigma1, t2, sigma2 =>
	WfJ{--}(_Lambda) -> --
	WfJ{-sem-}(_Delta `copyOEnv` (sigma1, envelopeOfType{--}(t1), sigma2, envelopeOfType{--}(t2))) -> --
	WfJ{-sem-}(IMM (sigma1) (objectFormat{--}(formatOfType{--}(t1)))) -> --
	WfJ{-sem-}(IMM (sigma2) (objectFormat{--}(formatOfType{--}(t2)))) ->
	{-sem-}(sizeOfType{--}(t1) == sizeOfType{--}(t2) && (sigma1 + sizeOfType{--}(t1) <= sigma2 || sigma1 >= sigma2 + sizeOfType{--}(t2) || sigma1 == sigma2)) ->
	{-sem-}(recTqOfType{--}(t1) == emptySet && recTqOfType{--}(t2) `subseteq` listToSet [CM]) ->
	{-sem-}(_Lambda, _Delta {->-}, setObject ({-sem-}(t1 {->-}, IMM (sigma1) (objectFormat{--}(formatOfType{--}(t1))))) --
							{-to-} ({-sem-}(t2 {->-}, IMM (sigma2) (objectFormat{--}(formatOfType{--}(t2))))))
	{-quad-}`EqvJ` {-sem-}(_Lambda, _Delta `copyOEnv` (sigma1, envelopeOfType{--}(t1), sigma2, envelopeOfType{--}(t2)) {->-}, RET [])
\end{haskell}

in which the notation “"_Delta `copyOEnv` (sigma1, xi_set1, sigma2, xi_set2)"” depicts the environment derived by “"setObject"”:

\BeginHaskellToken	{copyOEnv}				{Void}	{}
  \HaskellArgument							{OEnvT}
  \HaskellToken		{`copyOEnv`}				{Bin}	{\hbox{\slash\!\!\!\slash}}
  \HaskellArgument							{(Integer, EnvelopeT, Integer, EnvelopeT)}
  \HaskellResult							{OEnvT}
\EndHaskellToken

\NewHaskellToken	{`copyOEnv`}				{Op}	{\hbox{\slash\!\!\!\slash}}

\IndexTerm		{copyOEnv}					{\slash\slash o_env integer envelope integer envelope (o_env)}

\begin{haskell}
copyOEnv {-[*]-} `copyOEnv` {-[*]-} :: OEnvT -> (Integer, EnvelopeT, Integer, EnvelopeT) -> OEnvT
\end{haskell}

Similarly to all other object environment operations from Section \ref{Lambda/State}, the minimal algebraic properties of this function on every Etude implementation are captured by the following pair of Haskell theorems:

\NewHaskellToken	{copyOEnvAtomJ}				{Ord}	{\TKeyword{copy\sub{\alpha}}}
\NewHaskellDescription	{copyOEnvAtomJ}					{Object environment binding after a copy operation}
\IndexTheorem		{copyOEnvAtomJ}					{SET_A Object environment binding after a copy operation}

\NewHaskellToken	{copyOEnvInitJ}				{Ord}	{\TKeyword{copy\sub{\alpha}}}
\NewHaskellDescription	{copyOEnvInitJ}					{Object environment initialisers after a copy operation}
\IndexTheorem		{copyOEnvInitJ}					{SET_A Object environment initialisers after a copy operation}

\begin{haskell}
copyOEnvAtomJ 		:: forall _Delta, sigma1, xi_set1, sigma2, xi_set2, sigma, phi =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `copyOEnv` (sigma1, xi_set1, sigma2, xi_set2)) ->
	{-sem-}({-hv-}(n {-hv-}, phi {-hv-}, emptySet) 		`inEnvelope` xi_set1 && --
		{-hv-}(sigma1 + n {-hv-}, phi {-hv-}, {-hfil-}emptySet		) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}({-hv-}(n {-hv-}, phi {-hv-}, listToSet[CM]) 	`inEnvelope` xi_set2 && --
		{-hv-}(sigma2 + n {-hv-}, phi {-hv-}, {-hfil-}listToSet[CM]	) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(atomOfOEnv{--}(_Delta `copyOEnv` (sigma1, xi_set1, sigma2, xi_set2) {->-}, sigma1 + n, phi)) --
			`EqvJ` {-sem-}(atomOfOEnv{--}(_Delta `copyOEnv` (sigma1, xi_set1, sigma2, xi_set2) {->-}, sigma2 + n, phi))

copyOEnvInitJ		:: forall _Delta, sigma1, xi_set2, sigma2, xi_set2 =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `copyOEnv` (sigma1, xi_set1, sigma2, xi_set2)) ->
	{-sem-}(initOfOEnv{--}(_Delta `copyOEnv` (sigma1, xi_set1, sigma2, xi_set2)) == initOfOEnv{--}(_Delta) `purgeEnvelope` (sigma1, sigma1 + sizeOfEnvelope{--}(xi_set1)))
\end{haskell}

Such well-formed object update operations will never affect the contents of any non-overlapping memory-resident objects currently allocated in the same address space, or for that matter, any other properties of the original object environment:

\NewHaskellToken	{copyOEnvAtomsJ}			{Ord}	{\TKeyword{copy\sub{\macron\alpha}}}
\NewHaskellDescription	{copyOEnvAtomsJ}				{Object environment data after a copy operation}
\IndexTheorem		{copyOEnvAtomsJ}				{SET_A Object environment data after a copy operation}

\NewHaskellToken	{copyOEnvEnvelopeJ}			{Ord}	{\TKeyword{copy\sub{e}}}
\NewHaskellDescription	{copyOEnvEnvelopeJ}				{Object environment envelope after a copy operation}
\IndexTheorem		{copyOEnvEnvelopeJ}				{SET_E Object environment envelope after a copy operation}

\NewHaskellToken	{copyOEnvStackJ}			{Ord}	{\TKeyword{copy\sub{s}}}
\NewHaskellDescription	{copyOEnvStackJ}				{Object environment stack after a copy operation}
\IndexTheorem		{copyOEnvStackJ}				{SET_S Object environment stack after a copy operation}

\NewHaskellToken	{copyOEnvExtensionJ}			{Ord}	{\TKeyword{copy\sub{x}}}
\NewHaskellDescription	{copyOEnvExtensionJ}				{Object environment extensions after a copy operation}
\IndexTheorem		{copyOEnvExtensionJ}				{SET_X Object environment extensions after a copy operation}

\NewHaskellToken	{copyOEnvNewJ}				{Ord}	{\TKeyword{copy\sub{n}}}
\NewHaskellDescription	{copyOEnvNewJ}					{Object environment stack pointer after a copy operation}
\IndexTheorem		{copyOEnvNewJ}					{SET_X Object environment stack pointer after a copy operation}

\begin{haskell}
copyOEnvAtomsJ 		:: forall _Delta, sigma1, xi_set1, sigma2, xi_set2, sigma_k, phi_k, mu_set_k =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `copyOEnv` (sigma1, xi_set1, sigma2, xi_set2)) ->
	{-sem-}({-hv-}(sigma_k {-hv-},  phi_k {-hv-},  mu_set_k)  `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(sigma_k + sizeOfFormat{--}(phi_k) <= sigma1 || sigma_k >= sigma1 + sizeOfEnvelope{--}(xi_set1)) ->
	{-sem-}(atomOfOEnv{--}(_Delta `copyOEnv` (sigma1, xi_set1, sigma2, xi_set2) {->-}, sigma_k, phi_k)) `EqvJ` {-sem-}(atomOfOEnv{--}(_Delta {->-}, sigma_k, phi_k))

copyOEnvEnvelopeJ 	:: forall _Delta, sigma1, xi_set1, sigma2, xi_set2 =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `copyOEnv` (sigma1, xi_set1, sigma2, xi_set2)) ->
	{-sem-}(envelopeOfOEnv{--}(_Delta `copyOEnv` (sigma1, xi_set1, sigma2, xi_set2)) == envelopeOfOEnv{--}(_Delta))

copyOEnvStackJ 	:: forall _Delta, sigma1, xi_set1, sigma2, xi_set2 =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `copyOEnv` (sigma1, xi_set1, sigma2, xi_set2)) ->
	{-sem-}(stackOfOEnv{--}(_Delta `copyOEnv` (sigma1, xi_set1, sigma2, xi_set2)) == stackOfOEnv{--}(_Delta))

copyOEnvExtensionJ 	:: forall _Delta, sigma1, xi_set1, sigma2, xi_set2, xi_set =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `copyOEnv` (sigma1, xi_set1, sigma2, xi_set2)) -> WfJ{-sem-}(_Delta `extendOEnv` xi_set) ->
	WfJ{-sem-}((_Delta `copyOEnv` (sigma1, xi_set1, sigma2, xi_set2)) `extendOEnv` xi_set)

copyOEnvNewJ 		:: forall _Delta, sigma1, xi_set1, sigma2, xi_set2, xi_set =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `copyOEnv` (sigma1, xi_set1, sigma2, xi_set2)) -> WfJ{-sem-}(_Delta `extendOEnv` xi_set) ->
	{-sem-}(newInOEnv{--}(_Delta `copyOEnv` (sigma1, xi_set1, sigma2, xi_set2) {->-}, xi_set) == newInOEnv{--}(_Delta {->-}, xi_set))
\end{haskell}

On the other hand, all «compound assignments» of the form  “"e1 op{--}SimpleAssignment e2"” are essentially equivalent to the simple assignment “"e1 SimpleAssignment e1 op e2"”, effectively replenishing the targeted l-value with the result of applying the binary operator "op" to its existing content and the value of the second operand "e2", except that "e1" is evaluated only once by every instance of such an operation.

In all assignments of the form “"e1 CompoundRemainder e2"\!”, “"e1 CompoundLeftShift e2"\!”, “"e1 CompoundRightShift e2"\!”, “"e1 CompoundAnd e2"\!”, “"e1 CompoundExclusiveOr e2"\!” and “"e1 CompoundInclusiveOr e2"\!”, both the l-value operand "e1" and the value "e2" must have an integral type. A formal denotation of such constructs is depicted by the notation “"meaningOfIntegralAssignment{--}(_S, _T, _D, _I, op {->-}, e1, e2)"”, in which "op" is the corresponding Etude operator “"REM{-sub-}phi"\!”, “"SHL{-sub-}phi"\!”, “"SHR{-sub-}phi"\!”, “"AND{-sub-}phi"\!”, “"XOR{-sub-}phi"\!” or “"IOR{-!-}{-sub-}phi"\!”. In Haskell, this notation is implemented as follows:

\IndexTerm		{meaningOfIntegralAssignment}			{D_IA expression binary_op expression}
\InsertHaskell		{meaningOfIntegralAssignment}

Specifically, each of this compound assignment forms is mapped onto its Etude equivalent by the following set of Haskell definitions:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (Assignment (e1) ({-hfil-}CompoundRemainder		) (e2))) ---
  = meaningOfIntegralAssignment		{--}(_S, _T, _D, _I, {-sem-}({-hfil-}REM  ) {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e1))))))))))))), E1 e2)
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (Assignment (e1) ({-hfil-}CompoundLeftShift		) (e2))) ---
  = meaningOfIntegralAssignment		{--}(_S, _T, _D, _I, {-sem-}({-hfil-}SHL  ) {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e1))))))))))))), E1 e2)
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (Assignment (e1) ({-hfil-}CompoundRightShift		) (e2))) ---
  = meaningOfIntegralAssignment		{--}(_S, _T, _D, _I, {-sem-}({-hfil-}SHR  ) {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e1))))))))))))), E1 e2)
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (Assignment (e1) ({-hfil-}CompoundAnd			) (e2))) ---
  = meaningOfIntegralAssignment		{--}(_S, _T, _D, _I, {-sem-}({-hfil-}AND  ) {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e1))))))))))))), E1 e2)
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (Assignment (e1) ({-hfil-}CompoundExclusiveOr		) (e2))) ---
  = meaningOfIntegralAssignment		{--}(_S, _T, _D, _I, {-sem-}({-hfil-}XOR  ) {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e1))))))))))))), E1 e2)
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (Assignment (e1) ({-hfil-}CompoundInclusiveOr		) (e2))) ---
  = meaningOfIntegralAssignment		{--}(_S, _T, _D, _I, {-sem-}({-hfil-}IOR  ) {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e1))))))))))))), E1 e2)
\end{code}

Similarly, in the compound assignments “"e1 CompoundProduct e2"\!” and “"e1 CompoundQuotient e2"\!”, the two operands "e1" and "e2" may assume arbitrary arithmetic types, with the entire denotation derived from the corresponding Etude operator “"MUL{-sub-}phi"\!” or “"DIV{-sub-}phi"\!” by the following function "meaningOfArithmeticAssignment":

\IndexTerm		{meaningOfArithmeticAssignment}			{D_AA expression binary_op expression}
\InsertHaskell		{meaningOfArithmeticAssignment}

so that the actual meanings of these assignments can be captured in Haskell using the following translations:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (Assignment (e1) ({-hfil-}CompoundProduct		) (e2))) ---
  = meaningOfArithmeticAssignment	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}MUL  )  {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e1))))))))))))), E1 e2)
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (Assignment (e1) ({-hfil-}CompoundQuotient		) (e2))) ---
  = meaningOfArithmeticAssignment	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}DIV  )  {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e1))))))))))))), E1 e2)
\end{code}

The final two compound assignment forms “"e1 CompoundSum e2"\!” and “"e1 CompoundDifference e2"\!” also accept a pair of arithmetic operands, applying their corresponding Etude operators “"ADD{-sub-}phi"\!” or “"SUB{-sub-}phi"\!” in the constructed denotation. In addition, their l-value operand "e1" may also represent a pointer to an object type, in which case the value operand "e2" must have an integral type. As in the earlier additive expression forms, the value of "e2" is multiplied by the size of the pointer's referenced type, before adding it to or subtracting it from the value of the pointer "e1". In effect, given a pointer variable "p" which, at the beginning of the operation, contains the address of the "n"-th element in some C array object, the compound assignment “"p CompoundSum i"\,” sets "p" to the location of the element "n + i" in the same array. Conversely, “"p CompoundDifference i"\,”, sets "p" to the element "n - i", which, in Etude, is achieved by adding "i * negate sizeOfType{--}(baseOfType{--}(t))" to the original "p"'s value, assuming that "t" represents the type of the construction's l-value operand. In this work, the meaning of all such compound assignment forms is formalised by the following semantic translations:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (Assignment (e1) (CompoundSum) (e2)))

	=	meaningOfArithmeticAssignment{--}(_S, _T, _D, _I, {-sem-}(ADD) {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e1))))))))))))), E1 e2)

	<|>	do	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfModifiableLValue	{--}(_S,   _T,   _D,   _I {->-}, --
  												E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e1))))))))))))))
  			(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression	{--}(_S1,  _T1,  _D1,  _I {->-}, E1 e2)
			require $ (isPointerType{--}(t1) && isObjectType{--}(baseOfType{--}(t1)) && isIntegralType{--}(t2))
			return $	(_S2, _T2, _D2, _V1 `mapUnion` _V2, t1,
					{-quad-}{-sem-}( ---
						LET [{-raw-}[TVar 1] 	=: tau1 {-hv-}, {-raw-}[TVar 2] =: tau2] $
					 	LET [{-raw-}[TVar 3] 	=: GET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t1))) ({-sem-}(formatOfType{--}(t1)))] $
				 		LET [{-raw-}[TVar 4] 	=: RET  [OP2 (VAR (TVar 3)) (ADD) ({-sem-}(formatOfType{--}(t1))) --
					 						({-hv-}(OP2 ({-hv-}(OP1 (CNV (ZG`FMT`standardEncoding)) --
					 								({-sem-}(formatOfType{--}(t2))) {--} {-hv-}(VAR (TVar 2)))) --
					 							(MUL) (ZG`FMT`standardEncoding) --
					 							(IMM (toRational {-sem-}(sizeOfType{--}(baseOfType{--}(t1)))) --
												(ZG`FMT`standardEncoding))))]] $
			 	 		LET [{-hv-}[] 		=: SET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t1))) ({-sem-}(formatOfType{--}(t1))) --
			 	 							{-TO-} (VAR (TVar 4))] $
			 	 		RET [VAR (TVar 4)]))

meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (Assignment (e1) (CompoundDifference) (e2)))

	=	meaningOfArithmeticAssignment{--}(_S, _T, _D, _I, {-sem-}(SUB) {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e1))))))))))))), E1 e2)

	<|>	do	(_S1, _T1, _D1, _V1, t1, tau1) <- meaningOfModifiableLValue	{--}(_S,   _T,   _D,   _I {->-}, --
  												E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e1))))))))))))))
  			(_S2, _T2, _D2, _V2, t2, tau2) <- meaningOfValueExpression	{--}(_S1,  _T1,  _D1,  _I {->-}, E1 e2)
			require $ (isPointerType{--}(t1) && isObjectType{--}(baseOfType{--}(t1)) && isIntegralType{--}(t2))
			return $	(_S2, _T2, _D2, _V1 `mapUnion` _V2, t1,
					{-quad-}{-sem-}( ---
						LET [{-raw-}[TVar 1] 	=: tau1 {-hv-}, {-raw-}[TVar 2] =: tau2] $
					 	LET [{-raw-}[TVar 3] 	=: GET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t1))) ({-sem-}(formatOfType{--}(t1)))] $
					 	LET [{-raw-}[TVar 4] 	=: RET  [OP2 (VAR (TVar 3)) (ADD) ({-sem-}(formatOfType{--}(t1))) --
					 						({-hv-}(OP2 ({-hv-}(OP1 (CNV (ZG`FMT`standardEncoding)) --
					 								({-sem-}(formatOfType{--}(t2))) {--} {-hv-}(VAR (TVar 2)))) --
					 							(MUL) (ZG`FMT`standardEncoding) --
					 							(IMM (toRational {-sem-}(negate (sizeOfType{--}(baseOfType{--}(t1))))) --
												(ZG`FMT`standardEncoding))))]] $
				 	 	LET [{-hv-}[]		=: SET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t1))) ({-sem-}(formatOfType{--}(t1))) --
				 	 						{-TO-} (VAR (TVar 4))] $
				 	 	RET [VAR (TVar 4)]))
\end{code}

In C, the four expression forms “"Increment (e)"”, “"PostIncrement (e) {-++-}"”, “"Decrement (e)"” and “"PostDecrement (e) {----}"” represent specialised cases of compound assignments used to increase or decrease the value of a modifiable object "e" by the integer constant “"integerConstantOne"\!”. In particular, the two prefix variants of these expressions with a concrete syntax of “"Increment (e)"” and “"Decrement (e)"” are equivalent precisely to the corresponding compound assignments \hbox{“"e CompoundSum integerConstantOne"\!”} and “"e CompoundDifference integerConstantOne"\!”:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Increment (e)))))))))))))))) ---
  = meaningOfValueExpression{-sem-}(_S, _T, _D, _I {->-}, --
  	E1 (Assignment (e) (CompoundSum) (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (integerConstantOne))))))))))))))))))
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Decrement (e)))))))))))))))) ---
  = meaningOfValueExpression{-sem-}(_S, _T, _D, _I {->-}, --
  	E1 (Assignment (e) (CompoundDifference) (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (integerConstantOne))))))))))))))))))
\end{code}

On the other hand, their postfix variants differ from “"e CompoundSum integerConstantOne"\!” and “"e CompoundDifference integerConstantOne"\!” in that the original, rather than the updated value of "e" is returned by each of these operation. As a result, the formal denotations of these expressions differ from those of the earlier compound assignment forms only in the choice of Etude variable returned as the result of the entire operation. Like “"e CompoundSum integerConstantOne"\!” and “"e CompoundDifference integerConstantOne"\!”, “"PostIncrement (e) {-++-}"” and “"PostDecrement (e) {----}"” accept arbitrary modifiable l-value operands of a bit-field, arithmetic or pointer type, so that their denotations must be translated into Etude as follows:

\goodbreak

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (PostIncrement (e) {-++-}))))))))))))))))

	=	do	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) --
  				<- meaningOfModifiableLValue{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 e)))))))))))))))
			require $ (isBitFieldType{--}(t_e))
			return $	(_S_e, _T_e, _D_e, _V_e, t_e,
					{-quad-}{-sem-}( ---
						LET [{-raw-}[TVar 1] 	=: tau_e] $
					 	LET [{-raw-}[TVar 2] 	=: GET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t_e))) (bitFieldFormat)] $
					 	LET [{-raw-}[TVar 3] 	=: RET  [{-sem-}(unpackBitField{--}(t_e {->-}, VAR (TVar 2)))]] $
					 	LET [{-hv-}[] 		=: SET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t_e))) (bitFieldFormat) --
					 						{-TO-} ({-sem-}(packBitField{--}(t_e {->-}, VAR (TVar 2), --
					 								{-sem-}(OP2 (VAR (TVar 3)) (ADD) ({-sem-}(formatOfType{--}(t_e))) --
					 									(IMM (1) ({-sem-}(formatOfType{--}(t_e))))))))] $
					 	RET [VAR (TVar 3)]))

	<|>	do	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) --
  				<- meaningOfModifiableLValue{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 e)))))))))))))))
			require $ (isArithmeticType{--}(t_e))
			return $	(_S_e, _T_e, _D_e, _V_e, t_e,
					{-quad-}{-sem-}( ---
						LET [{-raw-}[TVar 1] 	=: tau_e] $
					 	LET [{-raw-}[TVar 2] 	=: GET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t_e))) ({-sem-}(formatOfType{--}(t_e)))] $
					 	LET [{-hv-}[] 		=: SET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t_e))) ({-sem-}(formatOfType{--}(t_e))) --
				 							{-TO-} ({-hv-}(OP2 (VAR (TVar 2)) (ADD) ({-sem-}(formatOfType{--}(t_e))) --
					 									(IMM (1) ({-sem-}(formatOfType{--}(t_e))))))] $
					 	RET [VAR (TVar 2)]))

	<|>	do	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) --
  				<- meaningOfModifiableLValue{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 e)))))))))))))))
			require $ (isPointerType{--}(t_e) && isObjectType{--}(baseOfType{--}(t_e)))
			return $	(_S_e, _T_e, _D_e, _V_e, t_e,
					{-quad-}{-sem-}( ---
						LET [{-raw-}[TVar 1] 	=: tau_e] $
					 	LET [{-raw-}[TVar 2] 	=: GET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t_e))) ({-sem-}(formatOfType{--}(t_e)))] $
					 	LET [{-hv-}[] 		=: SET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t_e))) ({-sem-}(formatOfType{--}(t_e))) --
					 						{-TO-} ({-hv-}(OP2 (VAR (TVar 2)) (ADD) ({-sem-}(formatOfType{--}(t_e))) --
					 									(IMM (toRational {-sem-}(sizeOfType{--}(baseOfType{--}(t_e)))) --
					 	 										(ZG`FMT`standardEncoding))))] $
				 	 	RET [VAR (TVar 2)]))

meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (PostDecrement (e) {----}))))))))))))))))

	=	do	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) --
  				<- meaningOfModifiableLValue{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 e)))))))))))))))
			require $ (isBitFieldType{--}(t_e))
			return $	(_S_e, _T_e, _D_e, _V_e, t_e,
					{-quad-}{-sem-}( ---
						LET [{-raw-}[TVar 1] 	=: tau_e] $
					 	LET [{-raw-}[TVar 2] 	=: GET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t_e))) (bitFieldFormat)] $
					 	LET [{-raw-}[TVar 3] 	=: RET  [{-sem-}(unpackBitField{--}(t_e {->-}, VAR (TVar 2)))]] $
					 	LET [{-hv-}[] 		=: SET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t_e))) (bitFieldFormat) --
					 						{-TO-} ({-sem-}(packBitField{--}(t_e {->-}, VAR (TVar 2), --
					 								{-sem-}(OP2 (VAR (TVar 3)) (SUB) ({-sem-}(formatOfType{--}(t_e))) --
					 									(IMM (1) ({-sem-}(formatOfType{--}(t_e))))))))] $
					 	RET [VAR (TVar 3)]))

	<|>	do	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) --
  				<- meaningOfModifiableLValue{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 e)))))))))))))))
			require $ (isArithmeticType{--}(t_e))
			return $	(_S_e, _T_e, _D_e, _V_e, t_e,
					{-quad-}{-sem-}( ---
						LET [{-raw-}[TVar 1] 	=: tau_e] $
					 	LET [{-raw-}[TVar 2] 	=: GET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t_e))) ({-sem-}(formatOfType{--}(t_e)))] $
					 	LET [{-hv-}[] 		=: SET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t_e))) ({-sem-}(formatOfType{--}(t_e))) --
					 						{-TO-} ({-hv-}(OP2 (VAR (TVar 2)) (SUB) ({-sem-}(formatOfType{--}(t_e))) --
					 									(IMM (1) ({-sem-}(formatOfType{--}(t_e))))))] $
					 	RET [VAR (TVar 2)]))

	<|>	do	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) --
  				<- meaningOfModifiableLValue{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 e)))))))))))))))
			require $ (isPointerType{--}(t_e) && isObjectType{--}(baseOfType{--}(t_e)))
			return $	(_S_e, _T_e, _D_e, _V_e, t_e,
					{-quad-}{-sem-}( ---
						LET [{-raw-}[TVar 1] 	=: tau_e] $
					 	LET [{-raw-}[TVar 2] 	=: GET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t_e))) ({-sem-}(formatOfType{--}(t_e)))] $
					 	LET [{-hv-}[] 		=: SET  (VAR (TVar 1) {-hv-}, {-sem-}(attrOfType{--}(t_e))) ({-sem-}(formatOfType{--}(t_e))) --
					 						{-TO-} ({-hv-}(OP2 (VAR (TVar 2)) (ADD) ({-sem-}(formatOfType{--}(t_e))) --
					 									(IMM (toRational --
					 										  {-sem-}(negate (sizeOfType{--}(baseOfType{--}(t_e))))) --
				 	 										(ZG`FMT`standardEncoding))))] $
				 	 	RET [VAR (TVar 2)]))
\end{code}

\goodbreak

Last but not least, a value constructed from a function designator denotes the same term as that designator itself, while denotations of values obtained from an implicit conversion of an l-value with an object or an array type represent the actual content of the corresponding C object, as obtained by an application of the "contentOfLValue" combinator to the l-value's ordinary meaning. In all cases, the result has the pointer-promoted type of the original expression:

\begin{code}
meaningOfValueExpression {-sem-}(_S, _T, _D, _I {->-}, e) = do
	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfFunctionDesignator{--}(_S, _T, _D, _I {->-}, e) <|> meaningOfLValue{--}(_S, _T, _D, _I {->-}, e)
	require $ (isFunctionType{--}(t_e) || isObjectType{--}(t_e) || isArrayType{--}(t_e))
	return $ (_S_e, _T_e, _D_e, _V_e, tqOfType{--}(t_e) `requalifiedType` pointerPromotedType{--}(t_e), contentOfLValue{--}(t_e {->-}, tau_e))
\end{code}

The retainment of type qualifiers by values obtained from such a conversion represents a small deviation from the strict letter of the C Standard, required for a proper formalisation of qualified structure and union types in the Etude framework. However, since these qualifiers are always ignored in all other semantic analysis of structure and union values, this abnormality does not impact the accuracy of our formalisation and, for all intents and purposes, may be safely ignored by more casual readers.

\end{subsubsection}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Expressions/VoidExpressions}
\set{title}		{Void Expressions}
\begin{subsubsection}

In the C programming language, the expression syntax is also used to describe a rather different kind of entities that are generally known as «void expressions». Intuitively, such constructs are only ever evaluated for their side effects, so that their type is always equal to a qualified or unqualified version of “"VoidType"\,” and their denotations represent monadic Etude terms that deliver an empty list of results “"RET []"”. Formally, the meaning of every well-formed void expression is depicted by the following Haskell derivation:

\BeginHaskellToken	{meaningOfVoidExpression}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{ve}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, V, TypeT, TermT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfVoidExpression}			{D_VE expression}

\begin{code}
meaningOfVoidExpression {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, ExpressionT) -> _M{--}(S, S, D, V, TermT VarT)
\end{code}

Most commonly, void expressions arise from calls to a C function with a “"VoidType"\,” returned type. In particular, if “"FunctionCall (e) (ael_opt)"\!” represents a well-formed function call operation, in which the function designator "e" has one such type, then the operation is performed identically to its earlier value expression variant, except that no temporary variable is ever allocated for the function's returned value and that the derived Etude term does not deliver any such values to the surrounding program. In particular, the operand "e" is first evaluated together with any operands found in the argument expression list "ael_opt", with values of all such arguments placed in appropriate temporary objects, whose complete list is then applied to the targeted Etude function. In Haskell, this translation can be expressed concisely as follows:

\begin{code}
meaningOfVoidExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (FunctionCall (e) (ael_opt))))))))))))))))) = do
	(_S_e,   _T_e,   _D_e,  _V_e,  t_e, tau_e) ---
  		<- meaningOfValueExpression 		{--}(_S,     _T,     _D,     _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 --
  														(E10 (E11 (E12 (E13 (E14 (E15 e)))))))))))))))
	require $ (isPointerType{--}(t_e) && isFunctionType{--}(baseOfType{--}(t_e)) && isVoidType{--}(baseOfType{--}(baseOfType{--}(t_e))))
	(_S_a,   _T_a,   _D_a,  _V_a,  _W_a, beta_set) ---
 		<-  meaningOfArgumentExpressionListOpt	{--}(_S_e,   _T_e,   _D_e,   _I, prototypeOfType{--}(baseOfType{--}(t_e)) {->-}, ael_opt)
	return $ (_S_a, _T_a, _D_a, _V_e `mapUnion` _V_a `mapUnion` _W_a, --
  		  {-sem-}(LET ({-sem-}([{-sem-}({-raw-}[TVar 1] =: tau_e)] ++ beta_set)) $ APP (VAR (TVar 1)) (coerceList (VAR) (setToList ({-sem-}(dom{--}(_W_a)))))))
\end{code}

Further, an arbitrary C expression can be converted into the “"VoidType"\,” type using an explicit cast operation, in which case the operand's meaning is always constructed by a separate function "meaningOfExpressionOpt" that is described later in the current section:

\begin{code}
meaningOfVoidExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (Cast (type_name) {-!-} e {-,-})))))))))))))) = do
	(_S_t,   _T_t,   _D_t,  t_t) ---
		<- meaningOfTypeName			{--}(_S,     _T,     _D,     _I {->-}, type_name)
	(_S_e,   _T_e,   _D_e,  _V_e, tau_e) ---
  		<- meaningOfExpressionOpt		{--}(_S_t,   _T_t,   _D_t,   _I {->-}, Just (E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e))))))))))))))
  	require $ (isVoidType{--}(t_t))
  	return $ (_S_e, _T_e, _D_e, _V_e, tau_e)
\end{code}

Void expressions may also assume the conditional or sequence syntax “"Conditional (e1) {-?-} (e2) {-:-} (e3)"\!” and “"e1 {-!-} `Comma` {-!-} e2"\!”, provided that their "e2" and "e3" operands represent void expressions, as stipulated by the following pair of Haskell definitions:

\begin{code}
meaningOfVoidExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (Conditional (e1) {-?-} (e2) {-:-} (e3)))) = do
	(_S1,    _T1,    _D1,   _V1, t1, tau1) ---
		<- meaningOfValueExpression		{--}(_S,     _T,     _D,     _I {->-}, E1 (E2 (E3 e1)))
	(_S2,    _T2,    _D2,   _V2, tau2) ---
		<- meaningOfVoidExpression		{--}(_S1,    _T1,    _D1,    _I {->-}, e2)
	(_S3,    _T3,    _D3,   _V3, tau3) ---
  		<- meaningOfVoidExpression		{--}(_S2,    _T2,    _D2,    _I {->-}, E1 (E2 e3))
	require $ (isScalarType{--}(t1))
	return $	(_S3, _T3, _D3, _V1 `mapUnion` _V2 `mapUnion` _V3, --
			{-sem-}(LET [{-raw-}[TVar 1] =: tau1] $ --
	  				IFF (OP2 (VAR (TVar 1)) (CNE) ({-sem-}(formatOfType{--}(t1))) (IMM (0) ({-sem-}(formatOfType{--}(t1))))) {-THEN-} --
						(tau2) --
					{-ELSE-} --
						(tau3))) {-!-} {-!-} {-!-} {-!-} {-!-} {-!-} {-!-} {-!-} {-!-} {-!-} {-!-} {-!-}

meaningOfVoidExpression {-sem-}(_S, _T, _D, _I {->-}, e1 {-!-} `Comma` {-!-} e2) = do
	(_S1,    _T1,    _D1,   _V1, tau1) ---
  		<- meaningOfExpressionOpt		{--}(_S,     _T,     _D,     _I {->-}, Just e1)
	(_S2,    _T2,    _D2,   _V2, tau2) ---
  		<- meaningOfVoidExpression		{--}(_S1,    _T1,    _D1,    _I {->-}, E1 e2)
	return $ (_S2, _T2, _D2, _V1 `mapUnion` _V2, {-sem-}(LET [{-hv-}[] =: tau1] $ tau2))
\end{code}

Last but not least, every parenthesised version of a void expression has the same denotation as its constituent, with no other semantic form ever permitted in such constructs:

\begin{code}
meaningOfVoidExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (NestedExpression (e)))))))))))))))))) ---
  = meaningOfVoidExpression{--}(_S, _T, _D, _I {->-}, e)
meaningOfVoidExpression {-sem-}(_S, _T, _D, _I {->-}, other) ---
  = fail ("Semantic error")
\end{code}

In many contexts, a void expression can be obtained by an implicit conversion of an arbitrary well-formed function designator, l-value or value expression, in which case the expression's result is assigned to a temporary variable, whose value is otherwise ignored by the translated program. In this work, denotations of such implicitly-void expressions are formalised by the above-mentioned Haskell \hbox{function "meaningOfExpressionOpt":}

\BeginHaskellToken	{meaningOfExpressionOpt}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{e}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, Maybe ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, V, TermT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfExpressionOpt}			{D_E expression_opt}

\begin{code}
meaningOfExpressionOpt {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, Maybe ExpressionT) -> _M{--}(S, S, D, V, TermT VarT)

meaningOfExpressionOpt {-sem-}(_S, _T, _D, _I {->-}, e_opt)

	=	do	require $ (e_opt == Nothing)
  			return $ (_S, _T, _D, emptyMap, {-sem-}(RET []))

	<|>	do	(_S_e, _T_e, _D_e, _V_e, tau_e) <- meaningOfVoidExpression{--}(_S, _T, _D, _I {->-}, unJust e_opt)
  			return $ (_S_e, _T_e, _D_e, _V_e, tau_e)

	<|>	do	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e)
			{-quad-}<- meaningOfFunctionDesignator{--}(_S, _T, _D, _I {->-}, unJust e_opt) --
						<|> meaningOfLValue{--}(_S, _T, _D, _I {->-}, unJust e_opt) --
						<|> meaningOfValueExpression {--}(_S, _T, _D, _I {->-}, unJust e_opt)
	  		return $ (_S_e, _T_e, _D_e, _V_e, {-sem-}(LET [{-raw-}[TVar 1] =: tau_e] $ RET []))
\end{code}

This construction is particularly useful for a concise formalisation of the “"`Comma`"\!” and void cast operations described earlier in this section, as well as «expression statements» discussed in Section \ref{C/Statements}. In the later context, the void expression operand may be omitted entirely, resulting in the «null statement» construct “"ExpressionStatement () {-;-}"\!” which, by the above translation, is taken to denote the trivial monadic term “"RET []"”. 

\end{subsubsection}
\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Expressions/FunctionArguments}
\set{title}		{Function Arguments}
\begin{subsection}

In every function call operation of the form “"FunctionCall (e) (ael_opt)"\!”, the optional operand "ael" represents a comma-separated list of assignment expressions which depict the individual argument values supplied by the program to the function designated by the expression operand "e". The concrete syntax of these lists is depicted in Haskell as follows:

\IndexType		{ArgumentExpressionListT}			{argument_expression_list (\tstar)}
\IndexCons		{E0}						{ assignment_expression (argument_expression_list)}
\IndexCons		{En}						{, argument_expression_list assignment_expression (argument_expression_list)}

\begin{syntax}
\InsertHaskell		{ArgumentExpressionListT}
\end{syntax}

It is convenient to view such constructs simply as sequences of zero or more expression entities, which can be easily derived from their concrete syntax by the following recursive process:

\BeginHaskellToken	{listOfArgumentExpressions}		{Open}	{list}
  \HaskellArgument							{Maybe ArgumentExpressionListT}
  \HaskellResult							{[ExpressionT]}
\EndHaskellToken

\IndexTerm		{listOfArgumentExpressions}			{list argument_expression_list_opt (expressions)}

\begin{code}
listOfArgumentExpressions {-[*]-} :: Maybe ArgumentExpressionListT -> [ExpressionT]

listOfArgumentExpressions {-sem-}(Just (ael `En` e))	= listOfArgumentExpressions{--}(Just ael) ++ [E1 e]
listOfArgumentExpressions {-sem-}(Just (E0 e))		= [E1 e]
listOfArgumentExpressions {-sem-}({-hide-}Nothing)	= {-empty-}[]
\end{code}

At the beginning of every function call operation, the value of each argument is converted into the pointer-promoted version of the corresponding prototype entry, or else into the argument-promoted version of the expression's own type if no explicit type information is provided for the argument by the underlying function designator. In both cases, this converted value is stored into a temporary memory-resident object of an appropriate C type. Formally, the denotation of every such argument expression "e" is represented by the construct “"meaningOfArgumentExpression{--}(_S, _T, _D, _I, t_opt {->-}, e)"”, in which "t_opt" represents either the pointer-promoted version of the corresponding prototype entry, or the omitted type “"Nothing"” if no such entry exists in the prototype:

\BeginHaskellToken	{meaningOfArgumentExpression}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{ae}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, Maybe TypeT {->-}, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, V, V, TermT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfArgumentExpression}			{D_AE expression}

\begin{code}
meaningOfArgumentExpression {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I, Maybe TypeT {->-}, ExpressionT) -> _M{--}(S, S, D, V, V, TermT VarT)
\end{code}

observing that all such denotations include, besides the usual "_S", "_T", "_D", "_V" and "tau" components common to ordinary C expressions, an additional set of variable bindings "_W", which, intuitively, specifies the actual set of temporary Etude objects allocated for the function's argument values.

The constraints on such argument expressions are identical to those described earlier for simple assignment operations, except that no particular restrictions are imposed on the l-value's type qualification, so that the actual modification of the temporary argument object must be performed using Etude's initialisation operator “"INI"” instead of the earlier “"SET"” term form applied in Section \ref{C/Expressions/Values}. For the same reason, values of structure and union arguments are initialised using a separate implementation-defined construct “"iniObject ((t1 {->-}, alpha1)) {-to-} ((t2 {->-}, alpha2))"”, which is essentially equivalent to the earlier “"setObject ((t1 {->-}, alpha1)) {-to-} ((t2 {->-}, alpha2))"” combinator, except that the “"iniObject"\!” form is also guaranteed to be defined for constant-qualified types "t", provided that the corresponding memory region also appears in the environment's initialiser envelope "initOfOEnv{--}(_Delta)". Formally, the type signature of “"iniObject"\!” is introduced as follows:

\ReindexTerm		{iniObject}

\begin{haskell}
iniObject {-[*]-} {-to-} {-[*]-} :: (TypeT {->-}, AtomT VarT) -> (TypeT {->-}, AtomT VarT) -> TermT VarT
\end{haskell}

and its minimal semantics are described by the following theorem:

\NewHaskellToken	{iniObjectJ}				{Ord}	{\TKeyword{set\sub{i}}}
\NewHaskellDescription	{iniObjectJ}					{Properties of the “\haskell{iniObject (..) {-to-} (..)}” operation}
\IndexTheorem		{iniObjectJ}					{SETI Properties of the SETI_TO operation}

\begin{haskell}
iniObjectJ :: forall _Lambda, _Delta, t1, sigma1, t2, sigma2 =>
	WfJ{--}(_Lambda) -> --
	WfJ{-sem-}(_Delta `copyOEnv` (sigma1, envelopeOfType{--}(t1), sigma2, envelopeOfType{--}(t2))) -> --
	WfJ{-sem-}(IMM (sigma1) (objectFormat{--}(formatOfType{--}(t1)))) -> --
	WfJ{-sem-}(IMM (sigma2) (objectFormat{--}(formatOfType{--}(t2)))) ->
	{-sem-}(sizeOfType{--}(t1) == sizeOfType{--}(t2) && (sigma1 + sizeOfType{--}(t1) <= sigma2 || sigma1 >= sigma2 + sizeOfType{--}(t2) || sigma1 == sigma2)) ->
	{-sem-}(recTqOfType{--}(t1) `subseteq` listToSet [CM] && recTqOfType{--}(t2) `subseteq` listToSet [CM] && envelopeOfType{--}(t1) `subseteq` initOfOEnv{--}(_Delta)) ->
	{-sem-}(_Lambda, _Delta {->-}, iniObject ({-sem-}(t1 {->-}, IMM (sigma1) (objectFormat{--}(formatOfType{--}(t1))))) --
							{-to-} ({-sem-}(t2 {->-}, IMM (sigma2) (objectFormat{--}(formatOfType{--}(t2))))))
	{-quad-}`EqvJ` {-sem-}(_Lambda, _Delta `copyOEnv` (sigma1, envelopeOfType{--}(t1), sigma2, envelopeOfType{--}(t2)) {->-}, RET [])
\end{haskell}

Using this definition, the meaning of a single argument expression with an explicitly-specified C type can be formalised relatively easily as follows:

\begin{code}
meaningOfArgumentExpression {-sem-}(_S, _T, _D, _I, Just t {->-}, e)

	=	do	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, e)
			require $	(isArithmeticType{--}(t) && isArithmeticType{--}(t_e)) ||
					(isPointerType{--}(t) && isPointerType{--}(t_e) && ---
						     (	(unqualifiedType{--}(baseOfType{--}(t)) `areCompatibleTypes` unqualifiedType{--}(baseOfType{--}(t_e))) ||
							((isObjectType{--}(baseOfType{--}(t)) || isIncompleteType{--}(baseOfType{--}(t))) && isVoidType{--}(baseOfType{--}(t_e))) ||
							((isObjectType{--}(baseOfType{--}(t_e)) || isIncompleteType{--}(baseOfType{--}(t_e))) && isVoidType{--}(baseOfType{--}(t))))) ||
					(isPointerType{--}(t) && isNullPointerConstant{--}(_S, _T, _D, _I {->-}, e))
  			return $  (_S_e, _T_e, --
  					 _D_e `mapUnion` listToMap [ {-raw-}(uvar{--}(_D_e) {-:-}, Nothing) ], --
  					 _V_e, --
  					 listToMap [ {-raw-}(uvar{--}(_D_e) {-:-}, envelopeOfType{--}(t)) ],
	  			  {-quad-}{-sem-}( ---
  						LET [{-raw-}[TVar 1] =: tau_e] $ --
  					 	INI (VAR {-sem-}(uvar{--}(_D_e)) {-hv-}, emptySet) ({-sem-}(formatOfType{--}(t))) --
  					 		{-TO-} ({-hv-}(OP1 (CNV {-sem-}(formatOfType{--}(t))) ({-sem-}(formatOfType{--}(t_e))) {--} {-hv-}(VAR (TVar 1))))))

	<|>	do	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, e)
			require $	(isCompoundType{--}(t) && isObjectType{--}(t) && unqualifiedType{--}(t) `areCompatibleTypes` unqualifiedType{--}(t_e))
  			return $  (_S_e, _T_e, --
  					 _D_e `mapUnion` listToMap [ {-raw-}(uvar{--}(_D_e) {-:-}, Nothing) ], --
  					 _V_e, --
  					 listToMap [ {-raw-}(uvar{--}(_D_e) {-:-}, envelopeOfType{--}(t)) ],
  				  {-quad-}{-sem-}( ---
  					 	LET [{-raw-}[TVar 1] =: tau_e] $ --
  					 	{-sem-}(iniObject ((t {->-}, VAR (uvar{--}(_D_e)))) {-to-} ({-sem-}(t_e {->-}, VAR (TVar 1))))))
\end{code}

Otherwise, if the function prototype has provided us with no explicit type information about a given argument, then the resulting temporary l-value has the argument-promoted version of the expression's own type and, in all other respects, its denotation is derived identically to the meaning of a similar explicitly-typed argument assignment:

\begin{code}
meaningOfArgumentExpression {-sem-}(_S, _T, _D, _I, Nothing {->-}, e)

	=	do	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, e)
			require $ (isScalarType{--}(argumentPromotedType{--}(t_e)))
  			return $	(_S_e, _T_e, --
  					 _D_e `mapUnion` listToMap [ {-raw-}(uvar{--}(_D_e) {-:-}, Nothing) ], --
  					 _V_e, --
  					 listToMap [ {-raw-}(uvar{--}(_D_e) {-:-}, envelopeOfType{--}(argumentPromotedType{--}(t_e))) ],
  					{-quad-}{-sem-}( ---
  					 	LET [{-raw-}[TVar 1] =: tau_e] $ --
  					 	INI (VAR {-sem-}(uvar{--}(_D_e)) {-hv-}, emptySet) ({-sem-}(formatOfType{--}(argumentPromotedType{--}(t_e)))) --
  					 		{-TO-} ({-hv-}(OP1 (CNV {-sem-}(formatOfType{--}(argumentPromotedType{--}(t_e)))) ({-sem-}(formatOfType{--}(t_e))) {--} --
  					 			{-hv-}(VAR (TVar 1))))))

	<|>	do	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, e)
			require $ (isCompoundType{--}(argumentPromotedType{--}(t_e)) && isObjectType{--}(argumentPromotedType{--}(t_e)))
  			return $	(_S_e, _T_e, --
  					 _D_e `mapUnion` listToMap [ {-raw-}(uvar{--}(_D_e) {-:-}, Nothing) ], --
  					 _V_e, --
  					 listToMap [ {-raw-}(uvar{--}(_D_e) {-:-}, envelopeOfType{--}(argumentPromotedType{--}(t_e))) ],
  					{-quad-}{-sem-}( ---
  						LET [{-raw-}[TVar 1] =: tau_e] $ --
  					 	{-sem-}(iniObject ((argumentPromotedType{--}(t_e) {->-}, VAR (uvar{--}(_D_e)))) {-to-} ({-sem-}(t_e {->-}, VAR (TVar 1))))))
\end{code}

The above derivation of individual argument assignments may be extended trivially to an entire list of argument expressions, whereby the meaning "tau_e" of every individual argument is converted into an Etude binding of the form “"{-hv-}[] =: tau_e"\!” as follows:

\BeginHaskellToken	{meaningOfArgumentExpressions}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{aes}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, Maybe TypesT {->-}, [ExpressionT])}
  \HaskellResult							{_M{--}(S, S, D, V, V, BindingsT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfArgumentExpressions}			{D_AES expressions}

\begin{code}
meaningOfArgumentExpressions {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, I, Maybe TypesT {->-}, [ExpressionT]) -> _M{--}(S, S, D, V, V, BindingsT VarT)

meaningOfArgumentExpressions {-sem-}(_S, _T, _D, _I, Nothing		{->-}, e_set {-,-})

	=	 do	require $ (lengthOfList{--}(e_set {-,-}) == 0)
  			return $ (_S, _T, _D, emptyMap, emptyMap, {-empty-}[])

	<|>	 do	(_S_e,   _T_e,   _D_e,  _V_e,  _W_e,  tau_e	  ) <- meaningOfArgumentExpression   {--}(_S,    _T,    _D,    _I, Nothing {->-}, head{--}(e_set {-,-}))
  			(_S_a,   _T_a,   _D_a,  _V_a,  _W_a,  beta_set_a  ) <- meaningOfArgumentExpressions  {--}(_S_e,  _T_e,  _D_e,  _I, Nothing {->-}, tail{--}(e_set {-,-}))
  			return $ (_S_a, _T_a, _D_a, _V_e `mapUnion` _V_a, _W_e `mapUnion` _W_a, {-sem-}[{-hv-}[] =: tau_e] ++ beta_set_a)

meaningOfArgumentExpressions {-sem-}(_S, _T, _D, _I, Just t_set	{->-}, e_set {-,-})

	=	 do	require $ (inRange (lengthOfList{--}(t_set {-,-})) (==) (lengthOfList{--}(e_set {-,-})) (==) (0))
  			return $ (_S, _T, _D, emptyMap, emptyMap, {-empty-}[])

	<|>	 do	(_S_e,   _T_e,   _D_e,  _V_e,  _W_e,  tau_e	  ) <- meaningOfArgumentExpression   {--}(_S,    _T,    _D,    _I, --
  											Just (pointerPromotedType{--}(head{--}(t_set {-,-}))) {->-}, head {--}(e_set {-,-}))
  			(_S_a,   _T_a,   _D_a,  _V_a,  _W_a,  beta_set_a  ) <- meaningOfArgumentExpressions  {--}(_S_e,  _T_e,  _D_e,  _I, --
  											Just (tail{--}(t_set {-,-})) {->-}, tail {--}(e_set {-,-}))
  			return $ (_S_a, _T_a, _D_a, _V_e `mapUnion` _V_a, _W_e `mapUnion` _W_a, {-sem-}[{-hv-}[] =: tau_e] ++ beta_set_a)
\end{code}

In the actual function call operation, the above definition is applied directly only if the targeted function either includes a prototype without “\!"{-...-}{--}"\!” suffix, or else if it omits the prototype altogether from its C type. As a special case, if the prototype consists entirely of a single “"VoidType"\,” type entry, then the corresponding argument expression list must be empty and its denotation is formed from an empty argument variable set "_W" and a null binding list "beta_set" as follows:

\BeginHaskellToken	{meaningOfArgumentExpressionListOpt}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{ael}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, Maybe PrototypeT {->-}, Maybe ArgumentExpressionListT)}
  \HaskellResult							{_M{--}(S, S, D, V, V, BindingsT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfArgumentExpressionListOpt}		{D_AEL argument_expression_list_opt}

\begin{code}
meaningOfArgumentExpressionListOpt {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, I, Maybe PrototypeT {->-}, Maybe ArgumentExpressionListT) ->
	_M{--}(S, S, D, V, V, BindingsT VarT)

meaningOfArgumentExpressionListOpt {-sem-}(_S, _T, _D, _I, Just (Types (t_set))		{->-}, ael_opt) ---
  =	 do	require $ (lengthOfList{--}(t_set {-,-}) == 1 && isVoidType{--}(head{--}(t_set {-,-})) && ael_opt == Nothing)
  		return $ (_S, _T, _D, emptyMap, emptyMap, {-empty-}[])
  <|>	 do	meaningOfArgumentExpressions{--}(_S, _T, _D, _I, Just t_set	 {->-}, listOfArgumentExpressions{--}(ael_opt))
meaningOfArgumentExpressionListOpt {-sem-}(_S, _T, _D, _I, Nothing 			{->-}, ael_opt) ---
  =	 do	meaningOfArgumentExpressions{--}(_S, _T, _D, _I, Nothing 	 {->-}, listOfArgumentExpressions{--}(ael_opt))
\end{code}

In presence of the prototype suffix “\!"{-...-}{--}"\!”, the construction is rather more interesting. If "ell" represents the number of C types explicitly included in such a prototype, then the first "ell" arguments of the function are said to be «fixed», while any remaining entries in the list are said to represent «variable arguments» of the function call operation, whose precise number may fluctuate between individual invocations of that C function. In lambda calculus, such constructs are inherently inexpressible, since every lambda abstraction always specifies a predetermined set of variable bindings utilised within its body. Accordingly, this aspect of the C language is most naturally modelled in Etude by combining all variable arguments into a single structure-like object of an unspecified layout, which is always appended to the ordinary portion of the argument list whenever the underlying function prototype ends in the “\!"{-...-}{--}"\!” suffix. Conveniently, this approach also enables a very natural implementation of the “va\_list” type in the Standard C Library that, ultimately, must accompany our completed C compiler, together with the associated macros “va\_start”, “va\_arg” and “va\_end” required by the C Standard.

In particular, any fixed arguments corresponding to the explicitly-typed prototype entries represent ordinary argument assignments to temporary l-values of these types, denoting a predetermined set of argument objects "_W1" and bindings "beta_set1", except that the special “"VoidType"\,” case is not applicable in such constructs. Further, a parameter-less denotation of any remaining variable arguments is obtained in an absence of explicit typing, producing a second set of temporary l-values "_W2" and corresponding assignment bindings "beta_set2". In the entire operation, these variable arguments are then combined into a single Etude object "nu_v", whose structure is obtained from the union of their individual envelopes. Each variable argument is placed in "nu_v" at a predetermined but unspecified byte offset, as computed by the unspecified function "layoutOfVaList" of the following Haskell type:

\ReindexTerm		{layoutOfVaList}

\begin{haskell}
layoutOfVaList {-[*]-} :: V -> (MapT VarT Integer)
\end{haskell}

Intuitively, given a set of individual argument objects "_W2" derived from the function call's variable argument list, "layoutOfVaList{--}(_W2)" maps every variable "nu_k @@ dom{--}(_W2)" to an offset "n_k" into the combined variable argument object "nu_v", so that, in the denotation of the entire operation, all of these temporary variables are replaced with a single variable "nu_v", whose envelope represents the union of the individual envelopes "_W2#(nu_k) `shiftEnvelope` n_k". Further, in the set of assignment bindings "beta_set2", every occurrence of such a variable "nu_k" is replaced by an atom of the form “"OP2 (nu_v) (ADD) (OG`FMT`standardEncoding) (IMM (n_k) (ZG`FMT`standardEncoding))"\!”, so that all of these variables are effectively eliminated from the program and, accordingly removed from the resulting item definition set "_D". Formally, this rather complicated derivation is implemented in Haskell as follows:

\begin{code}
meaningOfArgumentExpressionListOpt {-sem-}(_S, _T, _D, _I, Just (Ellipsis (t_set) {-...-}) {->-}, ael_opt) = do
	(_S1, _T1, _D1, _V1, _W1, beta_set1) ---
		<- meaningOfArgumentExpressions{--}(_S,   _T,   _D,   _I, Just t_set	{->-}, --
  					takeFromList  {--}(lengthOfList{--}(t_set {-,-}), listOfArgumentExpressions{--}(ael_opt)))
	(_S2, _T2, _D2, _V2, _W2, beta_set2) ---
		<- meaningOfArgumentExpressions{--}(_S1,  _T1,  _D1,  _I, Nothing	{->-}, --
  					dropFromList  {--}(lengthOfList{--}(t_set {-,-}), listOfArgumentExpressions{--}(ael_opt)))
	(nu_v) <- {-hide-}return $ uvar{--}(_D2 `mapSetDifference` dom{--}(_W2))
	return $	(_S2, _T2, _D2 `mapSetDifference` dom{--}(_W2) `mapUnion` listToMap [{-raw-}(nu_v {-:-}, Nothing)], _V1 `mapUnion` _V2,
			{-quad-}_W1 `mapUnion` listToMap [ {-raw-}(nu_v {-:-}, unions [ _W2#(nu_k) `shiftEnvelope` n_k --
								| {-raw-}(nu_k {-:-}, n_k) <- mapToList (layoutOfVaList{--}(_W2)) ]) ],
			{-quad-}beta_set1 ++ (beta_set2 `substBindings` --
				listToMap [ {-raw-}(nu_k {-:-}, {-sem-}(OP2 (VAR nu_v) (ADD) (OG`FMT`standardEncoding) (IMM (toRational n_k) (ZG`FMT`standardEncoding)))) --
				 				| {-raw-}(nu_k {-:-}, n_k) <- mapToList (layoutOfVaList{--}(_W2)) ]))
\end{code}

Although it may be argued that the above formalisation of the C function call semantics represents an over-specification of the language, extensive experience with implementation of C compilers has shown the above approach (or minor variations thereof) to be the only effective means of implementing variable argument lists on modern instruction set architectures. This empirical evidence strongly suggests that the semantic of variable argument lists are fundamentally connected to the so-called «C calling convention» modelled above.

Nevertheless, the reader should keep in mind that, since the program equivalence relation defined by the semantics of Etude in Chapter \ref{Lambda} is based only the sequence of the system calls generated by programs, the above formalisation of function call operations could, in principle, be proven equivalent to any other sensible definition, permitting optimising C compilers to tailor the C calling convention to the specific needs of a particular program's structure.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Expressions/Constants}
\set{title}		{Constant Expressions}
\begin{subsection}

A conditional expression whose result is always known at the time of translation is known as a «constant expression». In the C standard, this additional use of the expression syntax is captured by the following trivial Haskell type definition:

\IndexType		{ConstantExpressionT}				{constant_expression (\tstar)}
\IndexCons		{ConstantExpressionT'}				{ conditional_expression (constant_expression)}

\begin{syntax}
\InsertHaskell		{ConstantExpressionT}
\end{syntax}

Such expressions have a number of important applications within the C language. Their precise meanings, as well as any constraints on the syntax of such constructs, depends on the context in which a constant expression appears within the program. The following sections contain a detailed discussion of five different classes of constant expressions, known as «integral constant expressions», «null pointer constants», «static initialiser expressions», «arithmetic constant expressions» and «address constants». Each of these syntactic entities is described individually in Sections \ref{C/Expressions/Constants/Integral}, \ref{C/Expressions/Constants/NullPointers}, \ref{C/Expressions/Constants/Static} and \ref{C/Expressions/Constants/Arithmetic}, respectively.

Regardless of the precise nature of a particular constant expression "e", every such construct denotes an atomic value rather than a monadic Etude term. In certain contexts, this atomic value must be reduced further into a simple numeric quantity, as depicted by the following Haskell combinator:

\ReindexTerm		{valueOfAtom}

\begin{haskell}
valueOfAtom {-[*]-} {-[*]-} :: (MonadFix _M, Monad _M) => FormatT -> AtomT nu -> _M{--}(Rational)
\end{haskell}

For every well-formed arithmetic constant expression "e" of an integral C type "t", the atomic denotation "alpha" of the constant "e" is always equivalent to the constant atomic form \hbox{“"IMM ({-sem-}(valueOfAtom{--}(alpha))) ({-sem-}(formatOfType{--}(t)))"\!”.} Formally:

\NewHaskellToken	{valueOfIntegralAtomJ}			{Ord}	{\TKeyword{int\sub{\alpha}}}
\NewHaskellDescription	{valueOfIntegralAtomJ}				{Properties of integral constant expressions}
\IndexTheorem		{valueOfIntegralAtomJ}				{INT_\alpha Properties of integral constant expressions}

\begin{haskell}
valueOfIntegralAtomJ :: forall n :: Integer, _S, _S', _T, _T', _D, _D', I, e, t, alpha =>
	{-sem-}(isIntegralType{--}(t) && glbOfType{--}(t) <= n <= lubOfType{--}(t)) ->
	{-sem-}(meaningOfArithmeticConstantExpression{--}(_S, _T, _D, I {->-}, e)) = {-sem-}(Just (_S', _T', _D', t, alpha)) ->
	{-sem-}(alpha) `EqvJ` {-sem-}(IMM (n) ({-sem-}(formatOfType{--}(t)))) ->
	{-sem-}(valueOfAtom(formatOfType{--}(t)){--}(alpha) == n)
\end{haskell}

Further, if "t" represents a floating type and "alpha" is equivalent to some rational constant atom of the form “"IMM ({-sem-}((negate 1)^s * m * radixOfFormat{--}(formatOfType{--}(t))^(e - precOfFormat{--}(formatOfType{--}(t))))) ({-sem-}(formatOfType{--}(t)))"\!”, then "valueOfAtom({-!-}{-!-}formatOfType{--}(t)){--}(alpha)" differs from "alpha"'s true arithmetic value by less than "radixOfFormat{--}(formatOfType{--}(t))^(e - precOfFormat{--}(formatOfType{--}(t)))" in magnitude. The C Standard imposes no other requirements on the reduction of arithmetic constants, so that, in principle, it may produce results more precise than those achievable during actual execution of the program on any given floating point hardware. In Haskell, these relaxed constraints on the definition of "valueOfAtom({-!-}{-!-}phi)" can be described as follows:

\NewHaskellToken	{valueOfRationalAtomJ}			{Ord}	{\TKeyword{flt\sub{\alpha}}}
\NewHaskellDescription	{valueOfRationalAtomJ}				{Properties of floating constant expressions}
\IndexTheorem		{valueOfRationalAtomJ}				{FLT_\alpha Properties of floating constant expressions}

\begin{haskell}
valueOfRationalAtomJ :: forall s, m, e :: Integer, _S, _S', _T, _T', _D, _D', I, e, t, alpha =>
	WfJ{--}(alpha) ->
	{-sem-}(isFloatingType{--}(t) && --
		0 <= s <= 1 && --
		0 <= m < radixOfFormat{--}(formatOfType{--}(t))^(precOfFormat{--}(formatOfType{--}(t))) && --
		minExpOfFormat{--}(formatOfType{--}(t)) <= e <= maxExpOfFormat{--}(formatOfType{--}(t))) ->
	{-sem-}(meaningOfArithmeticConstantExpression{--}(_S, _T, _D, I {->-}, e)) = {-sem-}(Just (_S', _T', _D', t, alpha)) ->
	{-sem-}(alpha) `EqvJ` {-sem-}(IMM ({-sem-}((negate 1)^s * m * radixOfFormat{--}(formatOfType{--}(t))^(e - precOfFormat{--}(formatOfType{--}(t))))) --
						({-sem-}(formatOfType{--}(t)))) ->
	{-sem-}(abs ((negate 1)^s * m * radixOfFormat{--}(formatOfType{--}(t))^(e - precOfFormat{--}(formatOfType{--}(t))) - valueOfAtom(formatOfType{--}(t)){--}(alpha)) --
			< radixOfFormat{--}(formatOfType{--}(t))^(e - precOfFormat{--}(formatOfType{--}(t))))
\end{haskell}

A typical implementation of this function is presented separately in Appendix \ref{C/Implementation}.

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Expressions/Constants/Integral}
\set{title}		{Integral Constant Expressions}
\begin{subsubsection}

\BeginHaskellToken	{meaningOfFloatingConstantExpression}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{fce}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, TypeT, Integer)}
\EndHaskellToken

\IndexTerm		{meaningOfFloatingConstantExpression}		{D_ICE expression}

\set{capture}		{meaningOfFloatingConstantExpression}
\begin{code}
meaningOfFloatingConstantExpression {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, ExpressionT) -> _M{--}(S, S, D, TypeT, Integer)

meaningOfFloatingConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 --
										(E15 (E16 (Constant (FloatingConstant floating_constant)))))))))))))))))) ---
  =	do	(t, x) <- meaningOfFloatingConstant{--}(floating_constant)
  		return $ (_S, _T, _D, t, truncate (x))
meaningOfFloatingConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (NestedExpression (e)))))))))))))))))) ---
  =	do	meaningOfFloatingConstantExpression{--}(_S, _T, _D, _I {->-}, e)
meaningOfFloatingConstantExpression {-sem-}(_S, _T, _D, _I {->-}, other) ---
  =	do	fail ("Semantic Error")
\end{code}

\BeginHaskellToken	{meaningOfBinaryIntegralConstantExpression}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{bice}}}
  \HaskellContext								{(MonadFix _M)}
  \HaskellArgument								{(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT)}
  \HaskellResult								{_M{--}(S, S, D, TypeT, Integer)}
\EndHaskellToken

\IndexTerm		{meaningOfBinaryIntegralConstantExpression}		{D_BICE expression expression}

\set{capture}		{meaningOfBinaryIntegralConstantExpression}
\begin{code}
meaningOfBinaryIntegralConstantExpression {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT) -> _M{--}(S, S, D, TypeT, Integer)

meaningOfBinaryIntegralConstantExpression {-sem-}(_S, _T, _D, _I, op {->-}, e1, e2) = do
	(_S1, _T1, _D1, t1, n1) <- meaningOfIntegralConstantExpression {--}(_S,   _T,   _D,   _I {->-}, e1)
	(_S2, _T2, _D2, t2, n2) <- meaningOfIntegralConstantExpression {--}(_S1,  _T1,  _D1,  _I {->-}, e2)
	(n) <- valueOfAtom (formatOfType{--}(t1 `commonArithmeticType` t2)) --
  					{-sem-}(OP2 	(OP1 (CNV {-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) ({-sem-}(formatOfType{--}(t1))) {--} --
  								{-hv-}(IMM (toRational n1) ({-sem-}(formatOfType{--}(t1))))) --
  							(op) ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) --
  							(OP1 (CNV {-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) ({-sem-}(formatOfType{--}(t2))) {--} --
  								{-hv-}(IMM (toRational n2) ({-sem-}(formatOfType{--}(t2)))))) {-!-} {-!-} {-!-} {-!-} {-!-}
	return $ (_S2, _T2, _D2, t1 `commonArithmeticType` t2, discardFraction n)
\end{code}

\BeginHaskellToken	{meaningOfRelationalIntegralConstantExpression}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{rice}}}
  \HaskellContext								{(MonadFix _M)}
  \HaskellArgument								{(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT)}
  \HaskellResult								{_M{--}(S, S, D, TypeT, Integer)}
\EndHaskellToken

\IndexTerm		{meaningOfRelationalIntegralConstantExpression}		{D_RICE expression expression}

\set{capture}		{meaningOfRelationalIntegralConstantExpression}
\begin{code}
meaningOfRelationalIntegralConstantExpression {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT) -> _M{--}(S, S, D, TypeT, Integer)

meaningOfRelationalIntegralConstantExpression {-sem-}(_S, _T, _D, _I, op {->-}, e1, e2) = do
	(_S1, _T1, _D1, t1, n1) <- meaningOfIntegralConstantExpression {--}(_S,   _T,   _D,   _I {->-}, e1)
	(_S2, _T2, _D2, t2, n2) <- meaningOfIntegralConstantExpression {--}(_S1,  _T1,  _D1,  _I {->-}, e2)
	(n) <- valueOfAtom (formatOfType{--}(PlainIntType)) --
  					{-sem-}(OP2 	(OP1 (CNV {-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) ({-sem-}(formatOfType{--}(t1))) {--} --
  								{-hv-}(IMM (toRational n1) ({-sem-}(formatOfType{--}(t1))))) --
  							(op) ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) --
  							(OP1 (CNV {-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) ({-sem-}(formatOfType{--}(t2))) {--} --
  								{-hv-}(IMM (toRational n2) ({-sem-}(formatOfType{--}(t2)))))) {-!-} {-!-} {-!-} {-!-} {-!-}
	return $ (_S2, _T2, _D2, {-sem-}(PlainIntType), discardFraction n)
\end{code}

An «integral constant expression» represents a value expression of an integral type, whose precise result is always known at the time of translation. Such constructs are used extensively in Sections \ref{C/Declarations} and \ref{C/Statements} to describe lengths of array types and “"CaseStatement"” label values. Accordingly, all such expressions always denote predetermined integer quantities, obtained from reduction of the corresponding Etude atoms by the "valueOfAtom({-!-}{-!-}{-!-}phi)" combinator described above. Formally, their denotations are constructed by the following Haskell translation:

\BeginHaskellToken	{meaningOfIntegralConstantExpression}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{ice}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, TypeT, Integer)}
\EndHaskellToken

\IndexTerm		{meaningOfIntegralConstantExpression}		{D_ICE expression}

\begin{code}
meaningOfIntegralConstantExpression {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, ExpressionT) -> _M{--}(S, S, D, TypeT, Integer)
\end{code}

Intuitively, these expressions must be always constructed entirely from integral constants, floating constants applied to a cast operation, “"SizeOf"\,” operations, casts of integral constant expressions to another integral type, or else from well-formed applications of the C operators “"Plus"”, “"Minus"”, “"Not"”, “"LogicalNot"”, “"`Product`"”, “"`Quotient`"”, “"`Remainder`"”, “"`LeftShift`"”, “"`RightShift`"”, “"`LessThan`"”, “"`GreaterThan`"”, “"`LessOrEqualTo`"”, “"`GreaterOrEqualTo`"”, “"`EqualTo`"”, “"`NotEqualTo`"”, “"`And`"”, “"`ExclusiveOr`"”, “"`InclusiveOr`"”, “"`LogicalAnd`"”, “"`LogicalOr`"” and “"Conditional () {-?-} ({-!-}) {-:-} ()"\,”, whose operands must also constitute integral constant expressions and whose true arithmetic results must fall within the range of values representable under the operation's C type.

In particular, every well-formed integer, character and enumeration constant represents an integral constant expression, whose type and numeric value is derived directly from the constant's lexical syntax by the process discussed earlier in Section \ref{C/Constants} \hbox{as follows:}

\begin{code}
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 --
										(E15 (E16 (Constant (IntegerConstant integer_constant)))))))))))))))))) = do
	(t, n) <- meaningOfIntegerConstant{--}(integer_constant)
	return $ (_S, _T, _D, t, n)

meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 --
										(E15 (E16 (Constant (CharacterConstant character_constant)))))))))))))))))) = do
	(t, n) <- meaningOfCharacterConstant{--}(character_constant)
	return $ (_S, _T, _D, t, n)

meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 --
										(E15 (E16 (Constant (EnumerationConstant enumeration_constant)))))))))))))))))) = do
	require	$ (enumeration_constant @@ dom{--}(_S) && linkageOfDesignator{--}(d) == {-sem-}(ConstantLinkage {-sem-}(integerOfLinkage{--}(linkageOfDesignator{--}(d)))))
	return $ (_S, _T, _D, typeOfDesignator{--}(d), integerOfLinkage{--}(linkageOfDesignator{--}(d)))

 where d = _S#(enumeration_constant)
\end{code}

Further, every well-formed application of the “"SizeOf"\,” operator constitutes an integral constant expression of the “"sizeType"” type, whose value is equal to the size of the C type derived from the expression's l-value, value or type name operand:
	
\begin{code}
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (SizeOf (e {-,-})))))))))))))))) = do
	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfLValue{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e)))))))))))))) --
		  				 <|> meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e))))))))))))))
	require $ (isObjectType{--}(t_e))
	return $ (_S_e, _T_e, _D_e `mapSetDifference` dom{--}(_V_e), {-sem-}(sizeType), sizeOfType{--}(t_e))

meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (SizeOfType (type_name)))))))))))))))) = do
	(_S_t, _T_t, _D_t, t_t) <- meaningOfTypeName{--}(_S, _T, _D, _I {->-}, type_name)
	require $ (isObjectType{--}(t_t))
	return $ (_S_t, _T_t, _D_t, {-sem-}(sizeType), sizeOfType{--}(t_t))
\end{code}

More so, every cast operation of the form “\!"Cast (type_name) {-!-} e"\,” belongs to this semantic family, provided that the specified type name represents an integral type and "e" constitutes an integral constant expression or a floating constant. The value of such an expression is obtained from the corresponding atom “"OP1 (CNV phi' {-!-}{-!-}) (phi) {--} {-hv-}(IMM (n) (phi))"”, in which "phi" and "phi'" represent Etude formats derived from the respective C types of the entity's expression and type name operands, while "n" depicts the integral constant denotation of "e":

\begin{code}
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (Cast (type_name) {-!-} e {-,-})))))))))))))) = do
	(_S_t,   _T_t,   _D_t,  t_t) <- meaningOfTypeName{--}(_S, _T, _D, _I {->-}, type_name)
	(_S_e,   _T_e,   _D_e,  t_e, n_e) <- meaningOfIntegralConstantExpression{--}(_S_t, _T_t, _D_t, _I {->-}, --
												E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e))))))))))))) --
				  		<|> meaningOfFloatingConstantExpression{--}(_S_t, _T_t, _D_t, _I {->-}, --
				  								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
	require $ (isIntegralType{--}(t_t))
	(n) <- valueOfAtom (formatOfType{--}(t_t)) --
				{-sem-}(OP1 (CNV {-sem-}(formatOfType{--}(t_t))) ({-sem-}(formatOfType{--}(t_e))) (IMM (toRational n_e) ({-sem-}(formatOfType{--}(t_e)))))
	return $ (_S_e, _T_e, _D_e, t_t, discardFraction n)
\end{code}

in which the notation “"meaningOfFloatingConstantExpression{--}(_S, _T, _D, _I {->-}, e)"” obtains the meaning of an integral constant expression formed from a floating constant as follows:

\IndexTerm		{meaningOfFloatingConstantExpression}		{D_FCE expression}
\InsertHaskell		{meaningOfFloatingConstantExpression}

Similarly, the denotations of all parenthesised versions of an integral constant expression and all applications of the unary “"Plus"”, “"Minus"” and “"Not"” operator to such an operand constitute integral constant expressions with the following natural denotations:

\begin{code}
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (NestedExpression (e)))))))))))))))))) ---
  =	do	meaningOfIntegralConstantExpression{--}(_S, _T, _D, _I {->-}, e)
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (Plus) (e)))))))))))))))) ---
  =	do	(_S_e, _T_e, _D_e, t_e, n_e) <- meaningOfIntegralConstantExpression --
  					{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
  		(n) <- valueOfAtom (formatOfType{--}(integralPromotedType{--}(t_e))) --
  					{-sem-}(OP1 (CNV {-sem-}(formatOfType{--}(integralPromotedType{--}(t_e)))) ({-sem-}(formatOfType{--}(t_e))) {--} --
  								{-hv-}(IMM (toRational n_e) ({-sem-}(formatOfType{--}(t_e)))))
  		return $ (_S_e, _T_e, _D_e, integralPromotedType{--}(t_e), discardFraction n)
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (Minus) (e)))))))))))))))) ---
  =	do	(_S_e, _T_e, _D_e, t_e, n_e) <-  meaningOfIntegralConstantExpression --
  					{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
  		(n) <- valueOfAtom (formatOfType{--}(integralPromotedType{--}(t_e))) --
					{-sem-}(OP1 (NEG) ({-sem-}(formatOfType{--}(t_e))) {--} --
  							 {-hv-}(OP1 (CNV {-sem-}(formatOfType{--}(integralPromotedType{--}(t_e)))) ({-sem-}(formatOfType{--}(t_e))) {--} --
  									{-hv-}(IMM (toRational n_e) ({-sem-}(formatOfType{--}(t_e))))))
  		return $ (_S_e, _T_e, _D_e, integralPromotedType{--}(t_e), discardFraction n)
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (Not) (e)))))))))))))))) ---
  =	do	(_S_e, _T_e, _D_e, t_e, n_e) <- meaningOfIntegralConstantExpression --
  					{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
  		(n) <- valueOfAtom (formatOfType{--}(integralPromotedType{--}(t_e))) --
					{-sem-}(OP1 (NOT) ({-sem-}(formatOfType{--}(t_e))) {--} --
  							 {-hv-}(OP1 (CNV {-sem-}(formatOfType{--}(integralPromotedType{--}(t_e)))) ({-sem-}(formatOfType{--}(t_e))) {--} --
  									{-hv-}(IMM (toRational n_e) ({-sem-}(formatOfType{--}(t_e))))))
  		return $ (_S_e, _T_e, _D_e, integralPromotedType{--}(t_e), discardFraction n)
\end{code}

In the same vein, every well-formed application of the “"`Product`"”, “"`Quotient`"”, “"`Remainder`"”, “"`Sum`"”, “"`Difference`"”, “"`LeftShift`"”, “"`RightShift`"”, “"`And`"”, “"`ExclusiveOr`"” and “"`InclusiveOr`"” operator to a pair of integral constant operands belongs to this semantic family. For conciseness, the denotations of all such applications are derived from the corresponding binary Etude operator by the following common construction:

\IndexTerm		{meaningOfBinaryIntegralConstantExpression}	{D_BICE expression expression}
\InsertHaskell		{meaningOfBinaryIntegralConstantExpression}

so that:

\begin{code}			 					
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (e1 {-hfil-}`Product` 	e2))))))))))))) ---
  =	meaningOfBinaryIntegralConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}MUL  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e1))))))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e2)))))))))))))
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (e1 {-hfil-}`Quotient` 	e2))))))))))))) ---
  =	meaningOfBinaryIntegralConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}DIV  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e1))))))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e2)))))))))))))
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (e1 {-hfil-}`Remainder` 	e2))))))))))))) ---
  =	meaningOfBinaryIntegralConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}REM  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e1))))))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e2)))))))))))))
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (e1 {-hfil-}`Sum` 		e2)))))))))))) ---
  =	meaningOfBinaryIntegralConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}ADD  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e1)))))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e2))))))))))))
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (e1 {-hfil-}`Difference` 	e2)))))))))))) ---
  =	meaningOfBinaryIntegralConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}SUB  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e1)))))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e2))))))))))))
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (e1 {-hfil-}`LeftShift` 		e2))))))))))) ---
  =	meaningOfBinaryIntegralConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}SHL  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 e1))))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e2)))))))))))
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (e1 {-hfil-}`RightShift` 		e2))))))))))) ---
  =	meaningOfBinaryIntegralConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}SHR  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 e1))))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e2)))))))))))
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (e1 {-hfil-}`And` 					e2)))))))) ---
  =	meaningOfBinaryIntegralConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}AND  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 e1)))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 e2))))))))
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (e1 {-hfil-}`ExclusiveOr` 				e2))))))) ---
  =	meaningOfBinaryIntegralConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}XOR  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 e1))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 e2)))))))
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (e1 {-hfil-}`InclusiveOr` 					e2)))))) ---
  =	meaningOfBinaryIntegralConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}IOR  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 e1)))), --
 								E1 (E2 (E3 (E4 (E5 (E6 e2))))))
\end{code}

Similarly, integral constant denotations of the relational and equality operators “"`LessThan`"”, “"`GreaterThan`"”, “"`LessOrEqualTo`"”, “"`GreaterOrEqualTo`"”, “"`EqualTo`"” and “"`NotEqualTo`"” are also derived from their Etude equivalents using the "valueOfAtom" combinator, with the result always assuming the plain “"PlainIntType"” type:

\IndexTerm		{meaningOfRelationalIntegralConstantExpression}	{D_RICE expression expression}
\InsertHaskell		{meaningOfRelationalIntegralConstantExpression}

In particular:

\begin{code}
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (e1 {-hfil-}`LessThan` 		e2)))))))))) ---
  =	meaningOfRelationalIntegralConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}CLT  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 e1)))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 e2))))))))))
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (e1 {-hfil-}`GreaterThan` 		e2)))))))))) ---
  =	meaningOfRelationalIntegralConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}CGT  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 e1)))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 e2))))))))))
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (e1 {-hfil-}`LessOrEqualTo` 	e2)))))))))) ---
  =	meaningOfRelationalIntegralConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}CLE  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 e1)))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 e2))))))))))
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (e1 {-hfil-}`GreaterOrEqualTo` 	e2)))))))))) ---
  =	meaningOfRelationalIntegralConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}CGE  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 e1)))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 e2))))))))))
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (e1 {-hfil-}`EqualTo` 			e2))))))))) ---
  =	meaningOfRelationalIntegralConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}CEQ  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 e1))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 e2)))))))))
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (e1 {-hfil-}`NotEqualTo` 		e2))))))))) ---
  =	meaningOfRelationalIntegralConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}CNE  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 e1))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 e2)))))))))
\end{code}

Finally, the remaining four logical and conditional forms of integral constant expressions obtained from applications of the C operators “"LogicalNot"”, “"`LogicalAnd`"”, “"`LogicalOr`"” and “"Conditional () {-?-} ({-!-}) {-:-} ({-!-})"” are formalised as follows:

\begin{code}
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (LogicalNot) (e)))))))))))))))) ---
  =	do	(_S_e,  _T_e,  _D_e,  t_e,  n_e  ) <- meaningOfIntegralConstantExpression --
  								{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
  		return $ (_S_e,  _T_e,  _D_e,  {-sem-}(PlainIntType), fromBool (n_e == 0))
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (e1 `LogicalAnd` e2))))) ---
  =	do	(_S1,   _T1,   _D1,   t1,   n1   ) <- meaningOfIntegralConstantExpression --
  								{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 (E4 e1))))
		(_S2,   _T2,   _D2,   t2,   n2   ) <- meaningOfIntegralConstantExpression --
								{--}(_S1,  _T1,  _D1,  _I {->-}, E1 (E2 (E3 (E4 (E5 e2)))))
		return $ (_S2,   _T2,   _D2,   {-sem-}(PlainIntType), fromBool (n1 /= 0 && n2 /= 0))
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (e1 `LogicalOr` e2)))) ---
  =	do	(_S1,   _T1,   _D1,   t1,   n1   ) <- meaningOfIntegralConstantExpression --
  								{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 e1)))
		(_S2,   _T2,   _D2,   t2,   n2   ) <- meaningOfIntegralConstantExpression --
								{--}(_S1,  _T1,  _D1,  _I {->-}, E1 (E2 (E3 (E4 e2))))
		return $ (_S2,   _T2,   _D2,   {-sem-}(PlainIntType), fromBool (n1 /= 0 || n2 /= 0))
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (Conditional (e1) {-?-} (e2) {-:-} (e3)))) ---
  =	do	(_S1,   _T1,   _D1,   t1,   n1   ) <- meaningOfIntegralConstantExpression ---
  								{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 e1)))
		(_S2,   _T2,   _D2,   t2,   n2   ) <- meaningOfIntegralConstantExpression ---
  								{--}(_S1,  _T1,  _D1,  _I {->-}, e2)
		(_S3,   _T3,   _D3,   t3,   n3   ) <- meaningOfIntegralConstantExpression ---
  								{--}(_S2,  _T2,  _D2,  _I {->-}, E1 (E2 e3))
		return $ (_S3, _T3, _D3, t2 `commonArithmeticType` t3, if n1 /= 0 then n2 else n3)
meaningOfIntegralConstantExpression {-sem-}(_S, _T, _D, _I {->-}, other) ---
  =	fail ("Semantic error")
\end{code}

which, intuitively, computes the meaning of these operations from an appropriate comparison of their operands' integer values against the numeric constant 0. No other C operator may appear in the syntax of integral constant expression entities.

\end{subsubsection}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Expressions/Constants/NullPointers}
\set{title}		{Null Pointer Constants}
\begin{subsubsection}

Every well-formed integral constant expression with a zero value, as well as all casts of such an expression into a pointer to the “"VoidType"\,” type, are known as «null pointer constants». Formally, the denotations of all such entities are constructed by the following simple Haskell derivation:

\BeginHaskellToken	{meaningOfNullPointerConstant}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{null}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, TypeT, AtomT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfNullPointerConstant}			{D_NULL expression}

\begin{code}
meaningOfNullPointerConstant {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, ExpressionT) -> _M{--}(S, S, D, TypeT, AtomT VarT)

meaningOfNullPointerConstant {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (NestedExpression (e)))))))))))))))))) --
  = meaningOfNullPointerConstant{--}(_S, _T, _D, _I {->-}, e)

meaningOfNullPointerConstant {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (Cast (type_name) {-!-} e {-,-})))))))))))))) = do
	(_S_t,   _T_t,  _D_t,  t_t) <- meaningOfTypeName {--}(_S, _T, _D, _I {->-}, type_name)
	(_S_e,   _T_e,  _D_e,  t_e, n_e) <- meaningOfIntegralConstantExpression{--}(_S_t, _T_t, _D_t, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e))))))))))))) --
			  		  <|> meaningOfFloatingConstantExpression{--}(_S_t, _T_t, _D_t, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
	(n) <- valueOfAtom (formatOfType{--}(t_t)) --
				{-sem-}(OP1 (CNV {-sem-}(formatOfType{--}(t_t))) ({-sem-}(formatOfType{--}(t_e))) {--} {-hv-}(IMM (toRational n_e) ({-sem-}(formatOfType{--}(t_e)))))
	require $ (n == 0 && (isIntegralType{--}(t_t) || (isPointerType{--}(t_t) && baseOfType{--}(t_t) == {-sem-}(VoidType) && isIntegralType{--}(t_e))))
	return $ (_S_e, _T_e, _D_e, t_t, {-sem-}(IMM (0) ({-sem-}(formatOfType{--}(t_e)))))

meaningOfNullPointerConstant {-sem-}(_S, _T, _D, _I {->-}, e) = do
	(_S_e,   _T_e,  _D_e,  t_e, n_e) <- meaningOfIntegralConstantExpression {--}(_S, _T, _D, _I {->-}, e)
	require $ (n_e == 0)
	return $ (_S_e, _T_e, _D_e, t_e, {-sem-}(IMM (0) ({-sem-}(formatOfType{--}(t_e)))))
\end{code}

In many contexts, it is also convenient to devise a boolean predicate function “"isNullPointerConstant"”, which, given some expression "e" and its lexical context "(_S, _T, _D, _I)", returns a true value if and only if "e" represents a null pointer constant, without the need to construct the full denotation of that expression. In Haskell, such predicate can be implemented trivially as follows:

\BeginHaskellToken	{isNullPointerConstant}			{Open}	{NULL}
  \HaskellArgument							{(S, S, D, I {->-}, ExpressionT)}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isNullPointerConstant}				{NULL expression (bool)}

\begin{code}
isNullPointerConstant {-[*]-} :: (S, S, D, I {->-}, ExpressionT) -> Bool

isNullPointerConstant {-sem-}(_S, _T, _D, _I {->-}, e) = (meaningOfNullPointerConstant{--}(_S, _T, _D, _I {->-}, e) /= Nothing)
\end{code}

which derives the null pointer denotation of "e" in the standard “Maybe” monad and compares the resulting optional value against its monadic zero “Nothing”.

\end{subsubsection}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Expressions/Constants/Static}
\set{title}		{Static Initialiser Expressions}
\begin{subsubsection}

A further application of the constant expression syntax is found in the context of «initialiser» entities, whose precise structure and semantics are defined later in Section \ref{C/Declarations/Initialisers}. When used as an initialiser for an object with a static linkage, an expression must always assume one of the following four syntactic forms:

\begin{list}

•	an «arithmetic constant expression» of a structure defined in Section \ref{C/Expressions/Constants/Arithmetic}, or

•	a null pointer constant, or

•	an «address constant», described in Section \ref{C/Expressions/Constants/Address}, or

•	an application of the binary “"`Sum`"” or “"`Difference`"” operator to an address constant and an integral constant expression.

\end{list}

Formally, the all such static initialiser expressions denote Etude atoms, whose values are defined by the following Haskell construction:

\BeginHaskellToken	{meaningOfStaticInitialiserExpression}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{sie}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, TypeT, AtomT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfStaticInitialiserExpression}		{D_SIE expression}

\begin{code}
meaningOfStaticInitialiserExpression {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, ExpressionT) -> _M{--}(S, S, D, TypeT, AtomT VarT)

meaningOfStaticInitialiserExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (NestedExpression (e)))))))))))))))))) ---
  =	do	meaningOfStaticInitialiserExpression	{--}(_S, _T, _D, _I {->-}, e)
meaningOfStaticInitialiserExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (e1 `Sum` e2)))))))))))) ---
  =	do	meaningOfArithmeticConstantExpression	{-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (e1 `Sum` e2))))))))))))
  <|>	do	meaningOfNullPointerConstant		{-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (e1 `Sum` e2))))))))))))
  <|>	do	meaningOfAddressConstant		{-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (e1 `Sum` e2))))))))))))
  <|>	do	(_S1, _T1, _D1, t1, alpha) ---
			<- meaningOfAddressConstant ---
  					{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e1)))))))))))
  		(_S2, _T2, _D2, t2, n) ---
			<- meaningOfIntegralConstantExpression ---
  					{--}(_S1,  _T1,  _D1,  _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e2))))))))))))
		require $ (isPointerType{--}(t1) && isObjectType{--}(baseOfType{--}(t1)))
		return $ (_S2, _T2, _D2, t1, {-sem-}(OP2 (alpha) (ADD) ({-sem-}(formatOfType{--}(t1))) --
  								(IMM (toRational {-sem-}(n * sizeOfType{--}(baseOfType{--}(t1)))) ({-sem-}(formatOfType{--}(t2))))))
  <|>	do	(_S1, _T1, _D1, t1, n) ---
			<- meaningOfIntegralConstantExpression ---
  					{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e1)))))))))))
  		(_S2, _T2, _D2, t2, alpha) ---
			<- meaningOfAddressConstant ---
  					{--}(_S1,  _T1,  _D1,  _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e2))))))))))))
		require $ (isPointerType{--}(t2) && isObjectType{--}(baseOfType{--}(t2)))
  		return $ (_S2, _T2, _D2, t2, {-sem-}(OP2 (alpha) (ADD) ({-sem-}(formatOfType{--}(t2))) --
  								(IMM (toRational {-sem-}(n * sizeOfType{--}(baseOfType{--}(t2)))) ({-sem-}(formatOfType{--}(t1))))))
meaningOfStaticInitialiserExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (e1 `Difference` e2)))))))))))) ---
  =	do	meaningOfArithmeticConstantExpression	{-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (e1 `Difference` e2))))))))))))
  <|>	do	meaningOfNullPointerConstant		{-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (e1 `Difference` e2))))))))))))
  <|>	do	meaningOfAddressConstant		{-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (e1 `Difference` e2))))))))))))
  <|>	do	(_S1, _T1, _D1, t1, alpha) ---
			<- meaningOfAddressConstant ---
  					{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e1)))))))))))
  		(_S2, _T2, _D2, t2, n) ---
			<- meaningOfIntegralConstantExpression ---
  					{--}(_S1,  _T1,  _D1,  _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e2))))))))))))
		require $ (isPointerType{--}(t1) && isObjectType{--}(baseOfType{--}(t1)))
		return $ (_S2, _T2, _D2, t1, {-sem-}(OP2 (alpha) (ADD) ({-sem-}(formatOfType{--}(t1))) --
  					(IMM (toRational {-sem-}(n * negate (sizeOfType{--}(baseOfType{--}(t1))))) ({-sem-}(formatOfType{--}(t2)))))) {-!-} {-!-} {-!-} {-!-} {-!-}
meaningOfStaticInitialiserExpression {-sem-}(_S, _T, _D, _I {->-}, e) ---
  =	do	meaningOfArithmeticConstantExpression	{--}(_S, _T, _D, _I {->-}, e)
  <|>	do	meaningOfNullPointerConstant		{--}(_S, _T, _D, _I {->-}, e)
  <|>	do	meaningOfAddressConstant		{--}(_S, _T, _D, _I {->-}, e)
\end{code}

An astute reader will observe that the denotation of such a constant expression may, on occasions, represent an ill-formed Etude atom, so that the ultimate validity of C initialiser expressions must be always determined by a further semantic scrutiny of the resulting Etude program against a set of implementation-defined criteria outlined earlier in Chapter \ref{Lambda}.

\end{subsubsection}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Expressions/Constants/Arithmetic}
\set{title}		{Arithmetic Constant Expression}
\begin{subsubsection}

The family of «arithmetic constant expressions», used to initialise static objects of an integral or floating C type, represents a slight relaxation of the integral constant expression constraints. In particular, arithmetic constant expressions support all of the C operators admitted into the earlier integral family, and, further, allow for their applications to arbitrary constant operands of any well-formed arithmetic C type. Accordingly, a formal definition of their denotations follows a structure similar to that of the previous definition described in Section \ref{C/Expressions/Constants/Integral} above, except that no specific restriction is imposed on the use of floating point operand values and that the resulting denotation is represented by a constant atomic value rather than a simple numeric quantity. Formally:

\BeginHaskellToken	{meaningOfArithmeticConstantExpression}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{ace}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, TypeT, AtomT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfArithmeticConstantExpression}		{D_ACE expression}

\begin{code}
meaningOfArithmeticConstantExpression {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, ExpressionT) -> _M{--}(S, S, D, TypeT, AtomT VarT)
\end{code}

In particular, every well-formed integer, character and enumeration constants represents an arithmetic constant expression, whose type and atomic value is derived directly from the constant's lexical syntax by the process described earlier in Section \ref{C/Constants}:

\begin{code}
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (SizeOf (e {-,-})))))))))))))))) = do
	(_S_e, _T_e, _D_e, _V_e, t_e, tau_e) <- meaningOfLValue{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e)))))))))))))) --
		  				 <|> meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 e))))))))))))))
	require $ (isObjectType{--}(t_e))
	return $ (_S_e, _T_e, _D_e `mapSetDifference` dom{--}(_V_e), {-sem-}(sizeType), --
				{-sem-}(IMM (toRational {-sem-}(sizeOfType{--}(t_e))) ({-sem-}(formatOfType{--}(sizeType)))))

meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (SizeOfType (type_name)))))))))))))))) = do
	(_S_t, _T_t, _D_t, t_t) <- meaningOfTypeName{--}(_S, _T, _D, _I {->-}, type_name)
	require $ (isObjectType{--}(t_t))
	return $ (_S_t, _T_t, _D_t, {-sem-}(sizeType), {-sem-}(IMM (toRational {-sem-}(sizeOfType{--}(t_t))) ({-sem-}(formatOfType{--}(sizeType)))))

meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (Cast (type_name) {-!-} e {-,-})))))))))))))) = do
  	(_S_t,   _T_t,   _D_t,  t_t) <- meaningOfTypeName{--}(_S, _T, _D, _I {->-}, type_name)
	(_S_e,   _T_e,   _D_e,  t_e, alpha) --
					<- meaningOfArithmeticConstantExpression{--}(_S_t, _T_t, _D_t, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
	require $ (isArithmeticType{--}(t_t))
	(x) <- valueOfAtom (formatOfType{--}(t_t)) {-sem-}(OP1 (CNV {-sem-}(formatOfType{--}(t_t))) ({-sem-}(formatOfType{--}(t_e))) {--} {-hv-}(alpha))
	return $ (_S_e, _T_e, _D_e, t_t, {-sem-}(IMM (x) ({-sem-}(formatOfType{--}(t_t)))))

meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (Constant constant))))))))))))))))) ---
  = do		(t, x) <- meaningOfConstant {--}(_S {->-}, constant)
  		return $ (_S, _T, _D, t, {-sem-}(IMM (x) ({-sem-}(formatOfType{--}(t)))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (Plus) (e)))))))))))))))) ---
  = do		(_S_e, _T_e, _D_e, t_e, alpha) <- meaningOfArithmeticConstantExpression --
		  					{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
  		(x) <- valueOfAtom (formatOfType{--}(integralPromotedType{--}(t_e))) --
  					{-sem-}(OP1 (CNV {-sem-}(formatOfType{--}(integralPromotedType{--}(t_e)))) ({-sem-}(formatOfType{--}(t_e))) {--} {-hv-}(alpha))
  		return $ (_S_e, _T_e, _D_e, integralPromotedType{--}(t_e), {-sem-}(IMM (x) ({-sem-}(formatOfType{--}(integralPromotedType{--}(t_e))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (Minus) (e)))))))))))))))) ---
  = do		(_S_e, _T_e, _D_e, t_e, alpha) <- meaningOfArithmeticConstantExpression --
		  					{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
  		(x) <- valueOfAtom (formatOfType{--}(integralPromotedType{--}(t_e))) --
  					{-sem-}(OP1 (NEG) ({-sem-}(formatOfType{--}(t_e))) {--} --
  							 {-hv-}(OP1 (CNV {-sem-}(formatOfType{--}(integralPromotedType{--}(t_e)))) ({-sem-}(formatOfType{--}(t_e))) {--} {-hv-}(alpha)))
  		return $ (_S_e, _T_e, _D_e, integralPromotedType{--}(t_e), {-sem-}(IMM (x) ({-sem-}(formatOfType{--}(integralPromotedType{--}(t_e))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (Not) (e)))))))))))))))) ---
  = do		(_S_e, _T_e, _D_e, t_e, alpha) <- meaningOfArithmeticConstantExpression --
		  					{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
  		(x) <- valueOfAtom (formatOfType{--}(integralPromotedType{--}(t_e))) --
  					{-sem-}(OP1 (NOT) ({-sem-}(formatOfType{--}(t_e))) {--} --
  							 {-hv-}(OP1 (CNV {-sem-}(formatOfType{--}(integralPromotedType{--}(t_e)))) ({-sem-}(formatOfType{--}(t_e))) {--} {-hv-}(alpha)))
  		return $ (_S_e, _T_e, _D_e, integralPromotedType{--}(t_e), {-sem-}(IMM (x) ({-sem-}(formatOfType{--}(integralPromotedType{--}(t_e))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (LogicalNot) (e)))))))))))))))) ---
  = do		(_S_e, _T_e, _D_e, t_e, alpha) <- meaningOfArithmeticConstantExpression --
		  					{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
  		(x) <- valueOfAtom (formatOfType{-sem-}(PlainIntType)) --
  					{-sem-}(OP2 (alpha) (CEQ) ({-sem-}(formatOfType{--}(t_e))) (IMM (0) ({-sem-}(formatOfType{--}(t_e)))))
  		return $ (_S_e, _T_e, _D_e, {-sem-}(PlainIntType), {-sem-}(IMM (x) ({-sem-}(formatOfType{-sem-}(PlainIntType)))))


meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (e1 `LogicalAnd` e2))))) = do
	(_S1, _T1, _D1, t1, alpha1) <- meaningOfArithmeticConstantExpression{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 (E4 e1))))
	(_S2, _T2, _D2, t2, alpha2) <- meaningOfArithmeticConstantExpression{--}(_S1,  _T1,  _D1,  _I {->-}, E1 (E2 (E3 (E4 (E5 e2)))))
	(x) <- valueOfAtom (formatOfType{-sem-}(PlainIntType)) --
 					{-sem-}(OP2 ({-hv-}(OP2 (alpha1) (CNE) ({-sem-}(formatOfType{--}(t1))) (IMM (0) ({-sem-}(formatOfType{--}(t1)))))) --
  							(AND) ({-sem-}(formatOfType{-sem-}(PlainIntType))) --
  							({-hv-}(OP2 (alpha2) (CNE) ({-sem-}(formatOfType{--}(t2))) (IMM (0) ({-sem-}(formatOfType{--}(t2)))))))
	return $ (_S2, _T2, _D2, {-sem-}(PlainIntType), {-sem-}(IMM (x) ({-sem-}(formatOfType{-sem-}(PlainIntType)))))

meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (e1 `LogicalOr` e2)))) = do
	(_S1, _T1, _D1, t1, alpha1) <- meaningOfArithmeticConstantExpression{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 e1)))
	(_S2, _T2, _D2, t2, alpha2) <- meaningOfArithmeticConstantExpression{--}(_S1,  _T1,  _D1,  _I {->-}, E1 (E2 (E3 (E4 e2))))
	(x) <- valueOfAtom (formatOfType{-sem-}(PlainIntType)) --
  					{-sem-}(OP2 ({-hv-}(OP2 (alpha1) (CNE) ({-sem-}(formatOfType{--}(t1))) (IMM (0) ({-sem-}(formatOfType{--}(t1)))))) --
  							(IOR) ({-sem-}(formatOfType{-sem-}(PlainIntType))) --
  							({-hv-}(OP2 (alpha2) (CNE) ({-sem-}(formatOfType{--}(t2))) (IMM (0) ({-sem-}(formatOfType{--}(t2)))))))
	return $ (_S2, _T2, _D2, {-sem-}(PlainIntType), {-sem-}(IMM (x) ({-sem-}(formatOfType{-sem-}(PlainIntType)))))

meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (Conditional (e1) {-?-} (e2) {-:-} (e3)))) = do
	(_S1, _T1, _D1, t1, alpha1) <- meaningOfArithmeticConstantExpression{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 e1)))
	(_S2, _T2, _D2, t2, alpha2) <- meaningOfArithmeticConstantExpression{--}(_S1,  _T1,  _D1,  _I {->-}, e2)
	(_S3, _T3, _D3, t3, alpha3) <- meaningOfArithmeticConstantExpression{--}(_S2,  _T2,  _D2,  _I {->-}, E1 (E2 e3))
	(x) <- valueOfAtom (formatOfType{-sem-}(PlainIntType)) {--}(alpha1)
	return $ (_S3, _T3, _D3, t2 `commonArithmeticType` t3, if x /= 0 then alpha2 else alpha3)
\end{code}

Otherwise, all well-formed applications of the binary “"`Product`"”, “"`Quotient`"”, “"`Remainder`"”, “"`Sum`"”, “"`Difference`"”, “"`LeftShift`"”, “"`RightShift`"”, “"`And`"”, “"`ExclusiveOr`"”, “"`InclusiveOr`"”, “"`LessThan`"”, “"`GreaterThan`"”, “"`LessOrEqualTo`"”, “"`GreaterOrEqualTo`"”, “"`EqualTo`"”, “"`NotEqualTo`"” operators to constant arithmetic expression operands and all parenthesised versions of such an expression also belong to this semantic family. Their denotations are  derived from the following familiar mapping of these C operators to their Etude equivalents:

\begin{code}
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (e1 {-hfil-}`Product` 		e2))))))))))))) ---
  = meaningOfBinaryArithmeticConstantExpression	 	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}MUL  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e1))))))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e2)))))))))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (e1 {-hfil-}`Quotient` 		e2))))))))))))) ---
  = meaningOfBinaryArithmeticConstantExpression	 	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}DIV  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e1))))))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e2)))))))))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (e1 {-hfil-}`Remainder` 		e2))))))))))))) ---
  = meaningOfIntegralArithmeticConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}REM  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e1))))))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e2)))))))))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (e1 {-hfil-}`Sum` 			e2)))))))))))) ---
  = meaningOfBinaryArithmeticConstantExpression		{--}(_S, _T, _D, _I, {-sem-}({-hfil-}ADD  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e1)))))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e2))))))))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (e1 {-hfil-}`Difference` 		e2)))))))))))) ---
  = meaningOfBinaryArithmeticConstantExpression		{--}(_S, _T, _D, _I, {-sem-}({-hfil-}SUB  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e1)))))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 e2))))))))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (e1 {-hfil-}`LeftShift` 			e2))))))))))) ---
  = meaningOfIntegralArithmeticConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}SHL  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 e1))))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e2)))))))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (e1 {-hfil-}`RightShift` 			e2))))))))))) ---
  = meaningOfIntegralArithmeticConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}SHR  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 e1))))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 e2)))))))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (e1 {-hfil-}`And` 					e2)))))))) ---
  = meaningOfIntegralArithmeticConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}AND  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 e1)))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 e2))))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (e1 {-hfil-}`ExclusiveOr` 					e2))))))) ---
  = meaningOfIntegralArithmeticConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}XOR  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 e1))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 e2)))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (e1 {-hfil-}`InclusiveOr` 					e2)))))) ---
  = meaningOfIntegralArithmeticConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}IOR  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 e1)))), --
 								E1 (E2 (E3 (E4 (E5 (E6 e2))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (e1 {-hfil-}`LessThan` 				e2)))))))))) ---
  = meaningOfRelationalArithmeticConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}CLT  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 e1)))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 e2))))))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (e1 {-hfil-}`GreaterThan` 			e2)))))))))) ---
  = meaningOfRelationalArithmeticConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}CGT  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 e1)))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 e2))))))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (e1 {-hfil-}`LessOrEqualTo` 			e2)))))))))) ---
  = meaningOfRelationalArithmeticConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}CLE  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 e1)))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 e2))))))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (e1 {-hfil-}`GreaterOrEqualTo` 			e2)))))))))) ---
  = meaningOfRelationalArithmeticConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}CGE  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 e1)))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 e2))))))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (e1 {-hfil-}`EqualTo` 				e2))))))))) ---
  = meaningOfRelationalArithmeticConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}CEQ  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 e1))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 e2)))))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (e1 {-hfil-}`NotEqualTo` 				e2))))))))) ---
  = meaningOfRelationalArithmeticConstantExpression	{--}(_S, _T, _D, _I, {-sem-}({-hfil-}CNE  ) {->-}, --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 e1))))))), --
 								E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 e2)))))))))
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (NestedExpression (e)))))))))))))))))) ---
  = meaningOfArithmeticConstantExpression{--}(_S, _T, _D, _I {->-}, e)
meaningOfArithmeticConstantExpression {-sem-}(_S, _T, _D, _I {->-}, other) ---
  = fail ("Semantic error")
\end{code}

assuming that the individual meanings of all binary arithmetic and relational operations listed above are derived systematically from their Etude equivalents by the following Haskell constructions:

\BeginHaskellToken	{meaningOfBinaryArithmeticConstantExpression}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{bace}}}
  \HaskellContext									{(MonadFix _M)}
  \HaskellArgument									{(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT)}
  \HaskellResult									{_M{--}(S, S, D, TypeT, AtomT VarT)}
\EndHaskellToken

\BeginHaskellToken	{meaningOfIntegralArithmeticConstantExpression}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{iace}}}
  \HaskellContext									{(MonadFix _M)}
  \HaskellArgument									{(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT)}
  \HaskellResult									{_M{--}(S, S, D, TypeT, AtomT VarT)}
\EndHaskellToken

\BeginHaskellToken	{meaningOfRelationalArithmeticConstantExpression}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{race}}}
  \HaskellContext									{(MonadFix _M)}
  \HaskellArgument									{(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT)}
  \HaskellResult									{_M{--}(S, S, D, TypeT, AtomT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfBinaryArithmeticConstantExpression}			{D_BACE expression expression}
\IndexTerm		{meaningOfIntegralArithmeticConstantExpression}			{D_BACE expression expression}
\IndexTerm		{meaningOfRelationalArithmeticConstantExpression}		{D_RACE expression expression}

\begin{code}
meaningOfBinaryArithmeticConstantExpression {-[*]-}, meaningOfIntegralArithmeticConstantExpression {-[*]-}, meaningOfRelationalArithmeticConstantExpression {-[*]-} --
		:: (MonadFix _M, Monad _M) =>
	(S, S, D, I, BinaryOpT {->-}, ExpressionT, ExpressionT) -> _M{--}(S, S, D, TypeT, AtomT VarT)

meaningOfBinaryArithmeticConstantExpression 	 {-sem-}(_S, _T, _D, _I, op {->-}, e1, e2) = do
	(_S1, _T1, _D1, t1, alpha1) <- meaningOfArithmeticConstantExpression {--}(_S,   _T,   _D,   _I {->-}, e1)
	(_S2, _T2, _D2, t2, alpha2) <- meaningOfArithmeticConstantExpression {--}(_S1,  _T1,  _D1,  _I {->-}, e2)
	(x) <- valueOfAtom (formatOfType{--}(t1 `commonArithmeticType` t2)) ---
  					{-sem-}(OP2 ({-hv-}(OP1 (CNV {-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) ({-sem-}(formatOfType{--}(t1))) {--}{-hv-}(alpha1))) ---
  							(op) ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) --
  							({-hv-}(OP1 (CNV {-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) ({-sem-}(formatOfType{--}(t2))) {--}{-hv-}(alpha2))))
	return $ (_S2, _T2, _D2, t1 `commonArithmeticType` t2, {-sem-}(IMM (x) ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2)))))

meaningOfIntegralArithmeticConstantExpression 	 {-sem-}(_S, _T, _D, _I, op {->-}, e1, e2) = do
	(_S1, _T1, _D1, t1, alpha1) <- meaningOfArithmeticConstantExpression {--}(_S,   _T,   _D,   _I {->-}, e1)
	(_S2, _T2, _D2, t2, alpha2) <- meaningOfArithmeticConstantExpression {--}(_S1,  _T1,  _D1,  _I {->-}, e2)
	(x) <- valueOfAtom (formatOfType{--}(t1 `commonArithmeticType` t2)) ---
  					{-sem-}(OP2 ({-hv-}(OP1 (CNV {-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) ({-sem-}(formatOfType{--}(t1))) {--}{-hv-}(alpha1))) ---
  							(op) ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) --
  							({-hv-}(OP1 (CNV {-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) ({-sem-}(formatOfType{--}(t2))) {--}{-hv-}(alpha2))))
	require $ isIntegralType{--}(t1) && isIntegralType{--}(t2)
	return $ (_S2, _T2, _D2, t1 `commonArithmeticType` t2, {-sem-}(IMM (x) ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2)))))

meaningOfRelationalArithmeticConstantExpression	 {-sem-}(_S, _T, _D, _I, op {->-}, e1, e2) = do
	(_S1, _T1, _D1, t1, alpha1) <- meaningOfArithmeticConstantExpression {--}(_S,   _T,   _D,   _I {->-}, e1)
	(_S2, _T2, _D2, t2, alpha2) <- meaningOfArithmeticConstantExpression {--}(_S1,  _T1,  _D1,  _I {->-}, e2)
	(x) <- valueOfAtom (formatOfType{--}(t1 `commonArithmeticType` t2)) ---
  					{-sem-}(OP2 ({-hv-}(OP1 (CNV {-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) ({-sem-}(formatOfType{--}(t1))) {--}{-hv-}(alpha1))) ---
  							(op) ({-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) --
  							({-hv-}(OP1 (CNV {-sem-}(formatOfType{--}(t1 `commonArithmeticType` t2))) ({-sem-}(formatOfType{--}(t2))) {--}{-hv-}(alpha2))))
	return $ (_S2, _T2, _D2, {-sem-}(PlainIntType), {-sem-}(IMM (x) ({-sem-}(formatOfType{-sem-}(PlainIntType)))))
\end{code}

A careful reader will observe that the meaning of every well-formed arithmetic constant always represents a valid Etude atom of the form “"IMM (x) (phi)"\!”, whose reduction is performed internally by the compiler as part of the translation process, prior to an eventual execution of the translated program. This behaviour is explicitly mandated by the C standard and provides for a potential discrepancy between the evaluation rules of floating point operations in their ordinary and constant contexts, as discussed earlier in Section \ref{C/Constants}.

\end{subsubsection}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Expressions/Constants/Address}
\set{title}		{Address Constants}
\begin{subsubsection}

The final constant expression category, known as «address constants», is used by the language for representation of the initial values assigned to statically-linked pointer variables. Intuitively, such expressions must represent function designators, l-values that designate a statically-linked object or else pointers to such l-values. They may be formed from almost arbitrary applications of the unary “"Address"”, “"Indirection"” operators, as well as the “\!"`DirectMember`"\!”, “\,"`IndirectMember`"”, array subscripts and pointer cast operations, provided that their operands also represent address constants. Formally, this semantic family is defined \hbox{in Haskell as follows:}

\BeginHaskellToken	{meaningOfAddressConstant}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{addr}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, TypeT, AtomT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfAddressConstant}			{D_ADDR expression}

\begin{code}
meaningOfAddressConstant {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, ExpressionT) -> _M{--}(S, S, D, TypeT, AtomT VarT)
\end{code}

In particular, such constants must be always formed from applications of the unary operator “"Address"” to a «constant l-value object», or else by an implicit conversion of a constant l-value of a function or an array type. Cast operations are also permitted, provided that they convert a well-formed address or null pointer constant into another pointer type. Formally:

\begin{code}
meaningOfAddressConstant {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (NestedExpression (e)))))))))))))))))) ---
  = meaningOfAddressConstant{--}(_S, _T, _D, _I {->-}, e)

meaningOfAddressConstant {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (Cast (type_name) {-!-} e {-,-})))))))))))))) = do
	(_S_t, _T_t, _D_t, t_t) <- meaningOfTypeName{--}(_S, _T, _D, _I {->-}, type_name)
	(_S_e, _T_e, _D_e, t_e, alpha) <- meaningOfAddressConstant{--}(_S_t, _T_t, _D_t, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e))))))))))))) --
  					   <|> meaningOfNullPointerConstant{--}(_S_t, _T_t, _D_t, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
	require $ (isPointerType{--}(t_t))
	return $ (_S_e, _T_e, _D_e, t_t, {-sem-}(OP1 (CNV {-sem-}(formatOfType{--}(t_t))) ({-sem-}(formatOfType{--}(t_e))) {--} {-hv-}(alpha)))

meaningOfAddressConstant {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (Address) (e)))))))))))))))) = do
	(_S_e, _T_e, _D_e, t_e, alpha) <- meaningOfConstantLValue{--}(_S,  _T,  _D,  _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
	require $ (not (isBitFieldType{--}(t_e)))
	return $ (_S_e, _T_e, _D_e, {-sem-}(PointerType (t_e) {-*-}), alpha)

meaningOfAddressConstant {-sem-}(_S, _T, _D, _I {->-}, e) = do
	(_S_e, _T_e, _D_e, t_e, alpha) <- meaningOfConstantLValue{--}(_S,  _T,  _D,  _I {->-}, e)
	require $ (isFunctionType{--}(t_e) || isArrayType{--}(t_e))
	return $ (_S_e, _T_e, _D_e, pointerPromotedType{--}(t_e), alpha)
\end{code}

Further, the set and denotations of «constant l-values», which, intuitively, represent objects with an external, internal or static linkage, are formalised by the following Haskell construction:

\BeginHaskellToken	{meaningOfConstantLValue}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{clv}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, ExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, TypeT, AtomT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfConstantLValue}			{D_CLV expression}

\begin{code}
meaningOfConstantLValue {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, ExpressionT) -> _M{--}(S, S, D, TypeT, AtomT VarT)
\end{code}

In particular, every string literal and every variable bound to a designator with an external, internal or static linkage form, as well as every parenthesised version of another valid l-value constant always belongs to this semantic family, as captured by the following three translation rules:

\begin{code}
meaningOfConstantLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (StringLiteral string_literal {-,-}))))))))))))))))) = do
	(t, delta_set {-,,-}) <- meaningOfStringLiteral{--}(string_literal)
	return $ (_S, _T, _D `mapUnion` listToMap [ {-raw-}(uvar{--}(_D) {-:-}, Just {-sem-}(OBJI (delta_set) {-OF-} ({-sem-}(envelopeOfType{--}(t))))) ], t, VAR (uvar{--}(_D)))

meaningOfConstantLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (Variable x))))))))))))))))) = do
	require	$ (x @@ dom{--}(_S) && isStaticLinkage{--}(linkageOfDesignator{--}(_S#(x))))
	return $ (_S, _T, _D, typeOfDesignator{--}(_S#(x)), VAR (variableOfLinkage{--}(linkageOfDesignator{--}(_S#(x)))))

meaningOfConstantLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (NestedExpression (e)))))))))))))))))) ---
  = meaningOfConstantLValue{--}(_S, _T, _D, _I {->-}, e)
\end{code}

Further, a constant l-value may represent a member of a constant structure, union or array object, obtained by a well-formed application of the direct member operator “\!"`DirectMember`"\!” to a constant l-value operand, an application of the indirect member operator \hbox{“\,"`IndirectMember`"”} to an address constant, or else from an application of the array subscript operator “\,"ArraySubscript () ()"\,” to an address constant and an integral constant expression:

\begin{code}
meaningOfConstantLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (e {-!-} `DirectMember` {-!-} x)))))))))))))))) = do
	(_S_e, _T_e, _D_e, t_e, alpha) <- meaningOfConstantLValue{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 e)))))))))))))))
	(m) <- {-hide-}return $ setOfMembers (membersOfType{--}(t_e))#(x)
	require $ (isCompoundType{--}(t_e) && x @@ dom{--}(setOfMembers (membersOfType{--}(t_e))))
	return $ (_S_e, _T_e, _D_e, tqOfType{--}(t_e) `requalifiedType` typeOfMember{--}(m), --
		  {-sem-}(OP2 (OP1 (CNV {-sem-}(objectFormat{--}(formatOfType{--}(typeOfMember{--}(m))))) (OG`FMT`standardEncoding) {--} {-hv-} (alpha)) --
						(ADD) (OG`FMT`standardEncoding) --
						(IMM (toRational {-sem-}(offsetOfMember{--}(m))) (ZG`FMT`standardEncoding))))

meaningOfConstantLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (e {-,-} `IndirectMember` x)))))))))))))))) = do
	(_S_e, _T_e, _D_e, t_e, alpha) <- meaningOfAddressConstant{--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 e)))))))))))))))
	(m) <- {-hide-}return $ setOfMembers (membersOfType{--}(baseOfType{--}(t_e)))#(x)
	require $ (isCompoundType{--}(baseOfType{--}(t_e)) && x @@ dom{--}(setOfMembers (membersOfType{--}(baseOfType{--}(t_e)))))
	return $ (_S_e, _T_e, _D_e,  tqOfType{--}(baseOfType{--}(t_e)) `requalifiedType` typeOfMember{--}(m), --
		  {-sem-}(OP2 (OP1 (CNV {-sem-}(objectFormat{--}(formatOfType{--}(typeOfMember{--}(m))))) (OG`FMT`standardEncoding) {--} {-hv-} (alpha)) --
						(ADD) (OG`FMT`standardEncoding) --
						(IMM (toRational {-sem-}(offsetOfMember{--}(m))) (ZG`FMT`standardEncoding))))
						
meaningOfConstantLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (ArraySubscript (e1) (e2)))))))))))))))))

	= 	 do	(_S1, _T1, _D1, t1, alpha	) <- meaningOfAddressConstant ---
								{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 e1)))))))))))))))
			(_S2, _T2, _D2, t2, n		) <- meaningOfIntegralConstantExpression ---
		  						{--}(_S1,  _T1,  _D1,  _I {->-}, e2)
			require $ (isObjectType{--}(baseOfType{--}(t1)))
			return $ (_S2, _T2, _D2, baseOfType{--}(t1), --
				  {-sem-}(OP2 (alpha) (ADD) ({-sem-}(formatOfType{--}(t1))) --
				  	(IMM (toRational {-sem-}(n * sizeOfType{--}(baseOfType{--}(t1)))) (ZG`FMT`standardEncoding))))

	<|>	 do	(_S1, _T1, _D1, t1, n		) <- meaningOfIntegralConstantExpression ---
			  					{--}(_S,   _T,   _D,   _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 e1)))))))))))))))
  			(_S2, _T2, _D2, t2, alpha	) <- meaningOfAddressConstant ---
		  						{--}(_S1,  _T1,  _D1,  _I {->-}, e2)
			require $ (isObjectType{--}(baseOfType{--}(t2)))
			return $ (_S2, _T2, _D2, baseOfType{--}(t2), --
				  {-sem-}(OP2 (alpha) (ADD) ({-sem-}(formatOfType{--}(t2))) --
				  	(IMM (toRational {-sem-}(n * sizeOfType{--}(baseOfType{--}(t2)))) (ZG`FMT`standardEncoding))))
\end{code}

Last but not least, constant l-values may be constructed by applying the unary “"Indirection"” operator to the value of a well-formed address constant:

\begin{code}
meaningOfConstantLValue {-sem-}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (Unary (Indirection) (e)))))))))))))))) ---
  =	do	(_S_e, _T_e, _D_e, t_e, alpha) <- meaningOfAddressConstant {--}(_S, _T, _D, _I {->-}, E1 (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 e)))))))))))))
		return $ (_S_e, _T_e, _D_e, baseOfType{--}(t_e), alpha)
meaningOfConstantLValue {-sem-}(_S, _T, _D, _I {->-}, other) ---
  =	fail ("Semantic error")
\end{code}

observing that, by definition, every such constant always represents a pointer to an l-value, whose content may be described either by a single Etude atom or else a sum of such an atom with an integer constant of the form “"IMM (n) (ZG`FMT`standardEncoding)"\!”, which makes it possible to implement all such constants under every relocatable program representation found in the industry.

\end{subsubsection}
\end{subsection}
\end{section}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%