\set{capture}{hidden}
\begin{code}

module Lambda.Arch (
	byteWidth,
	standardEncoding,
	objectFormat,
	widthOfFormat,
	sizeOfFormat,
	radixOfFormat,
	precOfFormat,
	minExpOfFormat,
	maxExpOfFormat,
	lubOfFormat,
	glbOfFormat,
	evalAtom,
	evalAtoms,
	decodeAtom,
	encodeAtom,
	plusZero,
	minusZero,
	plusInfinity,
	minusInfinity,
	plusNaN,
	minusNaN,
	isFiniteRational,
	isNonRational,
	envelopeOfOEnv,
	stackOfOEnv,
	atomOfOEnv,
	atomToObject,
	atomToFunction,
	newInOEnv,
	extendOEnv,
	contractOEnv,
	setInOEnv,
) where

import Additional
import Lambda.Syntax
import Lambda.Atoms
import Lambda.State
import Lambda.Terms
import Lambda.Modules

infixl 6 `setInOEnv`, `extendOEnv`, `contractOEnv`

\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{Lambda/Arch}
\set{title}		{Etude on MMIX}
\begin{section}

In order to utilise the above semantic translation as a realistic operational interpretation of MMIX programs, we must also refine the generic Etude language described in Chapter \ref{Lambda} for the specific requirements of the MMIX instruction set architecture. In particular, all implementation-specified language parameters, such as the precise ranges of numeric quantities representable under individual Etude formats, the behaviour of mathematically-undefined operations, or the precise implementation and structure of the program's address space, must be assigned appropriate concrete values, in order to align the earlier generic algebraic interpretation of Etude programs with the operational reality of the underlying computational hardware.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Lambda/Arch/Atoms}
\set{title}{Atoms and Their Formats}
\begin{subsection}

First of all, every Etude implementation must describe its atomic representation of all numeric quantities that are supported by the language. As described in Section \ref{Lambda/Atoms}, this representation rests crucially on a language parameter known as the byte width "byteWidth", which, on every 64-bit architectures such as MMIX, is always assigned the value of 8:

\NewHaskellToken	{byteWidth}				{Ord}	{«\omega»}
\NewHaskellResult	{byteWidth}					{Integer}
\IndexTerm		{byteWidth}					{\omega (integer)}

\begin{code}
byteWidth :: Integer
byteWidth = 8
\end{code}

Further, in this chapter, the encoding component of every Etude format is represented simply by an integer value, as described by the following type equations:

\IndexType		{EncodingT}					{encoding (\tstar)}

\begin{syntax}
\InsertHaskell		{EncodingT}
\end{syntax}

Intuitively, such encoding value specifies the number of bits required for an accurate depiction of all numeric quantities representable under a given well-formed Etude format. Accordingly, the encoding of every natural and integer format must always represent a sensible bit width in the range "1..64" for the object. On the other hand, formats from all other genres must be assigned the standard encoding value of 64, except that three additional object formats “"OG`FMT`63"”, “"OG`FMT`62"” and “"OG`FMT`61"” may be used to describe memory locations which are guaranteed to be aligned on 2, 4 or 8 byte boundaries, respectively. Formally, the set of all valid formats on the MMIX architecture is characterised by the following well-formedness property:

\BeginHaskellToken	{WfFormatJ}				{Ord}	{\TKeyword{wf}}
  \HaskellArgument							{FormatT}
  \HaskellResult							{Prop}
\EndHaskellToken

\NewHaskellToken	{WfNgC}					{Ord}	{\TKeyword{wf\sub{N}}}
\NewHaskellToken	{WfZgC}					{Ord}	{\TKeyword{wf\sub{Z}}}
\NewHaskellToken	{WfOgC}					{Ord}	{\TKeyword{wf\sub{O}}}
\NewHaskellToken	{WfRgC}					{Ord}	{\TKeyword{wf\sub{R}}}
\NewHaskellToken	{WfFgC}					{Ord}	{\TKeyword{wf\sub{F}}}

\NewHaskellDescription	{WfNgC}						{Validity of natural formats}
\NewHaskellDescription	{WfZgC}						{Validity of integer formats}
\NewHaskellDescription	{WfOgC}						{Validity of object formats}
\NewHaskellDescription	{WfRgC}						{Validity of rational formats}
\NewHaskellDescription	{WfFgC}						{Validity of function formats}

\IndexProp		{WfFormatJ}					{WF format (\pstar)}
\IndexAxiom		{WfNgC}						{WF_N Validity of natural formats}
\IndexAxiom		{WfZgC}						{WF_Z Validity of integer formats}
\IndexAxiom		{WfOgC}						{WF_O Validity of object formats}
\IndexAxiom		{WfRgC}						{WF_R Validity of rational formats}
\IndexAxiom		{WfFgC}						{WF_F Validity of function formats}

\begin{haskell}
data WfFormatJ {-[*]-} :: FormatT -> Prop

 where	WfNgC  :: forall epsilon =>  {-hfill-}{-sem-}(1   <= epsilon <= 64) -> WfJ{-sem-}(NG  `FMT`epsilon)
	WfZgC  :: forall epsilon =>  {-hfill-}{-sem-}(1   <= epsilon <= 64) -> WfJ{-sem-}(ZG  `FMT`epsilon)
	WfOgC  :: forall epsilon =>  {-hfill-}{-sem-}(61  <= epsilon <= 64) -> WfJ{-sem-}(OG  `FMT`epsilon)
	WfRgC  :: WfJ{-sem-}(RG  `FMT`64)
	WfFgC  :: WfJ{-sem-}(FG  `FMT`64)
\end{haskell}

In particular, the standard encoding "standardEncoding" is always assigned the value of 64:

\NewHaskellToken	{standardEncoding}			{Ord}	{\Phi}
\NewHaskellResult	{standardEncoding}				{EncodingT}
\IndexTerm		{standardEncoding}				{\Phi}

\begin{code}
standardEncoding :: EncodingT
standardEncoding = 64
\end{code}

\set{label}		{Lambda/Arch/wfStandardFormatJ}
\set{title}		{Validity of standard formats}
\begin{theorem}

Every standard Etude format is well-formed under the above definition of "WfJ" and "standardEncoding":

\NewHaskellToken	{wfStandardFormatJ}			{Ord}	{\TKeyword{wf\sub{\Phi}}}
\NewHaskellDescription	{wfStandardFormatJ}				{Validity of standard formats}
\IndexTheorem		{wfStandardFormatJ}				{WF_\Phi Validity of standard formats}

\begin{haskell}
wfStandardFormatJ 	:: forall gamma 	=> WfJ{--}(gamma) -> WfJ{-sem-}(gamma`FMT`standardEncoding)
\end{haskell}
\begin{proof}

Trivial, by case inspection of the “"WfJ :: FormatT -> Prop"” predicate definition:

\begin{haskell}
wfStandardFormatJ {-sem-}(_P :: WfJ{-sem-}({-hfil-}NG  )) = WfNgC  (defn :: {-hfill-}{-sem-}(1   <= standardEncoding <= 64))
wfStandardFormatJ {-sem-}(_P :: WfJ{-sem-}({-hfil-}ZG  )) = WfZgC  (defn :: {-hfill-}{-sem-}(1   <= standardEncoding <= 64))
wfStandardFormatJ {-sem-}(_P :: WfJ{-sem-}({-hfil-}OG  )) = WfOgC  (defn :: {-hfill-}{-sem-}(61  <= standardEncoding <= 64))
wfStandardFormatJ {-sem-}(_P :: WfJ{-sem-}({-hfil-}RG  )) = WfRgC
wfStandardFormatJ {-sem-}(_P :: WfJ{-sem-}({-hfil-}FG  )) = WfFgC 	{-QED-}
\end{haskell}
\end{proof}
\end{theorem}

\set{label}		{Lambda/Arch/wfIntegralFormatJ}
\set{title}		{Validity of integral formats}
\begin{theorem}

For every valid integral format “"ZG`FMT`epsilon"”, its natural counterpart “"NG`FMT`epsilon"” is likewise well-formed:

\NewHaskellToken	{wfIntegralFormatJ}			{Ord}	{\TKeyword{wf\sub{i}}}
\NewHaskellDescription	{wfIntegralFormatJ}				{Validity of integral formats}
\IndexTheorem		{wfIntegralFormatJ}				{WF_I Validity of integral formats}

\begin{haskell}
wfIntegralFormatJ	:: forall epsilon	=> WfJ{-sem-}(ZG`FMT`epsilon) -> WfJ{-sem-}(NG`FMT`epsilon)
\end{haskell}
\begin{proof}

Once again, this theorem follows naturally from the earlier definition of the format well-formedness axioms "WfZgC" and "WfNgC":

\begin{haskell}
wfIntegralFormatJ {-sem-}(WfZgC _Q) = WfNgC _Q {-QED-}
\end{haskell}
\end{proof}
\end{theorem}

\noindent For conciseness, in the remainder of this section most of such trivially justified theorems are presented as simple corollaries, leaving a precise formulation of the required proof terms as an exercise to a keen reader.

As described earlier in Section \ref{Lambda/Terms}, every well-formed Etude format "phi" is associated with some predetermined object format "objectFormat{--}(phi)" that, intuitively, can be used to describe locations of "phi"-formatted objects in the program's address space. On MMIX, these formats are always given the encoding of \hbox{"64 - floorLogBase2((sizeOfFormat{--}(phi)))",} in which "sizeOfFormat{--}(phi)" depicts the byte size of the referenced objects, as described later in this section:

\BeginHaskellToken	{objectFormat}				{Open}	{\ZapfChancery{O}}
  \HaskellArgument							{FormatT}
  \HaskellResult							{FormatT}
\EndHaskellToken

\IndexTerm		{objectFormat}					{O format (format)}

\begin{code}
objectFormat {-[*]-} :: FormatT -> FormatT

objectFormat {-sem-}(phi) = {-sem-}(OG`FMT`{-sem-}(64 - floorLogBase2((toRational (sizeOfFormat{--}(phi))))))
\end{code}

Further, the «width» of every well-formed Etude format "phi", depicted in Chapter \ref{Lambda} by the notation “"widthOfFormat{--}(phi)"”, is always equal to the format's encoding value "epsilon":

\BeginHaskellToken	{widthOfFormat}				{Open}	{\ZapfChancery{W}}
  \HaskellArgument							{FormatT}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{widthOfFormat}					{W format (integer)}

\begin{code}
widthOfFormat {-[*]-} :: FormatT -> Integer

widthOfFormat {-sem-}(phi) = encodingOfFormat{--}(phi)
\end{code}

On the other hand, the size of a valid format is equal to the least integer \hbox{"k @@ listToSet [1, 2, 4, 8]"}, such that "8{--}k >= widthOfFormat{--}(phi)". Mathematically, this integer can be obtained by the following concise formula:

\BeginHaskellToken	{sizeOfFormat}				{Open}	{\ZapfChancery{S}}
  \HaskellArgument							{FormatT}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{sizeOfFormat}					{S format (integer)}

\begin{code}
sizeOfFormat {-[*]-} :: FormatT -> Integer

sizeOfFormat {-sem-}(phi) = 2^(floorLogBase2(toRational (floor ((widthOfFormat{--}(phi) + 7) % 8))))
\end{code}

\set{label}		{Lambda/Arch/sizeOfValidFormatJ}
\set{title}		{Geometry of well-formed formats}
\set{style}		{corollary}
\begin{theorem}

Every well-formed Etude format has a positive size and width, with the later no greater than the product of its size and the byte width parameter "byteWidth". Further, the size of every integer format “"ZG`FMT`epsilon"” is equal to that of the corresponding natural format “"NG`FMT`epsilon"”. Formally:

\NewHaskellToken	{sizeOfValidFormatJ}			{Ord}	{\TKeyword{size\sub{\phi}}}
\NewHaskellToken	{widthOfValidFormatJ}			{Ord}	{\TKeyword{width\sub{\phi}}}
\NewHaskellToken	{sizeOfIntegerFormatJ}			{Ord}	{\TKeyword{size\sub{z}}}

\NewHaskellDescription	{sizeOfValidFormatJ}				{Sizes of well-formed formats}
\NewHaskellDescription	{widthOfValidFormatJ}				{Widths of well-formed formats}
\NewHaskellDescription	{sizeOfIntegerFormatJ}				{Sizes of integral formats}

\IndexTheorem		{sizeOfValidFormatJ}				{SIZE_\phi Sizes of well-formed formats}
\IndexTheorem		{widthOfValidFormatJ}				{WIDTH_\phi Widths of well-formed formats}
\IndexTheorem		{sizeOfIntegerFormatJ}				{SIZE_Z\epsilon Sizes of integral formats}

\begin{haskell}
sizeOfValidFormatJ 	:: forall phi 		=> WfJ{--}(phi) 		-> {-sem-}(sizeOfFormat{--}(phi) > 0)
widthOfValidFormatJ 	:: forall phi 		=> WfJ{--}(phi) 		-> {-sem-}(0 < widthOfFormat{--}(phi) <= byteWidth * sizeOfFormat{--}(phi))
sizeOfIntegerFormatJ 	:: forall epsilon 	=> WfJ{-sem-}(ZG`FMT`epsilon)	-> {-sem-}(sizeOfFormat{-sem-}(ZG`FMT`epsilon) == sizeOfFormat{-sem-}(NG`FMT`epsilon))
\end{haskell}
\end{theorem}

\LocalHaskellToken	{\_min\_}				{Ord}	{«min»}
\LocalHaskellToken	{\_max\_}				{Ord}	{«max»}

\noindent The MMIX architecture supports a full range of double precision floating point arithmetic defined in IEEE 754 Standard for Binary Floating Point Arithmetic [IEEE754]. Accordingly, the four floating point parameters "r", "p", "_E{-sub-}_min_" and "_E{-sub-}_max_" assume the following values for the 64-bit rational format “"RG`FMT`64"”:

\BeginHaskellToken	{radixOfFormat}				{Open}	{r}
  \HaskellArgument							{FormatT}
  \HaskellResult							{Integer}
\EndHaskellToken

\BeginHaskellToken	{precOfFormat}				{Open}	{p}
  \HaskellArgument							{FormatT}
  \HaskellResult							{Integer}
\EndHaskellToken

\BeginHaskellToken	{minExpOfFormat}			{Open}	{E\sub{min}}
  \HaskellArgument							{FormatT}
  \HaskellResult							{Integer}
\EndHaskellToken

\BeginHaskellToken	{maxExpOfFormat}			{Open}	{E\sub{max}}
  \HaskellArgument							{FormatT}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{radixOfFormat}					{r format (integer)}
\IndexTerm		{precOfFormat}					{p format (integer)}
\IndexTerm		{minExpOfFormat}				{Emin format (integer)}
\IndexTerm		{maxExpOfFormat}				{Emax format (integer)}

\begin{code}
radixOfFormat {-[*]-}, precOfFormat {-[*]-}, minExpOfFormat {-[*]-}, maxExpOfFormat {-[*]-} :: FormatT -> Integer

radixOfFormat	{-sem-}(RG`FMT`64) =  {-hfill-} 2		{--}
precOfFormat 	{-sem-}(RG`FMT`64) =  {-hfill-} 53		{--}
minExpOfFormat 	{-sem-}(RG`FMT`64) =  {-hfill-} negate 1021	{--}
maxExpOfFormat 	{-sem-}(RG`FMT`64) =  {-hfill-} 1024		{--}
\end{code}

\set{label}		{Lambda/Arch/radixOfFormatJ}
\set{title}		{Minimal floating point requirements}
\set{style}		{corollary}
\begin{theorem}

\NewHaskellToken	{radixOfFormatJ}			{Ord}	{\TKeyword{radix}}
\NewHaskellToken	{precOfFormatJ}				{Ord}	{\TKeyword{prec}}
\NewHaskellToken	{minExpOfFormatJ}			{Ord}	{\TKeyword{emin}}
\NewHaskellToken	{maxExpOfFormatJ}			{Ord}	{\TKeyword{emax}}

\NewHaskellDescription	{radixOfFormatJ}				{Radix of well-formed rational formats}
\NewHaskellDescription	{precOfFormatJ}					{Precision of well-formed rational formats}
\NewHaskellDescription	{minExpOfFormatJ}				{Minimum exponent of well-formed rational formats}
\NewHaskellDescription	{maxExpOfFormatJ}				{Maximum exponent of well-formed rational formats}

\IndexTheorem		{radixOfFormatJ}				{RADIX Radix of well-formed rational formats}
\IndexTheorem		{precOfFormatJ}					{PREC Precision of well-formed rational formats}
\IndexTheorem		{minExpOfFormatJ}				{EMIN Minimum exponent of well-formed rational formats}
\IndexTheorem		{maxExpOfFormatJ}				{EMAX Maximum exponent of well-formed rational formats}

\begin{haskell}
radixOfFormatJ		:: forall epsilon => WfJ{-sem-}(RG`FMT`epsilon) -> {-sem-}(radixOfFormat	{-sem-}(RG`FMT`epsilon) > 1)
precOfFormatJ		:: forall epsilon => WfJ{-sem-}(RG`FMT`epsilon) -> {-sem-}(precOfFormat		{-sem-}(RG`FMT`epsilon) > 0)
minExpOfFormatJ		:: forall epsilon => WfJ{-sem-}(RG`FMT`epsilon) -> {-sem-}(minExpOfFormat	 {-sem-}(RG`FMT`epsilon) < 0)
maxExpOfFormatJ		:: forall epsilon => WfJ{-sem-}(RG`FMT`epsilon) -> {-sem-}(maxExpOfFormat	 {-sem-}(RG`FMT`epsilon) > 0)
\end{haskell}
\end{theorem}

\noindent As described in Section \ref{Lambda/Atoms}, the greatest finite value representable under a given well-formed Etude format "phi" is known as that format's «least upper bound», depicted by the notation “"lubOfFormat{--}(phi)"”.  For natural formats, this bound is always set to "2^(widthOfFormat{--}(phi)) - 1". On the other hand, the "2^(widthOfFormat{--}(phi))" possible bit patterns found in integer and pointer values are always interpreted under the «two's complement encoding», so that the greatest representable number has the value of "2^(widthOfFormat{--}(phi) - 1) - 1". Finally, in accordance with the IEEE Standard 754, the least upper bound of every rational format is equal to "(1 - radixOfFormat{--}(phi)^^(negate (precOfFormat{--}(phi)))) * radixOfFormat{--}(phi)^(maxExpOfFormat{--}(phi))". Formally, these three rules are captured by the following Haskell definition:

\BeginHaskellToken	{lubOfFormat}				{Open}	{lub}
  \HaskellArgument							{FormatT}
  \HaskellResult							{Rational}
\EndHaskellToken

\IndexTerm		{lubOfFormat}					{lub format (rational)}

\begin{code}
lubOfFormat {-[*]-} :: FormatT -> Rational

lubOfFormat {-sem-}(NG  `FMT`epsilon)	= 2^(widthOfFormat{-sem-}(NG`FMT`epsilon)) - 1
lubOfFormat {-sem-}(ZG  `FMT`epsilon)	= 2^(widthOfFormat{-sem-}(ZG`FMT`epsilon) - 1) - 1
lubOfFormat {-sem-}(RG  `FMT`epsilon)	= (1 - toRational (radixOfFormat{-sem-}(RG`FMT`epsilon))^^(negate (precOfFormat{-sem-}(RG`FMT`epsilon)))) --
			  			* toRational (radixOfFormat{-sem-}(RG`FMT`epsilon))^(maxExpOfFormat{-sem-}(RG`FMT`epsilon))
lubOfFormat {-sem-}(FG  `FMT`epsilon)	= 2^63 - 1
lubOfFormat {-sem-}(OG  `FMT`epsilon)	= 2^63 - 1
\end{code}

Further, the format's «greatest lower bound», or the least finite quantity representable under it, is always set to 0 for natural formats. For signed integer quantities and pointer address values, it is always set to the least value "negate 2^(widthOfFormat{--}(phi) - 1)" representable under the corresponding two's complement encoding within "widthOfFormat{--}(phi)" bits of information, while, for all well-formed rational formats, it is equal precisely to the negation of the corresponding least upper bound. \hbox{In Haskell:}

\BeginHaskellToken	{glbOfFormat}				{Open}	{glb}
  \HaskellArgument							{FormatT}
  \HaskellResult							{Rational}
\EndHaskellToken

\IndexTerm		{glbOfFormat}					{glb format (rational)}

\begin{code}
glbOfFormat {-[*]-} :: FormatT -> Rational

glbOfFormat {-sem-}(NG  `FMT`epsilon)	= 0
glbOfFormat {-sem-}(ZG  `FMT`epsilon)	= negate (2^(widthOfFormat{-sem-}(ZG`FMT`epsilon) - 1))
glbOfFormat {-sem-}(RG  `FMT`epsilon)	= negate (lubOfFormat{-sem-}(RG`FMT`epsilon))
glbOfFormat {-sem-}(FG  `FMT`epsilon)	= negate (2^63)
glbOfFormat {-sem-}(OG  `FMT`epsilon)	= negate (2^63)
\end{code}

\set{label}		{Lambda/Arch/boundsOfValidFormatJ}
\set{title}		{Minimal format bounds requirements}
\set{style}		{corollary}
\begin{theorem}

The bounds of all well-formed arithmetic formats satisfy the following minimal requirements:

\NewHaskellToken	{glbOfValidFormatJ}			{Ord}	{\TKeyword{glb\sub{\phi}}}
\NewHaskellToken	{glbOfNaturalFormatJ}			{Ord}	{\TKeyword{glb\sub{n}}}
\NewHaskellToken	{glbOfIntegerFormatJ}			{Ord}	{\TKeyword{glb\sub{z}}}
\NewHaskellToken	{glbOfRationalFormatJ}			{Ord}	{\TKeyword{glb\sub{r}}}

\NewHaskellToken	{lubOfValidFormatJ}			{Ord}	{\TKeyword{lub\sub{\phi}}}
\NewHaskellToken	{lubOfNaturalFormatJ}			{Ord}	{\TKeyword{lub\sub{n}}}
\NewHaskellToken	{lubOfIntegerFormatJ}			{Ord}	{\TKeyword{lub\sub{z}}}
\NewHaskellToken	{lubOfRationalFormatJ}			{Ord}	{\TKeyword{lub\sub{r}}}

\NewHaskellDescription	{glbOfValidFormatJ}				{Greatest lower bound of well-formed formats}
\NewHaskellDescription	{glbOfNaturalFormatJ}				{Greatest lower bound of natural formats}
\NewHaskellDescription	{glbOfIntegerFormatJ}				{Greatest lower bound of integer formats}
\NewHaskellDescription	{glbOfRationalFormatJ}				{Greatest lower bound of rational formats}

\NewHaskellDescription	{lubOfValidFormatJ}				{Least upper bound of well-formed formats}
\NewHaskellDescription	{lubOfNaturalFormatJ}				{Least upper bound of natural formats}
\NewHaskellDescription	{lubOfIntegerFormatJ}				{Least upper bound of integer formats}
\NewHaskellDescription	{lubOfRationalFormatJ}				{Least upper bound of rational formats}

\IndexTheorem		{glbOfValidFormatJ}				{GLB_\phi	Greatest lower bound of well-formed formats}
\IndexTheorem		{glbOfNaturalFormatJ}				{GLB_N		Greatest lower bound of natural formats}
\IndexTheorem		{glbOfIntegerFormatJ}				{GLB_Z		Greatest lower bound of integer formats}
\IndexTheorem		{glbOfRationalFormatJ}				{GLB_R		Greatest lower bound of rational formats}

\IndexTheorem		{lubOfValidFormatJ}				{LUB_\phi	Least upper bound of well-formed formats}
\IndexTheorem		{lubOfNaturalFormatJ}				{LUB_N		Least upper bound of natural formats}
\IndexTheorem		{lubOfIntegerFormatJ}				{LUB_Z		Least upper bound of integer formats}
\IndexTheorem		{lubOfRationalFormatJ}				{LUB_R		Least upper bound of rational formats}

\begin{haskell}
glbOfValidFormatJ 	:: forall phi 		=> WfJ{--}(phi) 		-> {-sem-}(glbOfFormat	{--}(phi) 			<= 0)
glbOfNaturalFormatJ 	:: forall epsilon 	=> WfJ{-sem-}(NG`FMT`epsilon)	-> {-sem-}(glbOfFormat	{-sem-}(NG`FMT`epsilon)		== 0)
glbOfIntegerFormatJ 	:: forall epsilon 	=> WfJ{-sem-}(ZG`FMT`epsilon)	-> {-sem-}(glbOfFormat	{-sem-}(ZG`FMT`epsilon) 	<= negate (lubOfFormat{-sem-}(ZG`FMT`epsilon)))
glbOfRationalFormatJ	:: forall epsilon 	=> WfJ{-sem-}(RG`FMT`epsilon) 	-> {-sem-}(glbOfFormat  {-sem-}(RG`FMT`epsilon) 	== --
	negate (radixOfFormat{-sem-}(RG`FMT`epsilon)^(precOfFormat{-sem-}(RG`FMT`epsilon)) - 1) --
		* radixOfFormat{-sem-}(RG`FMT`epsilon)^(maxExpOfFormat{-sem-}(RG`FMT`epsilon) - precOfFormat{-sem-}(RG`FMT`epsilon)))

lubOfValidFormatJ 	:: forall phi 		=> WfJ{--}(phi) 		-> {-sem-}(lubOfFormat	{--}(phi) 			> 0)
lubOfNaturalFormatJ 	:: forall epsilon 	=> WfJ{-sem-}(NG`FMT`epsilon)	-> {-sem-}(lubOfFormat	{-sem-}(NG`FMT`epsilon) 	== 2^(widthOfFormat{-sem-}(NG`FMT`epsilon)) - 1)
lubOfIntegerFormatJ 	:: forall epsilon 	=> WfJ{-sem-}(ZG`FMT`epsilon)	-> {-sem-}(lubOfFormat	{-sem-}(ZG`FMT`epsilon) 	< lubOfFormat{-sem-}(NG`FMT`epsilon))
lubOfRationalFormatJ	:: forall epsilon 	=> WfJ{-sem-}(RG`FMT`epsilon) 	-> {-sem-}(lubOfFormat  {-sem-}(RG`FMT`epsilon) 	== negate (glbOfFormat{-sem-}(RG`FMT`epsilon)))
\end{haskell}
\end{theorem}

\noindent Using the above definitions, we can now capture the precise well-formedness property applicable to all Etude atoms under the MMIX architecture. Perhaps surprisingly, this can be accomplished by only a pair of simple axioms "WfImmC" and "wfEqvAtomJ", The relevant Haskell property definition assumes the following form:

\BeginHaskellToken	{WfAtomJ}				{Ord}	{\TKeyword{wf}}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{AtomT nu}
  \HaskellResult							{Prop}
\EndHaskellToken

\NewHaskellToken	{WfImmC}				{Ord}	{\TKeyword{wf\sub{nc}}}
\NewHaskellToken	{wfEqvAtomJ}				{Ord}	{\TKeyword{wf\sub{\equiv}}}

\NewHaskellDescription	{WfImmC}					{Validity of constant atoms}
\NewHaskellDescription	{wfEqvAtomJ}					{Validity of equivalent atoms}

\IndexProp		{WfAtomJ}					{WF atom (\pstar)}
\IndexAxiom		{WfImmC}					{WF_NC Validity of constant atoms}
\IndexAxiom		{wfEqvAtomJ}					{WF_\alpha Validity of equivalent atoms}

\begin{haskell}
data WfAtomJ {-[*]-} :: (Ord nu) => AtomT nu -> Prop

 where	WfImmC      {-[*]-} 		:: forall x :: Integer => {-sem-}(0 <= x < 2^64) -> WfJ{-sem-}(IMM (x) (NG`FMT`64))
 	wfEqvAtomJ  {-[*]-} {-[*]-}	:: forall alpha1, alpha2 => WfJ{--}(alpha1) -> (alpha1 `EqvJ` alpha2) -> WfJ{--}(alpha2)
\end{haskell}

In other words, a constant natural atom of the form “"IMM (x) (phi)"\!” is always guaranteed to be well-formed if "x" depicts a non-negative integer representable in 64 bits of pure binary representation, i.e., one with a value less than "2^64". Otherwise, "wfEqvAtomJ" guarantees the validity of all elements found in a given equivalence class of Etude atoms, whenever at least one of these elements can be shown to be well-formed. Formally, the equivalence relation “"alpha1 `EqvJ` alpha2"\!” is defined as a definitional equality of the two atoms' respective normal forms, obtained from their reduction by the evaluation function "evalAtom". In particular:

\BeginHaskellToken	{EqvAtomJ}				{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{AtomT nu}
  \HaskellToken		{`EqvJ`}				{Rel}	{\equiv}
  \HaskellArgument							{AtomT nu}
  \HaskellResult							{Prop}
\EndHaskellToken

\NewHaskellToken	{EqvAtomC}				{Ord}	{\TKeyword{eqv\sub{\alpha}}}
\NewHaskellDescription	{EqvAtomC}					{Equivalence of atoms}

\IndexProp		{EqvAtomJ}					{\equiv atom atom (\pstar)}
\IndexAxiom		{EqvAtomC}					{EQV_\alpha Equivalence of atoms}

\begin{haskell}
data {-[*]-} `EqvJ` {-[*]-} :: (Ord nu) => AtomT nu -> AtomT nu -> Prop

 where	EqvAtomC {-[*]-} :: forall alpha1, alpha2 => {-sem-}(evalAtom{--}(alpha1) = evalAtom{--}(alpha2)) -> (alpha1 `EqvJ` alpha2)
\end{haskell}

\set{label}		{Lambda/Arch/eqvAtomJ}
\set{title}		{Atom equivalence}
\begin{theorem}

The “"`EqvJ`"” property constitutes a reflexive, symmetric and transitive relation over Etude atoms:

\NewHaskellToken	{eqvReflAtomJ}				{Ord}	{\TKeyword{refl\sub{\alpha}}}
\NewHaskellToken	{eqvSymmAtomJ}				{Ord}	{\TKeyword{symm\sub{\alpha}}}
\NewHaskellToken	{eqvTransAtomJ}				{Ord}	{\TKeyword{trans\sub{\alpha}}}

\NewHaskellDescription	{eqvReflAtomJ}					{Reflexivity of atom equivalence}
\NewHaskellDescription	{eqvSymmAtomJ}					{Symmetry of atom equivalence}
\NewHaskellDescription	{eqvTransAtomJ}					{Transitivity of atom equivalence}

\IndexTheorem		{eqvReflAtomJ}					{REFL_\alpha Reflexivity of atom equivalence}
\IndexTheorem		{eqvSymmAtomJ}					{SYMM_\alpha Symmetry of atom equivalence}
\IndexTheorem		{eqvTransAtomJ}					{TRANS_\alpha Transitivity of atom equivalence}

\begin{haskell}
eqvReflAtomJ	:: forall alpha -> (alpha `EqvJ` alpha)
eqvSymmAtomJ	:: forall alpha1, alpha2 => (alpha1 `EqvJ` alpha2) -> (alpha2 `EqvJ` alpha1)
eqvTransAtomJ	:: forall alpha1, alpha2, alpha3 => (alpha1 `EqvJ` alpha2) -> (alpha2 `EqvJ` alpha3) -> (alpha1 `EqvJ` alpha3)
\end{haskell}
\begin{proof}

All three of the above theorems are justified directly by the corresponding properties of Haskell's definitional equality “"="”:

\begin{haskell}
eqvReflAtomJ	{-sem-}(alpha)					= EqvAtomC (refl {-sem-}(evalAtom{--}(alpha)))
eqvSymmAtomJ	{-sem-}(EqvAtomC _P)				= EqvAtomC (symm _P)
eqvTransAtomJ	{-sem-}(EqvAtomC _P) {-sem-}(EqvAtomC _Q)	= EqvAtomC (trans _P _Q) 	{-QED-}
\end{haskell}
\end{proof}
\end{theorem}

\set{label}		{Lambda/Arch/eqvEvalAtomJ}
\set{title}		{Equivalence of identical atoms}
\begin{theorem}

All well formed pairs of structurally identical atoms are equivalent:

\NewHaskellToken	{eqvEvalAtomJ}				{Ord}	{\TKeyword{eqv\sub{\ZapfChancery{E}}}}
\NewHaskellDescription	{eqvEvalAtomJ}					{Equivalence of identical atoms}
\IndexTheorem		{eqvEvalAtomJ}					{EQV_E Equivalence of identical atoms}

\begin{haskell}
eqvEvalAtomJ 	:: forall alpha1, alpha2 => WfJ{--}(alpha1) -> WfJ{--}(alpha2) -> {-sem-}(evalAtom{--}(alpha1) == evalAtom{--}(alpha2)) -> (alpha1 `EqvJ` alpha2)
\end{haskell}
\begin{proof}

By definition of the automatically derived structural identity relation “"=="”, we have "{-sem-}(alpha1' == alpha2') -> {-sem-}(alpha1' = alpha2')" for all atomic expressions "alpha1'" and "alpha2'". Accordingly, we can establish the required equivalence by a simple application of that standard result to the assumed equality proof of "evalAtom{--}(alpha1) == evalAtom{--}(alpha2)" in the following manner:

\begin{haskell}
eqvEvalAtomJ {-sem-}(_P :: WfJ{--}(alpha1)) {-sem-}(_Q :: WfJ{--}(alpha2)) {-sem-}(_R :: {-sem-}(evalAtom{--}(alpha1) == evalAtom{--}(alpha2)))	= EqvAtomC (lemma _R)

 where	lemma1 :: forall alpha1', alpha2' => {-sem-}(alpha1' == alpha2') -> {-sem-}(alpha1' = alpha2') = triv {-QED-}
\end{haskell}
\end{proof}
\end{theorem}

\noindent In this work, "evalAtom" represents the operational semantics of all closed and well-formed Etude atoms under the MMIX architecture. By association, therefore, it also defines the precise operational behaviours of any C and MMIX programs whose meanings have been described through a translation into the present Etude variant.

A reader should observe that, according to the following definition of "evalAtom", an operational semantics of Etude atoms represents a partial Haskell function, undefined for open and otherwise meaningless syntactic entities. Accordingly, by theorem "EqvAtomC", all such illegal constructs are grouped into a single equivalence class, since, in Haskell, the definitional equality always holds between a pair of undefined values “\!"undefined"\!”.

When defined, however, the value produced by "evalAtom" will always represent some well-formed constant atom “"IMM (x) (NG`FMT`64)"\!”, i.e., one in which the rational number "x" depicts a non-negative integer quantity less than "2^64" in magnitude. Intuitively, this implies that all MMIX programs are geared solely towards manipulation of 64-bit unsigned integer values, whose binary representation may, however be interpreted variously as a signed integer "z" in the range "negate 2^63 <= z < 2^63", or even a rational number, with the later interpretation relying on the somewhat esoteric encoding of the IEEE 754 double precision values. Accordingly, on MMIX, every well-formed Etude atom “"IMM (x) (phi)"\!” is always equivalent to some other 64-bit natural number with a binary representation identical to that of "x" under the rules of the format "phi".

Formally, the binary encoding of a given numeric quantity "x" under a well-formed Etude format "phi" is depicted by the notation “"encodeAtom(phi){--}(x)"”. In particular, if "phi" represents a valid natural or integer format, then the encoding of "x" is always equal to its integral portion "truncate (x)" after its reduction modulo "2^(widthOfFormat{--}(phi))", except that, if "phi" belongs to the integer genre “"ZG"” and "truncate(x) `mod` 2^(widthOfFormat{--}(phi)) >= 2^(widthOfFormat{--}(phi) - 1)", then "2^64 - 2^(widthOfFormat{--}(phi))" is added to its value, effectively copying the integer's sign bit into all otherwise unoccupied bit positions in the resulting 64-bit natural number. On the other hand, the encoding of "x" under a format from the object or function genre represents the value of "truncate (x) `mod` 2^64 - truncate (x) `mod` 2^k", where "k" is equal to the number of «alignment bits» specified by the format, defined as "64 - encodingOfFormat{--}(phi)" and "2" for object and functional formats, respectively:

\BeginHaskellToken	{encodeAtom}				{Open}	{enc}
  \HaskellContext							{(Ord nu)}
  \HaskellModifier							{\sub}
  \HaskellArgument							{FormatT}
  \HaskellArgument							{Rational}
  \HaskellResult							{AtomT nu}
\EndHaskellToken

\IndexTerm		{encodeAtom}					{enc format rational (atom)}

\begin{code}
encodeAtom {-[*]-} {-[*]-} :: {- (Ord nu) => -} FormatT -> Rational -> AtomT nu

encodeAtom ({-sem-}(NG`FMT`epsilon)) 	{-sem-}(x) ---
  | inRange ({-hfill-}1) 	(<=) (epsilon) (<=) (64) {-ord-} ---
  	= {-sem-}(IMM (toRational {-sem-}(truncate (x) `mod` 2^epsilon)) (NG`FMT`64))
encodeAtom ({-sem-}(ZG`FMT`epsilon)) 	{-sem-}(x) ---
  | inRange ({-hfill-}1) 	(<=) (epsilon) (<=) (64) ---
  	= {-sem-}(IMM (toRational {-sem-}(truncate (x) `mod` 2^epsilon + (2^64 - 2^epsilon) * truncate (x) `bits` [epsilon - 1])) (NG`FMT`64))
encodeAtom ({-sem-}(OG`FMT`epsilon)) 	{-sem-}(x) ---
  | inRange ({-hfill-}61) 	(<=) (epsilon) (<=) (64) ---
  	= {-sem-}(IMM (toRational {-sem-}(truncate (x) `mod` 2^64 - truncate (x) `mod` 2^(64 - epsilon))) (NG`FMT`64))
encodeAtom ({-sem-}(FG`FMT`64)) 	{-sem-}(x) ---
	= {-sem-}(IMM (toRational {-sem-}(truncate (x) `mod` 2^64 - truncate (x) `mod` 2^2)) (NG`FMT`64))
\end{code}

\set{capture}		{encodeAtom/Rational}
\begin{code}
encodeAtom ({-sem-}(RG`FMT`64)) {-sem-}(x) ---
  | x == 0				= {-sem-}(IMM (toRational {-sem-}(0)) (NG`FMT`64))
  | e <= 0				= {-sem-}(IMM (toRational {-sem-}(2^63 #*# {--} s + round{--}(2^^(bias + prec - 1) #*# {--} abs(x)))) (NG`FMT`64))
  | e <= 2 #*# {--} bias		= {-sem-}(IMM (toRational {-sem-}(2^63 #*# {--} s + 2^prec #*# {--} e + round{--}(2^^(bias + prec - e) #*# {--} abs(x)) - 2^prec)) (NG`FMT`64))
  | e > 2 #*# {--} bias	{-ord-}		= {-sem-}(IMM (toRational {-sem-}(2^63 #*# {--} s + 2^prec #*# (2 #*# {--} bias + 1))) (NG`FMT`64))

 where	s  = fromBool (x < 0)
 	e  = floorLogBase2(abs(x)) + bias
\end{code}

The corresponding binary encoding of rational constants “"IMM (x) (RG`FMT`64)"\!” is somewhat more elaborate. In general, a precise representation of such constants relies on two additional parameters "prec" and "bias", whose values are one less than those of the corresponding floating point parameters "precOfFormat{-sem-}(RG`FMT`64)" and "maxExpOfFormat{-sem-}(RG`FMT`64)", respectively:

\NewHaskellToken	{prec}					{Ord}	{«p»}
\NewHaskellToken	{bias}					{Ord}	{«b»}

\NewHaskellResult	{prec}						{Integer}
\NewHaskellResult	{bias}						{Integer}

\IndexTerm		{prec}						{p (integer)}
\IndexTerm		{bias}						{b (integer)}

\begin{code}
prec, bias :: Integer

prec  = precOfFormat{-sem-}(RG`FMT`64) - 1
bias  = maxExpOfFormat{-sem-}(RG`FMT`64) - 1
\end{code}

The "bias" parameter is often referred to as the «bias» of a floating point format. Using these parameters, every well-formed rational constant can be encoded as a triple of three integers "s", "e" and "m" known as the «sign», «exponent» and «mantissa», respectively, such that "0 <= s <= 1", "0 <= e <= 2 #*# bias + 1" and "0 <= m < 2^prec". If "x == 0", then all three components are set to 0, so that “"IMM (0) (RG`FMT`64)"\!” is always encoded directly as “"IMM (0) (NG`FMT`64)"\!”. Otherwise, the 64-bit encoding "n" of the rational number "x" is equal to the sum "2^63#*#s + 2^prec#*#e + m", so that the sign "s" is stored in the most-significant bit of "n", the mantissa "m" is placed in the least-significant "p" bits of the resulting number and the exponent is slipped into the remaining 11 bits between "s" and "m". In all cases, the sign bit "s" is assigned the value of 1 if and only if "x < 0" and remains clear for all non-negative rational numbers. Further, the exponent "e" is generally equal to "floorLogBase2(abs(x)) + bias", clipped into the required range "0 <= e <= 2 {--} bias + 1". If "negate bias < floorLogBase2(abs(x)) <= bias", then "x" is said to represent a «normalised floating point number» and the mantissa "m" is set to "round{--}(2^^(bias + prec - e) #*# {--} abs(x)) - 2^prec". Otherwise, if "floorLogBase2(abs(x)) <= negate bias", then "x" is said to be «denormalised», resulting in the mantissa of "round{--}(2^^(bias + prec - 1) #*# {--} abs(x))". Finally, all values of "x" for which "floorLogBase2(abs(x)) > bias" are semantically equivalent to "plusMinusInfinity" and are encoded with "m == 0". Formally:

\InsertHaskell		{encodeAtom/Rational}

In the following definition of the operational semantics "evalAtom", it is often necessary to reverse the above process, in order to obtain a concrete numeric value of an atom with a given binary encoding. In particular, the notation “"decodeAtom(phi){-sem-}(IMM (n) (NG`FMT`64))"” depicts an integer or rational value reconstructed from its binary encoding "n" in accordance with the specified format "phi". In Haskell, it is implemented as follows:

\BeginHaskellToken	{decodeAtom}				{Open}	{dec}
  \HaskellContext							{(Ord nu)}
  \HaskellModifier							{\sub}
  \HaskellArgument							{FormatT}
  \HaskellArgument							{AtomT nu}
  \HaskellResult							{Rational}
\EndHaskellToken

\IndexTerm		{decodeAtom}					{dec format atom (rational)}

\begin{code}
decodeAtom {-[*]-} {-[*]-} :: {- (Ord nu) => -} FormatT -> AtomT nu -> Rational

decodeAtom ({-sem-}(NG`FMT`epsilon)) 	{-sem-}(IMM (n) (NG`FMT`64)) ---
  | inRange ({-hfill-}1) 	(<=) (epsilon) (<=) (64) {-ord-}	= n
decodeAtom ({-sem-}(ZG`FMT`epsilon)) 	{-sem-}(IMM (n) (NG`FMT`64)) ---
  | inRange ({-hfill-}1) 	(<=) (epsilon) (<=) (64)		= toRational (discardFraction n - 2^64 #*# (fromBool (n >= 2^63)))
decodeAtom ({-sem-}(OG`FMT`epsilon)) 	{-sem-}(IMM (n) (NG`FMT`64)) ---
  | inRange ({-hfill-}61) 	(<=) (epsilon) (<=) (64)		= toRational (discardFraction n - 2^64 #*# (fromBool (n >= 2^63)))
decodeAtom ({-sem-}(FG`FMT`64)) 	{-sem-}(IMM (n) (NG`FMT`64)) ---
  									= toRational (discardFraction n - 2^64 #*# (fromBool (n >= 2^63)))
decodeAtom ({-sem-}(RG`FMT`64)) 	{-sem-}(IMM (n) (NG`FMT`64)) ---
  | {-hfill-}e 			== 0					= (negate 1)^s * toRational m / 2^^(bias + prec - 1)
  | inRange ({-hfill-}1) 	(<=) (e) (<=) (2 #*# {--} bias) 	= (negate 1)^s * toRational ((m + 2^prec)) / 2^^(bias + prec - e)

 where	s  	= discardFraction n`bits`[63]
 	e  	= discardFraction n`bits`[prec .. 62]
 	m  	= discardFraction n`bits`[0 .. prec - 1]
\end{code}

in which the notations “"n`bits`[i..j {-,-}]"” represent the value of bits "i..j" in the binary representation of the integer "n", as obtained by the corresponding Haskell construction defined in Appendix \ref{Additional}. The reader should observe that the result of "decodeAtom(phi){-sem-}(IMM (n) (NG`FMT`64))" is meaningless if the integer "n" falls outside the required range "0 <= n < 2^64", or if it doesn't represent a properly sign-extended quantity as appropriate for the underlying format "phi". However, since, by construction, such applications of “"decodeAtom"” can never occur in the following specification of the operational semantics "evalAtom", is seems wasteful to enforce these restrictions directly in the above Haskell definition.

In addition to the finite floating point numbers whose values are interpreted by the above function, the IEEE 754 arithmetic model allows for two distinct representations of zero, written as "plusZero" and "minusZero" respectively, as well as two infinities "plusMinusInfinity" and, further, two distinguished non-numeric quantities depicted by the notations “"plusMinusNaN"”. Whenever one of these values is applied to an Etude operator under the format “"RG`FMT`64"”, the result is determined by the rules of the IEEE Standard rather than the conventional mathematical wisdom. In hexadecimal notation, the binary encodings of these six specialised quantities are specified as follows:

\NewHaskellToken	{plusZero}				{Ord}	{\hbox{\plus0}}
\NewHaskellToken	{minusZero}				{Ord}	{\hbox{\minus0}}
\NewHaskellToken	{plusMinusZero}				{Ord}	{\hbox{\pm0}}

\NewHaskellToken	{plusInfinity}				{Ord}	{\hbox{\plus\infty}}
\NewHaskellToken	{minusInfinity}				{Ord}	{\hbox{\minus\infty}}
\NewHaskellToken	{plusMinusInfinity}			{Ord}	{\hbox{\pm\infty}}

\NewHaskellToken	{plusNaN}				{Ord}	{\hbox{\plus NaN}}
\NewHaskellToken	{minusNaN}				{Ord}	{\hbox{\minus NaN}}
\NewHaskellToken	{plusMinusNaN}				{Ord}	{\hbox{\pm NaN}}

\NewHaskellResult	{plusZero}					{(Ord nu) => AtomT nu}
\NewHaskellResult	{minusZero}					{(Ord nu) => AtomT nu}
\NewHaskellResult	{plusInfinity}					{(Ord nu) => AtomT nu}
\NewHaskellResult	{minusInfinity}					{(Ord nu) => AtomT nu}
\NewHaskellResult	{plusNaN}					{(Ord nu) => AtomT nu}
\NewHaskellResult	{minusNaN}					{(Ord nu) => AtomT nu}

\IndexTerm		{plusZero}					{0+ (atom)}
\IndexTerm		{minusZero}					{0- (atom)}
\IndexTerm		{plusInfinity}					{\infty+ (atom)}
\IndexTerm		{minusInfinity}					{\infty- (atom)}
\IndexTerm		{plusNaN}					{NaN+ (atom)}
\IndexTerm		{minusNaN}					{NaN- (atom)}

\begin{code}
plusZero, minusZero, plusInfinity, minusInfinity, plusNaN, minusNaN :: {- (Ord nu) => -} AtomT nu

plusZero	= {-sem-}(IMM ({-sem-}({-hfill-}0x0000000000000000	)) (NG`FMT`64))
minusZero	= {-sem-}(IMM ({-sem-}({-hfill-}0x8000000000000000	)) (NG`FMT`64))
plusInfinity	= {-sem-}(IMM ({-sem-}({-hfill-}0x7FF0000000000000	)) (NG`FMT`64))
minusInfinity	= {-sem-}(IMM ({-sem-}({-hfill-}0xFFF0000000000000	)) (NG`FMT`64))
plusNaN		= {-sem-}(IMM ({-sem-}({-hfill-}0x7FF8000000000000	)) (NG`FMT`64))
minusNaN	= {-sem-}(IMM ({-sem-}({-hfill-}0xFFF8000000000000	)) (NG`FMT`64))
\end{code}

For convenience, we also define a pair of additional notations “"plusMinusZeroSet"” and “"plusMinusInfinitySet"” to stand for the following trivial subsets of these distinguished floating point values:

\NewHaskellToken	{plusMinusZeroSet}			{Ord}	{\{\hbox{\pm0}\}}
\NewHaskellToken	{plusMinusInfinitySet}			{Ord}	{\{\hbox{\pm\infty}\}}

\NewHaskellResult	{plusMinusZeroSet}				{(Ord nu) => SetT (AtomT nu)}
\NewHaskellResult	{plusMinusInfinitySet}				{(Ord nu) => SetT (AtomT nu)}

\IndexTerm		{plusMinusZeroSet}				{0{\pm} (atoms)}
\IndexTerm		{plusMinusInfinitySet}				{\infty{\pm} (atoms)}

\begin{code}
plusMinusZeroSet, plusMinusInfinitySet :: (Ord nu) => SetT (AtomT nu)

plusMinusZeroSet	= listToSet [ plusZero, minusZero ]
plusMinusInfinitySet	= listToSet [ plusInfinity, minusInfinity ]
\end{code}

In general, a given floating point encoding "n" represents a finite rational number only if its biased exponent "n`bits`[{-,,-}prec..62]" has a value less than or equal to "2{--}bias". Otherwise, "n" encodes either an infinity of an appropriate sign, or, if the mantissa "n`bits`[0..prec-1]" has a non-zero value, a special non-numeric quantity known as a «NaN». The two   floating point values depicted by the earlier notations “"plusMinusNaN"” constitute merely a representative example of a NaN that is synthesised internally by the processor as a result of certain undefined mathematical operation. Formally, the complete sets of all finite and NaN floating point quantities can be identified by the following pair of Haskell \hbox{predicates “"isFiniteRational"” and “"isNonRational"”:}

\BeginHaskellToken	{isFiniteRational}			{Open}	{FIN}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{AtomT nu}
  \HaskellResult							{Bool}
\EndHaskellToken

\BeginHaskellToken	{isNonRational}				{Open}	{NaN}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{AtomT nu}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isFiniteRational}				{FIN integer}
\IndexTerm		{isNonRational}					{NaN integer}

\begin{code}
isFiniteRational {-[*]-}, isNonRational {-[*]-} :: {- (Ord nu) => -} AtomT nu -> Bool
 
isFiniteRational 	{-sem-}(IMM (n) (NG`FMT`64)) = (inRange (0) (<=) (discardFraction n `bits` [prec..62]) (<=) (2 #*# {--} bias))
isNonRational 		{-sem-}(IMM (n) (NG`FMT`64)) = (discardFraction n `bits` [prec..62] == 2 #*# {--} bias + 1 && discardFraction n `bits` [0..prec-1] /= 0)
\end{code}

A reader interested in the actual motivation behind the above definitions is referred to the «IEEE 754 Standard for Binary Floating Point Arithmetic» [IEEE754], which covers all of their underlying intricacies with much more details than permitted by the restricted scope of the present work. It should, however, be observed that all of the above definitions presuppose a particular configuration of the MMIX processor, in which all floating point arithmetic is always rounded correctly to the nearest even value as per the standard Haskell “"round"” function. For simplicity, I assume that the system has been configured in this way prior to commencement of every program's execution and that no well-formed MMIX instruction sequence will ever attempt to deviate from that initial configuration.

Armed with these auxiliary definitions, we are now ready to proceed with a precise formulation of the operational semantics "evalAtom" assigned to all well-formed Etude atoms. Predictably, "evalAtom" is formulated as a mapping of such atoms to their normalised representations, which, on MMIX, always depict appropriate constructs of the form “"IMM (x) (NG`FMT`64)"\!”. Accordingly, the Haskell type signature of "evalAtom" can be specified as follows:

\BeginHaskellToken	{evalAtoms}				{Open}	{\ZapfChancery{E}}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{AtomsT nu}
  \HaskellResult							{AtomsT nu}
\EndHaskellToken

\IndexTerm		{evalAtoms}					{E atoms (atoms)}

\set{capture}		{hidden}
\begin{code}
evalAtoms {-[*]-} :: (Ord nu) => AtomsT nu -> AtomsT nu

evalAtoms {-sem-}(alpha_set) = [ evalAtom{--}(alpha_k) | alpha_k <- alpha_set ]
\end{code}

\BeginHaskellToken	{evalAtom}				{Open}	{\ZapfChancery{E}}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{AtomT nu}
  \HaskellResult							{AtomT nu}
\EndHaskellToken

\IndexTerm		{evalAtom}					{E atom (atom)}

\begin{code}
evalAtom {-[*]-} :: (Ord nu) => AtomT nu -> AtomT nu
\end{code}

In particular, every constant atom of the form “"IMM (x) (phi)"\!” is always reducible directly to its encoded value “"encodeAtom(phi){--}(x)"\!”, provided that this encoding can be subsequently reversed without any loss of information. On the other hand, all of the remaining closed atomic forms “"OP1 (op) (phi) {--} {-hv-}(alpha)"” and “"OP2 (alpha1) (op) (phi) (alpha2)"\!” are evaluated systematically by obtaining the respective normal forms of the operands "alpha", "alpha1" and "alpha2" and applying the resulting partially reduced atom to another function "evalAtom'", which performs further normalisation in accordance with the arithmetic semantics of the operator "op". Formally:

\begin{code}
evalAtom {-sem-}(IMM (x) (phi)) ---
  | x == decodeAtom(phi){--}(encodeAtom(phi){--}(x)) ---
  	= encodeAtom(phi){--}(x)
evalAtom {-sem-}(OP1 (op) (phi) {--} {-hv-}(alpha)) ---
  | evalAtom{--}(alpha) == evalAtom{--}(alpha) ---
  	= evalAtom' {-sem-}(OP1 (op) (phi) {--} {-sem-}(evalAtom{--}(alpha)))
evalAtom {-sem-}(OP2 (alpha1) (op) (phi) (alpha2)) ---
  | evalAtom{--}(alpha1) == evalAtom{--}(alpha1) && evalAtom{--}(alpha2) == evalAtom{--}(alpha2) ---
  	= evalAtom' {-sem-}(OP2 ({-sem-}(evalAtom{--}(alpha1))) (op) (phi) ({-sem-}(evalAtom{--}(alpha2))))
\end{code}

in which the cunning comparisons "evalAtom{--}(alpha) == evalAtom{--}(alpha)" ensure that "evalAtom" remains undefined for all diverging and otherwise malformed operand values.

\set{label}		{Lambda/Arch/eqvUnaryAtomJ}
\set{title}		{Atom compatibility}
\begin{theorem}

Two atoms with pairwise-equivalent components are themselves equivalent to each other:

\NewHaskellToken	{eqvUnaryAtomJ}				{Ord}	{\TKeyword{eqv\sub{\alpha1}}}
\NewHaskellToken	{eqvBinaryAtomJ}			{Ord}	{\TKeyword{eqv\sub{\alpha2}}}

\NewHaskellDescription	{eqvUnaryAtomJ}					{Compatibility of unary atoms}
\NewHaskellDescription	{eqvBinaryAtomJ}				{Compatibility of binary atoms}

\IndexTheorem		{eqvUnaryAtomJ}					{EQV_\alpha1 Compatibility of unary atoms}
\IndexTheorem		{eqvBinaryAtomJ}				{EQV_\alpha2 Compatibility of binary atoms}

\begin{haskell}
eqvUnaryAtomJ	:: forall alpha1, alpha2, phi, op => --
	(alpha1 `EqvJ` alpha2) -> --
	{-sem-}(OP1 (op) (phi) {--} {-hv-}(alpha1)) `EqvJ` {-sem-}(OP1 (op) (phi) {--} {-hv-}(alpha2))
eqvBinaryAtomJ	:: forall alpha11, alpha21, alpha12, alpha22, phi, op =>
	(alpha11 `EqvJ` alpha12) -> --
	(alpha21 `EqvJ` alpha22) -> --
	{-sem-}(OP2 (alpha11) (op) (phi) (alpha21)) `EqvJ` {-sem-}(OP2 (alpha12) (op) (phi) (alpha22))
\end{haskell}
\vskip -6pt
\begin{proof}
\begin{haskell}
eqvUnaryAtomJ {-sem-}(EqvAtomC (_P :: {-sem-}(evalAtom{--}(alpha1) = evalAtom{--}(alpha2)))) = EqvAtomC lemma4

 where	lemma1	:: {-sem-}(evalAtom{-sem-}(OP1 (op) (phi) {--} {-hv-}(alpha1)) = evalAtom'{-sem-}(OP1 (op) (phi) {--} {-sem-}(evalAtom{--}(alpha1)))) ---
 			= defn
 	lemma2	:: {-sem-}(evalAtom'{-sem-}(OP1 (op) (phi) {--} {-sem-}(evalAtom{--}(alpha1))) = evalAtom'{-sem-}(OP1 (op) (phi) {--} {-sem-}(evalAtom{--}(alpha2)))) ---
 			= compat (_P) in alpha -> evalAtom'{-sem-}(OP1 (op) (phi) {--} {-hv-}(alpha))
 	lemma3	:: {-sem-}(evalAtom{-sem-}(OP1 (op) (phi) {--} {-hv-}(alpha2)) = evalAtom'{-sem-}(OP1 (op) (phi) {--} {-sem-}(evalAtom{--}(alpha2)))) ---
 			= defn
 	lemma4	:: {-sem-}(evalAtom{-sem-}(OP1 (op) (phi) {--} {-hv-}(alpha1)) = evalAtom{-sem-}(OP1 (op) (phi) {--} {-hv-}(alpha2))) ---
 			= trans (lemma1 (trans lemma2 (symm lemma3)))
\end{haskell}

The analogous proof for binary atomic forms follows a similar structure. \QED

\end{proof}
\end{theorem}

The actual evaluation function "evalAtom'" can be defined for partially normalised unary and binary atomic forms as follows:

\BeginHaskellToken	{evalAtom'}				{Open}	{\ZapfChancery{E}\prime}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{AtomT nu}
  \HaskellResult							{AtomT nu}
\EndHaskellToken

\IndexTerm		{evalAtom'}					{E' atom (atom)}

\begin{code}
evalAtom' {-[*]-} :: (Ord nu) => AtomT nu -> AtomT nu
\end{code}

First of all, in reality, an MMIX processor does not distinguish its functional and object formats from the two genres “"FG"” and “"OG"” as autonomous representations of data. Instead, every atomic operation performed under one of these formats is always treated exactly as if it was applied under the standard integer format “"ZG`FMT`64"”:

\begin{code}
evalAtom' {-sem-}(OP1 (op) (phi) {--} {-hv-}(alpha))	| phi @@ listToSet [ FG`FMT`64, OG`FMT`61, OG`FMT`62, OG`FMT`63, OG`FMT`64 ] ---
  = evalAtom' {-sem-}(OP1 (op) (ZG`FMT`64) {--} {-hv-}(alpha))
evalAtom' {-sem-}(OP2 (alpha1) (op) (phi) (alpha2))	| phi @@ listToSet [ FG`FMT`64, OG`FMT`61, OG`FMT`62, OG`FMT`63, OG`FMT`64 ] ---
  = evalAtom' {-sem-}(OP2 (alpha1) (op) (ZG`FMT`64) (alpha2))
\end{code}

Next, it is convenient to dispense with a number of special cases in the definition of "evalAtom'" that pertain to all mathematically undefined applications of Etude's arithmetic operators. As already mentioned in Section \ref{MMIX/Syntax}, in the interest of conciseness I assume that our MMIX processor is permanently configured in such a way that all arithmetic traps for both integer and floating point operations remain disabled throughout the entire execution of a program. Under these conditions, if one of the two operands to the “"ADD{-sub-}phi"\!”, “"SUB{-sub-}phi"\!”,  “"MUL{-sub-}phi"\!” or “"DIV{-sub-}phi"\!” operator applied under the rational format “"RG`FMT`64"” represents a NaN, then the entire construction is reducible to that NaN, except that the bit "prec - 1" is always set in the result. If both operands represent non-numeric quantities, then the value of the second operand is chosen in preference to the first. On the other hand, if a NaN is supplied to a relational operator “"CEQ{-sub-}phi"\!”, “"CNE{-sub-}phi"\!”, “"CLT{-sub-}phi"\!”, “"CGT {-sub-}phi"\!”, “"CLE{-sub-}phi"\!” or “"CGE{-sub-}phi"\!”, then the result is always 0, so that no NaN is ever deemed equal to itself or any other well-formed floating point quantity. Formally:

\begin{code}
evalAtom' {-sem-}(OP2 (IMM (x) (NG`FMT`64)) (op) (RG`FMT`64) (IMM (y) (NG`FMT`64)))

	| op @@ listToSet [ ADD, SUB, MUL, DIV ] && isNonRational{-sem-}(IMM (y) (NG`FMT`64)) ---
  		= {-sem-}(IMM (toRational {-sem-}(discardFraction y - 2^(prec - 1) #*# {-!-} discardFraction y `bits` [prec - 1] + 2^(prec - 1))) (NG`FMT`64))
	| op @@ listToSet [ ADD, SUB, MUL, DIV ] && isNonRational{-sem-}(IMM (x) (NG`FMT`64)) ---
  		= {-sem-}(IMM (toRational {-sem-}(discardFraction x - 2^(prec - 1) #*# {-!-} discardFraction x `bits` [prec - 1] + 2^(prec - 1))) (NG`FMT`64))
	| op @@ listToSet [ CEQ, CNE, CLT, CGT, CLE, CGE ] && (isNonRational{-sem-}(IMM (x) (NG`FMT`64)) || isNonRational{-sem-}(IMM (y) (NG`FMT`64))) --
	  	= {-sem-}(IMM (0) (NG`FMT`64))
\end{code}

Further, if one or both of the two operands "alpha1" and "alpha2" involved in a floating point addition operation represents "plusMinusInfinity", then the entire atom is reducible to an infinity of the same sign, except that the sum of two infinite floating point values with opposite signs is, by definition, deemed equal to "plusMinusNaN" with the sign of "alpha2". Further, the IEEE Standard proclaims "minusZero + minusZero" to have the value of "minusZero" rather than the expected "plusZero". These three special cases are captured by "evalAtom'" as follows:  

\begin{code}
evalAtom' {-sem-}(OP2 (alpha1) (ADD) (RG`FMT`64) (alpha2)) ---
  | (alpha1 == minusInfinity && alpha2 == plusInfinity) ---
  	= plusNaN
  | (alpha1 == plusInfinity && alpha2 == minusInfinity) ---
  	= minusNaN
  | (alpha1 == plusInfinity || alpha2 == plusInfinity) ---
  	= plusInfinity
  | (alpha1 == minusInfinity || alpha2 == minusInfinity) ---
  	= minusInfinity
  | (inRange (alpha1) (==) (alpha2) (==) (minusZero)) ---
  	= minusZero
\end{code}

Similarly, in a floating point subtraction operation, the same five special cases have equivalent definitions, after flipping the sign bit of the second operand as follows:

\begin{code}
evalAtom' {-sem-}(OP2 (alpha1) (SUB) (RG`FMT`64) (alpha2)) ---
  | (inRange (alpha1) (==) (alpha2) (==) (minusInfinity)) ---
  	= plusNaN
  | (inRange (alpha1) (==) (alpha2) (==) (plusInfinity)) ---
  	= minusNaN
  | (alpha1 == plusInfinity 	|| alpha2 == minusInfinity) ---
  	= plusInfinity
  | (alpha1 == minusInfinity 	|| alpha2 == plusInfinity) ---
  	= minusInfinity
  | (alpha1 == minusZero 	&& alpha2 == plusZero) ---
  	= minusZero
\end{code}

On the other hand, floating point multiplication identifies only three special cases. First of all, the product of "plusMinusZero" with "plusMinusInfinity" (in any order) is equal to "plusNaN" if both operands have identical signs, or "minusNaN" otherwise. All other multiplications involving infinite or zero values result in "plusMinusInfinity" or "plusMinusZero" of an appropriate sign, which, in the following Haskell definition, is captured by adding "2^63" to the encoding of "plusNaN", "plusInfinity" or "plusZero" whenever the sign bits of the two operands have opposite values:

\begin{code}
evalAtom' {-sem-}(OP2 (alpha1) (MUL) (RG`FMT`64) (alpha2)) ---
  |  (alpha1 @@ plusMinusInfinitySet  	&& alpha2 @@ plusMinusZeroSet) 		||
     (alpha1 @@ plusMinusZeroSet	&& alpha2 @@ plusMinusInfinitySet) 	--
     	= {-sem-}(IMM ({-sem-}(toRational (discardFraction (decodeAtom({-sem-}(NG`FMT`64)){--}(plusNaN)) + 2^63 #*# {-!-} s))) (NG`FMT`64))
  |  (alpha1 @@ plusMinusInfinitySet  	|| alpha2 @@ plusMinusInfinitySet) 	--
     	= {-sem-}(IMM ({-sem-}(toRational (discardFraction (decodeAtom({-sem-}(NG`FMT`64)){--}(plusInfinity)) + 2^63 #*# {-!-} s))) (NG`FMT`64))
  |  (alpha1 @@ plusMinusZeroSet      	|| alpha2 @@ plusMinusZeroSet)		--
     	= {-sem-}(IMM ({-sem-}(toRational (discardFraction (decodeAtom({-sem-}(NG`FMT`64)){--}(plusZero)) + 2^63 #*# {-!-} s))) (NG`FMT`64))
  	
 where s = fromBool (discardFraction (decodeAtom({-sem-}(NG`FMT`64)){--}(alpha1)) `bits` [63] /= discardFraction (decodeAtom({-sem-}(NG`FMT`64)){--}(alpha2)) `bits` [63])
\end{code}

In the same manner, the floating point division operations "plusMinusZero DIV plusMinusZero" and "plusMinusInfinity DIV plusMinusInfinity" are semantically equivalent to "plusMinusNaN" of an appropriate sign. Otherwise, divisions of the form "plusMinusInfinity DIV y" and "x DIV plusMinusZero" result in "plusMinusInfinity", while "plusMinusZero DIV y" and "x DIV plusMinusInfinity" are reducible to "plusMinusZero". Formally:

\begin{code}
evalAtom' {-sem-}(OP2 (alpha1) (DIV) (RG`FMT`64) (alpha2)) ---
  |  (alpha1 @@ plusMinusZeroSet 	&& alpha2 @@ plusMinusZeroSet) 		||
     (alpha1 @@ plusMinusInfinitySet	&& alpha2 @@ plusMinusInfinitySet)	--
     	= {-sem-}(IMM ({-sem-}(toRational (discardFraction (decodeAtom({-sem-}(NG`FMT`64)){--}(plusNaN)) + 2^63 #*# {-!-} s))) (NG`FMT`64))
  |  (alpha1 @@ plusMinusInfinitySet 	|| alpha2 @@ plusMinusZeroSet) 		--
     	= {-sem-}(IMM ({-sem-}(toRational (discardFraction (decodeAtom({-sem-}(NG`FMT`64)){--}(plusInfinity)) + 2^63 #*# {-!-} s))) (NG`FMT`64))
  |  (alpha1 @@ plusMinusZeroSet 	|| alpha2 @@ plusMinusInfinitySet) 	--
     	= {-sem-}(IMM ({-sem-}(toRational (discardFraction (decodeAtom({-sem-}(NG`FMT`64)){--}(plusZero)) + 2^63 #*# {-!-} s))) (NG`FMT`64))
  	
 where s = fromBool (discardFraction (decodeAtom({-sem-}(NG`FMT`64)){--}(alpha1)) `bits` [63] /= discardFraction (decodeAtom({-sem-}(NG`FMT`64)){--}(alpha2)) `bits` [63])
\end{code}

Division by zero also crops up in integer arithmetic. If "phi" represents a non-rational format, then, with all arithmetic traps disabled, the MMIX processor reduces all atoms of the form “"OP2 (alpha1) (DIV) (phi) (IMM (0) (NG`FMT`64))"\!” and “"OP2 (alpha1) (REM) (phi) (IMM (0) (NG`FMT`64))"\!” to "alpha2" or "alpha1", respectively:

\begin{code}
evalAtom' {-sem-}(OP2 (alpha1) (DIV) (phi) (alpha2)) ---
  | (genreOfFormat{--}(phi) @@ listToSet [ NG, ZG ] && inRange (1) (<=) (encodingOfFormat{--}(phi)) (<=) (64) && decodeAtom(phi){--}(alpha2) == 0)	= alpha2
evalAtom' {-sem-}(OP2 (alpha1) (REM) (phi) (alpha2)) ---
  | (genreOfFormat{--}(phi) @@ listToSet [ NG, ZG ] && inRange (1) (<=) (encodingOfFormat{--}(phi)) (<=) (64) && decodeAtom(phi){--}(alpha2) == 0)	= alpha1
\end{code}

The final special case pertains to those applications of the six relational operators “"CEQ{-sub-}phi"\!”, “"CNE{-sub-}phi"\!”, “"CLT{-sub-}phi"\!”, “"CGT {-sub-}phi"\!”, “"CLE{-sub-}phi"\!” and “"CGE{-sub-}phi"\!” under the rational format “"RG`FMT`64"”, in which one or both of the two atomic operands has an infinite value. In this case, the result is always equal to “"IMM (1) (NG`FMT`64)"\!” or “"IMM (0) (NG`FMT`64)"\!” whenever the corresponding mathematical relation does or does not hold asymptotically for the supplied operands. Formally:

\begin{code}
evalAtom' {-sem-}(OP2 (alpha1) (CEQ) (RG`FMT`64) (alpha2)) ---
  | alpha1 @@ plusMinusInfinitySet || alpha2 @@ plusMinusInfinitySet ---
  	= {-sem-}(IMM (toRational {-sem-}(fromBool (alpha1 == alpha2))) (NG`FMT`64))
evalAtom' {-sem-}(OP2 (alpha1) (CNE) (RG`FMT`64) (alpha2)) ---
  | alpha1 @@ plusMinusInfinitySet || alpha2 @@ plusMinusInfinitySet ---
  	= {-sem-}(IMM (toRational {-sem-}(fromBool (alpha1 /= alpha2))) (NG`FMT`64))
evalAtom' {-sem-}(OP2 (alpha1) (CLT) (RG`FMT`64) (alpha2)) ---
  | alpha1 @@ plusMinusInfinitySet || alpha2 @@ plusMinusInfinitySet ---
  	= {-sem-}(IMM (toRational {-sem-}(fromBool (alpha1 /= plusInfinity && alpha2 /= minusInfinity))) (NG`FMT`64))
evalAtom' {-sem-}(OP2 (alpha1) (CGT) (RG`FMT`64) (alpha2)) ---
  | alpha1 @@ plusMinusInfinitySet || alpha2 @@ plusMinusInfinitySet ---
  	= {-sem-}(IMM (toRational {-sem-}(fromBool (alpha1 /= minusInfinity && alpha2 /= plusInfinity))) (NG`FMT`64))
evalAtom' {-sem-}(OP2 (alpha1) (CLE) (RG`FMT`64) (alpha2)) ---
  | alpha1 @@ plusMinusInfinitySet || alpha2 @@ plusMinusInfinitySet ---
  	= {-sem-}(IMM (toRational (fromBool {-sem-}(alpha1 == minusInfinity || alpha2 == plusInfinity))) (NG`FMT`64))
evalAtom' {-sem-}(OP2 (alpha1) (CGE) (RG`FMT`64) (alpha2)) ---
  | alpha1 @@ plusMinusInfinitySet || alpha2 @@ plusMinusInfinitySet ---
  	= {-sem-}(IMM (toRational (fromBool {-sem-}(alpha1 == plusInfinity || alpha2 == minusInfinity))) (NG`FMT`64))
\end{code}

Otherwise, every ordinary application of the binary Etude operator “"ADD{-sub-}phi"\!”, “"SUB{-sub-}phi"\!” and “"MUL{-sub-}phi"\!” is equivalent to the true arithmetic value of the corresponding mathematical operation “"+"”, “"-"” or “"*"”, encoded under the specified format "phi" as follows:

\begin{code}
evalAtom' {-sem-}(OP2 (alpha1) (ADD) (phi) (alpha2)) = encodeAtom(phi){--}(decodeAtom(phi){--}(alpha1) + decodeAtom(phi){--}(alpha2))
evalAtom' {-sem-}(OP2 (alpha1) (SUB) (phi) (alpha2)) = encodeAtom(phi){--}(decodeAtom(phi){--}(alpha1) - decodeAtom(phi){--}(alpha2))
evalAtom' {-sem-}(OP2 (alpha1) (MUL) (phi) (alpha2)) = encodeAtom(phi){--}(decodeAtom(phi){--}(alpha1) * decodeAtom(phi){--}(alpha2))
\end{code}

More so, the division operation “"OP2 (alpha1) (DIV) (phi) (alpha2)"\!” reduces into “"IMM ({-sem-}(decodeAtom(phi){--}(alpha1) / decodeAtom(phi){--}(alpha2))) (phi)"\!” if "phi" represents a rational format, or “"IMM ({-sem-}(floor(decodeAtom{--}(phi)(alpha1) / decodeAtom{--}(phi)(alpha2)))) (phi)"\!” under any other arithmetic genre, while the dual remainder operation “"OP2 (alpha1) (REM) (phi) (alpha2)"\!” produces an appropriate binary encoding of the numeric quantity “"discardFraction (decodeAtom(phi){--}(x)) - floor (decodeAtom(phi){--}(x) / decodeAtom(phi){--}(y)) * discardFraction (decodeAtom(phi){--}(x))"”. The later operation is, however, applicable only under integral formats and always remains undefined if "phi" is equal to “"RG`FMT`64"”:

\begin{code}
evalAtom' {-sem-}(OP2 (alpha1) (DIV) (phi) (alpha2)) ---
  | genreOfFormat{--}(phi) == {-sem-}(RG) --
	= encodeAtom(phi){--}(decodeAtom(phi){--}(alpha1) / decodeAtom(phi){--}(alpha2))
  | genreOfFormat{--}(phi) /= {-sem-}(RG) --
	= encodeAtom(phi) (toRational (floor (decodeAtom(phi){--}(alpha1) / decodeAtom(phi){--}(alpha2))))
evalAtom' {-sem-}(OP2 (alpha1) (REM) (phi) (alpha2)) ---
  | genreOfFormat{--}(phi) /= {-sem-}(RG) --
	= encodeAtom(phi){--}(toRational (discardFraction (decodeAtom(phi){--}(alpha1)) --
								- floor (decodeAtom(phi){--}(alpha1) / decodeAtom(phi){--}(alpha2)) --
								* discardFraction (decodeAtom(phi){--}(alpha1))))
\end{code}

In the bitwise operations “"OP2 (alpha1) (AND) (phi) (alpha2)"\!”, “"OP2 (alpha1) (XOR) (phi) (alpha2)"\!” and “"OP2 (alpha1) (IOR) ({-!-}{-!-}phi) (alpha2)"\!”, the format component "phi" constitutes a syntactic noise and is always ignored by the MMIX processor, even if it happens to have an invalid value. Instead, each of these three atomic forms is reducible into the bitwise product, difference or sum of the binary encodings "x" and "y", as captured by the sum of a series \hbox{"[2^k #*# {--}(fromBool (toBool (alpha1`bits`[k]) op toBool (alpha2`bits`[k]))) | k <- [0..63]]",} in which "op" is equal to one of the three boolean Haskell operators “"&&"”, “"/="” or “"||"” for “"AND{-sub-}phi"\!”, “"XOR{-sub-}phi"\!” and “"IOR{-!-}{-!-}{-sub-}phi"\!”, respectively. In other words, “"OP2 (alpha1) (AND) (phi) (alpha2)"\!” is always equivalent to a natural number "n", whose "k"th bit is set in its binary representation if and only if the same bit is set in both of the operands "alpha1" and "alpha2". Similarly, the “"XOR{-sub-}phi"\!” operator sets this result bit if the corresponding indices of "alpha1" and "alpha2" have opposite values, while “"IOR{-!-}{-!-}{-sub-}phi"\!” sets it whenever at least one of these input bits is set. Formally:

\begin{code}
evalAtom' {-sem-}(OP2 (IMM (x) (NG`FMT`64)) (AND) (phi) (IMM (y) (NG`FMT`64))) ---
  	= {-sem-}(IMM ({-sem-}(toRational --
  				(sum [2^k #*# {--}(fromBool (toBool (discardFraction x`bits`[k]) {-hfil-} &&  toBool (discardFraction y`bits`[k]))) | k <- [0..63]]))) (NG`FMT`64))
evalAtom' {-sem-}(OP2 (IMM (x) (NG`FMT`64)) (XOR) (phi) (IMM (y) (NG`FMT`64))) ---
  	= {-sem-}(IMM ({-sem-}(toRational --
  				(sum [2^k #*# {--}(fromBool (toBool (discardFraction x`bits`[k]) {-hfil-} /=  toBool (discardFraction y`bits`[k]))) | k <- [0..63]]))) (NG`FMT`64))
evalAtom' {-sem-}(OP2 (IMM (x) (NG`FMT`64)) (IOR) (phi) (IMM (y) (NG`FMT`64))) ---
  	= {-sem-}(IMM ({-sem-}(toRational --
  				(sum [2^k #*# {--}(fromBool (toBool (discardFraction x`bits`[k]) {-hfil-} ||  toBool (discardFraction y`bits`[k]))) | k <- [0..63]]))) (NG`FMT`64))
\end{code}

Every application of the “"SHL{-sub-}phi"\!” or “"SHR{-sub-}phi"\!” operator under a non-rational format is equivalent to multiplying or dividing the decoded value of its first operand by "2^n", where "n" is the natural number derived from the second operand under the natural format “"NG`FMT`64"”. For “"SHR{-sub-}phi"\!”, the result is rounded towards "minusInfinity" in the same way as was done earlier by the ordinary integer division operator “"DIV{-sub-}phi"\!”. Formally:

\begin{code}
evalAtom' {-sem-}(OP2 (alpha1) (SHL) (phi) (alpha2)) ---
  | genreOfFormat{--}(phi) /= {-sem-}(RG) --
	= encodeAtom(phi){--}(			decodeAtom(phi){--}(alpha1) * 2^(discardFraction (decodeAtom({-sem-}(NG`FMT`64)){--}(alpha2)))	)
evalAtom' {-sem-}(OP2 (alpha1) (SHR) (phi) (alpha2)) ---
  | genreOfFormat{--}(phi) /= {-sem-}(RG) --
	= encodeAtom(phi) (toRational (floor (	decodeAtom(phi){--}(alpha1) / 2^(discardFraction (decodeAtom({-sem-}(NG`FMT`64)){--}(alpha2)))	)))
\end{code}

On the other hand, when one of the six relational Etude operators “"CEQ{-sub-}phi"\!”, “"CNE{-sub-}phi"\!”, “"CLT{-sub-}phi"\!”, “"CGT {-sub-}phi"\!”, “"CLE{-sub-}phi"\!” or “"CGE{-sub-}phi"\!” is applied to a pair of finite numeric values, the entire operation is always equivalent to a natural atom of the form “"IMM (n) (NG`FMT`64)"\!”, in which the integer "n" has the value of "1" or "0", obtained from the numeric representation of the corresponding Haskell boolean expression “"x == y"”, “"x /= y"”, “"x < y"”, “"x > y"”, “"x <= y"” or “"x >= y"”, in which "x" and "y" represent the respective decoded numeric values of the construct's two operands. In Haskell:

\begin{code}
evalAtom' {-sem-}(OP2 (alpha1) (CEQ) (phi) (alpha2)) ---
	= encodeAtom({-sem-}(NG`FMT`64)){--}(toRational (fromBool (decodeAtom(phi){--}(alpha1) == decodeAtom(phi){--}(alpha2))))
evalAtom' {-sem-}(OP2 (alpha1) (CNE) (phi) (alpha2)) ---
	= encodeAtom({-sem-}(NG`FMT`64)){--}(toRational (fromBool (decodeAtom(phi){--}(alpha1) /= decodeAtom(phi){--}(alpha2))))
evalAtom' {-sem-}(OP2 (alpha1) (CLT) (phi) (alpha2)) ---
	= encodeAtom({-sem-}(NG`FMT`64)){--}(toRational (fromBool (decodeAtom(phi){--}(alpha1) < decodeAtom(phi){--}(alpha2))))
evalAtom' {-sem-}(OP2 (alpha1) (CGT) (phi) (alpha2)) ---
	= encodeAtom({-sem-}(NG`FMT`64)){--}(toRational (fromBool (decodeAtom(phi){--}(alpha1) > decodeAtom(phi){--}(alpha2))))
evalAtom' {-sem-}(OP2 (alpha1) (CLE) (phi) (alpha2)) ---
	= encodeAtom({-sem-}(NG`FMT`64)){--}(toRational (fromBool (decodeAtom(phi){--}(alpha1) <= decodeAtom(phi){--}(alpha2))))
evalAtom' {-sem-}(OP2 (alpha1) (CGE) (phi) (alpha2)) ---
	= encodeAtom({-sem-}(NG`FMT`64)){--}(toRational (fromBool (decodeAtom(phi){--}(alpha1) >= decodeAtom(phi){--}(alpha2))))
\end{code}

The only cases remaining in the definition of "evalAtom'" pertain to the three unary Etude operators “"NEG{-sub-}phi"\!”, “"NOT{-sub-}phi"\!” and “"phi'{-!-}{-sub-}phi"\!”. First of all, every atom of the form “"OP1 (NEG) (phi) {--} {-hv-}(alpha)"” is essentially equivalent to “"OP2 ({-sem-}(minusZero)) (SUB) (phi) (alpha)"” if "phi" belongs to the rational genre, or else to “"OP2 ({-sem-}(plusZero)) (SUB) (phi) (alpha)"” if it represents any other well-formed Etude format. Further, the «bit complement operation» “"OP1 (NOT) (phi) {--} {-hv-}(alpha)"” stands for a subtraction of "alpha"'s value from "2^64 - 1" under every Etude format other than “"RG`FMT`64"”:

\begin{code}
evalAtom' {-sem-}(OP1 (NEG) (phi) {--} {-hv-}(alpha)) ---
  | phi == {-sem-}(RG`FMT`64) 	= evalAtom' {-sem-}(OP2 ({-sem-}(minusZero)) (SUB) (phi) (alpha))
  | phi /= {-sem-}(RG`FMT`64) 	= evalAtom' {-sem-}(OP2 ({-sem-}(plusZero)) (SUB) (phi) (alpha))
evalAtom' {-sem-}(OP1 (NOT) (phi) {--} {-hv-}(alpha)) ---
  | phi /= {-sem-}(RG`FMT`64) 	= evalAtom' {-sem-}(OP2 (IMM ({-sem-}(2^64 - 1)) (NG`FMT`64)) (SUB) (phi) (alpha))
\end{code}

Last but not least, all Etude «conversion operations» of the form “"OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(alpha)"\!” denote the numeric value of "alpha" decoded under the given format "phi" and reinterpreted under the target format "phi'", except that, if "alpha" represents a non-finite floating point quantity, then its binary encoding is reinterpreted directly without further analysis and, if the source and target formats are identical, then the operation is always reducible directly to "alpha", even if "phi" does not represent a well-formed Etude format. Formally:

\begin{code}
evalAtom' {-sem-}(OP1 (CNV phi' {-!-}) (phi) {--} {-hv-} (alpha)) ---
  | phi == phi' ---
  	= {-sem-}(alpha)
  | phi == {-sem-}(RG`FMT`64) && not (isFiniteRational{--}(alpha)) ---
	= encodeAtom(phi'){--}(decodeAtom(phi){--}(alpha))
  | otherwise ---
	= encodeAtom(phi'){--}(decodeAtom({-sem-}(NG`FMT`64)){--}(alpha))
\end{code}

\set{label}		{Lambda/Arch/wfEvalAtomJ}
\set{title}		{Reduction of atoms}
\set{style}		{corollary}
\begin{theorem}

Under "evalAtom", every well-formed atom is reducible into a valid atomic constant:

\NewHaskellToken	{wfEvalAtomJ}				{Ord}	{\TKeyword{wf\sub{\ZapfChancery{E}}}}
\NewHaskellToken	{immEvalAtomJ}				{Ord}	{\TKeyword{imm\sub{\ZapfChancery{E}}}}

\NewHaskellDescription	{wfEvalAtomJ}					{Validity of reducible atoms}
\NewHaskellDescription	{immEvalAtomJ}					{Normalisation of reducible atoms}

\IndexTheorem		{wfEvalAtomJ}					{WF_E Validity of reducible atoms}
\IndexTheorem		{immEvalAtomJ}					{IMM_E Normalisation of reducible atoms}

\begin{haskell}
wfEvalAtomJ 	:: forall alpha => WfJ{--}(alpha) -> WfJ{-sem-}(evalAtom{--}(alpha))
immEvalAtomJ 	:: forall alpha => WfJ{--}(alpha) -> {-sem-}(isImmediateAtom{--}(evalAtom{--}(alpha)))
\end{haskell}
\end{theorem}

\set{label}		{Lambda/Arch/wfZeroJ}
\set{title}		{Generic validity of constant atoms}
\begin{theorem}

The following trivial atomic forms are always well-formed:

\NewHaskellToken	{wfZeroAtomJ}				{Ord}	{\TKeyword{wf\sub{0}}}
\NewHaskellToken	{wfImmIgAtomJ}				{Ord}	{\TKeyword{wf\sub{i}}}
\NewHaskellToken	{wfImmRgAtomJ}				{Ord}	{\TKeyword{wf\sub{r}}}

\NewHaskellDescription	{wfZeroAtomJ}					{Validity of “{\let\WhiteSpace\relax\haskell{IMM (0) (phi)}\!\!\!}”}
\NewHaskellDescription	{wfImmIgAtomJ}					{Validity of integral atoms}
\NewHaskellDescription	{wfImmRgAtomJ}					{Validity of rational atoms}

\IndexTheorem		{wfZeroAtomJ}					{WF_0 Validity of \#0\phi}
\IndexTheorem		{wfImmIgAtomJ}					{WF_I Validity of integral atoms}
\IndexTheorem		{wfImmRgAtomJ}					{WF_R Validity of rational atoms}

\begin{haskell}
wfZeroAtomJ 	:: forall phi => --
	WfJ{--}(phi) -> --
	WfJ{-sem-}(IMM (0) (phi))

wfImmIgAtomJ 	:: forall n :: Integer, phi =>
	WfJ{--}(phi) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG] && glbOfFormat{--}(phi) <= n <= lubOfFormat{--}(phi)) ->
	WfJ{-sem-}(IMM (x) (phi))
	
wfImmRgAtomJ 	:: forall s, m, e :: Integer, phi =>
 	WfJ{--}(phi) ->
 	{-sem-}(genreOfFormat{--}(phi) == {-sem-}(RG) && --
		0 <= s <= 1 && --
		0 <= m < radixOfFormat{--}(phi)^(precOfFormat{--}(phi)) && --
		minExpOfFormat{--}(phi) <= e <= maxExpOfFormat{--}(phi)) ->
	WfJ{-sem-}(IMM ({-sem-}((negate 1)^s * m * radixOfFormat{--}(phi)^(e - precOfFormat{--}(phi)))) (phi))
\end{haskell}
\begin{proof}

From the various cases in the definition of "evalAtom", we have "evalAtom{-sem-}(IMM (0) (phi)) = {-sem-}(IMM (0) (NG`FMT`64))", which is obviously well-formed by "WfImmC", since "0 <= 0 < 2^64". Accordingly, "wfZeroAtomJ" can be established by a simple case inspection of all well-formed Etude formats "phi", while utilising the guarantees provides by the supplied validity assumption for satisfaction of the required preconditions featured in the definition of "evalAtom". 

Similarly, for all well-formed integral formats "phi" and "glbOfFormat{--}(phi) <= n <= lubOfFormat{--}(phi)", we can establish the reduction chain "evalAtom{-sem-}(IMM (n) (phi)) = encodeAtom(phi){--}(x) = {-sem-}(IMM (toRational {-sem-}(truncate (x) `mod` 2^epsilon)) (NG`FMT`64))" if "phi" belongs to the natural genre “"NG"” and "evalAtom{-sem-}(IMM (n) (phi)) = {-sem-}(IMM (toRational {-sem-}(truncate (x) `mod` 2^epsilon + (2^64 - 2^epsilon) * truncate (x) `bits` [epsilon - 1])) (NG`FMT`64))" if it represents an integer format. Either way, by definition of the “"`mod`"” operator, "n `mod` m < m" for all possible integer values of "n" and "m", and, by well-formedness of "phi", "epsilon < 64", so that, in both cases, the ensuing natural constant "truncate (x) `mod` 2^epsilon" or, for signed integers, "truncate (x) `mod` 2^epsilon + (2^64 - 2^epsilon) * truncate (x) `bits` [epsilon - 1]", must always represent a non-negative integer less than "2^64" in magnitude, which is sufficient to establish a validity of the supplied atom directly from the axiom "WfImmC".

The proof of "wfImmRgAtomJ" has a similar structure, using the supplied constraints on the integers "s", "m" and "e" to satisfy the corresponding preconditions of "evalAtom". "WfImmC" can then be established directly by a simple arithmetic reasoning about the range of all possible \hbox{natural} numbers \hbox{"2^63 #*# {--} s + 2^prec #*# {--} e + round{--}(2^^(bias + prec - e) #*# {--} abs(x) - 2^prec)"} that are featured in the definition of “"encodeAtom"”. \QED

\end{proof}
\end{theorem}

An actual symbolic rendition of these proofs in the framework of the Haskell extensions from Chapter \ref{Lambda} would require an uncomplicated but rather extensive reasoning that, in Fermat's famous words, is simply too long to fit in the margin of this book, given that it would contribute little to our understanding of the critical verification processes at play within a linearly correct program translation system.

\set{label}		{Lambda/Arch/eqvAddAtomJ}
\set{title}		{Addition of rational values}
\begin{theorem}

Every well-formed Etude atom of the form “"OP2 (IMM (x) (phi)) (ADD) (phi) (IMM (y) (phi))"\!” is equivalent to “"IMM ({-sem-}(x + y)) (phi)"\!”, up to the rounding error of the specified rational format "phi", provided that "glbOfFormat{--}(phi) <= x + y <= lubOfFormat{--}(phi)":

\NewHaskellToken	{eqvAddAgAtomJ}				{Ord}	{\TKeyword{eqv\sub{\plus A}}}
\NewHaskellDescription	{eqvAddAgAtomJ}					{Equivalence of arithmetic “\haskell{ADD{-sub-}phi}\!\!\!” operations}
\IndexTheorem		{eqvAddAgAtomJ}					{EQV_\plus A Equivalence of arithmetc of \plus\phi operations}

\begin{haskell}
eqvAddAgAtomJ	:: forall x, y, z, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [ ZG, RG ] && glbOfFormat{--}(phi) <= x + y <= lubOfFormat{--}(phi)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (ADD) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM (z) (phi)) ->
	{-sem-}(abs (z - (x + y)) < ulpOfAtom(phi){--}(x + y))
\end{haskell}
\begin{proof}

First of all, we observe that, given the assumption "glbOfFormat{--}(phi) <= x + y <= lubOfFormat{--}(phi)", in the following proof we only need to concern ourselves with finite operand values, safely discarding all infinities and NaNs. Accordingly, let us begin with a detailed analysis of the common reduction chain of "evalAtom" for all such operands:

\begin{haskell}
evalAtom{-sem-}(OP2 (IMM (x) (phi)) (ADD) (phi) (IMM (y) (phi)))
	= evalAtom'{-sem-}(OP2 ({-sem-}(evalAtom{-sem-}(IMM (x) (phi)))) (ADD) (phi) ({-sem-}(evalAtom{-sem-}(IMM (y) (phi)))))			-- reduce "evalAtom"
	= evalAtom'{-sem-}(OP2 ({-sem-}(encodeAtom({-sem-}(phi)){-sem-}(IMM (x) (phi)))) --
				(ADD) (phi) ({-sem-}(encodeAtom({-sem-}(phi)){-sem-}(IMM (y) (phi)))))						-- reduce "evalAtom"
	= encodeAtom({-sem-}(phi)){--}(decodeAtom({-sem-}(phi)){--}(encodeAtom({-sem-}(phi)){-sem-}(IMM (x) (phi))) --
						+ decodeAtom({-sem-}(phi)){--}(encodeAtom({-sem-}(phi)){-sem-}(IMM (y) (phi))))			-- reduce "evalAtom"
	= encodeAtom({-sem-}(RG`FMT`64)){--}(x + y)												-- expand "evalAtom"
	= z'
\end{haskell}

where the third reduction step follows from the precondition "x == decodeAtom({-sem-}(phi)){--}(encodeAtom({-sem-}(phi)){-sem-}(IMM (x) (phi)))" found in the definition of "evalAtom" over Etude constants.

Now, if "z == 0", then, by definition of “"encodeAtom"”, the resulting atom is equivalent to “"IMM (0) (NG`FMT`64)"”. Under the IEEE floating point model, the decoded value of such an atom, represented in this work by the notation “"decodeAtom(phi){-sem-}(IMM (0) (NG`FMT`64))"”, is equal to the rational number "0", so that "abs (0 - 0) == 0 < ulpOfAtom(phi){--}(0) == 2^(negate 1021)" as required.

Otherwise, let "z == x + y", with the biased exponent "e" equal to "floorLogBase2(abs(z)) + bias" and let "s" be the sign of "z", defined by the numeric value of the relation "z < 0".  Further, let "s' == z'`bits`[63]", "e' == z'`bits`[52..62]" and "m' == z'`bits`[0..51]", so that these three variables represent the sign, exponent and mantissa of the encoded constant "z'".

If "z" is normalised, so that "0 < e <= 2{--}bias", then, by definition of “"ulpOfAtom"”, the required margin of error is equal to "2^(floorLogBase2 (abs (z)) - precOfFormat{--}(phi)) == 2^(e - bias - precOfFormat{--}(phi)) == 2^(e - bias - prec + 1)". Further, the “"encodeAtom"” function gives us "z == 2^63 #*# {--} s + 2^prec #*# {--} e + round{--}(2^^(bias + prec - e) #*# {--} abs(z)) - 2^prec", so that "s' == s", "e' == e" and "m' == round{--}(2^^(bias + prec - e) #*# {--} abs(z)) - 2^prec" by definition of bit extraction. Accordingly:

\begin{haskell}
abs (decodeAtom(phi){-sem-}(IMM (z) (NG`FMT`64)) - z)
	= abs ((negate 1)^s' * (m' + 2^prec) / 2^^(bias + prec - e') - z)							-- reduce “"decodeAtom"”
	= abs ((negate 1)^s * (m' + 2^prec) / 2^^(bias + prec - e) - z)								-- substitute "s'" and "e'"
	= abs ((negate 1)^s * (m' + 2^prec) / 2^^(bias + prec - e) - (negate 1)^(s) * abs(z))					-- properties of “"abs({-*-})"”
	= abs ((negate 1)^s * ((m' + 2^prec) / 2^^(bias + prec - e) - abs(z)))							-- distribution of "(negate 1)^(z < 0)"
	= (m' + 2^prec) / 2^^(bias + prec - e) - abs(z)										-- since "abs((negate 1)^(x)) == x" if "x > 0"
	= (round{--}(2^^(bias + prec - e) #*# {--} abs(z)) - 2^prec + 2^prec) / 2^^(bias + prec - e) - abs(z)			-- substitute "m'"
	= round{--}(2^^(bias + prec - e) #*# {--} abs(z)) / 2^^(bias + prec - e) - abs(z)					-- algebraic simplifications
	< (2^^(bias + prec - e) #*# {--} abs(z) + 1) / 2^^(bias + prec - e) - abs(z)						-- properties of “"round"”
	= 2^^(e - bias - prec)													-- algebraic simplifications
	< 2^^(e - bias - prec + 1)
\end{haskell}

as required. The increased precision of our result is due to the fact that our MMIX processor always performs exact rounding of all floating point quantities, so that none of its arithmetic operations can ever introduce more than half an ULP of error.

Finally, if "z" represents a denormalised quantity, i.e., if "e <= 0", then the result is encoded as "z' == 2^63 #*# {--} s + round{--}(2^^(bias + prec - 1) #*# {--} abs(z))", so that "m' == round{--}(2^^(bias + prec - 1) #*# {--} abs(z))", "s' == s" and \hbox{"e' == 0"} by definition of the bit extraction operator. Further, for all such denormalised numbers, the required unit of least precision "ulpOfAtom(phi){--}(z)" is defined as "2^(minExpOfFormat{--}(phi) - 1) == 2^(negate 1022)". Keeping these facts in mind, the exact amount of rounding error introduced by the operation can be computed as follows:

\begin{haskell}
abs (decodeAtom(phi){-sem-}(IMM (z) (NG`FMT`64)) - z)
	= abs ((negate 1)^(s') * m' / 2^^(bias + prec - 1) - z)									-- reduce “"decodeAtom"”
	= abs ((negate 1)^(s) * m' / 2^^(bias + prec - 1) - z)									-- substitute "s'"
	= abs ((negate 1)^(s) * m' / 2^^(bias + prec - 1) - (negate 1)^(z < 0) * abs(z))					-- properties of “"abs({-*-})"”
	= abs ((negate 1)^(s) * m' / 2^^(bias + prec - 1) - (negate 1)^(s) * abs(z))						-- substitute "s"
	= abs ((negate 1)^(s) * (m' / 2^^(bias + prec - 1) - abs(z)))								-- distribution of "(negate 1)^(z < 0)"
	= m' / 2^^(bias + prec - 1) - abs(z)											-- since "abs((negate 1)^(x)) == x" if "x > 0"
	= round{--}(2^^(bias + prec - 1) #*# {--} abs(x)) / 2^^(bias + prec - 1) - abs(z)					-- substitute "m'"
	< (2^^(bias + prec - 1) #*# {--} abs(z) + 1) / 2^^(bias + prec - 1) - abs(z)						-- properties of “"round"”
	= abs(z) + 2^^(1 - bias - prec) - abs(z)										-- distribution of "2^^(bias + prec - 1)"
	= 2^^(1 - 1023 - 52)													-- simplify, substitute "bias" and "prec"
	< 2^(negate 1022)
\end{haskell}

as required. Observe that, for denormalised numbers, MMIX buys us a whooping 52 bits of precision, as a result of its complete support for gradual underflow performed during such operations, that is permitted but not mandated by Etude. In the above definitions, steps 3 and 7 follow from the well-known identities “"round{--}(x) < x + 1"” and “"x == (negate 1)^(x < 0) * abs(x)"”, which hold for all rational numbers "x". \QED

\end{proof}
\end{theorem}

Section \ref{Lambda/Atoms} lists 45 additional theorems that are used to capture the generic algebraic semantics of all arithmetic Etude operations. However, for brevity, in this chapter I omit their detailed justifications, since all of the corresponding proofs follow a structure similar to, and generally much simpler than the above reasoning about the properties of rational addition, save for the less interesting details of Haskell arithmetic.

\NewHaskellToken	{wfNegAtomJ}				{Ord}	{\TKeyword{wf\sub{\lnot}}}
\NewHaskellToken	{wfAddAtomJ}				{Ord}	{\TKeyword{wf\sub{\plus}}}
\NewHaskellToken	{wfSubAtomJ}				{Ord}	{\TKeyword{wf\sub{\minus}}}
\NewHaskellToken	{wfMulAtomJ}				{Ord}	{\TKeyword{wf\sub{\times}}}
\NewHaskellToken	{wfDivAtomJ}				{Ord}	{\TKeyword{wf\sub{\div}}}
\NewHaskellToken	{wfRemAtomJ}				{Ord}	{\TKeyword{wf\sub{\rem}}}

\NewHaskellDescription	{wfNegAtomJ}					{Validity of the unary “\haskell{NEG{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{wfAddAtomJ}					{Validity of the “\haskell{ADD{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{wfSubAtomJ}					{Validity of the “\haskell{SUB{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{wfMulAtomJ}					{Validity of the “\haskell{MUL{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{wfDivAtomJ}					{Validity of the “\haskell{DIV{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{wfRemAtomJ}					{Validity of the “\haskell{REM{-sub-}phi}\!\!\!” operations}

\IndexTheorem		{wfNegAtomJ}					{WF_\lnot	Validity of the unary -\phi operations}
\IndexTheorem		{wfAddAtomJ}					{WF_\plus	Validity of the +\phi operations}
\IndexTheorem		{wfSubAtomJ}					{WF_\minus	Validity of the -\phi operations}
\IndexTheorem		{wfMulAtomJ}					{WF_\times	Validity of the \times\phi operations}
\IndexTheorem		{wfDivAtomJ}					{WF_\div	Validity of the \div\phi operations}
\IndexTheorem		{wfRemAtomJ}					{WF_\rem	Validity of the \rem\phi operations}

\set{capture}		{hidden}
\begin{haskell}
wfNegAtomJ 	:: forall x, phi =>
	WfJ{-sem-}(IMM (x) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) == {-sem-}(NG) || --
		(genreOfFormat{--}(phi) @@ listToSet [ZG, RG] && glbOfFormat{--}(phi) <= negate x <= lubOfFormat{--}(phi))) ->
	WfJ{-sem-}(OP1 (NEG) (phi) {--} {-raw-}(IMM (x) (phi)))

wfAddAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) == {-sem-}(NG) || --
		(genreOfFormat{--}(phi) @@ listToSet [ZG, RG] && glbOfFormat{--}(phi) <= x + y <= lubOfFormat{--}(phi))) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (ADD) (phi) (IMM (y) (phi)))
			
wfSubAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) == {-sem-}(NG) || --
		(genreOfFormat{--}(phi) @@ listToSet [ZG, RG] && glbOfFormat{--}(phi) <= x - y <= lubOfFormat{--}(phi))) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (SUB) (phi) (IMM (y) (phi)))

wfMulAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) == {-sem-}(NG) || --
		(genreOfFormat{--}(phi) @@ listToSet [ZG, RG] && glbOfFormat{--}(phi) <= x * y <= lubOfFormat{--}(phi))) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (MUL) (phi) (IMM (y) (phi)))

wfDivAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(y /= 0 && genreOfFormat{--}(phi) @@ listToSet [NG, ZG, RG] && glbOfFormat{--}(phi) <= x / y <= lubOfFormat{--}(phi)) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (DIV) (phi) (IMM (y) (phi)))

wfRemAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(y /= 0 && genreOfFormat{--}(phi) @@ listToSet [NG, ZG, RG] && glbOfFormat{--}(phi) <= x / y <= lubOfFormat{--}(phi)) -> --
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (REM) (phi) (IMM (y) (phi)))
\end{haskell}

\NewHaskellToken	{wfBitAtomJ}				{Ord}	{\TKeyword{wf\sub{bit}}}
\NewHaskellToken	{wfShlAtomJ}				{Ord}	{\TKeyword{wf\sub{\ll}}}
\NewHaskellToken	{wfShrAtomJ}				{Ord}	{\TKeyword{wf\sub{\gg}}}
\NewHaskellToken	{wfNotAtomJ}				{Ord}	{\TKeyword{wf\sub{\sim}}}

\NewHaskellDescription	{wfBitAtomJ}					{Validity of the “\haskell{AND{-sub-}phi}\!\!\!”,
										“\haskell{XOR{-sub-}phi}\!\!\!” and “\haskell{IOR{-!-}{-!-}{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{wfShlAtomJ}					{Validity of the “\haskell{SHL{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{wfShrAtomJ}					{Validity of the “\haskell{SHR{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{wfNotAtomJ}					{Validity of the unary “\haskell{NOT{-sub-}phi}\!\!\!” operations}

\IndexTheorem		{wfBitAtomJ}					{WF_BIT Validity of the \bitand\phi, \bitxor\phi and \bitor\phi operations}
\IndexTheorem		{wfShlAtomJ}					{WF_\ll Validity of the \ll\phi operations}
\IndexTheorem		{wfShrAtomJ}					{WF_\gg Validity of the \gg\phi operations}
\IndexTheorem		{wfNotAtomJ}					{WF_\sim Validity of the unary \sim\phi operations}

\set{capture}		{hidden}
\begin{haskell}
wfBitAtomJ	:: forall x, y, phi, op =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(x >= 0 && y >= 0 && op @@ listToSet [ AND, XOR, IOR ] && genreOfFormat{--}(phi) @@ listToSet [NG, ZG]) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (op) (phi) (IMM (y) (phi)))

wfShlAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(x >= 0 && 0 <= y < widthOfFormat{--}(phi) && (genreOfFormat{--}(phi) == {-sem-}(NG) || (genreOfFormat{--}(phi) == {-sem-}(ZG) && x * 2^y <= lubOfFormat{--}(phi)))) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (SHL) (phi) (IMM (y) (phi)))

wfShrAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(x >= 0 && 0 <= y < widthOfFormat{--}(phi) && genreOfFormat{--}(phi) @@ listToSet [ NG, ZG ]) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (SHR) (phi) (IMM (y) (phi)))

wfNotAtomJ	:: forall x, phi => ---
	WfJ{-sem-}(IMM (x) (phi)) -> --
	{-sem-}(genreOfFormat{--}(phi) == {-sem-}(NG)) -> --
	WfJ{-sem-}(OP1 (NOT) (phi) {--} {-raw-}(IMM (x) (phi)))
\end{haskell}

\NewHaskellToken	{wfEqlAtomJ}				{Ord}	{\TKeyword{wf\sub{eql}}}
\NewHaskellToken	{wfRelAtomJ}				{Ord}	{\TKeyword{wf\sub{rel}}}

\NewHaskellDescription	{wfEqlAtomJ}					{Validity of the “\haskell{CEQ{-sub-}phi}\!\!\!” and “\haskell{CNE{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{wfRelAtomJ}					{Validity of the “\haskell{CLT{-sub-}phi}\!\!\!”, “\haskell{CGT{-sub-}phi}\!\!\!”,
										“\haskell{CLE{-sub-}phi}\!\!\!” and “\haskell{CGE{-sub-}phi}\!\!\!” operations}

\IndexTheorem		{wfEqlAtomJ}					{WF_EQL Validity of the \eq\phi and \ne\phi operations}
\IndexTheorem		{wfRelAtomJ}					{WF_REL Validity of the \lt\phi, \gt\phi, \le\phi and \ge\phi operations}

\set{capture}		{hidden}
\begin{haskell}
wfEqlAtomJ	:: forall x, y, phi, op =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) -> --
	{-sem-}(op @@ listToSet [ CEQ, CNE ]) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (op) (phi) (IMM (y) (phi)))

wfRelAtomJ	:: forall x, y, phi, op =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) -> --
	{-sem-}(op @@ listToSet [ CLT, CGT, CLE, CGE ] && genreOfFormat{--}(phi) /= {-sem-}(FG)) ->
	WfJ{-sem-}(OP2 (IMM (x) (phi)) (op) (phi) (IMM (y) (phi)))
\end{haskell}

\NewHaskellToken	{wfCnvNgAtomJ}				{Ord}	{\TKeyword{wf\sub{cn}}}
\NewHaskellToken	{wfCnvAgAtomJ}				{Ord}	{\TKeyword{wf\sub{ca}}}
\NewHaskellToken	{wfCnvPgAtomJ}				{Ord}	{\TKeyword{wf\sub{cp}}}

\NewHaskellDescription	{wfCnvNgAtomJ}					{Validity of natural conversions}
\NewHaskellDescription	{wfCnvAgAtomJ}					{Validity of arithmetic conversions}
\NewHaskellDescription	{wfCnvPgAtomJ}					{Validity of pointer conversions}

\IndexTheorem		{wfCnvNgAtomJ}					{WF_CN Validity of natural conversions}
\IndexTheorem		{wfCnvAgAtomJ}					{WF_CA Validity of arithmetic conversions}
\IndexTheorem		{wfCnvPgAtomJ}					{WF_CP Validity of pointer conversions}

\set{capture}		{hidden}
\begin{haskell}
wfCnvNgAtomJ	:: forall x, phi, phi' =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG] && genreOfFormat{--}(phi') == {-sem-}(NG)) ->
	WfJ{-sem-}(OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(IMM (x) (phi)))
	
wfCnvAgAtomJ	:: forall x, phi, phi' =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	{-sem-}(listToSet [genreOfFormat{--}(phi), genreOfFormat{--}(phi')] `subset` listToSet [NG, ZG, RG] && --
		glbOfFormat{--}(phi') <= x <= lubOfFormat{--}(phi')) ->
	WfJ{-sem-}(OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(IMM (x) (phi)))
	
wfCnvPgAtomJ	:: forall phi, phi' =>
	WfJ{--}(phi) -> WfJ{--}(phi') -> --
	{-sem-}(genreOfFormat{--}(phi) /@ {-sem-}(RG) && genreOfFormat{--}(phi) /@ {-sem-}(RG)) ->
	WfJ{-sem-}(OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(IMM (0) (phi)))
\end{haskell}

\NewHaskellToken	{eqvAddNgAtomJ}				{Ord}	{\TKeyword{eqv\sub{\plus N}}}
\NewHaskellToken	{eqvSubNgAtomJ}				{Ord}	{\TKeyword{eqv\sub{\minus N}}}
\NewHaskellToken	{eqvMulNgAtomJ}				{Ord}	{\TKeyword{eqv\sub{\times N}}}
\NewHaskellToken	{eqvNegNgAtomJ}				{Ord}	{\TKeyword{eqv\sub{\lnot N}}}
\NewHaskellToken	{eqvNotNgAtomJ}				{Ord}	{\TKeyword{eqv\sub{\sim N}}}
\NewHaskellToken	{eqvCnvNgAtomJ}				{Ord}	{\TKeyword{eqv\sub{IN}}}

\NewHaskellDescription	{eqvAddNgAtomJ}					{Equivalence of natural “\haskell{ADD{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvSubNgAtomJ}					{Equivalence of natural “\haskell{SUB{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvMulNgAtomJ}					{Equivalence of natural “\haskell{MUL{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvNegNgAtomJ}					{Equivalence of natural unary “\haskell{NEG{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvNotNgAtomJ}					{Equivalence of natural unary “\haskell{NOT{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvCnvNgAtomJ}					{Equivalence of natural conversions}

\IndexTheorem		{eqvAddNgAtomJ}					{EQV_\plus N	Equivalence of natural \plus\phi operations}
\IndexTheorem		{eqvSubNgAtomJ}					{EQV_\minus N	Equivalence of natural \minus\phi operations}
\IndexTheorem		{eqvMulNgAtomJ}					{EQV_\times N	Equivalence of natural \times\phi operations}
\IndexTheorem		{eqvNegNgAtomJ}					{EQV_\lnot N	Equivalence of natural unary \minus\phi operations}
\IndexTheorem		{eqvNotNgAtomJ}					{EQV_\sim N	Equivalence of natural unary \sim\phi operations}
\IndexTheorem		{eqvCnvNgAtomJ}					{EQV_IN		Equivalence of natural conversions}

\set{capture}		{hidden}
\begin{haskell}
eqvAddNgAtomJ	:: forall x, y, epsilon =>
	WfJ{-sem-}(IMM (x) (NG`FMT`epsilon)) -> --
	WfJ{-sem-}(IMM (y) (NG`FMT`epsilon)) ->
	{-sem-}(OP2 (IMM (x) (NG`FMT`epsilon)) (ADD) (NG`FMT`epsilon) (IMM (y) (NG`FMT`epsilon))) --
		`EqvJ` {-sem-}(IMM ({-sem-}((x + y) `mod` (lubOfFormat{-sem-}(NG`FMT`epsilon) + 1))) (NG`FMT`epsilon))

eqvSubNgAtomJ	:: forall x, y, epsilon =>
	WfJ{-sem-}(IMM (x) (NG`FMT`epsilon)) -> --
	WfJ{-sem-}(IMM (y) (NG`FMT`epsilon)) ->
	{-sem-}(OP2 (IMM (x) (NG`FMT`epsilon)) (SUB) (NG`FMT`epsilon) (IMM (y) (NG`FMT`epsilon))) --
		`EqvJ` {-sem-}(IMM ({-sem-}((x - y) `mod` (lubOfFormat{-sem-}(NG`FMT`epsilon) + 1))) (NG`FMT`epsilon))

eqvMulNgAtomJ	:: forall x, y, epsilon =>
	WfJ{-sem-}(IMM (x) (NG`FMT`epsilon)) -> --
	WfJ{-sem-}(IMM (y) (NG`FMT`epsilon)) ->
	{-sem-}(OP2 (IMM (x) (NG`FMT`epsilon)) (MUL) (NG`FMT`epsilon) (IMM (y) (NG`FMT`epsilon))) --
		`EqvJ` {-sem-}(IMM ({-sem-}((x * y) `mod` (lubOfFormat{-sem-}(NG`FMT`epsilon) + 1))) (NG`FMT`epsilon))

eqvNegNgAtomJ	:: forall x, epsilon =>
	WfJ{-sem-}(IMM (x) (NG`FMT`epsilon)) -> 
	{-sem-}(OP1 (NEG) (NG`FMT`epsilon) {--} {-hv-}(IMM (x) (NG`FMT`epsilon))) --
		`EqvJ` {-sem-}(IMM ({-sem-}(negate x `mod` (lubOfFormat{-sem-}(NG`FMT`epsilon) + 1))) (NG`FMT`epsilon))

eqvNotNgAtomJ	:: forall x, epsilon =>
	WfJ{-sem-}(IMM (x) (NG`FMT`epsilon)) ->
	{-sem-}(OP1 (NOT) (NG`FMT`epsilon) {--} {-hv-}(IMM (x) (NG`FMT`epsilon))) --
		`EqvJ` {-sem-}(IMM ({-sem-}(lubOfFormat{-sem-}(NG`FMT`epsilon) - x)) (NG`FMT`epsilon))

eqvCnvNgAtomJ	:: forall x, phi, epsilon =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(NG`FMT`epsilon) -> --
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG]) ->
	{-sem-}(OP1 (CNV (NG`FMT`epsilon)) (phi) {--} {-hv-}(IMM (x) (phi))) --
		`EqvJ` {-sem-}(IMM ({-sem-}(x `mod` (lubOfFormat{-sem-}(NG`FMT`epsilon) + 1))) (NG`FMT`epsilon))
\end{haskell}

\NewHaskellToken	{eqvSubAgAtomJ}				{Ord}	{\TKeyword{eqv\sub{\minus A}}}
\NewHaskellToken	{eqvMulAgAtomJ}				{Ord}	{\TKeyword{eqv\sub{\times A}}}
\NewHaskellToken	{eqvDivAgAtomJ}				{Ord}	{\TKeyword{eqv\sub{\div A}}}
\NewHaskellToken	{eqvNegAgAtomJ}				{Ord}	{\TKeyword{eqv\sub{\lnot A}}}

\NewHaskellDescription	{eqvSubAgAtomJ}					{Equivalence of arithmetic “\haskell{SUB{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvMulAgAtomJ}					{Equivalence of arithmetic “\haskell{MUL{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvDivAgAtomJ}					{Equivalence of arithmetic “\haskell{DIV{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvNegAgAtomJ}					{Equivalence of arithmetic unary “\haskell{NEG{-sub-}phi}\!\!\!” operations}

\IndexTheorem		{eqvSubAgAtomJ}					{EQV_\minus A	Equivalence of arithmetic \minus\phi operations}
\IndexTheorem		{eqvMulAgAtomJ}					{EQV_\times A	Equivalence of arithmetic \times\phi operations}
\IndexTheorem		{eqvDivAgAtomJ}					{EQV_\div A	Equivalence of arithmetic \div\phi operations}
\IndexTheorem		{eqvNegAgAtomJ}					{EQV_\lnot A	Equivalence of arithmetic unary \minus\phi operations}

\set{capture}		{hidden}
\begin{haskell}
eqvSubAgAtomJ	:: forall x, y, z, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [ ZG, RG ] && glbOfFormat{--}(phi) <= x - y <= lubOfFormat{--}(phi)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (SUB) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM (z) (phi)) ->
	{-sem-}(abs (z - (x - y)) < ulpOfAtom(phi){--}(x - y))

eqvMulAgAtomJ	:: forall x, y, z, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [ ZG, RG ] && glbOfFormat{--}(phi) <= x * y <= lubOfFormat{--}(phi)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (MUL) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM (z) (phi)) ->
	{-sem-}(abs (z - (x * y)) < ulpOfAtom(phi){--}(x * y))

eqvDivAgAtomJ	:: forall x, y, z, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [ ZG, RG ] && y /= 0 && glbOfFormat{--}(phi) <= x / y <= lubOfFormat{--}(phi)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (DIV) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM (z) (phi)) ->
	{-sem-}(abs (z - (x / y)) < ulpOfAtom(phi){--}(x / y))

eqvNegAgAtomJ	:: forall x, z, phi =>
	WfJ{-sem-}(IMM (x) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [ ZG, RG ] && glbOfFormat{--}(phi) <= negate x <= lubOfFormat{--}(phi)) ->
	{-sem-}(OP1 (NEG) (phi) {--} {-hv-}(IMM (x) (phi))) `EqvJ` {-sem-}(IMM (z) (phi)) ->
	{-sem-}(abs (z - (negate x)) < ulpOfAtom(phi){--}(negate x))
\end{haskell}

\NewHaskellToken	{eqvDivIgAtomJ}				{Ord}	{\TKeyword{eqv\sub{\div I}}}
\NewHaskellToken	{eqvRemIgAtomJ}				{Ord}	{\TKeyword{eqv\sub{\rem I}}}

\NewHaskellDescription	{eqvDivIgAtomJ}					{Equivalence of integral “\haskell{DIV{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvRemIgAtomJ}					{Equivalence of integral “\haskell{REM{-sub-}phi}\!\!\!” operations}

\IndexTheorem		{eqvDivIgAtomJ}					{EQV_\div I	Equivalence of integral \div\phi operations}
\IndexTheorem		{eqvRemIgAtomJ}					{EQV_\rem I	Equivalence of integral \rem\phi operations}

\set{capture}		{hidden}
\begin{haskell}
eqvDivIgAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG] && x >= 0 && y > 0) ->
	{-sem-}(OP2 (IMM (x) (phi)) (DIV) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM ({-sem-}(floor (x / y))) (phi))

eqvRemIgAtomJ	:: forall x, y, z, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(IMM (y) (phi)) ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG] && y /= 0 && glbOfFormat{--}(phi) <= x - y*z <= lubOfFormat{--}(phi)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (DIV) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM (z) (phi)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (REM) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM ({-sem-}(x - y*z)) (phi))
\end{haskell}

\NewHaskellToken	{eqvCnvRgAtomJ}				{Ord}	{\TKeyword{eqv\sub{RA}}}
\NewHaskellToken	{eqvCnvIgAtomJ}				{Ord}	{\TKeyword{eqv\sub{AI}}}
\NewHaskellToken	{eqvCnvPgAtomJ}				{Ord}	{\TKeyword{eqv\sub{PP}}}

\NewHaskellDescription	{eqvCnvRgAtomJ}					{Equivalence of rational conversions}
\NewHaskellDescription	{eqvCnvIgAtomJ}					{Equivalence of integral conversions}
\NewHaskellDescription	{eqvCnvPgAtomJ}					{Equivalence of pointer conversions}

\IndexTheorem		{eqvCnvRgAtomJ}					{EQV_RA		Equivalence of rational conversions}
\IndexTheorem		{eqvCnvIgAtomJ}					{EQV_AI		Equivalence of integral conversions}
\IndexTheorem		{eqvCnvPgAtomJ}					{EQV_PP		Equivalence of pointer conversions}

\set{capture}		{hidden}
\begin{haskell}
eqvCnvRgAtomJ	:: forall x, z, phi, phi' =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(phi') ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG, RG] && --
		genreOfFormat{--}(phi') == {-sem-}(RG) && --
		glbOfFormat{--}(phi') <= x <= lubOfFormat{--}(phi')) ->
	{-sem-}(OP1 (CNV phi') (phi) {--} {-hv-}(IMM (x) (phi))) `EqvJ` {-sem-}(IMM (z) (phi')) ->
	{-sem-}(abs (z - x) < ulpOfAtom(phi'){--}(x))

eqvCnvIgAtomJ	:: forall x, phi, phi' =>
	WfJ{-sem-}(IMM (x) (phi)) -> WfJ{-sem-}(phi') ->
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG, RG] && --
		genreOfFormat{--}(phi') @@ listToSet [NG, ZG] && --
		glbOfFormat{--}(phi') <= truncate (x) <= lubOfFormat{--}(phi')) ->
	{-sem-}(OP1 (CNV phi') (phi) {--} {-hv-}(IMM (x) (phi))) `EqvJ` {-sem-}(IMM ({-sem-}(truncate (x))) (phi'))

eqvCnvPgAtomJ	:: forall phi, phi' =>
	WfJ{--}(phi) -> WfJ{--}(phi') ->
	{-sem-}(genreOfFormat{--}(phi) /= {-sem-}(RG) && genreOfFormat{--}(phi') /= {-sem-}(RG)) ->
	{-sem-}(OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(IMM (0) (phi))) `EqvJ` {-sem-}(IMM (0) (phi'))
\end{haskell}

\NewHaskellToken	{eqvAndAtomJ}				{Ord}	{\TKeyword{eqv\sub{\bitand}}}
\NewHaskellToken	{eqvXorAtomJ}				{Ord}	{\TKeyword{eqv\sub{\bitxor}}}
\NewHaskellToken	{eqvIorAtomJ}				{Ord}	{\TKeyword{eqv\sub{\bitor}}}

\NewHaskellDescription	{eqvAndAtomJ}					{Equivalence of “\haskell{AND{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvXorAtomJ}					{Equivalence of “\haskell{XOR{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvIorAtomJ}					{Equivalence of “\haskell{IOR{-!-}{-!-}{-sub-}phi}\!\!\!” operations}

\IndexTheorem		{eqvAndAtomJ}					{EQV_\bitand	Equivalence of \bitand\phi operations}
\IndexTheorem		{eqvXorAtomJ}					{EQV_\bitxor	Equivalence of \bitxor\phi operations}
\IndexTheorem		{eqvIorAtomJ}					{EQV_\bitor	Equivalence of \bitor\phi operations}

\set{capture}		{hidden}
\begin{haskell}
eqvAndAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) -> --
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG] && x >= 0 && y >= 0) ->
	{-sem-}(OP2 (IMM (x) (phi)) (AND) (phi) (IMM (y) (phi))) --
		`EqvJ` {-sem-}(IMM ({-sem-}(sum [ (x`bit`(k) && y`bit`(k)) * 2^i | k <- [0..widthOfFormat{--}(phi) - 1]])) (phi))

eqvXorAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) -> --
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG] && x >= 0 && y >= 0) ->
	{-sem-}(OP2 (IMM (x) (phi)) (XOR) (phi) (IMM (y) (phi))) --
		`EqvJ` {-sem-}(IMM ({-sem-}(sum [ (x`bit`(k) /= y`bit`(k)) * 2^i | k <- [0..widthOfFormat{--}(phi) - 1]])) (phi))

eqvIorAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) -> --
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG] && x >= 0 && y >= 0) ->
	{-sem-}(OP2 (IMM (x) (phi)) (IOR) (phi) (IMM (y) (phi))) --
		`EqvJ` {-sem-}(IMM ({-sem-}(sum [ (x`bit`(k) || y`bit`(k)) * 2^i | k <- [0..widthOfFormat{--}(phi) - 1]])) (phi))
\end{haskell}

\NewHaskellToken	{eqvShlNgAtomJ}				{Ord}	{\TKeyword{eqv\sub{\ll N}}}
\NewHaskellToken	{eqvShlZgAtomJ}				{Ord}	{\TKeyword{eqv\sub{\ll Z}}}
\NewHaskellToken	{eqvShrAtomJ}				{Ord}	{\TKeyword{eqv\sub{\gg}}}

\NewHaskellDescription	{eqvShlNgAtomJ}					{Equivalence of natural “\haskell{SHL{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvShlZgAtomJ}					{Equivalence of integer “\haskell{SHL{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvShrAtomJ}					{Equivalence of “\haskell{SHR{-sub-}phi}\!\!\!” operations}

\IndexTheorem		{eqvShlNgAtomJ}					{EQV_\ll N	Equivalence of natural \ll\phi operations}
\IndexTheorem		{eqvShlZgAtomJ}					{EQV_\ll Z	Equivalence of integer \ll\phi operations}
\IndexTheorem		{eqvShrAtomJ}					{EQV_\gg	Equivalence of \gg\phi operations}

\set{capture}		{hidden}
\begin{haskell}
eqvShlNgAtomJ	:: forall x, y, epsilon =>
	WfJ{-sem-}(IMM (x) (NG`FMT`epsilon)) -> --
	WfJ{-sem-}(IMM (y) (NG`FMT`epsilon)) -> --
	{-sem-}(0 <= y < widthOfFormat{-sem-}(NG`FMT`epsilon)) ->
	{-sem-}(OP2 (IMM (x) (NG`FMT`epsilon)) (SHL) (NG`FMT`epsilon) (IMM (y) (NG`FMT`epsilon))) --
		`EqvJ` {-sem-}(IMM ({-sem-}((x * 2^y) `mod` (lubOfFormat{-sem-}(NG`FMT`epsilon) + 1))) (NG`FMT`epsilon))

eqvShlZgAtomJ	:: forall x, y, epsilon =>
	WfJ{-sem-}(IMM (x) (ZG`FMT`epsilon)) -> --
	WfJ{-sem-}(IMM (y) (ZG`FMT`epsilon)) -> --
	{-sem-}(0 <= x * 2^y <= lubOfFormat{--}(ZG`FMT`epsilon) && 0 <= y < widthOfFormat{-sem-}(ZG`FMT`epsilon)) ->
	{-sem-}(OP2 (IMM (x) (ZG`FMT`epsilon)) (SHL) (ZG`FMT`epsilon) (IMM (y) (ZG`FMT`epsilon))) --
		`EqvJ` {-sem-}(IMM ({-sem-}(x * 2^y)) (ZG`FMT`epsilon))

eqvShrAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) -> --
	{-sem-}(genreOfFormat{--}(phi) @@ listToSet [NG, ZG] && x >= 0 && 0 <= y < widthOfFormat{--}(phi)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (SHR) (phi) (IMM (y) (phi))) --
		`EqvJ` {-sem-}(IMM ({-sem-}(floor (x / 2^y))) (phi))
\end{haskell}

\NewHaskellToken	{eqvCeqAtomJ}				{Ord}	{\TKeyword{eqv\sub{\eq}}}
\NewHaskellToken	{eqvCneAtomJ}				{Ord}	{\TKeyword{eqv\sub{\ne}}}

\NewHaskellDescription	{eqvCeqAtomJ}					{Equivalence of “\haskell{CEQ{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvCneAtomJ}					{Equivalence of “\haskell{CNE{-sub-}phi}\!\!\!” operations}

\IndexTheorem		{eqvCeqAtomJ}					{EQV_\eq	Equivalence of \eq\phi operations}
\IndexTheorem		{eqvCneAtomJ}					{EQV_\ne	Equivalence of \ne\phi operations}

\set{capture}		{hidden}
\begin{haskell}
eqvCeqAtomJ	:: forall x, y, phi => ---
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) -> --
	{-sem-}(OP2 (IMM (x) (phi)) (CEQ) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM ({-sem-}(x == y)) (ZG`FMT`standardEncoding))
eqvCneAtomJ	:: forall x, y, phi => ---
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) -> --
	{-sem-}(OP2 (IMM (x) (phi)) (CNE) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM ({-sem-}(x /= y)) (ZG`FMT`standardEncoding))
\end{haskell}

\NewHaskellToken	{eqvCltAtomJ}				{Ord}	{\TKeyword{eqv\sub{\lt}}}
\NewHaskellToken	{eqvCgtAtomJ}				{Ord}	{\TKeyword{eqv\sub{\gt}}}
\NewHaskellToken	{eqvCleAtomJ}				{Ord}	{\TKeyword{eqv\sub{\le}}}
\NewHaskellToken	{eqvCgeAtomJ}				{Ord}	{\TKeyword{eqv\sub{\ge}}}

\NewHaskellDescription	{eqvCltAtomJ}					{Equivalence of “\haskell{CLT{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvCgtAtomJ}					{Equivalence of “\haskell{CGT{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvCleAtomJ}					{Equivalence of “\haskell{CLE{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvCgeAtomJ}					{Equivalence of “\haskell{CGE{-sub-}phi}\!\!\!” operations}

\IndexTheorem		{eqvCltAtomJ}					{EQV_\lt	Equivalence of \lt\phi operations}
\IndexTheorem		{eqvCgtAtomJ}					{EQV_\gt	Equivalence of \gt\phi operations}
\IndexTheorem		{eqvCleAtomJ}					{EQV_\le	Equivalence of \le\phi operations}
\IndexTheorem		{eqvCgeAtomJ}					{EQV_\ge	Equivalence of \ge\phi operations}

\set{capture}		{hidden}
\begin{haskell}
eqvCltAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) -> --
	{-sem-}(genreOfFormat{--}(phi) /= {-sem-}(FG)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (CLT) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM ({-sem-}(x < y)) (ZG`FMT`standardEncoding))

eqvCgtAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) -> --
	{-sem-}(genreOfFormat{--}(phi) /= {-sem-}(FG)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (CGT) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM ({-sem-}(x > y)) (ZG`FMT`standardEncoding))

eqvCleAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) -> --
	{-sem-}(genreOfFormat{--}(phi) /= {-sem-}(FG)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (CLE) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM ({-sem-}(x <= y)) (ZG`FMT`standardEncoding))

eqvCgeAtomJ	:: forall x, y, phi =>
	WfJ{-sem-}(IMM (x) (phi)) -> --
	WfJ{-sem-}(IMM (y) (phi)) -> --
	{-sem-}(genreOfFormat{--}(phi) /= {-sem-}(FG)) ->
	{-sem-}(OP2 (IMM (x) (phi)) (CGE) (phi) (IMM (y) (phi))) `EqvJ` {-sem-}(IMM ({-sem-}(x >= y)) (ZG`FMT`standardEncoding))
\end{haskell}

\NewHaskellToken	{eqvAddZeroAtomJ}			{Ord}	{\TKeyword{eqv\sub{\plus0}}}
\NewHaskellToken	{eqvAddOgAtomJ}				{Ord}	{\TKeyword{eqv\sub{\plus O}}}
\NewHaskellToken	{eqvSubOgAtomJ}				{Ord}	{\TKeyword{eqv\sub{\minus O}}}
\NewHaskellToken	{eqvCnvOgAtomJ}				{Ord}	{\TKeyword{eqv\sub{OO}}}

\NewHaskellDescription	{eqvAddZeroAtomJ}				{Equivalence of object atoms}
\NewHaskellDescription	{eqvAddOgAtomJ}					{Equivalence of object “\haskell{ADD{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvSubOgAtomJ}					{Equivalence of object “\haskell{SUB{-sub-}phi}\!\!\!” operations}
\NewHaskellDescription	{eqvCnvOgAtomJ}					{Equivalence of object conversions}

\IndexTheorem		{eqvAddZeroAtomJ}				{EQV_\plus0	Equivalence of object atoms}
\IndexTheorem		{eqvAddOgAtomJ}					{EQV_\plus O	Equivalence of object \plus\phi operations}
\IndexTheorem		{eqvSubOgAtomJ}					{EQV_\minus O	Equivalence of object \minus\phi operations}
\IndexTheorem		{eqvCnvOgAtomJ}					{EQV_OO		Equivalence of object conversions}

\set{capture}		{hidden}
\begin{haskell}
eqvAddZeroAtomJ :: forall x, epsilon =>
	WfJ{-sem-}(IMM (x) (OG`FMT`epsilon)) ->
	{-sem-}(IMM (x) (OG`FMT`epsilon)) `EqvJ` {-sem-}(OP2 (IMM (x) (OG`FMT`epsilon)) (ADD) (OG`FMT`epsilon) (IMM (0) (ZG`FMT`standardEncoding)))
	
eqvAddOgAtomJ :: forall x, y, z, epsilon =>
	WfJ{-sem-}(OP2 (IMM (x) (OG`FMT`epsilon)) (ADD) (OG`FMT`epsilon) (IMM (y) (ZG`FMT`standardEncoding))) ->
	WfJ{-sem-}(OP2 (IMM (y) (ZG`FMT`standardEncoding)) (ADD) (ZG`FMT`standardEncoding) (IMM (z) (ZG`FMT`standardEncoding))) ->
	{-sem-}(OP2 ({-hv-}(OP2 (IMM (x) (OG`FMT`epsilon)) (ADD) (OG`FMT`epsilon) (IMM (y) (ZG`FMT`standardEncoding)))) --
			(ADD) (OG`FMT`epsilon) (IMM (z) (ZG`FMT`standardEncoding))) `EqvJ` --
	{-sem-}(OP2 (IMM (x) (OG`FMT`epsilon)) (ADD) (OG`FMT`epsilon) --
			({-hv-}(OP2 (IMM (y) (ZG`FMT`standardEncoding)) (ADD) (ZG`FMT`standardEncoding) (IMM (z) (ZG`FMT`standardEncoding)))))

eqvSubOgAtomJ :: forall x, y, z, epsilon =>
	WfJ{-sem-}(OP2 (IMM (x) (OG`FMT`epsilon)) (ADD) (OG`FMT`epsilon) (IMM (y) (ZG`FMT`standardEncoding))) ->
	WfJ{-sem-}(OP2 (IMM (x) (OG`FMT`epsilon)) (ADD) (OG`FMT`epsilon) (IMM (z) (ZG`FMT`standardEncoding))) ->
	WfJ{-sem-}(OP2 (IMM (y) (ZG`FMT`standardEncoding)) (SUB) (ZG`FMT`standardEncoding) (IMM (z) (ZG`FMT`standardEncoding))) ->
	{-sem-}(OP2 ({-hv-}(OP2 (IMM (x) (OG`FMT`epsilon)) (ADD) (OG`FMT`epsilon) (IMM (y) (ZG`FMT`standardEncoding)))) --
			(SUB) (OG`FMT`epsilon) ({-hv-}(OP2 (IMM (x) (OG`FMT`epsilon)) (ADD) (OG`FMT`epsilon) (IMM (z) (ZG`FMT`standardEncoding))))) `EqvJ` --
	{-sem-}(OP2 (IMM (y) (ZG`FMT`standardEncoding)) (SUB) (ZG`FMT`standardEncoding) (IMM (z) (ZG`FMT`standardEncoding)))

eqvCnvOgAtomJ :: forall x, epsilon =>
	WfJ{-sem-}(IMM (x) (OG`FMT`epsilon)) ->
	{-sem-}(IMM (x) (OG`FMT`epsilon)) `EqvJ` {-sem-}(OP1 (CNV (OG`FMT`epsilon)) (OG`FMT`standardEncoding) --
								({-hv-}(OP1 (CNV (OG`FMT`standardEncoding)) (OG`FMT`epsilon) (IMM (x) (OG`FMT`epsilon)))))
\end{haskell}
\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Lambda/Arch/State}
\set{title}{Evaluation State}
\begin{subsection}

In Section \ref{Lambda/State}, the object environment of an Etude program was represented by an abstract type “"OEnvT"”, whose properties were described solely through a set of suitable operations on such environments. It is now time to rectify this situation and specify the actual structure of the address space made available to all MMIX programs. Formally, this structure is defined by the following three Haskell types:

\IndexType		{OEnvT}						{o_env (\tstar)}
\IndexType		{M}						{M (\tstar)}
\IndexType		{A}						{A (\tstar)}

\begin{syntax}
\InsertHaskell		{OEnvT}
\end{syntax}

In other words, every object environment depicts a tuple \hbox{"(_M, _A, xi_set, xi_set_i, psi_set, sigma_i, sigma_c, sigma_f)",} in which the finite map "_M", known as the program's «memory image», defines the actual bindings of individual addresses "sigma" to their respective byte values stored at the corresponding memory locations. The current atomic content of a given object "(sigma, phi)" can be retrieved from "_M" by the construction "atomOfOEnv{--}(_Delta {->-}, sigma, phi)", which, on MMIX, is defined as follows:

\BeginHaskellToken	{atomOfOEnv}				{Open}	{«\macron \alpha»}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{(OEnvT {->-}, Integer, FormatT)}
  \HaskellResult							{AtomT nu}
\EndHaskellToken

\IndexTerm		{atomOfOEnv}					{\alpha- o_env integer format (atom)}

\begin{code}
atomOfOEnv {-[*]-} {-[*]-} :: (Ord nu) => (OEnvT {->-}, Integer, FormatT) -> AtomT nu

atomOfOEnv {-sem-}({-raw-}(_M, _A, xi_set, xi_set_i, psi_set, sigma_i, sigma_c, sigma_f) {->-}, sigma, phi) ---
  | phi @@ listToSet [ NG`FMT`8, NG`FMT`16, NG`FMT`32, NG`FMT`64, RG`FMT`64 ] --
  		= {-sem-}(IMM (toRational x) (NG`FMT`64))
  | otherwise	= evalAtom{-sem-}(OP1 (CNV phi) (NG`FMT`64) {--} {-hv-}(IMM (toRational x) (NG`FMT`64)))
  
 where	x   		= sum [ 2^(byteWidth #*# k) #*# _M#(sigma' + sizeOfFormat{--}(phi) - 1 - k) | k <- [ 0 .. sizeOfFormat{--}(phi) - 1 ] ]
 	sigma'  	= sigma - sigma `mod` sizeOfFormat{--}(phi)
\end{code}

Intuitively, the above definition renders MMIX into a big-endian architecture, whereby the proper encoded value of the memory-resident object requested by "atomOfOEnv{--}(_Delta {->-}, sigma, phi)" is assembled from the individual bytes of the corresponding memory image "_M", with the most significant 8 bits of the object fetched from the least address in the region. Unless "phi" specifies one of five blessed formats “"NG`FMT`8"”, “"NG`FMT`16"”, “"NG`FMT`32"”, “"NG`FMT`64"” or “"RG`FMT`64"”, the resulting natural atom is always converted into "phi", as if by the reduction of an appropriate term of the form “"OP1 (CNV phi) (NG`FMT`64) {--} {-hv-}(IMM (toRational x) (NG`FMT`64))"”. The reader should note that, unlike most other modern instruction set architectures, MMIX does not assume that the supplied address "sigma" represents a multiple of the format's size and always adjusts any misaligned addresses, implicitly discarding the least-significant 3, 2, 1 or 0 bits of "sigma" if required.

Further, the two envelopes "xi_set" and "xi_set_i" found in every MMIX object environment "(_M, _A, xi_set, xi_set_i, psi_set, sigma_i, sigma_c, sigma_f)" correspond directly to the similarly-named functions from Section \ref{Lambda/State}:

\BeginHaskellToken	{envelopeOfOEnv}			{Open}	{«\superimpose{\raise.5ex\hbox{\,\macron{}}}{\xi}»}
  \HaskellArgument							{OEnvT}
  \HaskellResult							{EnvelopeT}
\EndHaskellToken

\BeginHaskellToken	{initOfOEnv}				{Open}	{«\superimpose{\raise.5ex\hbox{\,\macron{}}}{\xi}\sub{i}»}
  \HaskellArgument							{OEnvT}
  \HaskellResult							{EnvelopeT}
\EndHaskellToken

\BeginHaskellToken	{stackOfOEnv}				{Open}	{«\superimpose{\,\macron{}}{\psi}»}
  \HaskellArgument							{OEnvT}
  \HaskellResult							{StackT}
\EndHaskellToken

\IndexTerm		{envelopeOfOEnv}				{\xi- o_env (envelope)}
\IndexTerm		{initOfOEnv}					{\xi-i o_env (envelope)}
\IndexTerm		{stackOfOEnv}					{\psi o_env (stack)}

\begin{code}
envelopeOfOEnv 	{-[*]-}, initOfOEnv {-[*]-} :: OEnvT -> EnvelopeT

envelopeOfOEnv	{-sem-}(_M, _A, xi_set, xi_set_i, psi_set, sigma_i, sigma_c, sigma_f) = xi_set
initOfOEnv 	{-sem-}(_M, _A, xi_set, xi_set_i, psi_set, sigma_i, sigma_c, sigma_f) = xi_set_i

{-{ while the environment's stack "stackOfOEnv{--}(_Delta)" is, by definition, equal to its component "psi_set"\,: }-}

stackOfOEnv	{-[*]-} :: OEnvT -> StackT

stackOfOEnv 	{-sem-}(_M, _A, xi_set, xi_set_i, psi_set, sigma_i, sigma_c, sigma_f) = psi_set
\end{code}

In practice, none of these three object environment components are used directly on the MMIX architecture, so that their inclusion in the structure of “\!"OEnvT"” has been effected only to facilitate identification of various memory allocation patterns that are considered illegal by the standard of the ultimate mapping between Etude programs and their MMIX representations in Section \ref{MMIX/Lambda}. Instead, the actual structure of the program's address space is defined solely by the environment's «address space» component "_A", which associates every byte address "sigma" accessible to the program with a boolean flag "_A#(sigma)", set to a true value if and only if the underlying operating system has granted our program the right to modify the content of the byte object located at that address. Within the domain of "_A", a smaller range of consecutive addresses "[sigma_f .. sigma_i]" is reserved for sole use by the program's data stack "stackOfOEnv{--}(_Delta)" and, further, the «current stack pointer» component "sigma_c" determines the least stack address that is currently utilised by the program. Accordingly, the construction "newInOEnv{--}(_Delta {->-}, xi_set)" that, in Section \ref{Lambda/State}, determined the location of a new envelope "xi_set" upon its introduction into the address space by the “"NEW (xi_set)"” term form, is defined on MMIX as follows:

\BeginHaskellToken	{newInOEnv}				{Open}	{«\sigma\sub{c}»}
  \HaskellArgument							{(OEnvT {->-}, EnvelopeT)}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{newInOEnv}					{\sigma_c o_env envelope (integer)}

\begin{code}
newInOEnv {-[*]-} :: (OEnvT {->-}, EnvelopeT) -> Integer

newInOEnv {-sem-}({-raw-}(_M, _A, xi_set, xi_set_i, psi_set, sigma_i, sigma_c, sigma_f) {->-}, xi_set') = sigma_c - ceiling (sizeOfEnvelope{--}(xi_set') % 8) * 8
\end{code}

so that each newly introduced stack frame is always placed below any that are already present in the program's address space and is assigned a memory location aligned to the architecture's word boundary of 8.

In Section \ref{Lambda/Modules}, numeric address values were derived from their atomic representations using the implementation-defined constructs "atomToObject{--}(alpha)" and "atomToFunction{--}(alpha)". On MMIX, both of these functions can be implemented by decoding the normalised form of "alpha" \hbox{as follows:}

\BeginHaskellToken	{atomToObject}				{Open}	{\ZapfChancery{L\sub{\LKeyword{O}}}}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{AtomT nu}
  \HaskellResult							{Integer}
\EndHaskellToken

\BeginHaskellToken	{atomToFunction}			{Open}	{\ZapfChancery{L\sub{\LKeyword{F}}}}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{AtomT nu}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{atomToObject}					{L_O atom (integer)}
\IndexTerm		{atomToFunction}				{L_F atom (integer)}

\begin{code}
atomToObject 	{-[*]-}, atomToFunction {-[*]-} :: (Ord nu) => AtomT nu -> Integer

atomToObject 	{-sem-}(alpha) = discardFraction (decodeAtom({-sem-}(OG`FMT`64)){--}(evalAtom{--}(alpha)))
atomToFunction 	{-sem-}(alpha) = discardFraction (decodeAtom({-sem-}(FG`FMT`64)){--}(evalAtom{--}(alpha)))
\end{code}

Throughout its entire execution, every well-formed Etude program must ensure that its environment "(_M, _A, xi_set, xi_set_i, psi_set, sigma_i, sigma_c, sigma_f)" always satisfies the following nine invariants:

\begin{list}

•	The envelope "xi_set" must be well-formed.

•	The initialiser envelope "xi_set_i" must represent a subset of "xi_set".

•	All frames of the stack "psi_set" must be allocated in "xi_set".

•	All frames of the stack "psi_set" must be allocated within the region \hbox{"[ sigma_f .. sigma_i ]"} that is reserved for this purpose by the operating system.

•	The stack pointer "sigma_c" must also fall within that address region, with "sigma_c `mod` 8 == 0", so that "sigma_c" represents a value of a well-formed atom under the object format “"OG`FMT`61"”.

•	Every address "sigma @@ dom{--}(_A)" must have a value in the range "0 <= sigma < 2^63", so that "sigma" depicts a well-formed numeric value of an Etude atom under the “"OG`FMT`64"” format. If "_A#(sigma)" is false, then the address must also appear in the domain of "_M", in order to ensure that the contents of all immutable memory objects are always prepopulated in the program's memory image "_M".

•	Every address "sigma @@ dom{--}(_M)" must be also mapped in "_A", with "0 <= _M#(sigma) <= 255".

•	Every address "sigma @@ [ sigma_f .. sigma_i ]" must be identified by "_A" as a modifiable.

•	Finally, for each envelope element "{-hv-}(sigma_k {-hv-}, phi_k {-hv-}, mu_set_k) @@ xi_set", every address "sigma" in the set "listToSet [sigma_k .. sigma_k + sizeOfFormat{--}(phi_k) - 1]" must always appear in the domain of the address space "_A" and, if "_A#(sigma)" is false for at least one such value of "sigma", then the set of memory access attributes "mu_set_k" associated with "sigma" must contain the access attribute “"CM"”.

\end{list}

Intuitively, these nine requirements are intended to provide the compiler with sufficient information about the behaviour of well-formed programs to facilitate an efficient transition between their Etude and native MMIX representations, as required for establishment of the linear correctness property in Section \ref{MMIX/Lambda}. The reader should observe that, under the above conditions, the three environment components "xi_set", "xi_set_i" "psi_set" echo all of the useful information stored in "_M" and "_I". In fact, as we shall soon discover, well-formed MMIX programs may, under certain conditions, discard some of the less interesting data from "xi_set" and "psi_set", in order to admit various useful optimising transformations of many common Etude term forms. However, any program that invokes such behaviour is inherently unportable and must be always scrutinised in the context of the specific MMIX implementation of Etude. Formally, the well-formedness property “"WfOEnvJ :: OEnvT -> Prop"” is defined for MMIX object environments as follows:

\BeginHaskellToken	{WfOEnvJ}				{Open}	{\TKeyword{wf}}
  \HaskellArgument							{OEnvT}
  \HaskellResult							{Prop}
\EndHaskellToken

\IndexProp		{WfOEnvJ}					{WF o_env (\pstar)}

\NewHaskellToken	{WfOEnvC}				{Ord}	{\TKeyword{wf\sub{\Delta}}}
\NewHaskellDescription	{WfOEnvC}					{Validity of object environments}
\IndexAxiom		{WfOEnvC}					{WF_\Delta Validity of object environments}

\begin{haskell}
data WfOEnvJ {-[*]-} :: OEnvT -> Prop

 where	WfOEnvC :: forall _M, _A, xi_set, xi_set_i, psi_set, sigma_i, sigma_c, sigma_f =>
 	{-quad-}	WfJ{--}(xi_set) ->
 			{-sem-}(xi_set_i `subseteq` xi_set) ->
 			{-sem-}(unions [ xi_set_k `shiftEnvelope` sigma_k | (sigma_k {-hv-}, xi_set_k) <- psi_set ] `subseteq` xi_set) ->
 			{-sem-}(unions [ listToSet [ sigma_k .. sizeOfEnvelope{--}(xi_set_k) - 1 ] | (sigma_k {-hv-}, xi_set_k) <- psi_set ] --
 															`subseteq` listToSet [ sigma_f .. sigma_i ]) ->
			{-sem-}(sigma_f <= sigma_c <= sigma_i && sigma_c `mod` 8 == 0) ->
			(forall sigma => {-sem-}(sigma @@ dom{--}(_A)) -> {-sem-}(0 <= sigma < 2^63 && (_A#(sigma) || sigma @@ dom{--}(_M)))) ->
			(forall sigma => {-sem-}(sigma @@ dom{--}(_M)) -> {-sem-}(sigma @@ dom{--}(_A) && 0 <= _M#(sigma) <= 255)) ->
			(forall sigma => {-sem-}(sigma_f <= sigma <= sigma_i) -> {-sem-}(_A#(sigma))) ->
			(forall sigma_k, phi_k, mu_set_k =>
			{-quad-}	{-sem-}({-hv-}(sigma_k {-hv-}, phi_k {-hv-}, mu_set_k) @@ xi_set && sigma_k <= sigma < sigma_k + sizeOfFormat{--}(phi_k)) ->
					(sigma @@ dom{--}(_A) && (_A#(sigma) || {-sem-}(CM) @@ mu_set_k))) ->
			WfOEnvJ{-sem-}(_M, _A, xi_set, psi_set, sigma_i, sigma_c, sigma_f)
\end{haskell}

\set{label}		{Lambda/Arch/envelopeOfOEnvJ}
\set{title}		{Validity of object environment envelopes and stacks}
\begin{theorem}

Every well-formed object environment "_Delta" must always specify a valid envelope "envelopeOfOEnv{--}(_Delta)" that includes at least all of the envelope elements from "initOfOEnv{--}(_Delta)", as well as any addresses that are mentioned by the individual stack frames of "stackOfOEnv{--}(_Delta)":

\NewHaskellToken	{envelopeOfOEnvJ}			{Ord}	{\TKeyword{env\sub{e}}}
\NewHaskellToken	{initOfOEnvJ}				{Ord}	{\TKeyword{env\sub{i}}}
\NewHaskellToken	{stackOfOEnvJ}				{Ord}	{\TKeyword{env\sub{s}}}

\NewHaskellDescription	{envelopeOfOEnvJ}				{Validity of object environment envelopes} 
\NewHaskellDescription	{initOfOEnvJ}					{Validity of object environment initialiser envelopes}
\NewHaskellDescription	{stackOfOEnvJ}					{Validity of object environment stacks}

\IndexTheorem		{envelopeOfOEnvJ}				{ENV_E Validity of object environment envelopes} 
\IndexTheorem		{initOfOEnvJ}					{ENV_I Validity of object environment initialiser envelopes} 
\IndexTheorem		{stackOfOEnvJ}					{ENV_S Validity of object environment stacks}

\begin{haskell}
envelopeOfOEnvJ 	:: forall _Delta => WfJ{--}(_Delta) -> WfJ{-sem-}(envelopeOfOEnv{--}(_Delta))
initOfOEnvJ 		:: forall _Delta => WfJ{--}(_Delta) -> {-sem-}(initOfOEnv{--}(_Delta) `subseteq` envelopeOfOEnv{--}(_Delta))
stackOfOEnvJ 		:: forall _Delta => WfJ{--}(_Delta) -> {-sem-}(unions [ xi_set_k `shiftEnvelope` sigma_k | (sigma_k {-hv-}, xi_set_k) <- stackOfOEnv{--}(_Delta) ] --
									`subseteq` envelopeOfOEnv{--}(_Delta))
\end{haskell}
\begin{proof}

Directly, by the earlier definition of "WfJ :: OEnvT -> Prop". \QED

\end{proof}
\end{theorem}

The environment extension operation "_Delta `extendOEnv` xi_set" can be modelled quite easily on MMIX, since it never affects the actual configuration of the address space "_A" or the associated memory image "_M". Instead, the current stack pointer "sigma_c" is assigned directly the required value of "newInOEnv{--}(_Delta {->-}, xi_set)" and, in order to satisfy the requirements from Section \ref{Lambda/State}, "xi_set" is also pushed onto the stack "stackOfOEnv{--}(_Delta)" and incorporated into the address space envelopes "envelopeOfOEnv{--}(_Delta)" and "initOfOEnv{--}(_Delta)" as follows:

\BeginHaskellToken	{extendOEnv}				{Void}	{}
  \HaskellArgument							{OEnvT}
  \HaskellToken		{{-/-}}					{Bin}	{\slash}
  \HaskellArgument							{EnvelopeT}
  \HaskellResult							{OEnvT}
\EndHaskellToken

\NewHaskellToken	{{-/-}}					{Op}	{\slash}
\NewHaskellToken	{`extendOEnv`}				{Op}	{\slash}

\IndexTerm		{extendOEnv}					{\slash o_env envelope (o_env)}

\begin{code}
extendOEnv {-[*]-} {-/-} {-[*]-} :: OEnvT -> EnvelopeT -> OEnvT

extendOEnv {-sem-}(_M, _A, xi_set, xi_set_i, psi_set, sigma_i, sigma_c, sigma_f) {-/-} {-sem-}(xi_set' {-,-})
{--}		  = {-sem-}(_M, _A, xi_set `union` (xi_set' `shiftEnvelope` sigma_c'), xi_set_i `union` (xi_set' `shiftEnvelope` sigma_c'), --
					{-raw-}[ {-hv-}(sigma_c' {-hv-}, xi_set') ] ++ psi_set, sigma_i, sigma_c', sigma_f)
 where	sigma_c'  = newInOEnv {-sem-}({-raw-}(_M, _A, xi_set, xi_set_i, psi_set, sigma_i, sigma_c, sigma_f) {->-}, xi_set')
\end{code}

\set{label}		{Lambda/Arch/extendOEnvJ}
\set{title}		{Algebraic semantics of object environment extensions}
\begin{theorem}

The environment extension operation "_Delta `extendOEnv` xi_set" preserves the semantics of "_Delta" as follows:

\NewHaskellToken	{extendOEnvStackJ}			{Ord}	{\TKeyword{ext\sub{s}}}
\NewHaskellDescription	{extendOEnvStackJ}				{Object environment stack after extension}
\IndexTheorem		{extendOEnvStackJ}				{EXT_S Object environment stack after extension}

\NewHaskellToken	{extendOEnvEnvelopeJ}			{Ord}	{\TKeyword{ext\sub{e}}}
\NewHaskellDescription	{extendOEnvEnvelopeJ}				{Object environment envelope after extension}
\IndexTheorem		{extendOEnvEnvelopeJ}				{EXT_E Object environment envelope after extension}

\NewHaskellToken	{extendOEnvInitJ}			{Ord}	{\TKeyword{ext\sub{i}}}
\NewHaskellDescription	{extendOEnvInitJ}				{Object environment initialisers after extension}
\IndexTheorem		{extendOEnvInitJ}				{EXT_I Object environment initialisers after extension}

\NewHaskellToken	{extendOEnvAtomsJ}			{Ord}	{\TKeyword{ext\sub{a}}}
\NewHaskellDescription	{extendOEnvAtomsJ}				{Object environment data after extension}
\IndexTheorem		{extendOEnvAtomsJ}				{EXT_A Object environment data after extension}

\begin{haskell}
extendOEnvStackJ	:: forall _Delta, xi_set => --
	WfJ{--}(_Delta) -> WfJ{--}(_Delta `extendOEnv` xi_set) -> --
	{-sem-}(stackOfOEnv	{--}(_Delta `extendOEnv` xi_set) == {-raw-}[{-hv-}(newInOEnv{--}(_Delta {->-}, xi_set) {-hv-}, xi_set)] ++ stackOfOEnv{--}(_Delta))
extendOEnvEnvelopeJ	:: forall _Delta, xi_set => --
	WfJ{--}(_Delta) -> WfJ{--}(_Delta `extendOEnv` xi_set) -> --
	{-sem-}(envelopeOfOEnv	{--}(_Delta `extendOEnv` xi_set) == envelopeOfOEnv	{--}(_Delta) `union` (xi_set `shiftEnvelope` newInOEnv{--}(_Delta {->-}, xi_set)))
extendOEnvInitJ		:: forall _Delta, xi_set => --
	WfJ{--}(_Delta) -> WfJ{--}(_Delta `extendOEnv` xi_set) -> --
	{-sem-}(initOfOEnv	{--}(_Delta `extendOEnv` xi_set) == initOfOEnv		{--}(_Delta) `union` (xi_set `shiftEnvelope` newInOEnv{--}(_Delta {->-}, xi_set)))

extendOEnvAtomsJ 	:: forall _Delta, xi_set, sigma_k, phi_k, mu_set_k =>
	WfJ{--}(_Delta) -> --
	WfJ{--}(_Delta `extendOEnv` xi_set) -> --
	{-sem-}({-hv-}(sigma_k {-hv-}, phi_k {-hv-}, mu_set_k) @@ envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(atomOfOEnv{--}(_Delta `extendOEnv` xi_set {->-}, sigma_k, phi_k)) `EqvJ` {-sem-}(atomOfOEnv{--}(_Delta {->-}, sigma_k, phi_k))
\end{haskell}
\begin{proof}

The first three theorems are justified immediately from our Haskell definition of the environment extension operation, whereby all of the respective environment components directly assume the precise forms required above. On the other hand, the fourth theorem "extendOEnvAtomsJ" follows from the fact that the MMIX definition of the environment inspection operation "atomOfOEnv{--}(_Delta {->-}, sigma, phi)" relies only on the memory image "_M{--}(_Delta)", which is clearly preserved under all of its well-formed extensions. \QED

\end{proof}
\end{theorem}

In contrast, the dual contraction operation "_Delta `contractOEnv` xi_set" may involve some additional processing whenever the precise envelope "xi_set" does not appear at the top of the environment's data stack "stackOfOEnv{--}(_Delta)". On MMIX, such operations are still deemed valid, provided that the resulting stack pointer "sigma_c + ceiling (sizeOfEnvelope{--}(xi_set') / 8) * 8" remains within its allotted region "[sigma_f .. sigma_i]", but, for the remainder of the program's execution, the "envelopeOfOEnv{--}(_Delta)", "initOfOEnv{--}(_Delta)" and "stackOfOEnv{--}(_Delta)" components are all reset to "emptySet", so that no guarantees can be ever made about such programs' semantics within the portable fragment of the Etude language described in Chapter \ref{Lambda}:

\BeginHaskellToken	{contractOEnv}				{Void}	{}
  \HaskellArgument							{OEnvT}
  \HaskellToken		{{-\\-}}				{Bin}	{\backslash}
  \HaskellArgument							{EnvelopeT}
  \HaskellResult							{OEnvT}
\EndHaskellToken

\NewHaskellToken	{{-\\-}}				{Op}	{\backslash}
\NewHaskellToken	{`contractOEnv`}			{Op}	{\backslash}

\IndexTerm		{contractOEnv}					{\backslash o_env envelope (o_env)}

\begin{code}
contractOEnv {-[*]-} {-\-} {-[*]-} :: OEnvT -> EnvelopeT -> OEnvT

contractOEnv {-sem-}(_M, _A, xi_set, xi_set_i, psi_set, sigma_i, sigma_c, sigma_f) {-\-} {-sem-}(xi_set' {-,-})

   | psi_set /= {-empty-}[] && head{--}(psi_set) == {-hv-}(sigma_c' {-hv-}, xi_set') ---
  			= {-sem-}(_M, _A, xi_set `difference` (xi_set' `shiftEnvelope` sigma_c'), xi_set_i `difference` (xi_set' `shiftEnvelope` sigma_c'), --
  					tail{--}(psi_set), sigma_i, sigma_c', sigma_f) {-!-} {-!-} {-!-} {-!-} {-!-} {-!-}
   | otherwise		= {-sem-}(_M, _A, emptySet, emptySet, {-empty-}[], sigma_i, sigma_c', sigma_f)

 where	 sigma_c'  = sigma_c + ceiling (sizeOfEnvelope{--}(xi_set') % 8) * 8
\end{code}

\set{label}		{Lambda/Arch/contractOEnvJ}
\set{title}		{Algebraic semantics of object environment contractions}
\begin{theorem}

The environment contraction operation "_Delta `contractOEnv` xi_set" preserves the semantics of "_Delta" as follows:

\NewHaskellToken	{wfContractOEnvJ}			{Ord}	{\TKeyword{con\sub{\Delta}}}
\NewHaskellDescription	{wfContractOEnvJ}				{Validity of object environment contractions}
\IndexTheorem		{wfContractOEnvJ}				{CON_\Delta Validity of object environment contractions}

\NewHaskellToken	{contractOEnvStackJ}			{Ord}	{\TKeyword{con\sub{s}}}
\NewHaskellDescription	{contractOEnvStackJ}				{Object environment stack after contraction}
\IndexTheorem		{contractOEnvStackJ}				{CON_S Object environment stack after contraction}

\NewHaskellToken	{contractOEnvEnvelopeJ}			{Ord}	{\TKeyword{con\sub{e}}}
\NewHaskellDescription	{contractOEnvEnvelopeJ}				{Object environment envelope after contraction}
\IndexTheorem		{contractOEnvEnvelopeJ}				{CON_E Object environment envelope after contraction}

\NewHaskellToken	{contractOEnvInitJ}			{Ord}	{\TKeyword{con\sub{i}}}
\NewHaskellDescription	{contractOEnvInitJ}				{Object environment initialisers after contraction}
\IndexTheorem		{contractOEnvInitJ}				{CON_I Object environment initialisers after contraction}

\NewHaskellToken	{contractOEnvAtomsJ}			{Ord}	{\TKeyword{con\sub{a}}}
\NewHaskellDescription	{contractOEnvAtomsJ}				{Object environment data after contraction}
\IndexTheorem		{contractOEnvAtomsJ}				{CON_A Object environment data after contraction}

\begin{haskell}
wfContractOEnvJ		:: forall _Delta, sigma_c, xi_set => --
	WfJ{--}(_Delta) -> {-sem-}(head{--}(stackOfOEnv{--}(_Delta) == {-hv-}(sigma_c {-hv-}, xi_set))) -> --
	WfJ{-sem-}(_Delta `contractOEnv` xi_set)
contractOEnvStackJ	:: forall _Delta, sigma_c, xi_set => --
	WfJ{--}(_Delta) -> {-sem-}(head{--}(stackOfOEnv{--}(_Delta) == {-hv-}(sigma_c {-hv-}, xi_set))) -> --
	{-sem-}(stackOfOEnv	{--}(_Delta `contractOEnv` xi_set) == tail{--}(stackOfOEnv{--}(_Delta)))
contractOEnvEnvelopeJ	:: forall _Delta, sigma_c, xi_set => --
	WfJ{--}(_Delta) -> {-sem-}(head{--}(stackOfOEnv{--}(_Delta) == {-hv-}(sigma_c {-hv-}, xi_set))) -> --
	{-sem-}(envelopeOfOEnv	{--}(_Delta `contractOEnv` xi_set) == envelopeOfOEnv	{--}(_Delta) `difference` (xi_set `shiftEnvelope` sigma_c))
contractOEnvInitJ	:: forall _Delta, sigma_c, xi_set => --
	WfJ{--}(_Delta) -> {-sem-}(head{--}(stackOfOEnv{--}(_Delta) == {-hv-}(sigma_c {-hv-}, xi_set))) -> --
	{-sem-}(initOfOEnv	{--}(_Delta `contractOEnv` xi_set) == initOfOEnv	{--}(_Delta) `difference` (xi_set `shiftEnvelope` sigma_c))

contractOEnvAtomsJ	:: forall _Delta, sigma_c, xi_set, sigma_k, phi_k, mu_set_k =>
	WfJ{--}(_Delta) ->
	{-sem-}(head{--}(stackOfOEnv{--}(_Delta) == {-hv-}(sigma_c {-hv-}, xi_set))) ->
	{-sem-}({-hv-}(sigma_k {-hv-}, phi_k {-hv-}, mu_set_k) @@ envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(sigma_k + sizeOfFormat{--}(phi_k) <= sigma_c || sigma_k >= sigma_c + sizeOfEnvelope{--}(xi_set)) ->
	{-sem-}(atomOfOEnv{--}(_Delta `contractOEnv` xi_set {->-}, sigma_k, phi_k)) `EqvJ` {-sem-}(atomOfOEnv{--}(_Delta {->-}, sigma_k, phi_k))
\end{haskell}
\begin{proof}

Once again, all five theorems follow directly from the MMIX implementation of environment contractions, observing that only the portable case of that definition is ever applicable under the supplied precondition “"head{--}(stackOfOEnv{--}(_Delta) == {-hv-}(sigma_c {-hv-}, xi_set))"”, that every subset of a well-formed Etude envelope is also well-formed itself and that the actual memory image "_M" of the program is never affected by the operation. \QED

\end{proof}
\end{theorem}

Finally, on MMIX, an update to the content of a given memory object "(sigma, phi)" with the new value of "alpha" is modelled by discarding any alignment bits of "sigma", then setting each byte "sigma + k @@ [ sigma' .. sigma' + sizeOfFormat{--}(phi) - 1]" in the program's memory image "_M" to the corresponding 8 bits "x`bits`[ byteWidth #*# k .. byteWidth #*# (k + 1) - 1 ]" of "alpha"'s encoded value "discardFraction (decodeAtom({-sem-}(NG`FMT`64)){--}(evalAtom{--}(alpha)))" \hbox{as follows:}

\BeginHaskellToken	{setInOEnv}				{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{OEnvT}
  \HaskellToken		{{-/-}}					{Bin}	{\slash}
  \HaskellArgument							{(Integer, FormatT, AtomT nu)}
  \HaskellResult							{OEnvT}
\EndHaskellToken

\NewHaskellToken	{{-/-}}					{Op}	{\slash}
\NewHaskellToken	{`setInOEnv`}				{Op}	{\slash}

\IndexTerm		{setInOEnv}					{\slash o_env integer format atom (o_env)}

\begin{code}
setInOEnv {-[*]-} {-/-} {-[*]-} :: (Ord nu) => OEnvT -> (Integer, FormatT, AtomT nu) -> OEnvT

setInOEnv {-sem-}(_M, _A, xi_set, xi_set_i, psi_set, sigma_i, sigma_c, sigma_f) {-/-} {-sem-}(sigma, phi, alpha) | and [ _A#(sigma' + k) | k <- [ 0 .. sizeOfFormat{--}(phi) - 1 ] ]
 {--}			= {-sem-}(_M', _A, xi_set, xi_set_i', psi_set, sigma_i, sigma_c, sigma_f)
 where	 sigma'		= sigma - sigma `mod` sizeOfFormat{--}(phi)
 	 x		= discardFraction (decodeAtom({-sem-}(NG`FMT`64)){--}(evalAtom{--}(alpha)))
	 _M'		= _M // listToMap [ {-raw-}((sigma' + sizeOfFormat{--}(phi) - 1 - k) {-:-}, x`bits`[ byteWidth #*# k .. byteWidth #*# (k + 1) - 1 ]) --
							| k <- [ 0 .. sizeOfFormat{--}(phi) - 1 ] ]
	 xi_set_i'	= xi_set_i `purgeEnvelope` (sigma', sigma' + sizeOfFormat{--}(phi))
\end{code}

\set{label}		{Lambda/Arch/wfSetInOEnvJ}
\set{title}		{Validity of object environment updates}
\begin{theorem}

Well-formedness of every environment "_Delta" is preserved under a valid environment update operation "_Delta `setInOEnv` (sigma, phi, alpha)":

\NewHaskellToken	{wfSetInOEnvJ}				{Ord}	{\TKeyword{set\sub{\Delta}}}
\NewHaskellDescription	{wfSetInOEnvJ}					{Validity of object environment updates}
\IndexTheorem		{wfSetInOEnvJ}					{SET_\Delta Validity of object environment updates}

\begin{haskell}
wfSetInOEnvJ 		:: forall _Delta, sigma, phi, x, mu_set =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(IMM (x) (phi)) -> {-sem-}({-hv-}(sigma {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	WfJ{-sem-}(_Delta `setInOEnv` (sigma, phi, {-sem-}(IMM (x) (phi))))
\end{haskell}
\begin{proof}

Recalling the earlier definition of the environment well-formedness property “"WfJ :: OEnvT -> Prop"” and, further, observing that the above definition of environment updates only ever modifies the two environment components "_M" and "xi_set_i", we are only required to establish three of the nine preconditions of "WfJ".

In particular, we must first demonstrate that, in the updated environment, "xi_set_i'" always represents a subset of "xi_set". Given the definitions of the finite map extension and contraction operators from Appendix \ref{Additional}, we observe that \hbox{"dom{--}(_M1) `subseteq` dom{--}(_M1 // _M2)"} for all well-defined finite maps "_M1" and "_M2". Accordingly, \hbox{"xi_set_i' == xi_set_i `purgeEnvelope` (sigma', sigma' + sizeOfFormat{--}(phi)) `subseteq` xi_set_i `subseteq` xi_set".}

For all \hbox{"sigma @@ dom{--}(_A)",} we also require that \hbox{"0 <= sigma < 2^63 && (_A{--}(sigma) || sigma @@ dom{--}(_M))".} Observing that, for every valid map "_M" and set "_S", the relation \hbox{"dom{--}(_M `mapSetDifference` _S) `subseteq` dom{--}(_M)"} is always satisfied by the finite map difference operator from Appendix \ref{Additional}, we have \hbox{"dom{--}(_M') `subseteq` dom{--}(_M // listToMap [ sigma_k' {-:-}, n_k' ]",} so that the second of the required preconditions is likewise guaranteed to hold in the resulting environment.

Finally, well-formedness of the resulting object environment also requires that, for every \hbox{"sigma @@ dom{--}(_M)"}, \hbox{"0 <= _M{--}(sigma) <= 255"}. If the address \hbox{"sigma + k"} falls outside of the region affected by the update, then \hbox{"_M'{--}(sigma + k) == _M{--}(sigma + k)",} so the property holds by definition. Otherwise, we have \hbox{"_M'{--}(sigma + k) == x`bits`[ byteWidth {--} k .. byteWidth {--} (k + 1) - 1 ] <= 2^(byteWidth {--} (k + 1) - 1 - byteWidth {--} k + 1)"}, which, as required, is always equal to "2^byteWidth" or "2^8", since the byte offset "k" cannot be greater than "7" in magnitude. \QED

\end{proof}
\end{theorem}

\set{label}		{Lambda/Arch/setInOEnvAtomJ}
\set{title}		{Object environment binding after an update}
\begin{theorem}

\NewHaskellToken	{setInOEnvAtomJ}			{Ord}	{\TKeyword{set\sub{\alpha}}}
\NewHaskellDescription	{setInOEnvAtomJ}				{Object environment binding after an update}
\IndexTheorem		{setInOEnvAtomJ}				{SET_A Object environment binding after an update}

\begin{haskell}
setInOEnvAtomJ 		:: forall _Delta, sigma, phi, alpha, mu_set =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `setInOEnv` (sigma, phi, alpha)) -> {-sem-}({-hv-}(sigma {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(atomOfOEnv{--}(_Delta `setInOEnv` (sigma, phi, alpha) {->-}, sigma, phi)) `EqvJ` {-sem-}(alpha)
\end{haskell}
\begin{proof}

Given the earlier definition of the atom equivalence relation in Section \ref{Lambda/Arch/Atoms}, it will suffice to show that the above theorem holds for the four distinguished natural formats "NG`FMT`8", "NG`FMT`16", "NG`FMT`32" or "NG`FMT`64", since every other well-formed Etude atom is always equivalent under reduction by "evalAtom" to some constant “"IMM (n) (NG`FMT`epsilon)"\!”, in which the integer "n" falls within the range of values "0 <= n < 2^(sizeOfFormat{--}(phi) {--} byteWidth)" that are representable under the format "phi". In order to establish theorem "setInOEnvAtomJ", it is therefore sufficient to show that every such value of "n" is always preserved under its insertion and a subsequent extraction from the same address. In particular, let "m" be the value stored at some address "sigma" in a memory image "_M", which has been derived by an update of that address with the value of "n". Then:

\begin{haskell}
m	= sum [ 2^(byteWidth #*# k) #*# _M#(sigma' + sizeOfFormat{--}(phi) - 1 - k) --
			| k <- [ 0 .. sizeOfFormat{--}(phi) - 1 ] ] 				
	= sum [ 2^(byteWidth #*# k) #*# n`bits`[ byteWidth #*# k .. byteWidth #*# (k + 1) - 1 ] --
			| k <- [ 0 .. sizeOfFormat{--}(phi) - 1 ] ]				
	= sum [ 2^(byteWidth #*# k) #*# sum [ 2^(j - byteWidth #*# k) #*# (floor (n % 2^j) `mod` 2) --
		| j <- [ byteWidth #*# k .. byteWidth #*# (k + 1) - 1 ] ] --
			| k <- [ 0 .. sizeOfFormat{--}(phi) - 1 ] ]				
	= sum [ sum [ 2^(byteWidth #*# k) #*# 2^(j - byteWidth #*# k) #*# (floor (n % 2^j) `mod` 2) --
		| j <- [ byteWidth #*# k .. byteWidth #*# (k + 1) - 1 ] ] --
			| k <- [ 0 .. sizeOfFormat{--}(phi) - 1 ] ]
	= sum [ 2^(byteWidth #*# k) #*# 2^(j - byteWidth #*# k) #*# (floor (n % 2^j) `mod` 2) --
			| k <- [ 0 .. sizeOfFormat{--}(phi) - 1 ], --
			  j <- [ byteWidth #*# k .. byteWidth #*# (k + 1) - 1 ] ]
	= sum [ 2^j #*# (floor (n % 2^j) `mod` 2) --
			| k <- [ 0 .. sizeOfFormat{--}(phi) - 1 ], --
			  j <- [ byteWidth #*# k .. byteWidth #*# (k + 1) - 1 ] ]
	= sum [ 2^j #*# (floor (n % 2^j) `mod` 2) --
			| j <- [ 0 .. byteWidth * sizeOfFormat{--}(phi) - 1 ] ]
	= n
\end{haskell}

as required. The first three steps in the above reduction sequence are justified, respectively, by the earlier definition of the environment inspection function "atomOfOEnv", the values of \hbox{"_M{--}(sigma' + sizeOfFormat{--}(phi) - 1 - k)"} inserted into the address space in accordance with the definition of environment update operations and, finally, by the definition of bit extraction from Appendix \ref{Additional}. The remaining five steps, on the other hand, represent simple arithmetic transformations justified directly by the axioms of the Haskell “"Integer"” type. \QED

\end{proof}
\end{theorem}

\set{label}		{Lambda/Arch/setInOEnvAtomsJ}
\set{title}		{Object environment data after an update}
\begin{theorem}

\NewHaskellToken	{setInOEnvAtomsJ}			{Ord}	{\TKeyword{set\sub{\macron\alpha}}}
\NewHaskellDescription	{setInOEnvAtomsJ}				{Object environment data after an update}
\IndexTheorem		{setInOEnvAtomsJ}				{SET_A Object environment data after an update}

\begin{haskell}
setInOEnvAtomsJ 	:: forall _Delta, sigma, phi, alpha, xi_set, sigma_k, phi_k, mu_set_k =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `setInOEnv` (sigma, phi, alpha)) ->
	{-sem-}({-hv-}(sigma {-hv-},    phi {-hv-},    mu_set)    `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}({-hv-}(sigma_k {-hv-},  phi_k {-hv-},  mu_set_k)  `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(sigma_k + sizeOfFormat{--}(phi_k) <= sigma || sigma_k >= sigma + sizeOfFormat{--}(phi)) ->
	{-sem-}(atomOfOEnv{--}(_Delta `setInOEnv` (sigma, phi, alpha) {->-}, sigma_k, phi_k)) `EqvJ` {-sem-}(atomOfOEnv{--}(_Delta {->-}, sigma_k, phi_k))
\end{haskell}
\begin{proof}

Trivial, observing that none of the relevant address values are ever affected in "_M" by the above Haskell definition of object environment updates. \QED

\end{proof}
\end{theorem}

\set{label}		{Lambda/Arch/eqvSetInOEnvJ}
\set{title}		{Compatibility of object environment updates}
\begin{theorem}

\NewHaskellToken	{eqvSetInOEnvJ}				{Ord}	{\TKeyword{set\sub{\equiv}}}
\NewHaskellDescription	{eqvSetInOEnvJ}					{Compatibility of object environment updates}
\IndexTheorem		{eqvSetInOEnvJ}					{SET_\Delta Compatibility of object environment updates}

\begin{haskell}
eqvSetInOEnvJ 		:: forall _Delta, sigma, phi, alpha1, alpha2 => --
	{-sem-}(alpha1) `EqvJ` {-sem-}(alpha2) -> --
	{-sem-}(_Delta `setInOEnv` (sigma, phi, alpha1) = _Delta `setInOEnv` (sigma, phi, alpha2))
\end{haskell}
\begin{proof}

In the definition of environment extension "_Delta `setInOEnv` (sigma, phi, alpha_i)", the value "n_i" inserted into the memory image "_M" at each affected address "sigma' + k" is defined as "discardFraction (decodeAtom({-sem-}(NG`FMT`64)){--}(evalAtom{--}(alpha_i)))", so that, given the equivalence of "alpha1" and "alpha2" or "evalAtom{--}(alpha1) = evalAtom{--}(alpha2)", we have "x1 = x2" and "_Delta `setInOEnv` (sigma, phi, alpha1) = _Delta `setInOEnv` (sigma, phi, alpha2)" as required. \QED

\end{proof}
\end{theorem}

\set{label}		{Lambda/Arch/setInOEnvInvJ}
\set{title}		{Algebraic invariants of object environment updates}
\begin{theorem}

Every well-formed environment update operation preserves the following properties of the program's address space:

\NewHaskellToken	{setInOEnvInitJ}			{Ord}	{\TKeyword{set\sub{i}}}
\NewHaskellDescription	{setInOEnvInitJ}				{Object environment initialisers after an update}
\IndexTheorem		{setInOEnvInitJ}				{SET_I Object environment initialisers after an update}

\NewHaskellToken	{setInOEnvEnvelopeJ}			{Ord}	{\TKeyword{set\sub{e}}}
\NewHaskellDescription	{setInOEnvEnvelopeJ}				{Object environment envelope after an update}
\IndexTheorem		{setInOEnvEnvelopeJ}				{SET_E Object environment envelope after an update}

\NewHaskellToken	{setInOEnvStackJ}			{Ord}	{\TKeyword{set\sub{s}}}
\NewHaskellDescription	{setInOEnvStackJ}				{Object environment stack after an update}
\IndexTheorem		{setInOEnvStackJ}				{SET_S Object environment stack after an update}

\NewHaskellToken	{setInOEnvExtensionJ}			{Ord}	{\TKeyword{set\sub{x}}}
\NewHaskellDescription	{setInOEnvExtensionJ}				{Object environment extensions after an update}
\IndexTheorem		{setInOEnvExtensionJ}				{SET_X Object environment extensions after an update}

\NewHaskellToken	{setInOEnvNewJ}				{Ord}	{\TKeyword{set\sub{n}}}
\NewHaskellDescription	{setInOEnvNewJ}					{Object environment stack pointer after an update}
\IndexTheorem		{setInOEnvNewJ}					{SET_X Object environment stack pointer after an update}

\begin{haskell}
setInOEnvInitJ 		:: forall _Delta, sigma, phi, alpha, mu_set =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `setInOEnv` (sigma, phi, alpha)) -> {-sem-}({-hv-}(sigma {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(initOfOEnv{--}(_Delta `setInOEnv` (sigma, phi, alpha) {->-}, sigma, phi) == initOfOEnv{--}(_Delta) `purgeEnvelope` (sigma, sigma + sizeOfFormat{--}(phi)))

setInOEnvEnvelopeJ 	:: forall _Delta, sigma, phi, alpha, mu_set =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `setInOEnv` (sigma, phi, alpha)) -> {-sem-}({-hv-}(sigma {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(envelopeOfOEnv{--}(_Delta `setInOEnv` (sigma, phi, alpha)) == envelopeOfOEnv{--}(_Delta))

setInOEnvStackJ 	:: forall _Delta, sigma, phi, alpha, mu_set =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `setInOEnv` (sigma, phi, alpha)) -> {-sem-}({-hv-}(sigma {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(stackOfOEnv{--}(_Delta `setInOEnv` (sigma, phi, alpha)) == stackOfOEnv{--}(_Delta))

setInOEnvExtensionJ 	:: forall _Delta, sigma, phi, alpha, mu_set, xi_set =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `setInOEnv` (sigma, phi, alpha)) -> {-sem-}({-hv-}(sigma {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)) -> --
	WfJ{-sem-}(_Delta `extendOEnv` xi_set) ->
	WfJ{-sem-}((_Delta `setInOEnv` (sigma, phi, alpha)) `extendOEnv` xi_set)

setInOEnvNewJ 		:: forall _Delta, sigma, phi, alpha, mu_set, xi_set =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `setInOEnv` (sigma, phi, alpha)) -> {-sem-}({-hv-}(sigma {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)) -> --
	WfJ{-sem-}(_Delta `extendOEnv` xi_set) ->
	{-sem-}(newInOEnv{--}(_Delta `setInOEnv` (sigma, phi, alpha) {->-}, xi_set) == newInOEnv{--}(_Delta {->-}, xi_set))
\end{haskell}
\begin{proof}

Our Haskell implementation of object environment updates always returns the original value of each environment component scrutinised above, so that all five of these theorems hold directly by the definition of "_Delta `setInOEnv` (sigma, phi, alpha_i)". \QED

\end{proof}
\end{theorem}
\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Lambda/Arch/Terms}
\set{title}{Terms}
\begin{subsection}

\set{capture}		{evalSysCall}
\begin{code}
evalSysCall {-[*]-} :: (Ord nu) => ({-,,,-}FEnvT nu, OEnvT, PrimT {->-}, AtomsT nu) -> (OEnvT, AtomsT nu)
\end{code}

\set{capture}		{hidden}
\begin{code}
evalSysCall {-sem-}(_Lambda, _Delta, pi {->-}, alpha_set) = (_Delta, {-empty-}[])
\end{code}

The final piece of Etude's operational model that must be specialised for the MMIX architecture is the precise semantics of all monadic term constructs provided by the language. In a nutshell, the meaning of Etude terms is formalised similarly to atoms, using a function "evalTerm" akin to the earlier atom evaluation algorithm "evalAtom". In particular, in the context of a given evaluation environment "(_Lambda, Delta)", "evalTerm{--}(_Lambda, _Delta {->-}, tau)" reduces the specified term "tau" into a list of result atoms "alpha_set", an updated object environment "_Delta'" and a list of zero or more abstract entities known as «actions», which, intuitively, describe the exact sequence of system calls invoked by the program during evaluation of "tau". Formally:

\BeginHaskellToken	{evalTerm}				{Open}	{\ZapfChancery{E}\sub{«\tau»}}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{({-,,,-}FEnvT nu, OEnvT {->-}, TermT nu)}
  \HaskellResult							{(OEnvT, ActionsT nu, AtomsT nu)}
\EndHaskellToken

\IndexTerm		{evalTerm}					{E_\tau term f_env o_env (o_env actions atoms)}

\begin{code}
evalTerm {-[*]-} :: (Ord nu) => ({-,,,-}FEnvT nu, OEnvT {->-}, TermT nu) -> (OEnvT, ActionsT nu, AtomsT nu)
\end{code}

Each of these actions is depicted simply by a pair "(pi, alpha_set)", in which "pi" represents one of the operating system facilities described by the Haskell type “"PrimT"” and "alpha_set" specifies the precise list of input arguments for the underlying system call. Formally:

\BeginHaskellToken	{ActionT}				{Ord}	{«action»}
  \HaskellContext							{(Ord nu)}
  \HaskellCommand						{1}	{\sub{#1}}
  \HaskellArgument							{nu}
  \HaskellResult							{Type}
\EndHaskellToken 

\BeginHaskellToken	{ActionsT}				{Ord}	{«actions»}
  \HaskellContext							{(Ord nu)}
  \HaskellCommand						{1}	{\sub{#1}}
  \HaskellArgument							{nu}
  \HaskellResult							{Type}
\EndHaskellToken 

\IndexType		{ActionT}					{action \nu (\tstar)}
\IndexType		{ActionsT}					{actions \nu (\tstar)}

\begin{syntax}
\begin{code}
type ActionT nu	= 	(PrimT, AtomsT nu)
type ActionsT nu = 	[ ActionT nu ]
\end{code}
\end{syntax}

The evaluation results obtained by "evalTerm" are used directly for determination of the actual term equivalence and well-formedness properties utilised in the generic algebraic description of Etude term semantics from Chapter \ref{Lambda}. Specifically, these term properties capture precisely the notions of definitional equality and well-formedness of the respective evaluation results obtained from the entity's syntax and context by "evalTerm", as expressed by the following pair of Haskell property definitions:

\BeginHaskellToken	{EqvTermJ}				{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{({-,,,-}FEnvT nu, OEnvT {->-}, TermT nu)}
  \HaskellToken		{`EqvJ`}				{Rel}	{\equiv}
  \HaskellArgument							{({-,,,-}FEnvT nu, OEnvT {->-}, TermT nu)}
  \HaskellResult							{Prop}
\EndHaskellToken

\BeginHaskellToken	{WfTermJ}				{Ord}	{\TKeyword{wf}}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{({-,,,-}FEnvT nu, OEnvT {->-}, TermT nu)}
  \HaskellResult							{Prop}
\EndHaskellToken

\NewHaskellToken	{EqvTermC}				{Ord}	{\TKeyword{eqv\sub{\tau}}}
\NewHaskellToken	{WfTermC}				{Ord}	{\TKeyword{wf\sub{\tau}}}

\NewHaskellDescription	{EqvTermC}					{Equivalence of terms}
\NewHaskellDescription	{WfTermC}					{Validity of terms}

\IndexProp		{EqvTermJ}					{\equiv term term (\pstar)}
\IndexAxiom		{EqvTermC}					{EQV_\tau Equivalence of terms}

\IndexProp		{WfTermJ}					{WF term (\pstar)}
\IndexAxiom		{WfTermC}					{WF_\tau Validity of terms}

\begin{haskell}
data {-[*]-} `EqvJ` {-[*]-} :: (Ord nu) => ({-,,,-}FEnvT nu, OEnvT {->-}, TermT nu) -> ({-,,,-}FEnvT nu, OEnvT {->-}, TermT nu) -> Prop

 where	EqvTermC :: forall _Lambda1, _Delta1, alpha1, Lambda2, _Delta2, alpha2 =>
 	{-quad-}	{-sem-}(evalTerm{--}(_Lambda1, _Delta1 {->-}, alpha1) = evalTerm{--}(_Lambda2, _Delta2 {->-}, alpha2)) ->
 			{-sem-}(_Lambda1, _Delta1 {->-}, alpha1) `EqvJ` {-sem-}(_Lambda2, _Delta2 {->-}, alpha2)

data WfJ {-[*]-} :: (Ord nu) => ({-,,,-}FEnvT nu, OEnvT {->-}, TermT nu) -> Prop

 where	WfTermC :: forall _Lambda, _Delta, alpha => --
 		WfJ{-sem-}(evalTerm{--}(_Lambda, _Delta {->-}, alpha)) -> --
 		WfJ{-sem-}(_Lambda, _Delta {->-}, alpha)
\end{haskell}

\set{label}		{Lambda/Arch/EqvTermJ}
\set{title}		{Standard equivalence properties of Etude terms}
\begin{theorem}

According to the above definition, “"`EqvJ`"” constitutes a reflexive, symmetric and transitive relation over all Etude term forms:

\NewHaskellToken	{eqvReflTermJ}				{Ord}	{\TKeyword{refl\sub{\tau}}}
\NewHaskellToken	{eqvSymmTermJ}				{Ord}	{\TKeyword{symm\sub{\tau}}}
\NewHaskellToken	{eqvTrnsTermJ}				{Ord}	{\TKeyword{trans\sub{\tau}}}

\NewHaskellDescription	{eqvReflTermJ}					{Reflexivity of term equivalence}
\NewHaskellDescription	{eqvSymmTermJ}					{Symmetry of term equivalence}
\NewHaskellDescription	{eqvTrnsTermJ}					{Transitivity of term equivalence}

\IndexTheorem		{eqvReflTermJ}					{REFL_\tau Reflexivity of term equivalence}
\IndexTheorem		{eqvSymmTermJ}					{SYMM_\tau Symmetry of term equivalence}
\IndexTheorem		{eqvTrnsTermJ}					{TRANS_\tau Transitivity of term equivalence}

\begin{haskell}
eqvReflTermJ	:: forall _Lambda, _Delta, tau =>
	{-sem-}(_Lambda,   _Delta   {->-}, tau   ) `EqvJ` {-sem-}(_Lambda,   _Delta   {->-}, tau  )

eqvSymmTermJ	:: forall _Lambda1, _Delta1, tau1, _Lambda2, _Delta2, tau2 =>
	{-sem-}(_Lambda1,  _Delta1  {->-}, tau1  ) `EqvJ` {-sem-}(_Lambda2,  _Delta2  {->-}, tau2  ) ->
	{-sem-}(_Lambda2,  _Delta2  {->-}, tau2  ) `EqvJ` {-sem-}(_Lambda1,  _Delta1  {->-}, tau1  )

eqvTrnsTermJ	:: forall _Lambda1, _Delta1, tau1, _Lambda2, _Delta2, tau2, _Lambda3, _Delta3, tau3 =>
	{-sem-}(_Lambda1,  _Delta1  {->-}, tau1  ) `EqvJ` {-sem-}(_Lambda2,  _Delta2  {->-}, tau2  ) ->
	{-sem-}(_Lambda2,  _Delta2  {->-}, tau2  ) `EqvJ` {-sem-}(_Lambda3,  _Delta3  {->-}, tau3  ) ->
	{-sem-}(_Lambda1,  _Delta1  {->-}, tau1  ) `EqvJ` {-sem-}(_Lambda3,  _Delta3  {->-}, tau3  )
\end{haskell}
\begin{proof}

Trivial, from the corresponding properties of definitional equality “"="”. \QED

\end{proof}
\end{theorem}

In particular, simple terms of the form “"RET (alpha_set)"” never deliver any actions or affect the program's memory image. Instead, the supplied list of atoms "alpha_set" is reduced by "evalAtoms" and returned directly as part of the term's evaluated result:

\begin{code}
evalTerm {-sem-}(_Lambda, _Delta {->-}, RET (alpha_set)) ---
		= (_Delta, {-empty-}[], evalAtoms{--}(alpha_set))
\end{code}

\set{label}		{Lambda/Arch/wfRetTermJ}
\set{title}		{Properties of “"RET"” terms}
\begin{theorem}

All Etude “"RET"” term forms satisfy the following properties under the MMIX architecture:

\NewHaskellToken	{wfRetTermJ}				{Ord}	{\TKeyword{wf\sub{ret}}}
\NewHaskellDescription	{wfRetTermJ}					{Validity of “\haskell{RET}” terms}
\IndexTheorem		{wfRetTermJ}					{WF_RET Algebraic semantics of RET terms}

\NewHaskellToken	{eqvRetTermJ}				{Ord}	{\TKeyword{eqv\sub{ret}}}
\NewHaskellDescription	{eqvRetTermJ}					{Compatibility of “\haskell{RET}” terms}
\IndexTheorem		{eqvRetTermJ}					{EQV_RET Equivalence of RET terms}

\begin{haskell}
wfRetTermJ 	:: forall _Lambda, _Delta, alpha_set => WfJ{--}(_Lambda) -> WfJ{--}(_Delta) -> WfJ{--}(alpha_set) -> WfJ{-sem-}(_Lambda, _Delta {->-}, RET (alpha_set))
eqvRetTermJ 	:: forall _Lambda, _Delta, alpha_set1, alpha_set2 => --
	(alpha_set1 `EqvJ` alpha_set2) -> --
	{-sem-}(_Lambda, _Delta {->-}, RET (alpha_set1)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, RET (alpha_set2))
\end{haskell}
\begin{proof}

From the corresponding properties of atom well-formedness "wfEqvAtomJ" and equivalence "EqvAtomC". \QED

\end{proof}
\end{theorem}

In every application term of the form “"APP (alpha) (alpha_set)"”, "alpha" must depict a well-formed atom that, in "_Lambda", is bound to some Etude function "FUN (nu_set) {-DOT-} (tau)", in which the parameter list "nu_set" has the same length as the number of argument atoms in "alpha_set". The construction simply evaluates the body term "tau", after substituting the normal form of each "alpha_k @@ evalAtoms{--}(alpha_set)" for any free occurrences of the corresponding variable "nu_k @@ nu_set" in "tau". Formally:

\begin{code}
evalTerm {-sem-}(_Lambda, _Delta {->-}, APP (alpha) (alpha_set)) | (lengthOfList{--}(alpha_set) == lengthOfList{--}(nu_set)) ---
		= evalTerm{--}(_Lambda, _Delta {->-}, tau `substTerm` (listToMap (nu_set `zipList` evalAtoms{--}(alpha_set))))

 where	{-sem-}(FUN (nu_set) {-DOT-} (tau)) = _Lambda#(discardFraction (decodeAtom(FG`FMT`64){--}(evalAtom{--}(alpha))))
\end{code}

\set{label}		{Lambda/Arch/wfAppTermJ}
\set{title}		{Properties of application terms}
\begin{theorem}

Every Etude application term satisfies the following properties under the MMIX architecture:

\NewHaskellToken	{wfAppTermJ}				{Ord}	{\TKeyword{wf\sub{app}}}
\NewHaskellDescription	{wfAppTermJ}					{Validity of function applications}
\IndexTheorem		{wfAppTermJ}					{WF_APP Validity of function applications}

\NewHaskellToken	{eqvAppTermJ}				{Ord}	{\TKeyword{eqv\sub{app}}}
\NewHaskellDescription	{eqvAppTermJ}					{Compatibility of function applications}
\IndexTheorem		{eqvAppTermJ}					{EQV_APP Compatibility of function applications}

\NewHaskellToken	{eqvAppBetaTermJ}			{Ord}	{\TKeyword{eqv\sub{\beta}}}
\NewHaskellDescription	{eqvAppBetaTermJ}				{Beta equivalence}
\IndexTheorem		{eqvAppBetaTermJ}				{EQV_\beta Beta equivalence}

\begin{haskell}
wfAppTermJ :: forall _Lambda, _Delta, x, alpha_set, nu_set, tau =>
	WfJ{--}(_Lambda) -> WfJ{--}(_Delta) -> WfJ{-sem-}(IMM (x) (FG`FMT`standardEncoding)) -> WfJ{--}(alpha_set) ->
	{-sem-}(_Lambda#(x) == {-sem-}(FUN (nu_set) {-DOT-} (tau)) && lengthOfList{--}(nu_set) == lengthOfList{--}(alpha_set)) ->
	WfJ{-sem-}(_Lambda, _Delta {->-}, tau `substTerm` (nu_set `zipList` alpha_set)) ->
	WfJ{-sem-}(_Lambda, _Delta {->-}, APP (IMM (x) (FG`FMT`standardEncoding)) (alpha_set))

eqvAppTermJ :: forall _Lambda, _Delta, alpha1, alpha_set1, alpha2, alpha_set2 =>
	(alpha1 `EqvJ` alpha2) -> --
	(alpha_set1 `EqvJ` alpha_set2) ->
	{-sem-}(_Lambda, _Delta {->-}, APP (alpha1) (alpha_set1)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, APP (alpha2) (alpha_set2))

eqvAppBetaTermJ :: forall _Lambda, _Delta, x, alpha_set, nu_set, tau =>
	WfJ{--}(_Lambda) -> WfJ{--}(_Delta) -> WfJ{-sem-}(IMM (x) (FG`FMT`standardEncoding)) -> WfJ{--}(alpha_set) ->
	{-sem-}(_Lambda#(x) == {-sem-}(FUN (nu_set) {-DOT-} (tau)) && lengthOfList{--}(nu_set) == lengthOfList{--}(alpha_set)) ->
	WfJ{-sem-}(_Lambda, _Delta {->-}, tau `substTerm` (nu_set `zipList` alpha_set)) ->
	{-sem-}(_Lambda, _Delta {->-}, APP (IMM (x) (FG`FMT`standardEncoding)) (alpha_set)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, tau `substTerm` (nu_set `zipList` alpha_set))
\end{haskell}
\begin{proof}

Since, by definition, every application term evaluates to the appropriately substituted function body "tau `substTerm` (nu_set `zipList` alpha_set)", theorem "wfAppTermJ" holds trivially by the virtue of its own precondition "WfJ{-sem-}(_Lambda, _Delta {->-}, tau `substTerm` (nu_set `zipList` alpha_set))". Further, "eqvAppTermJ" can be established by observing that "evalTerm" always reduces all of the atomic values involved in the operation into their respective normal forms, which, by "EqvAtomC", must be structurally identical if the theorem's two preconditions are to be satisfied. Accordingly, the entire theorem is justified by reflexivity of the term equivalence relation. Finally, the beta-equivalence theorem "eqvAppBetaTermJ" is established directly by the definition "evalTerm", which prescribes precisely the required behaviour to all reductions of Etude application expressions. \QED

\end{proof}
\end{theorem}

A complete operational description of Etude system call operations is, regrettably, impossible without a detailed scrutiny of the underlying operating system facilities. Fortunately, neither is it required for a successful establishment of the linear correctness property for our compiler, whereby it will suffice to represent meanings of all well-formed system primitives by the following unspecified Haskell function:

\BeginHaskellToken	{evalSysCall}				{Open}	{\ZapfChancery{E}\sub{«\delta»}}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{({-,,,-}FEnvT nu, OEnvT, PrimT {->-}, AtomsT nu)}
  \HaskellResult							{(OEnvT, AtomsT nu)}
\EndHaskellToken

\IndexTerm		{evalSysCall}					{E_\delta atoms o_env f_env prim (o_env atoms)}

\InsertHaskell		{evalSysCall}

Using this auxiliary definition, the semantic significance of each system call operation “"SYS (pi) (alpha_set)"” can be represented by the object environment and result atoms derived by "evalSysCall" from an application of "pi" to the normal forms of the arguments "alpha_set". In addition, the evaluation result always includes the invoked primitive in its action list as a pair of the form "(pi, evalAtoms{--}(alpha_set))". Formally:

\begin{code}
evalTerm {-sem-}(_Lambda, _Delta {->-}, SYS (pi) (alpha_set)) ---
		= (_Delta', [ (pi, evalAtoms{--}(alpha_set)) ], alpha_set')
 where	(_Delta', alpha_set') ---
 		= evalSysCall{--}(_Lambda, _Delta, pi {->-}, evalAtoms{--}(alpha_set))
\end{code}

\set{label}		{Lambda/Arch/wfSysTermJ}
\set{title}		{Properties of system calls}
\begin{theorem}

All Etude system calls satisfy the following compatibility law under the MMIX architecture:

\NewHaskellToken	{eqvSysTermJ}				{Ord}	{\TKeyword{eqv\sub{sys}}}
\NewHaskellDescription	{eqvSysTermJ}					{Compatibility of system calls}
\IndexTheorem		{eqvSysTermJ}					{EQV_SYS Compatibility of system calls}

\begin{haskell}
eqvSysTermJ :: forall _Lambda, _Delta, pi, alpha_set1, alpha_set2 => --
	(alpha_set1 `EqvJ` alpha_set2) -> --
	{-sem-}(_Lambda, _Delta {->-}, SYS (pi) (alpha_set1)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, SYS (pi) (alpha_set2))
\end{haskell}
\begin{proof}

By symmetry of the equivalence relation, after reduction of all argument atoms into their respective normal forms. \QED

\end{proof}
\end{theorem}

Every conditional Etude expression of the form “"IFF (alpha) {-THEN-} (tau1) {-ELSE-} (tau2)"\!” evaluates to either "tau1" or "tau2", whenever the specified atom "alpha" has a non-zero or zero normal form, respectively. In Haskell:

\begin{code}
evalTerm {-sem-}(_Lambda, _Delta {->-}, IFF (alpha) {-THEN-} (tau1) {-ELSE-} (tau2)) ---
  | (decodeAtom(NG`FMT`64){--}(evalAtom{--}(alpha)) /= 0)	{-ord-} = evalTerm{--}(_Lambda, _Delta {->-}, tau1)
  | (decodeAtom(NG`FMT`64){--}(evalAtom{--}(alpha)) == 0)	{-ord-} = evalTerm{--}(_Lambda, _Delta {->-}, tau2)
\end{code}

\set{label}		{Lambda/Arch/wfIffTermJ}
\set{title}		{Properties of conditional terms}
\begin{theorem}

All conditional terms satisfy the following properties under the MMIX architecture:

\NewHaskellToken	{wfIffTrueTermJ}			{Ord}	{\TKeyword{wf\sub{tt}}}
\NewHaskellToken	{wfIffFalseTermJ}			{Ord}	{\TKeyword{wf\sub{ff}}}

\NewHaskellDescription	{wfIffTrueTermJ}				{Validity of true “\haskell{IFF}” terms}
\NewHaskellDescription	{wfIffFalseTermJ}				{Validity of false “\haskell{IFF}” terms}

\IndexTheorem		{wfIffTrueTermJ}				{WF_TT Validity of true IF terms}
\IndexTheorem		{wfIffFalseTermJ}				{WF_FF Validity of false IF terms}

\NewHaskellToken	{eqvIffTermJ}				{Ord}	{\TKeyword{eqv\sub{if}}}
\NewHaskellDescription	{eqvIffTermJ}					{Compatibility of “\haskell{IFF}” terms}
\IndexTheorem		{eqvIffTermJ}					{EQV_IF Compatibility of IF terms}

\NewHaskellToken	{eqvIffTrueTermJ}			{Ord}	{\TKeyword{eqv\sub{tt}}}
\NewHaskellToken	{eqvIffFalseTermJ}			{Ord}	{\TKeyword{eqv\sub{ff}}}

\NewHaskellDescription	{eqvIffTrueTermJ}				{Equivalence of true “\haskell{IFF}” terms}
\NewHaskellDescription	{eqvIffFalseTermJ}				{Equivalence of false “\haskell{IFF}” terms}

\IndexTheorem		{eqvIffTrueTermJ}				{EQV_TT Equivalence of true IF terms}
\IndexTheorem		{eqvIffFalseTermJ}				{EQV_FF Equivalence of false IF terms}

\begin{haskell}
wfIffTrueTermJ   :: forall _Lambda, _Delta, x, tau1, tau2 =>
	WfJ{--}(_Lambda) -> WfJ{--}(_Delta) -> WfJ{-sem-}(IMM (x) (ZG`FMT`standardEncoding)) -> WfJ{--}(tau1) -> {-sem-}(x /= 0) ->
	WfJ{-sem-}(IFF (IMM (x) (ZG`FMT`standardEncoding)) {-THEN-} (tau1) {-ELSE-} (tau2))
	
wfIffFalseTermJ   :: forall _Lambda, _Delta, x, tau1, tau2 =>
	WfJ{--}(_Lambda) -> WfJ{--}(_Delta) -> WfJ{-sem-}(IMM (x) (ZG`FMT`standardEncoding)) -> WfJ{--}(tau2) -> {-sem-}(x == 0) ->
	WfJ{-sem-}(IFF (IMM (x) (ZG`FMT`standardEncoding)) {-THEN-} (tau1) {-ELSE-} (tau2))

eqvIffTermJ :: forall _Lambda1, _Delta1, alpha1, tau11, tau21, _Lambda2, _Delta2, alpha2, tau12, tau22 =>
	(alpha1 `EqvJ` alpha2) ->
	(_Lambda1, _Delta1 {->-}, tau11) `EqvJ` (_Lambda2, _Delta2 {->-}, tau12) -> --
	(_Lambda1, _Delta1 {->-}, tau12) `EqvJ` (_Lambda2, _Delta2 {->-}, tau22) ->
	{-sem-}(_Lambda1, _Delta1 {->-}, IFF (alpha1) {-THEN-} (tau11) {-ELSE-} (tau21)) `EqvJ` {-sem-}(_Lambda2, _Delta2 {->-}, IFF (alpha2) {-THEN-} (tau12) {-ELSE-} (tau22))

eqvIffTrueTermJ   :: forall _Lambda, _Delta, x, tau1, tau2 =>
	WfJ{--}(_Lambda) -> WfJ{--}(_Delta) -> WfJ{-sem-}(IMM (x) (ZG`FMT`standardEncoding)) -> WfJ{--}(tau1) -> {-sem-}(x /= 0) ->
	{-sem-}(IFF (IMM (x) (ZG`FMT`standardEncoding)) {-THEN-} (tau1) {-ELSE-} (tau2)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, tau1)

eqvIffFalseTermJ  :: forall _Lambda, _Delta, x, tau1, tau2 =>
	WfJ{--}(_Lambda) -> WfJ{--}(_Delta) -> WfJ{-sem-}(IMM (x) (ZG`FMT`standardEncoding)) -> WfJ{--}(tau2) -> {-sem-}(x == 0) ->
	{-sem-}(IFF (IMM (x) (ZG`FMT`standardEncoding)) {-THEN-} (tau1) {-ELSE-} (tau2)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, tau2)
\end{haskell}
\begin{proof}

Once again, all of the above theorems follow directly from the definition of "evalTerm" over Etude's conditional term forms, which always obtains the normal form of the supplied atomic operand "alpha", thus justifying the compatibility law "eqvIffTermJ", and proceeds by reducing the conditional term into one of its two branches "tau1" or "tau2", precisely as required by the above theorems. \QED

\end{proof}
\end{theorem}

Further, the five Etude term forms dedicated to the task of address space and memory image management correspond directly to the analogous object environment operations from Section \ref{Lambda/Arch/State}, so that their respective reductions can be formalised in Haskell \hbox{as follows:}

\begin{code}
evalTerm {-sem-}(_Lambda, _Delta {->-}, NEW  (xi_set {-,-})) ---
		= (_Delta `extendOEnv` xi_set, {-empty-}[], {-sem-}[ IMM (toRational {-sem-}(newInOEnv{--}(_Delta {->-}, xi_set))) (OG`FMT`standardEncoding) ])
evalTerm {-sem-}(_Lambda, _Delta {->-}, DEL  (xi_set {-,-})) ---
		= (_Delta `contractOEnv` xi_set, {-empty-}[], {-empty-}[])
evalTerm {-sem-}(_Lambda, _Delta {->-}, GET  (alpha   {-hv-}, mu_set) (phi)) ---
		= (_Delta, {-empty-}[], {-sem-}[ atomOfOEnv{--}(_Delta {->-}, discardFraction (decodeAtom(OG`FMT`64){--}(evalAtom{--}(alpha))), phi) ])
evalTerm {-sem-}(_Lambda, _Delta {->-}, SET  (alpha1  {-hv-}, mu_set) (phi) {-TO-} (alpha2)) ---
		= (_Delta `setInOEnv` (discardFraction (decodeAtom(OG`FMT`64){--}(evalAtom{--}(alpha1))), phi, evalAtom{--}(alpha2)), {-empty-}[], {-empty-}[])
evalTerm {-sem-}(_Lambda, _Delta {->-}, INI  (alpha1  {-hv-}, mu_set) (phi) {-TO-} (alpha2)) ---
		= (_Delta `setInOEnv` (discardFraction (decodeAtom(OG`FMT`64){--}(evalAtom{--}(alpha1))), phi, evalAtom{--}(alpha2)), {-empty-}[], {-empty-}[])
\end{code}

The reader should observe that, on MMIX, both of the “"SET"” and “"INI"” term constructors assume an identical semantic interpretation. Our architecture shares this peculiarity with almost all other modern instruction set designs, which rarely implement the constant access attributes “"CM"” for memory-resident objects that are allocated dynamically during a program's execution.

\set{label}		{Lambda/Arch/wfNewTermJ}
\set{title}		{Properties of object environment management terms}
\begin{theorem}

On MMIX, all Etude terms of the form “"NEW (xi_set {-,-})"” and “"DEL (xi_set {-,-})"” satisfy the following algebraic theorems from Section \ref{Lambda/Terms}:

\NewHaskellToken	{wfNewTermJ}				{Ord}	{\TKeyword{wf\sub{new}}}
\NewHaskellToken	{wfDelTermJ}				{Ord}	{\TKeyword{wf\sub{del}}}

\NewHaskellDescription	{wfNewTermJ}					{Validity of “\haskell{NEW}” terms}
\NewHaskellDescription	{wfDelTermJ}					{Validity of “\haskell{DEL}” terms}

\IndexTheorem		{wfNewTermJ}					{WF_NEW Validity of NEW terms}
\IndexTheorem		{wfDelTermJ}					{WF_DEL Validity of DEL terms}

\NewHaskellToken	{eqvNewTermJ}				{Ord}	{\TKeyword{eqv\sub{new}}}
\NewHaskellToken	{eqvDelTermJ}				{Ord}	{\TKeyword{eqv\sub{del}}}

\NewHaskellDescription	{eqvNewTermJ}					{Equivalence of “\haskell{NEW}” terms}
\NewHaskellDescription	{eqvDelTermJ}					{Equivalence of “\haskell{DEL}” terms}

\IndexTheorem		{eqvNewTermJ}					{EQV_NEW Equivalence of NEW terms}
\IndexTheorem		{eqvDelTermJ}					{EQV_DEL Equivalence of DEL terms}

\NewHaskellToken	{wfNewAtomsJ}				{Ord}	{\TKeyword{wf\sub{obj}}}
\NewHaskellDescription	{wfNewAtomsJ}					{Validity of stack addresses}
\IndexTheorem		{wfNewAtomsJ}					{WF_OBJ Validity of stack addresses}

\begin{haskell}
wfNewTermJ  :: forall _Lambda, _Delta, xi_set => WfJ{--}(_Lambda) -> WfJ{-sem-}(_Delta `extendOEnv`    xi_set) -> WfJ{-sem-}(_Lambda, _Delta {->-}, NEW  (xi_set))
wfDelTermJ  :: forall _Lambda, _Delta, xi_set => WfJ{--}(_Lambda) -> WfJ{-sem-}(_Delta `contractOEnv`  xi_set) -> WfJ{-sem-}(_Lambda, _Delta {->-}, DEL  (xi_set))

eqvNewTermJ  :: forall _Lambda, _Delta, xi_set =>
	WfJ{--}(_Lambda) -> WfJ{-sem-}(_Delta `extendOEnv` xi_set) ->
	{-sem-}(_Lambda, _Delta {->-}, NEW  (xi_set)) `EqvJ` {-sem-}(_Lambda, _Delta `extendOEnv` xi_set {->-}, --
		RET [IMM ({-sem-}(newInOEnv{--}(_Delta {->-}, xi_set))) (OG`FMT`standardEncoding)])

eqvDelTermJ  :: forall _Lambda, _Delta, xi_set =>
	WfJ{--}(_Lambda) -> WfJ{-sem-}(_Delta `contractOEnv` xi_set) ->
	{-sem-}(_Lambda, _Delta {->-}, DEL  (xi_set)) `EqvJ` {-sem-}(_Lambda, _Delta `contractOEnv` xi_set {->-}, RET [])

\end{haskell}
\set{capture}{hidden}
\begin{haskell}
wfNewAtomsJ :: forall _Lambda, _Delta, xi_set =>
	WfJ{-sem-}(_Lambda, _Delta {->-}, NEW (xi_set)) ->
	(forall n, phi, mu_set => --
		{-sem-}({-hv-}(n {-hv-}, phi {-hv-}, mu_set) @@ xi_set) -> --
		WfJ{-sem-}(OP2 ({-hv-}(OP1 (CNV {-sem-}(objectFormat{--}(phi))) (OG`FMT`standardEncoding) --
						{--}{-hv-}(IMM ({-sem-}(newInOEnv{--}(_Delta {->-}, xi_set))) (OG`FMT`standardEncoding)))) --
				(ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding))))
\end{haskell}
\begin{proof}

By definition of "evalTerm", also utilising the analogous results "stackOfOEnvJ" and "extendOEnvStackJ" from Section \ref{Lambda/Arch/State}. \QED

\end{proof}
\end{theorem}

\set{label}		{Lambda/Arch/wfGetTermJ}
\set{title}		{Properties of object environment inspection terms}
\begin{theorem}

Every object environment inspection term of the form “"GET (alpha {-hv-}, mu_set) (phi)"\!” satisfies the following three laws on the MMIX architecture:

\NewHaskellToken	{wfGetTermJ}				{Ord}	{\TKeyword{wf\sub{get}}}
\NewHaskellDescription	{wfGetTermJ}					{Validity of “\haskell{GET}” terms}
\IndexTheorem		{wfGetTermJ}					{WF_GET Validity of GET terms}

\NewHaskellToken	{eqvGetTermJ}				{Ord}	{\TKeyword{eqv\sub{get}}}
\NewHaskellDescription	{eqvGetTermJ}					{Compatibility of “\haskell{GET}” terms}
\IndexTheorem		{eqvGetTermJ}					{EQV_GET Compatibility of GET terms}

\NewHaskellToken	{eqvGetEnvTermJ}			{Ord}	{\TKeyword{eqv\sub{geto}}}
\NewHaskellDescription	{eqvGetEnvTermJ}				{Reduction of “\haskell{GET}” terms}
\IndexTheorem		{eqvGetEnvTermJ}				{EQV_GETO Reduction of GET terms}

\begin{haskell}
wfGetTermJ  :: forall _Lambda, _Delta, x, n, phi, mu_set =>
	WfJ{--}(_Lambda) -> --
	WfJ{-sem-}(OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding))) -> --
	WfJ{-sem-}(atomOfOEnv{--}(_Delta {->-}, x + n, phi)) ->
	{-sem-}({-hv-}(x + n {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	WfJ{-sem-}(_Lambda, _Delta {->-}, --
			GET (OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding)) {-hv-}, mu_set) (phi))

eqvGetTermJ :: forall _Lambda, _Delta, alpha1, alpha2, mu_set, phi =>
	(alpha1 `EqvJ` alpha2) ->
	{-sem-}(_Lambda, _Delta {->-}, GET (alpha1 {-hv-}, mu_set) (phi)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, GET (alpha2 {-hv-}, mu_set) (phi))

eqvGetEnvTermJ :: forall _Lambda, _Delta, x, n, phi, mu_set =>
	WfJ{--}(_Lambda) -> --
	WfJ{-sem-}(OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding))) -> --
	WfJ{-sem-}(atomOfOEnv{--}(_Delta {->-}, x + n, phi)) ->
	{-sem-}({-hv-}(x + n {-hv-}, phi {-hv-}, mu_set `difference` listToSet [VM]) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(_Lambda, _Delta {->-}, --
			GET (OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding)) {-hv-}, mu_set) (phi)) --
		`EqvJ` {-sem-}(_Lambda, _Delta {->-}, RET {-sem-}[atomOfOEnv{--}(_Delta {->-}, x + n, phi)])
\end{haskell}
\begin{proof}

First of all, we observe that, on MMIX, theorems "wfGetTermJ" and "eqvGetEnvTermJ" are not constrained to the specific structures of the address atom "alpha" that are described above. After reduction by "evalAtom" within the definition of "evalTerm", all such terms are rendered equivalent to their appropriate normal forms “"IMM (n) (NG`FMT`64)"\!”, so that theorem "eqvGetTermJ" holds trivially by the earlier reflexivity of term equivalence relation "eqvReflTermJ". Similarly, theorems "wfGetTermJ" and "eqvGetEnvTermJ" are justified by the definitional equality of the atoms delivered by "evalTerm" from “"GET"” using the object inspection construct "atomOfOEnv{--}(_Delta {->-}, x + n, phi)", which both of these constraints require to be well-formed as part of their respective preconditions. \QED

\end{proof}
\end{theorem}

\set{label}		{Lambda/Arch/wfSetTermJ}
\set{title}		{Properties of object environment update terms}
\begin{theorem}

Under the MMIX architecture, every object environment update term of the form \hbox{“"SET (alpha1 {-hv-}, mu_set) (phi) {-TO-} (alpha2)"\!”} and \hbox{“"INI (alpha1 {-hv-}, mu_set) (phi) {-TO-} (alpha2)"\!”} satisfies the following constraint theorems described earlier in Section \ref{Lambda/Terms}:

\NewHaskellToken	{wfSetTermJ}				{Ord}	{\TKeyword{wf\sub{set}}}
\NewHaskellDescription	{wfSetTermJ}					{Validity of “\haskell{SET}” terms}
\IndexTheorem		{wfSetTermJ}					{WF_SET Validity of SET terms}

\NewHaskellToken	{wfIniTermJ}				{Ord}	{\TKeyword{wf\sub{ini}}}
\NewHaskellDescription	{wfIniTermJ}					{Validity of “\haskell{INI}” terms}
\IndexTheorem		{wfIniTermJ}					{WF_INI Validity of SETI terms}

\NewHaskellToken	{eqvSetTermJ}				{Ord}	{\TKeyword{eqv\sub{set}}}
\NewHaskellDescription	{eqvSetTermJ}					{Compatibility of “\haskell{SET}” terms}
\IndexTheorem		{eqvSetTermJ}					{EQV_SET Compatibility of SET terms}

\NewHaskellToken	{eqvIniTermJ}				{Ord}	{\TKeyword{eqv\sub{ini}}}
\NewHaskellDescription	{eqvIniTermJ}					{Compatibility of “\haskell{INI}” terms}
\IndexTheorem		{eqvIniTermJ}					{EQV_INI Compatibility of SETI terms}

\NewHaskellToken	{eqvSetEnvTermJ}			{Ord}	{\TKeyword{eqv\sub{seto}}}
\NewHaskellDescription	{eqvSetEnvTermJ}				{Reduction of “\haskell{SET}” terms}
\IndexTheorem		{eqvSetEnvTermJ}				{EQV_SETO Reduction of SET terms}

\NewHaskellToken	{eqvIniEnvTermJ}			{Ord}	{\TKeyword{eqv\sub{inio}}}
\NewHaskellDescription	{eqvIniEnvTermJ}				{Reduction of “\haskell{INI}” terms}
\IndexTheorem		{eqvIniEnvTermJ}				{EQV_INIO Reduction of SETI terms}

\begin{haskell}
wfSetTermJ :: forall _Lambda, _Delta, x, n, phi, mu_set, alpha =>
	WfJ{--}(_Lambda) -> --
	WfJ{--}(_Delta `setInOEnv` (x + n, phi, alpha)) -> --
	WfJ{-sem-}(OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding))) ->
	{-sem-}({-hv-}(x + n {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	WfJ{-sem-}(_Lambda, _Delta {->-}, --
		SET (OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding)) {-hv-}, mu_set) (phi) {-TO-} (alpha))

wfIniTermJ :: forall _Lambda, _Delta, x, n, phi, mu_set, alpha =>
	WfJ{--}(_Lambda) -> --
	WfJ{--}(_Delta `setInOEnv` (x + n, phi, alpha)) -> --
	WfJ{-sem-}(OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding))) ->
	{-sem-}({-hv-}(x + n {-hv-}, phi {-hv-}, mu_set) `inEnvelope` initOfOEnv{--}(_Delta)) ->
	WfJ{-sem-}(_Lambda, _Delta {->-}, --
		INI (OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding)) {-hv-}, mu_set) (phi) {-TO-} (alpha))

eqvSetTermJ :: forall _Lambda, _Delta, alpha11, alpha21, alpha12, alpha22, mu_set, phi =>
	(alpha11 `EqvJ` alpha12) -> --
	(alpha21 `EqvJ` alpha22) ->
	{-sem-}(_Lambda, _Delta {->-}, SET (alpha11 {-hv-}, mu_set) (phi) {-TO-} (alpha21)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, SET (alpha12 {-hv-}, mu_set) (phi) {-TO-} (alpha22))

eqvIniTermJ :: forall _Lambda, _Delta, alpha11, alpha21, alpha12, alpha22, mu_set, phi =>
	(alpha11 `EqvJ` alpha12) -> --
	(alpha21 `EqvJ` alpha22) ->
	{-sem-}(_Lambda, _Delta {->-}, INI (alpha11 {-hv-}, mu_set) (phi) {-TO-} (alpha21)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, SET (alpha12 {-hv-}, mu_set) (phi) {-TO-} (alpha22))

eqvSetEnvTermJ :: forall _Lambda, _Delta, x, n, mu_set, phi, alpha =>
	WfJ{--}(_Lambda) -> --
	WfJ{--}(_Delta `setInOEnv` (x + n, phi, alpha)) -> --
	WfJ{-sem-}(OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding))) ->
	{-sem-}({-hv-}(x + n {-hv-}, phi {-hv-}, emptySet) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(_Lambda, _Delta {->-}, ---
		SET (OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding)) {-hv-}, mu_set) (phi) {-TO-} (alpha)) --
			`EqvJ` {-sem-}(_Lambda, _Delta `setInOEnv` (x + n, phi, alpha) {->-}, RET []) {-!-}{-!-}{-!-}{-!-}{-!-}

eqvIniEnvTermJ :: forall _Lambda, _Delta, x, n, mu_set, phi, alpha =>
	WfJ{--}(_Lambda) -> --
	WfJ{--}(_Delta `setInOEnv` (x + n, phi, alpha)) -> --
	WfJ{-sem-}(OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding))) ->
	{-sem-}({-hv-}(x + n {-hv-}, phi {-hv-}, listToSet [CM]) `inEnvelope` initOfOEnv{--}(_Delta)) ->
	{-sem-}(_Lambda, _Delta {->-}, ---
		INI (OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding)) {-hv-}, mu_set) (phi) {-TO-} (alpha)) --
			`EqvJ` {-sem-}(_Lambda, _Delta `setInOEnv` (x + n, phi, alpha) {->-}, RET []) {-!-}{-!-}{-!-}{-!-}{-!-}
\end{haskell}
\begin{proof}

A formal justification of these theorems follows a general pattern analogous to that outlined earlier for the object environment inspection operations, observing that, on MMIX, no specific restrictions are imposed on these constructs' access attribute sets "mu_set" or the precise structure of their address terms "alpha". \QED

\end{proof}
\end{theorem}

Last but not least, an Etude operation of the form \hbox{“"LET [{-raw-}(nu_set {-=-}, tau1)] (tau2)"\!”} is essentially reducible into its body term "tau2", except that any actions executed by "tau1" are prefixed to those produced by "tau2" itself and that all free occurrences of any variables from "nu_set" in "tau2" are replaced with the respective atomic values delivered by "tau1":

\begin{code}
evalTerm {-sem-}(_Lambda, _Delta {->-}, LET [{-raw-}(nu_set {-=-}, tau1)] (tau2)) | (lengthOfList{--}(alpha_set1) == lengthOfList{--}(nu_set)) = (_Delta2, a_set1 ++ a_set2, alpha_set2)

 where	(_Delta1, a_set1, alpha_set1) = evalTerm{--}(_Lambda, _Delta 	 {->-}, tau1)
 	(_Delta2, a_set2, alpha_set2) = evalTerm{--}(_Lambda, _Delta1	 {->-}, tau2 `substTerm` (listToMap (nu_set `zipList` alpha_set1)))
\end{code}

\set{label}		{Lambda/Arch/eqvLetRetTermJ}
\set{title}		{Properties of atom bindings}
\begin{theorem}

The binding of a trivial well-formed Etude term \hbox{“"RET (alpha_set)"”} to a list of variables "nu_set" of the same length as "alpha_set" is equivalent to the substitution of "alpha_set" for "nu_set" within the associated body term "tau":

\NewHaskellToken	{eqvLetRetTermJ}			{Ord}	{\TKeyword{eqv\sub{leta}}}
\NewHaskellDescription	{eqvLetRetTermJ}				{Reduction of trivial “\haskell{LET}” bindings}
\IndexTheorem		{eqvLetRetTermJ}				{WF_LETA Reduction of trivial LET bindings}

\begin{haskell}
eqvLetRetTermJ :: forall _Lambda, _Delta, nu_set, alpha_set =>
	WfJ{--}(nu_set) -> WfJ{--}(alpha_set) ->
	{-sem-}(lengthOfList{--}(nu_set) == lengthOfList{--}(alpha_set)) ->
	{-sem-}(_Lambda, _Delta {->-}, LET [nu_set =: RET (alpha_set)] (tau)) `EqvJ` (_Lambda, _Delta {->-}, tau `substTerm` (nu_set `zipList` alpha_set))
\end{haskell}
\begin{proof}

According to the earlier definition of \hbox{"evalTerm{--}(_Lambda, _Delta {->-}, RET (alpha_set))",} all such terms deliver an empty list of actions "a_set1", together with the normal forms of the specified atoms "alpha_set". The object environment "_Delta" is not affected by such reduction. In effect, \hbox{we have:}

\begin{haskell}
evalTerm{--}(_Lambda, _Delta {->-}, LET [nu_set =: RET (alpha_set)] (tau))
	{--}{-hfil-}={-hfil-}  		evalTerm{--}(_Lambda, _Delta {->-}, tau `substTerm` (listToMap (nu_set `zipList` evalAtoms{--}(alpha_set))))
	{--}{-hfil-}`EqvJ`{-hfil-}	(_Lambda, _Delta {->-}, tau `substTerm` (nu_set `zipList` alpha_set))
\end{haskell}

as required. \QED

\end{proof}
\end{theorem}

\set{label}		{Lambda/Arch/eqvLetSysTermJ}
\set{title}		{Properties of monadic term bindings}
\begin{theorem}

Given a pair of Etude constructs \hbox{“"LET [nu_set1 =: tau11] (tau21)"\!”} and \hbox{“"LET [nu_set2 =: tau12] (tau22)"\!”,} in which the respective variable lists have equal lengths and the two terms "tau11" and "tau12" are equivalent under the prevailing evaluation environments, the entire constructs are also equivalent, provided that, for every list of well-formed atoms "alpha_set'" of an appropriate length, a substitution of these atoms for "nu_set1" in "tau21" and, in "tau22" for the variable list "nu_set2" delivers a pair of Etude term forms that are equivalent under any well-formed function and object environment "(_Lambda', _Delta')". Formally:

\NewHaskellToken	{eqvLetSysTermJ}			{Ord}	{\TKeyword{eqv\sub{lets}}}
\NewHaskellDescription	{eqvLetSysTermJ}				{Reduction of system calls “\haskell{LET}” bindings}
\IndexTheorem		{eqvLetSysTermJ}				{EQV_LETS Reduction of system call LET bindings}

\begin{haskell}
eqvLetSysTermJ :: forall _Lambda, _Delta, nu_set1, tau11, tau21, nu_set2, tau12, tau22 =>
	{-sem-}(_Lambda, _Delta {->-}, tau11) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, tau12) ->
	{-sem-}(lengthOfList{--}(nu_set1) == lengthOfList{--}(nu_set2)) ->
	(	forall _Lambda', _Delta', alpha_set' =>
	 	{-quad-}	WfJ{--}(alpha_set') ->
	 	{-quad-}	{-sem-}(lengthOfList{--}(alpha_set') == lengthOfList{--}(nu_set1)) ->
		{-quad-}	{-sem-}(_Lambda', _Delta' {->-}, tau21 `substTerm` (nu_set1 `zipList` alpha_set')) --
					`EqvJ` {-sem-}(_Lambda', _Delta' {->-}, tau22 `substTerm` (nu_set2 `zipList` alpha_set'))) ->
	{-sem-}(_Lambda, _Delta {->-}, LET [nu_set1 =: tau11] (tau21)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, LET [nu_set2 =: tau12] (tau22))
\end{haskell}
\begin{proof}

First of all, observe that, from the equivalence of "tau11" and "tau12", we have \hbox{"evalTerm{--}(_Lambda, _Delta {->-}, tau11) = evalTerm{--}(_Lambda, _Delta {->-}, tau12) = (_Delta', a_set, alpha_set')",} for some well-formed value of "_Delta'" and "alpha_set'". Further, an application of the delivered atom list "alpha_set'" to the second assumption of "eqvLetSysTermJ" provides us with the equivalence of the substituted terms \hbox{"(_Lambda, _Delta' {->-}, tau21 `substTerm` (nu_set1 `zipList` alpha_set'))"} and \hbox{"(_Lambda, _Delta' {->-}, tau22 `substTerm` (nu_set2 `zipList` alpha_set'))",} or the definitional equality of \hbox{"evalTerm{--}(_Lambda, _Delta' {->-}, tau21 `substTerm` (nu_set1 `zipList` alpha_set'))"} and \hbox{"evalTerm{--}(_Lambda, _Delta' {->-}, tau22 `substTerm` (nu_set2 `zipList` alpha_set'))",} from which the equivalence of the entire corresponding “"LET"” forms can be obtained directly by the virtue of theorem "EqvTermC". \QED

\end{proof}
\end{theorem}

\set{label}		{Lambda/Arch/eqvLetLetTermJ}
\set{title}		{Properties of nested bindings}
\begin{theorem}

A nested “"LET"” term of the form \hbox{“"LET ({-hv-}[nu_set =: LET (beta_set) (tau1)]) (tau2)"\!”} is always guaranteed to be equivalent to its flattened variant \hbox{“"LET (beta_set) {-hv-}(LET [nu_set =: tau1] (tau2))"\!”}, provided that no variable bound in "beta_set" appears free within the body term "tau2", with the possible exception of any variables that are also bound in "nu_set":

\NewHaskellToken	{eqvLetLetTermJ}			{Ord}	{\TKeyword{eqv\sub{letn}}}
\NewHaskellDescription	{eqvLetLetTermJ}				{Reduction of nested “\haskell{LET}” bindings}
\IndexTheorem		{eqvLetLetTermJ}				{EQV_LETN Reduction of nested LET bindings}

\begin{haskell}
eqvLetLetTermJ :: forall _Lambda, _Delta, nu_set, beta_set, tau1, tau2 =>
	{-sem-}(bvOfBindings{--}(beta_set) `intersection` (fvOfTerm{--}(tau2) `difference` nu_set) == emptySet) ->
	{-sem-}(_Lambda, _Delta {->-}, LET ({-hv-}[nu_set =: LET (beta_set) (tau1)]) (tau2)) --
		`EqvJ` {-sem-}(_Lambda, _Delta {->-}, LET (beta_set) {-hv-}(LET [nu_set =: tau1] (tau2)))
\end{haskell}
\begin{proof}

For simplicity, let us only consider the case of "beta_set == {-sem-}[nu_set' =: tau']", since all more complex binding groups are always reduced by "evalTerm" into a sequence of such term forms. From the definition of "evalTerm{--}(_Lambda, _Delta {->-}, LET ({-hv-}[nu_set =: LET [nu_set' =: tau'] (tau1)]) (tau2))", we have:

\begin{haskell}
evalTerm {-sem-}(_Lambda, _Delta {->-}, LET ({-hv-}[nu_set =: LET [nu_set' =: tau'] (tau1)]) (tau2)) = (_Delta2, a_set1 ++ a_set2, alpha_set2)

 where	(_Delta1,  a_set1,   alpha_set1)   = evalTerm{-sem-}(_Lambda, _Delta 	 {->-}, LET [nu_set' =: tau'] (tau1)]) (tau2))
 	(_Delta2,  a_set2,   alpha_set2)   = evalTerm{-sem-}(_Lambda, _Delta1	 {->-}, tau2 `substTerm` (listToMap (nu_set `zipList` alpha_set1)))

{-{ and, for the inner “"LET"” binding: }-}

evalTerm{--}(_Lambda, _Delta {->-}, LET [nu_set' =: tau'] (tau1)) = (_Delta1, a_set' ++ a_set1', alpha_set1)

 where	(_Delta',  a_set',   alpha_set')   = evalTerm{-sem-}(_Lambda, _Delta 	 {->-}, tau')
 	(_Delta1,  a_set1',  alpha_set1)   = evalTerm{-sem-}(_Lambda, _Delta'	 {->-}, tau1 `substTerm` (listToMap (nu_set' `zipList` alpha_set')))

{-{ where "a_set1 == a_set' ++ a_set1'". Similarly, the reduction of “"LET [nu_set' =: tau'] {-hv-}(LET [nu_set =: tau1] (tau2))"” proceeds as follows: }-}

evalTerm {-sem-}(_Lambda, _Delta {->-}, LET [nu_set' =: tau'] {-hv-}(LET [nu_set =: tau1] (tau2))) = (_Delta2, a_set' ++ a_set2', alpha_set2)

 where	(_Delta',  a_set',   alpha_set')   = evalTerm{-sem-}(_Lambda, _Delta 	 {->-}, tau')
 	(_Delta2,  a_set2',  alpha_set2)   = evalTerm{-sem-}(_Lambda, _Delta'	 {->-}, {-sem-}(LET [nu_set =: tau1] (tau2)) `substTerm` (listToMap (nu_set' `zipList` alpha_set')))

{-{ Given that, by assumption, "nu_set'" does not appear free within "tau2", "tau2 `substTerm` (listToMap (nu_set' `zipList` alpha_set')) = tau2", so that the later “"LET"” form can be simplified as follows: }-}

evalTerm {-sem-}(_Lambda, _Delta {->-}, LET [nu_set =: {-sem-}(tau1 `substTerm` (listToMap (nu_set' `zipList` alpha_set')))] (tau2)) = (_Delta2, a_set1' ++ a_set2, alpha_set2)

 where	(_Delta1,  a_set1',  alpha_set1)   = evalTerm{-sem-}(_Lambda, _Delta 	 {->-}, tau1 `substTerm` (listToMap (nu_set' `zipList` alpha_set')))
 	(_Delta2,  a_set2,   alpha_set2)   = evalTerm{-sem-}(_Lambda, _Delta1	 {->-}, tau2 `substTerm` (listToMap (nu_set `zipList` alpha_set1)))
\end{haskell}

so that both the nested and the flattened variant of the “"LET"” statement are reducible to the common result value "(_Delta2, a_set' ++ a_set1' ++ a_set2, alpha_set2)" as required. \QED
	
\end{proof}
\end{theorem}

On MMIX, all bindings that have been grouped into a single “"LET"” term are always evaluated precisely in the order their appearance within the group, so that the reductions of all remaining “"LET"” term forms can be formalised trivially as follows:

\begin{code}
evalTerm {-sem-}(_Lambda, _Delta {->-}, LET [] (tau)) ---
		= evalTerm{--}(_Lambda, _Delta {->-}, tau)
evalTerm {-sem-}(_Lambda, _Delta {->-}, LET (beta_set) (tau)) ---
		= evalTerm{-sem-}(_Lambda, _Delta {->-}, LET [{-sem-}(head{--}(beta_set))] (LET ({-sem-}(tail{--}(beta_set))) (tau)))
\end{code}

\set{label}		{Lambda/Arch/eqvLetListTermJ}
\set{title}		{Properties of binding groups}
\begin{theorem}

All Etude binding groups satisfy the following requirements on the MMIX architecture:

\NewHaskellToken	{wfLetListTermJ}			{Ord}	{\TKeyword{wf\sub{letm}}}
\NewHaskellDescription	{wfLetListTermJ}				{Validity of “\haskell{LET}” binding groups}
\IndexTheorem		{wfLetListTermJ}				{WF_LETM Validity of LET binding groups}

\NewHaskellToken	{eqvLetNullTermJ}			{Ord}	{\TKeyword{eqv\sub{lete}}}
\NewHaskellDescription	{eqvLetNullTermJ}				{Reduction of degenerate “\haskell{LET}” terms}
\IndexTheorem		{eqvLetNullTermJ}				{WF_LETE Reduction of degenerate LET terms}

\NewHaskellToken	{eqvLetListTermJ}			{Ord}	{\TKeyword{eqv\sub{letm}}}
\NewHaskellDescription	{eqvLetListTermJ}				{Reduction of “\haskell{LET}” binding groups}
\IndexTheorem		{eqvLetListTermJ}				{EQV_LETM Reduction of LET binding groups}

\begin{haskell}
wfLetListTermJ :: forall _Lambda, _Delta, beta_set, tau =>
	WfJ{--}(_Lambda) -> WfJ{--}(_Delta) ->
	WfJ{-sem-}(_Lambda, _Delta {->-}, LET [{-sem-}(head{--}(beta_set))] (LET ({-sem-}(tail{--}(beta_set))) (tau))) ->
	{-sem-}(union [fvOfTerm{--}(tau_k) `difference` nu_set{-sub-}k | {-sem-}(nu_set{-sub-}k =: tau_k) <- beta_set {-,,-} ] `intersection` bvOfBindings{--}(beta_set) == emptySet) ->
	(	forall k => --
			{-sem-}(0 <= k < factorial (lengthOfList{--}(beta_set)) {-!-}) -> --
			{-sem-}(_Lambda, _Delta {->-}, LET (beta_set) (tau)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, LET ({-sem-}(permutation{--}(k, beta_set))) (tau))) -> --
				{-!-}{-!-}{-!-}{-!-}{-!-}{-!-}
	WfJ{-sem-}(_Lambda, _Delta {->-}, LET (beta_set) (tau))

eqvLetNullTermJ  :: forall _Lambda, _Delta, tau => {-sem-}(_Lambda, _Delta {->-}, LET [] (tau)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, tau)

eqvLetListTermJ :: forall _Lambda, _Delta, nu_set, beta_set, tau =>
	{-sem-}(lengthOfList{--}(beta_set) > 1) ->
	{-sem-}(_Lambda, _Delta {->-}, LET [beta_set] (tau)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, LET [{-sem-}(head{--}(beta_set))] (LET ({-sem-}(tail{--}(beta_set))) (tau)))
\end{haskell}
\begin{proof}

By definition of the "evalTerm" function. The convoluted precondition of "wfLetListTermJ", which was provided in the earlier algebraic semantics of Etude in order to forbid overlapping of side effects within a binding group, is irrelevant on MMIX, which always imposes a predetermined order of term reduction within such constructs. \QED

\end{proof}
\end{theorem}

Finally, one more auxiliary theorem must be established in order to complete our formalisation of Etude terms under the MMIX instruction set architecture:

\set{label}		{Lambda/Arch/wfEqvTermJ}
\set{title}		{Validity of equivalent term forms}
\begin{theorem}

Every Etude term from a given equivalent class is well-formed whenever at least one of the terms in that class is deemed valid:

\NewHaskellToken	{wfEqvTermJ}				{Ord}	{\TKeyword{wf\sub{eqv}}}
\NewHaskellDescription	{wfEqvTermJ}					{Validity of equivalent Etude terms}
\IndexTheorem		{wfEqvTermJ}					{WF_EQV Validity of equivalent Etude terms}

\begin{haskell}
wfEqvTermJ :: forall _Lambda1, _Delta1, tau1, _Lambda2, _Delta2, tau2 =>
	WfJ{-sem-}(_Lambda1, _Delta1 {->-}, tau1) -> --
	{-sem-}(_Lambda1, _Delta1 {->-}, tau1) `EqvJ` {-sem-}(_Lambda2, _Delta2 {->-}, tau2) -> --
	WfJ{-sem-}(_Lambda2, _Delta2 {->-}, tau2)
\end{haskell}
\begin{proof}

Trivial, from the analogous property "wfEqvAtomJ" that was established for Etude atoms in Section \ref{Lambda/Arch/Atoms}. \QED

\end{proof}
\end{theorem}

A complete operational description of Etude terms is, regrettably, impossible without a detailed scrutiny of the underlying operating system facilities. In particular, most of such facilities operate in the real world of hardware devices and user interactions, so that their complete formalisation would probably mandate an extension of the earlier object environment structure from Section \ref{Lambda/Arch/State} with some suitable abstract rendition of that real world. Fortunately, this added complexity is not required for a successful establishment of the linear correctness property for our compiler, whereby the above algebraic treatment of system calls turns out to be sufficiently detailed as to satisfy all of the proof obligations raised in the following section of this work. Accordingly, without further ado, let us now proceed to the final code generation stage \hbox{of the entire project.}

\end{subsection}
\end{section}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%